Package: emmeans
Type: Package
Title: Estimated Marginal Means, aka Least-Squares Means
Version: 1.10.2
Date: 2024-05-19
Authors@R: c(person("Russell V.", "Lenth", role = c("aut", "cre", "cph"), 
    email = "russell-lenth@uiowa.edu"),
    person("Ben", "Bolker", role = "ctb"),
    person("Paul", "Buerkner", role = "ctb"),
    person("Iago", "Giné-Vázquez", role = "ctb"),
    person("Maxime", "Herve", role = "ctb"),
    person("Maarten", "Jung", role = "ctb"),
    person("Jonathon", "Love", role = "ctb"),
    person("Fernando", "Miguez", role = "ctb"),
    person("Julia", "Piaskowski", role = "ctb"),
    person("Hannes", "Riebl", role = "ctb"),
    person("Henrik", "Singmann", role = "ctb"))
Depends: R (>= 4.1.0)
Imports: estimability (>= 1.4.1), graphics, methods, numDeriv, stats,
        utils, mvtnorm
Suggests: bayesplot, bayestestR, biglm, brms, car, coda (>= 0.17),
        ggplot2, lattice, logspline, mediation, mgcv, multcomp,
        multcompView, nlme, ordinal (>= 2014.11-12), pbkrtest (>=
        0.4-1), lme4, lmerTest (>= 2.0.32), MASS, MuMIn, rsm, knitr,
        rmarkdown, sandwich, scales, splines, testthat, tibble, xtable
        (>= 1.8-2)
Enhances: CARBayes, coxme, gee, geepack, MCMCglmm, MCMCpack, mice,
        nnet, pscl, rstanarm, sommer, survival
URL:
        https://rvlenth.github.io/emmeans/,https://rvlenth.github.io/emmeans/
BugReports: https://github.com/rvlenth/emmeans/issues
LazyData: yes
ByteCompile: yes
Description: Obtain estimated marginal means (EMMs) for many linear, generalized 
  linear, and mixed models. Compute contrasts or linear functions of EMMs,
  trends, and comparisons of slopes. Plots and other displays.
  Least-squares means are discussed, and the term "estimated marginal means"
  is suggested, in Searle, Speed, and Milliken (1980) Population marginal means 
  in the linear model: An alternative to least squares means, The American 
  Statistician 34(4), 216-221 <doi:10.1080/00031305.1980.10483031>.
License: GPL-2 | GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.1
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2024-05-20 02:51:17 UTC; rlenth
Author: Russell V. Lenth [aut, cre, cph],
  Ben Bolker [ctb],
  Paul Buerkner [ctb],
  Iago Giné-Vázquez [ctb],
  Maxime Herve [ctb],
  Maarten Jung [ctb],
  Jonathon Love [ctb],
  Fernando Miguez [ctb],
  Julia Piaskowski [ctb],
  Hannes Riebl [ctb],
  Henrik Singmann [ctb]
Maintainer: Russell V. Lenth <russell-lenth@uiowa.edu>
Repository: CRAN
Date/Publication: 2024-05-20 03:50:02 UTC
Built: R 4.2.3; ; 2024-05-20 06:24:14 UTC; unix

Package: emmeans
Type: Package
Title: Estimated Marginal Means, aka Least-Squares Means
Version: 2.0.3
Date: 2026-03-30
Authors@R: c(person("Russell V.", "Lenth", role = c("aut", "cph"), 
                    email = "russell-lenth@uiowa.edu"),
    person("Julia", "Piaskowski", role = c("cre", "aut"),
           email = "julia.piask@gmail.com"),
    person("Balazs", "Banfai", role = "ctb"),
    person("Ben", "Bolker", role = "ctb"),
    person("Paul", "Buerkner", role = "ctb"),
    person("Iago", "Giné-Vázquez", role = "ctb"),
    person("Maxime", "Hervé", role = "ctb"),
    person("Maarten", "Jung", role = "ctb"),
    person("Jonathon", "Love", role = "ctb"),
    person("Fernando", "Miguez", role = "ctb"),
    person("Hannes", "Riebl", role = "ctb"),
    person("Henrik", "Singmann", role = "ctb"))
Maintainer: Julia Piaskowski <julia.piask@gmail.com>
Depends: R (>= 4.1.0)
Imports: estimability (>= 1.4.1), graphics, methods, mvtnorm, numDeriv,
        rlang, stats, utils
Suggests: bayesplot, bayestestR, biglm, brms, car, coda (>= 0.17),
        compositions, ggplot2 (>= 4.0.0), knitr, lattice, lme4,
        lmerTest (>= 2.0.32), logspline, MASS, mediation, mgcv,
        multcomp, multcompView, MuMIn, nlme, ordinal (>= 2014.11-12),
        pbkrtest (>= 0.4-1), rmarkdown, robmixglm, rsm, sandwich,
        scales, splines, testthat, tibble, xtable (>= 1.8-2)
Enhances: CARBayes, coxme, gee, geepack, MCMCglmm, MCMCpack, mice,
        nnet, pscl, rstanarm, sommer, survival
URL: https://rvlenth.github.io/emmeans/,
        https://github.com/rvlenth/emmeans/
BugReports: https://github.com/rvlenth/emmeans/issues
LazyData: yes
ByteCompile: yes
Description: Obtain estimated marginal means (EMMs) for many linear, generalized 
  linear, and mixed models. Compute contrasts or linear functions of EMMs,
  trends, and comparisons of slopes. Plots and other displays.
  Least-squares means are discussed, and the term "estimated marginal means"
  is suggested, in Searle, Speed, and Milliken (1980) Population marginal means 
  in the linear model: An alternative to least squares means, The American 
  Statistician 34(4), 216-221 <doi:10.1080/00031305.1980.10483031>.
License: GPL-2 | GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-08 22:31:47 UTC; jpiaskowski
Author: Russell V. Lenth [aut, cph],
  Julia Piaskowski [cre, aut],
  Balazs Banfai [ctb],
  Ben Bolker [ctb],
  Paul Buerkner [ctb],
  Iago Giné-Vázquez [ctb],
  Maxime Hervé [ctb],
  Maarten Jung [ctb],
  Jonathon Love [ctb],
  Fernando Miguez [ctb],
  Hannes Riebl [ctb],
  Henrik Singmann [ctb]
Repository: CRAN
Date/Publication: 2026-04-09 10:00:20 UTC
Built: R 4.6.0; ; 2026-04-25 16:13:50 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method("+",emmGrid)
S3method("[",emmGrid)
S3method("[",summary_emm)
S3method("levels<-",emmGrid)
S3method(as.data.frame,emmGrid)
S3method(as.data.frame,emm_list)
S3method(as.data.frame,summary_eml)
S3method(as.data.frame,summary_emm)
S3method(as.glht,default)
S3method(as.glht,emmGrid)
S3method(as.glht,emm_list)
S3method(as.list,emmGrid)
S3method(as.list,emm_list)
S3method(coef,emmGrid)
S3method(coef,emm_list)
S3method(coef,glht_list)
S3method(confint,emmGrid)
S3method(confint,emm_list)
S3method(confint,glht_list)
S3method(contrast,emmGrid)
S3method(contrast,emm_list)
S3method(emm_basis,Gam)
S3method(emm_basis,MCMCglmm)
S3method(emm_basis,aovlist)
S3method(emm_basis,averaging)
S3method(emm_basis,betareg)
S3method(emm_basis,brmsfit)
S3method(emm_basis,carbayes)
S3method(emm_basis,clm)
S3method(emm_basis,clmm)
S3method(emm_basis,coxme)
S3method(emm_basis,coxph)
S3method(emm_basis,default)
S3method(emm_basis,gam)
S3method(emm_basis,gamlss)
S3method(emm_basis,gamm)
S3method(emm_basis,gee)
S3method(emm_basis,geeglm)
S3method(emm_basis,geese)
S3method(emm_basis,glmgee)
S3method(emm_basis,gls)
S3method(emm_basis,gnls)
S3method(emm_basis,hurdle)
S3method(emm_basis,lm)
S3method(emm_basis,lme)
S3method(emm_basis,lqm)
S3method(emm_basis,lqmm)
S3method(emm_basis,mblogit)
S3method(emm_basis,mcmc)
S3method(emm_basis,mcmc.list)
S3method(emm_basis,merMod)
S3method(emm_basis,mira)
S3method(emm_basis,mlm)
S3method(emm_basis,mmer)
S3method(emm_basis,multinom)
S3method(emm_basis,nlme)
S3method(emm_basis,nls)
S3method(emm_basis,polr)
S3method(emm_basis,qdrg)
S3method(emm_basis,rms)
S3method(emm_basis,rq)
S3method(emm_basis,rqs)
S3method(emm_basis,stanreg)
S3method(emm_basis,survreg)
S3method(emm_basis,svyolr)
S3method(emm_basis,zeroinfl)
S3method(emmip,default)
S3method(head,emmGrid)
S3method(linfct,default)
S3method(linfct,emm_list)
S3method(pairs,emmGrid)
S3method(pairs,emm_list)
S3method(plot,emmGrid)
S3method(plot,emm_list)
S3method(plot,glht_list)
S3method(plot,summary_emm)
S3method(predict,emmGrid)
S3method(predict,emm_list)
S3method(print,emmGrid)
S3method(print,emm_list)
S3method(print,pwpm)
S3method(print,summary_eml)
S3method(print,summary_emm)
S3method(qdrg,default)
S3method(rbind,emmGrid)
S3method(rbind,emm_list)
S3method(rbind,summary_eml)
S3method(rbind,summary_emm)
S3method(recover_data,MCMCglmm)
S3method(recover_data,aovlist)
S3method(recover_data,averaging)
S3method(recover_data,betareg)
S3method(recover_data,brmsfit)
S3method(recover_data,call)
S3method(recover_data,carbayes)
S3method(recover_data,clm)
S3method(recover_data,clmm)
S3method(recover_data,coxme)
S3method(recover_data,coxph)
S3method(recover_data,default)
S3method(recover_data,gam)
S3method(recover_data,gamlss)
S3method(recover_data,gamm)
S3method(recover_data,gee)
S3method(recover_data,geeglm)
S3method(recover_data,geese)
S3method(recover_data,glmgee)
S3method(recover_data,gls)
S3method(recover_data,gnls)
S3method(recover_data,hurdle)
S3method(recover_data,lm)
S3method(recover_data,lme)
S3method(recover_data,lqm)
S3method(recover_data,lqmm)
S3method(recover_data,manova)
S3method(recover_data,mblogit)
S3method(recover_data,mcmc)
S3method(recover_data,mcmc.list)
S3method(recover_data,merMod)
S3method(recover_data,mira)
S3method(recover_data,mmer)
S3method(recover_data,multinom)
S3method(recover_data,nlme)
S3method(recover_data,nls)
S3method(recover_data,polr)
S3method(recover_data,qdrg)
S3method(recover_data,rms)
S3method(recover_data,rq)
S3method(recover_data,rqs)
S3method(recover_data,stanreg)
S3method(recover_data,survreg)
S3method(recover_data,svyglm)
S3method(recover_data,svyolr)
S3method(recover_data,zeroinfl)
S3method(str,emmGrid)
S3method(str,emm_list)
S3method(subset,emmGrid)
S3method(summary,emmGrid)
S3method(summary,emm_list)
S3method(summary,glht_list)
S3method(summary,summary_eml)
S3method(summary,summary_emm)
S3method(tail,emmGrid)
S3method(test,emmGrid)
S3method(test,emm_list)
S3method(update,emmGrid)
S3method(update,emm_list)
S3method(update,summary_emm)
S3method(vcov,emmGrid)
S3method(vcov,emm_list)
S3method(vcov,glht_list)
S3method(vcov,qdrg)
S3method(weights,emmGrid)
export(.all.vars)
export(.aovlist.dffun)
export(.cmpMM)
export(.combine.terms)
export(.diag)
export(.emm_register)
export(.emm_vignette)
export(.get.excl)
export(.get.offset)
export(.hurdle.support)
export(.my.vcov)
export(.num.key)
export(.std.link.labels)
export(.zi.support)
export(add_grouping)
export(add_submodels)
export(as.emmGrid)
export(as.emm_list)
export(as.glht)
export(as.mcmc.emmGrid)
export(as.mcmc.emm_list)
export(as.mcmc.list.emmGrid)
export(as.mcmc.list.emm_list)
export(comb_facs)
export(consec.emmc)
export(contrast)
export(del.eff.emmc)
export(dunnett.emmc)
export(eff.emmc)
export(eff_size)
export(emm)
export(emm_basis)
export(emm_defaults)
export(emm_example)
export(emm_options)
export(emmeans)
export(emmip)
export(emmip_ggplot)
export(emmip_lattice)
export(emmobj)
export(emtrends)
export(force_regular)
export(get.lsm.option)
export(get_emm_option)
export(helmert.emmc)
export(hpd.summary)
export(identity.emmc)
export(inverse)
export(joint_tests)
export(linfct)
export(lsm)
export(lsm.options)
export(lsmeans)
export(lsmip)
export(lsmobj)
export(lstrends)
export(make.meanint)
export(make.symmint)
export(make.tran)
export(mean_chg.emmc)
export(meanint)
export(mvcontrast)
export(mvregrid)
export(nrmlz.emmc)
export(opoly.emmc)
export(pairwise.emmc)
export(permute_levels)
export(poly.emmc)
export(pwpm)
export(pwpp)
export(qdrg)
export(recover_data)
export(ref_grid)
export(regrid)
export(revpairwise.emmc)
export(split_fac)
export(symmint)
export(test)
export(trt.vs.ctrl.emmc)
export(trt.vs.ctrl1.emmc)
export(trt.vs.ctrlk.emmc)
export(tukey.emmc)
export(with_emm_options)
export(wtcon.emmc)
exportClasses(emmGrid)
import(estimability)
import(mvtnorm)
import(stats)
importFrom(grDevices,col2rgb)
importFrom(grDevices,hcl)
importFrom(grDevices,palette.colors)
importFrom(grDevices,rgb)
importFrom(grDevices,rgb2hsv)
importFrom(graphics,pairs)
importFrom(graphics,plot)
importFrom(methods,"slot<-")
importFrom(methods,as)
importFrom(methods,getClass)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,show)
importFrom(methods,slot)
importFrom(methods,slotNames)
importFrom(rlang,.data)
importFrom(stats,coef)
importFrom(stats,weights)
importFrom(utils,getS3method)
importFrom(utils,hasName)
importFrom(utils,head)
importFrom(utils,installed.packages)
importFrom(utils,methods)
importFrom(utils,str)
importFrom(utils,tail)

R package **emmeans**: Estimated marginal means <img src="man/figures/logo.png" align="right" />
====
## Website
https://rvlenth.github.io/emmeans/

## Features
Estimated marginal means (EMMs, also known as least-squares means in the
context of traditional regression models) are derived by using a model to make
predictions over a regular grid of predictor combinations (called a *reference
grid*). These predictions may possibly be averaged (typically with equal
weights) over one or more of the predictors. Such marginally-averaged
predictions are useful for describing the results of fitting a model,
particularly in presenting the effects of factors. The **emmeans** package can
easily produce these results, as well as various graphs of them
(interaction-style plots and side-by-side intervals).


  * Estimation and testing of pairwise comparisons of EMMs, and several other
    types of contrasts, are provided.
    
  * In rank-deficient models, the estimability of predictions is checked,
    to avoid outputting results that are not uniquely defined.
    
  * For models where continuous predictors interact with factors, the package's
    `emtrends` function works in terms of a reference grid of predicted slopes of
    trend lines for each factor combination.
    
  * Vignettes are provided on various aspects of EMMs and using the package. 
    See the [CRAN page](https://CRAN.R-project.org/package=emmeans).
    
  * We try to provide flexible (but pretty basic) graphics support for
    the `emmGrid` objects produced by the package. Also, support is provided
    for nested fixed effects.
    
  * Response transformations and link functions are supported via a `type`
    argument in many functions (e.g., `type = "response"` to back-transform
    results to the response scale). Also, a `regrid()` function is provided
    to reconstruct the object on any transformed scale that the user wishes.

  * Two-way support of the `glht` function in the **multcomp** package.
  

## Model support


  * The package incorporates support for many types of models, including 
    standard models fitted using `lm`, `glm`, and relatives, 
    various mixed models, GEEs, survival models, count models,
    ordinal responses, zero-inflated models, and others. Provisions for
    some models include special modes for accessing different types of 
    predictions; for example, with zero-inflated models, one may opt for
    the estimated response including zeros, just the linear predictor, 
    or the zero model.
    For details, see
    [`vignette("models", package = "emmeans")`](https://CRAN.R-project.org/package=emmeans/vignettes/models.html)
    
  * Various Bayesian models (**carBayes**, **MCMCglmm**, **MCMCpack**) are
    supported by way of creating a posterior sample of least-squares means or
    contrasts thereof, which may then be examined using tools such as in the
    **coda** package.
    
  * Package developers are encouraged to incorporate **emmeans** support for
  their models by writing `recover_data` and `emm_basis` methods.
    See [`vignette("extending", package = "emmeans")`](https://CRAN.R-project.org/package=emmeans/vignettes/xtending.html)
    

## Versions and installation


  * **CRAN** The latest CRAN version may be found at [https://CRAN.R-project.org/package=emmeans](https://CRAN.R-project.org/package=emmeans).
    Also at that site, formatted versions of this package's vignettes 
    may be viewed.

  * **GitHub** To install the latest development version from GitHub, 
    install the newest version of the **remotes** 
    package. If you are a Windows user, you should also first download and
      install the latest version of
      [`Rtools`](https://cran.r-project.org/bin/windows/Rtools/).
      Then run
    
```r
remotes::install_github("rvlenth/emmeans", dependencies = TRUE, build_vignettes = TRUE)
```
Omitting the `build_vignettes` argument can save some time if you don't want the vignettes. 
They can always be found [for the latest CRAN version](https://cran.r-project.org/package=emmeans) or 
-- perhaps more up-to-date -- the [emmeans site](https://rvlenth.github.io/emmeans/).

*Note:* 

For the latest release notes on this development version, see the 
[NEWS file](https://github.com/rvlenth/emmeans/blob/master/NEWS.md)

## Rounding
For its summary output, **emmeans** uses an optimal-digits algorithm that rounds
results to about the number of displayed digits that are useful, relative to estimates' 
confidence limits. This avoids cluttering the output, but it is unlike other R
results, which are typically less round. If this is annoying to you, there is an 
option (`opt.digits = FALSE`) that disables the optimal-digits routine. Note that
the values actually stored in `emm_summary` objects and such are *not* rounded.

## "Tidiness" can be dangerous
I see more and more users who are in a terrible hurry to get results. They develop
a "workflow" where they plan-out several steps at once and pipe them
together. That's useful when you don't have to think about what happens in
those steps; but *when you're doing the kinds of post hoc analyses offered by*
**emmeans**, *you should be thinking!*
Most functions in the **emmeans** package yield results that are accompanied by 
annotations such as transformations involved, P-value adjustments
made, the families for those adjustments, etc. If you just pipe the results
into some more code, *you never see those annotations*. 

**Please slow down!** Look at the actual results from each **emmeans** package
function without any post-processing -- None. That way, you'll see the annotated
summaries. Statistics is pretty hard stuff. Don't make it harder by blindfolding
yourself.


## Supersession plan
[Julia Piaskowski](https://github.com/jpiaskowski) is preparing to take over as maintainer.

##### *Note: **emmeans** supersedes the package **lsmeans**. The latter is just a front end for **emmeans**, and in fact, the `lsmeans()` function itself is part of **emmeans**.*

warp.lm <- lm(breaks ~ wool*tension, data = warpbreaks)
# Using 'emm'
summary(glht(warp.lm, emm(pairwise ~ tension | wool)))
# Combine the means and the comparisons into one family
summary(glht(warp.lm, emm(pairwise ~ tension | wool, 
                          which = 1:2, by = "wool")))
                          
# Same as first example, but use an existing 'emmeans' result
warp.emm <- emmeans(warp.lm, ~ tension | wool)
summary(as.glht(pairs(warp.emm)))
# Same contrasts, but treat as one family
summary(as.glht(pairs(warp.emm), by = NULL))

require("compositions")
data(AnimalVegetation)
AV <- as.data.frame(AnimalVegetation)
AV$region <- factor(ifelse(AV$regA == 1, "A", "B"))

AVmod <- lm(ilr(cbind(disc,spick,din,spin)) ~ region, data = AV)

AVRG <- ref_grid(AVmod)   |> suppressWarnings()
AVRG

lvls <- c("disc","spick","din","spin")
mvregrid(AVRG, newname = "comp", newlevs = lvls) |>
    confint(by = "region")

# The ref grid we'd have gotten if the 'clr' transform had been used...
mvregrid(AVRG, transform = "clr", newname = "comp", newlevs = lvls)

### Use of fcn argument...
arr.lm <- lm(as.matrix(USArrests) ~ state.region)
arr.emm <- emmeans(arr.lm, ~ crime | state.region, mult.name = "crime")

PC <- princomp(USArrests, cor = TRUE)
std.and.rot <- function(x, pc, ...) {
    z <- sweep(sweep(x, 2, pc$center, "-"), 2, pc$scale, "/")
    z %*% pc$loadings
}

# EMMs obtained if model had been fitted with PC$scores as the response ...
mvregrid(arr.emm, newname = "prin.comp", fcn = std.and.rot, pc = PC)

## ----echo = FALSE, results = "hide", message = FALSE--------------------------
require("emmeans")
require("ggplot2")
options(show.signif.stars = FALSE) 
knitr::opts_chunk$set(fig.width = 4.5, class.output = "ro") 

## -----------------------------------------------------------------------------
nutr.lm <- lm(gain ~ (age + group + race)^2, data = nutrition) 
car::Anova(nutr.lm)

## -----------------------------------------------------------------------------
emmeans(nutr.lm, ~ group * race, calc = c(n = ".wgt."))

## -----------------------------------------------------------------------------
with(nutrition, table(race, age))

## -----------------------------------------------------------------------------
emmeans(nutr.lm, pairwise ~ group | race, at = list(age = "3")) |>
    summary(by = NULL)

## -----------------------------------------------------------------------------
framing <- mediation::framing 
levels(framing$educ) <- c("NA","Ref","< HS", "HS", "> HS","Coll +") 
framing.glm <- glm(cong_mesg ~ age + income + educ + emo + gender * factor(treat), 
    family = binomial, data = framing)

## ----fig.alt = "Two panels, each showing a decreasing trend with educ except they increase again with educ=Coll+. In the male panel, the curve for treat 1 is higher than that for treat 2, while the reverse is true in the female panel"----
emmip(framing.glm, treat ~ educ | gender, type = "response") 

## ----fig.alt = "Similar to previous figure but now the curves flatten out with >HS about the same as Coll+. For the maile panel, we still have the trace for treat 1 higher than for treat 0; but in the female panel, they are about the same as each other, and a bit lower than treat 0 for males. To see these results in numerical form, call emmeans() with the same arguments *except* replace the second argument with ~treat*educ|gender"----
emmip(framing.glm, treat ~ educ | gender, type = "response", 
    cov.reduce = emo ~ treat*gender + age + educ + income)

## ----eval = FALSE-------------------------------------------------------------
# emo.adj <- resid(lm(emo ~ treat*gender + age + educ + income, data = framing))

## ----eval = FALSE-------------------------------------------------------------
# emmeans(..., cov.reduce = list(x1 ~ trt, x2 ~ trt + x1, x3 ~ trt + x1 + x2))

## ----eval = FALSE-------------------------------------------------------------
# emmeans(model, "A", weights = "outer")
# emmeans(model, c("A", "B"), weights = "prop") |>  emmeans(weights = "prop")

## ----message = FALSE----------------------------------------------------------
sapply(c("equal", "prop", "outer", "cells", "flat"), \(w)
    emmeans(nutr.lm, ~ race, weights = w) |> predict())

## -----------------------------------------------------------------------------
mtcars.lm <- lm(mpg ~ factor(cyl)*am + disp + hp + drat + log(wt) + vs + 
                  factor(gear) + factor(carb), data = mtcars)

## -----------------------------------------------------------------------------
rg.usual <- ref_grid(mtcars.lm)
rg.usual
nrow(linfct(rg.usual))
rg.nuis = ref_grid(mtcars.lm, non.nuisance = "cyl")
rg.nuis
nrow(linfct(rg.nuis))

## -----------------------------------------------------------------------------
emmeans(rg.usual, ~ cyl * am)
emmeans(rg.nuis, ~ cyl * am)

## -----------------------------------------------------------------------------
predict(emmeans(mtcars.lm, ~ cyl * am, non.nuis = c("cyl", "am"), 
                wt.nuis = "prop"))
predict(emmeans(mtcars.lm, ~ cyl * am, weights = "outer"))

## -----------------------------------------------------------------------------
emmeans(mtcars.lm, ~ gear | am, non.nuis = quote(all.vars(specs)))

## ----error = TRUE-------------------------------------------------------------
try({
ref_grid(mtcars.lm, rg.limit = 200)
})

## -----------------------------------------------------------------------------
neuralgia.glm <- glm(Pain ~ Sex + Age + Duration + Treatment,
                     data = neuralgia, family = binomial)
emmeans(neuralgia.glm, "Treatment", counterfactuals = "Treatment",
        vcov. = sandwich::vcovHC)

## -----------------------------------------------------------------------------
emmeans(neuralgia.glm, "Treatment", weights = "prop", type = "response")

## -----------------------------------------------------------------------------
emmeans(nutr.lm, pairwise ~ group | race, submodel = ~ age + group*race) |> 
        summary(by = NULL)

## -----------------------------------------------------------------------------
emmeans(nutr.lm, ~ group * race, submodel = "minimal")

## -----------------------------------------------------------------------------
joint_tests(nutr.lm, submodel = "type2")

## -----------------------------------------------------------------------------
cows <- data.frame (
    route = factor(rep(c("injection", "oral"), c(5, 9))),
    drug = factor(rep(c("Bovineumab", "Charloisazepam", 
              "Angustatin", "Herefordmycin", "Mollycoddle"), c(3,2,  4,2,3))),
    resp = c(34, 35, 34,   44, 43,      36, 33, 36, 32,   26, 25,   25, 24, 24)
)
cows.lm <- lm(resp ~ route + drug, data = cows)

## ----message = FALSE----------------------------------------------------------
cows.rg <- ref_grid(cows.lm)
cows.rg

## -----------------------------------------------------------------------------
route.emm <- emmeans(cows.rg, "route")
route.emm

## -----------------------------------------------------------------------------
drug.emm <- emmeans(cows.rg, "drug")
drug.emm

## -----------------------------------------------------------------------------
pairs(route.emm, reverse = TRUE)

pairs(drug.emm, by = "route", reverse = TRUE)

## ----fig.width = 5.5, fig.alt = "A panel for each route. This interaction plot has a lot of empty space because all 5 drugs are represented in each panel, and the x axis labels all overlap"----
emmip(cows.rg, ~ drug | route)

## ----fig.width = 5.5, fig.alt = "This plot shows the same means as the previous one, but each panel shows only the drugs that are nested in each route"----
require(ggplot2)
emmip(cows.rg, ~ drug, abbr.len = 6) + 
  facet_wrap(~ route, scales = "free_x", space = "free_x")

## ----fig.height = 2.5, fig.width = 5.5, fig.alt = "side-by-side CIs and PIs for drugs in each route. Again, with free_y scaling, each panel contains only the drugs involved"----
plot(drug.emm, by = "route", PIs = TRUE) + 
    facet_wrap(~ route, scales = "free_y", space = "free_y")

# Use a stored example having a posterior sample
# Model is based on the data in lme4::cbpp
post <- readRDS(system.file("extdata", "cbpplist", package = "emmeans"))$post.beta

rg2 <- qdrg(~ size + period, data = lme4::cbpp, mcmc = post, link = "logit")

summary(rg2, type = "response")

### A saved reference grid for a mixed logistic model (see lme4::cbpp)
cbpp.rg <- do.call(emmobj, 
  readRDS(system.file("extdata", "cbpplist", package = "emmeans")))

# Predictive distribution for herds of size 20
# (perhaps a bias adjustment should be applied; see "sophisticated" vignette)
pred.incidence <- coda::as.mcmc(regrid(cbpp.rg), likelihood = "binomial", trials = 20)
summary(pred.incidence)

pigs.lm <- lm(log(conc) ~ source + factor(percent), data = pigs)
pigs.emm <- emmeans(pigs.lm, "percent", type = "response")
multcomp::cld(pigs.emm, alpha = 0.10, Letters = LETTERS)

multcomp::cld(pigs.emm, alpha = 0.10, signif.sets = TRUE)

multcomp::cld(pigs.emm, delta = log(1.25), adjust = "sidak")

## ----echo = FALSE, results = "hide", message = FALSE--------------------------
require("emmeans")
knitr::opts_chunk$set(fig.width = 4.5, class.output = "ro")

## -----------------------------------------------------------------------------
pigs.lm <- lm(log(conc) ~ source + factor(percent), data = pigs)
pigs.emm.s <- emmeans(pigs.lm, "source")
pairs(pigs.emm.s)

## -----------------------------------------------------------------------------
pwpm(pigs.emm.s)

## -----------------------------------------------------------------------------
pwpm(pigs.emm.s, means = FALSE, flip = TRUE,     # args for pwpm()
     reverse = TRUE,                             # args for pairs()
     side = ">", delta = 0.05, adjust = "none")  # args for test()

## -----------------------------------------------------------------------------
eff_size(pigs.emm.s, sigma = sigma(pigs.lm), edf = 23)

## -----------------------------------------------------------------------------
eff_size(pigs.emm.s, sigma = sigma(pigs.lm), edf = Inf)

## ----eval = FALSE-------------------------------------------------------------
# eff_size(pairs(pigs.emm.s), sigma = sigma(pigs.lm), edf = 23, method = "identity")

## ----fig.height = 1.5, fig.alt = "side-by-side CIs with comparison arrows added"----
plot(pigs.emm.s, comparisons = TRUE)

## ----fig.alt = "Pairwise P-value plot that shows, for each pair of means, a vertical line segment whose horizontal position is the Tukey-adjusted P-value for that comparison. The endpoints of the line segments align with the vertical scale showing the soy levels and their means. This particular plot shows that skim-fish and soy-fish are highyly significant, while skim-soy has a P-value just over 0.05"----
pwpp(pigs.emm.s)

## ----fig.width = 9, fig.alt = "pwpp for all 78 pairwise comparisons of cell means. It is pretty hard to digest due to its complexity. To see all this information in text form, call pwpm(pigs.cells)"----
pigs.lmint <- lm(log(conc) ~ source * factor(percent), data = pigs)
pigs.cells <- emmeans(pigs.lmint, ~ source * percent)
pwpp(pigs.cells, type = "response")

## ----fig.width = 6, fig.alt = "pwpp presented in separate panels for each source. Each panel has just 6 P-value bars, making it much less cluttered than the previous figure"----
pwpp(pigs.cells, by = "source", type = "response")

## -----------------------------------------------------------------------------
coef(pairs(pigs.emm.s))

## -----------------------------------------------------------------------------
pigs.emm.p <- emmeans(pigs.lm, "percent")
ply <- contrast(pigs.emm.p, "poly")
ply

coef(ply)

## -----------------------------------------------------------------------------
nrmlz.emmc(1:4, family = "helmert")
contrast(pigs.emm.s, "nrmlz", family = "pairwise")

## -----------------------------------------------------------------------------
org.aov <- aov(sales1 ~ day + Error(store), data = oranges,
               contrasts = list(day = "contr.sum"))
org.emml <- emmeans(org.aov, consec ~ day)
org.emml

## -----------------------------------------------------------------------------
skip_comp.emmc <- function(levels, skip = 1, reverse = FALSE, ...) {
    if((k <- length(levels)) < skip + 1)
        stop("Need at least ", skip + 1, " levels")
    coef <- data.frame()
    coef <- as.data.frame(lapply(seq_len(k - skip - 1), function(i) {
        sgn <- ifelse(reverse, -1, 1)
        sgn * c(rep(0, i - 1), 1, rep(0, skip), -1, rep(0, k - i - skip - 1))
    }))
    names(coef) <- sapply(coef, function(x)
        paste(which(x == 1), "-", which(x == -1)))
    attr(coef, "adjust") = "fdr"   # default adjustment method
    coef
}

## -----------------------------------------------------------------------------
skip_comp.emmc(1:5)

skip_comp.emmc(1:5, skip = 0, reverse = TRUE)

## -----------------------------------------------------------------------------
contrast(org.emml[[1]], "skip_comp", skip = 2, reverse = TRUE)

## -----------------------------------------------------------------------------
LF <- contrast(pigs.emm.s, 
               list(lambda1 = c(1, 2, 0), lambda2 = c(0, 3, -2)),
               offset = c(-7, 1))
confint(LF, adjust = "bonferroni")

## -----------------------------------------------------------------------------
pairs(pigs.emm.s, type = "lp")

pairs(pigs.emm.s, type = "response")

# Create an emmGrid object from a system file
cbpp.rg <- do.call(emmobj, 
                   readRDS(system.file("extdata", "cbpplist", package = "emmeans")))
cbpp.emm <- emmeans(cbpp.rg, "period")

hpd.summary(cbpp.emm)   # or just summary(cbpp.emm) as it gets redirected

# Equivalence test for any two-fold difference
summary(pairs(cbpp.emm), type = "response", delta = log(2))

# Post hoc analysis of a "biglm" object -- not supported by emmeans
bigmod <- biglm(log(conc) ~ source + factor(percent), data = pigs)

rg1 <- qdrg(log(conc) ~ source + factor(percent), data = pigs, 
    coef = coef(bigmod), vcov = vcov(bigmod), df = bigmod$df.residual)

emmeans(rg1, "source", type = "response")

## But in this particular case, we could have done it the easy way:
##     rg1 <- qdrg(object = bigmod)

data(AnimalVegetation, package = "compositions")
AV <- as.data.frame(AnimalVegetation)

AVmod <- lm(compositions::ilr(cbind(disc,spick,din,spin)) ~ regA, data = AV)

AVRG <- ref_grid(AVmod)   |> suppressWarnings()
AVRG

mvregrid(AVRG, newname = "comp", 
         newlevs = c("disc","spick","din","spin"),
         fcn = "ilrInv") |>
    confint(by = "regA")
     

##############################################################################
#    Copyright (c) 2012-2019 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

# sommer package support

#' @exportS3Method recover_data mmer
recover_data.mmer = function(object, data, ...) {
    if (is.null(data))
        data = object$data
    fcall = call("mmer", formula = object$call$fixed, data = data)
    emmeans::recover_data(fcall, delete.response(terms(object$call$fixed)), 
                          object$call$na.method.V, ...)
}

#' @exportS3Method emm_basis mmer         
emm_basis.mmer = function(object, trms, xlev, grid, ...) {
    cf = object$Beta
    bhat = cf$Estimate
    m = suppressWarnings(model.frame(trms, grid, na.action = na.pass, xlev = xlev))
    # if we can get contrasts from the object, fix next line
    X = model.matrix(trms, m, contrasts.arg = NULL)
    V = .my.vcov(object, vcov. = function(., ...) .$VarBeta)
    
    if ((k <- length(bhat)) < ncol(X)) {  # we have rank deficiencies
        QR = qr(model.matrix(trms, object$data, contrasts.arg = NULL))
        bhat = rep(NA, ncol(X))
        bhat[QR$pivot[1:k]] = cf$Estimate
        nbasis = estimability::nonest.basis(QR)
    }
    else
        nbasis = estimability::all.estble 
    misc = list()
    # soup-up following if (1) glms allowed or (2) d.f. available
    dfargs = list(df = object$df.residual)
    dffun = function(k, dfargs) Inf
    bas = list(X = X, bhat = bhat, nbasis = nbasis, V = V, 
               dffun = dffun, dfargs = dfargs, misc = misc)
    # check for multiv resp
    k = length(levels(cf$Trait))
    if (k > 1) {
        bas$misc$ylevs = list(Trait = levels(cf$Trait))
        bas$X = kronecker(diag(rep(1, k)), bas$X)
        # reorder coefs to go one trait at a time
        ord = as.integer(matrix(seq_along(bas$bhat), ncol = k, byrow = TRUE))
        bas$bhat = bas$bhat[ord]
        bas$V = bas$V[ord, ord, drop = FALSE]
    }
    bas
}

##############################################################################
#    Copyright (c) 2012-2019 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

### functions to implement different families of contrasts
### All return a matrix or data frame whose columns are the desired contrasts coefs
### with appropriate row and column names
### Also they have two attributes: 
###   "desc" is an expanded description of the family,
###   "adjust" is the default multiplicity adjustment (used if adjust="auto" in emmeans)

#' Contrast families
#'
#' Functions with an extension of \code{.emmc} provide for named contrast
#' families. One of the standard ones documented here may be used, or the user
#' may write such a function.
#'
#' Each standard contrast family has a default multiple-testing adjustment as
#' noted below. These adjustments are often only approximate; for a more
#' exacting adjustment, use the interfaces provided to \code{glht} in the
#' \pkg{multcomp} package.
#'
#' \code{pairwise.emmc}, \code{revpairwise.emmc}, and \code{tukey.emmc} generate
#' contrasts for all pairwise comparisons among estimated marginal means at the
#' levels in levs. The distinction is in which direction they are subtracted.
#' For factor levels A, B, C, D, \code{pairwise.emmc} generates the comparisons
#' A-B, A-C, A-D, B-C, B-D, and C-D, whereas \code{revpairwise.emmc} generates
#' B-A, C-A, C-B, D-A, D-B, and D-C. \code{tukey.emmc} invokes
#' \code{pairwise.emmc} or \code{revpairwise.emmc} depending on \code{reverse}.
#' The default multiplicity adjustment method is \code{"tukey"}, which is only
#' approximate when the standard errors differ.
#'
#' \code{poly.emmc} and \code{opoly.emmc} generate orthogonal polynomial contrasts.
#' \code{poly.emmc} uses equally-spaced factor levels; coefficients are derived from the
#' \code{\link[stats]{poly}} function, but an \emph{ad hoc} algorithm is used to
#' scale them to integer coefficients that are (usually) the same as in
#' published tables of orthogonal polynomial contrasts. On the other hand, 
#' \code{opoly.emmc}'s coefficients are always normalized (sum of squares equals 1),
#' but allows the user to choose alternate reference points in \code{scores}, as in the
#' \code{\link{contr.poly}} function.
#' In both cases, the default multiplicity adjustment method is \code{"none"}.
#'
#' \code{trt.vs.ctrl.emmc} and its relatives generate contrasts for comparing
#' one level (or the average over specified levels) with each of the other
#' levels. The argument \code{ref} should be the index(es) (not the labels) of
#' the reference level(s). \code{trt.vs.ctrl1.emmc} is the same as
#' \code{trt.vs.ctrl.emmc} with a reference value of 1, and
#' \code{trt.vs.ctrlk.emmc} is the same as \code{trt.vs.ctrl} with a reference
#' value of \code{length(levs)}. \code{dunnett.emmc} is the same as
#' \code{trt.vs.ctrl}. The default multiplicity adjustment method is
#' \code{"dunnettx"}, a close approximation to the Dunnett adjustment.
#' \emph{Note} in all of these functions, it is illegal to have any overlap
#' between the \code{ref} levels and the \code{exclude} levels. If any is found,
#' an error is thrown.
#'
#' \code{consec.emmc} and \code{mean_chg.emmc} are useful for contrasting
#' treatments that occur in sequence. For a factor with levels A, B, C, D,
#' \code{consec.emmc} generates the comparisons B-A, C-B, and D-C, while
#' \code{mean_chg.emmc} generates the contrasts (B+C+D)/3 - A, (C+D)/2 -
#' (A+B)/2, and D - (A+B+C)/3. With \code{reverse = TRUE}, these differences go
#' in the opposite direction.
#'
#' \code{eff.emmc} and \code{del.eff.emmc} generate contrasts that compare each
#' level with the average over all levels (in \code{eff.emmc}) or over all other
#' levels (in \code{del.eff.emmc}). These differ only in how they are scaled.
#' For a set of k EMMs, \code{del.eff.emmc} gives weight 1 to one EMM and weight
#' -1/(k-1) to the others, while \code{eff.emmc} gives weights (k-1)/k and -1/k
#' respectively, as in subtracting the overall EMM from each EMM. The default
#' multiplicity adjustment method is \code{"fdr"}. This is a Bonferroni-based
#' method and is slightly conservative; see \code{\link[stats]{p.adjust}}.
#' 
#' \code{nrmlz.emmc} is a wrapper that can be used with any other \code{.emmc} 
#' function that will normalize the contrast coefficients so that the sum of its 
#' squares equals 1. Just provide the root name of the function in \code{family},
#' along with any other arguments to pass to it.
#' 
#' \code{wtcon.emmc} generates weighted contrasts based on the function 
#' \code{\link[multcomp]{contrMat}} function in the \pkg{multcomp} package,
#' using the provided \code{type} as documented there. If the user provides 
#' \code{wts}, they have to conform to the length of \code{levs}; however,
#' if \code{wts} is not specified, \code{contrast} will fill-in what is
#' required, and usually this is safer (especially when \code{by != NULL}
#' which usually means that the weights are different in each \code{by} group).
#'
#' \code{identity.emmc} simply returns the identity matrix (as a data frame),
#' minus any columns specified in \code{exclude}. It is potentially useful in
#' cases where a contrast function must be specified, but none is desired.
#'
#' @rdname emmc-functions
#' @aliases emmc-functions
#' @param levs Vector of factor levels
#' @param exclude integer vector of indices, or character vector of levels to
#'   exclude from consideration. These levels will receive weight 0 in all
#'   contrasts. Character levels must exactly match elements of \code{levs}.
#' @param include integer or character vector of levels to include (the
#'   complement of \code{exclude}). An error will result if the user specifies
#'   both \code{exclude} and \code{include}.
#' @param ... Additional arguments, passed to related methods as appropriate
#'
#' @return A data.frame, each column containing contrast coefficients for levs.
#'   The "desc" attribute is used to label the results in emmeans, and the
#'   "adjust" attribute gives the default adjustment method for multiplicity.
#'
#' @note Caution is needed in cases where the user alters the ordering of
#'   results (e.g., using the the \code{"[...]"} operator), because the
#'   contrasts generated depend on the order of the levels provided. For
#'   example, suppose \code{trt.vs.ctrl1} contrasts are applied to two \code{by}
#'   groups with levels ordered (Ctrl, T1, T2) and (T1, T2, Ctrl) respectively,
#'   then the contrasts generated will be for (T1 - Ctrl, T2 - Ctrl) in the
#'   first group and (T2 - T1, Ctrl - T1) in the second group, because the first
#'   level in each group is used as the reference level.
#'
#' @examples
#' warp.lm <- lm(breaks ~ wool*tension, data = warpbreaks)
#' warp.emm <- emmeans(warp.lm, ~ tension | wool)
#' contrast(warp.emm, "poly")
#' contrast(warp.emm, "trt.vs.ctrl", ref = "M")
#' \dontrun{
#' ## Same when enhanced labeling is used:
#' contrast(warp.emm, "trt.vs.ctrl", 
#'          enhance.levels = "tension", ref = "tensionM")}
#' 
#' # Comparisons with grand mean
#' contrast(warp.emm, "eff")
#' # Comparisons with a weighted grand mean
#' contrast(warp.emm, "eff", wts = c(2, 5, 3))
#'
#' # Compare only low and high tensions
#' # Note pairs(emm, ...) calls contrast(emm, "pairwise", ...)
#' pairs(warp.emm, exclude = 2)
#' # (same results using exclude = "M" or include = c("L","H") or include = c(1,3))
#'
#'# Same contrasts as above but with normalized contrast coefficients
#'contrast(warp.emm, "nrmlz", family = "pairwise", include = c(1, 3))
#'
#' ### Setting up a custom contrast function
#' revhelmert.emmc <- function(levs, ...) {
#'     M <- as.data.frame(contr.helmert(levs)[rev(seq_along(levs)), ])
#'     names(M) <- paste(rev(levs)[-1],"vs later")
#'     attr(M, "desc") <- "reverse Helmert contrasts"
#'     M
#' }
#' contrast(warp.emm, "revhelmert")
#' 
#' \dontrun{
#' # See what is used for polynomial contrasts with 6 levels
#' emmeans:::poly.emmc(1:6)
#' }
#' @name contrast-methods
#' @export
pairwise.emmc = function(levs, exclude = integer(0), include, ...) {
    exclude = .get.excl(levs, exclude, include)
    k = length(levs)
    M = data.frame(levs=levs)
    for (i in setdiff(seq_len(k-1), exclude)) {
        for (j in setdiff(i + seq_len(k-i), exclude)) { ###for (j in (i+1):k) {
            con = rep(0,k)
            con[i] = 1
            con[j] = -1
            nm = paste(levs[i], levs[j], sep = " - ")
            M[[nm]] = con
        }
    }
    row.names(M) = levs
    M = M[-1]
    attr(M, "desc") = "pairwise differences"
    attr(M, "adjust") = "tukey"
    attr(M, "type") = "pairs"
    attr(M, "famSize") = k - length(exclude)
    if(length(exclude) > 0)
        attr(M, "famSize") = length(levs) - length(exclude)
    M
}

# all pairwise trt[j] - trt[i], j > i
#' @rdname emmc-functions
#' @export
revpairwise.emmc = function(levs, exclude = integer(0), include, ...) {
    exclude = .get.excl(levs, exclude, include)
    k = length(levs)
    M = data.frame(levs=levs)
    for (i in setdiff(1 + seq_len(k - 1), exclude)) {
        for (j in setdiff(seq_len(i-1), exclude)) {
            con = rep(0,k)
            con[i] = 1
            con[j] = -1
            nm = paste(levs[i], levs[j], sep = " - ")
            M[[nm]] = con
        }
    }
    row.names(M) = levs
    M = M[-1]
    attr(M, "desc") = "pairwise differences"
    attr(M, "adjust") = "tukey"
    attr(M, "type") = "pairs"
    if(length(exclude) > 0)
        attr(M, "famSize") = length(levs) - length(exclude)
    M
}

# pseudonym
#' @rdname emmc-functions
#' @param reverse Logical value to determine the direction of comparisons
#' @export
tukey.emmc = function(levs, reverse = FALSE, ...) {
    if (reverse)
        revpairwise.emmc(levs, ...)
    else
        pairwise.emmc(levs, ...)
}

# Poly contrasts - scaled w/ integer levels like most tables
# ad hoc scaling works for up to 13 levels
#' @rdname emmc-functions
#' @param max.degree Integer specifying the maximum degree of polynomial contrasts
#' @export
poly.emmc = function(levs, max.degree = min(6, k-1), ...) {
    nm = c("linear", "quadratic", "cubic", "quartic", paste("degree",5:20))
    k = length(levs)
    M = as.data.frame(poly(seq_len(k), min(20,max.degree)))
    for (j in seq_len(ncol(M))) {
        con = M[, j]
        pos = which(con > .01)
        con = con / min(con[pos])
        z = max(abs(con - round(con)))
        while (z > .05) {
            con = con / z
            z = max(abs(con - round(con)))
        }
        M[ ,j] = round(con)
    }
    row.names(M) = levs
    names(M) = nm[seq_len(ncol(M))]
    attr(M, "desc") = "polynomial contrasts"
    attr(M, "adjust") = "none"
    M
}

# Orthonormal poly contrasts - not rescaled
#' @rdname emmc-functions
#' @param scores Set of values of length \code{length(levs)} over which
#'   orthogonal polynomials are computed. The default scores are the 
#'   consecutive integers \code{seq_along(levs)}.
#'   (If \code{exclude} or \code{include}
#'   are used, the default scores are subsetted accordingly; and if \code{scores} 
#'   is specified, its length must be the same as that of the subsetted \code{levs}).
#' @export
opoly.emmc = function(levs, max.degree = min(6, k-1), scores, 
                      exclude = integer(0), include, ...) {
    if (MS <- missing(scores))
        scores = seq_along(levs)
    exclude = .get.excl(levs, exclude, include)
    if(length(exclude) > 0) {
        oldlevs = levs
        if (MS)
            scores = scores[-exclude]
        levs = levs[-exclude]
    }
    nm = c("linear", "quadratic", "cubic", "quartic", paste("degree",5:20))
    k = length(levs)
    if (length(scores) != k)
        stop("In opoly.emmc: Lengths of 'scores' must equal ", length(levs), call. = FALSE)
    M = contr.poly(k, contrasts = TRUE, scores = scores, sparse = FALSE)
    if(length(exclude) > 0) {
        MM = M
        levs = oldlevs
        M = matrix(0, nrow = length(oldlevs), ncol = ncol(MM))
        M[-exclude, ] = MM
    }
    M = as.data.frame(M[, seq_len(min(k-1, max.degree)), drop = FALSE])
    row.names(M) = levs
    names(M) = nm[seq_len(ncol(M))]
    attr(M, "desc") = "normalized polynomial contrasts"
    attr(M, "adjust") = "none"
    M
}


# All comparisons with a control; ref = index of control group
# New version -- allows more than one control group (ref is a vector)
#' @rdname emmc-functions
#' @param ref Integer(s) or character(s) specifying which level(s) to use 
#'   as the reference. Character values must exactly match elements of \code{levs}
#'   (including any enhancements -- see examples)
#' @export
trt.vs.ctrl.emmc = function(levs, ref = 1, reverse = FALSE, 
                            exclude = integer(0), include, ...) {
    ref = .num.key(levs, ref)
    exclude = .get.excl(levs, exclude, include)
    if (length(ref) == 0 || (min(ref) < 1) || (max(ref) > length(levs)))
        stop("In trt.vs.ctrl.emmc(), 'ref' levels are out of range", call. = FALSE)
    k = length(levs)
    cnm = ifelse(length(ref)==1, 
                 levs[ref], 
                 paste("avg(", paste(levs[ref], collapse=","), ")", sep=""))
    templ = rep(0, length(levs))
    templ[ref] = -1 / length(ref)
    M = data.frame(levs=levs)
    if (length(intersect(exclude, ref)) > 0)
        stop("'exclude' set cannot overlap with 'ref'")
    skip = c(ref, exclude)
    for (i in seq_len(k)) {
        if (i %in% skip) next
        con = templ
        con[i] = 1
        if (reverse)
            nm = paste(cnm, levs[i], sep = " - ")
        else
            nm = paste(levs[i], cnm, sep = " - ")
        M[[nm]] = con
    }
    row.names(M) = levs
    M = M[-1]
    if (reverse)
        M = -M
    attr(M, "desc") = "differences from control"
    attr(M, "adjust") = "dunnettx"
    if(length(exclude) > 0)
        attr(M, "famSize") = length(levs) - length(exclude)
    M
}

# control is 1st level
#' @rdname emmc-functions
#' @export
trt.vs.ctrl1.emmc = function(levs, ref = 1, ...) {
    trt.vs.ctrl.emmc(levs, ref = ref, ...)
}

# control is last level
#' @rdname emmc-functions
#' @export
trt.vs.ctrlk.emmc = function(levs, ref = length(levs), ...) {
    trt.vs.ctrl.emmc(levs, ref = ref, ...)
}

# pseudonym for trt.vs.ctrl
#' @rdname emmc-functions
#' @export
dunnett.emmc = function(levs, ref = 1, ...) {
    trt.vs.ctrl.emmc(levs, ref = ref, ...)
}

# effects contrasts. Each mean versus the average of all

#' @rdname emmc-functions
#' @param wts Optional weights to use with \code{eff.emmc} and \code{del.eff.emmc} contrasts.
#'   These default to equal weights.
#'   If \code{exclude} or \code{include} are specified, \code{wts} may be
#'   either the same length as \code{levs} or the length of the included levels.
#'   In the former case, weights for any excluded levels are set to zero.
#'   \code{wts} has no impact on the results unless there are at least
#'   three levels included in the contrast.
#' @export
eff.emmc = function(levs, exclude = integer(0), include, wts = rep(1, length(levs)), ...) {
    exclude = .get.excl(levs, exclude, include)
    if ((length(exclude) > 0) && (length(wts) == length(levs) - length(exclude))) {
        tmp = rep(0, length(levs))
        tmp[-exclude] = wts
        wts = tmp
    }
    if (length(wts) != length(levs))
        stop("length of 'wts' must equal the number of levels",
             " or the number of included levels")
    wts[exclude] = 0
    
    M = data.frame(row.names = levs)
    wts = wts / sum(wts)
    for (i in setdiff(seq_along(levs), exclude)) {
        con = -wts
        con[i] = 1 + con[i]
        nm = paste(levs[i], "effect")
        M[[nm]] = con
    }
    attr(M, "desc") = "differences from grand mean"
    attr(M, "adjust") = "fdr"
    if(length(exclude) > 0)
        attr(M, "famSize") = length(levs) - length(exclude)
    M
}

#' @rdname emmc-functions
#' @export
del.eff.emmc = function(levs, exclude = integer(0), include, wts = rep(1, length(levs)), ...) {
    M = eff.emmc(levs, exclude, include, wts, ...)
    use = setdiff(seq_along(levs), .get.excl(levs, exclude, include))
    for(i in seq_along(M))
        M[[i]] = M[[i]] / M[[i]][use[i]]
    attr(M, "desc") = "differences from mean of others"
    M
}

# Contrasts to compare consecutive levels:
# (-1,1,0,0,...), (0,-1,1,0,...), ..., (0,...0,-1,1)
#' @rdname emmc-functions
#' @export
consec.emmc = function(levs, reverse = FALSE, exclude = integer(0), include, ...) {
    exclude = .get.excl(levs, exclude, include)
    sgn = ifelse(reverse, -1, 1)
    tmp = rep(0, length(levs))
    k = length(levs) - length(exclude)
    active.rows = setdiff(seq_along(levs), exclude)
    M = data.frame(levs=levs)
    nms = levs[active.rows]
    for (i in seq_len(k-1)) {
        con = rep(0, k)
        con[i] = -sgn
        con[i+1] = sgn
        tmp[active.rows] = con
        nm = ifelse(reverse,
                    paste(nms[i], "-", nms[i+1]),
                    paste(nms[i+1], "-", nms[i]))
        M[[nm]] = tmp
    }
    row.names(M) = levs
    M = M[-1]
    attr(M, "desc") = "changes between consecutive levels"
    attr(M, "adjust") = "mvt"
    if(length(exclude) > 0)
        attr(M, "famSize") = length(levs) - length(exclude)
    M
}

# Mean after minus mean before
# e.g., (-1, 1/3,1/3,1/3), (-1/2,-1/2, 1/2,1/2), (-1/3,-1/3,-1/3, 1)
#' @rdname emmc-functions
#' @export
mean_chg.emmc = function(levs, reverse = FALSE, exclude = integer(0), include, ...) {
    exclude = .get.excl(levs, exclude, include)
    sgn = ifelse(reverse, -1, 1)
    k = length(levs) - length(exclude)
    tmp = rep(0, length(levs))
    M = data.frame(levs=levs)
    active.rows = setdiff(seq_along(levs), exclude)
    nms = levs[active.rows]
    for (i in seq_len(k-1)) {
        kmi = k - i
        con = rep(c(-sgn/i, sgn/kmi), c(i, kmi)) 
        nm = paste(nms[i], nms[i+1], sep="|")
        tmp[active.rows] = con
        M[[nm]] = tmp
    }
    row.names(M) = levs
    M = M[-1]
    attr(M, "desc") = "mean after minus mean before"
    attr(M, "adjust") = "mvt"
    if(length(exclude) > 0)
        attr(M, "famSize") = length(levs) - length(exclude)
    M
}

#' @rdname emmc-functions
#' @export
helmert.emmc <- function(levs, exclude = integer(0), include, ...) {
    exclude = .get.excl(levs, exclude, include)
    k = length(levs) - length(exclude)
    M = contr.helmert(k)
    if (length(exclude) > 0) {
        M = matrix(0, nrow = length(levs), ncol = k-1)
        M[-exclude, ] = contr.helmert(k)
        lbl = levs[-exclude]
    }
    else
        lbl = levs
    M <- as.data.frame(M)
    names(M) <- paste(lbl[-1],"vs earlier")
    attr(M, "desc") <- "Helmert contrasts"
    attr(M, "adjust") = "none"
    if(length(exclude) > 0)
        attr(M, "famSize") = k
    M
}

#' @rdname emmc-functions
#' @param family name of contrast family to use
#' @export
nrmlz.emmc = function(levs, family, ...) {
    M = get(paste0(family, ".emmc"))(levs, ...)
    for (i in seq_len(ncol(M)))
        M[, i] = M[, i] / sqrt(sum(M[, i]^2))
    attr(M, "desc") = paste("Normalized", attr(M, "desc"))
    M
}

# weighted contrasts
#' @rdname emmc-functions
#' @param cmtype the \code{type} argument passed to \code{\link[multcomp]{contrMat}}
#' @export
wtcon.emmc = function(levs, wts = rep(1, length(levs)), cmtype = "GrandMean", ...) {
    if (!requireNamespace("multcomp"))
        stop("The 'multcomp' package must be installed to use 'wtcon' contrasts", call. = FALSE)
    
    names(wts) = levs
    M = data.frame(t(multcomp::contrMat(wts, type = cmtype, ...)))
    M
}



# Non-contrasts -- just pass thru, possibly excluding some levels
#' @rdname emmc-functions
#' @export
identity.emmc = function(levs, exclude = integer(0), include, ...) {
    exclude = .get.excl(levs, exclude, include)
    k = length(levs) - length(exclude)
    M = as.data.frame(diag(length(levs)))
    names(M) = levs
    if(length(exclude) > 0)
        M = M[ , -exclude, drop = FALSE]
    attr(M, "desc") = "Identity"
    attr(M, "famSize") = k
    attr(M, "adjust") = "none"
    M
}



### utility to translate character keys to index keys
#' @rdname extending-emmeans
#' @order 43
#' @param levs,key The \code{.num.key} function returns the numeric indices of
#' the levels in \code{levs} to the set of all levels in \code{key}
#' @export
.num.key = function(levs, key) {
    if(!is.null(raw <- attr(levs, "raw")))
        levs = raw
    orig.key = key
    if (is.character(key))
        key = match(key, levs)
    # if (any(is.na(key)))
    #     warning("One or more of: '", paste(orig.key, collapse = "','"), "' not found in '",
    #          paste(levs, collapse = "','"), "'", call. = FALSE)
    # if (any(key > length(levs)) || any(key < 1))
    #     stop("Numeric index not in 1 : length(levs)")
    key[key %in% seq_along(levs)] # I think I'll just silently remove unmatched levels
}

### utility to find exclude levels from either exclude or include
### Also returns numeric version
#' @rdname extending-emmeans
#' @order 34
#' @param exc,inc Arguments for \code{.get.excl} which is useful
#' in writing \code{.emmc} functions for generating contrast coefficients,
#' and supports arguments \code{exclude} or \code{include} for excluding
#' or specifying which levels to use.
#' @export
.get.excl = function(levs, exc, inc) {
    if (!missing(inc)) {
        if(length(exc) > 0)
            stop("Cannot specify both 'exclude' and 'include'", call. = FALSE)
        inc = .num.key(levs, inc)
        exc = setdiff(seq_along(levs), inc)
    }
    .num.key(levs, exc)
}

##############################################################################
#    Copyright (c) 2012-2022 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

# Extra examples

#' Run or list additional examples
#' 
#' This function exists so as to provide cleaner-looking examples in
#' help files when it must be run conditionally on another package.
#' Typically we want to run the code (\code{run = TRUE} is the default),
#' or otherwise just list it on the console (\code{list = TRUE}).
#'
#' @param name Character name of file to run. We look for a file with this name
#'   (with \code{".R"} appended) in the system files provided with \pkg{emmeans}.
#' @param run Logical choosing whether or not to run the example code
#' @param list Logical choosing whether or not to list the example code
#' @param ... Used only by the developer
#' 
#' @export
#' 
#' @examples
#' # List an example
#' emm_example("qdrg-biglm", list = TRUE)
#' 
#' # Run an example
#' if (require(biglm))
#'     emm_example("qdrg-biglm")
#' 
emm_example = function(name, run = !list, list = FALSE, ...) {
    test = (\(test = FALSE, ...) test)(...)
    if(test) {
        file = paste0("inst/extexamples/", name, ".R")
        filestg = paste0("inst/extexamples/", name, ".R")
    }
    else {
        file = system.file("extexamples", paste0(name, ".R"), package = "emmeans")
        if (file == "") stop("File '", name, ".R' not found")
        filestg = paste0("system.file(\"extexamples\", \"", paste0(name, ".R"),
        "\", package = \"emmeans\")")
    }
    if (list) {
        cat(readLines(file), sep = "\n")
        cat("\n")
    }
    if(run) {
        prompt.echo = try(get("prompt.echo", parent.frame(3)), silent = TRUE)
        continue.echo = try(get("continue.echo", parent.frame(3)), silent = TRUE)
        if (inherits(prompt.echo, "try-error")) {
            prompt.echo = getOption("prompt")
            continue.echo = getOption("continue")
        }
        message("\n--- Running code from '", filestg, "'")
        source(file, echo = TRUE, verbose = FALSE, 
               prompt.echo = prompt.echo, continue.echo = continue.echo)
        cat("\n")
    }
    invisible()
}

# Cohen's effect sizes


#' Calculate Cohen effect sizes and confidence bounds thereof
#' 
#' Standardized effect sizes are typically calculated using pairwise differences of estimates,
#' divided by the SD of the population providing the context for those effects.
#' This function calculates effect sizes from an \code{emmGrid} object,
#' and confidence intervals for them, accounting for uncertainty in both the estimated
#' effects and the population SD. 
#' 
#' Any \code{by} variables specified in \code{object} will remain in force in the returned
#' effects, unless overridden in the optional arguments.
#' 
#' For models having a single random effect, such as those fitted using
#' \code{\link{lm}}; in that case, the \code{stats::sigma} and
#' \code{stats::df.residual} functions may be useful for specifying \code{sigma}
#' and \code{edf}. For models with more than one random effect, \code{sigma} may
#' be based on some combination of the random-effect variances. 
#' 
#' Specifying \code{edf} can be rather unintuitive but is also relatively
#' uncritical; but the smaller the value, the wider the confidence intervals for
#' effect size. The value of \code{sqrt(2/edf)} can be interpreted as the
#' relative accuracy of \code{sigma}; for example, with \code{edf = 50},
#' \eqn{\sqrt(2/50) = 0.2}, meaning that \code{sigma} is accurate to plus or
#' minus 20 percent. Note in an example below, we tried two different \code{edf}
#' values as kind of a bracketing/sensitivity-analysis strategy. A value of
#' \code{Inf} is allowable, in which case you are assuming that \code{sigma} is
#' known exactly. Obviously, this narrows the confidence intervals for the
#' effect sizes -- unrealistically if in fact \code{sigma} is unknown.
#' 
#'
#' @param object an \code{\link[=emmGrid-class]{emmGrid}} object, 
#' typically one defining the EMMs to 
#' be contrasted. If instead, \code{class(object) == "emm_list"},
#' such as is produced by \code{emmeans(model, pairwise ~ treatment)},
#' a message is displayed; the contrasts already therein are used; and 
#' \code{method} is replaced by \code{"identity"}.
#' @param sigma numeric scalar, value of the population SD. 
#' @param edf numeric scalar that specifies the equivalent degrees of freedom
#'   for the \code{sigma}. This is a way of specifying the uncertainty in \code{sigma},
#'   in that we regard our estimate of \code{sigma^2} as being proportional to
#'   a chi-square random variable with \code{edf} degrees of freedom. (\code{edf} should
#'   not be confused with the \code{df} argument that may be passed via \code{...}
#'   to specify the degrees of freedom to use in \eqn{t} statistics and confidence intervals.)
#' @param method the contrast method to use to define the effects.
#'   This is passed to \code{\link{contrast}} after the elements of \code{object}
#'   are scaled.
#' @param ... Additional arguments passed to \code{contrast}
#'
#' @return an \code{\link[=emmGrid-class]{emmGrid}} object containing the effect sizes
#' 
#' @section Computation:
#' This function uses calls to \code{\link{regrid}} to put the estimated
#' marginal means (EMMs) on the log scale. Then an extra element is added to
#' this grid for the log of \code{sigma} and its standard error (where we assume
#' that \code{sigma} is uncorrelated with the log EMMs). Then a call to
#' \code{\link{contrast}} subtracts \code{log{sigma}} from each of the log EMMs,
#' yielding values of \code{log(EMM/sigma)}.
#' Finally, the results are re-gridded back to the original scale and the
#' desired contrasts are computed using \code{method}. In the log-scaling
#' part, we actually rescale the absolute values and keep track of the signs.
#' 
#' @note
#' The effects are always computed on the scale of the \emph{linear-predictor};
#' any response transformation or link function is completely ignored. If you
#' wish to base the effect sizes on the response scale, it is \emph{not} enough
#' to replace \code{object} with \code{regrid(object)}, because this
#' back-transformation changes the SD required to compute effect sizes. 
#' 
#' @note
#' \strong{Paired data:} Be careful with paired-data situations, where Cohen's d is typically referenced to
#' the SD of the \emph{paired differences} rather than the \emph{residual} SD.
#' You may need to enlarge \code{sigma} by a factor of \code{sqrt(2)} to obtain
#' comparable results with other software.
#' 
#' @note
#' \strong{Disclaimer:} There is substantial disagreement among practitioners on
#' what is the appropriate \code{sigma} to use in computing effect sizes; or,
#' indeed, whether \emph{any} effect-size measure is appropriate for some
#' situations. The user is completely responsible for specifying 
#' appropriate parameters (or for failing to do so).
#' 
#' Cohen effect sizes do not even exist for generalized linear models or other
#' models lacking an additive residual error term. 
#' 
#' @export
#' @note 
#' The examples here illustrate a sobering message that effect sizes are often not nearly as accurate as you may think.
#'
#' @examples
#' fiber.lm <- lm(strength ~ diameter + machine, data = fiber)
#' 
#' emm <- emmeans(fiber.lm, "machine")
#' eff_size(emm, sigma = sigma(fiber.lm), edf = df.residual(fiber.lm))
#' 
#' # or equivalently:
#' eff_size(pairs(emm), sigma(fiber.lm), df.residual(fiber.lm), method = "identity")
#' 
#' 
#' ### Mixed model example:
#' if (require(nlme)) withAutoprint({
#'   Oats.lme <- lme(yield ~ Variety + factor(nitro), 
#'                   random = ~ 1 | Block / Variety,
#'                   data = Oats)
#'   # Combine variance estimates
#'   VarCorr(Oats.lme)
#'   (totSD <- sqrt(214.4724 + 109.6931 + 162.5590))
#'   # I figure edf is somewhere between 5 (Blocks df) and 51 (Resid df)
#'   emmV <- emmeans(Oats.lme, ~ Variety)
#'   eff_size(emmV, sigma = totSD, edf = 5)
#'   eff_size(emmV, sigma = totSD, edf = 51)
#' }, spaced = TRUE)
#' 
#' 

eff_size = function(object, sigma, edf, method = "pairwise", ...) {
    ### Lame attempt to warn if unsuitable...
    # obj.sig = object@misc$sigma
    # if(is.null(obj.sig) || is.na(obj.sig[1]))
    #     warning("'eff_size()' results are highly suspect for many models.\n",
    #             " Cohen effect sizes are not even defined for generalized linear models.\n",
    #             " See documentation notes.", call. = FALSE)
    if (inherits(object, "emm_list") && ("contrasts" %in% names(object))) {
        message("Since 'object' is a list, we are using the contrasts already present.")
        object = object$contrasts
        method = "identity"
    }
    SE.logsigma = sqrt(1 / (2 * edf))
 
    object = update(object, tran = NULL)
    object = regrid(object, transform = "response")
    
    # put on absolute scale
    emm = object@bhat
    sgn = sign(emm)
    emm = abs(emm)
    sgn[emm < 1e-15] = 0
    emm[sgn == 0] = 1
    object@bhat = emm
    
    # put on log scale and incorporate the SD estimate
    logobj = regrid(object, transform = "log")
    logobj@bhat = c(logobj@bhat, log(sigma))
    V = rbind(cbind(logobj@V, 0), 0)
    n = nrow(V)
    V[n,n] = SE.logsigma^2
    logobj@V = V
    logobj@levels = list(dummy = 1:n)
    logobj@grid = data.frame(dummy = 1:n)
    logobj@linfct = diag(n)
    logobj@misc$by = NULL
    
    con = contrast(logobj, "trt.vs.ctrlk", by = NULL)
    con = regrid(con, transform = "response")
    object@bhat = con@bhat * sgn
    object@V = con@V
    update(contrast(object, method, adjust = "none", ...), 
           estName = "effect.size", infer = c(TRUE, FALSE),
           initMesg = paste("sigma used for effect sizes:", signif(sigma, digits = 4)))
}

##############################################################################
#    Copyright (c) 2012-2017 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

# Support for objects in the *rms* package

#' @exportS3Method recover_data rms
recover_data.rms = function(object, ...) {
    fcall = object$call
    recover_data(fcall, delete.response(terms(object)), object$na.action$omit, ...)
}

# TODO: (postponed indefinitely because Harrell doesn't like emmeans anyway)
# 1. If multivariate - like mlm method?
# 2. orm cases?

#' @exportS3Method emm_basis rms          
emm_basis.rms = function(object, trms, xlev, grid, 
        mode = c("middle", "latent", "linear.predictor", "cum.prob", "exc.prob", "prob", "mean.class"), 
        vcov., ...) {
    mode = match.arg(mode)
    bhat = coef(object) 
    if (missing(vcov.))
        V = vcov(object, intercepts = "all")
    else
        V = .my.vcov(object, vcov.)
    misc = list()
    
    X = predict(object, newdata = grid, type = "x")
    #xnames = dimnames(X)[[2]]
    #intcpts = setdiff(names(bhat), xnames)
    nint = length(bhat) - ncol(X)
    intcpts = names(bhat)[seq_len(nint)]
    xnames = setdiff(names(bhat), intcpts)
        
    if (length(intcpts) == 1) 
        mode = "single" # stealth mode for ordinary single-intercept case
    if (mode %in% c("single", "middle", "latent")) {
        X = cbind(1, X)
        mididx = ifelse(mode != "middle", 1, as.integer((1 + length(intcpts)) / 2))
        dimnames(X)[[2]][1] = switch(mode,
            single = intcpts,
            middle = intcpts[mididx],
            latent = "avg.intercept")
        if (mode == "middle") {
            nms = c(intcpts[mididx], xnames)
            bhat = bhat[nms]
            V = V[nms, nms, drop = FALSE]
        }
        else if (mode == "latent") {
            bhat = c(mean(bhat[intcpts]), bhat[xnames])
            nx = length(xnames)
            J1 = rbind(rep(1/nint, nint), 
                       matrix(0, nrow = nx, ncol = nint))
            J2 = rbind(0, diag(1, nx))
            J = cbind(J1, J2)
            V = J %*% V %*% t(J)
        }
        ### else mode == "single" and all is OK as it is
    }
    else { # mode %in% c("linear.predictor", "cum.prob", "exc.prob", "prob", "mean.class")
        misc$ylevs = list(cut = intcpts)
        I = diag(1, nint)
        J = matrix(1, nrow = nrow(X))
        JJ = matrix(1, nrow=nint)
        X = cbind(kronecker(I, J), kronecker(JJ, X))
        # Note V is correct as-is
        dimnames(X)[[2]] = c(intcpts, xnames)
        if (mode != "linear.predictor") {
            misc$mode = mode
            misc$postGridHook = .clm.postGrid
            misc$respName = as.character.default(object$terms)[2]
        }
    }
    
    # I think rms does not allow rank deficiency...
    nbasis = estimability::all.estble
    if (!is.null(object$family)) {
        if (!is.character(object$family))
            misc = .std.link.labels(object$family, misc)
        else {
            misc$tran = object$family
            if (misc$tran == "logistic") misc$tran = "logit"
            misc$inv.lbl = switch(class(object)[1], 
                orm = "exc.prob",
                lrm = ifelse(nint == 1, "prob", "exc.prob"),
                "response")
        }
        dffun = function(k, dfargs) Inf
        dfargs = list()
    }
    else {
        dfargs = list(df = object$df.residual)
        if (is.null(dfargs$df)) 
            dfargs$df = Inf
        dffun = function(k, dfargs) dfargs$df
    }
    list(X=X, bhat=bhat, nbasis=nbasis, V=V, 
         dffun=dffun, dfargs=dfargs, misc=misc)
}

##############################################################################
#    Copyright (c) 2012-2024 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

### This file has summary.emmGrid S3 method and related functions

# `as.data.frame` for a summary - just passes thru
# A former version created an infinite loop if not careful with using `as.data.frame`
# to coerce a summary_emm object to a regular data frame. Now we cave to insistent
# repeat calls and remove the `summary_emm` class the second time.
#' @export
as.data.frame.summary_emm = function(x, ...) {
    if(!is.null(attr(x, "digits")))
        oldClass(x) = "data.frame"
    else
        attr(x, "digits") = getOption("digits")
    x
}

# S3 summary method

#' Summaries, predictions, intervals, and tests for \code{emmGrid} objects
#' 
#' These are the primary methods for obtaining numerical or tabular results from
#' an \code{emmGrid} object. \code{summary.emmGrid} is the general function for
#' summarizing \code{emmGrid} objects. It also serves as the print method for
#' these objects; so for convenience, \code{summary()} arguments may be included
#' in calls to functions such as \code{\link{emmeans}} and
#' \code{\link{contrast}} that construct \code{emmGrid} objects. Note that by
#' default, summaries for Bayesian models are diverted to
#' \code{\link{hpd.summary}}. 
#' 
#' \code{confint.emmGrid} is equivalent to \code{summary.emmGrid with 
#' infer = c(TRUE, FALSE)}. The function \code{test.emmGrid}, when called with 
#' \code{joint = FALSE}, is equivalent to \code{summary.emmGrid} with \code{infer = c(FALSE, TRUE)}. 
#' 
#' With \code{joint = TRUE}, \code{test.emmGrid} calculates the Wald test of the
#' hypothesis \code{linfct \%*\% bhat = null}, where \code{linfct} and
#' \code{bhat} refer to slots in \code{object} (possibly subsetted according to
#' \code{by} or \code{rows}). An error is thrown if any row of \code{linfct} is
#' non-estimable. It is permissible for the rows of \code{linfct} to be linearly
#' dependent, as long as \code{null == 0}, in which case a reduced set of 
#' contrasts is tested. Linear dependence and nonzero \code{null} cause an 
#' error. The returned object has an additional \code{"est.fcns"} attribute, which
#' is a list of the linear functions associated with the joint test.
#'
#' @param object An object of class \code{"emmGrid"} (see \link{emmGrid-class})
#' @param infer A vector of one or two logical values. The first determines
#'   whether confidence intervals are displayed, and the second determines
#'   whether \emph{t} tests and \emph{P} values are displayed. If only one value
#'   is provided, it is used for both.
#' @param level Numerical value between 0 and 1. Confidence level for confidence
#'   intervals, if \code{infer[1]} is \code{TRUE}.
#' @param adjust Character value naming the method used to adjust \eqn{p} values
#'   or confidence limits; or to adjust comparison arrows in \code{plot}. See
#'   the P-value adjustments section below.
#' @param by Character name(s) of variables to use for grouping into separate 
#'   tables. This affects the family of tests considered in adjusted \emph{P}
#'   values. 
#' @param cross.adjust Character: \eqn{p}-value adjustment method to 
#'   additionally apply \emph{across} 
#'   the \code{by} groups. See the section on P-value adjustments for details.
#' @param type Character: type of prediction desired. This only has an effect if
#'   there is a known transformation or link function. \code{"response"} 
#'   specifies that the inverse transformation be applied. \code{"mu"} (or 
#'   equivalently, \code{"unlink"}) is usually the same as \code{"response"},
#'   but in the case where the model has both a link function and a response 
#'   transformation, only the link part is back-transformed. Other valid values 
#'   are \code{"link"}, \code{"lp"}, and \code{"linear.predictor"}; these are
#'   equivalent, and request that results be shown for the linear predictor,
#'   with no back-transformation. The default is \code{"link"}, unless the 
#'   \code{"predict.type"} option is in force; see \code{\link{emm_options}},
#'   and also the section below on transformations and links.
#' @param df Numeric. If non-missing, a constant number of degrees of freedom to
#'   use in constructing confidence intervals and \emph{P} values (\code{NA}
#'   specifies asymptotic results).
#' @param calc Named list of character value(s) or formula(s).
#'   The expressions in \code{char} are evaluated and appended to the
#'   summary, just after the \code{df} column. The expression may include
#'   any names up through \code{df} in the summary, any additional names in 
#'   \code{object@grid} (such as \code{.wgt.} or \code{.offset.}), or any
#'   earlier elements of \code{calc}.
#' @param null Numeric. Null hypothesis value(s), on the linear-predictor scale,
#'   against which estimates are tested. May be a single value used for all, or
#'   a numeric vector of length equal to the number of tests in each family
#'   (i.e., \code{by} group in the displayed table).
#' @param delta Numeric value (on the linear-predictor scale). If zero, ordinary
#'   tests of significance are performed. If positive, this specifies a
#'   threshold for testing equivalence (using the TOST or two-one-sided-test
#'   method), non-inferiority, or non-superiority, depending on \code{side}. See
#'   Details for how the test statistics are defined.
#' @param side Numeric or character value specifying whether the test is
#'   left-tailed (\code{-1}, \code{"-"}, \code{"<"}, \code{"left"}, or
#'   \code{"nonsuperiority"}); right-tailed (\code{1}, \code{"+"}, \code{">"},
#'   \code{"right"}, or \code{"noninferiority"}); or two-sided (\code{0},
#'   \code{2}, \code{"!="}, \code{"two-sided"}, \code{"both"},
#'   \code{"equivalence"}, or \code{"="}). See the special section below for
#'   more details.
#' @param frequentist Ignored except if a Bayesian model was fitted. If missing
#'   or \code{FALSE}, the object is passed to \code{\link{hpd.summary}}. Otherwise, 
#'   a logical value of \code{TRUE} will have it return a frequentist summary.
#' @param bias.adjust Logical value for whether to adjust for bias in
#'   back-transforming (\code{type = "response"}). This requires a valid value of 
#'   \code{sigma} to exist in the object or be specified.
#' @param sigma Error SD assumed for bias correction (when 
#'   \code{type = "response"} and a transformation
#'   is in effect), or for constructing prediction intervals. If not specified,
#'   \code{object@misc$sigma} is used, and a warning is issued if it is not found
#'   or not valid.
#'   \emph{Note:} \code{sigma} may be a vector, but be careful that it correctly
#'   corresponds (perhaps after recycling) to the order of the reference grid.
#' @param ... Optional arguments such as \code{scheffe.rank} 
#'   (see \dQuote{P-value adjustments}). 
#'   In \code{confint.emmGrid}, 
#'   \code{predict.emmGrid}, and 
#'   \code{test.emmGrid}, these arguments are passed to
#'   \code{summary.emmGrid}. These arguments are also passed to \code{mvregrid}
#'   when we have a multivariate transformation.
#'
#' @return \code{summary.emmGrid}, \code{confint.emmGrid}, and
#'   \code{test.emmGrid} return an object of class \code{"summary_emm"}, which
#'   is an extension of \code{\link{data.frame}} but with a special \code{print}
#'   method that displays it with custom formatting. For models fitted using
#'   MCMC methods, the call is diverted to \code{\link{hpd.summary}} (with 
#'   \code{prob} set to \code{level}, if specified); one may
#'   alternatively use general MCMC summarization tools with the 
#'   results of \code{as.mcmc}.
#'   
#' @section Defaults:
#'   The \code{misc} slot in \code{object} may contain default values for
#'   \code{by}, \code{calc}, \code{infer}, \code{level}, \code{adjust}, 
#'   \code{type}, \code{null}, \code{side}, and \code{delta}. 
#'   These defaults vary depending
#'   on the code that created the object. The \code{\link{update}} method may be
#'   used to change these defaults. In addition, any options set using 
#'   \samp{emm_options(summary = ...)} will trump those stored in the object's 
#'   \code{misc} slot.
#' 
#' @section Transformations and links:
#'   With \code{type = "response"}, the transformation assumed can be found in
#'   \samp{object@misc$tran}, and its label, for the summary is in
#'   \samp{object@misc$inv.lbl}. Any \eqn{t} or \eqn{z} tests are still performed
#'   on the scale of the linear predictor, not the inverse-transformed one.
#'   Similarly, confidence intervals are computed on the linear-predictor scale,
#'   then inverse-transformed. 
#'   
#'   Be aware that only univariate transformations and links are
#'   supported in this way. Some multivariate transformations are supported by 
#'   \code{\link{mvregrid}}.
#'   
#' @section Bias adjustment when back-transforming:
#'   When \code{bias.adjust} is \code{TRUE}, then back-transformed estimates
#'   are adjusted by adding 
#'   \eqn{0.5 h''(u)\sigma^2}, where \eqn{h} is the inverse transformation and
#'   \eqn{u} is the linear predictor. This is based on a second-order Taylor
#'   expansion. There are better or exact adjustments for certain specific
#'   cases, and these may be incorporated in future updates.
#'   
#'   Note: In certain models, e.g., those with non-gaussian families,
#'   \code{sigma} is initialized as \code{NA}, and so by default, bias adjustment
#'   is skipped and a warning is issued. You may override this by specifying a
#'   value for \code{sigma}. However, \emph{with ordinary generalized linear models,
#'   bias adjustment is inappropriate} and you should not try to do it. With GEEs and GLMMs,
#'   you probably should \emph{not} use \code{sigma(model)}, and instead you should create an
#'   appropriate value using the estimated random effects, e.g., from \code{VarCorr(model)}.
#'   An example is provided in the \dQuote{transformations} vignette.
#'   
#'   \emph{A word of caution:} This bias-adjustment method is merely a one-term
#'   correction, and it multiplies \code{sigma^2}. When \code{sigma} is reasonably
#'   small relative to the scale of the response, this is fine; but when
#'   \code{sigma} is large, this bias adjustment can produce bizarre and
#'   out-of-range results. Consider for example a logit or probit model where
#'   the back-transformed response is constrained to the interval [0, 1]; if
#'   \code{sigma} is more than 1, the adjustment can 
#'   easily pull the estimate outside of these constraints.
#' 
#' @section P-value adjustments:
#'   The \code{adjust} argument specifies a multiplicity adjustment for tests or
#'   confidence intervals. This adjustment always is applied \emph{separately}
#'   to each table or sub-table that you see in the printed output (see
#'   \code{\link{rbind.emmGrid}} for how to combine tables). If there are non-estimable
#'   cases in a \code{by} group, those cases are \emph{excluded} before determining
#'   the adjustment; that means there could be different adjustments in different groups.
#'   
#'   The valid values of \code{adjust} are as follows:
#'   \describe{
#'   \item{\code{"tukey"}}{Uses the Studentized range distribution with the number
#'     of means in the family. (Available for two-sided cases only.)}
#'   \item{\code{"scheffe"}}{Computes \eqn{p} values from the \eqn{F}
#'     distribution, according to the Scheffe critical value of
#'     \eqn{\sqrt{rF(\alpha; r, d)}}{sqrt[r*qf(alpha, r, d)]}, where \eqn{d} is
#'     the error degrees of freedom and \eqn{r} is the rank of the set of linear
#'     functions under consideration. By default, the value of \code{r} is
#'     computed from \code{linfct(object)} for each by group; however, if the
#'     user specifies an argument matching \code{scheffe.rank}, its value will
#'     be used instead. Ordinarily, if there are \eqn{k} means involved, then
#'     \eqn{r = k - 1} for a full set of contrasts involving all \eqn{k} means, and
#'     \eqn{r = k} for the means themselves. (The Scheffe adjustment is available
#'     for two-sided cases only.)}
#'   \item{\code{"sidak"}}{Makes adjustments as if the estimates were independent
#'     (a conservative adjustment in many cases).}
#'   \item{\code{"bonferroni"}}{Multiplies \eqn{p} values, or divides significance
#'     levels by the number of estimates. This is a conservative adjustment.}
#'   \item{\code{"dunnettx"}}{Uses our own\emph{ad hoc} approximation to the 
#'     Dunnett distribution for a family of estimates having pairwise
#'     correlations of \eqn{0.5} (as is true when comparing treatments with a
#'     control with equal sample sizes). The accuracy of the approximation
#'     improves with the number of simultaneous estimates, and is much faster
#'     than \code{"mvt"}. (Available for two-sided cases only.)}
#'   \item{\code{"mvt"}}{Uses the multivariate \eqn{t} distribution to assess the
#'     probability or critical value for the maximum of \eqn{k} estimates. This
#'     method produces the same \eqn{p} values and intervals as the default
#'     \code{summary} or \code{confint} methods to the results of
#'     \code{\link{as.glht}}. In the context of pairwise comparisons or comparisons
#'     with a control, this produces \dQuote{exact} Tukey or Dunnett adjustments,
#'     respectively. However, the algorithm (from the \pkg{mvtnorm} package) uses a
#'     Monte Carlo method, so results are not exactly repeatable unless the same
#'     random-number seed is used (see \code{\link[base:Random]{set.seed}}). As the family
#'     size increases, the required computation time will become noticeable or even
#'     intolerable, making the \code{"tukey"}, \code{"dunnettx"}, or others more
#'     attractive.}
#'   \item{\code{"none"}}{Makes no adjustments to the \eqn{p} values.}
#'   } %%%%%%%%%%%%%%%% end \describe {}
#' 
#'   For tests, not confidence intervals, the Bonferroni-inequality-based adjustment
#'   methods in \code{\link{p.adjust}} are also available (currently, these
#'   include \code{"holm"}, \code{"hochberg"}, \code{"hommel"},
#'   \code{"bonferroni"}, \code{"BH"}, \code{"BY"}, \code{"fdr"}, and
#'   \code{"none"}). If a \code{p.adjust.methods} method other than
#'   \code{"bonferroni"} or \code{"none"} is specified for confidence limits, the
#'   straight Bonferroni adjustment is used instead. Also, if an adjustment method
#'   is not appropriate (e.g., using \code{"tukey"} with one-sided tests, or with
#'   results that are not pairwise comparisons), a more appropriate method
#'   (usually \code{"sidak"}) is substituted.
#' 
#'   In some cases, confidence and \eqn{p}-value adjustments are only approximate
#'   -- especially when the degrees of freedom or standard errors vary greatly
#'   within the family of tests. The \code{"mvt"} method is always the correct
#'   one-step adjustment, but it can be very slow. One may use
#'   \code{\link{as.glht}} with methods in the \pkg{multcomp} package to obtain
#'   non-conservative multi-step adjustments to tests.
#'   
#'   \emph{Warning:} Non-estimable cases are \emph{included} in the family to which adjustments
#'   are applied. You may wish to subset the object using the \code{[]} operator
#'   to work around this problem.
#'   
#'   The \code{cross.adjust} argument is a way of specifying a multiplicity
#'   adjustment across the \code{by} groups (otherwise by default, each group is
#'   treated as a separate family in regards to multiplicity adjustments). It
#'   applies only to \eqn{p} values. Valid options are one of the
#'   \code{p.adjust.methods} or \code{"sidak"}. This argument is ignored unless
#'   it is other than \code{"none"}, there is more than one \code{by} group, and
#'   they are all the same size. Under those conditions, we first use
#'   \code{adjust} to determine the within-group adjusted \eqn{p} values.
#'   Imagine each group's adjusted \eqn{p} values arranged in side-by-side
#'   columns, thus forming a matrix with the number of columns equal to the
#'   number of \code{by} groups. Then we use the \code{cross.adjust} method to
#'   further adjust the adjusted \eqn{p} values in each row of this matrix. Note
#'   that an \emph{overall} Bonferroni (or Sidak) adjustment is obtainable by
#'   specifying \emph{both} \code{adjust} and \code{cross.adjust} as
#'   \code{"bonferroni"} (or \code{"sidak"}). However, less conservative (but
#'   yet conservative) overall adjustments are available when it is possible to
#'   use an \dQuote{exact} within-group method (e.g., \code{adjust = "tukey"}
#'   for pairwise comparisons) and \code{cross.adjust} as a conservative
#'   adjustment. [\code{cross.adjust} methods other than \code{"none"}, 
#'   \code{"bonferroni"}, or \code{"sidak"} do not seem advisable, but other 
#'   \code{p.adjust} methods are available if you can make sense of them.]
#'
#' @section Tests of significance, nonsuperiority, noninferiority, or equivalence:
#'   When \code{delta = 0}, test statistics are the usual tests of significance.
#'   They are of the form 
#'   \samp{(estimate - null)/SE}. Notationally: 
#'   \describe{
#'     \item{Significance}{\eqn{H_0: \theta = \theta_0}  versus \cr
#'        \eqn{H_1: \theta < \theta_0} (left-sided), or\cr
#'       \eqn{H_1: \theta > \theta_0} (right-sided), or\cr
#'       \eqn{H_1: \theta \ne \theta_0} (two-sided)\cr
#'       The test statistic is\cr
#'       \eqn{t = (Q - \theta_0)/SE}\cr 
#'       where \eqn{Q} is our estimate of \eqn{\theta};
#'       then left, right, or two-sided \eqn{p} values are produced, 
#'       depending on \code{side}.}
#'   }
#'   When \code{delta} is positive, the test statistic depends on \code{side} as
#'   follows.
#'   \describe{
#'   \item{Left-sided (nonsuperiority)}{\eqn{H_0: \theta \ge \theta_0 + \delta}
#'     versus \eqn{H_1: \theta < \theta_0 + \delta}\cr 
#'     \eqn{t = (Q - \theta_0 - \delta)/SE}\cr 
#'     The \eqn{p} value is the lower-tail probability.}
#'   \item{Right-sided (noninferiority)}{\eqn{H_0: \theta \le \theta_0 - \delta}
#'     versus \eqn{H_1: \theta > \theta_0 - \delta}\cr 
#'     \eqn{t = (Q - \theta_0 + \delta)/SE}\cr
#'     The \eqn{p} value is the upper-tail probability.}
#'   \item{Two-sided (equivalence)}{\eqn{H_0: |\theta - \theta_0| \ge \delta}
#'     versus \eqn{H_1: |\theta - \theta_0| < \delta}\cr
#'     \eqn{t = (|Q - \theta_0| - \delta)/SE}\cr
#'     The \eqn{p} value is the \emph{lower}-tail probability.\cr
#'     Note that \eqn{t} is the maximum of \eqn{t_{nonsup}} and \eqn{-t_{noninf}}. 
#'     This is equivalent to choosing the less 
#'     significant result in the two-one-sided-test (TOST) procedure.}
#'   } %%%%%%%%%%%% end \describe{}
#'
#' 
#' @section Non-estimable cases:
#'   When the model is rank-deficient, each row \code{x} of \code{object}'s
#'   \code{linfct} slot is checked for estimability. If \code{sum(x*bhat)}
#'   is found to be non-estimable, then the string \code{NonEst} is displayed for the
#'   estimate, and associated statistics are set to \code{NA}. 
#'   The estimability check is performed
#'   using the orthonormal basis \code{N} in the \code{nbasis} slot for the null
#'   space of the rows of the model matrix. Estimability fails when
#'   \eqn{||Nx||^2 / ||x||^2} exceeds \code{tol}, which by default is
#'   \code{1e-8}. You may change it via \code{\link{emm_options}} by setting
#'   \code{estble.tol} to the desired value.
#'   
#'   See the warning above that non-estimable cases are still included when
#'   determining the family size for \emph{P}-value adjustments.
#'   
#' @section Warning about potential misuse of P values:
#'   Some in the statistical and scientific community argue that
#'   the term \dQuote{statistical significance} should be completely abandoned, and
#'   that criteria such as \dQuote{p < 0.05} never be used to assess the
#'   importance of an effect. These practices can be too misleading and are prone to abuse.
#'   See \href{../doc/basics.html#pvalues}{the \dQuote{basics} vignette} for more
#'   discussion.
#'   
#' 
#' @note In doing testing and a transformation and/or link is in force, any
#'   \code{null} and/or \code{delta} values specified must always be on the
#'   scale of the linear predictor, regardless of the setting for `type`. If
#'   \code{type = "response"}, the null value displayed in the summary table 
#'   will be back-transformed from the value supplied by the user. But the
#'   displayed \code{delta} will not be changed, because there (often) is
#'   not a natural way to back-transform it.
#'   
#' @note When we have \code{type = "response"}, and \code{bias.adj = TRUE},
#'   the \code{null} value displayed in the output is both back-transformed
#'   and bias-adjusted, leading to a rather non-intuitive-looking null value.
#'   However, since the tests themselves are performed on the link scale,
#'   this is the response value at which a *P* value of 1 would be obtained.
#' 
#' @note The default \code{show} method for \code{emmGrid} objects (with the
#'   exception of newly created reference grids) is \code{print(summary())}.
#'   Thus, with ordinary usage of \code{\link{emmeans}} and such, it is
#'   unnecessary to call \code{summary} unless there is a need to
#'   specify other than its default options.
#'   
#' @note If a data frame is needed, \code{summary}, \code{confint},
#'   and \code{test} serve this need. \code{as.data.frame} routes to
#'   \code{summary} by default; calling it with \code{destroy.annotations = TRUE}
#'   is not recommended for exactly that reason.
#'   If you want to see more digits in the output, use
#'   \code{print(summary(object), digits = ...)}; and if you \emph{always} want
#'   to see more digits, use \code{emm_options(opt.digits = FALSE)}.
#' @seealso \code{\link{hpd.summary}}
#' 
#' @method summary emmGrid  
#' @export
#' @order 1
#'
#' @examples
#' warp.lm <- lm(breaks ~ wool * tension, data = warpbreaks)
#' warp.emm <- emmeans(warp.lm, ~ tension | wool)
#' warp.emm    # implicitly runs 'summary'
#' 
#' confint(warp.emm, by = NULL, level = .90)
#' 
#' # --------------------------------------------------------------
#' pigs.lm <- lm(log(conc) ~ source + factor(percent), data = pigs)
#' pigs.emm <- emmeans(pigs.lm, "percent", type = "response")
#' summary(pigs.emm)    # (inherits type = "response")
#' summary(pigs.emm, calc = c(n = ".wgt."))  # Show sample size
#' 
#' # For which percents is EMM non-inferior to 35, based on a 10% threshold?
#' # Note the test is done on the log scale even though we have type = "response"
#' test(pigs.emm, null = log(35), delta = log(1.10), side = ">")
#' 
#' con <- contrast(pigs.emm, "consec")
#' test(con)
#' 
#' test(con, joint = TRUE)
#' 
#' # default Scheffe adjustment - rank = 3
#' summary(con, infer = c(TRUE, TRUE), adjust = "scheffe")
#' 
#' # Consider as some of many possible contrasts among the six cell means
#' summary(con, infer = c(TRUE, TRUE), adjust = "scheffe", scheffe.rank = 5)
#'
#' # Show estimates to more digits
#' print(test(con), digits = 7)
#' 
#' # --------------------------------------------------------------
#' # Cross-adjusting P values
#' prs <- pairs(warp.emm)   # pairwise comparisons of tension, by wool
#' test(prs, adjust = "tukey", cross.adjust = "bonferroni")
#' 
#' # Same comparisons taken as one big family (more conservative)
#' test(prs, adjust = "bonferroni", by = NULL)
#' 
summary.emmGrid <- function(object, infer, level, adjust, by, 
                            cross.adjust = "none", type, df, calc,
                            null, delta, side, frequentist, 
                            bias.adjust = get_emm_option("back.bias.adj"),
                            sigma, ...) {
    if (missing(type))
        type = .get.predict.type(object@misc)
    else
        type = .validate.type(type)
    
    # if we have a multivariate transformation, try to replace it with mvregrid
    if((type == "response") && !is.null(tran <- object@misc$tran) && 
       is.character(tran) && (tran %in% mult.trans)) {
        foo = try(mvregrid(object, ...), silent = TRUE)
        if(!inherits(foo, "try-error")) {
            object = foo
            message("Note: Multivariate transformation found, object has been re-gridded;\n",
                    "this may distort levels or order of listing")
        }
        else {
            warning("Multivariate transformation found that we can't invert.\n",
                    "Perhaps add a 'mult.name' argument?")
        }
    }
    if(missing(sigma))
        sigma = object@misc$sigma
    if(missing(frequentist) && !is.null(object@misc$frequentist))
        frequentist = object@misc$frequentist
    if(missing(bias.adjust)) {
        if (!is.null(object@misc$bias.adjust)) 
            bias.adjust = object@misc$bias.adjust
        else
            bias.adjust = get_emm_option("back.bias.adj")
    }
    
    if(!is.na(object@post.beta[1]) && (missing(frequentist) || !frequentist))
        return (hpd.summary(object, prob = level, by = by, type = type, delta = delta,
                            bias.adjust = bias.adjust, sigma = sigma, ...))

    
    # Any "summary" options override built-in
    opt = get_emm_option("summary")
    if(!is.null(opt)) {
        opt$object = object
        object = do.call("update.emmGrid", opt)
    }
    
    if(!missing(sigma))
        object = update(object, sigma = sigma)  # we'll keep sigma in misc
    
    misc = object@misc
    use.elts = .reconcile.elts(object)
    grid = object@grid[use.elts, , drop = FALSE]
    
    ### For missing arguments, get from misc, else default    
    if(missing(infer))
        infer = misc$infer
    if(missing(level))
        level = misc$level
    if(missing(adjust))
        adjust = misc$adjust
    if(missing(cross.adjust) && !is.null(misc$cross.adjust))
        cross.adjust = misc$cross.adjust
    if(missing(by))
        by = misc$by.vars
    
    # Disable Tukey if by vars don't match those used in construction
    if((!is.na(misc$estType) && misc$estType == "pairs") && (paste(c("", by), collapse = ",") != misc$.pairby))
        misc$estType = object@misc$estType = "contrast"
    
    
    
    # if there are two transformations and we want response, then we need to undo both
    if ((type == "response") && (!is.null(misc$tran2))) {
        tmp = match.call()
        tmp$object = quote(object)
        tmp$type = "unlink"
        summ.unlink = eval(tmp) #this is summary with type = "unlink"
        object = regrid(object, transform = "mu")
        two.trans = TRUE
    }
    else
        two.trans = FALSE
    ### Commented out because we handled this earlier
    # Check if we have a log-ratio transformation. At least then we can guess the problem.
    # if ((type == "response") && is.character(object@misc$tran) && endsWith(object@misc$tran, "lr"))
    #     warning("It looks like you have a multivariate transformation -- not handled by type = 'response'.\n",
    #             "Perhaps instead you wanted to use 'mvregrid()'?")
    

    
    if ((type %in% c("mu", "unlink")) && (!is.null(t2 <- misc$tran2))) {
        if (!is.character(t2))
            t2 = "tran"
        object = update(object, inv.lbl = paste0(t2, "(resp)"))
    }
    
    if(missing(df)) 
        df = misc$df
    if(!is.null(df)) {
        object@dffun = function(k, dfargs) df[1]
        attr(object@dffun, "mesg") = "user-specified"
    }
    
    # for missing args that default to zero unless provided or in misc slot
    .nul.eq.zero = function(val) {
        if(is.null(val)) 0
        else val
    }
    if(missing(null))
        null = .nul.eq.zero(misc$null)
    if(missing(delta))
        delta = .nul.eq.zero(misc$delta)
    if(missing(side))
        side = .nul.eq.zero(misc$side)
    
    
    # reconcile all the different ways we could specify the alternative
    # ... and map each to one of the first 3 subscripts
    side.opts = c("left","both","right","two-sided","noninferiority","nonsuperiority","equivalence","superiority","inferiority","0","2","-1","1","+1","<",">","!=","=")
    side.map =  c( 1,     2,     3,      2,          3,               1,               2,            3,            1,            2,  2,   1,  3,   3,  1,  3,  2,   2)
    side = side.map[pmatch(side, side.opts, 2)[1]] - 2
    delta = abs(delta)
    
    result = .est.se.df(object, ...)
    
    lblnms = setdiff(names(grid), 
                     c(object@roles$responses, ".offset.", ".wgt."))
    lbls = grid[lblnms]
    
    zFlag = (all(is.na(result$df) | is.infinite(result$df)))
    inv = (type %in% c("response", "mu", "unlink")) # flag to inverse-transform
    link = attr(result, "link")
    if (inv && is.null(link))
        inv = FALSE
    
    if ((length(infer) == 0) || !is.logical(infer)) 
        infer = c(FALSE, FALSE)
    if(length(infer == 1)) 
        infer = c(infer,infer)
    
    if(inv && !is.null(misc$tran)) {
        if (!is.null(misc$inv.lbl)) {
            names(result)[1] = misc$inv.lbl
            if (!is.null(misc$log.contrast))  # contrast of logs - relabel as ratios
                for (ell in seq_along(lbls)){
                    lbls[[ell]] = factor(lbls[[ell]])
                    levels(lbls[[ell]]) = gsub(" - ", " / ", levels(lbls[[ell]]))
                }
        }
        else
            names(result)[1] = "response"
    }
    
    attr(result, "link") = NULL
    estName = names(result)[1]
    
    mesg = misc$initMesg
    
    # Look for "mesg" attribute in dffun
    if (!is.null(dfm <- attr(object@dffun, "mesg")))
        mesg = c(mesg, paste("Degrees-of-freedom method:", dfm))
    
    ### Add an annotation when we show results on lp scale and
    ### there is a transformation
    if (!inv && !is.null(link)) {
        mesg = c(mesg, paste("Results are given on the", link$name, "(not the response) scale."))
    }
    if (inv && !is.null(link$unknown)) {
        mesg = c(mesg, paste0('Unknown transformation "', link$name, '": no transformation done'))
        inv = FALSE
        link = NULL
    }
    if(inv && bias.adjust && !is.null(link)) {
        link = .make.bias.adj.link(link, sigma)
        bias.adjust = attr(link, "bias.adjust")  # disables later message if skipped
    }
    
    # et = 1 if a prediction, 2 if a contrast (or unmatched or NULL), 3 if pairs
    et = pmatch(c(misc$estType, "c"), c("prediction", "contrast", "pairs"), nomatch = 2)[1]
    
    by.size = nrow(grid)
    by.rows = .find.by.rows(grid, by)
    if (!is.null(by)) {
        if (length(unique(sapply(by.rows, length))) > 1) {
            by.size = misc$famSize = "(varies)"
        }
        else
            by.size = by.size / length(by.rows)
    }
    fam.info = c(misc$famSize, by.size, et)
    cnm = NULL
    adjust = tolower(adjust)
    if (adjust %in% c("bh", "by"))  # special cases in p.adjust.methods
        adjust = toupper(adjust)
    
    # get vcov matrix only if needed (adjust == "mvt")
    corrmat = sch.rank = NULL
    if (adjust %.pin% "mvt") { ##(!is.na(pmatch(adjust, "mvt"))) {
        vvv = vcov(object)
        if(any(is.na(vvv))) {
            warning("'mvt' adjustment not available due to missing values")
            corrmat = .diag(rep(NA, nrow(vvv)))
        }
        else
            corrmat = cov2cor(vvv)
        attr(corrmat, "by.rows") = by.rows
    }
    else if (adjust %.pin% "scheffe") {  ##(!is.na(pmatch(adjust, "scheffe"))) {
        if(is.null(sch.rank <- .match.dots("scheffe.rank", ...)))
            sch.rank = sapply(by.rows, function(.) qr(zapsmall(object@linfct[., , drop = FALSE]))$rank)
        if(length(unique(sch.rank)) > 1)
            fam.info[1] = "uneven"   # This forces ragged.by = TRUE in .adj functions
    }
    
    # Add calculated columns
    if(!missing(calc) || !is.null(calc <- misc$calc)) {
        env = c(result, grid[setdiff(names(grid), names(result))])
        for (v in names(calc)) {
            elt = rev(as.character(calc[[v]]))[1] # pick out rhs if a formula
            val = try(eval(parse(text = elt), envir = env), silent = TRUE)
            if(!inherits(val, "try-error"))
                result[[v]] = env[[v]] = val
            else
                warning("The column '", v, "' could not be calculated, ",
                " so it is omitted")
        }
    }
    
    
    # add linkname attribute
    if (two.trans)
        linkname = paste0(link$name, "[", attr(summ.unlink, "linkname"), "]")
    else
        linkname = link$name
    
    if(infer[1]) { # add CIs
        acv = .adj.critval(result[[1]], level, result$df, adjust, fam.info, side, corrmat, by.rows, sch.rank)
        ###adjust = acv$adjust # in older versions, I forced same adj method for tests
        cv = acv$cv
        cv = switch(side + 2, cbind(-Inf, cv), cbind(-cv, cv), cbind(-cv, Inf))
        cnm = if (zFlag) c("asymp.LCL", "asymp.UCL") else c("lower.CL","upper.CL")
        if(!is.null(misc$.predFlag)) {
            cnm = c("lower.PL", "upper.PL")
            sigma = misc$sigma
            mesg = c(mesg, paste0(
                "Prediction intervals and SEs are based on an error SD of ", 
                .fmt.sigma(sigma)))
            estName = names(result)[1] = "prediction"
        }
        if (!two.trans) {
            result[[cnm[1]]] = result[[1]] + cv[, 1]*result$SE
            result[[cnm[2]]] = result[[1]] + cv[, 2]*result$SE
        }
        else {
            result[cnm] = summ.unlink[cnm]
        }
        mesg = c(mesg, paste("Confidence level used:", level), acv$mesg)
        if (inv) {
            clims = with(link, cbind(linkinv(result[[cnm[1]]]), linkinv(result[[cnm[2]]])))
            tmp = apply(clims, 1, function(x) { 
                z = diff(x); ifelse(is.na(z), 0, z) })
            idx = if (all(tmp >= 0)) 1:2 else 2:1
            result[[cnm[1]]] = clims[, idx[1]]
            result[[cnm[2]]] = clims[, idx[2]]
            mesg = c(mesg, paste("Intervals are back-transformed from the", linkname, "scale"))
        }
    }
    if(infer[2]) { # add tests
        tnm = ifelse (zFlag, "z.ratio", "t.ratio")
        tail = ifelse(side == 0, -sign(abs(delta)), side)
        if(!two.trans) {
            result[["null"]] = null
            if (inv && !is.null(link))
                result[["null"]] = link$linkinv(null)
            if (all(result$null == 0))
                result[["null"]] = NULL
            if (side == 0) {
                if (delta == 0) # two-sided sig test
                    t.ratio = result[[tnm]] = (result[[1]] - null) / result$SE
                else
                    t.ratio = result[[tnm]] = (abs(result[[1]] - null) - delta) / result$SE
            }
            else {
                t.ratio = result[[tnm]] = (result[[1]] - null + side * delta) / result$SE            
            }
            apv = .adj.p.value(t.ratio, result$df, adjust, fam.info, tail, corrmat, by.rows, sch.rank)
            adjust = apv$adjust   # in case it was abbreviated
            result$p.value = apv$pval
            mesg = c(mesg, apv$mesg)
        }
        else {
            result$null = ifelse(is.null(summ.unlink$null), link$linkinv(0), link$linkinv(summ.unlink$null))
            result[[tnm]] = summ.unlink[[tnm]]
            result$p.value = summ.unlink$p.value
            apv = .adj.p.value(0, result$df, adjust, fam.info, tail, corrmat, by.rows, sch.rank)
            # we ignore everything about apv except the message
            mesg = c(mesg, apv$mesg)
        }
        
        # Handle cross-adjustments
        if ( (length(by.rows) > 1) && 
             (length(len <- sapply(by.rows, length)) > 1) &&
             is.na(pmatch(cross.adjust, "none")) ) {
            val = c("sidak", p.adjust.methods)
            w = pmatch(tolower(cross.adjust), tolower(val))
            if (!is.na(w)) {
                bridx = unlist(by.rows)
                cross.adjust = val[w]
                mat = matrix(result$p.value[bridx], nrow = len)
                apv = apply(mat, 1, function(p) {
                    if (w > 1)   p.adjust(p, cross.adjust)
                    else         1 - (1 - p)^ncol(mat)
                })
                result$p.value[bridx] = as.numeric(t(apv))
                mesg = c(mesg, paste("Cross-group P-value adjustment:", cross.adjust, 
                                     "method for", ncol(mat), "tests"))
            }
            else
                message("Invalid cross-adjustment method: '", cross.adjust, "'")
        }
        
        if (delta > 0)
            mesg = c(mesg, paste("Statistics are tests of", c("nonsuperiority","equivalence","noninferiority")[side+2],
                                 "with a threshold of", signif(delta, 5)))
        if(tail != 0) 
            mesg = c(mesg, paste("P values are ", ifelse(tail<0,"left-","right-"),"tailed", sep=""))
        if (inv) 
            mesg = c(mesg, paste("Tests are performed on the", linkname, "scale"))
    }
    if (inv) {
        result[["SE"]] = with(link, abs(mu.eta(result[[1]]) * result[["SE"]]))
        result[[1]] = with(link, linkinv(result[[1]]))
        if(bias.adjust)
            mesg = c(mesg, paste("Bias adjustment applied based on sigma =", 
                                 .fmt.sigma(sigma)))
    }
    
    if (length(misc$avgd.over) > 0) {
        qual = attr(misc$avgd.over, "qualifier")
        if (is.null(qual)) qual = ""
        mesg = c(paste0("Results are averaged over", qual, " the levels of: ",
                        paste(misc$avgd.over, collapse = ", ")), mesg)
    }
    summ = cbind(lbls, result)
    attr(summ, "estName") = estName
    attr(summ, "clNames") = cnm  # will be NULL if infer[1] is FALSE
    if (is.null(misc$pri.vars) || length(misc$pri.vars) == 0)
        misc$pri.vars = names(object@levels)
    attr(summ, "pri.vars") = setdiff(union(misc$pri.vars, misc$by.vars), c(by, ".wgt.", ".offset."))
    attr(summ, "by.vars") = by
    attr(summ, "adjust") = adjust
    attr(summ, "side") = side
    attr(summ, "delta") = delta
    attr(summ, "type") = type
    attr(summ, "mesg") = unique(mesg)
    attr(summ, "linkname") = linkname
    class(summ) = c("summary_emm", "data.frame")
    summ
}

# S3 predict method

#' @rdname summary.emmGrid
#' @order 4
#' @method predict emmGrid
#' @param interval Type of interval desired (partial matching is allowed): 
#' \code{"none"} for no intervals,
#'   otherwise confidence or prediction intervals with given arguments, 
#'   via \code{\link{confint.emmGrid}}. 
#'   Note: prediction intervals are not available
#'   unless the model family is \code{"gaussian"}.
#'   
#' @export
#' @return \code{predict} returns a vector of predictions for each row of \code{object@grid}.
predict.emmGrid <- function(object, type, 
                            interval = c("none", "confidence", "prediction"),
                            level = 0.95,
                            bias.adjust = get_emm_option("back.bias.adj"), sigma, 
                            ...) 
{
    # update with any "summary" options
    opt = get_emm_option("summary")
    if(!is.null(opt)) {
        opt$object = object
        object = do.call("update.emmGrid", opt)
    }

    interval = match.arg(interval)
    if (interval %in% c( "confidence", "prediction")) {
        if (interval == "prediction") {
            ok = object@misc$.predFlag = .chk.predict(object)
            if (!ok)
                return(NULL)
        }
        return(confint.emmGrid(object, type = type, level = level, 
                               bias.adjust = bias.adjust, sigma = sigma, ...))
    }
    
    if (missing(type))
        type = .get.predict.type(object@misc)
    else
        type = .validate.type(type)
    
    # if there are two transformations and we want response, then we need to undo both
    if ((type == "response") && (!is.null(object@misc$tran2)))
        object = regrid(object, transform = "mu")
    
    pred = .est.se.df(object, do.se = FALSE, ...)
    result = pred[[1]]
    
    if (type %in% c("response", "mu", "unlink")) {
        link = attr(pred, "link")
        if (!is.null(link)) {
            if (bias.adjust) {
                if(missing(sigma))
                    sigma = object@misc$sigma
                link = .make.bias.adj.link(link, sigma)
            }
            result = link$linkinv(result)
            if (is.logical(link$unknown) && link$unknown)
                warning("Unknown transformation: \"", link$name, "\" -- no transformation applied.")
        }
    }
    result
}

# as.data.frame method
#' @rdname summary.emmGrid
#' @order 5
#' @param x object of the given class
#' @param destroy.annotations Logical value. If \code{FALSE}, an object of class
#'   \code{summary_emm} is returned (which inherits from \code{data.frame}),
#'   but if displayed, details like confidence levels, P-value adjustments, 
#'   transformations, etc. are also shown. But unlike the result
#'   of \code{summary}, the number of digits displayed
#'   is obtained from \code{getOption("digits")} rather than using the
#'   optimal digits algorithm we usually use. Thus, it is formatted more like a 
#'   regular data frame, but with any annotations and groupings still intact.
#'   If \code{TRUE} (not recommended), a \dQuote{plain vanilla} data frame is 
#'   returned, based on \code{row.names} and \code{check.names}.
#' @param row.names passed to \code{\link{as.data.frame}}
#' @param optional required argument, but ignored in \code{as.data.frame.emmGrid}
#' @param check.names passed to \code{\link{data.frame}}
#' @return The \code{as.data.frame} method returns an object that inherits 
#'   from \code{"data.frame"}.
#' @export
#' @method as.data.frame emmGrid
as.data.frame.emmGrid = function(x, 
                                 row.names = NULL, optional, check.names = TRUE, 
                                 destroy.annotations = FALSE, ...) {
    rtn = summary.emmGrid(x, ...)
    attr(rtn, "digits") = getOption("digits")
    if(destroy.annotations)
        rtn = as.data.frame.data.frame(rtn, row.names = row.names, check.names = check.names)
    rtn
}


#' @rdname summary.emmGrid
#' @order 6
#' @method [ summary_emm
#' @param as.df Logical value. With \code{x[..., as.df = TRUE]}, the result is
#'   object is coerced to a \code{\link{data.frame}} before the subscripting 
#'   is applied. With \code{as.df = FALSE}, the result is
#'   returned as a \code{summary_emm} object when possible.
#' @export
"[.summary_emm" = function(x, ..., as.df = FALSE) {
    attr(x, "by.vars") = NULL
    rtn = as.data.frame.data.frame(x)[...]
    if ((!as.df) && (!is.null(attr(rtn, "estName"))))
        class(rtn) = c("summary_emm", "data.frame")
    rtn
}    


# Computes the quadratic form y'Xy after subsetting for the nonzero elements of y
.qf.non0 = function(X, y) {
    ii = (zapsmall(y) != 0)
    rtn = if (any(ii))
        sum(y[ii] * (X[ii, ii, drop = FALSE] %*% y[ii]))
    else 0
    if (!is.na(rtn) && (rtn < 0)) {
        warning("Negative variance estimate obtained!")
        rtn = NaN
    }
    rtn
}


# Workhorse for getting estimates etc.
.est.se.df = function(object, do.se=TRUE, tol = get_emm_option("estble.tol"), ...) {
    if (nrow(object@grid) == 0) {
        result = data.frame(NA, NA, NA)
        names(result) = c(object@misc$estName, "SE", "df")
        return(result[-1, ])
    }
    misc = object@misc
    use.elts = .reconcile.elts(object)

    if (!is.null(hook <- misc$estHook)) {
        if (is.character(hook)) hook = get(hook)
        result = hook(object, do.se=do.se, tol=tol, ...)
    }
    else {
        active = which(!is.na(object@bhat))
        bhat = object@bhat[active]
        result = t(apply(object@linfct[use.elts, , drop = FALSE], 1, function(x) {
            if (!any(is.na(x)) && estimability::is.estble(x, object@nbasis, tol)) {
                x = x[active]
                est = sum(bhat * x)
                if(do.se) {
                    se = sqrt(.qf.non0(object@V, x))
                    df = object@dffun(x, object@dfargs)
                    ### Brute force: if (is.na(df)) df = Inf    ### Added
                }
                else # if these unasked-for results are used, we're bound to get an error!
                    se = df = 0
                c(est, se, df)
            }
            else c(NA,NA,NA)
        }))
            
        if (!is.null(object@grid$.offset.))
            result[, 1] = result[, 1] + object@grid$.offset.[use.elts]
    }
    result[1] = as.numeric(result[1]) # silly bit of code to avoid getting a data.frame of logicals if all are NA
    result = as.data.frame(result)
    names(result) = c(misc$estName, "SE", "df")
    if (!is.null(misc$.predFlag)) {
        if (is.null(misc$sigma))
            stop("No 'sigma' is available for obtaining Prediction intervals.", 
                 call. = FALSE)
        result$SE = sqrt(result$SE^2 + misc$sigma^2)
    }
    if (!is.null(misc$tran)) {
        attr(result, "link") = .get.link(misc)
        if(is.character(misc$tran) && (misc$tran == "none"))
            attr(result, "link") = NULL
    }
    result
}

# workhorse for nailing down the link function
.get.link = function(misc) {
    link = if(is.character(misc$tran))
        .make.link(misc$tran)
    else if (is.list(misc$tran))
        misc$tran
    else 
        NULL
    
    if (is.list(link)) {  # See if multiple of link is requested, or variable is offset
        if (!is.null(misc$tran.mult) || !is.null(misc$tran.offset)) {
            name = link$name
            mult = link$mult = ifelse(is.null(misc$tran.mult), 1, misc$tran.mult)
            off = link$offset = ifelse(is.null(misc$tran.offset), 0, misc$tran.offset)
            link = with(link, list(
                linkinv = function(eta) linkinv(eta / mult) - offset,
                mu.eta = function(eta) mu.eta(eta / mult) / mult))
            if(mult != 1)
                name =  paste0(round(mult, 3), "*", name)        
            if(off != 0)
                name = paste0(name, "(mu + ", round(off, 3), ")")
            link$name = name         
        }
    }
    
    if (!is.null(link) && is.null(link$name))
        link$name = "linear-predictor"
    link
}

# patch-in alternative back-transform stuff for bias adjustment
# Currently, we just use a 2nd-order approx for everybody:
#   E(h(nu + E))  ~=  h(nu) + 0.5*h"(nu)*var(E)
# We also return an attribute "bias.adjust" which is TRUE if ok, FALSE if we aborted
.make.bias.adj.link = function(link, sigma) {
    ###if (is.null(sigma) || (!is.null(sigma) && (is.na(sigma) || (sigma < 0)))) { ###old code
    if(is.null(sigma) || (length(sigma) == 0) || is.na(sigma[1]) || (sigma[1] < 0)) {
         warning("Bias adjustment skipped: No valid 'sigma' provided\n", 
                "(And perhaps bias.adjust should NOT be used; see ? summary.emmGrid)", 
                call. = FALSE)
        attr(link, "bias.adjust") = FALSE
        return(link)
    }
    link$inv = link$linkinv
    link$der = link$mu.eta
    link$sigma22 = sigma^2 / 2
    link$der2 = function(eta) with(link, 1000 * (der(eta + .0005) - der(eta - .0005)))
    link$linkinv = function(eta) with(link, inv(eta) + sigma22 * der2(eta))
    link$mu.eta = function(eta) with(link, der(eta) +
                                         1000 * sigma22 * (der2(eta + .0005) - der2(eta - .0005)))
    attr(link, "bias.adjust") = TRUE
    link
}
####!!!!! TODO: Re-think whether we are handling Scheffe adjustments correctly
####!!!!!       if/when we shift around 'by' specs, etc.

### utility for changing adjustments
.chg.adjust = function(old, new, reason) {
    message("Note: adjust = \"", old, "\" was changed to \"", new, 
            "\"\nbecause \"", old, "\" is ", reason)
    new
}

# utility to compute an adjusted p value
# tail is -1, 0, 1 for left, two-sided, or right
# Note fam.info is c(famsize, ncontr, estTypeIndex)
# lsmeans >= 2.14: added corrmat arg, dunnettx & mvt adjustments
# emmeans > 1.3.4: we have sch.rank of same length as by.rows
# NOTE: corrmat is NULL unless adjust == "mvt"
.adj.p.value = function(t, DF, adjust, fam.info, tail, corrmat, by.rows, sch.rank) {
    fam.size = fam.info[1]
    n.contr = fam.info[2]
    et = as.numeric(fam.info[3])

    # Force no adjustment when just one test, unless we're using scheffe
    if ((n.contr == 1) && !(adjust %.pin% "scheffe"))  ##(pmatch(adjust, "scheffe", 0) != 1)) 
        adjust = "none"
    
    # check if we need to adjust adaptively for only estimable cases
    adaptive = (adjust != "none") && any(is.na(t))
    
    # do a pmatch of the adjust method
    adj.meths = c("sidak", "tukey", "scheffe", "dunnettx", "mvt", p.adjust.methods)
    
    k = pmatch(adjust, adj.meths)
    if(is.na(k))
        stop("Adjust method '", adjust, "' is not recognized or not valid")
    adjust = adj.meths[k]
    if ((tail != 0) && (adjust %in% c("tukey", "scheffe", "dunnettx"))) # meth not approp for 1-sided
        adjust = .chg.adjust(adjust, "sidak", "not appropriate for one-sided inferences")
    if ((et != 3) && adjust == "tukey") # not pairwise
        adjust = .chg.adjust(adjust, "sidak", "only appropriate for one set of pairwise comparisons")
    
    ragged.by = (is.character(fam.size) || adjust %in% c("mvt", setdiff(p.adjust.methods, "none")))   # flag that we need to do groups separately
    ragged.by = ragged.by | adaptive   # keep it ragged if we need adaptive adj
    if (!ragged.by)
        by.rows = list(seq_along(t))       # not ragged, we can do all as one by group
    
    # asymptotic results when df is NA
    DF[is.na(DF)] = Inf
    
    # if estType is "prediction", use #contrasts + 1 as family size
    # (produces right Scheffe CV; Tukey ones are a bit strange)
    # deprecated - used to try to keep track of scheffe.adj = ifelse(et == 1, 0, - 1)
    if (tail == 0)
        p.unadj = 2*pt(abs(t), DF, lower.tail=FALSE)
    else
        p.unadj = pt(t, DF, lower.tail = (tail<0))
    
    pval = numeric(length(t))
    for(jj in seq_along(by.rows)) { 
        rows = by.rows[[jj]]
        unadj.p = p.unadj[rows]
        abst = abs(t[rows])
        df = DF[rows]
        if (ragged.by) {
            n.contr = max(sum(!is.na(unadj.p)), 1)
            fam.size = (1 + sqrt(1 + 8*n.contr)) / 2   # tukey family size - e.g., 6 pairs -> family of 4
        }
        if (adjust %in% p.adjust.methods) # simple now because we forced ragged.rows
            pval[rows] = p.adjust(unadj.p, adjust)
        else pval[rows] = switch(adjust,
                           sidak = 1 - (1 - unadj.p)^n.contr,
                           # NOTE: tukey, scheffe, dunnettx all assumed 2-sided!
                           tukey = ptukey(sqrt(2)*abst, fam.size, zapsmall(df), lower.tail=FALSE),
                           scheffe = pf(t[rows]^2 / (sch.rank[jj]), sch.rank[jj], 
                                        df, lower.tail = FALSE),
                           dunnettx = 1 - .pdunnx(abst, n.contr, df),
                           mvt = 1 - .my.pmvt(t[rows], df, corrmat[rows,rows,drop=FALSE], -tail) # tricky - reverse the tail because we're subtracting from 1 
        )
    }

    chk.adj = match(adjust, c("none", "tukey", "scheffe"), nomatch = 99)
    
    if (ragged.by) {
        nc = max(sapply(by.rows, length))
        fs = (1 + sqrt(1 + 8*nc)) / 2
        scheffe.dim = "(varies)"
    }
    else {
        nc = n.contr
        fs = fam.size
        scheffe.dim = sch.rank[1]
    }
    do.msg = (chk.adj > 1) && !((fs < 3) && (chk.adj < 10)) ### WAS (chk.adj > 1) && (nc > 1) && !((fs < 3) && (chk.adj < 10)) 
    if (do.msg) {
#         xtra = if(chk.adj < 10) paste("a family of", fam.size, "tests")
#         else             paste(n.contr, "tests")
        xtra = if (!adaptive || (length(by.rows) == 1)) switch(adjust, 
                      tukey = paste("for comparing a family of", fam.size, "estimates"),
                      scheffe = paste("with rank", scheffe.dim),
                      paste("for", n.contr, "tests")
                )
        else switch(adjust, 
                    tukey = "for varying family sizes",
                    scheffe = "with varying rank",
                    "for varying numbers of tests"
        )
        
        mesg = paste("P value adjustment:", adjust, "method", xtra)
    }
    else mesg = NULL
    list(pval = pval, mesg = mesg, adjust = adjust)
}

# Code needed for an adjusted critical value
# returns a list similar to .adj.p.value
# lsmeans >= 2.14: Added tail & corrmat args, dunnettx & mvt adjustments
# emmeans > 1.3.4: Added sch.rank
# NOTE: corrmat is NULL unless adjust == "mvt"
.adj.critval = function(est, level, DF, adjust, fam.info, tail, corrmat, by.rows, sch.rank) {
    mesg = NULL
    
    fam.size = fam.info[1]
    n.contr = fam.info[2]
    et = as.numeric(fam.info[3])
    
    ragged.by = (is.character(fam.size))   # flag that we need to do groups separately
    if (!ragged.by && (n.contr == 1) && !(adjust %.pin% "scheffe"))  ##(pmatch(adjust, "scheffe", 0) != 1)) # Force no adjustment when just one interval, unless using Scheffe
        adjust = "none"
    
    adj.meths = c("sidak", "tukey", "scheffe", "dunnettx", "mvt", "bonferroni", "none")
    k = pmatch(adjust, adj.meths)
    if(is.na(k))
        k = which(adj.meths == "bonferroni") 
    adjust = adj.meths[k]
    
    # do we need to adapt for different by-grouup sizes?
    adaptive = any(is.na(est))
    
    ragged.by = ragged.by || adaptive || (adjust == "mvt")
    
    if (!ragged.by && (length(unique(DF)) == 1))
        by.rows = list(seq_along(DF))       # not ragged, we can do all as one by group
    
    if ((tail != 0) && (adjust %in% c("tukey", "scheffe", "dunnettx"))) # meth not approp for 1-sided
        adjust = .chg.adjust(adjust, "sidak", "not appropriate for one-sided inferences")
    if ((et != 3) && adjust == "tukey") # not pairwise
        adjust = .chg.adjust(adjust, "sidak", "only appropriate for one set of pairwise comparisons")
    
    # asymptotic results when df is NA
    DF[is.na(DF)] = Inf
    #### No longer used scheffe.adj = ifelse(et == 1, 0, - 1)
    
    chk.adj = match(adjust, c("none", "tukey", "scheffe"), nomatch = 99)
    if (ragged.by) {
        nc = max(sapply(by.rows, length))
        fs = (1 + sqrt(1 + 8*nc)) / 2
        scheffe.dim = "(varies)"
    }
    else {
        nc = n.contr
        fs = fam.size
        scheffe.dim = sch.rank[1]
    }
    do.msg = (chk.adj > 1) && ###   (nc > 1) && 
        !((fs < 3) && (chk.adj < 10)) 
    
    adiv = ifelse(tail == 0, 2, 1) # divisor for alpha where needed
    
    ###cvs = lapply(by.rows, function(rows) {
    cv = numeric(sum(sapply(by.rows, length)))
    for (jj in seq_along(by.rows)) { ####(rows in by.rows) {
        rows = by.rows[[jj]]
        df = DF[rows]
        if (ragged.by) {
            n.contr = max(1, sum(!is.na(est[rows])))
            fam.size = (1 + sqrt(1 + 8*n.contr)) / 2   # tukey family size - e.g., 6 pairs -> family of 4
        }
        cv[rows] = switch(adjust,
               none = -qt((1-level)/adiv, df),
               sidak = -qt((1 - level^(1/n.contr))/adiv, df),
               bonferroni = -qt((1-level)/n.contr/adiv, df),
               tukey = qtukey(level, fam.size, df) / sqrt(2),
               scheffe = sqrt((sch.rank[jj]) * qf(level, sch.rank[jj], df)),
               dunnettx = .qdunnx(level, n.contr, df),
               mvt = .my.qmvt(level, df, corrmat[rows, rows, drop = FALSE], tail)
        )
    }
    
    if (do.msg) {
        #        xtra = if(chk.adj < 10) paste("a family of", fam.size, "estimates")
        #        else             paste(n.contr, "estimates")
        xtra = if(!adaptive || (length(by.rows) == 1)) 
            switch(adjust, 
                   tukey = paste("for comparing a family of", fam.size, "estimates"),
                   scheffe = paste("with rank", scheffe.dim),
                   paste("for", n.contr, "estimates"))
        else switch(adjust,
                    tukey = "for varying family sizes",
                    scheffe = "with varying rank",
                    "for varying numbers of estimates")
        
        mesg = paste("Conf-level adjustment:", adjust, "method", xtra)
    }
    
    
    list(cv = cv, mesg = mesg, adjust = adjust)
}


### My own functions to ease access to mvt functions
### These use one argument at a time and expands each (lower, upper) or p to a k-vector
### Use tailnum = -1, 0, or 1
### NOTE: corrmat needs "by.rows" attribute to tell which rows
###   belong to which submatrix.
.my.pmvt = function(x, df, corrmat, tailnum) {
    lower = switch(tailnum + 2, -Inf, -abs(x), x)
    upper = switch(tailnum + 2, x, abs(x), Inf)
    by.rows = attr(corrmat, "by.rows")
    if (is.null(by.rows)) 
        by.rows = list(seq_len(length(x)))
    by.sel = numeric(length(x))
    for (i in seq_along(by.rows))
        by.sel[by.rows[[i]]] = i
    df = .fix.df(df)
    apply(cbind(lower, upper, df, by.sel), 1, function(z) {
        idx = by.rows[[z[4]]]
        k = length(idx)
        pval = try(mvtnorm::pmvt(rep(z[1], k), rep(z[2], k), 
                        df = as.integer(z[3]), corr = corrmat[idx, idx]), 
                    silent = TRUE)
        if (inherits(pval, "try-error"))   NA
        else                               pval
    })
}

# Vectorized for df but needs p to be scalar
.my.qmvt = function(p, df, corrmat, tailnum) {
    tail = c("lower.tail", "both.tails", "lower.tail")[tailnum + 2] 
    df = .fix.df(df)
    by.rows = attr(corrmat, "by.rows")
    if (is.null(by.rows)) 
        by.rows = list(seq_len(length(df)))
    by.sel = numeric(length(df))
    for (i in seq_along(by.rows))
        by.sel[by.rows[[i]]] = i
    # If df all equal, compute just once for each by group
    eq.df = (diff(range(df)) == 0)
    i1 = if (eq.df)   sapply(by.rows, function(r) r[1])
         else         seq_along(df)
    result = apply(cbind(p, df[i1], by.sel[i1]), 1, function(z) {
        idx = by.rows[[z[3]]]
        cv = try(mvtnorm::qmvt(z[1], tail = tail, 
                    df = as.integer(z[2]), corr = corrmat[idx, idx])$quantile,
                 silent = TRUE)
        if (inherits(cv, "try-error"))     NA
        else                               cv
    })
    if (eq.df) {
        res = result
        result = numeric(length(df))
        for(i in seq_along(by.rows))
            result[by.rows[[i]]] = res[i]
    } 
    result
}

# utility to get appropriate integer df
.fix.df = function(df) {
    sapply(df, function(d) {
        if (d > 0) d = max(1, d)
        if (is.infinite(d) || (d > 9999)) d = 0
        floor(d + .25) # tends to round down
    })
}

### My approximate dunnett distribution 
### - a mix of the Tukey cdf and Sidak-corrected t
.pdunnx = function(x, k, df, twt = (k - 1)/k) {
    tukey = ptukey(sqrt(2)*x, (1 + sqrt(1 + 8*k))/2, df)
    sidak = (pf(x^2, 1, df))^k
    twt*tukey + (1 - twt)*sidak
}

# Uses linear interpolation to get quantile
.qdunnx = function(p, k, df, ...) {
     if (k < 1.005)
         return(qt(1 - .5*(1 - p), df))
    xtuk = qtukey(p, (1 + sqrt(1 + 8*k))/2, df) / sqrt(2)
    xsid = sqrt(qf(p^(1/k), 1, df))
    fcn = function(x, d) 
        .pdunnx(x, k, d, ...) - p
    apply(cbind(xtuk, xsid, df), 1, function(r) {
        if (abs(diff(r[1:2])) < .0005)
            return (r[1])
        x = try(uniroot(fcn, r[1:2], tol = .0005, d = r[3]), silent = TRUE)
        if (inherits(x, "try-error")) {
            warning("Root-finding failed; using qtukey approximation for Dunnett quantile")
            return(xtuk)
        }
        else
            x$root
    })
}



### Support for different prediction types ###

# Valid values for type arg or predict.type option
#   "link", "lp", "linear" are all legal but equivalent
#   "mu" and "response" are usually equivalent -- but in a GLM with a response transformation,
#      "mu" (or "unlink") would back-transform the link only, "response" would do both
.valid.types = c("link","lp","linear", "response", "mu", "unlink")

# get "predict.type" option from misc, and make sure it's legal
.get.predict.type = function(misc) {
    type = misc$predict.type
    if (is.null(type))
        .valid.types[1]
    else
        .validate.type(type)
}

# check a "type" arg to make it legal
# NOTE: if not matched, returns "link", i.e., no back-transformation will be done
.validate.type = function (type) {
    type = .valid.types[pmatch(type, .valid.types, 1)]
    if (length(type) > 1) {
        type = type[1]
        warning("You specified more than one prediction type. Only type = \"", type, "\" was used")
    }
    type
}





# left-or right-justify column labels for m depending on "l" or "R" in just
.just.labs = function(m, just) {
    nm = dimnames(m)[[2]]
    for (j in seq_len(length(nm))) {
        if(just[nm[j]] == "L") 
            nm[j] = format(nm[j], width = nchar(m[1,j]), justify = "left")
    }
    dimnames(m) = list(rep("", nrow(m)), nm)
    m
}

# Format a data.frame produced by summary.emmGrid
#' @method print summary_emm
#' @export
print.summary_emm = function(x, ..., digits=NULL, quote=FALSE, right=TRUE, export = FALSE) {
    estn = attr(x, "estName")
    if (is.null(estn)) # uh-oh, somebody messed it up - so Hail Mary
        return(invisible(print(data.frame(x))))
    
    if(!is.null(attr(x, "digits")))
        digits = attr(x, "digits")
    
    pval.digits = suppressWarnings(as.integer(get_emm_option("pval.digits")))[1]
    if(is.na(pval.digits) || (pval.digits < 2) || (pval.digits > 6)) {
        pval.digits = ifelse(is.na(pval.digits), emm_defaults$pval.digits, pval.digits)
        pval.digits = max(2, min(6, pval.digits))
        emm_options(pval.digits = pval.digits)
    }

    test.stat.names = c("t.ratio", "z.ratio", "F.ratio", "T.square")  # format these w 3 dec places
    x.save = x
    if(export) x.save = list()
    for(i in which(sapply(x, is.matrix))) 
        x[[i]] = NULL   # hide matrices
    for (i in seq_along(names(x)))   # zapsmall the numeric columns
        if (is.numeric(x[[i]]))  
            x[[i]] = zapsmall(x[[i]])
    just = sapply(x, function(col) if(is.numeric(col)) "R" else "L")  ### was later
    if (!is.null(x$df)) x$df = round(x$df, 2)
    for (nm in test.stat.names)
        if(!is.null(x[[nm]]))
            x[[nm]] = format(round(x[[nm]], 3), nsmall = 3, sci = FALSE)
    if (!is.null(x$p.value)) {
        pval_min = 10^-(pval.digits)
        pval_min_display = format(pval_min, nsmall = pval.digits, sci = FALSE) 
        x$p.value = ifelse(x$p.value < pval_min, paste0("<", pval_min_display),
                            format(round(x$p.value, pval.digits), nsmall = pval.digits, sci = FALSE))
    }

    est = x[[estn]]
    if (get_emm_option("opt.digits") && is.null(digits)) {
        qnms = names(x)
        if (!is.null(x$SE))
            x$SE = signif(x$SE, 3)
        if (!is.null(x[["SE"]]))
            tmp = est + x[["SE"]] * cbind(rep(-2, nrow(x)), 0, 2)
        # else if (!is.null(x[["lower.HPD"]]))
        #     tmp = x[, c("lower.HPD", estn, "upper.HPD"), drop = FALSE]
        else if (any(startsWith(names(x), "lower."))) {
            lwr = qnms[startsWith(qnms, "lower.")]
            upr = qnms[startsWith(qnms, "upper.")]
            tmp = x[, c(lwr, estn, upr), drop = FALSE]
        }
        else tmp = NULL
        if (!is.null(tmp))
            digits = max(apply(tmp, 1, .opt.dig))
    }
    if (any(is.na(est))) {
        x[[estn]] = format(est, digits=digits)
        x[[estn]][is.na(est)] = "nonEst"
    }
    xc = as.matrix(format.data.frame(x, digits=digits, na.encode=TRUE))
    m = apply(rbind(just, names(x), xc), 2, function(x) {
        w = max(sapply(x, nchar))
        if (x[1] == "R") format(x[-seq_len(2)], width = w, justify="right")
        else format(x[-seq_len(2)], width = w, justify="left")
    })
    if(!is.matrix(m)) m = t(as.matrix(m))
    by.vars = attr(x, "by.vars")
    if (is.null(by.vars)) {
        m = .just.labs(m, just)
        if (export)
            x.save$summary = m
        else {
            print(m, quote=FALSE, right=TRUE)
            cat("\n")
        }
    }
    else { # separate listing for each by variable
        m = .just.labs(m[, setdiff(names(x), by.vars), drop = FALSE], just)
        if(export) 
            x.save$summary = list()
        pargs = unname(as.list(x[, by.vars, drop=FALSE]))
        # We want labels to come out in order though...
        ord = do.call(order, rev(pargs))
        m = m[ord, , drop = FALSE]
        xc = xc[ord, , drop = FALSE]
        pargs = unname(as.list(x[ord, by.vars, drop=FALSE]))
        pargs$sep = ", "
        lbls = do.call(paste, pargs)
        for (lb in unique(lbls)) {
            rows = which(lbls==lb)
            levs = paste(by.vars, "=", xc[rows[1], by.vars])
            levs = paste(levs, collapse=", ")
            if(export)
                x.save$summary[[levs]] = m[rows, , drop = FALSE]
            else {
                cat(paste(levs, ":\n", sep=""))
                print(m[rows, , drop=FALSE], ..., quote=quote, right=right)
                cat("\n")
            }
        }
    }
    
    msg = unique(attr(x, "mesg"))
    if (!is.null(msg) && !export)
        for (j in seq_len(length(msg))) cat(paste(msg[j], "\n"))
    else (x.save$annotations = msg)
    
    invisible(x.save)
}

#' @method update summary_emm
#' @export
#' @rdname update.emmGrid
#' @order 9
#' @param by.vars,mesg Attributes that can be altered in \code{update.summary_emm}
#' @section Method for \code{summary_emm} objects:
#' This method exists so that we can change the way a summary is displayed,
#' by changing the by variables or the annotations.
#' 
#' @examples
#' ### Compactify results with a by variable
#' update(joint_tests(pigs.rg, by = "source"), by = NULL)
update.summary_emm = function(object, by.vars, mesg, ...) {
    args = match.call()[-1]
    args$object = NULL
    for (nm in names(args))
        attr(object, nm) = args[[nm]]
    object
}

# determine optimum digits to display based on a conf or cred interval
# (but always at least 3)
.opt.dig = function(x) {
    z = range(x) / max(abs(x))
    dz = zapsmall(c(diff(z), z), digits = 8)[1]
    zz = round(1.51 - log(dz, 10))  # approx 1 - log(diff(z/3))
    zz[is.infinite(zz)] = 3   # we get z = Inf when SE is 0
    max(zz, 3, na.rm = TRUE)  
}

# Utility -- When misc$display present, reconcile which elements to use.
# Needed if we messed with previous nesting
.reconcile.elts = function(object) {
    display = object@misc$display
    nrows = nrow(object@grid)
    use.elts = rep(TRUE, nrows)
    if (!is.null(display) && (length(display) == nrows))  
        use.elts = display
    use.elts
}

##############################################################################
#    Copyright (c) 2012-2017 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

# documentation for models

#' Models supported in \pkg{emmeans}
#' 
#' Documentation for models has been moved to a vignette. To access it,
#' use \href{../doc/models.html}{\code{vignette("models", "emmeans")}}.
#' 
#' @name models
NULL

##############################################################################
#    Copyright (c) 2012-2020 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

#' Pairwise P-value plot
#' 
#' Constructs a plot of P values associated with pairwise comparisons of 
#' estimated marginal means.
#' 
#' Factor levels (or combinations thereof) are plotted on the vertical scale, and P values
#' are plotted on the horizontal scale. Each P value is plotted twice -- at
#' vertical positions corresponding to the levels being compared -- and connected by
#' a line segment. Thus, it is easy to visualize which P values are small and large,
#' and which levels are compared. In addition, factor levels are color-coded, and the points
#' and half-line segments appear in the color of the other level.
#' The P-value scale is nonlinear, so as to stretch-out smaller P values and
#' compress larger ones.
#' P values smaller than 0.0004 are altered and plotted in a way that makes 
#'   them more distinguishable from one another.
#' 
#' If \code{xlab}, \code{ylab}, and \code{xsub} are not provided, reasonable labels
#' are created. \code{xsub} is used to note special features; e.g., equivalence
#' thresholds or one-sided tests.
#' 
#' @param emm An \code{emmGrid} object
#' @param method Character or list. Passed to \code{\link{contrast}}, and defines 
#'           the contrasts to be displayed. Any contrast method may be used,
#'           provided that each contrast includes one coefficient of \code{1},
#'           one coefficient of \code{-1}, and the rest \code{0}. That is, calling
#'           \code{contrast(object, method)} produces a set of comparisons, each with
#'           one estimate minus another estimate.
#' @param by Character vector of variable(s) in the grid to condition on. These will
#'           create different panels, one for each level or level-combination.
#'           Grid factors not in \code{by} are the \emph{primary} factors: 
#'           whose levels or level combinations are compared pairwise.
#' @param sort Logical value. If \code{TRUE}, levels of the factor combinations are
#'             ordered by their marginal means. If \code{FALSE}, they appear in
#'             order based on the existing ordering of the factor levels involved.
#'             Note that the levels are ordered the same way in all panels, and in
#'             many cases this implies that the means in any particular panel
#'             will \emph{not} be ordered even when \code{sort = TRUE}.
#' @param values Logical value. If \code{TRUE}, the values of the EMMs are included
#'               in the plot. When there are several side-by-side panels due
#'               to \code{by} variable(s), the labels showing values start
#'               stealing a lot of space from the plotting area; in those cases,
#'               it may be desirable to specify \code{FALSE} or use \code{rows}
#'               so that some panels are vertically stacked.
#' @param rows Character vector of which \code{by} variable(s) are used to define
#'           rows of the panel layout. Those variables in \code{by} not included in 
#'           \code{rows} define columns in the array of panels.
#'           A \code{"."} indicates that only one row
#'           is used, so all panels are stacked side-by-side.
#' @param xlab Character label to use in place of the default for the P-value axis.
#' @param ylab Character label to use in place of the default for the primary-factor axis.
#' @param xsub Character label used as caption at the lower right of the plot.
#' @param plim numeric vector of value(s) between 0 and 1. These are included
#'   among the observed p values so that the range of tick marks includes at
#'   least the range of \code{plim}. Choosing \code{plim = c(0,1)} will ensure
#'   the widest possible range.
#' @param add.space Numeric value to adjust amount of space used for value labels. Positioning
#'                  of value labels is tricky, and depends on how many panels and the
#'                  physical size of the plotting region. This parameter allows the user to
#'                  adjust the position. Changing it by one unit should shift the position by
#'                  about one character width (right if positive, left if negative).
#'                  Note that this interacts with \code{aes$label} below.
#' @param aes optional named list of lists. Entries considered are \code{point}, 
#'   \code{segment}, and \code{label}, and contents are passed to the respective
#'   \code{ggplot2::geom_xxx()} functions. These affect rendering of points, 
#'   line segments joining them, and value labels. 
#'   Defaults are \code{point = list(size = 2)},
#'   \code{segment = list()}, and \code{label = list(size = 2.5)}.
#' @param ... Additional arguments passed to \code{contrast} and \code{\link{summary.emmGrid}}, 
#'   as well as to \code{geom_segment} and \code{geom_label}
#' 
#' @note If \code{emm} is the result of a Bayesian analysis, the plot is based on
#'   summaries with \code{frequentist = TRUE}.
#' @note The \pkg{ggplot2} and \pkg{scales} packages must be installed in order 
#'   for \code{pwpp} to work.
#' @note Additional plot aesthetics are available by adding them to the returned object;
#'   see the examples
#'
#' @seealso A numerical display of essentially the same results is available
#'   from \code{\link{pwpm}}
#' @export
#' @examples
#' pigs.lm <- lm(log(conc) ~ source * factor(percent), data = pigs)
#' emm = emmeans(pigs.lm, ~ percent | source)
#' pwpp(emm)
#' pwpp(emm, method = "trt.vs.ctrl1", type = "response", side = ">")
#'
#' # custom aesthetics:
#' my.aes <- list(point = list(shape = "square"), 
#'                segment = list(linetype = "dashed", color = "red"),
#'                label = list(family = "serif", fontface = "italic"))
#' my.pal <- c("darkgreen", "blue", "magenta", "orange")
#' pwpp(emm, aes = my.aes) + ggplot2::scale_color_manual(values = my.pal)
#' 
pwpp = function(emm, method = "pairwise", by, sort = TRUE, values = TRUE, 
                rows = ".",
                xlab, ylab, xsub = "", plim = numeric(0), add.space = 0, 
                aes, ...) {

        emm = .chk.list(emm, ...)
    if(missing(by)) 
        by = emm@misc$by.vars
    
    ### set up aesthetics
    if(missing(aes))
        aes = list()
    # defaults if other than system ones...
    daes = list(point = list(size = 2), segment = list(), label = list(size = 2.5))
    # fill aes w/ defaults if not present, at either level
    for(a in names(daes)) { 
        if(is.null(aes[[a]]))
            aes[[a]] = daes[[a]]
        else for (b in names(daes[[a]]))
            if (is.null(aes[[a]][[b]]))
                aes[[a]][[b]] = daes[[a]][[b]]
    }
    
    if(rows != "." && !(rows %in% by))
        stop("'rows' must be a subset of the 'by' variables")
    
    args = list(object = emm, method = method, by = by, ...)
    args$interaction = args$simple = args$offset = NULL
    con = do.call(contrast, args)
    
    # We allow the con call to change the by variable via options...
    by = con@misc$by
    
    args = list(object = emm, infer = c(FALSE, FALSE), by = by, frequentist = TRUE, ...)
    emm.summ = do.call(summary.emmGrid, args)
    
    args = list(object = con, infer = c(FALSE, TRUE), frequentist = TRUE, ...)
    args$null = NULL
    con.summ = do.call(summary.emmGrid, args)
    
    if(missing(xlab)) {
        adjust = .cap(attr(con.summ, "adjust"))
        delta = attr(con.summ, "delta")
        side = attr(con.summ, "side")
        xlab = "P value"
        if (adjust != "None")
            xlab = paste0(adjust, "-adjusted ", xlab)
        
        if (delta != 0)
            xsub = paste(c("Nonsuperiority", "Equivalence", "Noninferiority")[side + 2],
                         "test with threshold", delta)
        else
            xsub = c("Left-sided tests", "", "Right-sided tests")[side + 2]
    }
    
    sep = get_emm_option("sep")
    if(missing(ylab))
        ylab = paste(attr(emm.summ, "pri.vars"), collapse = ":")
    
    # figure out levels being compared
    cf = coef(con)
    use = setdiff(names(cf), names(con@misc$orig.grid))
    idx = apply(as.matrix(cf[use]), 2, function(x) {
        if(!all(range(x) == c(-1,1)) || (sum(abs(x)) != 2))
            stop("Each contrast must be a comparison of two estimates")
        c(which(x == 1), which(x == -1))
    })
    primv = setdiff(attr(emm.summ, "pri.vars"), by)
    pf = do.call(paste, c(unname(emm.summ[primv]), sep = sep))
    pemm = suppressMessages(emmeans(emm, primv))
    levs = do.call(paste, c(unname(pemm@grid[primv]), sep = sep))
    if(sort) ord = order(predict(pemm))
    else ord = seq_along(pf)
    pf = emm.summ$pri.fac = factor(pf, levels = levs[ord])
    
    con.summ$plus = pf[idx[1, ]]
    con.summ$minus = pf[idx[2, ]]
    
    estName = attr(emm.summ, "estName")
    
    
########## The rest should probably be done in a separate function ################
    
    .requireNS("ggplot2", 
               "pwpp requires the 'ggplot2' package be installed.", call. = FALSE)
    
        # granulate values in each group so they won't overlap
        # do this on the transformed (plotted) scale
        byr = .find.by.rows(con.summ, by)
        for (r in byr) {
            pv = con.summ$p.value[r]
            con.summ$p.value[r] = gran(pv)
        }
        
        # form the reverse half & get midpoints
        con.summ$midpt = (as.numeric(con.summ$plus) + as.numeric(con.summ$minus)) / 2
        tmp = con.summ
        tmp$plus = con.summ$minus
        tmp$minus = con.summ$plus
        con.summ = rbind(con.summ, tmp)
        
        # find ranges to ensure we get tick marks:
        exmaj = c(0, .pvmaj.brk)
        pvtmp = c(plim, con.summ$p.value)
        pvtmp = pvtmp[!is.na(pvtmp)]
        tick.min = max(exmaj[exmaj <= min(pvtmp)])
        tick.max = min(exmaj[exmaj >= max(pvtmp)])
        
        # args for geom_segment
        sarg = c(list(mapping = quote(ggplot2::aes(xend = .data$p.value, yend = .data$midpt)),
                    data = NULL, stat = "identity", position = "identity"), 
                    aes$segment)
        grobj = ggplot2::ggplot(data = con.summ, 
                                ggplot2::aes(x = .data$p.value, y = .data$plus,
                                              color = .data$minus, group = .data$minus)) +
            do.call(ggplot2::geom_point, aes$point) +
            do.call(ggplot2::geom_segment, sarg) +
            ggplot2::geom_point(ggplot2::aes(x = tick.min, y = 1), alpha = 0) +
            ggplot2::geom_point(ggplot2::aes(x = tick.max, y = 1), alpha = 0)
        if (!is.null(by)) {
            cols = setdiff(by, rows)
            if (length(cols) > 0)
                ncols = length(unique(do.call(paste, unname(con.summ[cols]))))
            else {
                ncols = 1
                cols = "."
            }
            grobj = grobj + ggplot2::facet_grid(
                as.formula(paste( paste(rows, collapse = "+"), "~", 
                                  paste(cols, collapse = "+"))), 
                labeller = "label_both")
        }
        else
            ncols = 1
        if (values) {
            emm.summ$minus = emm.summ$pri.fac # for consistency in grouping/labeling
            dig = .opt.dig(emm.summ[, estName])
            emm.summ$fmtval = format(emm.summ[[estName]], digits = dig)
            tminp = .pval.tran(min(c(tick.min, con.summ$p.value), na.rm=TRUE))
            pos = .pval.inv(tminp - .025)
            lpad = .012 * (add.space + max(nchar(emm.summ$fmtval))) * ncols # how much space needed for labels rel to (0,1)
            lpad = lpad * (1.1 - tminp) # scale closer to actual width of scales
            lpos = .pval.inv(tminp - lpad)   # pvalue at left end of label
            
            larg = c(list(mapping = quote(ggplot2::aes(x = pos, y = .data$minus,
                                            label = .data$fmtval, hjust = "right")),
                        data = emm.summ, stat = "identity", position = "identity"),  
                        aes$label)
            grobj = grobj + 
                do.call(ggplot2::geom_label, larg) +
                ggplot2::geom_point(ggplot2::aes(x = lpos, y = 1), alpha = 0) # invisible point to stake out space
        }
        else
            lpad = 0
        
        .requireNS("scales", 
                   "pwpp requires the 'scales' package be installed.", call. = FALSE)
        .pvtrans = scales::trans_new("Scaled P value", 
                                     transform = function(x) .pval.tran(x), 
                                     inverse = function(p) .pval.inv(p),
                                     format = function(x) format(x, drop0trailing = TRUE, scientific = FALSE),
                                     domain = c(0,1) )
        grobj = grobj + ggplot2::scale_x_continuous(trans = .pvtrans, 
                                                    breaks = .pvmaj.brk, minor_breaks = .pvmin.brk) +
            #### I plotted an extra point instead of expanding scale
            #### expand = ggplot2::expand_scale(add = c(.025 + lpad, .025))) +
            ggplot2::guides(color = "none")
        
        grobj + ggplot2::labs(x = xlab, y = ylab, caption = xsub) + theme_emm()
}
    
# capitalize
.cap = function(s)
    paste0(toupper(substring(s, 1, 1)), substring(s, 2))

          
### Scale-transformation code: We stretch out small P values without stretching 
### extremely small ones too much -- via a combination of log and normal cdf functions

.tran.ctr = -2.5  # params of normal cdf transf of log(.value)
.tran.sd = 3
.tran.div = pnorm(0, .tran.ctr, .tran.sd)

.pvmaj.brk = c(.001, .01, .05, .1, .2, .5, 1)
#.pvmin.brk = c(.0001, .0005, seq(.001, .009, by = .001), seq(.02 ,.09, by = .01), .2, .3, .4, .6, .7, .8, .9)
.pvmin.brk = c(.0005, .001, .005, seq(.01 ,.1, by = .01), .2, .3, .4, .5, 1)

# transforms x in (0, 1] to t in (0,1], while any x's < 0 or > 1 are preserved as is
# This allows me to position marginal labels etc. where I want
.pval.tran = function(x) {
    xx = sapply(x, function(.) min(max(., .00005), 1))
    rtn = pnorm(log(xx), .tran.ctr, .tran.sd) / .tran.div
    spec = which((x < .00005) | (x > 1))
    rtn[spec] = x[spec]
    rtn
}

.pval.inv = function(p) {
    pp = sapply(p, function(.) min(max(., .00005), .99995))
    rtn = exp(qnorm(.tran.div * pp, .tran.ctr, .tran.sd))
    spec = which((p < .00005) | (p > .99995))
    rtn[spec] = p[spec]
    rtn
}

# For scale_x_continuous(): (moved to body of fcn so I can check w/ requyireNamespace)
# .pvtrans = scales::trans_new("Scaled P value", 
#                              transform = function(x) .pval.tran(x), 
#                              inverse = function(p) .pval.inv(p),
#                              format = function(x) format(x, drop0trailing = TRUE, scientific = FALSE),
#                              domain = c(0,1) )

### quick & dirty algorithm to Stretch out P values so more distinguishable on transformed scale
gran = function(x, min_incr = .01) {
    savex = x
    x = x[!is.na(x)]
    if ((length(x) <= 3) || (diff(range(x)) == 0))
        return(savex)
    
    kink = function(xx) { # linear spline basis; call with knot subtracted
        xx[xx < 0] = 0
        xx
    }
    
    ### x[x < .00009] = .00009   # forces granulation of extremely small P
    # spread-out the p values less than .0004
    rnk = rank(x)
    sm = which(x < .0004)
    if(length(sm) > 0)
        x[sm] = .0004 - .0003 * rnk[sm] / length(sm)
    
    ord = order(x)
    tx = log(x[ord])
    df = diff(tx)
    incr = sapply(df, function(.) max(., min_incr))
    ttx = cumsum(c(tx[1], incr))
    if (length(x) > 5)
        tx = predict(lm(tx ~ ttx + kink(ttx + 1.5) + kink(ttx + 3) + kink(ttx + 4.5)))
    else
        tx = ttx
    tx[tx > 0] = 0
    x[ord] = exp(tx)
    savex[!is.na(savex)] = x
    savex
}



#' Pairwise P-value matrix (plus other statistics)
#'
#' This function presents results from \code{emmeans} and pairwise comparisons
#' thereof in a compact way. It displays a matrix (or matrices) of estimates,
#' pairwise differences, and P values. The user may opt to exclude any of these
#' via arguments \code{means}, \code{diffs}, and \code{pvals}, respectively.
#' To control the direction of the pairwise differences, use \code{reverse};
#' and to control what appears in the upper and lower triangle(s), use \code{flip}.
#' Optional arguments are passed to \code{contrast.emmGrid} and/or 
#' \code{summary.emmGrid}, making it possible to control what estimates 
#' and tests are displayed.
#'
#' @param emm An \code{emmGrid} object
#' @param by Character vector of variable(s) in the grid to condition on. These
#'   will create different matrices, one for each level or level-combination.
#'   If missing, \code{by} is set to \code{emm@misc$by.vars}.
#'   Grid factors not in \code{by} are the \emph{primary} factors:
#'   whose levels or level combinations are compared pairwise.
#' @param reverse Logical value passed to \code{\link{pairs.emmGrid}}.
#'   Thus, \code{FALSE} specifies \code{"pairwise"} comparisons 
#'   (earlier vs. later), and \code{TRUE} specifies \code{"revpairwise"}
#'   comparisons (later vs. earlier).
#' @param pvals Logical value. If \code{TRUE}, the pairwise differences 
#'   of the EMMs are included in each matrix according to \code{flip}.
#' @param means Logical value. If \code{TRUE}, the estimated marginal means
#'   (EMMs) from \code{emm} are included in the matrix diagonal(s).
#' @param diffs Logical value. If \code{TRUE}, the pairwise differences 
#'   of the EMMs are included in each matrix according to \code{flip}.
#' @param flip Logical value that determines where P values and differences 
#'   are placed. \code{FALSE} places the P values in the upper triangle
#'   and differences in the lower, and \code{TRUE} does just the opposite.
#' @param digits Integer. Number of digits to display. If missing,
#'   an optimal number of digits is determined.
#' @param ... Additional arguments passed to \code{\link{contrast.emmGrid}} and 
#'   \code{\link{summary.emmGrid}}. You should \emph{not} include \code{method}
#'   here, because pairwise comparisons are always used. 
#'
#' @return A matrix or `list` of matrices, one for each `by` level.
#' 
#' @seealso A graphical display of essentially the same results is available
#'   from \code{\link{pwpp}}
#' @export
#' 
#' @note If \code{emm} is the result of a Bayesian analysis, \code{pwpm} is
#'   based on a frequentist analysis
#'
#' @examples
#' warp.lm <- lm(breaks ~ wool * tension, data = warpbreaks)
#' warp.emm <- emmeans(warp.lm, ~ tension | wool)
#' 
#' pwpm(warp.emm)
#' 
#' # use dot options to specify noninferiority tests
#' pwpm(warp.emm, by = NULL, side = ">", delta = 5, adjust = "none")
pwpm = function(emm, by, reverse = FALSE,
                pvals = TRUE, means = TRUE, diffs = TRUE, 
                flip = FALSE, digits, ...) {
    emm = .chk.list(emm, ...)
    if(missing(by)) 
        by = emm@misc$by.vars
    
    emm = update(emm, by = by)
    pri = paste(emm@misc$pri.vars, collapse = ":")
    mns = confint(emm, frequentist = TRUE, ...)
    mns$lbls = do.call(paste, c(unname(mns[attr(mns, "pri.vars")]), sep = get_emm_option("sep")))
    estName = attr(mns, "estName")
    prs = test(pairs(emm, reverse = reverse, frequentist = TRUE, ...), ...)
    diffName = attr(prs, "estName")
    null.hyp = "0"
    if (!reverse) 
        trifcn = lower.tri
    else {
        flip = !flip
        trifcn = upper.tri
    }
    
    if (!is.null(prs$null)) {
        null.hyp = as.character(signif(unique(prs$null), digits = 5))
        if (length(null.hyp) > 1)
            null.hyp = "(various values)"
    }
    mby = .find.by.rows(mns, by)
    pby = .find.by.rows(prs, by)

    
    if(opt.dig <- missing(digits)) {
        tmp = mns[[estName]] + mns[["SE"]] * cbind(rep(-2, nrow(mns)), 0, 2)
        digits = max(apply(tmp, 1, .opt.dig))
        opt.dig = TRUE
    }

    result = lapply(seq_along(mby), function(i) {
        if(opt.dig) {
            pv = prs$p.value[pby[[i]]]
            fpv = sprintf("%6.4f", pv) 
            fpv[pv < 0.0001] = "<.0001"
        }
        else
            fpv = format(prs$p.value[pby[[i]]], digits = digits)
        fmn = format(mns[mby[[i]], estName], digits = digits)
        fdiff = format(prs[pby[[i]], diffName], digits = digits) 
        
        lbls = mns$lbls[mby[[i]]]
        n = length(lbls)
        mat = matrix("", nrow = n, ncol = n, dimnames = list(lbls, lbls))
        if(pvals) {
            mat[trifcn(mat)] = fpv
            mat = t(mat)
        }
        if (diffs)
            mat[trifcn(mat)] = fdiff
        if (means)
            diag(mat) = fmn
        else { # trim off empty row and col
            idx = seq_len(n - 1)
            if (pvals && !diffs)
                mat = mat[idx, 1 + idx]
            if (!pvals && diffs)
                mat = mat[1 + idx, idx]
        }
        if (flip) t(mat)
        else mat
    })
    if (reverse) 
        flip = !flip
    names(result) = paste(paste(by, collapse = ", "), "=", names(mby))
    if (length(result) == 1)
        result = result[[1]]
    class(result) = c("pwpm", "list")
    attr(result, "parms") = c(pvals = pvals, diffs = diffs, means = means, pri = pri,
            estName = estName, diffName = diffName, reverse = reverse, flip = flip,
            type = attr(mns, "type"), adjust = attr(prs, "adjust"),
            side = attr(prs, "side"), delta = attr(prs, "delta"),
            null = null.hyp)
    result
}

#' @export
print.pwpm = function(x, ...) {
    parms = attr(x, "parms")
    attr(x, "class") = attr(x, "parms") = NULL
    if ((islist <- !is.matrix(x)))
        entries = seq_along(x)
    else {
        entries = 1
        m = x 
    }

    for (i in entries) {
        if (islist) {
            cat(paste0("\n", names(x)[i], "\n"))
            m = x[[i]]
        }
        if (parms["means"])
            diag(m) = paste0("[", diag(m), "]")
        print(m, quote = FALSE, right = TRUE, na.print = "nonEst")
    }
    
    # print a parm and its name if present unless it's in excl
    # optional subst is NAMED vector where each possibilitty MUST be present
    catparm = function(f, excl = "0", delim = "  ", quote = TRUE, subst) {
        if(!is.na(pf <- parms[f]) && !(pf %in% excl)) {
            if (!missing(subst)) pf = subst[pf]
            if (quote) pf = dQuote(pf)
            cat(paste0(delim, f, " = ", pf))
        }
    }
    cat(paste0("\nRow and column labels: ", parms["pri"], "\n"))
    if (parms["pvals"]) {
        cat(paste0(ifelse(parms["flip"], "Lower", "Upper"), " triangle: P values "))
        catparm("null", quote = FALSE)
        catparm("side", subst = c("-1" = "<", "1" = ">"))
        catparm("delta", quote = FALSE)
        catparm("adjust", "none")
        cat("\n")
    }
    if (parms["means"]) {
        cat(paste0("Diagonal: [Estimates] (", parms["estName"], ") "))
        catparm("type", "link")
        cat("\n")
    }
    if (parms["diffs"]) {
        cat(paste0(ifelse(parms["flip"], "Upper", "Lower"), " triangle: Comparisons (",
                   parms["diffName"], ")   "))
        if (parms["reverse"]) cat("later vs. earlier\n")
        else cat("earlier vs. later\n")
    }
    invisible(x)
}
    

context("Nested structures")

set.seed(412.1948)
foo = data.frame(
    nest = factor(c(rep("A",8),rep("B",12),rep("C",4))),
    bird = factor(rep(1:6, each=4)),
    m = rep(c(2,3,0,-1,-2,5), each=4),
    b = rep(c(0,-1,1,2,2,3), each=4),
    x = 3 + 5*runif(24),
    e = 0.3*rnorm(24)
)
foo = transform(foo, resp = m*x + b + e)
foo1.lm = lm(resp ~ nest + bird*x, data = foo)
rg1 = ref_grid(foo1.lm)
foo2.lm = lm(resp ~ (nest + bird)*x, data = foo)
rg2 = ref_grid(foo2.lm)

test_that("nested EMM works", {
    emm1 = emmeans(rg1, "bird")
    expect_equal(nrow(summary(emm1)), 6)
    expect_equal(names(emm1@grid[1:2]), c("bird", "nest"))
    p1bn = predict(emmeans(emm1, "nest"))
    p1n = predict(emmeans(rg1, "nest"))
    p2n = predict(emmeans(rg2, "nest"))
    expect_equal(p1bn, p1n, tol = 1e-6)
    expect_equal(p1n, p2n, tol = 1e-6)
})

test_that("nested trends works", {
    emtb = emtrends(foo1.lm, "bird", "x")
    emtn = emtrends(foo1.lm, "nest", "x")
    emtbn = emmeans(emtb, "nest")
    expect_equal(nrow(summary(emtb)), 6)
    expect_equal(nrow(summary(emtn)), 3)
    expect_equal(predict(emtn), predict(emtbn), tol = 1e-6)
})

context("Reference grids")

pigs.lm = lm(log(conc) ~ source + factor(percent), data = pigs)
rg = ref_grid(pigs.lm)
rg1 = ref_grid(pigs.lm, at = list(source = "soy", percent = 12))

pigs.lm2 = update(pigs.lm, conc ~ source + percent, data = pigs)
rg2 = ref_grid(pigs.lm2)
rg2a = ref_grid(pigs.lm2, at = list(source = c("fish", "soy"), percent = 10))
rg2c = ref_grid(pigs.lm2, cov.reduce = FALSE)
rg2m = ref_grid(pigs.lm2, cov.reduce = min)

pigs.lm3 = update(pigs.lm2, . ~ source + source:factor(percent))
pigs = transform(pigs, sp = interaction(source, percent))
pigs.lm4 = update(pigs.lm2, . ~ source + sp)

test_that("Reference grid is constructed correctly", {
    expect_equal(nrow(rg@grid), 12)
    expect_equal(nrow(rg1@grid), 1)
    expect_equal(nrow(rg2@grid), 3)
    expect_equal(nrow(rg2a@grid), 2)
    expect_equal(nrow(rg2c@grid), 12)
    expect_equal(nrow(rg1@grid), 1)
    expect_equal(length(rg@levels), 2)
    expect_equal(rg2@levels$percent, mean(pigs$percent))
    expect_equal(rg2m@levels$percent, min(pigs$percent))
})

test_that("Reference grid extras are detected", {
    expect_equal(rg@misc$tran, "log")
    expect_true(is.null(rg2@misc$tran))
    expect_true(is.null(rg2@model.info$nesting))
    expect_is(ref_grid(pigs.lm3)@model.info$nesting, "list") # see note above
    expect_is(ref_grid(pigs.lm4)@model.info$nesting, "list") # see note above
})

colnames(ToothGrowth) <- c('len', 'choice of supplement', 'dose')
model <- stats::aov(`len` ~ `choice of supplement`, ToothGrowth)

test_that("Reference grid handles variables with spaces", {
    expect_output(str(ref_grid(model, ~`choice of supplement`)), "choice of supplement")
})

# models outside of data.frames
x = 1:10
y = rnorm(10)
mod1 = with(pigs, lm(log(conc) ~ source + factor(percent)))
test_that("ref_grid works with no data or subset", {
    expect_silent(ref_grid(lm(y ~ x)))
    expect_silent(ref_grid(mod1))
})

# Multivariate models
MOats.lm <- lm (yield ~ Block + Variety, data = MOats)
MOats.rg <- ref_grid (MOats.lm, mult.levs = list(
    trt = LETTERS[1:2], dose = as.character(1:2))
)
test_that("We can construct multivariate reference grid", {
    expect_equal(nrow(MOats.rg@grid), 72)
    expect_equal(length(MOats.rg@levels), 4)
})
MOats.nrg <- ref_grid (MOats.lm, mult.levs = list(nitro = c(0,.2,.4,.6)),
                       at = list(nitro = c(0.0001, .39998, .5)))
test_that("Fuzzy matching of numerical mult.levels works", {
    expect_equal(length(MOats.nrg@levels$nitro), 2)
    expect_equal(MOats.nrg@levels$nitro, c(0, .4), 0.001)
})

### Nuisance factors
MOats.rgn = ref_grid(MOats.lm, nuisance = "Block")
MOats.emm = emmeans(MOats.lm, ~ Variety * rep.meas)
test_that("We get same predictions with and without nuisance specs", {
    expect_equal(predict(MOats.rgn), predict(MOats.emm), 0.001)
})

# Missing and NA levels
miss.df = data.frame(x = factor(c("a", "a", "b", NA), levels = c("a", "b", "c", NA)), y = 1:4)
miss.lm = lm(y ~ x, data = miss.df)
miss.rg1 = ref_grid(miss.lm)
miss.rg2 = ref_grid(miss.lm, data = miss.df)
# Now try allowing NA levels
miss.dfa = transform(miss.df, x = factor(x, exclude = NULL))
miss.lma = lm(y ~ x, data = miss.dfa)
miss.rg1a = ref_grid(miss.lma)
miss.rg2a = ref_grid(miss.lma, data = miss.dfa)
test_that("Reference grid handles missing values", {
    expect_equal(length(miss.rg1@levels$x), 2)
    expect_equal(length(miss.rg2@levels$x), 2)
    expect_equal(length(miss.rg1a@levels$x), 3)
    expect_equal(length(miss.rg2a@levels$x), 3)
    expect_equal(inherits(with_emm_options(allow.na.levs = FALSE, ref_grid(miss.lma)), 
                 "try-error"), TRUE)
})

##############################################################################
#    Copyright (c) 2012-2019 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

### Support for objects involving several models (e.g., model averaging, multiple imputation)

### MuMIn::averaging
### This is just bare-bones. Need to provide data,
### and if applicable, df.
### Optional argument tran sets the tran property

# $data is NOT a standard member, but if it's there, we'll use it
# Otherwise, we need to provide data or its name in the call
#' @exportS3Method recover_data averaging
recover_data.averaging = function(object, data, formula = object$formula, ...) {
    ml = attr(object, "modelList")
    if (is.null(ml))
        return(paste0("emmeans support for 'averaging' models requires a 'modelList' attribute.\n",
                      "Re-fit the model from a model list or with fit = TRUE"))
    if (is.null(formula)) {
        lhs = as.formula(paste(formula(ml[[1]])[[2]], "~."))
        rhs = sapply(ml, function(m) {f = formula(m); f[[length(f)]]})
        formula = update(as.formula(paste("~", paste(rhs, collapse = "+"))), lhs)
    }
    if (is.null(data))
        data = ml[[1]]$call$data
    ### temporary patch -- $formula often excludes the intercept
    formula = update(formula, .~.+1) ####[-2]
    trms = attr(model.frame(formula, data = data), "terms")
    fcall = call("model.avg", formula = formula, data = data)
    recover_data(fcall, trms, na.action = NULL, ...)
}

# optional argument 'subset' is named character "prefix:pfx" or "wrap:wrap"
# or NAMED vector of indexes
#' @exportS3Method emm_basis averaging
emm_basis.averaging = function(object, trms, xlev, grid, subset, ...) {
    bhat = coef(object, full = TRUE)
    bnms = names(bhat)
    V = .my.vcov(object, function(., ...) vcov(., full = TRUE), ...)

    # support functions for 'what' ...
    prefixed_by = function(pfx) {  # picks elements prefixed by pfx
        w = which(startsWith(bnms, pfx))
        names(w) = gsub(pfx, "", bnms[w])
        w
    }
    wrapped_by = function(wrp) {  # picks elements wrapped by wrp(name)
        pat = paste0(wrp, "\\(")
        w = grep(pat, bnms)
        tmp = sub(pat, "", bnms[w])
        names(w) = substr(tmp, 1, nchar(tmp) - 1)
        ### names(w) = sub("([a-z]+\\()([0-9:_() A-Za-z]+)(\\))", "\\2", bnms[w], perl = TRUE)
        w
    }
    if(!missing(subset)) {
        if (is.character(subset)) {  # wrap:stg or prefix:stg
            tmp = strsplit(subset, ":")[[1]]
            if(tmp[[1]] == "wrap")
                subset = wrapped_by(tmp[2])
            else # anything else is interpreted as prefix
                subset = prefixed_by(rev(tmp)[1])
            if(length(subset) == 0)
                stop("subset specification eliminates all coefficients!")
        }
        bhat = bhat[subset]
        bnms = names(subset)
        bnms[bnms == "(Int)"] = "(Intercept)"  # covers a peculiarity in glmmTMB
        V = V[subset, subset]
    }
    
    # we're gonna assemble all the main-effects model-matrix components
    # and build the model matrix to match the names of the coefs
    fcts = rownames(attr(trms, "factors"))
    mmlist = lapply(fcts, \(f) {
        frm = reformulate(c(-1, f))
        suppressWarnings(model.matrix(frm, data = grid, xlev = xlev))
    })
    ME = cbind("(Intercept)" = 1, do.call(cbind, mmlist))
    cols = strsplit(bnms, ":")  # the cols we need
    ext = setdiff(intersect(unlist(cols), names(grid)), colnames(ME)) # extra expressions we need to compute
    if(length(ext) > 0) {
        xcols = sapply(ext, \(expr) eval(parse(text = expr), envir = grid))
        ME = cbind(ME, xcols)
    }
    # now get the factors for each coef
    X = sapply(cols, \(nms)
        apply(ME[, nms, drop = FALSE], 1, prod))
    colnames(X) = bnms
    # Now, sometimes a coefficient is duplicated, so we need to zero those out
    srt = sapply(cols, \(nms) paste(sort(nms), collapse = ":"))
    tmp = table(srt)
    for (nm in names(tmp[tmp > 1])) {  # work on the dupes
        zap = which(srt == nm)[-1]   # indices of all but the first
        bhat[zap] = 0
    }
    
    
    nbasis = estimability::all.estble
    ml = attr(object, "modelList")
    ml1 = ml[[1]]
    misc = list()
    fam <- try(family(ml1), silent = TRUE)
    if (!inherits(fam, "try-error"))
        misc = .std.link.labels(fam, misc)
    if (!is.null(df.residual(ml1))) {
        dffun = function(k, dfargs) dfargs$df
        dfargs = list(df = min(sapply(ml, df.residual)))
    }
    else {
        dffun = function(k, dfargs) Inf
        dfargs = list()
    }
    list(X = X, bhat = bhat, nbasis = nbasis, V = V, 
         dffun = dffun, dfargs = dfargs, misc = misc)
}




### minc::mira support -----------------------------------------
# Here we rely on the methods already in place for elements of $analyses

#' @exportS3Method recover_data mira
recover_data.mira = function(object, data = NULL, ...) {
    rdlist = lapply(object$analyses, recover_data, data = data, ...)
    rd = rdlist[[1]]
    # we'll average the numeric columns...
    numcols = which(sapply(rd, is.numeric))
    for (j in numcols)
        rd[, j] = apply(sapply(rdlist, function(.) .[, j]), 1, mean)
    rd
}

#' @exportS3Method emm_basis mira         
emm_basis.mira = function(object, trms, xlev, grid, ...) {
    # In case our method did a "pass it on" with the data, we need to add that attribute
    data = list(...)$misc$data
    if(!is.null(data))
        object$analyses = lapply(object$analyses, function(a) {attr(a, "data") = data; a})
    bas = emm_basis(object$analyses[[1]], trms, xlev, grid, ...)
    k = length(object$analyses)
    # we just average the V and bhat elements...
    V = 1/k * bas$V
    allb = cbind(bas$bhat, matrix(0, nrow = length(bas$bhat), ncol = k - 1))
    for (i in 1 + seq_len(k - 1)) {
        basi = emm_basis(object$analyses[[i]], trms, xlev, grid, ...)
        V = V + 1/k * basi$V
        allb[, i] = basi$bhat
    }
    bas$bhat = apply(allb, 1, mean)  # averaged coef
    notna = which(!is.na(bas$bhat))
    bas$dfargs$m = k
    bas$dfargs$df1 = bas$dffun  # this is the dffun for the 1st analysis
    bas$dfargs$B = cov(t(allb[notna, , drop = FALSE]))
    bas$V = bas$dfargs$T = V + (k + 1)/k * bas$dfargs$B   # pooled via Rubin's rules
    bas$dffun = function(a, dfargs) { # pretty much copied from mice:::barnard.rubin
        dfcom = dfargs$df1(a, dfargs)
        with(dfargs, { 
             b = sum(a * (B %*% a))
             t = sum(a * (T %*% a))
             lambda = (1 + 1/m) * b / t
             dfold = (m - 1)/lambda^2
             dfobs = (dfcom + 1)/(dfcom + 3) * dfcom * (1 - lambda)
             ifelse(is.infinite(dfcom), dfold, 
                    dfold * dfobs/(dfold + dfobs))
        }) }
    bas
}

##############################################################################
#    Copyright (c) 2012-2017 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

# Support for 'betareg' class

# mode is same as 'type' in predict.betareg, PLUS 
# mode = "phi.link" refers to link function before back-transforming to "precision"

#' @exportS3Method recover_data betareg
recover_data.betareg = function(object, mode = c("response", "link", "precision", "phi.link", "variance", "quantile"), ...) {
    fcall = object$call
    mode = match.arg(mode)
    if (mode  %in% c("response", "link"))
        mode = "mean"
    if (mode == "phi.link") 
        mode = "precision"
    if(mode %in% c("mean", "precision"))
        trms = delete.response(terms(object, model = mode))
    else
        trms = delete.response(object$terms$full)
    # Make sure there's an offset function available
    env = new.env(parent = attr(trms, ".Environment"))
    env$offset = function(x) x
    attr(trms, ".Environment") = env
    recover_data(fcall, trms, object$na.action, ...)
}

# PRELIMINARY...
# Currently works correctly only for "resp", "link", "precision", "phi" modes
#' @exportS3Method emm_basis betareg
emm_basis.betareg = function(object, trms, xlev, grid, 
        mode = c("response", "link", "precision", "phi.link", "variance", "quantile"), 
        quantile = .5, ...) {
    mode = match.arg(mode)
#     if (mode %in% c("variance", "quantile"))
#         stop(paste0('"', mode, '" mode is not yet supported.'))
    
    # figure out which parameters we need
    model = if (mode %in% c("response", "link")) "mean"
        else if (mode %in% c("precision", "phi.link")) "precision"
        else "full"
    V = .pscl.vcov(object, model = model) # borrowed from pscl methods
    bhat = coef(object, model = model)
    
    nbasis = estimability::all.estble
    dffun = function(k, dfargs) Inf
    dfargs = list()
    
    
    if (mode %in% c("response", "link", "precision", "phi.link")) {
        m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
        X = model.matrix(trms, m, contrasts.arg = object$contrasts[[model]])
        misc = list(tran = object$link[[model]]$name)
        if (mode %in% c("response", "precision")) {
            misc$postGridHook = ".betareg.pg"
        }
    }
    else { ### (mode %in% c("variance", "quantile"))
        m.trms = delete.response(terms(object, "mean"))
        m.m = model.frame(m.trms, grid, na.action = na.pass, xlev = xlev)
        X = model.matrix(m.trms, m.m, contrasts.arg = object$contrasts$mean)
        m.idx = seq_len(ncol(X))
        m.lp = as.numeric(X %*% bhat[m.idx] + .get.offset(m.trms, grid))
        mu = object$link$mean$linkinv(m.lp)
            
        p.trms = delete.response(terms(object, "precision"))
        p.m = model.frame(m.trms, grid, na.action = na.pass, xlev = xlev)
        Z = model.matrix(p.trms, p.m, contrasts.arg = object$contrasts$precision)
        p.lp = as.numeric(Z %*% bhat[-m.idx] + .get.offset(p.trms, grid))
        phi = object$link$precision$linkinv(p.lp)
        
        if (mode == "variance") {
            bhat = mu * (1 - mu) / (1 + phi)
            dbhat.dm = (1 - 2 * mu) / (1 + phi)
            dbhat.dp = -bhat / (1 + phi)
            # delta = cbind(diag(dbhat.dm) %*% X, diag(dbhat.dp) %*% Z)
            delta = cbind(sweep(X, 1, dbhat.dm, "*"), sweep(Z, 1, dbhat.dp, "*"))
            V = delta %*% tcrossprod(V, delta)
            misc = list()
        }
        else {  ### (mode = "quantile")
            bhat = as.numeric(sapply(quantile, function(q)
                stats::qbeta(q, phi * mu, phi * (1 - mu))))
            V = matrix(NA, nrow = length(bhat), ncol = length(bhat))
            misc = list(ylevs = list(quantile = quantile))
        }
        X = diag(1, length(bhat))
    }
    list(X=X, bhat=bhat, nbasis=nbasis, V=V, dffun=dffun, dfargs=dfargs, misc=misc)
}

# Post-grid hook for simple back-transforming
.betareg.pg = function(object, ...) {
    object@misc$postGridHook = NULL
    regrid(object, transform = TRUE)
}


### predict methods
# link: X%*%beta + off_m
# response: mu = h_m(link)
# 
# phi.link: Z%*%gamma + off_p
# precision: phi = h_p(phi.link)
# 
# variance: mu*(1 - mu) / (1 + phi)
# quantile: qbeta(p, mu*phi, (1 - mu)*phi)
#
# Defns:
#   phi = a + b
#   mu = a / (a + b)
# so that phi*mu = a and phi*(1 - mu) = b,
#   Variance = ab / [(a + b)^2 * (a + b + 1)]

##############################################################################
#    Copyright (c) 2012-2024 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

### Encouragement to wean ourselvs from the titdyverse
#' Dare to be un-"tidy"!
#' 
#' Users who use \pkg{emmeans} functions as part of a pipeline -- or post-process 
#' those results in some other way -- are likely missing some important information. 
#' 
#' Your best bet is to display the actual results without any post-processing.
#' That's because \code{emmeans} and its relatives have their own \code{summary} 
#' and \code{print} methods that display annotations that may be helpful in
#' explaining what you have. If you just pipe the results into the next step,
#' those annotations are stripped away and you never see them. Statistical
#' analysis is not just a workflow; it is a discipline that involves care in
#' interpreting intermediate results, and thinking before moving on. 
#' 
#' @examples
#' neur.glm <- glm(Pain ~ Treatment + Sex + Age, family = binomial(),
#'             data = neuralgia)
#'             
#' ### The actual results with annotations (e.g. ests are on logit scale):
#' emmeans(neur.glm, "Treatment")
#' 
#' ### Post-processed results lose the annotations
#' if(requireNamespace("tibble")) {
#'     emmeans(neur.glm, "Treatment") |> tibble::as_tibble()
#' }
#' 
#' @name untidy
NULL



# NOTE: Excluded from documentation
# Custom Vignette format
# 
# This is used to format HTML vignettes the way its developer wants them.
#
# @param ... Arguments passed to \code{rmarkdown::html_document}
#
# @return R Markdown format used by \code{rmarkdown::render}
#' @rdname extending-emmeans
#' @order 51
#' @param css,package,highlight Arguments for \code{.emm_vignette}, which is
#' a clean and simple alternative to such as \code{html_document} for use
#' as the output style of a Markdown file. All the vignettes in the
#' \pkg{emmeans} package use this output style.
#' @export
.emm_vignette = function(css = system.file("css", "clean-simple.css", package = "emmeans"), 
                         highlight = NULL,  ...) {
    rmarkdown::html_document(theme = NULL, highlight = highlight,
                             fig_width = 3, fig_height = 3, 
                             css = css, ###pandoc_args = "", 
                             ...)
###    css = css, pandoc_args = "--strip-comments", ...)
}


### Dynamic registration of S3 methods
# Code borrowed from hms pkg. I omitted some type checks etc. because
# this is only for internal use and I solemnly promise to behave myself.
register_s3_method = function(pkg, generic, class, envir = parent.frame()) {
    fun = get(paste0(generic, ".", class), envir = envir)
    if (isNamespaceLoaded(pkg)) {
        registerS3method(generic, class, fun, envir = asNamespace(pkg))
    }
    # Register hook in case package is later unloaded & reloaded
    setHook(
        packageEvent(pkg, "onLoad"),
        function(...) {
            registerS3method(generic, class, fun, envir = asNamespace(pkg))
        }
    )
}

.onLoad = function(libname, pkgname) {
    if (.requireNS("coda", fail = .nothing)) {
        register_s3_method("coda", "as.mcmc", "emmGrid")
        register_s3_method("coda", "as.mcmc.list", "emmGrid")
        register_s3_method("coda", "as.mcmc", "emm_list")
        register_s3_method("coda", "as.mcmc.list", "emm_list")
    }
    if (.requireNS("multcomp", fail = .nothing)) {
        register_s3_method("multcomp", "glht", "emmlf")
        register_s3_method("multcomp", "glht", "emmGrid")
        register_s3_method("multcomp", "cld", "emmGrid")
        register_s3_method("multcomp", "cld", "emm_list")
        register_s3_method("multcomp", "modelparm", "emmwrap")
    }
    if(.requireNS("xtable", fail = .nothing)) {
        register_s3_method("xtable", "xtable", "emmGrid")
        register_s3_method("xtable", "xtable", "summary_emm")
        register_s3_method("xtable", "print", "xtable_emm")
    }
}

.onAttach <- function(libname, pkgname) {
    packageStartupMessage(
        "Welcome to emmeans.\n",
        "Caution: You lose important information if you filter this package's results.\n",
        "See '? untidy'")
}



#' @rdname extending-emmeans
#' @order 29
#' @section Registering S3 methods for a model class:
#' The \code{.emm_register} function is provided as a convenience to conditionally 
#' register your
#' S3 methods for a model class, \code{recover_data.foo} and \code{emm_basis.foo},
#' where \code{foo} is the class name. Your package should implement an
#' \code{.onLoad} function and call \code{.emm_register} if \pkg{emmeans} is
#' installed. See the example.
#'
#' @param classes Character names of one or more classes to be registered.
#'   The package must contain the functions \code{recover_data.foo} and
#'   \code{emm_basis.foo} for each class \code{foo} listed in \code{classes}.
#' @param pkgname Character name of package providing the methods (usually
#'    should be the second argument of \code{.onLoad})
#' @param qdrg Logical value. If \code{FALSE}, the \code{recover_data} and
#'     \code{emm_basis} methods are registered. If \code{TRUE}, the \code{qdrg}
#'     method for each class is registered instead.
#'
#' @export
#'
#' @examples
#' \dontrun{
#' #--- If your package provides recover_data and emm_grid methods for class 'mymod',
#' #--- put something like this in your package code -- say in zzz.R:
#'   .onLoad <- function(libname, pkgname) {
#'     if (requireNamespace("emmeans", quietly = TRUE))
#'       emmeans::.emm_register("mymod", pkgname)
#'   }
#' }
.emm_register = function(classes, pkgname, qdrg = FALSE) {
    envir = asNamespace(pkgname)
    for (class in classes) {
        if(qdrg) 
            register_s3_method("emmeans", "recover_data", class, envir)
        else {
            register_s3_method("emmeans", "recover_data", class, envir)
            register_s3_method("emmeans", "emm_basis", class, envir)
        }
    }
}

##############################################################################
#    Copyright (c) 2012-2019 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

# Functions used only internally and of fairly general use
# More of these with specific uses only within a certain context remain there.

## create an lm-compatible set of coefficients in singular cases
## requires both bhat and X to be named; if not, silently returns bhat
.impute.NAs = function(bhat, X) {
    if (is.null(bnm <- names(bhat)) || is.null(colnames(X)))
        return(bhat)
    nm = intersect(bnm, (xnm <- colnames(X)))
    if(length(nm) == length(xnm))
        return(bhat[nm])
    tmp = rep(NA, length(xnm))
    names(tmp) = xnm
    tmp[nm] = bhat[nm]
    tmp
}

## %in%-style operator with partial matching
## e.g.,  ("bonf" %.pin% p.adjust.methods)  is TRUE
"%.pin%" = function (x, table) pmatch(x, table, nomatch = 0L) > 0L

## Like is.numeric() but returns TRUE for character vectors like c("1", "3", "2")
## (but will return FALSE if any x is NA)
.is.num = function(x) {
    nx = suppressWarnings(as.numeric(as.character(x)))
    !any(is.na(nx))
}

# return TRUE if obj@misc$sigma is not NA
.chk.predict = function(obj) {
    sig = obj@misc$sigma
    ok = (is.null(sig) || (!is.null(sig) && !is.na(sig)))
    if(!ok)
        warning("Prediction intervals are not available for this object", call. = FALSE)
    ok
}

### Internal function to implement 'allow.na.levs' option
.chk.fac = function(x) {
    if(get_emm_option("allow.na.levs"))
        factor(x, exclude = NULL)
    else
        factor(x)
}



## Alternative to all.vars, but keeps vars like foo$x and foo[[1]] as-is
##   Passes ... to all.vars
#' @rdname extending-emmeans
#' @order 41
#' @param expr,retain Arguments for \code{.all.vars}, which is an alternative to \code{\link{all.vars}}
#'   that has special provisions for retaining the special characters in \code{retain},
#'   thus allowing model specifications like \code{y ~ data$trt * df[["dose"]]}
#' @export
.all.vars = function(expr, retain = c("\\$", "\\[\\[", "\\]\\]", "'", '"'), ...) {
    if (is.null(expr) || length(expr) == 0)
        return(character(0))
    if (!inherits(expr, "formula")) {
        expr = try(eval(expr), silent = TRUE)
        if(inherits(expr, "try-error")) {
            return(character(0))
        }
    }
    repl = paste(".Av", seq_along(retain), ".", sep = "")
    for (i in seq_along(retain))
        expr = gsub(retain[i], repl[i], expr)
    subs = switch(length(expr), 1, c(1,2), c(2,1,3))
    vars = all.vars(as.formula(paste(expr[subs], collapse = "")), ...)
    retain = gsub("\\\\", "", retain)
    for (i in seq_along(retain))
        vars = gsub(repl[i], retain[i], vars)
    if(length(vars) == 0) vars = "1"   # no vars ---> intercept
    vars
}

### returns TRUE iff there is one or more function call in a formula
.has.fcns = function(form) {
    fcns = setdiff(.all.vars(form, functions = TRUE), 
                   c("~", "+", "-", "*", "/", ":", "(", "|", .all.vars(form)))
    length(fcns) > 0
}

### parse a formula of the form lhs ~ rhs | by into a list
### of variable names in each part
### Returns character(0) for any missing pieces
.parse.by.formula = function(form) {
    allv = .all.vars(form)
    ridx = ifelse(length(form) == 2, 2, 3)
    allrhs = as.vector(form, "character")[ridx]
    allrhs = gsub("\\|", "+ .by. +", allrhs) # '|' --> '.by.'
    allrhs = .all.vars(stats::reformulate(allrhs))
    bidx = grep(".by.", allrhs, fixed = TRUE)
    if (length(bidx) == 0) { # no '|' in formula
        by = character(0)
        rhs = allrhs
    }
    else {
        rhs = allrhs[seq_len(bidx[1] - 1)]
        by = setdiff(allrhs, c(rhs, ".by."))
    }
    lhs = setdiff(allv, allrhs)
    list(lhs = lhs, rhs = rhs, by = by)
}

### Check specs for 'all' and if there, create a list of specs
# Return the specs as they were, or if 'all' is specified, return revised list of specs
# with 'form.rtn' attribute
.parse.specs.for.all = function(object, specs, by) {
    all.key = "."  # The key to use to request all sets of means
    if(is.list(specs))
        return(specs)
    rtn = NULL
    if (inherits(specs, "formula")) {
        rtn = .parse.by.formula(specs)
        if ((length(rtn$rhs) == 0) || (rtn$rhs[1] != all.key))
            return(specs)
        specs = rtn$rhs
        rtn$by = setdiff(rtn$by, specs)
        if(length(rtn$by) > 0)
            by = rtn$by
    }
    # now we have is.character(specs) ...
    if ((length(specs) != 1) || (specs[1] != all.key))
        return(specs)
    # hack the object to bypass estimability checking. 
    # Doesn't matter what stats are as we use only the labels
    if(any(is.na(object@bhat))) {
        k = ncol(object@linfct)
        object@bhat = seq_len(k)
        object@V = diag(k)
        object@nbasis = estimability::all.estble
    }
    stgs = unique(setdiff(joint_tests(object, by = by)[, "model term"], "(confounded)"))
    # I think we need to add nested factors that involve 'by'
    if ((length(by) > 0) && !is.null(nst <- object@model.info$nesting)) {
        for (fac in names(nst))
            if (all(by %in% nst[[fac]]))
                stgs = c(stgs, fac)
        stgs = unique(stgs)
    }
    if(length(stgs) == 0)
        stop("'", all.key, "' specification yielded no terms", call. = FALSE)
    
    result = strsplit(stgs, ":")
    if(!is.null(rtn))
        attr(result, "form.rtn") = rtn
    result
}


# Utility to pick out the args that can be passed to a function
.args.for.fcn = function(fcn, args) {
    oknames = names(as.list(args(fcn)))
    mat = pmatch(names(args), oknames)
    args = args[!is.na(mat)]
    mat = mat[!is.na(mat)]
    names(args) = oknames[mat]
    args
}

# Create a list and give it class class.name
.cls.list <- function(class.name, ...) {
    result <- list(...)
    class(result) <- c(class.name, "list")
    result
}

### Not-so-damn-smart replacement of diag() that will 
### not be so quick to assume I want an identity matrix
### returns matrix(x) when x is a scalar
#' @rdname extending-emmeans
#' @order 42
#' @param x,nrow,ncol Arguments for \code{.diag}, which is an alternative to 
#'   \code{\link{diag}} that lacks its idiosyncrasy of returning an
#'   identity matrix when \code{x} is of length 1.
#' @export
#' 
.diag = function(x, nrow, ncol) {
    if(is.matrix(x))
        diag(x)
    else if((length(x) == 1) && missing(nrow) && missing(ncol)) 
        matrix(x)
    else 
        diag(x, nrow, ncol)
}

# Utility that returns TRUE if getOption("emmeans")[[opt]] is TRUE
.emmGrid.is.true = function(opt) {
    x = get_emm_option(opt)
    if (is.logical(x))  x
    else FALSE
}


# return list of row indexes in tbl for each combination of by
# tbl should be a data.frame
.find.by.rows = function(tbl, by) {
    if (is.null(by))
        return(list(seq_len(nrow(tbl))))
    if (any(is.na(match(by, names(tbl)))))
        stop("'by' variables are not all in the grid")    
    bylevs = tbl[ , by, drop = FALSE]
    by.id = do.call("paste", unname(bylevs))
    uids = unique(by.id)
    result = lapply(uids, function(id) which(by.id == id))
    names(result) = uids
    result
}

# calculate the offset for the given grid
#' @rdname extending-emmeans
#' @order 35
#' @param terms A \code{terms} component
#' @return \code{.get.offset} returns the values, based on \code{grid}, of 
#' any \code{offset} component in \code{terms}
#' @export
.get.offset = function(terms, grid) {
    off.idx = attr(terms, "offset")
    offset = rep(0, nrow(grid))
    tvars = attr(terms, "variables")
    for (i in off.idx)
        offset = offset + eval(tvars[[i+1]], grid)
    offset
}

# combine variables in several `terms` objects
#' @rdname extending-emmeans
#' @order 31
#' @return \code{combine.terms} returns a \code{terms} object resulting
#'   from combining all the terms or formulas in \code{...}.
#' @export
.combine.terms = function(...) {
    trms = list(...)
    vars = unlist(lapply(trms, .all.vars))
    terms(.reformulate(vars, env = environment(trms[[1]])))
}

######################################################################
### Contributed by Jonathon Love, https://github.com/jonathon-love ###
### and adapted by RVL to exclude terms like df$trt or df[["trt"]] ###
######################################################################
# reformulate for us internally in emmeans
# same as stats::reformulate, except it surrounds term labels with backsticks
#
# RVL note: I renamed it .reformulate to avoid certain issues.
#   For example I need reformulate() sometimes to strip off function calls
#   and this .reformulate works quite differently.
#
.reformulate <- function (termlabels, response = NULL, intercept = TRUE, env = parent.frame())
{
    if (!is.character(termlabels) || !length(termlabels))
        stop("'termlabels' must be a character vector of length at least one")
    has.resp = !is.null(response)
    termlabels = sapply(trimws(termlabels), function(x) 
        if (length(grep("\\$|\\[\\[", x)) > 0) x
        else paste0("`", x, "`"))
    termtext = paste(if (has.resp) "response", "~", 
                     paste(termlabels, collapse = "+"), collapse = "")
# prev version:                     paste0("`", trimws(termlabels), "`", collapse = "+"), collapse = "")
    if (!intercept)
        termtext = paste(termtext, "- 1")
    rval = eval(parse(text = termtext, keep.source = FALSE)[[1L]])
    if (has.resp)
        rval[[2L]] = if (is.character(response))
            as.symbol(response)
    else response
    environment(rval) = env
    rval
}

### Find variable names of the form df$x or df[["x"]]
# Returns indexes. In addition, return value has attribute
#   "details"  matrix with 1st row being dataset names, second row is variable names
.find.comp.names = function(vars) {
    comp = grep("\\$|\\[\\[", vars) # untick vars containing "$"
    if (length(comp) > 0) {
        attr(comp, "details") = gsub("\"", "",
            sapply(strsplit(vars[comp], "\\$|\\[\\[|\\]\\]"), function(.) .[1:2]))
    }
    comp
}


# returns a list of all matches to ... or lst with full names from args
.match.dots.list = function(args, ..., lst) {
    if(missing(lst))
        lst = list(...)
    idx = pmatch(names(lst), args, nomatch = 0)
    rtn = lst[idx > 0]
    names(rtn) = args[idx]
    rtn
}

### Find single arg in `...`. If pmatched, return its value, else NULL
.match.dots = function(arg, ..., lst) {
    rtn = .match.dots.list(arg[1], ..., lst = lst)
    if (length(rtn) > 0) rtn[[1]] else NULL
}


# return a list from ..., omitting any that pmatch omit
.zap.args = function(..., omit) {
    args = list(...)
    args[!is.na(pmatch(names(args), omit))] = NULL
    args
}

# return updated object with option list AND dot list
# optionally may exclude any opts in 'exclude.opts'
.update.options = function(object, options, ..., exclude.opts) {
    if (!is.list(options))
        options = as.list(options)
    dot.opts = .match.dots.list(.valid.misc, ...)
    if (!missing(exclude.opts))
        for (nm in exclude.opts)
            dot.opts[[nm]] = NULL
    # entries in both lists are overridden by those in ...
    for (nm in names(dot.opts))
        options[[nm]] = dot.opts[[nm]]
    options[["object"]] = object
    do.call(update.emmGrid, options)
}

# my own model.frame function. Intercepts compound names
# and fixes up the data component accordingly. We do this
# by creating data.frames within data having required variables of simple names 
model.frame = function(formula, data, ...) {
    if (is.null(data))
        return (stats::model.frame(formula, ...))
    idx = .find.comp.names(names(data))
    if (length(idx) > 0) {
        nm = names(data)[idx]
        others = names(data[-idx])
        details =  attr(idx, "details")
        num = suppressWarnings(as.numeric(details[2, ]))
        data = as.list(data)
        for (dfnm in unique(details[1, ])) {
            w = which(details[1, ] == dfnm)
            data[[dfnm]] = as.data.frame(data[nm[w]])
            names(data[[dfnm]]) = details[2, w]
            data[[dfnm]] = .reorder.cols(data[[dfnm]], num[w])
        }
        data[nm] = NULL # toss out stuff we don't need
        # save subst table in environ
        if (get_emm_option("simplify.names")) {
            details[1, ] = nm # top row is now fancy names, bottom is plain names
            all.nms = c(details[2, ], others)
            dup.cnt = sapply(details[2, ], function(.) sum(. == all.nms))
            dup.cnt[!is.na(num)] = 3 # don't simplify numeric ones
            details = details[, dup.cnt == 1, drop = FALSE]
            if (ncol(details) > 0)
                environment(formula)$.simplify.names. = details
        }
    }
    stats::model.frame(formula, data = data, ...)
}

# Utility to simplify names. each elt of top row of tbl is changed to bottom row
.simplify.names = function(nms, tbl) {
    for (j in seq_along(tbl[1,]))
        nms[nms == tbl[1, j]] = tbl[2, j]
    nms
}

# utility to make all names in a summary syntactically valid
.validate.names = function(object) {
    for (a in c("names", "pri.vars", "by.vars"))
        if (!is.null(att <- attr(object, a))) 
            attr(object, a) = make.names(att)
    object
}

# reorder columns of data frame to match numeric positions in num (if any)
.reorder.cols = function(data, num) {
    if (all(is.na(num)))
        return(data)
    m = max(num[!is.na(num)])
    nm = names(data)
    if (m > length(nm)) {
        k = length(nm)
        data = cbind(data, matrix(NA, nrow = nrow(data), ncol = m - k))
        nm = names(data) = c(nm, paste0(".xtra.", seq_len(m - k), "."))
    }
    for (i in num[!is.na(num)]) {
        j = which(nm == as.character(i))
        if (i != j) {
            tmp = nm[i]
            nm[i] = nm[j]
            nm[j] = tmp
        }
    }
    data[, nm, drop = FALSE]
}

# format sigma for use in  messages
.fmt.sigma = function(sigma) {
    if (length(sigma) == 1)
        round(sigma, 4 - floor(log10(sigma)))
    else
        "(various values)"
}

# format a transformation for messages
.fmt.tran = function(misc) {
    tran = misc$tran
    if (is.list(tran)) 
        tran = ifelse(is.null(tran$name), "custom", tran$name)
    if (!is.null(mul <- misc$tran.mult))
        tran = paste0(mul, "*", tran)
    if(!is.null(off <- misc$tran.offset))
        tran = paste0(tran, "(mu + ", off, ")")
    tran
}


# My own utility for requiring a namespace and handling case where it is not available
#   pkg      package name
#   ...      passed to fail
#   fail     function to call if namespace not found
#   quietly passed to requireNamespace()
## I can't decide definitively if I want to suppress S3 masking messages or not...
.requireNS = function(pkg, ..., fail = stop, quietly = TRUE) {
    ### result = suppressMessages(requireNamespace(pkg, quietly = quietly))
    result = requireNamespace(pkg, quietly = TRUE)
    if (!result) fail(...)
    result
}
# of possible use as fail in .requireNS
.nothing = function(...) invisible()


### Utilities for converting symm matrices to and from lower-triangle storage mode
.get.lt = function(X) {
    rtn = X[lower.tri(X, diag = TRUE)]
    attr(rtn, "nrow") = nrow(X)
    rtn
}

.lt2mat = function(lt) {
    if (is.null(n <- attr(lt, "nrow")))
        n = (sqrt(8 * length(lt) + 1) - 1) / 2
    X = matrix(NA, nrow = n, ncol = n)
    lti = which(lower.tri(X, diag = TRUE))
    X[lti] = lt
    X = t(X)
    X[lti] = lt
    X
}

### submodel support...

# Compact a model matrix
# This returns just the R part of X's QR decomposition.
# This is sufficient in lieu of the whole model matrix
# Ideally, X is already a qr object in which case weights is assumed already incorporated
# assign should be right if input is generated by model.matrix() or is $qr slot of lm
#' @rdname extending-emmeans
#' @order 33
#' @param X,weights,assign Arguments for \code{.cmpMM}, which compacts a model
#'   matrix \code{X} into a much smaller matrix that has the same row space.
#'   Specifically, it returns the R portion of its QR decomposition. If \code{X}
#'   is already of class \code{qr}, it is used directly. \code{weights} should be
#'   the weights used in the model fit, and \code{assign} is used for unravelling
#'   any pivoting done by \code{\link{qr}}.
#' @export
.cmpMM = function(X, weights = rep(1, nrow(X)), assign = attr(X$qr, "assign")) {
    if (!get_emm_option("enable.submodel"))
        return(NULL)
    if(!is.qr(X)) {
        if(any(is.na(X)))
            return(NULL)
        X = try({
            X = sweep(X, 1, sqrt(weights), "*") # scale rows by sqrt(weights)
            qr(X)
        }, silent = TRUE)
        if (inherits(X, "try-error"))
            return(NULL)
    }
    R = qr.R(X, complete = FALSE)
    R[, X$pivot] = R
    colnames(R)[X$pivot] = colnames(R)
    attr(R, "assign") = assign
    R
}

# Get 'factors' table and simplify it (remove function calls from term labels)
.smpFT = function(trms) {
    tbl = attr(trms, "factors")
    rn = rownames(tbl)
    newrn = sapply(rn, function(x) all.vars(as.formula(paste("~", x)))[1])
    if (any(newrn != rn)) {
        rownames(tbl) = newrn
        colnames(tbl) = apply(tbl, 2, function(x) paste(newrn[x > 0], collapse = ":"))
    }
    tbl
}

# Alias matrix. Goal here is to find indexes
#    i1 = indices of columns of smaller model
#    i2 = indices of all other terms
# Then for a given linear hypothesis L_R %*% bhat_R (_R subscripts smaller model)
#    we have L_R %*% bhat_R = L_R %*% (A %*% bhat_F) = (L_R %*% A) %*% bhat_F
#    (where _F subscripts full model).
# Moreover, L_R is just columns i1 of the linfct for the effect of interest.
.alias.matrix = function(object, submodel) {
    X = object@model.info$model.matrix  # assumed to have attributes "factors" and "assign"
    if (is.character(X)) {  # model.matrix is a message
        if (nchar(X) > 0) warning(X, call. = FALSE)
        return(NULL)
    }
    assign = attr(X, "assign")
    if (is.null(assign)) {  ### either missing model matrix or assign attribute
        warning("submodel information is not available for this object", call. = FALSE)
        return(NULL)
    }
    tbl = attr(X, "factors")
    if (is.character(submodel)) {
        type2 = pmatch(submodel[1], "type2", nomatch = 0) # 1 if type2, 0 otherwise
        submodel = as.formula(paste("~", paste(names(object@levels), collapse="*")))
    }
    else
        type2 = 0
    # now submodel is a formula
    # create term labels in compatible factor order
    subtbl = attr(terms(update(object@model.info$terms, submodel)), "factors")
    com = intersect(rownames(tbl), rownames(subtbl))
    if(length(com) == 0) { # No matching factors at all
        com = 1; 
        subtbl = matrix(0)
    }
    sublab = apply(subtbl[com, , drop = FALSE], 2, function(x) paste(com[x > 0], collapse = ":"))
    usecols = intersect(colnames(tbl), sublab)
    incl = c(0, which(colnames(tbl) %in% usecols))
    if (type2) {
        incl = max(incl)  # just the last one
        overlap = apply(tbl, 2, function(x) sum(x * tbl[, incl]))
        zaplist = which(overlap < sum(tbl[, incl])) # don't contain our term
        i0 = which(assign %in% c(0, zaplist))
    }
    else
        i0 = integer(0)
    rcols = which(assign %in% incl)
    if (length(i0) > 0) {
        X[, rcols] = qr.resid(qr(X[, i0, drop = FALSE]), X[, rcols, drop = FALSE])
        X[, i0] = 0
    }
    A = qr.coef(qr(X[, rcols, drop = FALSE]), X)  # alias matrix
    A[is.na(A)] = 0  ## NA coefs are really ones constrained to zero
    attr(A, "rcols") = rcols   # cols in submodel
    attr(A, "submodstr") = paste(colnames(tbl)[incl], collapse = " + ")
    A
}

##############################################################################
#    Copyright (c) 2012-2022 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################



# Combine two or more factors (named in facs) into a new factor named newname
#' Manipulate factors in a reference grid
#' 
#' These functions manipulate the levels of factors comprising a reference
#' grid by combining factor levels, splitting a factor's levels into 
#' combinations of newly-defined factors, creating a grouping factor in which 
#' factor(s) levels are nested, or permuting the order of levels of a factor
#' 
#' 
#' @param object An object of class \code{emmGrid}
#' @param facs Character vector. The names of the factors to combine
#' @param newname Character value. The name of the new factor
#' @param drop Logical value. If \code{TRUE}, any levels of the new factor that 
#'     are dropped if all occurrences in the newly reconstructed object have 
#'     weight zero. If \code{FALSE}, all levels are retained. 
#'     (This argument is ignored if there is no \code{.wgt.} column
#'     in \code{object@grid}.)
#' @param ... arguments passed to other methods
#' @rdname manip-factors
#'
#' @return A modified object of class \code{emmGrid}
#'
#' @section The \code{comb_facs} function:
#' \code{comb_facs} combines the levels of factors into a single factor
#' in the reference grid (similar to \code{\link{interaction}}). This new factor
#' replaces the factors that comprise it.
#' 
#' \emph{Additional note:}
#' The choice of whether to drop levels or not can make a profound difference.
#' If the goal is to combine factors for use in \code{joint_tests}, we advise \emph{against} 
#' \code{drop = TRUE} because that might change the weights used in deriving marginal means.
#' If combining factors in a nested structure, dropping unused cases can considerably reduce 
#' the storage required.
#'  
#'
#' @export
#'
#' @examples
#' mtcars.lm <- lm(mpg ~ factor(vs)+factor(cyl)*factor(gear), data = mtcars)
#' (v.c.g <- ref_grid(mtcars.lm))
#' (v.cg <- comb_facs(v.c.g, c("cyl", "gear")))
#'   
#' # One use is obtaining a single test for the joint contributions of two factors:
#' joint_tests(v.c.g)
#' 
#' joint_tests(v.cg)
#' 
#' # undo the 'comb_facs' operation:
#' split_fac(v.cg, "cyl.gear", list(cyl = c(4, 6, 8), gear = 3:5))
#' 
comb_facs = function(object, facs, newname = paste(facs, collapse = "."),
                     drop = FALSE, ...) {
    object = .chk.list(object, ...)
    if((length(facs)  < 1))
        stop("No factors have been specified to combine")
    levs = object@levels
    if(any(sapply(facs, function(x) !(x %in% names(levs)))))
        stop("Unknown factor(s)")
    if (newname %in% facs)
        stop("newname of '", newname, "' cannot be used. Specify something else")
    grid = object@grid
    idx = sapply(facs, function(x) which(names(levs) == x))
    levs[[newname]] = do.call(paste, c(do.call(expand.grid, levs[idx]), sep = ":"))
    grid[[newname]] = do.call(paste, c(grid[idx], sep = ":"))
    levs[idx] = grid[idx] = NULL
    if (drop && !is.null(w <- grid$.wgt.)) {
        rows = lapply(levs[[newname]], function(nm) which(grid[[newname]] == nm))
        maxw = sapply(rows, function(i) max(w[i]))
        zw = which(maxw == 0)
        if(length(zw) > 0) {
            r = unlist(rows[zw])
            levs[[newname]] = levs[[newname]][-r]
            grid = grid[-r, , drop = FALSE]
            object@linfct = object@linfct[-r, , drop = FALSE]
            if(!is.null(object@misc$display))
                object@misc$display = object@misc$display[-r]
        }
    }
    if(!is.null(nests <- object@model.info$nesting)) {
        # which of facs are nested?
        m = match(facs, names(nests), nomatch = 0)
        if (length(m[m > 0]) == length(facs)) { # all are nested, find common nests
            nst = nests[[m[1]]]
            for (mm in m[-1])
                nst = intersect(nst, nests[[mm]])
            if (length(nst) > 0)
                nests[newname] = nst
        }
        nests[m] = NULL # zap out all facs that are nested
        
        # look for where other factors are nested in facs factors
        for (nm in names(nests)) {
            # factors nested in any facs are also nested in the new factor
            if (any(facs %in% nests[[nm]])) {
                m = match(facs, nests[[nm]], nomatch = 0)
                nests[[nm]] = c(nests[[nm]][-m], newname)
            }
        }
        if (length(nests) == 0) nests = NULL
        object@model.info$nesting = nests
    }
    
    object@levels = levs
    object@grid = grid
    object@roles$predictors = c(object@roles$predictors[-idx], newname)
    
    ord = .std.order(grid, levs)
    object[ord]
}

#' @rdname manip-factors
#' @param fac The name of a factor that is part of the grid in \code{object}
#' @param newfacs A named list with the names of new factors
#'   and their levels. The names must not already exist in the object,
#'   and the product of the lengths of the levels must equal the number
#'   of levels of \code{fac}.
#'   
#' @section The \code{split_fac} function:
#' The levels in \code{newfacs} are expanded via \code{\link{expand.grid}} into
#' combinations of levels, and the factor \code{fac} is replaced by those 
#' factor combinations. Unlike \code{add_grouping}, this creates a crossed, 
#' rather than a nested structure. Note that the order of factor combinations
#' is systematic with the levels of first factor in \code{newfacs} varying 
#' the fastest; and those factor combinations are assigned respectively
#' to the levels of \code{fac} as displayed in \code{str(object)}.
#'
#' @export
#'
#' @examples
#' IS.glm <- glm(count ~ spray, data = InsectSprays, family = poisson)
#' IS.emm <- emmeans(IS.glm, "spray")
#' IS.new <- split_fac(IS.emm, "spray", list(A = 1:2, B = c("low", "med", "hi")))
#' str(IS.new)
#'
split_fac = function(object, fac, newfacs, ...) {
    object = .chk.list(object, ...)
    if (!(fac %in% names(object@grid)))
        stop("The factor '", fac, "' is not in the reference grid")
    newg = expand.grid(newfacs)
    ref = object@levels[[fac]]
    if(nrow(newg) != length(ref))
        stop("Mismatch between 'fac' levels (", length(ref),
             ") and new factor combinations (", nrow(newg), ")")
    iy = as.numeric(factor(object@grid[[fac]], levels = ref))
    repl = newg[iy, , drop = FALSE]
    idx = which(names(object@levels) == fac)
    i = seq_along(object@grid)
    object@grid = cbind(object@grid[i < idx], repl, object@grid[i>idx])
    i = seq_along(object@levels)
    object@levels = c(object@levels[i < idx], newfacs, object@levels[i > idx])
    object@roles$predictors = object@misc$pri.vars = names(object@levels)
    object@misc$by.vars = NULL
    if(!is.null(nests <- object@model.info$nesting)) {
        if(fac %in% names(nests)) {
            for (f in names(newfacs))
                nests[[f]] = nests[[fac]]
            nests[[fac]] = NULL
        }
        for (nm in names(nests)) 
            if (fac %in% nests[[nm]])
                nests[[nm]] = union(setdiff(nests[[nm]], fac), names(newfacs))
        object@model.info$nesting = nests
    }
    object
}


### Create a grouping factor and add it to a ref grid

#' @rdname manip-factors
#'
#' @param newname Character name of grouping factor to add (different from any
#'   existing factor in the grid)
#' @param refname Character name(s) of the reference factor(s)
#' @param newlevs Character vector or factor of the same length as that of the (combined) levels for 
#'   \code{refname}. The grouping factor \code{newname} will have the unique
#'   values of \code{newlevs} as its levels. The order of levels in \code{newlevs}
#'   is the same as the order of the level combinations produced by 
#'   \code{\link{expand.grid}} applied to the levels of \code{refname} -- that is, the
#'   first factor's levels change the fastest and the last one's vary the slowest.
#'
#' @section The \code{add_grouping} function:
#' This function adds a grouping factor to an existing reference grid or other 
#' \code{emmGrid} object, such that the levels of one or more existing factors (call them the
#' reference factors) are mapped to a smaller number of levels of the new
#' grouping factor. The reference factors are then nested in a 
#' new grouping factor named \code{newname}, and a new nesting structure
#' \code{refname \%in\% newname}.
#' This facilitates obtaining marginal means of the grouping factor, and 
#' contrasts thereof.
#' 
#' 
#' \emph{Additional notes:} By default, the levels of \code{newname} will be ordered
#'   alphabetically. To dictate a different ordering of levels, supply 
#'   \code{newlevs} as a \code{factor} having its levels in the desired order.
#'   
#' When \code{refname} specifies more than one factor, this can
#'   fundamentally (and permanently) change what is meant by the levels of those
#'   individual factors. For instance, in the \code{gwrg} example below, there
#'   are two levels of \code{wool} nested in each \code{prod}; and that implies
#'   that we now regard these as four different kinds of wool. Similarly, there
#'   are five different tensions (L, M, H in prod 1, and L, M in prod 2).
#'   
#' @export
#'
#' @examples
#' fiber.lm <- lm(strength ~ diameter + machine, data = fiber)
#' ( frg <- ref_grid(fiber.lm) )
#' 
#' # Suppose the machines are two different brands
#' brands <- factor(c("FiberPro", "FiberPro", "Acme"), levels = c("FiberPro", "Acme"))
#' ( gfrg <- add_grouping(frg, "brand", "machine", brands) )
#' 
#' emmeans(gfrg, "machine")
#' 
#' emmeans(gfrg, "brand")
#' 
#' ### More than one reference factor
#' warp.lm <- lm(breaks ~ wool * tension, data = warpbreaks)
#' gwrg <- add_grouping(ref_grid(warp.lm), 
#'     "prod",  c("tension", "wool"),  c(2, 1, 1,  1, 2, 1))
#'         # level combinations:         LA MA HA  LB MB HB
#' 
#' emmeans(gwrg, ~ wool * tension)   # some NAs due to impossible combinations
#' 
#' emmeans(gwrg, "prod")
#' 
add_grouping = function(object, newname, refname, newlevs, ...) {
    object = .chk.list(object, ...)
    if(!is.null(object@model.info$nesting[[refname]]))
        stop("'", refname, "' is already nested in another factor; cannot re-group it")
    if(newname %in% object@roles$predictors)
        stop("'", newname, "' is already the name of an existing predictor")
    rlevs = do.call(paste, do.call(expand.grid, object@levels[refname]))
    if (length(newlevs) != length(rlevs))
        stop("Length of 'newlevs' doesn't match # levels of '", refname, "'")
    newlevs = factor(newlevs)
    glevs = levels(newlevs)
    k = length(glevs)
    
    one = matrix(1, nrow = k, ncol = 1)
    object@linfct = kronecker(one, object@linfct)
    object@levels[[newname]] = glevs
    object@roles$predictors = c(object@roles$predictors, newname)
    
    ref = do.call(paste, object@grid[refname]) # obs levels of rlevs
    wgt = object@grid$.wgt.
    if (is.null(wgt)) wgt = rep(1, nrow(object@grid))
    offset = object@grid$.offset.
    ogrid = object@grid[setdiff(names(object@grid), c(".wgt.", ".offset."))]
    grid = data.frame()
    valid = logical(0) # flag for rows that make sense
    for (i in 1:k) {
        g = ogrid
        g[[newname]] = glevs[i]
        g$.wgt. = wgt
        g$.offset. = offset
        grid = rbind(grid, g)
        alevs = rlevs[newlevs == glevs[i]]
        valid = c(valid, ref %in% alevs)
    }
    # screen out invalid rows
    grid[!valid, ".wgt."] = 0
    object@linfct[!valid, ] = NaN
    object@misc$pri.vars = c(object@misc$pri.vars, newname)
    if(is.null(disp <- object@misc$display))
        object@misc$display = valid
    else
        object@misc$display = disp & valid
    object@grid = grid
    
    # update nesting structure
    nesting = object@model.info$nesting
    if (is.null(nesting))
        nesting = list()
    for (nm in names(nesting))
        if (any(refname %in% nesting[[nm]]))
            nesting[[nm]] = c(nesting[[nm]], newname)
    for (nm in refname)
        nesting[[nm]] = newname   ### ??? should it be c(nesting[[nm]], newname)
    object@model.info$nesting = nesting
    
    object
}

#' @rdname manip-factors
#' @export
#' 
#' @section The \code{add_submodels} function:
#' This function updates \code{object} with a named list of submodels specified in 
#' \code{...}. These are \code{rbind}ed together and the corresponding rows
#' for each submodel are assigned a factor named \code{newname} with levels 
#' equal to the names in \code{...}. This facilitates comparing estimates obtained
#' from different submodels. For this to work, the underlying model object must be of
#' a class supported by the \code{submodel} argument of \code{\link{update.emmGrid}}.
#' 
#' @examples
#' ## Using 'add_submodels' to compare adjusted and unadjusted means
#' fibint.lm <- lm(strength ~ machine * diameter, data = fiber)
#' fibsub <- add_submodels(emmeans(fibint.lm, "machine"), 
#'     full = ~ ., additive = ~ . - machine:diameter, unadj = ~ machine)
#' emmeans(fibsub, pairwise ~ model | machine, adjust = "none")
#' 
add_submodels = function(object, ..., newname = "model") {
    all = lapply(list(...), \(s) update(object, submodel = s))
    class(all) = c("emm_list", "list")
    comb = rbind(all)
    levels(comb)[[newname]] = names(all)
    comb
}



#' @rdname manip-factors
#' @param pos Integer vector consisting of some permutation of the sequence
#' \code{1:k}, where \code{k} is the number of levels of \code{fac}.
#' This determines which position each level of \code{fac} will occupy
#' after the levels are permuted; thus, if the
#' levels of \code{fac} are \code{A,B,C,D}, and \code{pos = c(3,1,2,4)}, 
#' then the permuted levels will be \code{B,C,A,D}.
#' 
#' @section The \code{permute_levels} function:
#' This function permutes the levels of \code{fac}. The returned object
#' has the same factors, same \code{by} variables, but with the levels
#' of \code{fac} permuted. 
#' The order of the columns in \code{object@grid} may be altered.
#' 
#' NOTE: \code{fac} must not be nested in another factor. \code{permute_levels} 
#' throws an error when \code{fac} is nested.
#' 
#' NOTE: Permuting the levels of a numeric predictor is tricky. For example,
#' if you want to display the new ordering of levels in \code{emmip()},
#' you must add the arguments \code{style = "factor"} and \code{nesting.order = TRUE}.
#' 
#' @examples
#' # Permuting factor levels...
#' str(v.c.g)
#' str(permute_levels(v.c.g, "cyl", c(2,3,1)))
#' 
#' @export
permute_levels = function(object, fac, pos) {
    by.orig = object@misc$by.vars
    newlevs = object@levels[[fac]]
    newlevs[pos] = newlevs
    f = facs = names(object@levels)
    # Here's the trick: nest fac in .tmp. (one group per level)
    obj1 = add_grouping(object, ".tmp.", fac, pos)
    f[facs == fac] = ".tmp."
    # Then average-out fac, leaving .tmp. w/ same means
    obj2 = emmeans(obj1, f, by = NULL)
    # Finally, replace the names and levels of .tmp
    l = obj2@levels
    l[[which(names(l) == ".tmp.")]] = newlevs
    names(l)[names(l) == ".tmp."] = fac
    levels(obj2) = l
    obj2@misc$by.vars = by.orig
    obj2
}

context("Contrast function")

pigs.lm = lm(log(conc) ~ source + factor(percent), data = pigs)
rg = ref_grid(pigs.lm)
rgg = add_grouping(rg, "group", "source", c("1", "2", "2"))
emms = emmeans(rg, "source")
emmg = emmeans(rgg, "group")
emmgs = emmeans(rgg, "source")

pigs.lmi = lm(log(conc) ~ source * factor(percent), data = pigs)
rgi = ref_grid(pigs.lmi)

test_that("Non-nested contrasts work", {
    expect_equal(nrow(summary(contrast(emms))), 3)
    expect_equal(nrow(summary(contrast(emms, "consec"))), 2)
    expect_equal(nrow(summary(contrast(rg, by = "source"))), 12)
    expect_equal(nrow(summary(pairs(rg, by = "source"))), 18)
    expect_equal(nrow(summary(pairs(rg))), 66)
})

test_that("Nested contrasts work", {
    expect_equal(nrow(summary(contrast(emmg, "consec"))), 1)
    expect_warning(summary(contrast(emmgs, "consec", by = "group"))) 
        # warning from cov2cor due to mvt adjustment
    expect_equal(nrow(summary(pairs(emmgs, by = "group"))), 2)
})

test_that("Interaction contrasts work", {
    expect_equal(nrow(summary(contrast(rgi, interaction = TRUE))), 12)
    expect_equal(nrow(summary(contrast(rgi, interaction = "consec"))), 6)
    expect_equal(nrow(summary(contrast(rgi, interaction = c("consec", "pairwise")))), 12)
    expect_equal(nrow(summary(contrast(rgi, interaction = c("pairwise","consec")))), 9)
})

context("Estimated marginal means")

pigs.lm = lm(log(conc) ~ source + factor(percent), data = pigs)
rg = ref_grid(pigs.lm)
rgg = add_grouping(rg, "group", "source", c("1", "2", "2"))

test_that("Character interface works", {
    expect_equal(confint(emmeans(rg, "source"))$emmean, c(3.39, 3.67, 3.80), tol = 0.01)
    expect_equal(nrow(emmeans(rg, c("source", "percent"))@grid), 12)
    expect_equal(nrow(emmeans(rg, "source", by = "percent")@grid), 12)
    expect_equal(nrow(emmeans(rg, c("1"))@grid), 1)
})

test_that("Formula interface works", {
    expect_equal(confint(emmeans(rg, ~ source))$emmean, c(3.39, 3.67, 3.80), tol = 0.01)
    expect_equal(nrow(emmeans(rg, ~ source * percent)@grid), 12)
    expect_equal(nrow(emmeans(rg, ~ source | percent)@grid), 12)
    expect_equal(nrow(emmeans(rg, ~ 1)@grid), 1)
    expect_equal(nrow(emmeans(rg, ~ 1 | percent)@grid), 4)
    expect_equal(nrow(emmeans(rg, ~ percent | 1)@grid), 4)
})

# nesting
test_that("Nested EMMs work", {
    expect_equal(nrow(emmeans(rgg, ~ group)@grid), 2)
    expect_equal(nrow(emmeans(rgg, ~ source)@grid), 6) #   3 rows have 0 weight
    expect_equal(nrow(confint(emmeans(rgg, ~ source))), 3)
    expect_equal(colnames(emmeans(rgg, ~ source)@grid)[1:2], c("source","group"))
})

## ---- echo = FALSE, results = "hide", message = FALSE-------------------------
require("emmeans")
knitr::opts_chunk$set(fig.width = 4.5, class.output = "ro")

## -----------------------------------------------------------------------------
pigs.lm <- lm(log(conc) ~ source + factor(percent), data = pigs)
pigs.emm.s <- emmeans(pigs.lm, "source")
pairs(pigs.emm.s)

## -----------------------------------------------------------------------------
pwpm(pigs.emm.s)

## -----------------------------------------------------------------------------
pwpm(pigs.emm.s, means = FALSE, flip = TRUE,     # args for pwpm()
     reverse = TRUE,                             # args for pairs()
     side = ">", delta = 0.05, adjust = "none")  # args for test()

## -----------------------------------------------------------------------------
eff_size(pigs.emm.s, sigma = sigma(pigs.lm), edf = 23)

## -----------------------------------------------------------------------------
eff_size(pigs.emm.s, sigma = sigma(pigs.lm), edf = Inf)

## ---- eval = FALSE------------------------------------------------------------
#  eff_size(pairs(pigs.emm.s), sigma = sigma(pigs.lm), edf = 23, method = "identity")

## ----fig.height = 1.5, fig.alt = "side-by-side CIs with comparison arrows added"----
plot(pigs.emm.s, comparisons = TRUE)

## ----fig.alt = "Pairwise P-value plot that shows, for each pair of means, a vertical line segment whose horizontal position is the Tukey-adjusted P-value for that comparison. The endpoints of the line segments align with the vertical scale showing the soy levels and their means. This particular plot shows that skim-fish and soy-fish are highyly significant, while skim-soy has a P-value just over 0.05"----
pwpp(pigs.emm.s)

## ---- fig.width = 9, fig.alt = "pwpp for all 78 pairwise comparisons of cell means. It is pretty hard to digest due to its complexity. To see all this information in text form, call pwpm(pigs.cells)"----
pigs.lmint <- lm(log(conc) ~ source * factor(percent), data = pigs)
pigs.cells <- emmeans(pigs.lmint, ~ source * percent)
pwpp(pigs.cells, type = "response")

## ---- fig.width = 6, fig.alt = "pwpp presented in separate panels for each source. Each panel has just 6 P-value bars, making it much less cluttered than the previous figure"----
pwpp(pigs.cells, by = "source", type = "response")

## -----------------------------------------------------------------------------
coef(pairs(pigs.emm.s))

## -----------------------------------------------------------------------------
pigs.emm.p <- emmeans(pigs.lm, "percent")
ply <- contrast(pigs.emm.p, "poly")
ply

coef(ply)

## -----------------------------------------------------------------------------
org.aov <- aov(sales1 ~ day + Error(store), data = oranges,
               contrasts = list(day = "contr.sum"))
org.emml <- emmeans(org.aov, consec ~ day)
org.emml

## -----------------------------------------------------------------------------
skip_comp.emmc <- function(levels, skip = 1, reverse = FALSE) {
    if((k <- length(levels)) < skip + 1)
        stop("Need at least ", skip + 1, " levels")
    coef <- data.frame()
    coef <- as.data.frame(lapply(seq_len(k - skip - 1), function(i) {
        sgn <- ifelse(reverse, -1, 1)
        sgn * c(rep(0, i - 1), 1, rep(0, skip), -1, rep(0, k - i - skip - 1))
    }))
    names(coef) <- sapply(coef, function(x)
        paste(which(x == 1), "-", which(x == -1)))
    attr(coef, "adjust") = "fdr"   # default adjustment method
    coef
}

## -----------------------------------------------------------------------------
skip_comp.emmc(1:5)

skip_comp.emmc(1:5, skip = 0, reverse = TRUE)

## -----------------------------------------------------------------------------
contrast(org.emml[[1]], "skip_comp", skip = 2, reverse = TRUE)

## -----------------------------------------------------------------------------
LF <- contrast(pigs.emm.s, 
               list(lambda1 = c(1, 2, 0), lambda2 = c(0, 3, -2)),
               offset = c(-7, 1))
confint(LF, adjust = "bonferroni")

## -----------------------------------------------------------------------------
pairs(pigs.emm.s, type = "lp")

pairs(pigs.emm.s, type = "response")

##############################################################################
#    Copyright (c) 2012-2022 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

### support for the ordinal package

#' @exportS3Method recover_data clm
recover_data.clm = function(object, mode = "latent", ...) {
    wts = object$weights = object$model[["(weights)"]]
    if (mode %.pin% "scale") {  ###(!is.na(pmatch(mode, "scale"))) {
        if (is.null(trms <- object$S.terms))
            return("Specified mode=\"scale\", but no scale model is present") # ref_grid's error handler takes it from here
        recover_data(object$call, trms, object$na.action, pwts = wts, ...)
    }
    else if (is.null(object$S.terms) && is.null(object$nom.terms))
        recover_data.lm(object, ...)
    else { # bring-in predictors from loc, scale, and nom models
        trms = delete.response(object$terms)
        x.preds = union(.all.vars(object$S.terms), .all.vars(object$nom.terms))
        x.trms = terms(update(trms, .reformulate(c(".", x.preds))))
        recover_data(object$call, x.trms, object$na.action, pwts = wts, ...)
    }
}

# For now at least, clmm doesn't cover scale, nominal options
#' @exportS3Method recover_data clmm
recover_data.clmm = function(object, ...) {
    object$weights = object$model[["(weights)"]]
    recover_data.lm(object, ...)
}

# Note: For ALL thresholds, object$Theta has all the threshold values
# for the different cuts (same as object$alpha when threshold=="flexible")
# and object$tJac is s.t. tJac %*% alpha = Theta
# Note also that some functions of cut are constrained to be zero when
# threshold != "flexible". Can get basis using nonest.basis(t(tJac))
#
# opt arg 'mode' - determines what goes into ref_grid
#         'rescale' - (loc, scale) for linear transformation of latent result

#' @exportS3Method emm_basis clm          
emm_basis.clm = function (object, trms, xlev, grid, 
                          mode = c("latent", "linear.predictor", "cum.prob", "exc.prob", "prob", "mean.class", "scale"), 
                          rescale = c(0,1), ...) {
    # general stuff
    mode = match.arg(mode)
    if (mode == "scale")
        return (.emm_basis.clm.scale(object, trms, xlev, grid, ...))
    
    # if (is.null(object$contrasts))
    #     warning("Contrasts used to fit the model are unknown.\n",
    #             "Defaulting to system option, but results may be wrong.")
    
    bhat = coef(object)
    V = .my.vcov(object, ...)
    tJac = object$tJac
    dffun = function(...) Inf
    link = as.character(object$info$link)
    cnm = dimnames(object$tJac)[[1]]
    if (is.null(cnm))
        cnm = paste(seq_len(nrow(tJac)), "|", 1 + seq_len(nrow(tJac)), sep = "")
    misc = list()
    
    # My strategy is to piece together the needed matrices for each threshold parameter
    # Then assemble the results
    
    ### ----- Location part ----- ###
    contrasts = object$contrasts
    # Remember trms was trumped-up to include scale and nominal predictors.
    # Recover the actual terms for the principal model
    trms = delete.response(object$terms)
    m = model.frame(trms, grid, na.action = na.pass, xlev = object$xlevels)
    X = model.matrix(trms, m, contrasts.arg = contrasts)
    # Need following code because clmm objects don't have NAs for dropped columns...
    nms.needed = c(names(object$alpha), setdiff(colnames(X), "(Intercept)"))
    if (length(setdiff(nms.needed, bnm <- names(bhat))) > 0) {
        bhat = seq_along(nms.needed) * NA
        names(bhat) = nms.needed
        bhat[bnm] = coef(object)
        object$coefficients = bhat # will be needed by model.matrix
        object$beta = bhat[setdiff(nms.needed, names(object$alpha))]
    }
    xint = match("(Intercept)", colnames(X), nomatch = 0L)
    if (xint > 0L) {
        X = X[, -xint, drop = FALSE]
    }
    
    ### ----- Nominal part ----- ###
    if (is.null(object$nom.terms))
        NOM = matrix(1, nrow = nrow(X))
    else {
        mn = model.frame(object$nom.terms, grid, na.action = na.pass, xlev = object$nom.xlevels)
        NOM = model.matrix(object$nom.terms, mn, contrasts.arg = object$nom.contrasts)
    }
    bigNom = kronecker(tJac, NOM)
    # cols are in wrong order... I'll get the indexes by transposing a matrix of subscripts
    if (ncol(NOM) > 1)
        bigNom = bigNom[, as.numeric(t(matrix(seq_len(ncol(bigNom)), nrow=ncol(NOM))))]
    
    ### ----- Scale part ----- ###
    if (!is.null(object$S.terms)) {
        ms = model.frame(object$S.terms, grid, na.action = na.pass, xlev = object$S.xlevels)
        S = model.matrix(object$S.terms, ms, contrasts.arg = object$S.contrasts)
        S = S[, names(object$zeta), drop = FALSE]
        if (!is.null(attr(object$S.terms, "offset"))) {
            soff = .get.offset(object$S.terms, grid)
            # we'll add a column to S and adjust bhat and V accordingly
            S = cbind(S, offset = soff)
            bhat = c(bhat, offset = 1)
            V = rbind(cbind(V, offset = 0), offset = 0)
        }
        si = misc$scale.idx = length(object$alpha) + length(object$beta) + seq_len(ncol(S))
        # Make sure there are no name clashes
        names(bhat)[si] = paste(".S", names(object$zeta), sep=".")
        misc$estHook = ".clm.estHook"
        misc$vcovHook = ".clm.vcovHook"
    }
    else
        S = NULL
    
    ### ----- Get non-estimability basis ----- ###
    nbasis = snbasis = estimability::all.estble
    if (any(is.na(bhat))) {
        obj = object   # work around fact that model.matrix.clmm doesn't work
        class(obj) = "clm"
        mm = try(model.matrix(obj), silent = TRUE)
        if (inherits(mm, "try-error"))
            stop("Currently, it is not possible to construct a reference grid for this\n",
                 "object, because it is rank-deficient and no model matrix is available.")
        
        # note: mm has components X, NOM, and S
        if (any(is.na(c(object$alpha, object$beta)))) {
            NOMX = if (is.null(mm$NOM)) mm$X
                   else                 cbind(mm$NOM, mm$X[, -1])
            nbasis = estimability::nonest.basis(NOMX)
            # replicate and reverse the sign of the NOM parts
            nomcols = seq_len(ncol(NOM))
            nomreps = length(object$alpha) / ncol(NOM)
            nbasis = apply(nbasis, 2, function(x)
                c(rep(-x[nomcols], each = nomreps), x[-nomcols]))
        }
        if (!is.null(mm$S)) {
            if (any(is.na(object$zeta))) {
                snbasis = estimability::nonest.basis(mm$S)
                # put intercept part at end
                snbasis = rbind(snbasis[-1, , drop=FALSE], snbasis[1, ])
                if (!is.null(attr(object$S.terms, "offset")))
                    snbasis = rbind(snbasis, 0)
                snbasis = rbind(matrix(0, ncol=ncol(snbasis), nrow=min(si)-1), snbasis)
                 # Note scale intercept is included, so tack it on to the end of everything
                S = cbind(S, .S.intcpt = 1)
                bhat = c(bhat, .S.intcpt = 0)
                V = rbind(cbind(V, .S.intcpt = 0), .S.intcpt = 0)
                si = misc$scale.idx = c(si, 1 + max(si))
            }
        }
        if (is.na(nbasis[1])) # then only nonest part is scale
            nbasis = snbasis
        else { 
            if (!is.null(S)) # pad nbasis with zeros when there's a scale model
                nbasis = rbind(nbasis, matrix(0, nrow=length(si), ncol=ncol(nbasis)))
            if (!is.na(snbasis[1]))
                nbasis = cbind(nbasis, snbasis)
        }
    }
    
    if (mode == "latent") {
        # Create constant columns for means of scale and nominal parts
        J = matrix(1, nrow = nrow(X))
        nomm = rescale[2] * apply(bigNom, 2, mean)
        X = rescale[2] * X
        if (!is.null(S)) {
            sm = apply(S, 2, mean)
            X = cbind(X, kronecker(-J, matrix(sm, nrow = 1)))
        }
        bigX = cbind(kronecker(-J, matrix(nomm, nrow = 1)), X)
        misc$offset.mult = misc$offset.mult * rescale[2]
        intcpt = seq_len(ncol(tJac))
        bhat[intcpt] = bhat[intcpt] - rescale[1] / rescale[2]
    }
    else { ### ----- Piece together big matrix for each threshold ----- ###
        misc$ylevs = list(cut = cnm)
        # support for links not in make.link
        if (is.character(link) && !(link %in% c("logit", "probit", "cauchit", "cloglog"))) {
            setLinks = get("setLinks", asNamespace("ordinal"))
            env = new.env()
            setLinks(env, link)
            link = list(linkfun = quote(stop), 
                        linkinv=env$pfun, mu.eta = env$dfun, name = env$link, 
                        lambda = env$lambda)
        }
        misc$tran = link
        misc$inv.lbl = "cum.prob"
        misc$offset.mult = -1
        if (!is.null(S))
            X = cbind(X, S)
        J = matrix(1, nrow=nrow(tJac))
        bigX = cbind(bigNom, kronecker(-J, X))
        if (mode != "linear.predictor") {
            misc$mode = mode
            misc$respName = as.character.default(object$terms)[2]
            misc$postGridHook = ".clm.postGrid"
        }
    }
    
    dimnames(bigX)[[2]] = names(bhat)
    
    list(X = bigX, bhat = bhat, nbasis = nbasis, V = V, dffun = dffun, 
         dfargs = list(), misc = misc)
}

# function called at end of ref_grid
# I use this for polr as well
# Also used for stanreg result of stan_polr & potentially other MCMC ordinal models
.clm.postGrid = function(object, ...) {
    mode = object@misc$mode
    object@misc$postGridHook = object@misc$mode = NULL
    object = regrid(object, transform = "response", ...)
    if(object@misc$estName == "exc.prob") { # back-transforming yields exceedance probs
        object@bhat = 1 - object@bhat
        if(!is.null(object@post.beta[1]))
            object@post.beta = 1 - object@post.beta
        object@misc$estName = "cum.prob"
    }
    if (mode == "prob") {
        object = .clm.prob.grid(object, ...)
    }
    else if (mode == "mean.class") {
        object = .clm.mean.class(object, ...)
    }
    else if (mode == "exc.prob") {
        object@bhat = 1 - object@bhat
        if(!is.null(object@post.beta[1]))
            object@post.beta = 1 - object@post.beta
        object@misc$estName = "exc.prob"        
    }
    # (else mode == "cum.prob" and it's all OK)
    object@misc$respName = NULL # cleanup
    object
}


# Make the linear-predictor ref_grid into one for class probabilities
# This assumes that object has already been re-gridded and back-transformed
.clm.prob.grid = function(object, thresh = "cut", newname = object@misc$respName, ...) {
    byv = setdiff(names(object@levels), thresh)
    newrg = contrast(object, ".diff_cum", by = byv, ...)
    newrg@grid$.offset. = (apply(newrg@linfct, 1, sum) < 0) + 0
    if (!is.null(wgt <- object@grid[[".wgt."]])) {
        km1 = length(object@levels[[thresh]])
        wgt = wgt[seq_len(length(wgt) / km1)] # unique weights for byv combs
        newrg = force_regular(newrg)
        key = do.call(paste, object@grid[byv])[seq_along(wgt)]
        tgt = do.call(paste, newrg@grid[byv])
        for (i in seq_along (wgt))
            newrg@grid[[".wgt."]][tgt == key[i]] = wgt[i]
    }
    # proceed to disavow that this was ever exposed to 'emmeans' or 'contrast'
    ## class(newrg) = "ref.grid"
    misc = newrg@misc
    if(!is.null(misc$display) && all(misc$display))
        misc$display = NULL
    misc$is.new.rg = TRUE
    misc$infer = c(FALSE,FALSE)
    misc$estName = "prob"
    misc$pri.vars = misc$by.vars = misc$con.coef = misc$orig.grid = NULL
    newrg@misc = misc
    conid = which(names(newrg@levels) == "contrast")
    names(newrg@levels)[conid] = names(newrg@grid)[conid] = newname
    newrg@roles = object@roles
    newrg@roles$multresp = newname
    newrg
}

# special 'contrast' fcn used by .clm.mean.class
.meanclass.emmc = function(levs, lf, ...) 
    data.frame(mean = lf)

.clm.mean.class = function(object, ...) {
    prg = .clm.prob.grid(object, newname = "class", ...)
    byv = setdiff(names(prg@levels), "class")
    lf = as.numeric(prg@levels$class)
    newrg = contrast(prg, ".meanclass", lf = lf, by = byv, ...)
    newrg = update(newrg, infer = c(FALSE, FALSE), 
        pri.vars = NULL, by.vars = NULL, estName = "mean.class")
    newrg@levels$contrast = newrg@grid$contrast = NULL
    prg@roles$multresp = NULL
    newrg@roles = prg@roles
    ## class(newrg) = "ref.grid"
    newrg@grid$.wgt. = prg@grid$.wgt.[sapply(.find.by.rows(prg@grid, byv), head, 1)]
    update(force_regular(newrg), is.new.rg = TRUE)
}

# Contrast fcn for turning estimates of cumulative probabilities
# into cell probabilities
.diff_cum.emmc = function(levs, sep = "|", ...) {
    plevs = unique(setdiff(unlist(strsplit(levs, sep, TRUE)), sep))
    k = 1 + length(levs)
    if (length(plevs) != k)
        plevs = seq_len(k)
    M = matrix(0, nrow = length(levs), ncol = k)
    for (i in seq_along(levs))
        M[i, c(i,i+1)] = c(1,-1)
    dimnames(M) = list(levs, plevs)
    M = as.data.frame(M)
    attr(M, "desc") = "Differences of cumulative probabilities"
    attr(M, "adjust") = "none"
    attr(M, "offset") = c(rep(0, k-1), 1)
    M
}

#### replacement estimation routines for cases with a scale param

## workhorse for estHook and vcovHook functions
.clm.hook = function(object, tol = 1e-8, ...) {
    scols = object@misc$scale.idx
    bhat = object@bhat
    active = !is.na(bhat)
    bhat[!active] = 0
    linfct = object@linfct
    estble = estimability::is.estble(linfct, object@nbasis, tol) ###apply(linfct, 1, .is.estble, object@nbasis, tol)
    estble[!estble] = NA
    rsigma = estble * as.numeric(linfct[, scols, drop = FALSE] %*% object@bhat[scols])
    rsigma = exp(rsigma) * estble
    # I'll do the scaling later
    eta = as.numeric(linfct[, -scols, drop = FALSE] %*% bhat[-scols])
    if (!is.null(object@grid$.offset.))
        eta = eta + object@grid$.offset.
    for (j in scols) linfct[, j] = eta * linfct[, j]
    # linfct = (.diag(rsigma) %*% linfct) [, active, drop = FALSE]
    linfct = sweep(linfct, 1, rsigma, "*") [, active, drop = FALSE]
    list(est = eta * rsigma, V = linfct %*% tcrossprod(object@V, linfct))
}

.clm.estHook = function(object, do.se = TRUE, tol = 1e-8, ...) {
    raw.matl = .clm.hook(object, tol, ...)
    SE = if (do.se) sqrt(diag(raw.matl$V))  else NA
    cbind(est = raw.matl$est, SE = SE, df = Inf)
}

.clm.vcovHook = function(object, tol = 1e-8, ...) {
    .clm.hook(object, tol, ...)$V
}

### Special emm_basis fcn for the scale model
.emm_basis.clm.scale = function(object, trms, xlev, grid, ...) {
    m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
    X = model.matrix(trms, m, contrasts.arg = object$S.contrasts)
    bhat = c(`(intercept)` = 0, object$zeta)
    nbasis = estimability::all.estble
    if (any(is.na(bhat)))
        nbasis = estimability::nonest.basis(model.matrix(object)$S)
    k = sum(!is.na(bhat)) - 1
    V = .my.vcov(object, ...)
    pick = nrow(V) - k + seq_len(k)
    V = V[pick, pick, drop = FALSE]
    V = cbind(0, rbind(0,V))
    misc = list(tran = "log")
    list(X = X, bhat = bhat, nbasis = nbasis, V = V, 
         dffun = function(...) Inf, dfargs = list(), misc = misc)
}

#' @exportS3Method emm_basis clmm          
emm_basis.clmm = function (object, trms, xlev, grid, ...) {
    if(is.null(object$Hessian)) {
        message("Updating the model to obtain the Hessian...")
        object = update(object, Hess = TRUE)
    }
    # borrowed from Maxime's code -- need to understand this better, e.g. when it happens
    H = object$Hessian
    if (any(apply(object$Hessian, 1, function(x) all(x == 0)))) {
        H = H[names(coef(object)), names(coef(object))]
        object$Hessian = H
    }
    result = emm_basis.clm(object, trms, xlev, grid, ...)
    # strip off covariances of random effects
    keep = seq_along(result$bhat[!is.na(result$bhat)])
    result$V = result$V[keep,keep]
    result
}


### Support for 'svyolr' objects (survey package)
### It appears all we have to do is pretend it is class polr
### and get the data via model.frame
#' @exportS3Method recover_data svyolr
recover_data.svyolr = function(object, data = NULL, ...) {
    if (is.null(data))
        data = get("model.frame.svyolr", asNamespace("survey"))(object)
    class(object) = "polr"
    recover_data(object, data = data, ...)
}
#' @exportS3Method emm_basis svyolr       
emm_basis.svyolr = function(object, ...) {
    class(object) = "polr"
    emm_basis(object, ...)
}

##############################################################################
#    Copyright (c) 2012-2017 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

# emmeans and related functions


# emmeans utility for a list of specs
emmeans.list = function(object, specs, ...) {
    result = list()
    nms = names(specs)
    # Format a string describing the results
    .make.desc = function(meth, pri, by) {
        pri = paste(pri, collapse = ", ")
        desc = paste(meth, "of", pri)
        if (!is.null(by)) {
            by = paste(by, collapse = ", ")
            desc = paste(desc, "|", by)
        }
        desc
    }
    name.arg = match.call()$name
    
    for (i in seq_len(length(specs))) {
        nm = nms[i]
        # We'll rename the contrasts if spec is named
        if (is.null(name.arg)) {
            contr.name = ifelse(nm == "", "contrast", paste0(nm, ".contrast"))
            res = emmeans(object=object, specs = specs[[i]], name = contr.name, ...)
        }
        else
            res = emmeans(object=object, specs = specs[[i]], ...)
        if (is.data.frame(res)) { # happens e.g. when cld is used
            if (is.null(nm))
                nm = .make.desc("summary", attr(res, "pri.vars"), attr(res, "by.vars"))
            result[[nm]] = res
        }
        else if (is.list(res)) {
            for (j in seq_len(length(res))) {
                m = res[[j]]@misc
                if (is.null(nm))
                    names(res)[j] = .make.desc(m$methDesc, res[[1]]@misc$pri.vars, m$by.vars)
                else
                    names(res)[j] = paste(nm, m$methDesc)
            }
            result = c(result,res)
        }
        else{
            if (is.null(nm))
                nm = .make.desc(res@misc$methDesc, res@misc$pri.vars, res@misc$by.vars)
            result[[nm]] = res
        }
    }
    class(result) = c("emm_list", "list")
    result
}



# Here's our flagship function!
#' Estimated marginal means (Least-squares means)
#' 
#' Compute estimated marginal means (EMMs) for specified factors
#' or factor combinations in a linear model; and optionally, comparisons or
#' contrasts among them. EMMs are also known as least-squares means.
#' 
#' Users should also consult the documentation for \code{\link{ref_grid}}, 
#' because many important options for EMMs are implemented there, via the 
#' \code{...} argument.
#'
#' @param object An object of class \code{emmGrid}; or a fitted model object
#'   that is supported, such as the result of a call to \code{lm} or
#'   \code{lmer}. Many fitted-model objects are supported; see
#'   \href{../doc/models.html}{\code{vignette("models", "emmeans")}} for details.
#' @param specs A \code{character} vector specifying the names of the predictors
#'   over which EMMs are desired. \code{specs} may also be a \code{formula}
#'   or a \code{list} (optionally named) of valid \code{spec}s. Use of formulas
#'   is described in the Overview section below. 
#'   Specifying \code{.} as the only factor name
#'   creates a list of specifications for all model terms.
#'   
#'   \bold{Note:} We recommend \emph{against} using two-sided formulas; see the
#'   note below for \code{contr}.
#' @param by A character vector specifying the names of predictors to condition on.
#' @param fac.reduce A function that combines the rows of a matrix into a single
#'   vector. This implements the ``marginal averaging'' aspect of EMMs. 
#'   The default is the mean of the rows. Typically if it is overridden,
#'   it would be some kind of weighted mean of the rows. If \code{fac.reduce} is
#'   nonlinear, bizarre results are likely, and EMMs will not be
#'   interpretable. NOTE: If the \code{weights} argument is non-missing,
#'   \code{fac.reduce} is ignored.
#' @param contr A character value or \code{list} specifying contrasts to be
#'   added. See \code{\link{contrast}}. \bold{Note:} \code{contr} is ignored when
#'   \code{specs} is a formula. \bold{Note 2:}: We recommend \emph{against} using this
#'   argument; obtaining means and obtaining contrasts are two different things,
#'   and it is best to do them in separate steps, using the \code{\link{contrast}}
#'   function for the contrasts.
#' @param options If non-\code{NULL}, a named \code{list} of arguments to pass
#'   to \code{\link{update.emmGrid}}, just after the object is constructed.
#'   (Options may also be included in \code{...}; see the \sQuote{options}
#'   section below.)
#' @param weights Character value, numeric vector, or numeric matrix specifying
#'   weights to use in averaging predictions. See \dQuote{Weights} section below.
#'   Also, if \code{object} is not already a reference grid, \code{weights}
#'   (if it is character) is passed to \code{ref_grid} as \code{wt.nuis} in case 
#'   nuisance factors are specified. We can override this by specifying 
#'   \code{wt.nuis} explicitly.
#'   This more-or-less makes the weighting of nuisance factors consistent with
#'   that of primary factors.
#' @param offset Numeric vector or scalar. If specified, this adds an offset to
#'   the predictions, or overrides any offset in the model or its
#'   reference grid. If a vector of length differing from the number of rows in 
#'   the result, it is subsetted or cyclically recycled.
#' @param ... When \code{object} is not already a \code{"emmGrid"}
#'   object, these arguments are passed to \code{\link{ref_grid}}. Common
#'   examples are \code{at}, \code{cov.reduce}, \code{data}, \code{type}, 
#'   \code{regrid}, \code{df}, \code{nesting}, and \code{vcov.}.
#'   Model-type-specific options (see
#'   \href{../doc/models.html}{\code{vignette("models", "emmeans")}}), commonly
#'   \code{mode}, may be used here as well. In addition, if the model formula
#'   contains references to variables that are not predictors, you must provide
#'   a \code{params} argument with a list of their names.
#'   These arguments may also be used in lieu of \code{options}. See the 
#'   \sQuote{Options} section below.
#' @param tran Placeholder to prevent it from being included in \code{...}.
#'   If non-missing, it is added to `options`. See the \sQuote{Options}
#'   section.
#'   
#' @return   When \code{specs} is a \code{character} vector or one-sided formula,
#'   an object of class \code{"emmGrid"}. A number of methods
#'   are provided for further analysis, including
#'   \code{\link{summary.emmGrid}}, \code{\link{confint.emmGrid}},
#'   \code{\link{test.emmGrid}}, \code{\link{contrast.emmGrid}},
#'   and \code{\link{pairs.emmGrid}}.

#' When \code{specs} is a \code{list} or a \code{formula} having a left-hand
#' side, the return value is an \code{\link{emm_list}} object, which is simply a
#' \code{list} of \code{emmGrid} objects. 
#' 
#' @section Overview:
#' Estimated marginal means or EMMs (sometimes called least-squares means) are
#' predictions from a linear model over a \emph{reference grid}; or marginal
#' averages thereof. The \code{\link{ref_grid}} function identifies/creates the
#' reference grid upon which \code{emmeans} is based.
#' 
#' For those who prefer the terms \dQuote{least-squares means} or
#' \dQuote{predicted marginal means}, functions \code{lsmeans} and
#' \code{pmmeans} are provided as wrappers. See \code{\link{wrappers}}.
#' 
#' If \code{specs} is a \code{formula}, it should be of the form \code{~ specs},
#' \code{~ specs | by}, \code{contr ~ specs}, or \code{contr ~ specs | by}. The
#' formula is parsed and the variables therein are used as the arguments
#' \code{specs}, \code{by}, and \code{contr} as indicated. The left-hand side is
#' optional (and we don't recommend it), but if specified it should be the 
#' name of a contrast family (e.g., \code{pairwise}). Operators like
#' \code{*} or \code{:} are needed in the formula to delineate names, but
#' otherwise are ignored.
#' 
#' We now also allow using \code{.} in \code{specs}. If this is done, we run
#' \code{\link{joint_tests}} on the side to determine all relevant model terms,
#' then replace \code{specs} with a corresponding list of specifications. This is
#' a convenience, but it can create a sizeable \code{emm_list} object and it is
#' coded rather inefficiently. While it is permissible to include contrasts
#' via a \code{contr} argument or formula left-hand-side, we recommend instead
#' doing this in a follow-up test with \code{\link{contrast.emm_list}}. \emph{Caution}:
#' In models with nested fixed effects, using \code{.} creates results where
#' nested factors interact with nesting factors; in those cases, any contrasts 
#' you specify will go across nests, which is likely not what is desired.
#' 
#' In the special case where the mean (or weighted mean) of all the predictions
#' is desired, specify \code{specs} as \code{~ 1} or \code{"1"}.
#' 
#' A number of standard contrast families are provided. They can be identified 
#' as functions having names ending in \code{.emmc} -- see the documentation
#' for \code{\link{emmc-functions}} for details -- including how to write your
#' own \code{.emmc} function for custom contrasts.
#' 
#' @section Weights:
#' If \code{weights} is a vector, its length must equal
#'   the number of predictions to be averaged to obtain each EMM.
#'   If a matrix, each row of the matrix is used in turn, wrapping back to the
#'   first row as needed.  When in doubt about what is being averaged (or how
#'   many), first call \code{emmeans} with \code{weights = "show.levels"}.
#'   
#' If \code{weights} is a string, it should partially match one of the following:
#' \describe{
#' \item{\code{"equal"}}{Use an equally weighted average.}
#' \item{\code{"proportional"}}{Weight in proportion to the frequencies (in the
#'   original data) of the factor combinations that are averaged over.}
#' \item{\code{"outer"}}{Weight in proportion to each individual factor's
#'   marginal frequencies. Thus, the weights for a combination of factors are the
#'   outer product of the one-factor margins}
#' \item{\code{"cells"}}{Weight according to the frequencies of the cells being
#'   averaged.}
#' \item{\code{"flat"}}{Give equal weight to all cells with data, and ignore
#'   empty cells.}
#' \item{\code{"show.levels"}}{This is a convenience feature for understanding
#'   what is being averaged over. Instead of a table of EMMs, this causes the
#'   function to return a table showing the levels that are averaged over, in the
#'   order that they appear.}
#' }
#' Outer weights are like the 'expected' counts in a chi-square test of
#' independence, and will yield the same results as those obtained by
#' proportional averaging with one factor at a time. All except \code{"cells"}
#' uses the same set of weights for each mean. In a model where the predicted
#' values are the cell means, cell weights will yield the raw averages of the
#' data for the factors involved. Using \code{"flat"} is similar to
#' \code{"cells"}, except nonempty cells are weighted equally and empty cells
#' are ignored.
#' 
#' @section Counterfactuals:
#' Counterfactual reference grids (see the documentation for \code{\link{ref_grid}})
#' contain pairs of imputed and actual factor levels, and are handled in a special way.
#' For starters, the \code{weights} argument is ignored and we always use 
#' \code{"cells"} weights.
#' Our understanding is that if factors \code{A, B} are specified as counterfactuals, 
#' the marginal means for \code{A} should still be the same as if \code{A} were the only 
#' counterfactual. Accordingly, in computing these marginal means, we
#' exclude all cases where \code{B != actual_B}, because if \code{A} were the only
#' counterfactual, \code{B} will stay at its actual level.
#' We also take special pains to "remember" information about actual and 
#' imputed levels of counterfactuals so that appropriate results are obtained when
#' \code{emmeans} is applied to a previous \code{emmeans} result.
#' 
#' 
#' @section Offsets:
#' Unlike in \code{ref_grid}, an offset need not be scalar. If not enough values
#' are supplied, they are cyclically recycled. For a vector of offsets, it is 
#' important to understand that the ordering of results goes with the first 
#' name in \code{specs} varying fastest. If there are any \code{by} factors,
#' those vary slower than all the primary ones, but the first \code{by} variable
#' varies the fastest within that hierarchy. See the examples.
#' 
#' @section Options and \code{...}:
#'   Arguments that could go in \code{options} may instead be included in \code{...},
#'   typically, arguments such as \code{type}, \code{infer}, etc. that in essence
#'   are passed to \code{\link{summary.emmGrid}}. Arguments in both places are 
#'   overridden by the ones in \code{...}.
#'   
#'   There is a danger that \code{...} arguments could partially match those used
#'   by both \code{ref_grid} and \code{update.emmGrid}, creating a conflict.
#'   If these occur, usually they can be resolved by providing complete (or at least 
#'   longer) argument names; or by isolating non-\code{ref_grid} arguments in
#'   \code{options}; or by calling \code{ref_grid} separately and passing the
#'   result as \code{object}. See a not-run example below.
#'   
#'   Also, when \code{specs} is a two-sided formula, or \code{contr} is specified,
#'   there is potential confusion concerning which \code{...} arguments
#'   apply to the means, and which to the contrasts. When such confusion is possible,
#'   we suggest doing things separately 
#'   (a call to \code{emmeans} with no contrasts, followed by a call to 
#'   \code{\link{contrast}}). We treat
#'   \code{adjust} as a special case: it is applied to the \code{emmeans} results 
#'   \emph{only} if there are
#'   no contrasts specified, otherwise it is passed only to \code{contrast}.

#'
#' @export
#' 
#' @seealso \code{\link{ref_grid}}, \code{\link{contrast}}, 
#' \href{../doc/models.html}{vignette("models", "emmeans")}
#'
#' @examples
#' warp.lm <- lm(breaks ~ wool * tension, data = warpbreaks)
#' emmeans (warp.lm,  ~ wool | tension)
#' # or equivalently emmeans(warp.lm, "wool", by = "tension")
#' 
#' # 'adjust' argument ignored in emmeans, passed to contrast part...
#' emmeans (warp.lm, poly ~ tension | wool, adjust = "sidak")
#' 
#' # 'adjust' argument NOT ignored ...
#' emmeans (warp.lm, ~ tension | wool, adjust = "sidak")
#' 
#' # Get all sets of EMMs for this model
#' ( allsets <- emmeans(warp.lm, ".") )
#' contrast(allsets, "eff")    # all effects
#' 
#' 
#' \dontrun{
#'   ### Offsets: Consider a silly example:
#'   emmeans(warp.lm, ~ tension | wool, offset = c(17, 23, 47)) @ grid
#'   # note that offsets are recycled so that each level of tension receives
#'   # the same offset for each wool.
#'   # But using the same offsets with ~ wool | tension will probably not
#'   # be what you want because the ordering of combinations is different.
#' }
emmeans = function(object, specs, by = NULL, 
                   fac.reduce = function(coefs) apply(coefs, 2, mean), 
                   contr, options = get_emm_option("emmeans"), 
                   weights, offset, ..., tran) {
    
    object = .chk.list(object, ...)
    if(!is(object, "emmGrid")) {
        args = .zap.args(object = object, ..., omit = "submodel")
        if (is.null(args$wt.nuis)) # pass weights as wt.nuis
            args$wt.nuis = ifelse(!missing(weights) && is.character(weights), weights, "equal")
        object = do.call(ref_grid, args)
    }
    
    # Check to see if we want all sets of means
    specs = .parse.specs.for.all(object, specs, by)
    if(!is.null(rtn <- attr(specs, "form.rtn"))) {
        if(length(rtn$by) > 0)
            by = rtn$by
        if(length(rtn$lhs) > 0) 
            contr = rtn$lhs
    }
    
    if (is.list(specs)) {
        return (emmeans.list(object, specs, by = by, 
                             contr = contr, weights = weights, 
                             offset = offset, ...))
    }
    if (inherits(specs, "formula")) {
        spc = .parse.by.formula(specs)
        specs = spc$rhs
        if (length(spc$by) > 0) 
            by = setdiff(union(spc$by, by), spc$rhs)
        if (length(spc$lhs) > 0) 
            contr = spc$lhs
    }
    
    if (!missing(tran)) {
        options $tran = tran
    }
    
    if(is.null(nesting <- object@model.info$nesting)) 
        {
        RG = object
        facs = union(specs, by)
        
        # Check that grid is complete
        # This isn't a 100% reliable check, but...
        if(nrow(RG@grid) != prod(sapply(RG@levels, length)))
            stop("Irregular reference grid: Marginal means cannot be determined.\n",
                 "You can possibly fix this with the 'force_regular' function.")
        
        if (!is.null(RG@misc$display)) {
            RG@misc$display = NULL
            warning("emmeans() results may be corrupted by removal of a nesting structure")
        }
        
        ## Handle counterfactuals...
        # cf.grid is either NULL, logical, or a "parent" grid that should replace RG
        if(!is.null(cf.grid <- RG@misc$cf.grid)) {
            weights = "cells"  # always use cells weights with counterfactuals
            if (inherits(cf.grid, "emmGrid"))
                RG = cf.grid
            # which counterfactuals do we average over?
            cf.ao = intersect(setdiff(names(RG@levels), facs), RG@roles$counterfactuals)
            # zero-out weights for any cases where cf.ao levels differ from actual levels
            for(f in cf.ao) {
                fc = paste0("actual_", f)
                excl = (RG@grid[ , f] != RG@grid[, fc])
                RG@grid[excl, ".wgt."] = 0
            }
            # Fix up the returned grid to play along
            cf.surv = intersect(facs, RG@roles$counterfactuals)
            if(length(cf.surv) == 0) # no cfs remain
                RG@misc$cf.grid = RG@roles$counterfactuals = NULL
            else {
                if(length(intersect(facs, paste0("actual_", cf.surv))) == length(cf.surv))
                    RG@misc$cf.grid = TRUE
                else
                    RG@misc$cf.grid = RG   # save this as "parent" grid
            }
        }
        
        # Ensure object is in standard order
        ord = .std.order(RG@grid, RG@levels) ###do.call(order, unname(RG@grid[rev(names(RG@levels))]))
        if(any(ord != seq_along(ord)))
            RG = RG[ord]
        
        # xxx if ((length(facs) == 1) && (facs == "1")) {  ### just want grand mean
        if("1" %in% facs) {
            RG@levels[["1"]] = "overall"
            RG@grid[ ,"1"] = 1
        }

        
        # Figure out the structure of the grid
        wgt = RG@grid[[".wgt."]]
        if(!is.null(wgt) && all(zapsmall(wgt) == 0)) wgt = wgt + 1 ### repl all zero wgts with 1
        dims = sapply(RG@levels, length)
        row.idx = array(seq_len(nrow(RG@linfct)), dims)
        use.mars = match(facs, names(RG@levels)) # which margins to use
        avgd.mars = setdiff(seq_along(dims)[dims>1], use.mars) # margins that we average over
        
        # Reconcile weights, if there are any margins left
        if ((length(avgd.mars) > 0) && !missing(weights)) {
            if (is.character(weights)) {
                if (is.null(wgt))
                    warning("'weights' requested but no weighting information is available")
                else {
                    wopts = c("equal","proportional","outer","cells","flat","show.levels","invalid")
                    weights = switch(wopts[pmatch(weights, wopts, 7)],
                                     equal = rep(1, prod(dims[avgd.mars])),
                                     proportional = as.numeric(apply(row.idx, avgd.mars,
                                                                           function(idx) sum(wgt[idx]))),
                                     outer = {
                                         ftbl = apply(row.idx, avgd.mars,
                                                            function(idx) sum(wgt[idx]))
                                         # Fix up the dimensions
                                         ftbl = array(ftbl, dim(row.idx)[avgd.mars])
                                         w = N = sum(ftbl)
                                         for (d in seq_along(dim(ftbl)))
                                             w = outer(w, apply(ftbl, d, sum) / N)
                                         as.numeric(w)
                                     },
                                     cells = "fq",
                                     flat = "fl",
                                     show.levels = {
                                         cat("emmeans are obtained by averaging over these factor combinations\n")
                                         return(do.call(expand.grid, RG@levels[avgd.mars]))
                                     },
                                     invalid = stop("Invalid 'weights' option: '", weights, "'")
                    )
                }
            }
            if (is.matrix(weights)) {
                wtrow = 0
                fac.reduce = function(coefs) {
                    wtvec = weights[wtrow+1, ] / sum(weights[wtrow+1, ])
                    ans = apply(sweep(coefs, 1, wtvec, "*"), 2, sum)
                    wtrow <<- (1 + wtrow) %% nrow(weights)
                    ans
                }
            }
            else if (is.numeric(weights)) {
                wtsum = sum(weights)
                if (wtsum <= 1e-8) wtsum = NA
                fac.reduce = function(coefs) {
                    if (nrow(coefs) != length(weights))
                        stop("Nonconforming number of weights -- need ", nrow(coefs))
                    apply(sweep(coefs, 1, weights, "*"), 2, sum) / wtsum
                }
            }
        }
        
        # Get the required factor combs
        levs = list()
        for (f in facs) {
            levs[[f]] = RG@levels[[f]]
            if (!hasName(levs, f))
                stop(paste("No variable named", f, "in the reference grid"))
        }
        combs = do.call("expand.grid", levs)
        if (!missing(weights) && is.character(weights) && (weights %in% c("fq", "fl")))
            K = apply(row.idx, use.mars, function(idx) {
                fq = RG@grid[[".wgt."]][idx]
                if (weights == "fl")
                    fq = 0 + (fq > 0)  # fq = 1 if > 0, else 0
                apply(sweep(RG@linfct[idx, , drop=FALSE], 1, fq, "*"), 2, sum) / sum(fq)
            })
        else
            K = apply(row.idx, use.mars, function(idx) {
                fac.reduce(RG@linfct[idx, , drop=FALSE])
            })
        
        linfct = t(matrix(K, nrow = ncol(RG@linfct),
                          dimnames = list(colnames(RG@linfct), NULL)))

        if(.some.term.contains(union(facs, RG@roles$trend), RG@model.info$terms))
            if(get_emm_option("msg.interaction"))
                message("NOTE: Results may be misleading due ", 
                        "to involvement in interactions")
        
        # Figure offset, if any
        if (hasName(RG@grid, ".offset.")) {
            combs[[".offset."]] = as.numeric(apply(row.idx, use.mars, function(idx)
                fac.reduce(as.matrix(RG@grid[idx, ".offset.", drop = FALSE]))))
        }
        
        avgd.over = names(RG@levels[avgd.mars])
        
        # add/override .offset. column if requested
        if(!missing(offset)) {
            combs[[".offset."]] = rep(offset, nrow(combs))[seq_len(nrow(combs))]
            
        }
        
        # Update .wgt column of grid, if it exists
        if (!is.null(wgt)) {
            combs[[".wgt."]] = as.numeric(apply(row.idx, use.mars, 
                                                      function(idx) sum(wgt[idx])))
        }
        
        RG@roles$responses = character()
        RG@misc$is.new.rg = NULL
        RG@misc$famSize = nrow(linfct)
        if(RG@misc$estName == "prediction") 
            RG@misc$estName = "emmean"
        RG@misc$adjust = "none"
        RG@misc$infer = c(TRUE,FALSE)
        RG@misc$pri.vars = setdiff(facs, by)
        RG@misc$by.vars = by
        RG@misc$avgd.over = union(RG@misc$avgd.over, avgd.over)
        RG@misc$methDesc = "emmeans"
        RG@roles$predictors = setdiff(names(levs), RG@roles$multresp)
        # if ((length(RG@roles$multresp) > 0) && !(RG@roles$multresp %in% names(levs))) 
        #     RG@roles$multresp = character(0)
        # REPLACED BY:
        RG@roles$multresp = intersect(RG@roles$multresp, names(levs))
        
        result = as.emmGrid(RG)
        result@linfct = linfct
        result@levels = levs
        result@grid = combs
        
        result = if (missing(contr))
            .update.options(result, options, ...)
        else
            .update.options(result, options, ..., exclude.opts = "adjust")
    }
    
    else {  # handle a nested structure
        object@model.info$nesting = NULL
        result = .nested_emm(object, specs, by = by, fac.reduce = fac.reduce, 
                       options = options, weights = weights, offset = offset, nesting = nesting)
        if(!is.null(type <- list(...)$type))
            result = update(result, type = type)
    }
    

    if(!missing(contr)) { # return a list with emmeans and contrasts
        # # skip doing contrasts when we have no primary factor
        # if(length(result@misc$pri.vars) == 0) 
        #     return(result)
        warn.contr = interactive() && (sys.parent() == 0) && is.character(contr) && 
            (length(result@misc$pri.vars) > 1)
        # NULL-out a bunch of arguments to not pass. 
        dontpass = c("data", "avgd.over", "by.vars", "df", "initMesg", "estName", "estType",
                     "famSize", "inv.lbl", "methDesc", "nesting", "pri.vars", 
                     "tran", "tran.mult", "tran.offset", "tran2", "is.new.rg")
        args = .zap.args(object = result, method = contr, by = by, ..., omit = dontpass)
        ctrs = do.call(contrast, args)
        result = .cls.list("emm_list", emmeans = result, contrasts = ctrs)
        if(!is.null(lbl <- object@misc$methDesc))
            names(result)[1] = lbl
        
        if (warn.contr && (nrow(result$contr@grid) > 21))
            warning("You may have generated more contrasts than you really wanted. In the future,\n",
                    "we suggest you avoid things like 'pairwise ~ fac1*fac2' when you have\n",
                    "more than one factor. Instead, call emmeans() with just '~ fac1*fac2' and do the\n",
                    "contrasts you need in a later step. See vignette(\"QuickStart\", \"emmeans\").\n",
                     call. = FALSE)
    }

    result
}



# Construct a new emmGrid object with given arguments

#' Construct an \code{emmGrid} object from scratch
#' 
#' This allows the user to incorporate results obtained by some analysis
#' into an \code{emmGrid} object, enabling the use of \code{emmGrid} methods
#' to perform related follow-up analyses.
#' 
#'  The arguments must be conformable. This includes that the length of
#'  \code{bhat}, the number of columns of \code{linfct}, and the number of
#'  columns of \code{post.beta} must all be equal. And that the product of
#'  lengths in \code{levels} must be equal to the number of rows of
#'  \code{linfct}. The \code{grid} slot of the returned object is generated 
#'  by \code{\link{expand.grid}} using \code{levels} as its arguments. So the
#'  rows of \code{linfct} should be in corresponding order.
#'  
#' The functions \code{qdrg} and \code{\link{emmobj}} are close cousins, in that
#' they both produce \code{emmGrid} objects. When starting with summary
#' statistics for an existing grid, \code{emmobj} is more useful, while
#' \code{qdrg} is more useful when starting from an unsupported fitted model.
#'
#'
#' @param bhat Numeric. Vector of regression coefficients
#' @param V Square matrix. Covariance matrix of \code{bhat}
#' @param levels Named list or vector. Levels of factor(s) that define the
#'   estimates defined by \code{linfct}. If not a list, we assume one factor
#'   named \code{"level"}
#' @param linfct Matrix. Linear functions of \code{bhat} for each combination 
#'   of \code{levels}. 
#' @param df Numeric value or function with arguments \code{(x, dfargs)}. If a
#'   number, that is used for the degrees of freedom. If a function, it should
#'   return the degrees of freedom for \code{sum(x*bhat)}, with any additional
#'   parameters in \code{dfargs}.
#' @param dffun Overrides \code{df} if specified. This is a convenience
#'   to match the slot names of the returned object.
#' @param dfargs List containing arguments for \code{df}.
#'   This is ignored if df is numeric.
#' @param post.beta Matrix whose columns comprise a sample from the posterior
#'   distribution of the regression coefficients (so that typically, the column
#'   averages will be \code{bhat}). A 1 x 1 matrix of \code{NA} indicates that
#'   such a sample is unavailable.
#' @param nesting Nesting specification as in \code{\link{ref_grid}}. This is
#'   ignored if \code{model.info} is supplied.
#' @param se.bhat,se.diff Alternative way of specifying \code{V}. If \code{se.bhat}
#'   is specified, \code{V} is constructed using \code{se.bhat}, the standard errors of \code{bhat},
#'   and \code{se.diffs}, the standard errors of its pairwise differences. \code{se.diff}
#'   should be a vector of length \code{k*(k-1)/2} where \code{k} is the length
#'   of \code{se.bhat}, and its elements should be in the order \code{12,13,...,1k,23,...2k,...}.
#'   If \code{se.diff} is missing, \code{V} is computed as if the \code{bhat} are independent.
#' @param ... Arguments passed to \code{\link{update.emmGrid}}
#' 
#' @seealso \code{\link{qdrg}}, an alternative that is useful when starting 
#'   with a fitted model not supported in \pkg{emmeans}.
#'
#' @return An \code{emmGrid} object
#' @export
#'
#' @examples
#' # Given summary statistics for 4 cells in a 2 x 2 layout, obtain 
#' # marginal means and comparisons thereof. Assume heteroscedasticity
#' # and use the Satterthwaite method
#' levels <- list(trt = c("A", "B"), dose = c("high", "low"))
#' ybar <- c(57.6, 43.2, 88.9, 69.8)
#' s <-    c(12.1, 19.5, 22.8, 43.2)
#' n <-    c(44,   11,   37,   24)
#' se2 = s^2 / n
#' Satt.df <- function(x, dfargs)
#'     sum(x * dfargs$v)^2 / sum((x * dfargs$v)^2 / (dfargs$n - 1))
#'     
#' expt.emm <- emmobj(bhat = ybar, V = diag(se2),
#'     levels = levels, linfct = diag(c(1, 1, 1, 1)),
#'     df = Satt.df, dfargs = list(v = se2, n = n), estName = "mean")
#' 
#' ( trt.emm <- emmeans(expt.emm, "trt") )
#' ( dose.emm <- emmeans(expt.emm, "dose") )
#' 
#' rbind(pairs(trt.emm), pairs(dose.emm), adjust = "mvt")
#' 
#' ### Create an emmobj from means and SEs
#' ### (This illustration reproduces the MOats example for Variety = "Victory")
#' means = c(71.50000,  89.66667, 110.83333, 118.50000)
#' semeans = c(5.540591, 6.602048, 8.695358, 7.303221)
#' sediffs = c(7.310571,  9.894724,  7.463615, 10.248306,  4.935698,  8.694507)
#' foo = emmobj(bhat = means, se.bhat = semeans, se.diff = sediffs, 
#'              levels = list(nitro = seq(0, .6, by = .2)), df = 10)
#' plot(foo, comparisons = TRUE)
#' 
emmobj = function(bhat, V, levels, linfct = diag(length(bhat)), df = NA, dffun, dfargs = list(), 
                  post.beta = matrix(NA), nesting = NULL, 
                  se.bhat, se.diff, ...) {
    if (!missing(se.bhat)) {
        V = D = diag(vb <- se.bhat^2)
        if (!missing(se.diff)) {
            V[lower.tri(V)] = se.diff^2
            V = (-V - t(V) + sweep(D, 1, vb, "+") + + sweep(D, 2, vb, "+"))/2
            diag(V) = vb
        }
    }
    if ((nrow(V) != ncol(V)) || (nrow(V) != ncol(linfct)) || (length(bhat) != ncol(linfct)))
        stop("bhat, V, and linfct are incompatible")
    if (!is.list(levels))
        levels = list(level = levels)
    grid = do.call(expand.grid, levels)
    if (nrow(grid) != nrow(linfct))
        stop("linfct should have ", nrow(grid), "rows")
    pri.vars = names(grid)
    dotargs = list(...)
    model.info = dotargs$model.info
    if(is.null(model.info))
        model.info = list(call = str2lang("emmobj"), xlev = levels, 
                          nesting = .parse_nest(nesting))
    roles = list(predictors= names(grid), responses=character(0), 
                 multresp=character(0))
    for (nm in names(dotargs$extras))
        grid[[nm]] = dotargs$extras[[nm]]
    if (!missing(dffun))
        df = dffun
    if (is.function(df)) {
        dffun = df
    } 
    else {
        dffun = function(x, dfargs) dfargs$df
        dfargs = list(df = df)
    }
    ### let's not be so brute-force about what's needed in misc
    # misc = list(estName = "estimate", estType = "prediction", infer = c(TRUE,FALSE), level = .95,
    #             adjust = "none", famSize = nrow(linfct), 
    #             avgd.over = character(0), pri.vars = pri.vars,
    #             methDesc = "emmobj", display = dotargs$display, .pairby = dotargs$.pairby)
    dflt = list(estName = "estimate", estType = "prediction", infer = c(TRUE,FALSE), level = .95,
                adjust = "none", famSize = nrow(linfct),
                avgd.over = character(0), pri.vars = pri.vars,
                methDesc = "emmobj")
    misc = dflt[setdiff(names(dflt), names(dotargs))]  # use defaults when not specified
    nm = setdiff(names(dotargs), c(names(misc), "extras", "model.info")) # otherwise use dots
    misc = c(misc, dotargs[nm])
    
    result = new("emmGrid", model.info=model.info, roles=roles, grid=grid,
                 levels = levels, matlevs=list(),
                 linfct=linfct, bhat=bhat, nbasis=all.estble, V=V,
                 dffun=dffun, dfargs=dfargs, misc=misc, post.beta=post.beta)
    dotargs$model.info = dotargs$extras = dotargs$display = NULL
    do.call(update, c(object = result, dotargs, silent = TRUE))
}

#' Convert to and from \code{emmGrid} objects
#' 
#' These are useful utility functions for creating a compact version of an
#' \code{emmGrid} object that may be saved and later reconstructed, or for
#' converting old \code{ref.grid} or \code{lsmobj} objects into \code{emmGrid}
#' objects.
#' 
#' An \code{emmGrid} object is an S4 object, and as such cannot be saved in a
#' text format or saved without a lot of overhead. By using \code{as.list},
#' the essential parts of the object are converted to a list format that can be
#' easily and compactly saved for use, say, in another session or by another user.
#' Providing this list as the arguments for \code{\link{emmobj}} allows the user 
#' to restore a working \code{emmGrid} object.
#' 
#' @param object Object to be converted to class \code{emmGrid}. It may
#'   be a \code{list} returned by \code{as.list.emmGrid}, or a \code{ref.grid}
#'   or \code{lsmobj} object created by \pkg{emmeans}'s predecessor, the 
#'   \pkg{lsmeans} package. An error is thrown if \code{object} cannot
#'   be converted.
#' @param ... In \code{as.emmGrid}, additional arguments passed to 
#'   \code{\link{update.emmGrid}} before returning the object. This
#'   argument is ignored in \code{as.list.emmGrid}
#'   
#' @return \code{as.emmGrid} returns an object of class \code{emmGrid}. 
#'     However, in fact, both \code{as.emmGrid} and \code{as.emm_list} check for an
#'     attribute in \code{object} to decide whether to return an \code{emmGrid} 
#'     or \code{emm_list)} object.
#' 
#' @seealso \code{\link{emmobj}}
#' @export
#' 
#' @examples
#' pigs.lm <- lm(log(conc) ~ source + factor(percent), data = pigs)
#' pigs.sav <- as.list(ref_grid(pigs.lm))
#' 
#' pigs.anew <- as.emmGrid(pigs.sav)
#' emmeans(pigs.anew, "source")
as.emmGrid = function(object, ...) {
    if (cls <- class(object)[1] %in% c("ref.grid", "lsmobj")) {
        object = as.list.emmGrid(object)
        if (is.null(object$misc$is.new.rg))
            object$misc$is.new.rg = (cls == "ref.grid")
    }
    # above keeps us from having to define these classes in emmeans
    if (is.list(object)) {
        if (!is.null(attr(object, "emm_list")))
            return(as.emm_list(object))
        else
            result = do.call(emmobj, object)
    }
    else {
        result = try(as(object, "emmGrid", strict = FALSE), silent = TRUE)
        if (inherits(result, "try-error"))
            stop("Object cannot be coerced to class 'emmGrid'")
    }
    update(result, ...)
}

#' @rdname as.emmGrid
#' @order 2
#' @param x An \code{emmGrid} object
#' @param model.info.slot Logical value: Include the \code{model.info} slot?
#'   Set this to \code{TRUE} if you want to preserve the original call and 
#'   information needed by the \code{submodel} option.
#'   If \code{FALSE}, only the nesting information (if any) is saved
#' @return \code{as.list.emmGrid} returns an object of class \code{list}.
#' @method as.list emmGrid
#' @export
as.list.emmGrid = function(x, model.info.slot = FALSE, ...) {
    slots = c("bhat", "V", "levels", "linfct", "dffun", "dfargs", "post.beta")
    result = lapply(slots,function(nm) slot(x, nm))
    names(result) = slots
    result = c(result, x@misc)
    if(model.info.slot)
        result$model.info = x@model.info
    else
        result$nesting = x@model.info$nesting
    nm = intersect(names(x@grid), c(".wgt.", ".offset."))
    if (length(nm) > 0)
        result$extras = x@grid[nm]
    result$pri.vars = NULL
    result
}



#### --- internal stuff used only by emmeans -------------

# Check if model contains a term containing all elts of facs
# terms is terms() component of model
.some.term.contains = function(facs, terms) {
    for (trm in attr(terms, "term.labels")) {
        if(all(sapply(facs, function(f) length(grep(f,trm))>0)))
            if (length(.all.vars(as.formula(paste("~",trm)))) > length(facs)) 
                return(TRUE)
    }
    return(FALSE)
}

### Sort grid in standard order according to ordering of entries in levels. 
### Thus .std.order(do.call(expand.grid, levels), levels) --> 1,2,...,nrow
.std.order = function(grid, levels) {
    tmp = lapply(rev(names(levels)), function(nm) {
        x = grid[[nm]]
        if (inherits(x, "factor")) as.integer(x)
        else as.integer(factor(x, levels = as.character(levels[[nm]])))
        # Note: need as.character(levels) here so we handle such as Date vectors correctly
    })
    do.call(order, tmp)
}

context("cross-group P-value adjustment")

# cross.adjust is documented to be ignored unless all 'by' groups are the
# same size. Previously the guard only checked the NUMBER of by groups, so
# unequal sizes caused matrix() recycling and silently wrong adjusted P values.
test_that("cross.adjust is ignored for unequal-size by groups", {
    warp.lm <- lm(breaks ~ wool * tension, data = warpbreaks)
    prs <- pairs(emmeans(warp.lm, ~ tension | wool))
    uneq <- prs[-1]   # wool A: 2 comparisons, wool B: 3 -> unequal sizes

    base <- suppressMessages(
        test(uneq, by = "wool", adjust = "tukey", cross.adjust = "none"))

    # must not warn (no recycling) ...
    expect_warning(
        cross <- suppressMessages(
            test(uneq, by = "wool", adjust = "tukey", cross.adjust = "bonferroni")),
        regexp = NA)
    # ... and must equal the 'none' result (cross.adjust ignored as documented)
    expect_equal(cross$p.value, base$p.value)
})

# Positive control: with EQUAL-size by groups, cross.adjust must still be
# applied (this is the documented, working case - the fix must not disable it).
test_that("cross.adjust is still applied for equal-size by groups", {
    warp.lm <- lm(breaks ~ wool * tension, data = warpbreaks)
    eq <- pairs(emmeans(warp.lm, ~ tension | wool))   # wool A: 3, wool B: 3

    base  <- suppressMessages(
        test(eq, by = "wool", adjust = "tukey", cross.adjust = "none"))
    cross <- suppressMessages(
        test(eq, by = "wool", adjust = "tukey", cross.adjust = "bonferroni"))

    # cross.adjust must change (inflate) the P values across the two by groups
    expect_false(isTRUE(all.equal(cross$p.value, base$p.value)))
    expect_true(all(cross$p.value >= base$p.value - 1e-12))
})

##############################################################################
#    Copyright (c) 2012-2021 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

#' Multivariate contrasts
#' 
#' This function displays tests of multivariate comparisons or contrasts.
#' The contrasts are constructed at each level of the variable in \code{mult.name},
#' and then we do a multivariate test that the vector of estimates is equal to
#' \code{null} (zero by default). The \emph{F} statistic and degrees
#' of freedom are determined via the Hotelling distribution. that is, if there are
#' \eqn{m} error degrees of freedom and multivariate dimensionality \eqn{d}, then
#' the resulting \eqn{F} statistic has degrees of freedom \eqn{(d, m - d + 1)}
#' as shown in Hotelling (1931).
#' 
#' 
#' @param object An object of class \code{emmGrid}
#' @param method A contrast method, per \code{\link{contrast.emmGrid}}
#' @param mult.name Character vector of names of the factors whose levels
#'   define the multivariate means to contrast. If the model itself has a
#'   multivariate response, that is what is used. Otherwise, \code{mult.name}
#'   \emph{must} be specified.
#' @param null Scalar or conformable vector of null-hypothesis values to test against
#' @param by Any \code{by} variable(s). These should not include the primary
#'   variables to be contrasted. For convenience, the \code{by} variable is
#'   nulled-out if it would result in no primary factors being contrasted.
#' @param adjust Character value of a multiplicity adjustment method
#'   (\code{"none"} for no adjustment). The available adjustment methods are
#'   more limited that in \code{contrast}, and any default adjustment returned
#'   via \code{method} is ignored.
#' @param show.ests Logical flag determining whether the multivariate means 
#'   are displayed
#' @param ... Additional arguments passed to \code{contrast}
#'
#' @return An object of class \code{summary_emm} containing the multivariate
#'   test results; or a list of the estimates and the tests if \code{show.ests}
#'   is \code{TRUE}. The test results include the Hotelling \eqn{T^2} statistic,
#'   \eqn{F} ratios, degrees of freedom, and \eqn{P} values.
#' @note
#' If some interactions among the primary and \code{mult.name} factors are
#' absent, the covariance of the multivariate means is singular; this situation
#' is accommodated, but the result has reduced degrees of freedom and a message
#' is displayed. If there are other abnormal conditions such as non-estimable
#' results, estimates are shown as \code{NA}.
#'
#' While designed primarily for testing contrasts, multivariate tests of the
#' mean vector itself can be implemented via \code{method = "identity")} (see
#' the examples).
#'
#' @references Hotelling, Harold (1931) "The generalization of Student's ratio", 
#'   \emph{Annals of Mathematical Statistics} 2(3), 360–378. doi:10.1214/aoms/1177732979
#' 
#' @export
#'
#' @examples
#' MOats.lm <- lm(yield ~ Variety + Block, data = MOats)
#' MOats.emm <- emmeans(MOats.lm, ~ Variety | rep.meas)
#' mvcontrast(MOats.emm, "consec", show.ests = TRUE)  # mult.name defaults to rep.meas
#' 
#' # Test each mean against a specified null vector
#' mvcontrast(MOats.emm, "identity", name = "Variety", 
#'            null = c(80, 100, 120, 140), adjust = "none")
#' # (Note 'name' is passed to contrast() and overrides default name "contrast")
#' 
#' # 'mult.name' need not refer to a multivariate response
#' mvcontrast(MOats.emm, "trt.vs.ctrl1", mult.name = "Variety")
#' 
mvcontrast = function(object, method = "eff", mult.name = object@roles$multresp, null = 0,
                      by = object@misc$by.vars, adjust = c("sidak", p.adjust.methods),
                      show.ests = FALSE, ...) {
    object = .chk.list(object, ...)
    if (is.null(mult.name) || length(mult.name) == 0)
        stop("Must specify at least one factor in 'mult.name'")
    if(length(setdiff(names(object@levels), union(by, mult.name))) == 0)
        by = NULL   # avoid the case where we're left with no variables
    con = contrast(object, method = method, by = union(by, mult.name), ...)
    mvnm = paste(mult.name, collapse = " ")
    con = contrast(con, "identity", simple = mult.name, name = mvnm) # just re-orders it
    ese = .est.se.df(con)
    est = ese$est
    df = ese$df
    V = vcov(con)
    rows = .find.by.rows(con@grid, con@misc$by.vars)
    red.rank = FALSE  # flag for red.rank cases
    result = lapply(rows, function(r) {
        QR = try(qr(V[r, r, drop = FALSE]), silent = TRUE)
        if (inherits(QR, "try-error"))
            T2 = df1 = df2 = F = NA
        else {
            df1 = QR$rank
            if(df1 < length(r)) red.rank <<- TRUE
            rawdf = mean(df[r])
            df2 = rawdf - df1 + 1
            qe = qr.coef(QR, est[r] - null)
            qe[is.na(qe)] = 0
            T2 = sum(qe * (est[r] - null))
            F = T2 / df1 * (df2 / rawdf)
        }
        data.frame(T.square = T2, df1 = df1, df2 = df2, F.ratio = F)
    })
    result = cbind(con@grid[sapply(rows, function(r) r[1]), ], do.call(rbind, result))
    result[[mvnm]] = NULL
    class(result) = c("summary_emm", "data.frame")

    by = setdiff(by, mult.name)
    if (length(by) == 0) 
        by = NULL
    rows = .find.by.rows(result, by)
    adjust = match.arg(adjust)
    result$p.value = NA
    for (r in rows) {
        pv = with(result[r, ], pf(F.ratio, df1, df2, lower.tail = FALSE))
        result$p.value[r] = switch(adjust, sidak = 1 - (1 - pv)^length(r),
            p.adjust(pv, adjust))
    }
    attr(result, "estName") = "F.ratio"
    attr(result, "by.vars") = by
    if (adjust == "none") 
        mesg = NULL
    else
        mesg = paste("P value adjustment:", adjust)
    if (red.rank)
        mesg = c(mesg, "NOTE: Some or all d.f. are reduced due to singularities")
    if(any(is.na(result$T.square)))
        mesg = c(mesg, 
            "NAs indicate non-estimabile cases or other errors")
    attr(result, "mesg") = mesg
    if (show.ests)
        list(estimates = con, tests = result)
    else
        result
}


# List of multivariate transformations we recognize
mult.trans = c("alr", "apt", "cdt", "clr", "cpt", "idt", "iit", "ilr", "ilt", "ipt", "ult")
log.mult.trans = mult.trans[endsWith(mult.trans, "lr")]  # only the log-ratio-based ones

#' Multivariate regridding
#' 
#' This function is similar to \code{\link{regrid}} except it performs a
#' multivariate transformation. This is useful, for instance, in multivariate
#' models that have a compositional response.
#' 
#' If a multivariate response transformation was used in fitting the model,
#' its name is auto-detected, and in that case we need not specify \code{fcn}
#' as long as its inverse can be found in the namespace of the \pkg{compositions}
#' package. (That package need not be installed unless \code{fcn} is a character 
#' value.) For some such models, auto-detection process throws a
#' warning message, especially if \code{cbind} is also present in the model
#' formula.
#' 
#' Currently, no bias-adjustment option is available.
#'
#' @param object An \code{emmGrid} object
#' @param transform The transformation to use in re-gridding. If \code{"response"},
#'   we apply the inverse of the multivariate transformation in \code{object@misc$tran};
#'   otherwise, we re-grid as if \code{transform} had been applied to the multivariate
#'   response. (Note that this will entail first re-gridding to the response scale
#'   if necessary.)
#' @param mult.name The name of the multivariate factor to be transformed.
#'   If missing, we use \code{object@roles$multresp}, and throw an error message 
#'   if it is \code{NULL} or ambiguous; in that case, the user must repeat the
#'   call with \code{mult.name} specified.
#' @param newname the name to be given to the newly transformed variable
#' @param newlevs levels of the newly created factor (must conform to
#'   the number of columns created by \code{fcn}). If missing, we use the column names
#'   of the newly created variable.
#' @param fcn The multivariate function to apply. If character, we look for it 
#'   in the namespace of the \pkg{compositions} package.
#' @param ... Additional arguments passed to \code{fcn}
#'
#' @return A new \code{emmGrid} object with the newly created factor as its last factor
#' @export
#' 
#' @examples
#' if(requireNamespace("compositions"))
#'     emm_example("mvregrid")
#'     # Use emm_example("mvregrid", list = TRUE) # to see just the code
#' 
mvregrid = function(object, transform = "response", mult.name, newname = mult.name, newlevs, fcn, ...) {
    tran = object@misc$tran
    if((transform != "response") && !is.null(tran)) {    # non-response transformation -- 1st regrid to response scale
        object = mvregrid(object, transform = "response", mult.name = mult.name)
    }
    levels = levels(object)
    if(is.character(tran))
        tran = rev(strsplit(tran, "\\.")[[1]])[1]
    by = object@misc$by.vars
    if (missing(mult.name)) {
        mult.name = object@roles$multresp
        if(is.null(mult.name) || (length(mult.name) != 1))
            stop("No default for 'mult.name' identified; must supply it")
    }
    idx = which(names(levels) == mult.name)
    if(length(idx) == 0) 
        stop("we don't have a factor named '", mult.name, "'")
    levels = c(levels[-idx], levels[idx])

    ord = .std.order(object@grid, levels)
    if(any(ord != seq_along(ord)))
        object = object[ord] |> update(by.vars = by)
    
    yhat = matrix(predict(object), ncol = (p <- length(levels[[mult.name]])))
    if(missing(fcn))
        fcn = ifelse(transform == "response", paste0(tran, "Inv"), transform)
    if((is.character(fcn)) && requireNamespace("compositions")) {
        fcn = try(get(fcn, envir = asNamespace("compositions")), silent = TRUE)
        if(inherits(fcn, "try-error"))
            stop("The 'compositions' package must be installed to proceed")
    }
    
    newy = fcn(yhat, ...)
    k = ncol(newy)
    if(missing(newlevs)) {
        newlevs = colnames(newy)
        if(is.null(newlevs))
            newlevs = seq_len(ncol(newy))
    }
    else if(length(newlevs) != k)
        stop("Nonconforming 'newlevs': Must be of length ", k)
    if(!is.na(object@post.beta)) {
        pb = apply(object@linfct %*% t(object@post.beta), 2, function(y) {
            as.numeric(fcn(matrix(y, ncol = p), ...))
        })
        object@post.beta = t(pb)
    }
    Jac = numDeriv::jacobian(\(x, ...) as.numeric(fcn(x, ...)), yhat, ...)
    object@V = Jac %*% vcov(object) %*% t(Jac)
    object@bhat = as.numeric(newy)
    object@linfct = diag(length(object@bhat))
    names(levels)[length(levels)] = object@roles$multresp = newname
    object@roles$predictors = setdiff(object@roles$predictors, mult.name)
    levels[[newname]] = newlevs
    object@levels = levels
    wgt = object@grid[[".wgt."]]
    g = object@grid = object@grid[seq_len(n <- nrow(newy)), setdiff(names(object@grid), c(mult.name, ".wgt.", ".offset.")), drop = FALSE]
    for (i in seq_len(k - 1)) object@grid = rbind(object@grid, g)
    object@grid[[newname]] = rep(newlevs, each = n)
    if(!is.null(wgt))
        object@grid[[".wgt."]] = rep(wgt[seq_len(n)], k)
    object@misc$tran = object@misc$inv.lbl = object@misc$sigma = object@misc$is.new.rg = NULL
    if(transform != "response")
        object@misc$tran = transform
    object@misc$pri.vars = setdiff(names(object@grid), c(object@misc$by.vars, ".wgt'"))
    object@misc$methDesc = "mvregrid"
    
    object
}

##############################################################################
#    Copyright (c) 2012-2017 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

### S4 class definitions for emmeans package


### emmGrid object - For reference grids, emmeans results, etc.

#' The \code{emmGrid} class
#' 
#' The \code{emmGrid} class encapsulates linear functions of regression
#' parameters, defined over a grid of predictors. This includes reference
#' grids and grids of marginal means thereof (aka estimated marginal means).
#' Objects of class `emmGrid` may be used independently of the underlying model
#' object. Instances are created primarily by \code{\link{ref_grid}} and
#' \code{\link{emmeans}}, and several related functions.
#'   
#' @rdname emmGrid-class 
#' @slot model.info list. Contains the elements \code{call} (the call that
#'   produced the model), \code{terms} (its \code{terms} object), and
#'   \code{xlev} (factor-level information)
#' @slot roles list. Contains at least the elements \code{predictors}, 
#'   \code{responses}, and \code{multresp}. Each is a character vector of names 
#'   of these variables.
#' @slot grid data.frame. Contains the combinations of the variables that define
#'   the reference grid. In addition, there is an auxiliary column named
#'   \code{".wgt."} holding the observed frequencies or weights for each factor
#'   combination (excluding covariates). If the model has one or more
#'   \code{\link{offset}()} calls, there is an another auxiliary column named
#'   \code{".offset."}. Auxiliary columns are not considered part of the
#'   reference grid. (However, any variables included in \code{offset} calls
#'   \emph{are} in the reference grid.)
#' @slot levels list. Each entry is a character vector with the distinct levels
#'   of each variable in the reference grid. Note that \code{grid} is obtained
#'   by applying the function \code{\link{expand.grid}} to this list
#' @slot matlevs list. Like \code{levels} but has the levels of any matrices in
#'   the original dataset. Matrix columns are always concatenated and treated as
#'   a single variable for purposes of the reference grid
#' @slot linfct matrix. Each row consists of the linear function of the
#'   regression coefficients for predicting its corresponding element of the
#'   reference grid. The rows of this matrix go in one-to-one correspondence
#'   with the rows of \code{grid}, and the columns with elements of \code{bhat}.
#' @slot bhat numeric. The regression coefficients. If there is a multivariate
#'   response, the matrix of coefficients is flattened to a single vector, and
#'   \code{linfct} and \code{V} redefined appropriately. Important: \code{bhat}
#'   must \emph{include} any \code{NA} values produced as a result of 
#'   collinearity in the predictors. These are taken care of later in the 
#'   estimability check.
#' @slot nbasis matrix. The basis for the non-estimable functions of the
#'   regression coefficients. Every EMM will correspond to a linear combination
#'   of rows of \code{linfct}, and that result must be orthogonal to all the
#'   columns of \code{nbasis} in order to be estimable. If everything is
#'   estimable, \code{nbasis} should be a 1 x 1 matrix of \code{NA}.
#' @slot V matrix. The symmetric variance-covariance matrix of \code{bhat}
#' @slot dffun function having two arguments. \code{dffun(k, dfargs)} should
#'   return the degrees of freedom for the linear function \code{sum(k*bhat)},
#'   or \code{NA} if unavailable
#' @slot dfargs list. Used to hold any additional information needed by
#'   \code{dffun}.
#' @slot misc list. Additional information used by methods. These include at
#'   least the following: \code{estName} (the label for the estimates of linear
#'   functions), and the default values of \code{infer}, \code{level}, and
#'   \code{adjust} to be used in the \code{\link{summary.emmGrid}} method. Elements in
#'   this slot may be modified if desired using the \code{\link{update.emmGrid}}
#'   method.
#' @slot post.beta matrix. A sample from the posterior distribution of the
#'   regression coefficients, if MCMC methods were used; or a 1 x 1 matrix of
#'   \code{NA} otherwise. When it is non-trivial, the \code{\link{as.mcmc.emmGrid}}
#'   method returns \code{post.beta \%*\% t(linfct)}, which is a sample from the
#'   posterior distribution of the EMMs.
#' 
#' @section Methods:
#'   All methods for these objects are S3 methods except for \code{show}. 
#'   They include \code{\link{[.emmGrid}}, \code{\link{as.glht.emmGrid}},
#'   \code{\link{as.mcmc.emmGrid}}, \code{\link{as.mcmc.list.emmGrid}} (see \pkg{coda}),
#'   \code{\link{cld.emmGrid}} (see \pkg{multcomp}),
#'   \code{\link{coef.emmGrid}}, \code{\link{confint.emmGrid}}, 
#'   \code{\link{contrast.emmGrid}}, \code{\link{pairs.emmGrid}},
#'   \code{\link{plot.emmGrid}}, \code{\link{predict.emmGrid}}, \code{\link{print.emmGrid}},
#'   \code{\link{rbind.emmGrid}}, \code{show.emmGrid}, \code{\link{str.emmGrid}}, 
#'   \code{\link{summary.emmGrid}}, \code{\link{test.emmGrid}}, 
#'   \code{\link{update.emmGrid}}, \code{\link{vcov.emmGrid}}, and 
#'   \code{\link{xtable.emmGrid}}
#' 
#' @export
setClass("emmGrid", slots = c(
    model.info = "list",
    roles = "list",
    grid = "data.frame", 
    levels = "list",
    matlevs = "list",
    linfct = "matrix",
    bhat = "numeric",
    nbasis = "matrix",
    V = "matrix",
    dffun = "function",
    dfargs = "list",
    misc = "list",
    post.beta = "matrix"
))
# Note: misc will hold various extra params,
# including at least the following req'd by the summary method
#   estName: column name for the estimate in the summary ["prediction"]
#   infer: booleans (CIs?, tests?)  [(FALSE,FALSE)]
#   level: default conf level [.95]
#   adjust: default adjust method ["none"]
#   famSize: number of means in family

# NOTE: Old ref.grid and lsmobj classes moved to deprecated.R

##############################################################################
#    Copyright (c) 2012-2016 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

# Support for MCMCglmm class and possibly more MCMC-based models

# Method to create a coda 'mcmc' or 'mcmc.list' object from a ref.grid
# (dots not supported, unfortunately)
# If sep.chains is TRUE and there is more than one chain, an mcmc.list is returned

# NOTE: S3 registration of as.mcmc and as.mcmc.list is done dynamically in zzz.R

#' Support for MCMC-based estimation
#' 
#' When a model is fitted using Markov chain Monte Carlo (MCMC) methods, 
#' its reference grid contains a \code{post.beta} slot. These functions 
#' transform those posterior samples to posterior samples of EMMs or
#' related contrasts. They can then be summarized or plotted using,
#' e.g., functions in the \pkg{coda} package.
#'
#' @rdname mcmc-support
#' @aliases mcmc-support
#' @param x An object of class \code{emmGrid}
#' @param names Logical scalar or vector specifying whether variable names are
#'   appended to levels in the column labels for the \code{as.mcmc} or
#'   \code{as.mcmc.list} result -- e.g., column names of \code{treat A} and
#'   \code{treat B} versus  just \code{A} and \code{B}. When there is more than
#'   one variable involved, the elements of \code{names} are used cyclically.
#' @param sep.chains Logical value. If \code{TRUE}, and there is more than one
#'   MCMC chain available, an \code{\link[coda]{mcmc.list}} object is returned
#'   by \code{as.mcmc}, with separate EMMs posteriors in each chain.
#' @param likelihood Character value or function. If given, simulations are made from 
#'   the corresponding posterior predictive distribution. If not given, we obtain
#'   the posterior distribution of the parameters in \code{object}. See Prediction
#'   section below.
#' @param NE.include Logical value. If \code{TRUE}, non-estimable columns are
#'   kept but returned as columns of \code{NA} values (this may create errors or
#'   warnings in subsequent analyses using, say, \pkg{coda}). If \code{FALSE},
#'   non-estimable columns are dropped, and a warning is issued. (If all are
#'   non-estimable, an error is thrown.)
#' @param ... arguments passed to other methods
#'
#' @return An object of class \code{\link[coda]{mcmc}} or \code{\link[coda]{mcmc.list}}.
#' 
#' @section Details:
#' When the object's \code{post.beta} slot is non-trivial, \code{as.mcmc} will
#' return an \code{\link[coda]{mcmc}} or \code{\link[coda]{mcmc.list}} object
#' that can be summarized or plotted using methods in the \pkg{coda} package.
#' In these functions, \code{post.beta} is transformed by post-multiplying it by
#' \code{t(linfct)}, creating a sample from the posterior distribution of LS
#' means. In \code{as.mcmc}, if \code{sep.chains} is \code{TRUE} and there is in
#' fact more than one chain, an \code{mcmc.list} is returned with each chain's
#' results. The \code{as.mcmc.list} method is guaranteed to return an
#' \code{mcmc.list}, even if it comprises just one chain. 
#' 
#' @section Prediction:
#' When \code{likelihood} is specified, it is used to simulate values from the
#' posterior predictive distribution corresponding to the given likelihood and
#' the posterior distribution of parameter values. Denote the likelihood 
#' function as \eqn{f(y|\theta,\phi)}, where \eqn{y} is a response, \eqn{\theta}
#' is the parameter estimated in \code{object}, and \eqn{\phi} comprises zero or
#' more additional parameters to be specified. If \code{likelihood} is a 
#' function, that function should take as its first argument a vector of 
#' \eqn{\theta} values (each corresponding to one row of \code{object@grid}).
#' Any \eqn{\phi} values should be specified as additional named function
#' arguments, and passed to \code{likelihood} via \code{...}. This function should 
#' simulate values of \eqn{y}.
#' 
#' A few standard likelihoods are available by specifying \code{likelihood} as
#' a character value. They are:
#' \describe{
#'   \item{\code{"normal"}}{The normal distribution with mean \eqn{\theta} and
#'   standard deviation specified by additional argument \code{sigma}}
#'   \item{\code{"binomial"}}{The binomial distribution with success probability 
#'     \eqn{theta}, and number of trials specified by \code{trials}}
#'   \item{\code{"poisson"}}{The Poisson distribution with mean \eqn{theta} 
#'     (no additional parameters)}
#'   \item{\code{"gamma"}}{The gamma distribution with scale parameter \eqn{\theta}
#'     and shape parameter specified by \code{shape}}
#' }
#' 
#' @method as.mcmc emmGrid
#' @export as.mcmc.emmGrid
#' @examples
#' if(requireNamespace("coda")) 
#'     emm_example("as.mcmc-coda")
#'     # Use emm_example("as.mcmc-coda", list = TRUE) # to see just the code
#'     
as.mcmc.emmGrid = function(x, names = TRUE, sep.chains = TRUE, 
                           likelihood, NE.include = FALSE, ...) {
    if (is.na(x@post.beta[1])) {
        stop("No posterior sample -- can't make an 'mcmc' object")
    }
# notes on estimability issues:
# 1. Use @bhat to determine which coefs to use
# 2. @nabasis as in freq models
# 3. @post.beta we will EXCLUDE cols corresp to NAs in @bhat
# See stanreg support for hints/details
    use = which(!is.na(x@bhat))
    est = estimability::is.estble(x@linfct, x@nbasis)
    if (!any(est))
        stop("Aborted -- No estimates in the grid are estimable")
    else if(!all(est) && !NE.include) {
        rows = paste(which(!est), collapse = ", ")
        warning("Cases  ", rows, "  were dropped due to non-estimability", call. = FALSE)
    }
    mat = x@post.beta %*% t(x@linfct[, use, drop = FALSE])
    if (NE.include)
        mat[, !est] = NA
    else {
        mat = mat[, est, drop = FALSE]
        x@grid = x@grid[est, , drop = FALSE]
    }
    if(!is.null(offset <- x@grid[[".offset."]])) {
        n = nrow(mat)
        mat = mat + matrix(rep(offset, each = n), nrow = n)
    }
    if (!missing(likelihood)) {
        if (is.character(likelihood)) {
            likelihood = match.arg(likelihood, c("normal", "binomial", "poisson", "gamma"))
            likelihood = switch(likelihood,
                normal = function(theta, sigma, ...) 
                    rnorm(length(theta), mean = theta, sd = sigma),
                binomial = function(theta, trials, ...) 
                    rbinom(length(theta), size = trials, prob = theta),
                poisson = function(theta, ...)
                    rpois(length(theta), lambda = theta),
                gamma = function(theta, shape, ...)
                    rgamma(length(theta), scale = theta, shape = shape)
                #, stop("There is no predefined likelihood named '", likelihood, "'")
                )
        }
        mat = apply(mat, 2, likelihood, ...)
##! TODO: Add "multinomial" support. This will require a flag to observe
##! the 'by' variable(s), then we get parameter values from the columns
##! corresponding to each 'by' group 
    }
    nm = setdiff(names(x@grid), c(".wgt.",".offset."))
    if (any(names)) {
        names = rep(names, length(nm))
        for (i in seq_along(nm))
            if(names[i]) x@grid[nm[i]] = paste(nm[i], x@grid[[nm[i]]])
    }
    if(is.null(dimnames(mat)))
        dimnames(mat) = list(seq_len(nrow(mat)), seq_len(ncol(mat)))
    dimnames(mat)[[2]] = do.call(paste, c(unname(x@grid[, nm, drop = FALSE]), 
                                          sep = get_emm_option("sep")))
    n.chains = attr(x@post.beta, "n.chains")
    if (!sep.chains || is.null(n.chains) || (n.chains == 1))
        coda::mcmc(mat)
    else {
        n = nrow(mat) / n.chains
        seqn = seq_len(n)
        chains = lapply(seq_len(n.chains), function(i) coda::mcmc(mat[n*(i - 1) + seqn, , drop = FALSE]))
        coda::mcmc.list(chains)
    }
}

#' @method as.mcmc emm_list
#' @export as.mcmc.emm_list
#' @rdname mcmc-support
#' @param which item in the \code{emm_list} to use
as.mcmc.emm_list = function(x, which = 1, ...) {
    as.mcmc.emmGrid(x[[which]], ...)
}


### as.mcmc.list - guaranteed to return a list
#' @method as.mcmc.list emmGrid
#' @export as.mcmc.list.emmGrid
#' @rdname mcmc-support
as.mcmc.list.emmGrid = function(x, names = TRUE, ...) {
    result = as.mcmc.emmGrid(x, names = names, sep.chains = TRUE, ...)
    if(!inherits(result, "mcmc.list"))
        result = coda::mcmc.list(result)
    result
}

#' @method as.mcmc.list emm_list
#' @export as.mcmc.list.emm_list
#' @rdname mcmc-support
as.mcmc.list.emm_list = function(x, which = 1, ...) {
    as.mcmc.list.emmGrid(x[[which]], ...)
}



#' Summarize an emmGrid from a Bayesian model
#' 
#' This function computes point estimates and HPD (or quantile) intervals for each
#' factor combination in \code{object@emmGrid}. While this function
#' may be called independently, it is called automatically by the S3 method
#' \code{\link{summary.emmGrid}} when the object is based on a Bayesian model.
#' (Note: the \code{level} argument, or its default, is passed as \code{prob}).
#'
#' @param object an \code{emmGrid} object having a non-missing \code{post.beta} slot
#' @param prob numeric probability content for HPD intervals (note: when not specified,
#'   the current \code{level} option is used; see \code{\link{emm_options}})
#' @param by factors to use as \code{by} variables
#' @param type prediction type as in \code{\link{summary.emmGrid}}
#' @param point.est function to use to compute the point estimates from the 
#'   posterior sample for each grid point
#' @param ci.method character value matching \code{"HPD"} (default) or \code{"quantile"}
#'   (but actually not case-sensitive).
#'   The default is to use HPD intervals (which are the shortest possible intervals). 
#'   Alternatively, choosing \code{"quantile"} uses the quantiles of the posterior
#'   having equal tail probabilities.
#' @param delta Numeric equivalence threshold (on the linear predictor scale 
#'   regardless of \code{type}).
#'   See the section below on equivalence testing.
#' @param bias.adjust Logical value for whether to adjust for bias in
#'   back-transforming (\code{type = "response"}). This requires a value of 
#'   \code{sigma} to exist in the object or be specified.
#' @param sigma Error SD assumed for bias correction (when 
#'   \code{type = "response"}. If not specified,
#'   \code{object@misc$sigma} is used, and a warning if it is not found or invalid.
#'   \emph{Note:} \code{sigma} may be a vector, as long as it conforms to the 
#'   number of observations in the posterior sample.
#' @param ... required but not used
#'
#' @return an object of class \code{summary_emm}
#' 
#' @section Equivalence testing note:
#'   If \code{delta} is positive, two columns labeled \code{p.equiv} and
#'   \code{odds.eq} are appended to the summary. \code{p.equiv} is the fraction
#'   of posterior estimates having absolute values less than \code{delta}. The
#'   \code{odds.eq} column is just \code{p.equiv} converted to an odds ratio; so
#'   it is the posterior odds of equivalence.
#'   
#'   A high value of \code{p.equiv} is evidence
#'   in favor of equivalence. It can be used to obtain something equivalent
#'   (in spirit) to the frequentist Schuirmann (TOST) procedure, whereby we would
#'   conclude equivalence at significance level \eqn{\alpha} if the \eqn{(1 - 2\alpha)}
#'   confidence interval falls entirely in the interval \eqn{[-\delta, \delta]}.
#'   Similarly in the Bayesian context, an equally strong argument for
#'   equivalence is obtained if \code{p.equiv} exceeds \eqn{1 - 2\alpha}.
#'   
#'   A closely related quantity is the ROPE (region of practical equivalence),
#'   obtainable via \code{bayestestR::rope(object, range = c(-delta, delta))}.
#'   Its value is approximately \code{100 * p.equiv / 0.95} if the default
#'   \code{ci = 0.95} is used. See also \pkg{bayestestR}'s 
#'   \href{https://github.com/easystats/bayestestR/issues/567}{issue #567}.
#'   
#'   Finally, a Bayes factor for equivalence is obtainable by dividing 
#'   \code{odds.eq} by the prior odds of equivalence, assessed or elicited separately.
#'   
#' @note
#' HPD intervals require the \pkg{coda} package to be installed on your system;
#' otherwise an error is thrown. (So one way to sidestep that error is to specify 
#' \code{ci.method = "quantile"}).
#' 
#' @seealso summary.emmGrid
#' 
#' @export
#'
#' @examples
#' if(require("coda")) 
#'     emm_example("hpd.summary-coda")
#'     # Use emm_example("hpd.summary-coda", list = TRUE) # to see just the code
#' 
hpd.summary = function(object, prob, by, type, point.est = median, 
                       ci.method = get_emm_option("post.ci.method"), 
                       delta,
                       bias.adjust = get_emm_option("back.bias.adj"), sigma, 
                       ...) {
    if(!is.null(object@misc$.predFlag))
        stop("Prediction intervals for MCMC models should be done using 'frequentist = TRUE'\n",
             "or using 'as.mcmc(object, ..., likelihood = ...)'")
    
    ci.method = match.arg(tolower(ci.method), choices = c("hpd", "quantile"))
    
    if (ci.method == "HPD")
        .requireNS("coda", "Bayesian summary requires the 'coda' package")
    
    # Steal some init code from summary.emmGrid:
    opt = get_emm_option("summary")
    if(!is.null(opt)) {
        opt$object = object
        object = do.call("update.emmGrid", opt)
    }
    
    misc = object@misc
    use.elts = if (is.null(misc$display))  
                    rep(TRUE, nrow(object@grid)) 
                else                        
                    misc$display
    grid = object@grid[use.elts, , drop = FALSE]
    
    if(missing(prob))
        prob = misc$level
    if(missing(by))
        by = misc$by.vars
    
    if (missing(type))
        type = .get.predict.type(misc)
    else
        type = .validate.type(type)
    # if there are two transformations and we want response, then we need to undo both
    if ((type == "response") && (!is.null(misc$tran2)))
        object = regrid(object, transform = "mu")
    if ((type %in% c("mu", "unlink")) && (!is.null(t2 <- misc$tran2))) {
        if (!is.character(t2))
            t2 = "tran"
        object = update(object, inv.lbl = paste0(t2, "(resp)"))
    }
    
    link = .get.link(misc)
    inv = (type %in% c("response", "mu", "unlink")) # flag to inverse-transform
    if (inv && is.null(link))
        inv = FALSE
    
    
    ### OK, finally, here is the real stuff
    pe.lbl = as.character(substitute(point.est))
    if(length(pe.lbl) > 1) 
        pe.lbl = "user-supplied function"
    mesg = c(misc$initMesg, paste("Point estimate displayed:", pe.lbl))
    if (length(misc$avgd.over) > 0) {
        qual = attr(misc$avgd.over, "qualifier")
        if (is.null(qual)) qual = ""
        mesg = c(paste0("Results are averaged over", qual, " the levels of: ",
                        paste(misc$avgd.over, collapse = ", ")), mesg)
    }

    mcmc = as.mcmc.emmGrid(object, names = FALSE, sep.chains = FALSE, 
                           NE.include = TRUE, ...)
    mcmc = mcmc[, use.elts, drop = FALSE]
    p.equiv = odds.eq = NULL
    if (!missing(delta) && (delta > 0)) {
        p.equiv = apply(mcmc, 2, \(x) mean(abs(x) < delta))
        mesg = c(mesg, paste0("'p.equiv' and 'odds.eq' based on posterior P(|lin. pred.| < ", 
                              round(delta, digits = 4), ")"))
        odds.eq = p.equiv / (1 - p.equiv)
    }
    if (inv) {
        if (bias.adjust) {
            if (missing(sigma))
                sigma = object@misc@sigma
            link = .make.bias.adj.link(link, sigma)
            bias.adjust = attr(link, "bias.adjust")
        }
        
        for (j in seq_along(mcmc[1, ]))
            mcmc[, j] = with(link, linkinv(mcmc[, j]))
        mesg = c(mesg, paste("Results are back-transformed from the", link$name, "scale"))
        if(bias.adjust)
            mesg = c(mesg, paste("Bias adjustment applied based on sigma =",
                                 .fmt.sigma(sigma)))
    }
    else if(!is.null(link))
        mesg = c(mesg, paste("Results are given on the", link$name, "(not the response) scale."))
    
    est = !is.na(mcmc[1, ])
    mcmc[, !est] = 0 # temp so we don't get errors
    pt.est = data.frame(apply(mcmc, 2, point.est))
    names(pt.est) = object@misc$estName
    if (ci.method == "hpd") {
        summ = as.data.frame(coda::HPDinterval(mcmc, prob = prob))[c("lower","upper")]
        names(summ) = cnm = paste0(names(summ), ".HPD")
        mesg = c(mesg, paste("HPD interval probability:", prob))
    }
    else {
        al = (1 - prob)/2
        summ = as.data.frame(t(apply(mcmc, 2, quantile, c(al, 1-al))))
        names(summ) = cnm = c("lower.QL", "upper.QL")
        mesg = c(mesg, paste("quantile interval probability:", prob))
    }
    lblnms = setdiff(names(grid), 
                     c(object@roles$responses, ".offset.", ".wgt."))
    lbls = grid[lblnms]
    if(!is.null(p.equiv)) {
        summ$p.equiv = p.equiv
        summ$odds.eq = odds.eq
    }
    if (inv) {
        if (!is.null(misc$inv.lbl)) {
            names(pt.est) = misc$estName = misc$inv.lbl
            if (!is.null(misc$log.contrast))  # contrast of logs - relabel as ratios
                for (ell in seq_along(lbls)){
                    lbls[[ell]] = factor(lbls[[ell]])
                    levels(lbls[[ell]]) = gsub(" - ", " / ", levels(lbls[[ell]]))
                }
        }
        else
            names(pt.est) = misc$estName = "response"
    }
    
    summ[!est, ] = NA
    pt.est[!est, ] = NA
    summ = cbind(lbls, pt.est, summ)
    attr(summ, "estName") = misc$estName
    attr(summ, "clNames") = cnm
    if (is.null(misc$pri.vars) || length(misc$pri.vars) == 0)
        misc$pri.vars = names(object@levels)
    attr(summ, "pri.vars") = setdiff(union(misc$pri.vars, misc$by.vars), by)
    attr(summ, "by.vars") = by
    attr(summ, "mesg") = unique(mesg)
    class(summ) = c("summary_emm", "data.frame")
    summ
}




# Currently, data is required, as call is not stored
#' @exportS3Method recover_data MCMCglmm
recover_data.MCMCglmm = function(object, data, trait, ...) {
    if (is.null(data) && !is.null(object$data)) # allow for including data in object
        data = eval(object$data)
    # if a multivariate response, stack the data with `trait` variable
    yvars = .all.vars(update(object$Fixed$formula, ". ~ 1"))
    if ("trait" %in% names(data)) {
        # don't do anything, just use what's provided
    }
    else if(length(yvars) > 1) {
#        for (v in yvars) data[[v]] = NULL
        dat = data
        for (i in seq_len(length(yvars) - 1))
            data = rbind(data, dat)
        data$trait = factor(rep(yvars, each = nrow(dat)))
    }
    else if(!missing(trait)) {
        # we'll create a fake "trait" variable with specified variable
        n = nrow(data)
        levs = levels(data[[trait]])
        attr(data, "misc") = list(resp.levs = levs, trait = trait)
        data$trait = rep(levs[-1], n)[1:n] # way overkill, but easy coding
    }
    attr(data, "call") = object$Fixed
    attr(data, "terms") = trms = delete.response(terms(object$Fixed$formula))
    attr(data, "predictors") = .all.vars(delete.response(trms))
    data
}

# misc may be NULL or a list generated by trait spec
#' @exportS3Method emm_basis MCMCglmm
emm_basis.MCMCglmm = function(object, trms, xlev, grid, vcov., 
                              mode = c("default", "multinomial"), misc, ...) {
    nobs.MCMCglmm = function(object, ...) 1   # prevents warning about nobs
    m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
    X = model.matrix(trms, m, contrasts.arg = NULL)
    Sol = as.matrix(object$Sol)[, seq_len(object$Fixed$nfl)] # toss out random effects if included
    bhat = apply(Sol, 2, mean)
    if (missing(vcov.))
        V = cov(Sol)
    else
        V = .my.vcov(object, vcov.)
    if (is.null(misc))
        misc = list()
    mode = match.arg(mode)
    if (mode == "multinomial") {
        misc$postGridHook = .MCMCglmm.multinom.postGrid
    }
    else { # try to figure out the link
        fam = unique(object$family)
        if (length(fam) > 1)
            stop("There is more than one 'family' in this model - too complex for emmeans support")
        link = switch(fam,
                      poisson = "log",
                      multinomial = "log",
                      categorical = "logit",
                      ordinal = "logit") # maybe more later?
        if (!is.null(link))
            misc = .std.link.labels(list(link = link), misc)
    }
    list(X = X, bhat = bhat, nbasis = matrix(NA), V = V, 
         dffun = function(k, dfargs) Inf, dfargs = list(), 
         misc = misc, post.beta = Sol)
}



.MCMCglmm.multinom.postGrid = function(object, ...) {
    linfct = object@linfct
    misc = object@misc
    post.lp = object@post.beta %*% t(linfct)
    sel = .find.by.rows(object@grid, "trait")
    k = length(sel)
    cols = unlist(sel)
    scal = sqrt(1 + 2 * (16 * sqrt(3) / (15 * pi))^2 / (k + 1))  # scaling const for logistic
    # I'm assuming here that diag(IJ) = 2 / (k + 1)
    object@post.beta = post.p = t(apply(post.lp, 1, function(l) {
        expX = exp(cbind(0, matrix(l[cols], ncol = k)) / scal)
        as.numeric(apply(expX, 1, function(z) z / sum(z)))
    })) # These results come out with response levels varying the fastest.
    
    object@bhat = apply(post.p, 2, mean)
    object@V = cov(post.p)
    preds = c(misc$trait, object@roles$predictors)
    object@roles$predictors  = preds[preds != "trait"]
    object@levels[["trait"]] = NULL
    object@levels = c(list(misc$resp.levs), object@levels)
    names(object@levels)[1] = misc$trait
    object@grid = do.call(expand.grid, object@levels)
    
    misc$postGridHook = misc$tran = misc$inv.lbl = 
        misc$trait = misc$resp.levs = NULL
    misc$display = object@model.info$nesting = NULL
    misc$estName = "prob"
    object@linfct = diag(1, ncol(post.p))
    object@misc = misc
    
    object
}


### Support for MCMCpack , maybe others that produce mcmc objects
### Whether it works depends on:
###    1. if there is a "call" attribute with a formula or fixed member
###    2. if it's right, even then
### Alternatively, maybe providing formula and data will do the trick

#' @exportS3Method recover_data mcmc
recover_data.mcmc = function(object, formula, data, ...) {
    if (missing(formula)) {
        cl = attr(object, "call")
        if (is.null(cl$formula))
            cl$formula = cl$fixed
        if (is.null(cl$formula))
            return("No fixed-effects formula found")
        data = NULL
    }
    else {
        if (missing(formula) || missing(data))
            return("Requires both formula and data to proceed")
        cl = call("mcmc.proxy", formula = formula, data = quote(data))
    }
    trms = delete.response(terms(eval(cl$formula, parent.frame())))
    recover_data(cl, trms, NULL, data, ...)
}

#' @exportS3Method emm_basis mcmc         
emm_basis.mcmc = function(object, trms, xlev, grid, vcov., contrasts.arg = NULL, ...) {
    m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
    X = model.matrix(trms, m, contrasts.arg = contrasts.arg)
    samp = as.matrix(object)[, seq_len(ncol(X)), drop = FALSE]
    bhat = apply(samp, 2, mean)
    if (missing(vcov.))
        V = cov(samp)
    else
        V = .my.vcov(object, vcov.)
    misc = list()
    list(X = X, bhat = bhat, nbasis = matrix(NA), V = V, 
         dffun = function(k, dfargs) Inf, dfargs = list(), 
         misc = misc, post.beta = samp)
}


### Support for mcmc.list
#' @exportS3Method recover_data mcmc.list
recover_data.mcmc.list = function(object, formula, data, ...) {
    recover_data.mcmc(object[[1]], formula, data, ...)
}

#' @exportS3Method emm_basis mcmc.list    
emm_basis.mcmc.list = function(object, trms, xlev, grid, vcov., ...) {
    result = emm_basis.mcmc(object[[1]], trms, xlev, grid, vcov., ...)
    cols = seq_len(ncol(result$post.beta))
    for (i in 2:length(object))
        result$post.beta = rbind(result$post.beta, 
            as.matrix(object[[i]])[, cols, drop = FALSE])
    attr(result$post.beta, "n.chains") = length(object)
    result
}


### support for CARBayes package - currently MUST supply data and have
### default contrasts matching what was used in fitting the mdoel
#' @exportS3Method recover_data carbayes
recover_data.carbayes = function(object, data, ...) {
    if(is.null(data)) # Try to recover data from parent frame
        data = model.frame(object$formula, data = parent.frame())
    cl = call("carbayes.proxy", formula = object$formula, data = quote(data))
    trms = delete.response(terms(eval(object$formula, parent.frame())))
    recover_data(cl, trms, NULL, data, ...)
}

#' @exportS3Method emm_basis carbayes
emm_basis.carbayes = function(object, trms, xlev, grid, ...) {
    m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
    X = model.matrix(trms, m, contrasts.arg = attr(object$X, "contrasts"))
    samp = as.matrix(object$samples$beta)
    bhat = apply(samp, 2, mean)
    V = cov(samp)
    misc = list()
    list(X = X, bhat = bhat, nbasis = matrix(NA), V = V, 
         dffun = function(k, dfargs) Inf, dfargs = list(), 
         misc = misc, post.beta = samp)
}



### Support for the rstanarm package (stanreg objects)
###
#' @exportS3Method recover_data stanreg
recover_data.stanreg = function(object, ...) {
    recover_data.lm(object, ...)
}

# note: mode and rescale are ignored for some models
#' @exportS3Method emm_basis stanreg      
emm_basis.stanreg = function(object, trms, xlev, grid, mode, rescale, ...) {
    misc = list()
    if (!is.null(object$family)) {
        if (is.character(object$family)) # work around bug for stan_polr
            misc$tran = object$method
        else
            misc = .std.link.labels(object$family, misc)
    }
    # Previous code...
    ### m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
    ### if(is.null(contr <- object$contrasts))
    ###     contr = attr(model.matrix(object), "contrasts")
    ### X = model.matrix(trms, m, contrasts.arg = contr)
    ### bhat = rstanarm::fixef(object)
    ### nms = intersect(colnames(X), names(bhat))
    ### bhat = bhat[nms]
    ### V = vcov(object)[nms, nms, drop = FALSE]

    # Instead, use internal routine in rstanarm to get the model matrix
    # Later, we'll get bhat and V from the posterior sample because
    # the vcov(object) doesn't always jibe with fixef(object)
    if(is.null(object$contrasts)) # old version of rstanarm where contrasts may get lost.
        object$contrasts = attr(model.matrix(object), "contrasts")
    pp_data = get("pp_data", envir = getNamespace("rstanarm"))
    X = pp_data(object, newdata = grid, re.form = ~0, ...)[[1]]
    nms = colnames(X)
    
    if(!is.null(object$zeta)) {   # Polytomous regression model
        if (missing(mode))
            mode = "latent"
        else
            mode = match.arg(mode, 
                             c("latent", "linear.predictor", "cum.prob", "exc.prob", "prob", "mean.class"))
        
        xint = match("(Intercept)", nms, nomatch = 0L)
        if (xint > 0L) 
            X = X[, -xint, drop = FALSE]
        k = length(object$zeta)
        if (mode == "latent") {
            ### if (missing(rescale))  #------ (Disabl rescale)
                rescale = c(0,1)
            X = rescale[2] * cbind(X, matrix(- 1/k, nrow = nrow(X), ncol = k))
            ### bhat = c(bhat, object$zeta - rescale[1] / rescale[2])
            misc = list(offset.mult = rescale[2])
        }
        else {
            ### bhat = c(bhat, object$zeta)
            j = matrix(1, nrow=k, ncol=1)
            J = matrix(1, nrow=nrow(X), ncol=1)
            X = cbind(kronecker(-j, X), kronecker(diag(1,k), J))
            link = object$method
            if (link == "logistic") link = "logit"
            misc = list(ylevs = list(cut = names(object$zeta)), 
                        tran = link, inv.lbl = "cumprob", offset.mult = -1)
            if (mode != "linear.predictor") {
                misc$mode = mode
                misc$postGridHook = ".clm.postGrid" # we probably need to adapt this
            }
        }
        nms = colnames(X) = c(nms, names(object$zeta))
        
        misc$respName = as.character.default(terms(object))[2]
    }
    samp = as.matrix(object$stanfit)
    nms = intersect(nms, colnames(samp)) # in case some are excl by rank def.
    samp = samp[, nms, drop = FALSE]
    attr(samp, "n.chains") = object$stanfit@sim$chains

    bhat = apply(samp, 2, mean)
    V = cov(samp)
    
    # estimability...
    nbasis = estimability::all.estble
    all.nms = colnames(X)
    if (length(nms) < length(all.nms)) {
        if(is.null(contr <- object$contrasts))
            contr = attr(model.matrix(object), "contrasts")
        coef = NA * X[1, ]
        coef[names(bhat)] = bhat
        bhat = coef
        data = object$data
        if(!is.null(subset <- object$call$subset)) {
            subset = eval(subset, envir = data, enclos = environment(terms(object)))
            data = data[subset, , drop = FALSE]
        }
        mmat = model.matrix(trms, data, contrasts.arg = contr)
        nbasis = estimability::nonest.basis(mmat)
    }
    
    
    list(X = X, bhat = bhat, nbasis = nbasis, V = V, 
         dffun = function(k, dfargs) Inf, dfargs = list(), 
         misc = misc, post.beta = samp)
}

##############################################################################
#    Copyright (c) 2012-2017 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

# Runs the function multcompLetters from the multcompView package
# returns an error if not installed
.mcletters = function(..., Letters=c("1234567890",LETTERS,letters), reversed=FALSE) {
    .requireNS("multcompView", 
               "The 'multcompView' package must be installed to use CLD methods")

    # Expand strings to individual letters
    Letters = as.character(unlist(sapply(Letters, function(stg) {
        sapply(seq_len(nchar(stg)), function(i) substr(stg, i, i))
    })))
    
    result = multcompView::multcompLetters(..., Letters=Letters, reversed=reversed)
    if (is.null(result$monospacedLetters))
        result$monospacedLetters = result$Letters
    result
}


### Lingering support for multcomp::cld -- registered dynamically in zzz.R
### NOTE: MUST KEEP the rdname of CLD.emmGrid 
###       because it's referenced by augmentedRCBD package
#' Compact letter displays
#' 
#' A method for \code{multcomp::cld()} is provided for users desiring to produce 
#' compact-letter displays (CLDs). 
#' This method uses the Piepho (2004) algorithm (as implemented in the
#' \pkg{multcompView} package) to generate a compact letter display of all
#' pairwise comparisons of estimated marginal means. The function obtains (possibly
#' adjusted) P values for all pairwise comparisons of means, using the
#' \code{\link{contrast}} function with \code{method = "pairwise"}. When a P
#' value exceeds \code{alpha}, then the two means have at least one letter in
#' common.
#' 
#' @rdname CLD.emmGrid
#' @order 1
#' @param object An object of class \code{emmGrid}
#' @param details Logical value determining whether detailed information on tests of
#'   pairwise comparisons is displayed
#' @param sort Logical value determining whether the EMMs are sorted before the comparisons
#'   are produced. When \code{TRUE}, the results are displayed according to
#'   \code{reversed}.
#' @param by Character value giving the name or names of variables by which separate
#'   families of comparisons are tested. If NULL, all means are compared.
#'   If missing, the object's \code{by.vars} setting, if any, is used.
#' @param alpha Numeric value giving the significance level for the comparisons
#' @param Letters Character vector of letters to use in the display. Any strings of
#'   length greater than 1 are expanded into individual characters
#' @param reversed,decreasing Logical value (passed to \code{multcompView::multcompLetters}.)
#'   If \code{TRUE}, the order of use of the letters is reversed.
#'   Either \code{reversed} or \code{decreasing} may be specified, thus providing
#'   compatibility with both \code{multcompView::multcompLetters(..., reversed, ...)} and
#'   \code{multcomp::cld(..., decreasing, ...)}.
#'   In addition, if both \code{sort} and \code{reversed} are TRUE, the sort
#'   order of results is reversed.
#' @param signif.sets Logical value. If \code{FALSE} (and \code{delta = 0}), a 
#'   \sQuote{traditional}
#'   compact-letter display is constructed with groupings representing sets of
#'   estimates that are not statistically different. If \code{TRUE}, the criteria
#'   are reversed so that two estimates sharing the same symbol test as significantly
#'   different. See also \code{delta}.
#' @param delta Numeric value passed to \code{\link{test.emmGrid}}. If this
#'   is positive, it is used as an equivalence threshold in the TOST procedure for
#'   two-sided equivalence testing. In the resulting compact letter display,
#'   two estimates share the same grouping letter only if they are found to be
#'   statistically equivalent -- that is, groupings reflect actual \emph{findings}
#'   of equivalence rather than failure to find a significant difference.
#'   When \code{delta} is nonzero, \code{signif.sets} is ignored.
#' @param ... Arguments passed to \code{\link{contrast}} (for example,
#'   an \code{adjust} method)
#' @references Piepho, Hans-Peter (2004) An algorithm for a letter-based 
#'   representation of all pairwise comparisons, 
#'   Journal of Computational and Graphical Statistics, 
#'   13(2), 456-466.
#' 
#' @return
#' A \code{\link[=summary.emmGrid]{summary_emm}} object showing the estimated marginal means
#' plus an additional column labeled \code{.group} (when \code{signif.sets = FALSE}), 
#' \code{.signif.set} (when \code{signif.sets = TRUE}), or \code{.equiv.set} 
#' (when \code{delta > 0}).
#' 
#' 
#'   
#' @note
#' We warn that the default display encourages a poor
#' practice in interpreting significance tests. Such CLDs are misleading because they
#' visually group means with comparisons \emph{P} > \code{alpha} as though they 
#' are equal, when in fact we have only failed to prove that they differ.
#' A better alternative if one wants to show groupings is to specify an equivalence
#' threshold \code{delta}; then groupings will be based on actual findings of
#' equivalence. Another way to display actual findings is to set
#' \code{signif.sets = TRUE}, so that estimates in the same group are those 
#' found to be statistically \emph{different}. Obviously, these different options
#' require different interpretations of the results; the annotations and the label
#' given the final column help guide how to assess the results.
#' 
#' As further alternatives, consider \code{\link{pwpp}} (graphical display of \emph{P} 
#' values) or \code{\link{pwpm}} (matrix display).
#'
#' @method cld emmGrid
#' @examples
#' if(requireNamespace("multcomp"))
#'     emm_example("cld-multcomp")
#'     # Use emm_example("cld-multcomp", list = TRUE) # to just list the code
#'
cld.emmGrid = function(object, details = FALSE, sort = TRUE, 
                       by, alpha = .05, 
                       Letters = c("1234567890",LETTERS,letters), 
                       reversed = decreasing, decreasing = FALSE, signif.sets = FALSE, delta = 0, ...) {
    if (!is.na(object@post.beta)[1]) {
        message("NOTE: Summary and groupings are based on frequentist results")
        object@post.beta = matrix(NA)
    }
    emmtbl = summary(object, ...)
    if(missing(by)) 
        by = object@misc$by.vars
    if (sort) {
        args = list()
        for (nm in by) args[[nm]] = emmtbl[[nm]]
        args$.emmGrid. = emmtbl[[attr(emmtbl, "estName")]]
        ord = do.call("order", unname(args))
        emmtbl = emmtbl[ord, , as.df = FALSE]
        if (!is.null(object@misc$display)) {
            use = which(object@misc$display)
            object@linfct = object@linfct[use, , drop = FALSE]
            object@grid = object@grid[use, , drop = FALSE]
            object@misc$display = NULL
        }
        object@grid = object@grid[ord, , drop = FALSE]
        object@linfct = object@linfct[ord, , drop = FALSE]
    }
    attr(emmtbl, "by.vars") = by
    object@misc$by.vars = by
    
    prwise = contrast(object, "revpairwise", by = by)
    testargs = list(object = prwise, delta = delta, ...)
    testargs$side = NULL   # can't be doin' 1-sided tests!
    pwtbl = do.call(test, testargs)
    
    if (delta > 0)
        signif.sets = TRUE
    
    p.boo = if(signif.sets)
        (pwtbl$p.value >= alpha)
    else
        (pwtbl$p.value < alpha)
    if(is.null(by)) {
        by.rows = list(seq_len(nrow(pwtbl)))
        by.out = list(seq_len(nrow(emmtbl)))
    }
    else {
        by.rows = .find.by.rows(pwtbl, by)
        by.out = .find.by.rows(emmtbl, by)
    }
    
    ### This code moved to inside the loop
    # Create comps matrix reflecting order generated by pairwise.emmc
    # icol = jcol = numeric(0)
    # create fake row indexes in revpairwise order for use by .mcletters
    # k = length(by.out[[1]])
    # for (i in 2:k) {
    #     icol = c(icol, seq_len(i-1))
    #     jcol = c(jcol, rep(i, i-1))
    # }
    
    ltrs = rep("", nrow(emmtbl))
    na.p = which(is.na(p.boo))
    # Take care of non-est cases. This is surprisingly complicated,
    # because it's possible we have some emmeans that are non-est
    # but comparisons are est'ble. So cases to exclude must be missing in
    # the table of means, AND appar somewhere in the indexes of NA p values
    # All that said, it still messes up because I didn't track the indexes correctly
    # excl.rows = intersect(which(is.na(emmtbl$SE)), union(icol[na.p], jcol[na.p]))
    # So I'll just go with which est's are missing
    excl.rows = which(is.na(emmtbl$SE))
    p.boo[na.p] = FALSE
    
    for (i in seq_len(length(by.rows))) {
        # Create comps matrix reflecting order generated by pairwise.emmc
        icol = jcol = numeric(0)
        k = length(by.out[[i]])
        for (j in 2:k) {
            icol = c(icol, seq_len(j-1))
            jcol = c(jcol, rep(j, j-1))
        }
        labs = paste(icol, jcol, sep="-")
        pb = p.boo[by.rows[[i]]]
        names(pb) = labs
        mcl = .mcletters(pb, Letters = Letters, reversed = reversed)$monospacedLetters
        ltrs[by.out[[i]]] = paste0(" ", mcl[seq_along(by.out[[i]])])
    }
    # any missing estimates get blanks...
    ltrs[excl.rows] = ""
    
    emmtbl[[".group"]] = ltrs
    if(sort && reversed) for (i in seq_len(length(by.out))) {
        r = by.out[[i]]
        emmtbl[r, ] = emmtbl[rev(r), ]
    }
    if (!signif.sets) {
        dontusemsg = paste0("NOTE: If two or more means share the same grouping symbol,\n",
                            "      then we cannot show them to be different.\n",
                            "      But we also did not show them to be the same.")
        
        attr(emmtbl, "mesg") = c(attr(emmtbl,"mesg"), attr(pwtbl, "mesg"), 
                                 paste("significance level used: alpha =", alpha), dontusemsg)
    }
    else if (delta == 0) {
        names(emmtbl)[names(emmtbl) == ".group"] = ".signif.set"
        attr(emmtbl, "mesg") = attr(emmtbl, "mesg") = c(attr(emmtbl,"mesg"), attr(pwtbl, "mesg"), 
            paste("significance level used: alpha =", alpha), 
            "Estimates sharing the same symbol are significantly different")
    }
    else {
        names(emmtbl)[names(emmtbl) == ".group"] = ".equiv.set"
        attr(emmtbl, "mesg") = attr(emmtbl, "mesg") = c(attr(emmtbl,"mesg"), attr(pwtbl, "mesg"), 
            paste("significance level used: alpha =", alpha), 
            "Estimates sharing the same symbol test as equivalent")
    }
    
    if (details)
        list(emmeans = emmtbl, comparisons = pwtbl)
    else
        emmtbl
}

### Registered dynamically in zzz.R
### NOTE: MUST KEEP the rdname of CLD.emmGrid 
###       because it's referenced by augmentedRCBD package
#' @rdname CLD.emmGrid
#' @order 2
#' @method cld emm_list
#' @param which Which element of the \code{emm_list} object to process
#'   (If length exceeds one, only the first one is used)
cld.emm_list = function(object, ..., which = 1) {
    multcomp::cld(object[[which[1]]], ...)
}

##############################################################################
#    Copyright (c) 2012-2022 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

# summary of a summary - just pass it thru
#' @export
summary.summary_emm = function(object, ...) object

# confint and test methods

# confint method
#' @rdname summary.emmGrid
#' @order 2
#' @param parm (Required argument for \code{confint} methods, but not used)
#' @method confint emmGrid
#' @export
confint.emmGrid = function(object, parm, level = .95, ...) {
    summary(object, infer = c(TRUE, FALSE), level = level, ...)
}

#' @rdname summary.emmGrid
#' @order 3
#' @export
test = function(object, null, ...) {
    UseMethod("test")
}

#' @rdname summary.emmGrid
#' @order 3
#' @param joint Logical value. If \code{FALSE}, the arguments are passed to 
#'   \code{\link{summary.emmGrid}} with \code{infer=c(FALSE, TRUE)}. If \code{joint = 
#'   TRUE}, a joint test of the hypothesis L beta = null is performed, where L 
#'   is \code{linfct(object)} and beta is the vector of fixed effects estimated 
#'   by \code{object@betahat}. This will be either an \emph{F} test or a 
#'   chi-square (Wald) test depending on whether degrees of freedom are 
#'   available. See also \code{\link{joint_tests}}.
#' @param verbose Logical value. If \code{TRUE} and \code{joint = TRUE}, a table
#'   of the effects being tested is printed.
#' @param rows Integer values. The rows of L to be tested in the joint test. If
#'   missing, all rows of L are used. If not missing, \code{by} variables are
#'   ignored.
#' @param status logical. If \code{TRUE}, a \code{note} column showing status
#'   flags (for rank deficiencies and estimability issues) is displayed even 
#'   when empty. If \code{FALSE}, the column is included only if there are 
#'   such issues.
#' @method test emmGrid
#' @export
test.emmGrid = function(object, null = 0, 
                    joint = FALSE, verbose = FALSE, rows, by, status = FALSE, ...) {
    # if joint = FALSE, this is a courtesy method for 'contrast'
    # else it computes the F test or Wald test of H0: L*beta = null
    # where L = object@linfct    
    if (!joint) {
        if (missing(by))
            summary(object, infer=c(FALSE,TRUE), null = null, ...)
        else
            summary(object, infer=c(FALSE,TRUE), null = null, by = by, ...)
    }
    else {
        if(verbose) {
            cat("Joint test of the following linear predictions\n")
            print(cbind(object@grid, equals = null))
        } 
        L = object@linfct
        bhat = object@bhat
        estble.idx = which(!is.na(object@bhat))
        bhat = bhat[estble.idx]
        est.flag = !is.na(object@nbasis[1])
        if(est.flag) {
            est.tol = get_emm_option("estble.tol")
            nbasis = zapsmall(object@nbasis)
        }

        if (!missing(rows))
            by.rows = list(sel.rows = rows)
        else {
            by.rows = list(all = seq_len(nrow(L)))
            if(missing(by)) 
                by = object@misc$by.vars
            if (!is.null(by)) 
                by.rows = .find.by.rows(object@grid, by)
        }
        
        # my own zapsmall fcn - sets a hard limit
        zapsm = function(x, tol = 1e-7) {
            x[abs(x) < tol] = 0
            x
        }

        result = lapply(by.rows, function(rows) {
            LL = L[rows, , drop = FALSE]

            # discard any rows that have NAs
            narows = apply(LL, 1, function(x) any(is.na(x)) | all(x == 0))
            LL = LL[!narows, , drop = FALSE]
            rrflag = 0 + 2 * any(narows)  ## flag for estimability issue
            
            if(est.flag)  { 
                if (any(!estimability::is.estble(LL, nbasis, est.tol))) {
                    LL = estimability::estble.subspace(zapsm(LL), nbasis)
                    rrflag = bitwOr(rrflag, 2)
                }
            }
            # Check rank
            qrLt = qr(zapsm(t(LL))) # this will work even if LL has 0 rows
            r = qrLt$rank
            if (r == 0)
                return(c(df1 = 0, df2 = NA, F.ratio = NA, p.value = NA, note = 3))
            
            if (r < nrow(LL)) {
                if(!all(null == 0))
                    stop("Rows are linearly dependent - cannot do the test when 'null' != 0")
                rrflag = bitwOr(rrflag, 1)
            }
            nz = 1:r   # indexes to use
            tR = t(qr.R(qrLt))[nz, nz, drop = FALSE]
            # for some reason I am getting overly optimistic ranks sometimes...
            nz = which(zapsmall(diag(tR)) != 0)
            if (length(nz) < r) {
                r = length(nz)
                tR = tR[nz, nz, drop = FALSE]
            }
            if(length(null) < r) null = rep(null, r)
            tQ = tQ.all = t(qr.Q(qrLt))[nz, , drop = FALSE]
            # tQ.all will have all the columns. tQ may get subsetted
            # NOW get rid of the NA parts...
            if (est.flag)
                tQ = tQ[, estble.idx, drop = FALSE]
            z = try(tQ %*% bhat - solve(tR, null[nz]), silent = TRUE)
            zcov = tQ %*% object@V %*% t(tQ)
            F = try(sum(z * solve(zcov, z)) / r)
            if (inherits(F, "try-error"))
                c(df1 = r, df2 = NA,  F.ratio = NA, p.value = NA, note = 1)
            else {
                if(is.null(df2 <- object@misc$df))
                    df2 = min(apply(tQ, 1, function(.) object@dffun(., object@dfargs)))
                if (is.na(df2)) 
                    df2 = Inf
                p.value = suppressWarnings(pf(F, r, df2, lower.tail = FALSE))
                rtn = c(round(c(df1 = r, df2 = df2), 2), 
                        F.ratio = round(F, 3), p.value = p.value, note = rrflag)
                # Note: Following will screw-up joint_tests if some df2's are finite and others infinite
                if (is.infinite(df2)) {
                    rtn = c(rtn[1:3], Chisq = round(F*r, 3), rtn[4:5])
                }
                attr(rtn, "L") = tQ.all
                rtn
            }
        })
        
        ef = lapply(result, function(r) attr(r, "L"))
        if (length(unique(sapply(result, length))) > 1) { # some results don't have chisq
            result = lapply(result, \(x) {
                if (length(x) == 5) 
                    x = c(x[1:3], Chisq = NA, x[4:5])
                x
            })
        }
        result = as.data.frame(t(as.data.frame(result)))
        if (!missing(by)) {
            fbr = sapply(by.rows, "[", 1)
            result = cbind(object@grid[fbr, by, drop = FALSE], result)
        }
        class(result) = c("summary_emm", "data.frame")
        attr(result, "estName") = "F.ratio"
        attr(result, "est.fcns") = lapply(ef, zapsmall)        
        if (!status && all(result$note == 0))
            result$note = NULL
        else {
            if (any(result$note %in% c(1,3)))
                attr(result, "mesg") = .dep.msg
            if (any(result$note %in% c(2,3)))
                attr(result, "mesg") = c(attr(result, "mesg"), .est.msg)
            result$note = sapply(result$note, function(x) 
                switch(x + 1, "", " d", "   e", " d e"))
        }
        result
    }
}

# messages (also needed by joint_tests())
.dep.msg = "d: df1 reduced due to linear dependence"
.est.msg = "e: df1 reduced due to non-estimability"

# Do all joint tests of contrasts. by, ... passed to emmeans() calls

#' Compute joint tests of the terms in a model
#'
#' This function produces an analysis-of-variance-like table based on linear
#' functions of predictors in a model or \code{emmGrid} object. Specifically,
#' the function constructs, for each combination of factors (or covariates
#' reduced to two or more levels), a set of (interaction) contrasts via
#' \code{\link{contrast}}, and then tests them using \code{\link{test}} with
#' \code{joint = TRUE}. Optionally, one or more of the predictors may be used as
#' \code{by} variable(s), so that separate tables of tests are produced for
#' each combination of them.
#' 
#' In models with only factors, no covariates, these tests correspond to
#' \dQuote{type III} tests a la \pkg{SAS}, as long as equal-weighted averaging
#' is used and there are no estimability issues. When covariates are present and
#' they interact with factors, the results depend on how the covariate is
#' handled in constructing the reference grid. See the section on covariates
#' below. The point that one must always remember is that \code{joint_tests}
#' always tests contrasts among EMMs, in the context of the reference grid,
#' whereas SAS's type III tests are tests of model coefficients -- which may or may
#' not have anything to do with EMMs or contrasts.
#' 
#' @param object a fitted model, \code{emmGrid}, or \code{emm_list}. If the
#'   latter, its first element is used.
#' @param cov.reduce a function.
#'    If \code{object} is a fitted model, it is
#'    replaced by \code{ref_grid(object, cov.reduce = cov.reduce, ...)}.
#'    For this purpose, the functions \code{meanint} and \code{symmint} are
#'    available for returning an interval around the mean or around zero,
#'    respectively. Se the section below on covariates.
#' @param by character names of \code{by} variables. Separate sets of tests are
#'    run for each combination of these.
#' @param show0df logical value; if \code{TRUE}, results with zero numerator
#'    degrees of freedom are displayed, if \code{FALSE} they are skipped
#' @param showconf logical value.
#'    When we have models with estimability issues (e.g., missing cells), then with
#'    \code{showconf = TRUE}, we test any remaining effects that are not purely
#'    due to contrasts of a single term. If found, they are labeled
#'    \code{(confounded)}. See
#'    \code{vignette("xplanations")} for more information.
#' @param ... additional arguments passed to \code{ref_grid} and \code{emmeans}
#'
#' @return a \code{summary_emm} object (same as is produced by 
#'   \code{\link{summary.emmGrid}}). All effects for which there are no
#'   estimable contrasts are omitted from the results. 
#'   There may be an additional row named \code{(confounded)} which accounts
#'   for additional degrees of freedom for effects not accounted for in the 
#'   preceding rows.
#'   
#'   The returned object also includes an \code{"est.fcns"} attribute, which is a
#'   named list containing the linear functions associated with each joint test.
#'   Each row of these is standardized to have length 1. 
#'   No estimable functions are included for confounded effects.
#'   
#' @section Dealing with covariates:
#' A covariate (or any other predictor) must have \emph{more than one value in 
#' the reference grid} in order to test its effect and be included in the results.
#' Therefore, when \code{object} is a model, we default to \code{cov.reduce = meanint}
#' which sets each covariate at a symmetric interval about its mean. But
#' when \code{object} is an existing reference grid, it often has only one value 
#' for covariates, in which case they are excluded from the joint tests.
#' 
#' While having two points is sufficient when the covariate term has a linear trend,
#' you need more than two when some kind of curved trend (polynomial, spline, etc.)
#' is present -- else \code{joint_tests()} will not show enough degrees of freedom 
#' for terms involving the covariate. You may specify these points manually using \code{at},
#' or by including an \code{npts} argument in \code{cov.reduce}, via \code{make.meanint}
#' or \code{make.symmint()}. With some kinds of curved trends, the joint tests of
#' covariate terms may become somewhat meaningless.
#' 
#' Covariates present further complications in that their values in the
#' reference grid can affect the joint tests of \emph{other} effects. When
#' covariates are centered around their means (the default), then the tests we
#' obtain can be described as joint tests of covariate-adjusted means; and that
#' is our intended use here. However, some software such as \pkg{SAS} and
#' \code{car::Anova} adopt the convention of centering covariates around zero;
#' and for that purpose, one can use \code{cov.reduce = symmint(0)} when calling
#' with a model object (or in constructing a reference grid). However, adjusted
#' means with covariates set at or around zero do not make much sense in the
#' context of interpreting estimated marginal means, unless the covariate means
#' really are zero.
#' 
#' See the examples below with the \code{toy} dataset.
#' 
#' @note \code{joint_tests} is flaky with models having nested fixed effects. In
#' some cases, terms that could be relevant are not identified, or confounded
#' with unidentifiable terms.
#' 
#' @seealso \code{\link{test}}
#' @order 1
#' @export
#'
#' @examples
#' pigs.lm <- lm(log(conc) ~ source * factor(percent), data = pigs)
#' 
#' (jt <- joint_tests(pigs.lm))             ## will be same as type III ANOVA
#' 
#' ### Estimable functions associated with "percent"
#' attr(jt, "est.fcns") $ "percent"
#' 
#' joint_tests(pigs.lm, weights = "outer")  ## differently weighted
#' 
#' joint_tests(pigs.lm, by = "source")      ## separate joint tests of 'percent'
#' 
#' ### Comparisons with type III tests in SAS
#' toy = data.frame(
#'     treat = rep(c("A", "B"), c(4, 6)),
#'     female = c(1, 0, 0, 1,   0, 0, 0, 1, 1, 0 ),
#'     resp = c(17, 12, 14, 19, 28, 26, 26, 34, 33, 27))
#' toy.fac = lm(resp ~ treat * factor(female), data = toy)
#' toy.cov = lm(resp ~ treat * female, data = toy)
#' # (These two models have identical fitted values and residuals)
#' 
#' # -- SAS output we'd get with toy.fac --
#' ## Source          DF    Type III SS    Mean Square   F Value   Pr > F
#' ## treat            1    488.8928571    488.8928571    404.60   <.0001
#' ## female           1     78.8928571     78.8928571     65.29   0.0002
#' ## treat*female     1      1.7500000      1.7500000      1.45   0.2741
#' # 
#' # -- SAS output we'd get with toy.cov --
#' ## Source          DF    Type III SS    Mean Square   F Value   Pr > F
#' ## treat            1    252.0833333    252.0833333    208.62   <.0001
#' ## female           1     78.8928571     78.8928571     65.29   0.0002
#' ## female*treat     1      1.7500000      1.7500000      1.45   0.2741
#' 
#' joint_tests(toy.fac)
#' joint_tests(toy.cov)   # female is regarded as a 2-level factor by default
#' 
#' ## Treat 'female' as a numeric covariate (via cov.keep = 0)
#' ## ... then tests depend on where we center things
#' 
#' # Center around the mean
#' joint_tests(toy.cov, cov.keep = 0, cov.reduce = make.meanint(delta = 1))
#' # Center around zero (like SAS's results for toy.cov)
#' joint_tests(toy.cov, cov.keep = 0, cov.reduce = make.symmint(ctr = 0, delta = 1))
#' # Center around 0.5 (like SAS's results for toy.fac)
#' joint_tests(toy.cov, cov.keep = 0, cov.reduce = range)
#' 
#' ### Example with empty cells and confounded effects
#' low3 <- unlist(attr(ubds, "cells")[1:3]) 
#' ubds.lm <- lm(y ~ A*B*C, data = ubds, subset = -low3)
#' 
#' # Show overall joint tests by C:
#' ref_grid(ubds.lm, by = "C") |> contrast("consec") |> test(joint = TRUE)
#' 
#' # Break each of the above into smaller components:
#' joint_tests(ubds.lm, by = "C")
#' 
joint_tests = function(object, by = NULL, show0df = FALSE, 
                       showconf = TRUE,
                       cov.reduce = make.meanint(1), ...) {
    
    # hidden defaults for contrast methods and which basis to use for all contrasts
    use.contr = (function(use.contr = c("consec", "consec"), ...) use.contr)(...)

    object = .chk.list(object,...)
    if (!inherits(object, "emmGrid")) {
        args = .zap.args(object = object, cov.reduce = cov.reduce, ..., omit = "submodel")
        object = do.call(ref_grid, args)
    }
    facs = setdiff(names(object@levels), c(by, "1"))

    if(length(facs) == 0)
        stop("There are no factors to test")
    
    # Use "factors" attr if avail to screen-out interactions not in model
    # For any factors not in model (created by emmeans fcns), assume they interact w/ everything
    trmtbl = attr(object@model.info$terms, "factors")
    if (is.null(trmtbl) || (length(trmtbl) == 0))
        trmtbl = matrix(1, nrow = length(facs), dimnames = list(facs, NULL))
    else
        row.names(trmtbl) = sapply(row.names(trmtbl), function(x)
            .all.vars(reformulate(x)))
    xtras = setdiff(facs, row.names(trmtbl))
    if (length(xtras) > 0) {
        xt = matrix(1, nrow = length(xtras), ncol = ncol(trmtbl), dimnames = list(xtras, NULL))
        trmtbl = rbind(trmtbl, xt)
    }
    nesting = object@model.info$nesting
    for (nst in names(nesting)) { # make sure all complete nests are in the table
        trmtbl = cbind(trmtbl, 0)
        n = ncol(trmtbl)
        trmtbl[c(nst, nesting[[nst]]), n] = 1
    }
    est.fcns = list()
    ef.ord = character(0)

    do.test = function(these, facs, result, ...) {
        if ((k <- length(these)) > 0) {
            if(any(apply(trmtbl[these, , drop = FALSE], 2, prod) != 0)) { # term is in model
                nesters = NULL
                if (!is.null(nesting)) {
                    nst = intersect(these, names(nesting))
                    if (length(nst) > 0)
                        nesters = unlist(nesting[nst]) # proceed only if these includes all nesters
                }
                if (is.null(nesting) || length(setdiff(nesters, these)) == 0) {   
                    emm = emmeans(object, these, by = by, ...)
                    tst = test(contrast(emm, interaction = use.contr[1], by = union(by, nesters)), 
                               by = by, joint = TRUE, status = TRUE)
                    ef = attr(tst, "est.fcns") # get this before we subset the results
                    tst = tst[names(tst) != "Chisq"]   # could have some with Chisq and some without
                    mt = paste(these, collapse = ":")
                    if (length(ef) > 1)
                        ef = list(ef)
                    names(ef) = mt
                    est.fcns <<- c(est.fcns, ef)
                    ef.ord <<- c(ef.ord, k)
                    tst = cbind(ord = k, `model term` = mt, tst)
                    result = rbind(result, tst)
                }
            }
            last = max(match(these, facs))
        }
        else
            last = 0
        if (last < (n <- length(facs)))
            for (i in last + seq_len(n - last))
                result = do.test(c(these, facs[i]), facs, result, ...)
        result
    }
    result = suppressMessages(do.test(character(0), facs, NULL, ...))
    if (!is.null(result) && all(is.infinite(result$df2) | is.na(result$df2))) {
        w = which(names(result) == "F.ratio")
        result = cbind(result[, 1:w], Chisq = result$F.ratio * result$df1, 
                       result[, (w+1):ncol(result)])
    }
    
    
    ## look at as-yet-unexplained effects
    if (showconf) {
        tmp = contrast(object, use.contr[2], by = by, name = ".cnt.", ...)
        br = .find.by.rows(tmp@grid, by)
        ef = est.fcns
        if (!is.null(ef)) {
            lf = rows = NULL
            for (i in seq_along(br)) {  # assemble est fcns for each by variable
                r = br[[i]]
                nm = names(br)[i]
                efi = if (!is.null(nm)) lapply(ef, function(e) e[[nm]])
                else ef
                efi = do.call(rbind, efi)
                if(!is.null(efi)) {
                    lf = rbind(lf, efi)     # stack 'em up into lf
                    rows = c(rows, r[seq_len(nrow(efi))])   # rows w/ same by combs
                }
                else {
                    lf = rbind(lf, NA * tmp@linfct[r[1], ])
                    rows = c(rows, r[1])
                }
            }
            tmpe = tmp
            tmpe@linfct = lf
            tmpe@grid = tmp@grid[rows, , drop = FALSE]
            
            ref = conf = test(tmp, joint = TRUE, status = TRUE)
            tst = test(tmpe, joint = TRUE)
            conf$df1 = ref$df1 - tst$df1
            conf$F.ratio = (ref$df1 * ref$F.ratio - tst$df1 * tst$F.ratio) / conf$df1
            conf$p.value = pf(conf$F.ratio, conf$df1, conf$df2, lower.tail = FALSE)
            conf$note = ""
            conf = cbind(ord = 999, `model term` = "(confounded)", conf)
            result = rbind(result, conf)
        }
    }
    
    result = result[order(result[[1]]), -1, drop = FALSE]
    est.fcns = est.fcns[order(ef.ord)]
    if(!show0df) {
        result = result[result$df1 > 0, , drop = FALSE]
        if(!is.null(by))
            est.fcns = lapply(est.fcns, function(x) x[!sapply(x, is.null)])
        est.fcns = est.fcns[!sapply(est.fcns, is.null)]
    }
        
    class(result) = c("summary_emm", "data.frame")
    attr(result, "estName") = "F.ratio"
    attr(result, "by.vars") = by
    nms = colnames(object@linfct)
    if(is.null(by)) 
        est.fcns = lapply(est.fcns, function(x) {
            if(!is.null(x)) colnames(x) = nms; x})
    else 
        est.fcns = lapply(est.fcns, function(L) lapply(L, function(x) {
            if(!is.null(x)) colnames(x) = nms; x}))
    attr(result, "est.fcns") = est.fcns
    if (any(result$note != "")) {
        msg = character(0)
        if (any(result$note %in% c(" d", " d e")))  
            msg = .dep.msg
        if (any(result$note %in% c("   e", " d e")))
            msg = c(msg, .est.msg)
        attr(result, "mesg") = msg
    }
    else
        result$note = NULL
    result
}


#' @rdname joint_tests
#' @order 6
#'
#' @param delta,ctr arguments for \code{make.meanint} and \code{make.symmint}.
#'   \code{delta} sets the distance each side of the center, so that the
#'   width of the interval is \code{2*delta}.
#' @param npts  number of points to include in the interval
#'
#' @return \code{make.meanint} returns the function 
#' \code{function(x) mean(x) + delta * c(-1, 1)},
#'   and \code{make.symmint(ctr, delta)} returns the function
#' \code{function(x) ctr + delta * c(-1, 1)}
#'         (which does not depend on \code{x}).
#' The cases with \code{delta = 1}, \code{meanint = make.meanint(1)} and
#' \code{symmint(ctr) = make.symmint(ctr, 1)} are retained for
#' back-compatibility reasons. These functions are available primarily for use
#' with \code{cov.reduce}.
#' @export
make.meanint = function(delta = 1, npts = 2) 
    function(x) mean(x) + delta * seq(-1, 1, length.out = npts)

#' @rdname joint_tests
#' @order 7
#' @param x argument for \code{meanint} and \code{symmint}
#' @export
meanint = function(x) mean(x) + c(-1, 1)


# const pm 1
#' @rdname joint_tests
#' @order 8
#' @export
make.symmint = function(ctr, delta = 1, npts = 2) {
    function(x) ctr + delta * seq(-1, 1, length.out = npts) 
}

#' @rdname joint_tests
#' @order 9
#' @export
symmint = function(ctr) 
    make.symmint(ctr, 1)

##############################################################################
#    Copyright (c) 2012-2025 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

# S3 plot method for emmGrid objects
# ... are arguments sent to update()

# DELETED: #' @import ggplot2


#' @rdname plot
#' @importFrom graphics plot
#' @method plot emmGrid
#' @export
plot.emmGrid = function(x, y, type, CIs = TRUE, PIs = FALSE, comparisons = FALSE, 
                    colors,
                    alpha = .05, adjust = "tukey", int.adjust = "none", intervals, ...) {
    
    if(!missing(intervals))
        CIs = intervals
    nonlin.scale = FALSE
    if(!missing(type)) {
        # If we say type = "scale", set it to "response" and set a flag
        if (nonlin.scale <- (type %.pin% "scale"))  ##(pmatch(type, "scale", 0) == 1)) 
            type = "response"
        object = update(x, predict.type = type, ..., silent = TRUE)
    }
    else
        object = update(x, ..., silent = TRUE)
    ptype = ifelse(is.null(object@misc$predict.type), "lp", object@misc$predict.type)
    # when we want comparisons, we have a transformation, and we want non-link scale, it's mandatory to regrid first:
    ### Nope, we are bypassing that as we will ALWAYS do things on the link scale
    # if(comparisons && !is.null(object@misc$tran) && 
    #         !(ptype %in% c("link", "lp", "linear.predictor")))
    #     object = regrid(object, transform = ptype)
    if (missing(int.adjust)) {
        int.adjust = object@misc$adjust
        if (is.null(int.adjust))
            int.adjust = "none"
    }
    
    frequentist = (\(frequentist = FALSE, ...) frequentist)(...)
    # we will do everything on link scale and back-transform later in .plot.srg...
    summ = summary(object, infer = c(TRUE, FALSE), adjust = int.adjust, type = "lp", ...)
    if (is.null(attr(summ, "pri.vars"))) { ## new ref_grid - use all factors w/ > 1 level
        pv = names(x@levels)
        len = sapply(x@levels, length)
        if (max(len) > 1)
            pv = pv[len > 1]
        attr(summ, "pri.vars") = pv
    }
    if (PIs) {
        prd = predict(object, interval = "pred", frequentist = frequentist, type = "lp", ...)
        if (PIs <- !is.null(prd)) {
            summ$lpl = prd$lower.PL
            summ$upl = prd$upper.PL
        }
    }
    
    estName = attr(summ, "estName")
    extra = NULL
    bayes = 
    if(comparisons) {
        if ((!is.na(object@post.beta[1])) && (!frequentist))
            stop("Comparison intervals are not implemented for Bayesian analyses")
        extra = object
        extra@misc$comp.alpha = alpha
        extra@misc$comp.adjust = adjust
    }
    if (nonlin.scale)
        scale = .make.scale(object@misc)
    else
        scale = NULL
    .plot.srg(x = summ, CIs = CIs, PIs = PIs, colors = colors, extra = extra, 
              backtran = !(ptype %in% c("link", "lp", "linear.predictor")), 
              link = attr(.est.se.df(object), "link"), scale = scale, ...)
}

# May use in place of plot.emmGrid but no control over level etc.
# extra is a placeholder for comparison-interval stuff



#' Plot an \code{emmGrid} or \code{summary_emm} object
#' 
#' Methods are provided to plot EMMs as side-by-side CIs, and optionally to display 
#'   \dQuote{comparison arrows} for displaying pairwise comparisons.
#'
#' @rdname plot
#' @param x Object of class \code{emmGrid} or \code{summary_emm}
#' @param y (Required but ignored)
#' @param horizontal Logical value specifying whether the intervals should be
#'   plotted horizontally or vertically
#' @param xlab Character label for horizontal axis
#' @param ylab Character label for vertical axis
#' @param facetlab Character or function method used to label facets in a 
#'   multi-panel plot (with the \code{ggplot} engine).
#'   Default is \code{"label_both"}, meaning that both factor names and levels are shown,
#'   You can use \code{"label_value"} to save space, or \code{"label_context"}
#'   to decide automatically (often wrongly). See the help page for 
#'   \code{ggplot2::labellers}
#' @param layout Numeric value passed to \code{\link[lattice:xyplot]{dotplot}}
#'   when \code{engine == "lattice"}.
#' @param type Character value specifying the type of prediction desired
#'   (matching \code{"linear.predictor"}, \code{"link"}, or \code{"response"}).
#'   See details under \code{\link{summary.emmGrid}}.
#'   In addition, the user may specify \code{type = "scale"}, in which case
#'   a transformed scale (e.g., a log scale) is displayed based on the transformation
#'   or link function used. Additional customization of this scale is available through
#'   including arguments to \code{ggplot2::scale_x_continuous} in \code{...} .
#' @param scale Object of class \code{trans} (in the \pkg{scales} package) to
#'   specify a nonlinear scale. This is used in lieu of \code{type = "scale"} when
#'   plotting a \code{summary_emm} object created with \code{type = "response"}.
#'   This is ignored with other types of summaries.
#' @param CIs Logical value. If \code{TRUE}, confidence intervals are
#'   plotted for each estimate.
#' @param PIs Logical value. If \code{TRUE}, prediction intervals are
#'   plotted for each estimate. If \code{object} is a Bayesian model,
#'   this requires the \code{...} arguments to include
#'   \code{frequentist = TRUE} and \code{sigma =} (some value).
#'   Note that the \code{PIs} option is \emph{not} available with
#'   \code{summary_emm} objects -- only for \code{emmGrid} objects.
#'   Also, prediction intervals are not available
#'   with \code{engine = "lattice"}.
#' @param comparisons Logical value. If \code{TRUE}, \dQuote{comparison arrows}
#'   are added to the plot, in such a way that the degree to which arrows
#'   overlap reflects as much as possible the significance of the comparison of
#'   the two estimates. (A warning is issued if this can't be done.)
#'   Note that comparison arrows are not available with `summary_emm` objects.
#' @param colors Optional character vector of colors to use for estimates, CIs, PIs, 
#'   and comparison arrows, respectively. If missing, these are derived from
#'   \code{ink}, \code{paper}, and \code{accent} in the theme's \code{geom} element.
#'   \code{ink} is used for the estimates, \code{accent} is used for comparison arrows,
#'   and a contrasting color to \code{accent} is blended with \code{paper} for
#'   the intervals. 
#'   If just one color is given, estimates and intervals are made from
#'   various blends of that color and \code{paper}, and \code{accent}
#'   color is used for comparison arrows.
#' @param alpha The significance level to use in constructing comparison arrows
#' @param adjust Character value: Multiplicity adjustment method for comparison arrows \emph{only}.
#' @param int.adjust Character value: Multiplicity adjustment method for the plotted confidence intervals \emph{only}.
#' @param intervals If specified, it is used to set \code{CIs}. This is the previous
#'   argument name for \code{CIs} and is provided for backward compatibility.
#' @param plotit Logical value. If \code{TRUE}, a graphical object is returned;
#'   if \code{FALSE}, a data.frame is returned containing all the values
#'   used to construct the plot.
#' @param ... Additional arguments passed to \code{\link{update.emmGrid}}, 
#'   \code{\link{summary.emmGrid}},
#'   \code{\link{predict.emmGrid}}, or
#'   \code{\link[lattice:xyplot]{dotplot}}
#'
#' @return If \code{plotit = TRUE}, a graphical object is returned.
#' 
#'   If \code{plotit = FALSE}, a \code{data.frame} with the table of
#'   EMMs that would be plotted. In the latter case, the estimate being plotted
#'   is named \code{the.emmean}, and any factors involved have the same names as
#'   in the object. Confidence limits are named \code{lower.CL} and
#'   \code{upper.CL}, prediction limits are named \code{lpl} and \code{upl}, and
#'   comparison-arrow limits are named \code{lcmpl} and \code{ucmpl}.
#'   There is also a variable named \code{pri.fac} which contains the factor 
#'   combinations that are \emph{not} among the \code{by} variables.

#' @section Details:
#' If any \code{by} variables are in force, the plot is divided into separate
#' panels. For
#' \code{"summary_emm"} objects, the \code{\dots} arguments in \code{plot}
#' are passed \emph{only} to \code{dotplot}, whereas for \code{"emmGrid"}
#' objects, the object is updated using \code{\dots} before summarizing and
#' plotting.
#' 
#' In plots with \code{comparisons = TRUE}, the resulting arrows are only
#' approximate, and in some cases may fail to accurately reflect the pairwise
#' comparisons of the estimates -- especially when estimates having large and
#' small standard errors are intermingled in just the wrong way. Note that the
#' maximum and minimum estimates have arrows only in one direction, since there
#' is no need to compare them with anything higher or lower, respectively. See
#' the \href{../doc/xplanations.html#arrows}{\code{vignette("xplanations",
#' "emmeans")}} for details on how these are derived.
#' 
#' If \code{adjust} or \code{int.adjust} are not supplied, they default to the 
#' internal \code{adjust} setting saved in \code{pairs(x)} and \code{x} 
#' respectively (see \code{\link{update.emmGrid}}).
#' 
#' @note In order to play nice with the plotting functions,
#' any variable names that are not syntactically correct (e.g., contain spaces)
#' are altered using \code{\link{make.names}}.
#' 
#' @importFrom graphics plot
#' 

#' @method plot summary_emm
#' @export
#'
#' @examples
#' warp.lm <- lm(breaks ~ wool * tension, data = warpbreaks)
#' warp.emm <- emmeans(warp.lm, ~ tension | wool)
#' plot(warp.emm)
#' 
#' plot(warp.emm, PIs = TRUE, comparisons = TRUE)
#' 
#' with_emm_options(gg.theme = ggplot2::theme_dark(), 
#'     plot(warp.emm, PIs = TRUE, comparisons = TRUE))
#' 
#' plot(warp.emm, by = NULL, comparisons = TRUE, adjust = "none", 
#'      horizontal = FALSE, colors = "blue")
#' 
#' ### Using a transformed scale (also demonstrating 'facetlab' argument)
#' pigs.lm <- lm(log(conc + 2) ~ source * factor(percent), data = pigs)
#' pigs.emm <- emmeans(pigs.lm, ~ percent | source)
#' plot(pigs.emm, type = "scale", breaks = seq(20, 100, by = 10), 
#'      facetlab = "label_value")
#' 
#' # Based on a summary. 
#' # To get a transformed axis, must specify 'scale'; but it does not necessarily
#' # have to be the same as the actual response transformation
#' pigs.ci <- confint(pigs.emm, type = "response")
#' plot(pigs.ci, scale = scales::log10_trans(), 
#'      facetlab = \(x) ggplot2::label_both(x, sep = " = "))
plot.summary_emm = function(x, y, horizontal = TRUE, CIs = TRUE,
                            xlab, ylab, facetlab = "label_both", layout, scale = NULL,
                            colors, intervals, 
                            plotit = TRUE, ...) {
    if(!missing(intervals))
        CIs = intervals
    if(attr(x, "type") != "response")   # disable scale when no response transformation
        scale = NULL
    .plot.srg (x, y, horizontal, xlab, ylab, facetlab = facetlab, layout, scale = scale,
               CIs = CIs, colors = colors, plotit = plotit, ...)
}

# Workhorse for plot.summary_emm
#' @importFrom grDevices col2rgb hcl rgb rgb2hsv
.plot.srg = function(x, y, 
                     horizontal = TRUE, xlab, ylab, facetlab = "label_both", layout, colors,
                     engine = get_emm_option("graphics.engine"),
                     CIs = TRUE, PIs = FALSE, extra = NULL, 
                     plotit = TRUE, backtran = FALSE, link, scale = NULL, ...) {
    
    engine = match.arg(engine, c("ggplot", "lattice"))
    if (engine == "ggplot") 
        .requireNS("ggplot2", 
                   "The 'ggplot' engine requires the 'ggplot2' package be installed.")
    if (engine == "lattice")
        .requireNS("lattice", 
                   "The 'lattice' engine requires the 'lattice' package be installed.")
    
    summ = x # so I don't get confused
    estName = "the.emmean"
    names(summ)[which(names(summ) == attr(summ, "estName"))] = estName
    clNames = attr(summ, "clNames")
    if (is.null(clNames)) {
        warning("No information available to display confidence limits")
        lcl = ucl = summ[[estName]]
    }
    else {
        lcl = summ[[clNames[1]]]
        ucl = summ[[clNames[2]]]
    }
    
    # ensure all names are syntactically valid
    summ = .validate.names(summ)
    
    if (engine == "lattice") { # ---------- lattice-specific stuff ----------
        
        # Panel functions...
        prepanel.ci = function(x, y, horizontal=TRUE, CIs=TRUE,
                               lcl, ucl, subscripts, ...) {
            x = as.numeric(x)
            lcl = as.numeric(lcl[subscripts])
            ucl = as.numeric(ucl[subscripts])
            if (!CIs) # no special scaling needed
                list()
            else if (horizontal)
                list(xlim = range(x, ucl, lcl, finite = TRUE)) 
            else
                list(ylim = range(y, ucl, lcl, finite = TRUE)) 
        }
        panel.ci <- function(x, y, horizontal=TRUE, CIs=TRUE,
                             lcl, ucl, lcmpl, rcmpl,                          subscripts, pch = 16, 
                             lty = dot.line$lty, lwd = dot.line$lwd, 
                             col = dot.symbol$col, col.line = dot.line$col, ...) {
            dot.line <- lattice::trellis.par.get("dot.line")
            dot.symbol <- lattice::trellis.par.get("dot.symbol")
            x = as.numeric(x)
            y = as.numeric(y)
            lcl = as.numeric(lcl[subscripts])
            ucl = as.numeric(ucl[subscripts])
            compare = !is.null(lcmpl)
            if(compare) {
                lcmpl = as.numeric(lcmpl[subscripts])
                rcmpl = as.numeric(rcmpl[subscripts])
            }
            if(horizontal) {
                lattice::panel.abline(h = unique(y), col = col.line, lty = lty, lwd = lwd)
                if(CIs) 
                    lattice::panel.arrows(lcl, y, ucl, y, col = col, length = .6, unit = "char", angle = 90, code = 3)
                if(compare) {
                    s = (x > min(x))
                    lattice::panel.arrows(lcmpl[s], y[s], x[s], y[s], length = .5, unit = "char", code = 1, col = "red", type = "closed", fill="red")
                    s = (x < max(x))
                    lattice::panel.arrows(rcmpl[s], y[s], x[s], y[s], length = .5, unit = "char", code = 1, col = "red", type = "closed", fill="red")
                }
            } 
            else {
                lattice::panel.abline(v = unique(x), col = col.line, lty = lty, lwd = lwd)
                if(CIs)
                    lattice::panel.arrows(x, lcl, x, ucl, col=col, length = .6, unit = "char", angle = 90, code = 3)
                if(compare) {
                    s = (y > min(y))
                    lattice::panel.arrows(x[s], lcmpl[s], x[s], y[s], length = .5, unit = "char", code = 1, col = "red", type = "closed", fill="red")
                    s = (y < max(y))
                    lattice::panel.arrows(x[s], rcmpl[s], x[s], y[s], length = .5, unit = "char", code = 1, col = "red", type = "closed", fill="red")
                }
            }
            lattice::panel.xyplot(x, y, pch=16, ...)
        }
        my.strip = lattice::strip.custom(strip.names = c(TRUE,TRUE), strip.levels = c(TRUE,TRUE), sep = " = ")
        
    } # ---------- end lattice-specific -----------
    
    sep = get_emm_option("sep")
    priv = attr(summ, "pri.vars")
    pf = do.call(paste, c(unname(summ[priv]), sep = sep))
    if (length(pf) == 0)
        pf = "1"
    summ$pri.fac = factor(pf, levels=unique(pf))
    chform = ifelse(horizontal,
                    paste("pri.fac ~", estName),
                    paste(estName, "~ pri.fac"))
    
    byv = attr(summ, "by.vars")
    if (!is.null(byv) && length(byv) > 0) {
        chform = paste(chform, "|", paste(byv, collapse="*"))
        lbv = do.call("paste", c(unname(summ[byv]), sep = sep)) # strings for matching by variables
        ubv = unique(lbv)
    }
    else {
        lbv = rep(1, nrow(summ))
        ubv = 1
    }
    
    
    # Obtain comparison limits
    if (!is.null(extra)) {
        ### We will ALWAYS be working on LP scale now...
        # # we need to work on the linear predictor scale
        # # typeid = 1 -> response, 2 -> other
        # typeid = pmatch(extra@misc$predict.type, "response", nomatch = 2)
        # if(length(typeid) < 1) typeid = 2        
        # if (typeid == 1)
        #     est = predict(extra, type = "lp")
        # else
        est = summ[[estName]]
        
        alpha = extra@misc$comp.alpha
        adjust = extra@misc$comp.adjust
        psumm = suppressMessages(confint(pairs(extra), level = 1 - alpha, type = "lp", adjust = adjust))
        psumm = .validate.names(psumm)
        k = ncol(psumm)
        del = (psumm[[k]] - psumm[[k-1]]) / 4 # half the halfwidth, on lp scale
        diff = psumm[[attr(psumm, "estName")]]
        overlap = apply(psumm[ ,(k-1):k], 1, function(x) 2*min(-x[1],x[2])/(x[2]-x[1]))
        
        # figure out by variables and indexes (lbv, ubv already defined)
        if(is.null(byv))
            pbv = rep(1, nrow(psumm))
        else
            pbv = do.call("paste", c(unname(psumm[byv]), sep = sep))
        neach = length(lbv) / length(ubv)
        # indexes for pairs results -- est[id1] - est[id2]
        id1 = rep(seq_len(neach-1), rev(seq_len(neach-1)))
        id2 = unlist(sapply(seq_len(neach-1), function(x) x + seq_len(neach-x)))
        # list of psumm row numbers involved in each summ row
        involved = lapply(seq_len(neach), function(x) union(which(id2==x), which(id1==x)))
        
        # initialize arrays
        mind = numeric(length(lbv))   # for minima of del
        llen = rlen = numeric(neach)  # for left and right arrow lengths
        npairs = length(id1)
        iden = diag(rep(1, 2*neach))
        
        for (by in ubv) {
            d = del[pbv == by]
            rows = which(lbv == by)
            for(i in seq_len(neach)) 
                mind[rows[i]] = min(d[involved[[i]]])
            
            # Set up regression equations to match arrow overlaps with interval overlaps
            # We'll add rows later (with weights 1) to match with mind values
            lmat = rmat = matrix(0, nrow = npairs, ncol = neach)
            y = numeric(npairs)
            v1 = 1 - overlap[pbv == by]
            dif = diff[pbv == by]
            for (i in which(!is.na(v1))) {
                #wgt = 6 * max(0, ifelse(v1[i] < 1, v1[i], 2-v1[i]))
                wgt = 3 + 20 * max(0, .5 - (1 - v1[i])^2)
                # really this is sqrt of weight
                if (dif[i] > 0)   # id2  <----->  id1
                    lmat[i, id1[i]] = rmat[i, id2[i]] = wgt*v1[i]
                else  # id1  <----->  id2
                    rmat[i, id1[i]] = lmat[i, id2[i]] = wgt*v1[i]
                y[i] = wgt * abs(dif[i])
            }
            X = rbind(cbind(lmat, rmat),iden)
            y = c(y, rep(mind[rows], 2))
            y[is.na(y)] = 0
            soln = qr.coef(qr(X), y)
            soln[is.na(soln)] = 0
            ll = llen[rows] = soln[seq_len(neach)]
            rl = rlen[rows] = soln[neach + seq_len(neach)]
            
            # Abort if negative lengths
            if (any(c(rl, ll) < 0)) {
                stop("Aborted -- Some comparison arrows have negative length!\n",
                     "(in group \"", by, "\")",
                     call. = FALSE)
            }
            
            # Overlap check
            for (i in which(!is.na(v1))) {
                v = 1 - v1[i]
                obsv = 1 - abs(dif[i]) / ifelse(dif[i] > 0, 
                                                ll[id1[i]] + rl[id2[i]], 
                                                rl[id1[i]] + ll[id2[i]])
                if (v*obsv < 0)
                    warning("Comparison discrepancy in group \"", by, 
                            "\", ", psumm[i, 1], 
                            ":\n    Target overlap = ", round(v, 4),
                            ", overlap on graph = ", round(obsv, 4),
                            call. = FALSE)
            }
            
            # shorten arrows that go past the data range
            estby = est[rows]
            rng = suppressWarnings(range(estby, na.rm = TRUE))
            diffr = diff(rng)
            ii = which(estby - ll < rng[1])
            llen[rows][ii] = estby[ii] - rng[1] + .02 * diffr
            ii = which(estby + rl > rng[2])
            rlen[rows][ii] = rng[2] - estby[ii] + .02 * diffr
            
            # remove arrows completely from extremes
            llen[rows][estby < rng[1] + .0001 * diffr] = NA
            rlen[rows][estby > rng[2] - .0001 * diffr] = NA
        }
        
        ### Unneeded now as we are always on LP scale
        # invtran = I
        # if (typeid == 1) {
        #     tran = extra@misc$tran
        #     if(is.character(tran)) {
        #         link = try(make.link(tran), silent=TRUE)
        #         if (!inherits(link, "try-error"))
        #             invtran = link$linkinv
        #     }
        #     else if (is.list(tran))
        #         invtran = tran$linkinv
        # }
        
        lcmpl = summ$lcmpl = as.numeric(est - llen)
        rcmpl = summ$rcmpl = as.numeric(est + rlen)
    }
    else lcmpl = rcmpl = NULL
    
    if(backtran && !missing(link) && !is.null(link)) { ### we need to back-transform stuff... link must be non-missing
        summ$the.emmean = with(link, linkinv(summ$the.emmean))
        summ[[clNames[1]]] = lcl = with(link, linkinv(lcl))
        summ[[clNames[2]]] = ucl = with(link, linkinv(ucl))
        if (PIs) {
            summ$lpl = with(link, linkinv(summ$lpl))
            summ$upl = with(link, linkinv(summ$upl))
        }
        if(!is.null(extra)) {
            summ$lcmpl = lcmpl = with(link, linkinv(summ$lcmpl))
            summ$rcmpl = rcmpl = with(link, linkinv(summ$rcmpl))
        }
    }
    
    if(!plotit) 
        return(as.data.frame(summ))
    
    
    facName = paste(priv, collapse=":")
    
    # private lil fcn to mix colors with white
    mix_col = function(col1, col2, frac) {
        x = grDevices::colorRamp(c(col1, col2))(frac) / 255
        rgb(x[1], x[2], x[3], 1)
    }
    
    if (engine == "lattice") {
        if(missing(colors))
            colors = c("black", mix_col("white", "blue", .35), mix_col("white", "blue", .20), "red")
        if(length(colors) == 1)
            colors = c(colors, mix_col("white", colors, .35), mix_col("white", colors, .20), "red")
        
        if (missing(layout)) {
            layout = c(1, length(ubv))
            if(!horizontal) 
                layout = rev(layout)
        }
        
        form = as.formula(chform)
        if (horizontal) {
            if (missing(xlab)) xlab = attr(summ, "estName")
            if (missing(ylab)) ylab = facName
            lattice::dotplot(form, prepanel=prepanel.ci, panel=panel.ci, 
                             strip = my.strip, horizontal = TRUE,
                             ylab = ylab, xlab = xlab,
                             data = summ, CIs = CIs, lcl=lcl, ucl=ucl, 
                             lcmpl=lcmpl, rcmpl=rcmpl, layout = layout, ...)
        }
        else {
            if (missing(xlab)) xlab = facName
            if (missing(ylab)) ylab = attr(summ, "estName")
            lattice::dotplot(form, prepanel=prepanel.ci, panel=panel.ci, 
                             strip = my.strip, horizontal = FALSE,
                             xlab = xlab, ylab = ylab,
                             data = summ, CIs = CIs, lcl=lcl, ucl=ucl, 
                             lcmpl=lcmpl, rcmpl=rcmpl, layout = layout, ...)
        }
    } # --- end lattice plot
    else {  ## ggplot method
        thm = theme_emm()
        
        if(missing(colors) && is.numeric(thopt <- get_emm_option("gg.theme")) && thopt == 1)
            colors = c("black", mix_col("white", "blue", .35), mix_col("white", "blue", .20), "red")
        
        if(missing(colors) || (length(colors) == 1)) {
            dot.col = ifelse(missing(colors), thm$geom@ink, colors)
            paper = thm$geom@paper
            if(missing(colors)) {   # derive a contrasting color to 'accent' color
                xxx = (rgb2hsv(col2rgb(thm$geom@accent))[1] + 0.54) %% 1
                clr = hcl(xxx * 360, 80, 50)
            }
            else
                clr = dot.col
            colors = c(dot.col, mix_col(paper, clr, 0.25), mix_col(paper, clr, 0.15), thm$geom@accent)
        }
        dot.col = colors[1]
        CI.col = colors[2]
        PI.col = colors[3]
        comp.col = colors[4]
        
        summ$lcl = lcl
        summ$ucl = ucl

        # construct horizontal plot - flip coords later if necessary
        grobj = ggplot2::ggplot(summ, ggplot2::aes(x = .data$the.emmean, y = .data$pri.fac)) +
            ggplot2::geom_point(color = CI.col, alpha = .0) 
            # invisible points to get grouping order right
        if (PIs) 
            grobj = grobj + ggplot2::geom_segment(ggplot2::aes(x = .data$lpl, xend = .data$upl, 
                                y = .data$pri.fac, yend = .data$pri.fac), 
                                color = PI.col, lwd = 2.5)
        if (CIs) 
            grobj = grobj + ggplot2::geom_segment(ggplot2::aes(x = .data$lcl, xend = .data$ucl, 
                                y = .data$pri.fac, yend = .data$pri.fac), 
                                color = CI.col, lwd = 4)

        if (!is.null(extra)) {
            grobj = grobj + 
                ggplot2::geom_segment(ggplot2::aes(x = .data$the.emmean, linewidth = 1,
                        xend = .data$lcmpl, y = .data$pri.fac, yend = .data$pri.fac), linewidth = 0.9,
                        arrow = ggplot2::arrow(length = ggplot2::unit(.07, "inches"),  
                            type = "closed"), color = comp.col, 
                        data = summ[!is.na(summ$lcmpl), ]) +
                ggplot2::geom_segment(ggplot2::aes(x = .data$the.emmean, 
                    xend = .data$rcmpl, y = .data$pri.fac, yend = .data$pri.fac), linewidth = 0.9,
                    arrow = ggplot2::arrow(length = ggplot2::unit(.07, "inches"), 
                            type = "closed"), color = comp.col,
                    data = summ[!is.na(summ$rcmpl), ])
        }
        if (length(byv) > 0)
            grobj = grobj + ggplot2::facet_grid(as.formula(paste(paste(byv, collapse = "+"), " ~ .")), 
                                                labeller = facetlab)
        grobj = grobj + ggplot2::geom_point(color = dot.col, size = 4, shape = 18)
        
        if(!is.null(scale)) {
            args = list(...)
            pass = (names(args) %.pin% names(as.list(args(ggplot2::scale_x_continuous))))
            args = c(list(trans = scale), args[pass])
            grobj = grobj + do.call(ggplot2::scale_x_continuous, args)
        }
            
        if (missing(xlab)) xlab = attr(summ, "estName")
        if (missing(ylab)) ylab = facName
            
        if(!horizontal)
            grobj = grobj + ggplot2::coord_flip()
        
        grobj + ggplot2::labs(x = xlab, y = ylab) + thm
    }
}

##############################################################################
#    Copyright (c) 2012-2017 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

# datasets (provided as .rda files in data/) -- this file is for documentation

### auto.noise ###
#' Auto Pollution Filter Noise
#' 
#' Three-factor experiment comparing pollution-filter noise for two filters,
#' three sizes of cars, and two sides of the car.
#' 
#' The data are from a statement by Texaco, Inc., to the Air and Water Pollution
#' Subcommittee of the Senate Public Works Committee on June 26, 1973.    
#' Mr. John McKinley, President of Texaco, cited an automobile filter developed
#' by Associated Octel Company as effective in reducing pollution. However, 
#' questions had been raised about the effects of filters on vehicle performance, 
#' fuel consumption, exhaust gas back pressure, and silencing. On the last 
#' question, he referred to the data included here as evidence that the silencing
#' properties of the Octel filter were at least equal to those of standard silencers.
#'  
#' @format A data frame with 36 observations on the following 4 variables.
#'   \describe{
#'   \item{\code{noise}}{Noise level in decibels (but see note) - a numeric vector.}
#'   \item{\code{size}}{The size of the vehicle - an ordered factor with
#'     levels \code{S}, \code{M}, \code{L}.}
#'   \item{\code{type}}{Type of anti-pollution filter - a factor with levels
#'     \code{Std} and \code{Octel}}
#'   \item{\code{side}}{The side of the car where measurement was taken -- a
#'     factor with levels \code{L} and \code{R}.}
#'   }
#' @source The dataset was obtained from the Data and Story Library (DASL)
#'   at Carnegie-Mellon University. Apparently it has since been removed. The
#'   original dataset was altered by assigning meaningful names to the factors
#'   and sorting the observations in random order as if this were the run order
#'   of the experiment.
#' @note While the data source claims that \code{noise} is measured in decibels,
#'   the values are implausible. I believe that these measurements are actually
#'   in tenths of dB (centibels?). Looking at the values in the dataset, note 
#'   that every measurement ends in 0 or 5, and it is reasonable to believe that
#'   measurements are accurate to the nearest half of a decibel.
#'   %%% Thanks to an email communication from a speech/hearing scientist
#' @examples
#' # (Based on belief that noise/10 is in decibel units)
#' noise.lm <- lm(noise/10 ~ size * type * side, data = auto.noise)
#' 
#' # Interaction plot of predictions
#' emmip(noise.lm, type ~ size | side)
#' 
#' # Confidence intervals
#' plot(emmeans(noise.lm, ~ size | side*type))
#' 
"auto.noise"

# This is where it used to be...
#  \url{http://lib.stat.cmu.edu/DASL/Datafiles/airpullutionfiltersdat.html}


### feedlot ###
#' Feedlot data
#' 
#' This is an unbalanced analysis-of-covariance example, where one covariate is
#' affected by a factor. Feeder calves from various herds enter a feedlot, where
#' they are fed one of three diets. The weight of the animal at entry is the
#' covariate, and the weight at slaughter is the response.
#' 
#' The data arise from a Western Regional Research Project conducted at New
#' Mexico State University. Calves born in 1975 in commercial herds entered a
#' feedlot as yearlings. Both diets and herds are of interest as factors. The
#' covariate, \code{ewt}, is thought to be dependent on \code{herd} due to
#' different genetic backgrounds, breeding history, etc. The levels of
#' \code{herd} ordered to similarity of genetic background.
#' 
#' Note: There are some empty cells in the cross-classification of 
#' \code{herd} and \code{diet}.
#' @format A data frame with 67 observations and 4 variables:
#' \describe{
#'   \item{\code{herd}}{a factor with levels \code{9} \code{16} \code{3}
#'     \code{32} \code{24} \code{31} \code{19} \code{36} \code{34} \code{35}
#'     \code{33}, designating the herd that a feeder calf came from.}
#'   \item{\code{diet}}{a factor with levels \code{Low} \code{Medium}
#'     \code{High}: the energy level of the diet given the animal.}
#'   \item{\code{swt}}{a numeric vector: the weight of the animal at slaughter.}
#'   \item{\code{ewt}}{a numeric vector: the weight of the animal at entry to the feedlot.}
#' }
#' @source Urquhart NS (1982) Adjustment in covariates when one factor affects
#'   the covariate. \emph{Biometrics} 38, 651-660.
#' @examples
#' feedlot.lm <- lm(swt ~ ewt + herd*diet, data = feedlot)
#' 
#' # Obtain EMMs with a separate reference value of ewt for each 
#' # herd. This reproduces the last part of Table 2 in the reference
#' emmeans(feedlot.lm,  ~ diet | herd,  cov.reduce = ewt ~ herd)
#' 
"feedlot"




### fiber ###
#' Fiber data
#' 
#' Fiber data from Montgomery Design (8th ed.), p.656 (Table 15.10). Useful as a
#' simple analysis-of-covariance example.
#' 
#' The goal of the experiment is to compare the mean breaking strength of fibers
#' produced by the three machines. When testing this, the technician also
#' measured the diameter of each fiber, and this measurement may be used as a
#' concomitant variable to improve precision of the estimates.
#' @format A data frame with 15 observations and 3 variables:
#' \describe{
#'   \item{\code{machine}}{a factor with levels \code{A} \code{B} \code{C}. 
#'     This is the primary factor of interest.}
#'   \item{\code{strength}}{a numeric vector. The response variable.}
#'   \item{\code{diameter}}{a numeric vector. A covariate.}
#' }
#' @source Montgomery, D. C. (2013) \emph{Design and Analysis of Experiments}
#'   (8th ed.). John Wiley and Sons, ISBN 978-1-118-14692-7.
#' @examples
#' fiber.lm <- lm(strength ~ diameter + machine, data=fiber)
#' ref_grid(fiber.lm)
#' 
#' # Covariate-adjusted means and comparisons
#' emmeans(fiber.lm, pairwise ~ machine)
#' 
"fiber"




### MOats ###
#' Oats data in multivariate form
#' 
#' This is the \code{Oats} dataset provided in the \pkg{nlme} package, but it is
#' rearranged as one multivariate observation per plot.
#' 
#' These data arise from a split-plot experiment reported by Yates (1935) and
#' used as an example in Pinheiro and Bates (2000) and other texts. Six blocks
#' were divided into three whole plots, randomly assigned to the three varieties
#' of oats. The whole plots were each divided into 4 split plots and randomized
#' to the four concentrations of nitrogen.
#' @format A data frame with 18 observations and 3 variables
#' \describe{
#'   \item{\code{Variety}}{a factor with levels \code{Golden Rain},
#'     \code{Marvellous}, \code{Victory}}
#'   \item{\code{Block}}{an ordered factor with levels \code{VI} < \code{V} <
#'     \code{III} < \code{IV} < \code{II} < \code{I}}
#'   \item{\code{yield}}{a matrix with 4 columns, giving the yields with
#'     nitrogen concentrations of 0, .2, .4, and .6.}
#' }
#' @source The dataset \code{\link[nlme]{Oats}} in the \pkg{nlme} package.
#' @references
#' Pinheiro, J. C. and Bates D. M. (2000) \emph{Mixed-Effects Models in S and
#' S-PLUS}, Springer, New York. (Appendix A.15)
#' 
#' Yates, F. (1935) Complex experiments, \emph{Journal of the Royal Statistical
#' Society} Suppl. 2, 181-247
#' @examples
#' MOats.lm <- lm (yield ~ Block + Variety, data = MOats)
#' MOats.rg <- ref_grid (MOats.lm, mult.name = "nitro")
#' emmeans(MOats.rg, ~ nitro | Variety)
"MOats"



### neuralgia ###
#' Neuralgia data
#' 
#' These data arise from a study of analgesic effects of treatments of elderly
#' patients who have neuralgia. Two treatments and a placebo are compared. The 
#' response variable is whether the patient reported pain or not. Researchers
#' recorded the age and gender of 60 patients along with the duration of
#' complaint before the treatment began.
#' 
#' @format A data frame with 60 observations and 5 variables:
#' \describe{
#'   \item{\code{Treatment}}{Factor with 3 levels \code{A}, \code{B}, and \code{P}.
#'     The latter is placebo}
#'   \item{\code{Sex}}{Factor with two levels \code{F} and \code{M}}
#'   \item{\code{Age}}{Numeric covariate -- patient's age in years}
#'   \item{\code{Duration}}{Numeric covariate -- duration of the condition before
#'     beginning treatment}
#'   \item{\code{Pain}}{Binary response factor with levels \code{No} and \code{Yes}}
#' }
#' @source Cai, Weijie (2014) \emph{Making Comparisons Fair: How LS-Means Unify 
#'   the Analysis of Linear Models}, SAS Institute, Inc. Technical paper 142-2014,
#'   page 12, 
#'   \url{http://support.sas.com/resources/papers/proceedings14/SAS060-2014.pdf}
#' @examples
#' # Model and analysis shown in the SAS report:
#' neuralgia.glm <- glm(Pain ~ Treatment * Sex + Age, family = binomial(),
#'    data = neuralgia) 
#' pairs(emmeans(neuralgia.glm, ~ Treatment, at = list(Sex = "F")), 
#'     reverse = TRUE, type = "response", adjust = "bonferroni")
#' 
"neuralgia"

### nutrition ###
#' Nutrition data
#' 
#' This observational dataset involves three factors, but where several factor 
#' combinations are missing. It is used as a case study in Milliken and Johnson,
#' Chapter 17, p.202. (You may also find it in the second edition, p.278.)
#' 
#' A survey was conducted by home economists ``to study how much
#' lower-socioeconomic-level mothers knew about nutrition and to judge the
#' effect of a training program designed to increase their knowledge of
#' nutrition.'' This is a messy dataset with several empty cells.
#' @format A data frame with 107 observations and 4 variables:
#' \describe{
#'   \item{\code{age}}{a factor with levels \code{1}, \code{2}, \code{3},
#'     \code{4}. Mother's age group.}
#'   \item{\code{group}}{a factor with levels \code{FoodStamps}, \code{NoAid}.
#'     Whether or not the family receives food stamp assistance.}
#'   \item{\code{race}}{a factor with levels \code{Black}, \code{Hispanic},
#'     \code{White}. Mother's race.}
#'   \item{\code{gain}}{a numeric vector (the response variable). Gain score
#'     (posttest minus pretest) on knowledge of nutrition.}
#' }
#' @source Milliken, G. A. and Johnson, D. E. (1984)
#' \emph{Analysis of Messy Data -- Volume I: Designed Experiments}. 
#' Van Nostrand, ISBN 0-534-02713-7.
#' @examples
#' nutr.aov <- aov(gain ~ (group + age + race)^2, data = nutrition)
#' 
#' # Summarize predictions for age group 3
#' nutr.emm <- emmeans(nutr.aov, ~ race * group, at = list(age="3"))
#'                    
#' emmip(nutr.emm, race ~ group)
#' 
#' # Hispanics seem exceptional; but this doesn't test out due to very sparse data
#' pairs(nutr.emm, by = "group")
#' pairs(nutr.emm, by = "race")
"nutrition"




### oranges ###
#' Sales of oranges
#' 
#' This example dataset on sales of oranges has two factors, two covariates, and
#' two responses. There is one observation per factor combination.
#' @format A data frame with 36 observations and 6 variables:
#' \describe{
#'   \item{\code{store}}{a factor with levels \code{1} \code{2} \code{3}
#'     \code{4} \code{5} \code{6}. The store that was observed.}
#'   \item{\code{day}}{a factor with levels \code{1} \code{2} \code{3}
#'     \code{4} \code{5} \code{6}. The day the observation was taken (same for
#'     each store).}
#'   \item{\code{price1}}{a numeric vector. Price of variety 1.}
#'   \item{\code{price2}}{a numeric vector. Price of variety 2.}
#'   \item{\code{sales1}}{a numeric vector. Sales (per customer) of variety 1.}
#'   \item{\code{sales2}}{a numeric vector. Sales (per customer) of variety 2.}
#' }
#' @source This is (or once was) available as a SAS sample dataset. 
#' @references
#' Littell, R., Stroup W., Freund, R. (2002) \emph{SAS For Linear Models} (4th
#' edition). SAS Institute. ISBN 1-59047-023-0.
#' @examples
#' # Example on p.244 of Littell et al.
#' oranges.lm <- lm(sales1 ~ price1*day, data = oranges)
#' emmeans(oranges.lm, "day")
#' 
#' # Example on p.246 of Littell et al.
#' emmeans(oranges.lm, "day", at = list(price1 = 0))
#' 
#' # A more sensible model to consider, IMHO (see vignette("interactions"))
#' org.mlm <- lm(cbind(sales1, sales2) ~ price1 * price2 + day + store, 
#'               data = oranges)
"oranges"




### pigs ###
#' Effects of dietary protein on free plasma leucine concentration in pigs
#' 
#' A two-factor experiment with some observations lost
#' 
#' @format A data frame with 29 observations and 3 variables:
#' \describe{
#'   \item{source}{Source of protein in the diet (factor with 3 levels: 
#'     fish meal, soybean meal, dried skim milk)}
#'   \item{percent}{Protein percentage in the diet (numeric with 4 values:
#'     9, 12, 15, and 18)}
#'   \item{conc}{Concentration of free plasma leucine, in mcg/ml}
#' }
#' @source Windels HF (1964) PhD thesis, Univ. of Minnesota. (Reported as
#'   Problem 10.8 in Oehlert G (2000) \emph{A First Course in Design and
#'   Analysis of Experiments}, licensed under Creative Commons,
#'   \url{http://users.stat.umn.edu/~gary/Book.html}.) Observations 7, 22, 23,
#'   31, 33, and 35 have been omitted, creating a more notable imbalance.
#' @examples
#'   pigs.lm <- lm(inverse(conc) ~ source + factor(percent), data = pigs)
#'   emmeans(pigs.lm, "source")
"pigs"

### ubds ###
#' Unbalanced dataset
#' 
#' This is a simulated unbalanced dataset with three factors
#' and two numeric variables. There are true relationships among these variables.
#' This dataset can be useful in testing or illustrating messy-data situations.
#' There are no missing data, and there is at least one observation for every 
#' factor combination; however, the \code{"cells"} attribute makes it simple
#' to construct subsets that have empty cells.
#' 
#' @format A data frame with 100 observations, 5 variables,
#'   and a special \code{"cells"} attribute:
#' \describe{
#'   \item{A}{Factor with levels 1, 2, and 3}
#'   \item{B}{Factor with levels 1, 2, and 3}
#'   \item{C}{Factor with levels 1, 2, and 3}
#'   \item{x}{A numeric variable}
#'   \item{y}{A numeric variable}
#' }
#' In addition, \code{attr(ubds, "cells")} consists of a named list of length 27 with the row numbers for
#' each combination of \code{A, B, C}. For example, 
#' \code{attr(ubds, "cells")[["213"]]} has the row numbers corresponding
#' to levels \code{A == 2, B == 1, C == 3}. The entries are ordered by
#' length, so the first entry is the cell with the lowest frequency.
#' @examples
#'  # Omit the three lowest-frequency cells
#'  low3 <- unlist(attr(ubds, "cells")[1:3]) 
#'  messy.lm <- lm(y ~ (x + A + B + C)^3, data = ubds, subset = -low3)
#'   

"ubds"

##############################################################################
#    Copyright (c) 2012-2020 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

# lqmm and lqm support

#' @exportS3Method recover_data lqmm
recover_data.lqmm = function(object, data = object$mfArgs$data, ...) {
    fcall = object$call
    trms = delete.response(terms(eval(fcall$fixed)))
    recover_data(fcall, trms, object$mfArgs$na.action, data = data, ...)
}

#' @exportS3Method emm_basis lqmm         
emm_basis.lqmm = function(object, trms, xlev, grid, tau = 0.5, ...) {
    taudiff = abs(object$tau - tau)
    col = which(taudiff < 0.0001)
    if (length(col) == 0)
        stop("No coefficients available for tau = ", tau)
    bhat = coef(object)
    # Very touchy here because their boot() function doesn't take dots...
    nm = intersect(names(list(...)), c("method", "R", "seed", "startQR"))
    vargs = c(list(object = object, covariance = TRUE), list(...)[nm])
    V = do.call("summary", vargs)$Cov[names(bhat), names(bhat)]
    if (length(taudiff) > 1) {
        bhat = bhat[, col[1]]
        V = V[, , col]
    }
    m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
    X = model.matrix(trms, m, contrasts.arg = object$contrasts)
    nbasis = estimability::all.estble
    dfargs = list(df = object$rdf)
    dffun = function(k, dfargs) dfargs$df
    list(X = X, bhat = bhat, nbasis = nbasis, V = V, 
         dffun = dffun, dfargs = dfargs, misc = list())
}


# Use same functions for lqm objects
#' @exportS3Method recover_data lqm
recover_data.lqm = function(object, ...) {
    recover_data.lm(object, frame = NULL, ...)
}

#' @exportS3Method emm_basis lqm          
emm_basis.lqm = function(object, ...)
    emm_basis.lqmm(object, ...)



#### rq objects (quantreg)

#' @exportS3Method recover_data rq
recover_data.rq = function(object, ...) {
    recover_data.lm(object, frame = object$model, ...)
}
    
#' @exportS3Method emm_basis rq           
emm_basis.rq = function(object, trms, xlev, grid, tau = object$tau, ...) {
    taucols = sapply(tau, \(t) {
        w = which(abs(object$tau - t) < 0.0001)
        ifelse(length(w) == 1, w, NA) })
    tau = tau[!is.na(taucols)]
    taucols = taucols[!is.na(taucols)]
    if (length(taucols) == 0)
        stop("No valid 'tau' values were specified")
    bhat = object$coefficients
    summ = summary(object, covariance = TRUE, ...)
    nm = if(is.null(names(bhat))) row.names(bhat) else names(bhat)
    m = suppressWarnings(model.frame(trms, grid, na.action = na.pass, xlev = xlev))
    X = model.matrix(trms, m, contrasts.arg = object$contrasts)
    assign = attr(X, "assign")
    X = X[, nm, drop = FALSE]
    if(is.matrix(bhat)) {
        bhat = bhat[, taucols, drop = FALSE]
        k = ncol(bhat)
        X = kronecker(diag(k), X)
        p = nrow(bhat)
        V = matrix(NA, nrow = length(bhat), ncol = length(bhat))
        for(j in 1:k) {
            jj = seq_len(p) + p*(j - 1)
            V[jj, jj] = summ[[taucols[j]]] $ cov
        }
        df = summ[[1]] $ rdf
    }
    else {
        V = summ$cov
        df = summ$rdf
    }
    bhat = as.numeric(bhat)
    nbasis = estimability::all.estble
    misc = list(ylevs = list(tau = tau))
    dfargs = list(df = df)
    dffun = function(k, dfargs) dfargs$df
    list(X = X, bhat = bhat, nbasis = nbasis, V = V, 
         dffun = dffun, dfargs = dfargs, misc = misc)
}

# we just reroute rqs objects to emm_basis.rq, as pretty similar
#' @exportS3Method recover_data rqs
recover_data.rqs = recover_data.rq

#' @exportS3Method emm_basis rqs          
emm_basis.rqs = function(object, ...)
    emm_basis.rq(object, ...)

##############################################################################
#    Copyright (c) 2012-2016 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

# emmeans support for aovlist objects

#' @export
recover_data.aovlist = function(object, ...) {
    fcall = match.call(aov, attr(object, "call")) # matches even if called via `manova()`
    trms = terms(object)
    # Find the Error terms
    lbls = attr(trms, "term.labels")
    err.idx = grep("^Error\\(", lbls)
    newf = as.formula(paste(c(".~.", lbls[err.idx]), collapse = "-"))
    trms = terms(update(trms, newf))
    dat = recover_data(fcall, delete.response(trms), na.action = attr(object, "na.action"), ...)
    attr(dat, "pass.it.on") = TRUE
    dat
}

# This works great for balanced experiments, and goes horribly wrong
# even for slightly unbalanced ones. So I abort on these kinds of cases
#' @export
emm_basis.aovlist = function (object, trms, xlev, grid, vcov., ...) {
    m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
    contr = attr(object, "contrasts")
    non.orth = sapply(contr, function(x) {
        !is.character(x) || !(x %in% c("contr.helmert", "contr.poly", "contr.sum"))
    })
    if (any(non.orth)) { # refit with sum-to-zero contrasts
        contr[non.orth] = "contr.sum"
        message("Note: re-fitting model with sum-to-zero contrasts")
        cl = attr(object, "call")
        cl$contrasts = contr
        object = eval(cl)
    }
    X = model.matrix(trms, m, contrasts.arg = contr)
    xnms = dimnames(X)[[2]]
    
    # Check for situations we can't handle...
    colsums = apply(X[, setdiff(xnms, "(Intercept)"), drop=FALSE], 2, sum)
    if (any(round(colsums,3) != 0))
        warning("Some predictors are correlated with the intercept - results may be very biased")
    if (length(unlist(lapply(object, function(x) names(coef(x))))) > length(xnms))
        message("NOTE: Results are based on intra-block estimates and are biased.")
    
    # initialize arrays
    nonint = setdiff(names(object), "(Intercept)")
    
    k = npar = length(xnms)
    bhat1 = rep(NA, k) # I'll use NAs in 1st dim of bhat to track which slots I've filled
    # check for multivariate response
    m = ifelse (is.matrix(coefm <- object[[1]]$coefficients), ncol(coefm), 1)
    bhat = rep(NA, k*m)
    zmm1 = seq_len(m) - 1   # seq 0 : (m-1)
    
    
    #### utility functions...
    # return indices of as.numeric(x[i, ]) where x has nr rows
    indx = function(i, nr = npar) 
        as.numeric(sapply(zmm1, function(j) nr * j + i))
    
    # get names or rownames
    mynames = function(x)
        if (m == 1) names(x) else rownames(x)
    
    V = matrix(0, nrow = k*m, ncol = k*m)
    names(bhat1) = xnms
    allxnms = xnms
    if (m > 1) {
        ylevs = colnames(coefm)
        allxnms = as.character(sapply(ylevs, function(.) paste0(xnms, .)))
    }
    names(bhat) = allxnms
    dimnames(V) = list(allxnms, allxnms)
    empty.list = as.list(nonint)
    names(empty.list) = nonint
    Vmats = Vidx = Vdf = empty.list
    wts = matrix(0, nrow = length(nonint), ncol = k*m)
    dimnames(wts) = list(nonint, allxnms)
    # NOTE: At present, I just do intra-block analysis: wts are all 0 and 1
    btemp = bhat1 #++ temp for tracking indexes
    #++Work thru strata in reverse order
    for (nm in rev(nonint)) {
        x = object[[nm]]
        if (m > 1) class(x) = c("mlm", "lm")   # because vcov.aov is NOT suitable
        bi = coef(x)
        rn = mynames(bi)
        nr = length(rn)
        idx = which(!is.na(bi[seq_along(rn)]))
        bi = bi[indx(idx, nr)]
        ii = match(rn[idx], xnms)
        use = setdiff(ii, which(!is.na(bhat1))) #++ omit elts already filled
        if(length(use) > 0) {
            ii.left = seq_along(ii)[!is.na(match(ii,use))]
            wts[nm, indx(use)] = 1
            bhat1[use] = bi[ii.left]
            allii.left = indx(ii.left, nr)
            alluse = Vidx[[nm]] = indx(use)
            bhat[alluse] = bi[allii.left]
            # following is OK now that we have class(x) = "mlm"
            Vi = vcov(x, complete = FALSE)[allii.left, allii.left, drop = FALSE]
            Vmats[[nm]] = Vi
            V[alluse, alluse] = Vi
        }
        else {
            Vmats[[nm]] = matrix(0, nrow=0, ncol=0)
            Vidx[[nm]] = integer(0)
        }
        # Any cases with 0 df will have NaN for covariances. I make df = -1 
        # in those cases so I don't divide by 0 later in Satterthwaite calcs
        Vdf[[nm]] = ifelse(x$df > 0, x$df, -1)
    }
    
    x <- object[["(Intercept)"]]
    if (!is.null(x)) {
        # The intercept belongs in the 1st error stratum
        # So have to add a row and column to its covariance matrix
        idx1 = indx(1)
        bhat[idx1] = as.numeric(coef(x))
        wts[1, idx1] = 1
        x = object[[nonint[1]]] # 1st non-intercept stratum
        Vidx[[1]] = ii = sort(c(idx1, Vidx[[1]]))
        k = length(ii)
        vv = matrix(0, nrow = k, ncol = k)
        i2k = indx(2:(k / m), k / m)
        if (k > m) vv[i2k, i2k] = Vmats[[1]]
        # Variance of intercept is EMS of this stratum divided by N
        # Here I'm assuming there are no weights
        N = sum(sapply(object, function(x) length(x$residuals))) / m
        i1 = indx(1, k / m)
        if (m > 1)
            V[idx1, idx1] = vv[i1,i1] = estVar(x) / N
        else
            V[1,1] = vv[1,1] = sum(resid(x)^2) / x$df / N
        #dimnames(vv) = list(c(xnms[ii], xnms[ii]))
        Vmats[[1]] = vv
    }
    # override V if vcov. is supplied
    if(!missing(vcov.)) {
        V = .my.vcov(object, vcov.)
        dfargs = list()
        dffun = function(k, dfargs) Inf
    }
    else {
        dfargs = list(Vmats=Vmats, Vidx=Vidx, Vdf=unlist(Vdf), wts = wts)
        dffun = function(k, dfargs) {
            emmeans::.aovlist.dffun(k, dfargs)
        }
    }
    nbasis = estimability::all.estble  # Consider this further?
    misc = list(initMesg = "Warning: EMMs are biased unless design is perfectly balanced")
    if (m > 1) {
        misc$ylevs = list(rep.meas = ylevs)
        X = kronecker(diag(1, m), X)
    }
    
    # submodel support
    mm = NULL
    if(!is.null(dat <- attr(object, "data"))) {
        m = model.frame(trms, dat, na.action = na.pass, xlev = xlev)
        mm = model.matrix(trms, m, contrasts.arg = contr)
        mm = .cmpMM(mm, assign = attr(mm, "assign"))
    }
    
    list(X = X, bhat = bhat, nbasis = nbasis, V = V, dffun = dffun, 
         dfargs = dfargs, misc = misc, model.matrix = mm)
}

#' @rdname extending-emmeans
#' @order 32
#' @param k,dfargs Arguments to \code{.aovlist.dffun}, which is made available as a 
#'   convenience to developers providing support similar to that provided for 
#'   \code{aovlist} objects
#' @export
.aovlist.dffun = function(k, dfargs) {
    if(is.matrix(k) && (nrow(k) > 1)) {
        dfs = apply(k, 1, .aovlist.dffun, dfargs)
        min(dfs)
    }
    else {
        v = sapply(seq_along(dfargs$Vdf), function(j) {
            ii = dfargs$Vidx[[j]]
            kk = (k * dfargs$wts[j, ])[ii]            
            #sum(kk * .mat.times.vec(dfargs$Vmats[[j]], kk))
            .qf.non0(dfargs$Vmats[[j]], kk)
        })
        sum(v)^2 / sum(v^2 / dfargs$Vdf) # Good ole Satterthwaite
    }
}

##############################################################################
#    Copyright (c) 2012-2017 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

### Wrappers for those who want other (familiar) terminology: lsmeans 
                                                    #xxx  and pmmeans xxx#

### general-purpose wrapper for creating pmxxxxx and lsxxxxx functions
### use subst arg to specify e.g. "ls" #xxx or "pm"
.emwrap = function(emmfcn, subst, ...) {
    result = emmfcn(...)

        if (inherits(result, "emmGrid"))
        result = .sub.em(result, subst)
    else if(inherits(result, "emm_list")) {
        for (i in seq_along(result))
            result[[i]] = .sub.em(result[[i]], subst)
        names(result) = gsub("^em", subst, names(result))
    }
    result
}

# returns an updated emmGrid object with estName "em..." replaced by "xx..."
.sub.em = function(object, subst) {
    nm = object@misc$estName
    update(object, estName = gsub("^em", subst, nm))
}



### Exported implementations

# lsmeans family
#' Wrappers for alternative naming of EMMs
#' 
#' These are wrappers for \code{\link{emmeans}} and related functions to provide
#' backward compatibility, or for users who may prefer to
#' use other terminology than \dQuote{estimated marginal means} -- namely 
#' \dQuote{least-squares means}. These functions also provide the functionality
#' formerly provided by the \pkg{lsmeans} package, which is now just a front-end
#' for \pkg{emmeans}.
#' 
#' For each function with \code{ls}\emph{xxxx} in its name,
#' the same function named \code{em}\emph{xxxx} is called. Any estimator names or 
#' list items beginning with \dQuote{em} are replaced with \dQuote{ls} 
#' before the results are returned
#' 
#' @param ... Arguments passed to the corresponding \code{em}\emph{xxxx} function
#' 
#' @return The result of the call to \code{em}\emph{xxxx}, suitably modified.
#' @rdname wrappers
#' @aliases wrappers
#' @seealso \code{\link{emmeans}}, \code{\link{emtrends}}, \code{\link{emmip}},
#'          \code{\link{emm}}, \code{\link{emmobj}}, \code{\link{emm_options}},
#'          \code{\link{get_emm_option}}
#' @export
#' @examples
#' pigs.lm <- lm(log(conc) ~ source + factor(percent), data = pigs)
#' lsmeans(pigs.lm, "source")
lsmeans = function(...)
    .emwrap(emmeans, subst = "ls", ...)


#' @rdname wrappers
#' @export
lstrends = function(...)
    .emwrap(emtrends, subst = "ls", ...)


#' @rdname wrappers
#' @export
lsmip = function(...)
    emmip(...)


#' @rdname wrappers
#' @export
lsm = function(...)
    emm(...)


#' @rdname wrappers
#' @export
lsmobj = function(...)
    .emwrap(emmobj, subst = "ls", ...)

#' @rdname wrappers
#' @export
lsm.options = function(...) {
    .Deprecated("emm_options")
    args = list(...)
    nms = names(args)
    nms = gsub("ref.grid", "ref_grid", nms)
    nms = gsub("lsmeans", "emmeans", nms)
    names(args) = nms
    do.call(emm_options, args)
}

#' @rdname wrappers
#' @param x Character name of desired option
#' @param default default value to return if \code{x} not found
#' 
#' @return \code{get.lsm.option} and \code{lsm.options} remap options from
#'   and to corresponding options in the \pkg{emmeans} options system.
#' @export
get.lsm.option = function(x, default = emm_defaults[[x]]) {
    .Deprecated("get_emm_option")
    if(x == "ref.grid") x = "ref_grid"
    if(x == "lsmeans") x = "emmeans"
    get_emm_option(x, default = default)
}
    

##############################################################################
#    Copyright (c) 2012-2025 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

# rbind method for emmGrid objects
# ... and some other methods

#' Combine or subset \code{emmGrid} objects
#'
#' These functions provide methods for \code{\link[base:cbind]{rbind}} and
#' \code{\link[base:Extract]{[}} that may be used to combine \code{emmGrid} objects
#' together, or to extract a subset of cases. The primary reason for 
#' doing this would be to obtain multiplicity-adjusted results for smaller
#' or larger families of tests or confidence intervals. 
#' 
#' @param ... In \code{rbind}, object(s) of class \code{emmGrid} or \code{summary_emm}.
#'   In others, additional arguments passed to other methods
#' @param deparse.level (required but not used)
#' @param adjust Character value passed to \code{\link{update.emmGrid}}
#' 
#' @note \code{rbind} throws an error if there are incompatibilities in
#'   the objects' coefficients, covariance structures, etc. But they 
#'   are allowed to have different factors; a missing level \code{'.'}
#'   is added to factors as needed.
#' 
#' @note These functions generally reset \code{by.vars} to \code{NULL};
#' so if you want to keep any \dQuote{by} variables, you should follow-up
#' with \code{\link{update.emmGrid}}.
#'
#' @return A revised object of class \code{emmGrid}
#' @order 1
#' @method rbind emmGrid
#' @export
#' @examples
#' warp.lm <- lm(breaks ~ wool * tension, data = warpbreaks)
#' warp.rg <- ref_grid(warp.lm)
#' 
#' # Do all pairwise comparisons within rows or within columns, 
#' # all considered as one faily of tests:
#' w.t <- pairs(emmeans(warp.rg, ~ wool | tension))
#' t.w <- pairs(emmeans(warp.rg, ~ tension | wool))
#' rbind(w.t, t.w, adjust = "mvt")
#' update(w.t + t.w, adjust = "fdr")  ## same as above except for adjustment
rbind.emmGrid = function(..., deparse.level = 1, adjust = "bonferroni") {
    objs = list(...)
    if (!all(sapply(objs, inherits, "emmGrid")))
        stop("All objects must inherit from 'emmGrid'")
    bhats = lapply(objs, function(o) o@bhat)
    bhat = bhats[[1]]
    if(!all(sapply(bhats, function(b) (length(b) == length(bhat)) 
                   && (sum((b - bhat)^2, na.rm = TRUE) == 0))))
        stop("All objects must have the same fixed effects")
    Vs = lapply(objs, function(o) o@V)
    V = Vs[[1]]
    if(!all(sapply(Vs, function(v) sum((v - V)^2) == 0)))
        stop("All objects must have the same covariances")
    obj = objs[[1]]
    linfcts = lapply(objs, function(o) o@linfct)
    obj@linfct = do.call(rbind, linfcts)
    bnms = unlist(lapply(objs, function(o) o@misc$by.vars))
    grids = lapply(objs, function(o) o@grid)
    gnms = unique(c(bnms, unlist(lapply(grids, names))))
    gnms = setdiff(gnms, c(".wgt.", ".offset.")) # exclude special names
    grid = data.frame(.tmp. = seq_len(n <- nrow(obj@linfct)))
    for (g in gnms)
        grid[[g]] = rep(".", n)
    grid[[".wgt."]] = grid[[".offset."]] = 0
    grid$.tmp. = NULL
    n.before = 0
    for (g in grids) {
        rows = n.before + seq_along(g[[1]])
        n.before = max(rows)
        for (nm in setdiff(names(g), c(".wgt.", ".offset.")))
            grid[rows, nm] = as.character(g[[nm]])
        if (!is.null(g$.wgt.)) grid[rows, ".wgt."] = g$.wgt.
        if (!is.null(g$.offset.)) grid[rows, ".offset."] = g$.offset.
    }
    if (all(grid$.wgt. == 0)) 
        grid$.wgt. = 1
    if (all(grid$.offset. == 0)) 
        grid$.offset. = NULL
    avgd.over = unique(unlist(lapply(objs, function(o) o@misc$avgd.over)))
    attr(avgd.over, "qualifier") = " some or all of"
    obj@grid = grid
    obj@levels = lapply(gnms, function(nm) unique(grid[[nm]]))
    names(obj@levels) = gnms
    obj@roles$predictors = setdiff(names(obj@levels), obj@roles$multresp)
    obj@misc$con.coef = obj@misc$orig.grid = obj@misc$.pairby = NULL
    update(obj, pri.vars = gnms, by.vars = NULL, adjust = adjust, initMesg = NULL,
           estType = "rbind",
           famSize = n,   # instead of round((1 + sqrt(1 + 8*n)) / 2, 3),
           avgd.over = avgd.over)
}
#' @rdname rbind.emmGrid
#' @order 4
#' @param e1,e2,x,object Objects of class \code{emmGrid}
#' @return The result of \code{e1 + e2} is the same as \code{rbind(e1, e2)}
#' @method + emmGrid
#' @export
"+.emmGrid" = function(e1, e2) {
    if(!is(e2, "emmGrid"))
        stop("'+.emmGrid' works only when all objects are class `emmGrid`", call. = FALSE)
    rbind(e1, e2)
}


### Subset a reference grid
# if drop = TRUE, the levels of factors are reduced
#' @rdname rbind.emmGrid
#' @order 5
#' @param i Integer vector of indexes
#' @param drop.levels Logical value. If \code{TRUE}, the \code{"levels"} slot in
#'   the returned object is updated to hold only the predictor levels that actually occur
#'   
#' @method [ emmGrid
#' @export
#' @examples
#'
#' # Show only 3 of the 6 cases
#' summary(warp.rg[c(2, 4, 5)])
"[.emmGrid" = function(x, i, adjust, drop.levels = TRUE, ...) {
    x@linfct = x@linfct[i, , drop = FALSE]
    x@grid = x@grid[i, , drop = FALSE]                  
    x = update(x, pri.vars = names(x@grid), famSize = length(i), estType = "[")
    x@misc$orig.grid = x@misc$con.coef = x@misc$.pairby = NULL
    x@misc$by.vars = NULL
    if(!missing(adjust))
        x@misc$adjust = adjust
    if(!is.null(disp <- x@misc$display))
        x@misc$display = disp[i]
    if (drop.levels) {
        for (nm in names(x@levels))
            x@levels[[nm]] = unique(x@grid[[nm]])
    }
    x
}

#' @rdname rbind.emmGrid
#' @order 9
#' @param subset logical expression indicating which rows of the grid to keep
#' @method subset emmGrid
#' @export
#' @examples
#' 
#' # After-the-fact 'at' specification
#' subset(warp.rg, wool == "A")  ## or warp.rg |> subset(wool == "A")
#' 
subset.emmGrid = function(x, subset, ...) {
    sel = eval(substitute(subset), envir = x@grid, enclos = parent.frame())
    x[sel, ...]
}

#' @rdname rbind.emmGrid
#' @order 6
#' @param n integer number of entries to include (or exclude if negative)
#' @importFrom utils head
#' @method head emmGrid
#' @export
head.emmGrid = function(x, n = 6, ...) {
    s = sign(n[1])
    n = min(abs(n[1]), nrow(x@grid))
    x[s * seq_len(n)]
}

#' @rdname rbind.emmGrid
#' @order 7
#' @importFrom utils tail
#' @method tail emmGrid
#' @export
tail.emmGrid = function(x, n = 6, ...) {
    s = sign(n[1])
    N = nrow(x@grid)
    n = min(abs(n[1]), N)
    x[s * (seq_len(n) - n + N)]
}

##############################################################################
#    Copyright (c) 2012-2022 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################


### =========== Various methods for emmGrid class =============================
# (note: some major ones have their own file)



### S4 show method
## use S3 for this setMethod("summary", "emmGrid", summary.emmGrid)
setMethod("show", "emmGrid", function(object) {
    isnewrg = object@misc$is.new.rg
    if (is.null(isnewrg)) 
        isnewrg = FALSE
    
    if (isnewrg)
        str.emmGrid(object)
    else
        print(summary.emmGrid(object))
})


### Others are all S3 methods

#' @rdname emmGrid-methods
#' @method str emmGrid
#' @export
str.emmGrid <- function(object, ...) {
    showlevs = function(x) { # internal convenience function
        if (is.null(x)) cat("(predicted by other variables)")
        else {
            if (length(x) > 9) x = c(x[1:9], "...", x[length(x)])
            cat(paste(format(x, digits = 5, justify = "none"), collapse=", "))
        }
    }
    showtran = function(misc, label) { # internal convenience fcn
        cat(paste(label, dQuote(.fmt.tran(misc)), "\n"))
    }
    levs = object@levels
    cat(paste("'", class(object)[1], "' object with variables:\n", sep=""))
    for (nm in union(object@roles$predictors, union(object@roles$multresp, object@roles$responses))) {
        cat(paste("    ", nm, " = ", sep = ""))
        if (hasName(object@matlevs, nm)) {
            if (nm %in% object@roles$responses)
                cat("multivariate response with means: ")
            else
                cat("matrix with column reference values: ")
            cat("\n        ")
            showlevs(object@matlevs[[nm]])
        }
        else if (nm %in% object@roles$multresp) {
            cat("multivariate response levels: ")
            showlevs(levs[[nm]])
        }
        else if (nm %in% object@roles$responses) {
            cat("response variable with mean ")
            showlevs(levs[[nm]])
        }
        else
            showlevs(levs[[nm]])
        cat("\n")
    }
    if (length(nuis <- object@roles$nuisance) > 0) {
        cat("Nuisance factors that have been collapsed by averaging:\n    ")
        tmp = paste0(names(nuis), "(", sapply(nuis, length), ")")
        cat(paste(tmp, collapse = ", "))
        cat("\n")
    }
    if (length(cf <- object@roles$counterfactuals) > 0) {
        cat("Counterfactuals\n    ")
        cat(paste(cf, collapse = ", "))
        cat("\n")
    }
    if(!is.null(object@model.info$nesting)) {
        cat("Nesting structure:  ")
        cat(.fmt.nest(object@model.info$nesting))
        cat("\n")
    }
    if(!is.null(tran <- object@misc$tran)) {
        showtran(object@misc, "Transformation:")
        if (!is.null(tran2 <- object@misc$tran2))
            showtran(list(tran = tran2), "Additional response transformation:")
    }
    if(!is.na(object@nbasis[1])) {
        cat(paste0("Some things are non-estimable (null space dim = ",
                  ncol(object@nbasis), ")\n"))
    }
}


#' @rdname emmGrid-methods
#' @param export Logical value. If \code{FALSE}, the object is printed.
#'   If \code{TRUE}, a list is invisibly returned, which contains character
#'   elements named \code{summary} and \code{annotations} that may be saved
#'   or displayed as the user sees fit. \code{summary} is a character matrix
#'   (or list of such matrices, if a \code{by} variable is in effect).
#'   \code{annotations} is a character vector of the annotations that would
#'   have been printed below the summary or summaries.
#' @method print emmGrid
#' @param x An \code{emmGrid} object
#' @export
print.emmGrid = function(x, ..., export = FALSE) {
    print(summary.emmGrid(x, ...), export = export)
    # if(export)
    #     print(summary.emmGrid(x, ...), export = export)
    # else
    #     show(x)
}

### Former print method which I changed to work around a bug in pkgdown
# print.emmGrid = function(x, ..., export = FALSE)
#     print(summary.emmGrid(x, ...), export = export)

# vcov method
#' Miscellaneous methods for \code{emmGrid} objects
#' @rdname emmGrid-methods
#' 
#' @param object An \code{emmGrid} object
#' @param ... (required but not used)
#' @param sep separator for pasting levels in creating row and column
#'        names for \code{vcov()} results
#' 
#' @return The \code{vcov} method returns a symmetric matrix of variances and
#'   covariances for \code{predict.emmGrid(object, type = "lp")}
#'
#' @method vcov emmGrid
#' @export
#' @examples
#' warp.lm <- lm(breaks ~ wool * tension, data = warpbreaks)
#' warp.emm <- emmeans(warp.lm, ~ tension | wool)
#' vcov(warp.emm) |> zapsmall()
#' 
#' vcov(pairs(warp.emm), sep = "|") |> zapsmall()
vcov.emmGrid = function(object, ..., sep = get_emm_option("sep")) {
    tol = get_emm_option("estble.tol")
    if (!is.null(hook <- object@misc$vcovHook)) {
        if (is.character(hook)) 
            hook = get(hook)
        hook(object, tol = tol, ...)
    }
    else {
        if(is.null(disp <- object@misc$display))
            disp = seq_len(nrow(object@linfct))
        X = object@linfct[disp, , drop = FALSE]
        estble = estimability::is.estble(X, object@nbasis, tol) 
        X[!estble, ] = NA
        X = X[, !is.na(object@bhat), drop = FALSE]
        rtn = X %*% tcrossprod(object@V, X)
        largs = as.list(object@grid[disp, seq_along(object@levels), drop = FALSE])
        largs$sep = sep
        rownames(rtn) = colnames(rtn) = do.call(paste, largs)
        rtn
    }
}


#' @rdname emmGrid-methods
#'
#' @returns The \code{linfct} function and method returns the \code{linfct} slot of \code{object}.
#' @export
linfct = function(object, ...)
    UseMethod("linfct")

#' @rdname emmGrid-methods
#' @export
linfct.default = function(object, ...) {
    attr(object, "linfct")
}


# Method to alter contents of misc slot

#' Update an \code{emmGrid} object
#' 
#' Objects of class \code{emmGrid} contain several settings that affect such things as
#' what arguments to pass to \code{\link{summary.emmGrid}}. 
#' The \code{update} method allows safer management of these settings than
#' by direct modification of its slots.
#'
#' @param object An \code{emmGrid} object
#' @param ... Options to be set. These must match a list of known options (see
#'   Details)
#' @param silent Logical value. If \code{FALSE} (the default), a message is
#'   displayed if any options are not matched. If \code{TRUE}, no messages are
#'   shown.
#'
#' @return an updated \code{emmGrid} object.
#' 
#' @method update emmGrid
#' @order 1
#' @export
#' 
#' @section Details:
#' The names in \code{\dots} are partially matched against those that are valid, and if a match is found, it adds or replaces the current setting. The valid names are
#' 
#' \describe{
#' \item{\code{tran}, \code{tran2}}{(\code{list} or \code{character}) specifies
#' the transformation which, when inverted, determines the results displayed by
#' \code{\link{summary.emmGrid}}, \code{\link{predict.emmGrid}}, or \code{\link{emmip}} when
#' \code{type="response"}. The value may be the name of a standard
#' transformation from \code{\link{make.link}} or additional ones supported by
#' name, such as \code{"log2"}; or, for a custom transformation, a \code{list}
#' containing at least the functions \code{linkinv} (the inverse of the
#' transformation) and \code{mu.eta} (the derivative thereof). The
#' \code{\link{make.tran}} function returns such lists for a number of popular
#' transformations. See the help page of \code{\link{make.tran}} for details as
#' well as information on the additional named transformations that are
#' supported. \code{tran2} is just like \code{tran} except it is a second
#' transformation (i.e., a response transformation in a generalized linear
#' model).}
#' 
#' \item{\code{tran.mult}}{Multiple for \code{tran}. For example, for the
#' response transformation \samp{2*sqrt(y)} (or \samp{sqrt(y) + sqrt(y + 1)},
#' for that matter), we should have \code{tran = "sqrt"} and \code{tran.mult =
#' 2}. If absent, a multiple of 1 is assumed.}
#' 
#' \item{\code{tran.offset}}{Additive constant before a transformation is applied.
#' For example, a response transformation of \code{log(y + pi)} has
#' \code{tran.offset  = pi}. If no value is present, an offset of 0 is assumed.}
#' 
#' \item{\code{estName}}{(\code{character}) is the column label used for
#' displaying predictions or EMMs.}
#' 
#' \item{\code{inv.lbl}}{(\code{character)}) is the column label to use for
#' predictions or EMMs when \code{type="response"}.}
#' 
#' \item{\code{by.vars}}{(\code{character)} vector or \code{NULL}) the variables
#' used for grouping in the summary, and also for defining subfamilies in a call
#' to \code{\link{contrast}}.}
#' 
#' \item{\code{pri.vars}}{(\code{character} vector) are the names of the grid
#' variables that are not in \code{by.vars}. Thus, the combinations of their
#' levels are used as columns in each table produced by \code{\link{summary.emmGrid}}.}
#' 
#' \item{\code{alpha}}{(numeric) is the default significance level for tests, in
#' \code{\link{summary.emmGrid}} as well as \code{\link{plot.emmGrid}}
#' when \samp{CIs = TRUE}. Be cautious that methods that depend on
#' specifying \code{alpha} are prone to abuse. See the
#' discussion in \href{../doc/basics.html#pvalues}{\code{vignette("basics", "emmeans")}}.}
#' 
#' \item{\code{adjust}}{(\code{character)}) is the default for the \code{adjust}
#' argument in \code{\link{summary.emmGrid}}.}
#' 
#' \item{\code{cross.adjust}}{(\code{character)}) is the default for the \code{cross.adjust}
#' argument in \code{\link{summary.emmGrid}} (used for adjusting between groups).}
#' 
#' \item{\code{famSize}}{(integer) is the number of means involved in a family of
#' inferences; used in Tukey adjustment}
#' 
#' \item{\code{infer}}{(\code{logical} vector of length 2) is the default value
#' of \code{infer} in \code{\link{summary.emmGrid}}.}
#' 
#' \item{\code{level}}{(numeric) is the default confidence level, \code{level},
#' in \code{\link{summary.emmGrid}}. \emph{Note:} You must specify all five letters 
#' of \sQuote{level} to distinguish it from the slot name \sQuote{levels}.}
#' 
#' \item{\code{df}}{(numeric) overrides the default degrees of freedom with a
#' specified single value.}
#' 
#' \item{\code{calc}}{(list) additional calculated columns. See \code{\link{summary.emmGrid}}.}
#' 
#' \item{\code{null}}{(numeric) null hypothesis for \code{summary} or
#' \code{test} (taken to be zero if missing).}
#' 
#' \item{\code{side}}{(numeric or character) \code{side} specification for for
#' \code{summary} or \code{test} (taken to be zero if missing).}
#' 
#' \item{\code{sigma}}{(numeric) Error SD to use in predictions and for bias-adjusted
#' back-transformations}
#' 
#' \item{\code{delta}}{(numeric) \code{delta} specification for \code{summary}
#' or \code{test} (taken to be zero if missing).}
#' 
#' \item{\code{predict.type} or \code{type}}{(character) sets the default method
#' of displaying predictions in \code{\link{summary.emmGrid}},
#' \code{\link{predict.emmGrid}}, and \code{\link{emmip}}. Valid values are
#' \code{"link"} (with synonyms \code{"lp"} and \code{"linear"}), or
#' \code{"response"}.}
#' 
#' \item{\code{bias.adjust}, \code{frequentist}}{(logical) These
#' are used by \code{summary} if the value of these arguments are not specified.}
#' 
#' \item{\code{estType}}{(\code{character}) is used internally to determine 
#' what \code{adjust} methods are appropriate. It should match one of 
#' \samp{c("prediction", "contrast", "pairs")}. As an example of why this is needed,
#' the Tukey adjustment should only be used for pairwise comparisons 
#' (\code{estType = "pairs"}); if \code{estType} is some other string, Tukey
#' adjustments are not allowed.}
#' 
#' \item{\code{avgd.over}}{(\code{character)} vector) are the names of the 
#' variables whose levels are averaged over in obtaining marginal averages of 
#' predictions, i.e., estimated marginal means. Changing this might produce a 
#' misleading printout, but setting it to \code{character(0)} will suppress the 
#' \dQuote{averaged over} message in the summary.}
#' 
#' \item{\code{initMesg}}{(\code{character}) is a string that is added to the
#' beginning of any annotations that appear below the \code{\link{summary.emmGrid}}
#' display.}
#' 
#' \item{\code{methDesc}}{(\code{character}) is a string that may be used for
#' creating names for a list of \code{emmGrid} objects. }
#' 
#' \item{\code{nesting}}{(Character or named \code{list}) specifies the nesting
#' structure. See \dQuote{Recovering or overriding model information} in the
#' documentation for \code{\link{ref_grid}}. The current nesting structure is
#' displayed by \code{\link{str.emmGrid}}.}
#'
#' \item{\code{levels}}{named \code{list} of new levels for the elements of the
#' current \code{emmGrid}. The list name(s) are used as new variable names, and
#' if needed, the list is expanded using \code{expand.grid}. These results replace
#' current variable names and levels. This specification changes the \code{levels},
#' \code{grid}, \code{roles}, and \code{misc} slots in the updated \code{emmGrid},
#' and resets \code{pri.vars}, \code{by.vars}, \code{adjust}, \code{famSize},
#' and \code{avgd.over}. In addition, if there is nesting of factors, that may be 
#' altered; a warning is issued if it involves something other than mere name changes.
#' \emph{Note:} All six letters of \code{levels} is needed in order to distinguish
#' it from \code{level}.}
#' 
#' \item{\code{submodel}}{\code{formula} or \code{character} value specifying a 
#' submodel (requires this feature being supported by underlying methods 
#' for the model class). When specified, the \code{linfct} slot is replaced by 
#' its aliases for the specified sub-model. Any factors in the sub-model that 
#' do not appear in the model matrix are ignored, as are any interactions that 
#' are not in the main model, and any factors associate with multivariate responses. 
#' The estimates displayed are then computed as if 
#' the sub-model had been fitted. (However, the standard errors will be based on the
#' error variance(s) of the full model.) 
#' \emph{Note:} The formula should refer only to predictor names, \emph{excluding} any
#' function calls (such as \code{factor} or \code{poly}) that appear in the 
#' original model formula. See the example.
#' 
#' The character values allowed should partially 
#' match \code{"minimal"} or \code{"type2"}. With \code{"minimal"}, the sub-model
#' is taken to be the one only involving the surviving factors in \code{object}
#' (the ones averaged over being omitted). Specifying \code{"type2"} is the same as
#' \code{"minimal"} except only the highest-order term in the submodel is retained,
#' and all effects not containing it are orthogonalized-out. Thus, in a purely linear
#' situation such as an \code{lm} model, the joint test
#' of the modified object is in essence a type-2 test as in \code{car::Anova}.
#' 
#' Please note that it is possible (or even likely) that there will be disparity
#' between the \code{grid} and \code{linfct} slots when a submodel is used. This is
#' because \code{grid} contains the \emph{claimed} values of the predictors and
#' \code{linfct} contains \emph{aliases} of them computed from the submodel.
#' 
#' For some objects such as generalized linear models, specifying \code{submodel}
#' will typically not produce the same estimates or type-2 tests as would be
#' obtained by actually fitting a separate model with those specifications.
#' The reason is that those models are fitted by iterative-reweighting methods,
#' whereas the \code{submodel} calculations preserve the final weights used in
#' fitting the full model.}
#' 
#' \item{(any other slot name)}{If the name matches an element of
#' \code{slotNames(object)} other than \code{levels}, that slot is replaced by 
#' the supplied value, if it is of the required class (otherwise an error occurs). 
#' 
#' The user must be very careful in
#' replacing slots because they are interrelated; for example, the lengths
#' and dimensions of \code{grid}, \code{linfct}, \code{bhat}, and \code{V} must
#' conform.}
#' } %%%%%%% end \describe 
#' 
#' @note
#' When it makes sense, an option set by \code{update} will persist into 
#' future results based on that object. But some options are disabled as well.
#' For example, a \code{calc} option will be nulled-out if \code{contrast}
#' is called, because it probably will not make sense to do the same 
#' calculations on the contrast results, and in fact the variable(s) needed
#' may not even still exist.
#' \code{factor(percent)}.
#'
#' @seealso \code{\link{emm_options}}
#' @examples
#' # Using an already-transformed response:
#' pigs.lm <- lm(log(conc) ~ source * factor(percent), data = pigs)
#' 
#' # Reference grid that knows about the transformation
#' # and asks to include the sample size in any summaries:
#' pigs.rg <- update(ref_grid(pigs.lm), tran = "log", 
#'                     predict.type = "response",
#'                     calc = c(n = ~.wgt.))
#' emmeans(pigs.rg, "source")
#' 
#' # Obtain estimates for the additive model
#' # [Note that the submodel refers to 'percent', not 'factor(percent)']
#' emmeans(pigs.rg, "source", submodel = ~ source + percent)
#' 
#' # Type II ANOVA
#' joint_tests(pigs.rg, submodel = "type2")
update.emmGrid = function(object, ..., silent = FALSE) {
    args = list(...)
    # see .valid.misc below this function for list of legal options
    valid.slots = slotNames(object)
    valid.choices = union(.valid.misc, valid.slots)
    misc = object@misc
    for (nm in names(args)) {
        fullname = try(match.arg(nm, valid.choices), silent=TRUE)
        if(inherits(fullname, "try-error")) {
            if (!silent)
                message("Argument ", sQuote(nm), " was ignored. Valid choices are:\n",
                        paste(valid.choices, collapse=", "))
        }
        else {
            if (fullname == "type") fullname = "predict.type"
            if (fullname == "levels") {
                lvls = args[[nm]]
                if (!is.list(lvls))
                    stop("'levels' must be a named list.")
                nm = names(lvls)
                if (is.null(nm) || any(nm == ""))
                     stop("'levels' must be a named list.")
                grd = do.call(expand.grid, lvls)
                if (nrow(object@grid) != nrow(grd))
                    stop("Length of replacement levels does not match the number of rows in the grid")
                oldlvls = object@levels
                if(!is.null(object@model.info$nesting) &&
                   ((length(oldlvls) != length(lvls)) ||
                    any(sapply(oldlvls, length) != sapply(lvls, length))))
                    warning("Changes to levels may have altered nesting structure.\n",
                            "You likely need to also run 'update(..., nesting = ...)'")
                object@levels = lvls
                for (nm in c(".wgt.", ".offset."))
                        grd[[nm]] = object@grid[[nm]]
                object@grid = grd
                object@roles$predictors = misc$pri.vars = names(lvls)
                misc$by.vars = misc$avgd.over = NULL
                if (!is.null(object@model.info$nesting))
                    object@model.info$nesting = .find_nests(grd, NULL, FALSE, lvls)
                misc$adjust = "none"
                misc$famSize = nrow(grd)
            }
            else if((fullname == "estName") && 
                    !is.null(misc$predict.type) && startsWith(misc$predict.type, "r"))
                misc$inv.lbl = args[[nm]]
            else if (fullname %in% valid.slots) # all slots but "levels"
                slot(object, fullname) = args[[nm]]
            else {
                if (fullname == "by.vars") {
                    allvars = union(misc$pri.vars, misc$by.vars)
                    misc$pri.vars = setdiff(allvars, args[[nm]])
                }
                if (fullname == "pri.vars") {
                    allvars = union(misc$pri.vars, misc$by.vars)
                    misc$by.vars = setdiff(allvars, args[[nm]])
                }
                # special case - I keep nesting in model.info. Plus add'l checks
                if (fullname == "nesting") {
                    object@model.info$nesting = lst = .parse_nest(args[[nm]])
                    if(!is.null(lst)) {
                        nms = union(names(lst), unlist(lst))
                        if(!all(nms %in% names(object@grid)))
                            stop("Nonexistent variables specified in 'nesting'")
                        object@misc$display = .find.nonempty.nests(object, nms)
                    }
                }
                if (fullname == "submodel") {
                    if(!is.null(A <- .alias.matrix(object, args[[nm]]))) {
                        rcols = attr(A, "rcols")
                        L = object@linfct
                        k = ncol(L) / ncol(A)
                        if(abs(k - (k<-as.integer(k))) > .01)
                            stop("Incompatible columns in alias setup")
                        ixmat = matrix(seq_along(L[1,]), ncol = k)
                        for (j in seq_len(k)) {
                            rc = ixmat[rcols, j]
                            fc =ixmat[, j]
                            object@linfct[, fc] = L[, rc, drop = FALSE] %*% A
                        }
                        object@model.info$model.matrix = "" # silent message
                        misc$initMesg = c(misc$initMesg, paste("submodel: ~", attr(A, "submodstr")))
                    }
                }
                    
                else
                    misc[[fullname]] = args[[nm]]
            }
        }
    }
    object@misc = misc
    object
}

### List of valid strings to match in update() ###
.valid.misc = c("adjust","alpha","avgd.over","bias.adjust","by.vars","calc",
                "cross.adjust", "delta","df",
               "initMesg","estName","estType","famSize","frequentist","infer","inv.lbl",
               "level","methDesc","nesting","null","predict.type","pri.vars",
               "side","sigma","tran","tran.mult","tran.offset","tran2","type","is.new.rg",
               "submodel")


#' Set or change emmeans options
#'
#' Use \code{emm_options} to set or change various options that are used in
#' the \pkg{emmeans} package. These options are set separately for different contexts in
#' which \code{emmGrid} objects are created, in a named list of option lists.
#' 
#' \pkg{emmeans}'s options are stored as a list in the system option \code{"emmeans"}. 
#' Thus, \code{emm_options(foo = bar)} is the same as 
#' \code{options(emmeans = list(..., foo = bar))} where \code{...} represents any
#' previously existing options. The list \code{emm_defaults} contains the default
#' values in case the corresponding element of system option \code{emmeans} is \code{NULL}.
#' 
#' Currently, the following main list entries are supported:
#' \describe{
#' \item{\code{ref_grid}}{A named \code{list} of defaults for objects created by
#' \code{\link{ref_grid}}. This could affect other objects as well. For example,
#' if \code{emmeans} is called with a fitted model object, it calls
#' \code{ref_grid} and this option will affect the resulting \code{emmGrid}
#' object.}
#' \item{\code{emmeans}}{A named \code{list} of defaults for objects created by
#'   \code{\link{emmeans}} or \code{\link{emtrends}}.}
#' \item{\code{contrast}}{A named \code{list} of defaults for objects created by
#'   \code{\link{contrast.emmGrid}} or \code{\link{pairs.emmGrid}}.}
#' \item{\code{summary}}{A named \code{list} of defaults used by the methods
#'   \code{\link{summary.emmGrid}}, \code{\link{predict.emmGrid}}, \code{\link{test.emmGrid}},
#'   \code{\link{confint.emmGrid}}, and \code{\link{emmip}}. The only option that can
#'   affect the latter four is \code{"predict.type"}.}
#' \item{\code{allow.na.levs}}{A logical value that if \code{TRUE} (the default), allows
#'   \code{NA} to be among the levels of a factor. Older versions of \pkg{emmeans} did not
#'   allow this. So if problems come up (say in a messy dataset that includes incomplete cases), 
#'   try setting this to \code{FALSE}.}
#' \item{\code{sep}}{A character value to use as a separator in labeling factor combinations.
#'   Such labels are potentially used in several places such as \code{\link{contrast}} and 
#'   \code{\link{plot.emmGrid}} when combinations of factors are compared or plotted.
#'   The default is \code{" "}.}
#' \item{\code{parens}}{Character vector that determines which labels are parenthesized
#'   when they are contrasted. The first element is a regular expression, and the second and
#'   third elements are used as left and right parentheses. 
#'   See details for the \code{parens} argument in \code{\link{contrast}}. The default
#'   will parenthesize labels containing the four arithmetic operators, 
#'   using round parentheses.}
#' \item{\code{cov.keep}}{The default value of \code{cov.keep} in \code{\link{ref_grid}}.
#'   Defaults to \code{"2"}, i.e., two-level covariates are treated like factors.}
#' \item{\code{graphics.engine}}{A character value matching 
#'   \code{c("ggplot", "lattice")}, setting the default engine to use in
#'   \code{\link{emmip}} and \code{\link{plot.emmGrid}}.  Defaults to \code{"ggplot"}.}
#' \item{\code{gg.theme}}{An integer. Set it to 1 or 2 if you want the appearance
#'   of plots used in \pkg{emmeans} version 1.x.x or 2.x.x. Or set it to the actual theme
#'   that you want to use, e.g., \code{emm_options(gg.theme = ggplot2::theme_dark())}}
#' \item{\code{msg.interaction}}{A logical value controlling whether or not
#'   a message is displayed when \code{emmeans} averages over a factor involved
#'   in an interaction. It is probably not appropriate to do this, unless
#'   the interaction is weak. Defaults to \code{TRUE}.}
#' \item{\code{msg.nesting}}{A logical value controlling whether or not to
#'   display a message when a nesting structure is auto-detected. The existence
#'   of such a structure affects computations of EMMs. Sometimes, a nesting
#'   structure is falsely detected -- namely when a user has omitted some
#'   main effects but included them in interactions. This does not change the
#'   model fit, but it produces a different parameterization that is picked
#'   up when the reference grid is constructed. Defaults to \code{TRUE}.}
#' \item{\code{rg.limit}}{An integer value setting a limit on the number of rows
#'   in a newly constructed reference grid. This is checked based on the number of
#'   levels of the factors involved; but it excludes the levels of any multivariate
#'   responses because those are not yet known. The reference grid consists of all
#'   possible combinations of the predictors, and this can become huge if there are
#'   several factors. An error is thrown if this limit is exceeded. One can use the 
#'   \code{nuisance} argument of \code{\link{ref_grid}} to collapse on nuisance
#'   factors, thus making the grid smaller. Defaults to 10,000.}
#' \item{\code{simplify.names}}{A logical value controlling whether to
#'   simplify (when possible) names in the model formula that refer to datasets --
#'   for example, should we simplify a predictor name like \dQuote{\code{data$trt}}
#'   to just \dQuote{\code{trt}}? Defaults to \code{TRUE}.}
#' \item{\code{opt.digits}}{A logical value controlling the precision with which
#'   summaries are printed. If \code{TRUE} (default), the number of digits
#'   displayed is just enough to reasonably distinguish estimates from the ends
#'   of their confidence intervals; but always at least 3 digits. If
#'   \code{FALSE}, the system value \code{getOption("digits")} is used.}
#' \item{\code{pval.digits}}{An integer indicating the precision with which
#'   p-values are printed. Integers between \code{2} and \code{6} are acceptable; the default is 
#' \code{4}. Floating point numbers are truncated. If a number outside the accepted range is
#' used, the nearest acceptable integer will be used instead. Non-numeric arguments are ignored.}
#' \item{\code{back.bias.adj}}{A logical value controlling whether we 
#'   try to adjust bias when back-transforming. If \code{FALSE}, we use naive
#'   back transformation. If \code{TRUE} \emph{and \code{sigma} is available and valid}, a
#'   second-order adjustment is applied to estimate the mean on the response
#'   scale. A warning is issued if no valid \code{sigma} is available}
#' \item{\code{enable.submodel}}{A logical value. If \code{TRUE}, enables support 
#'   for selected model classes to implement the \code{submodel} option. If
#'   \code{FALSE}, this support is disabled. Setting this option to \code{FALSE}
#'   could save excess memory consumption.}
#'   
#' }%%% end describe{}
#' Some other options have more specific purposes:
#' \describe{
#' \item{\code{estble.tol}}{Tolerance for determining estimability in
#' rank-deficient cases. If absent, the value in \code{emm_defaults$estble.tol)}
#' is used.}
#' \item{\code{save.ref_grid}}{Logical value of \code{TRUE} if you wish the 
#' latest reference grid created to be saved in \code{.Last.ref_grid}.
#' The default is \code{FALSE}.}
#' \item{Options for \code{lme4::lmerMod} models}{Options \code{lmer.df},
#' \code{disable.pbkrtest}, \code{pbkrtest.limit}, \code{disable.lmerTest},
#' and \code{lmerTest.limit}
#' options affect how degrees of freedom are computed for \code{lmerMod} objects
#' produced by the \pkg{lme4} package). See that section of the "models" vignette
#' for details.}
#' \item{\code{post.ci.method}}{method for estimating confidence intervals for Bayesian models. 
#' Options are \code{"HPD"} (default) or \code{"quantile"}.} 
#' } %%%%%% end \describe
#'
#' @param ... Option names and values (see Details)
#' @param disable If non-missing, this will reset all options to their defaults 
#'   if \code{disable} tests \code{TRUE} (but first save them for possible later 
#'   restoration). Otherwise, all previously saved options
#'   are restored. This is important for bug reporting; please see the section below
#'   on reproducible bugs. When \code{disable} is specified, the other arguments are ignored.
#' 
#' @return \code{emm_options} returns the current options (same as the result 
#'   of \samp{getOption("emmeans")}) -- invisibly, unless called with no arguments.
#' 
#' @section Reproducible bugs:
#' Most options set display attributes and such that are not likely to be associated
#' with bugs in the code. However, some other options (e.g., \code{cov.keep})
#' are essentially configuration settings that may affect how/whether the code
#' runs, and the settings for these options may cause subtle effects that may be
#' hard to reproduce. Therefore, when sending a bug report, please create a reproducible
#' example and make sure the bug occurs with all options set at their defaults.
#' This is done by preceding it with  \code{emm_options(disable = TRUE)}. 
#' 
#' By the way, \code{disable} works like a stack (LIFO buffer), in that \code{disable = TRUE}
#' is equivalent to \code{emm_options(saved.opts = emm_options())} and 
#' \code{emm_options(disable = FALSE)} is equivalent to 
#' \code{options(emmeans = get_emm_option("saved.opts"))}. To completely erase
#' all options, use \code{options(emmeans = NULL)}
#' 
#' @section Startup options:
#' \pkg{emmeans}'s options are kept in the system options in a list names \code{emmeans}.
#' Thus, if the user wants certain options set on startup, this can be arranged by
#' setting the \code{emmeans} option in one's startup script. For example,
#' the user's \code{.Rprofile} file could contain:
#' \preformatted{    options(emmeans = list(summary = list(predict.type = "response"), 
#'                            lmer.df = "satterthwaite"))}
#' 
#' 
#' @seealso \code{\link{update.emmGrid}}
#' @export
#' @examples
#' \dontrun{
#' emm_options(ref_grid = list(level = .90),
#'             contrast = list(infer = c(TRUE,FALSE)),
#'             estble.tol = 1e-6)
#' # Sets default confidence level to .90 for objects created by ref.grid
#' # AS WELL AS emmeans called with a model object (since it creates a 
#' # reference grid). In addition, when we call 'contrast', 'pairs', etc.,
#' # confidence intervals rather than tests are displayed by default.
#' }
#' 
#' \dontrun{
#' emm_options(disable.pbkrtest = TRUE)
#' # This forces use of asymptotic methods for lmerMod objects.
#' # Set to FALSE or NULL to re-enable using pbkrtest.
#' }
#' 
#' # See tolerance being used for determining estimability
#' get_emm_option("estble.tol")
#' 
#' \dontrun{
#' # Set all options to their defaults
#' emm_options(disable = TRUE)
#' # ... and perhaps follow with code for a minimal reproducible bug,
#' #     which may include emm_options() clls if they are pertinent ...
#' 
#' # restore options that had existed previously
#' emm_options(disable = FALSE)
#' }
#'
emm_options = function(..., disable) {
    opts = getOption("emmeans", list())
    newopts = list(...)
    display = TRUE  # flag to display all options if ... is empty
    if (!missing(disable)) {
        if (disable)
            opts = list(saved.opts = opts)
        else if (!is.null(saved <- opts$saved.opts))
            opts = saved
        newopts = list()
        display = FALSE
    }
    for (nm in names(newopts))
        opts[[nm]] = newopts[[nm]]
    options(emmeans = opts)
    if (length(newopts) > 0)
        invisible(opts)
    else if (display) {
        opts = c(opts, emm_defaults)
        opts[sort(names(opts))]
    }
}

# equivalent of getOption()
#' @rdname emm_options
#' @param x Character value - the name of an option to be queried
#' @param default Value to return if \code{x} is not found
#' @return \code{get_emm_option} returns the currently stored option for \code{x}, 
#'   or its default value if not found.
#' @export
get_emm_option = function(x, default = emm_defaults[[x]]) {
    opts = getOption("emmeans", list())
    if(is.null(default) || hasName(opts, x))
        opts[[x]]
    else 
        default
}

#' @rdname emm_options
#' @param expr Expression to evaluate. If missing, the last element of \code{...} is used.
#' @return \code{with_emm_options()} temporarily sets the options in \code{...}, then 
#'   evaluates \code{try(expr)} and returns the result.
#' @export
#' @examples
#' 
#' # Temporarily alter the formatting of labels in an interaction contrast:
#' pigs.lm <- lm(inverse(conc) ~ source * factor(percent), data = pigs)
#' with_emm_options(parens = c("\\-", "<", ">"), sep = ": ",
#'     contrast(contrast(ref_grid(pigs.lm), "consec", by = "percent"),
#'         "consec", by = NULL))
#' 
with_emm_options = function(..., expr) {
    cl = match.call()
    if(missing(expr)) {
        expr = cl[[length(cl)]]
        cl = cl[-length(cl)]
    }
    cl[[1]] = as.name("emm_options")
    oldopts = getOption("emmeans")
    eval(cl, parent.frame())
    result = try(eval(expr, parent.frame()), silent = TRUE)
    options(emmeans = oldopts)
    result
}



### Exported defaults for certain options
#' @rdname emm_options
#' @export
emm_defaults = list (
    ref_grid = list(is.new.rg = TRUE, infer = c(FALSE, FALSE)),
    emmeans = list(infer = c(TRUE, FALSE)),
    contrast = list(infer = c(FALSE, TRUE)),
    save.ref_grid = FALSE,    # save new ref_grid in .Last.ref_grid
    allow.na.levs = TRUE,     # Do we allow factors to have NA as a level?
    cov.keep = "2",           # default for cov.keep arg in ref_grid
    sep = " ",                # separator for combining factor levels
    parens = c("-|\\+|\\/|\\*", "(", ")"), # patterns for what/how to parenthesize in contrast
    graphics.engine = "ggplot",  # default for emmip and plot.emmGrid
    gg.theme = 2,             # Use the ggplot theme used in this emmeans version 
###    msg.data.call = TRUE,     # message when there's a call in data or subset
    msg.interaction = TRUE,   # message about averaging w/ interactions
    msg.nesting = TRUE,       # message when nesting is detected
    estble.tol = 1e-8,        # tolerance for estimability checks
    simplify.names = TRUE,    # simplify names like data$x to just "x"
    back.bias.adj = FALSE,    # Try to bias-adjust back-transformations?
    opt.digits = TRUE,        # optimize displayed digits for estimates and CI's?
    pval.digits = 4L,          # how many digits to list for p-values
    enable.submodel = TRUE,   # enable saving extra info for submodel
    rg.limit = 10000,         # limit on number of rows in a reference grid
    lmer.df = "kenward-roger",  # Use Kenward-Roger for df
    disable.pbkrtest = FALSE, # whether to bypass pbkrtest routines for lmerMod
    pbkrtest.limit = 3000,    # limit on N for enabling K-R
    disable.lmerTest = FALSE, # whether to bypass lmerTest routines for lmerMod
    lmerTest.limit = 3000,     # limit on N for enabling Satterthwaite
    post.ci.method = "HPD"    #  set CI method for Bayesian methods

)


# override levels<- method
#' @rdname update.emmGrid
#' @order 5
#' @export
#' @param x an \code{emmGrid} object
#' @param value \code{list} or replacement levels. See the documentation for
#' \code{update.emmGrid} with the \code{levels} argument, 
#' as well as the section below on \dQuote{Replaciong levels}
#'   
#' @return \code{levels<-} replaces the levels of the object in-place.
#'   See the section on replacing levels for details.
#' @section Replacing levels:
#' The \code{levels<-} method uses \code{update.emmGrid} to replace the
#' levels of one or more factors. This method allows selectively replacing
#' the levels of just one factor (via subsetting operators), whereas 
#' \code{update(x, levels = list(...))} requires a list of \emph{all} factors
#' and their levels. If any factors are to be renamed, we must replace all
#' levels and include the new names in the replacements. See the examples.
#' 
#' @examples
#' 
#' ## Changing levels of one factor
#' newrg <- pigs.rg
#' levels(newrg)$source <- 1:3
#' newrg
#' 
#' ## Unraveling a previously standardized covariate
#' zd = scale(fiber$diameter)
#' fibz.lm <- lm(strength ~ machine * zd, data = fiber)
#' (fibz.rg <- ref_grid(fibz.lm, at = list(zd = -2:2)))   ### 2*SD range
#' lev <- levels(fibz.rg)
#' levels(fibz.rg) <- list (
#'     machine = lev$machine,
#'     diameter = with(attributes(zd), 
#'                     `scaled:center` + `scaled:scale` * lev$zd) )
#' fibz.rg
#' 
"levels<-.emmGrid" = function(x, value) {
    update.emmGrid(x, levels = value)
}


### Utility to change the internal structure of an emmGrid object
### Returned emmGrid object has linfct = I and bhat = estimates
### Primary reason to do this is with transform = TRUE, then can 
### work with linear functions of the transformed predictions

#' Reconstruct a reference grid with a new transformation or simulations
#' 
#' The typical use of this function is to cause EMMs to be computed on
#' a different scale, e.g., the back-transformed scale rather than the 
#' linear-predictor scale. In other words, if you want back-transformed 
#' results, do you want to average and then back-transform, or 
#' back-transform and then average?
#' 
#' The \code{regrid} function reparameterizes an existing \code{ref.grid} so
#' that its \code{linfct} slot is the identity matrix and its \code{bhat} slot
#' consists of the estimates at the grid points. If \code{transform} is
#' \code{TRUE}, the inverse transform is applied to the estimates. Outwardly,
#' when \code{transform = "response"}, the result of \code{\link{summary.emmGrid}}
#' after applying \code{regrid} is identical to the summary of the original
#' object using \samp{type="response"}. But subsequent EMMs or
#' contrasts will be conducted on the new scale -- which is
#' the reason this function exists. 
#' 
#' This function may also be used to simulate a sample of regression
#' coefficients for a frequentist model for subsequent use as though it were a
#' Bayesian model. To do so, specify a value for \code{N.sim} and a sample is
#' simulated using the function \code{sim}. The grid may be further processed in
#' accordance with the other arguments; or if \code{transform = "pass"}, it is
#' simply returned with the only change being the addition of the simulated
#' sample.
#' 
#' @param object An object of class \code{emmGrid}
#' @param transform Character, list, or logical value. If \code{"response"},
#'   \code{"mu"}, or \code{TRUE}, the inverse transformation is applied to the
#'   estimates in the grid (but if there is both a link function and a response
#'   transformation, \code{"mu"} back-transforms only the link part); if
#'   \code{"none"} or \code{FALSE}, \code{object} is re-gridded so that its
#'   \code{bhat} slot contains \code{predict(object)} and its \code{linfct} slot
#'   is the identity. Any internal transformation information is preserved. If
#'   \code{transform = "pass"}, the object is not re-gridded in any way (this
#'   may be useful in conjunction with \code{N.sim}).
#'   
#'   If \code{transform} is a character value in \code{links} (which is the set
#'   of valid arguments for the \code{\link{make.link}} function, excepting
#'   \code{"identity"}), or if \code{transform} is a list of the same form as
#'   returned by \code{make.links} or \code{\link{make.tran}}, the results are
#'   formulated as if the response had been transformed with that link function.
#'   
#' @param inv.link.lbl Character value. This applies only when \code{transform} 
#'   is in \code{links}, and is used to label the predictions if subsequently summarized
#'   with \code{type = "response"}.
#' @param predict.type Character value. If provided, the returned object is
#'   updated with the given type to use by default by \code{summary.emmGrid}
#'   (see \code{\link{update.emmGrid}}).  This may be useful if, for example,
#'   when one specifies \code{transform = "log"} but desires summaries to be
#'   produced by default on the response scale.
#' @param bias.adjust Logical value for whether to adjust for bias in
#'   back-transforming (\code{transform = "response"}). This requires a valid value of 
#'   \code{sigma} to exist in the object or be specified.
#' @param sigma Error SD assumed for bias correction (when 
#'   \code{transform = "response"} and a transformation
#'   is in effect). If not specified,
#'   \code{object@misc$sigma} is used, and a warning is issued if it is not found.
#' @param N.sim Integer value. If specified and \code{object} is based on a 
#'   frequentist model (i.e., does not have a posterior sample), then a fake 
#'   posterior sample is generated using the function \code{sim}.
#' @param sim A function of three arguments (no names are assumed).
#'   If \code{N.sim} is supplied with a frequentist model, this function is called
#'   with respective arguments \code{N.sim}, \code{object@bhat}, and \code{object@V}.
#'   The default is the multivariate normal distribution.
#' @param ... Ignored.
#' 
#' @section Degrees of freedom:  
#' In cases where the
#' degrees of freedom depended on the linear function being estimated (e.g.,
#' Satterthwaite method), the d.f.
#' from the reference grid are saved, and a kind of \dQuote{containment} method
#' is substituted in the returned object, whereby the calculated d.f. for a new
#' linear function will be the minimum d.f. among those having nonzero
#' coefficients. This is kind of an \emph{ad hoc} method, and it can
#' over-estimate the degrees of freedom in some cases. An annotation is
#' displayed below any subsequent summary results stating that the 
#' degrees-of-freedom method is inherited from the previous method at
#' the time of re-gridding.
#'
#' @note Another way to use \code{regrid} is to supply a \code{regrid} 
#'   argument to \code{\link{ref_grid}} (either directly of indirectly via
#'   \code{\link{emmeans}}), in which case its value is passed to \code{regrid} as
#'   \code{transform}. This is often a simpler approach if the reference
#'   grid has not already been constructed.
#'
#' @return An \code{emmGrid} object with the requested changes
#' @export
#'
#' @examples
#' pigs.lm <- lm(log(conc) ~ source + factor(percent), data = pigs)
#' rg <- ref_grid(pigs.lm)
#' 
#' # This will yield EMMs as GEOMETRIC means of concentrations:
#' (emm1 <- emmeans(rg, "source", type = "response"))
#' pairs(emm1) ## We obtain RATIOS
#' 
#' # This will yield EMMs as ARITHMETIC means of concentrations:
#' (emm2 <- emmeans(regrid(rg, transform = "response"), "source"))
#' pairs(emm2)  ## We obtain DIFFERENCES
#' # Same result, useful if we hadn't already created 'rg'
#' # emm2 <- emmeans(pigs.lm, "source", regrid = "response")
#' 
#' # Simulate a sample of regression coefficients
#' set.seed(2.71828)
#' rgb <- regrid(rg, N.sim = 200, transform = "pass")
#' emmeans(rgb, "source", type = "response")  ## similar to emm1
regrid = function(object, transform = c("response", "mu", "unlink", "none", "pass", links), 
                  inv.link.lbl = "response", predict.type, 
                  bias.adjust = get_emm_option("back.bias.adj"), sigma, 
                  N.sim, sim = mvtnorm::rmvnorm, ...) 
{
    object = .chk.list(object, ...)
    links = c("logit", "probit", "cauchit", "cloglog", "log", "log10", "log2", "sqrt", "1/mu^2", "inverse")
    if (is.logical(transform))   # for backward-compatibility
        transform = ifelse(transform, "response", "none")
    else if (is.list(transform)) {
        userlink = transform
        transform = "user"
    }
    else
        transform = match.arg(transform)
    
    if (is.na(object@post.beta[1]) && !missing(N.sim)) {
        message("Simulating a sample of size ", N.sim, " of regression coefficients.")
        bh = object@bhat[!is.na(object@bhat)]
        object@post.beta = sim(N.sim, bh, object@V)
    }

    if (transform == "pass")
        return(object)

    # if we have two transformations to undo, do the first one recursively
    if ((transform == "response") && (!is.null(object@misc$tran2)))
             object = regrid(object, transform = "mu")

    # Save post.beta stuff
    PB = object@post.beta
    NC = attr(PB, "n.chains")
    
    if (!is.na(PB[1])) { # fix up post.beta BEFORE we overwrite parameters
        PB = PB %*% t(object@linfct[ , !is.na(object@bhat), drop = FALSE])
        if (".offset." %in% names(object@grid))
            PB = t(apply(PB, 1, function(.) . + object@grid[[".offset."]]))
    }
    
    est = .est.se.df(object, do.se = TRUE) ###FALSE)
    
    estble = !(is.na(est[[1]]))
    object@V = vcov(object)[estble, estble, drop = FALSE]
    object@bhat = est[[1]]
    object@linfct = diag(1, length(estble))
    object@misc$regrid.flag = TRUE
    pargs = object@grid[names(object@levels)]
    lbls = do.call(paste, c(pargs, sep = "."))
    if (!is.null(disp <- object@misc$display)) {  # fix up for the bookkeeping in nested models
        object@V = object@V[estble, estble, drop = FALSE]
        object@linfct = matrix(0, nrow = length(disp), ncol = length(estble))
        object@linfct[disp, ] = diag(1, length(estble))
        lbls = lbls[disp]
    }
    colnames(object@linfct) = lbls
    if(all(estble))
        object@nbasis = estimability::all.estble
    else
        object@nbasis = object@linfct[, !estble, drop = FALSE]
    
    # override the df function
    df = est$df
    edf = df[estble]
    if (length(edf) == 0) edf = NA
    # note both NA/NA and Inf/Inf test is.na() = TRUE
    prev.df.msg = attr(object@dffun, "mesg")
    if (any(is.na(edf/edf)) || (diff(range(edf)) < .01)) { # use common value
        object@dfargs = list(df = mean(edf, na.rm = TRUE))
        object@dffun = function(k, dfargs) dfargs$df
    }
    else { # use containment df
        object@dfargs = list(df = df)
        object@dffun = function(k, dfargs) {
            idx = which(zapsmall(k) != 0)
            ifelse(length(idx) == 0, NA, min(dfargs$df[idx], na.rm = TRUE))
        }
    }
    if(!is.null(prev.df.msg)) 
        attr(object@dffun, "mesg") = ifelse(
            startsWith(prev.df.msg, "inherited"), prev.df.msg,
            paste("inherited from", prev.df.msg, "when re-gridding"))
    
    if(transform %in% c("response", "mu", "unlink", links, "user") && !is.null(object@misc$tran)) {
        flink = link = attr(est, "link")
        if (bias.adjust) {
            if(missing(sigma))
                sigma = object@misc$sigma
            link = .make.bias.adj.link(link, sigma)
            #### (not needed)   bias.adjust = attr(link, "bias.adjust")
            if (!is.na(PB[1])) # special frequentist version when sigma is MCMC sample
                flink = .make.bias.adj.link(flink, mean(sigma))
            else
                flink = link
        }
        if (!is.na(PB[1]))
            PB = matrix(flink$linkinv(PB), ncol = ncol(PB))
        D = flink$mu.eta(object@bhat[estble])
        object@bhat = flink$linkinv(object@bhat)
        # efficient repl for D'VD with D <- diag(D)
        object@V = sweep(sweep(object@V, 1, D, "*"), 2, D, "*")

        inm = object@misc$inv.lbl
        if (!is.null(inm)) {
            object@misc$estName = inm
            if (!is.null(object@misc$log.contrast) && object@misc$log.contrast) # relabel ratios
                for (v in setdiff(object@misc$pri.vars, object@misc$by.vars))
                    object@grid[[v]] = gsub(" - ", "/", object@grid[[v]])
        }
        if((transform %in% c("mu", "unlink")) && !is.null(object@misc$tran2)) {
            object@misc$tran = object@misc$tran2
            object@misc$tran2 = object@misc$tran.mult = object@misc$tran.offset = object@misc$inv.lbl = NULL
        }
        else
            object@misc$tran = object@misc$tran.mult = object@misc$tran.offset = object@misc$inv.lbl = NULL
        sigma = object@misc$sigma = NULL
    }
    if (transform %in% c(links, "user")) { # fake a transformation
        link = if (transform == "user") userlink
               else                     .make.link(transform)
        bounds = range(link$linkinv(c(-1e6, -100, -1, 0, 1, 100, 1e6)))
        nas = which(is.na(object@bhat)) # already NA
        incl = vincl = which((object@bhat > bounds[1]) & (object@bhat < bounds[2]))
        if (length(nas) > 0) 
            vincl = which((object@bhat[-nas] > bounds[1]) & (object@bhat[-nas] < bounds[2]))
        negs = setdiff(seq_along(object@bhat), incl)
        if (length(negs) > 0) {
            message("Invalid response predictions are flagged as non-estimable")
            object@bhat[negs] = NA
            tmp = seq_along(object@bhat)
            object@nbasis = sapply(c(nas, negs), function(ii) 0 + (tmp == ii))
        }
        object@bhat = link$linkfun(object@bhat)
        Vee = object@V
        if(length(incl) > 0) {
            D = 1 / link$mu.eta(object@bhat[incl])
            object@V = sweep(sweep(Vee[vincl, vincl, drop = FALSE], 1, D, "*"), 2, D, "*")
        }
        if (!is.na(PB[1])) {
            PB[PB <= 0] = NA
            PB = link$linkfun(PB)
            PB[1] = ifelse(is.na(PB[1]), 0, PB[1]) # make sure 1st elt isn't NA
        }
        object@misc$tran = if (transform == "user") link   
                           else                     transform
        object@misc$inv.lbl = inv.link.lbl
    }
    
    if(!is.na(PB[1])) {
        PB = PB[ , !is.na(object@bhat), drop = FALSE]
        attr(PB, "n.chains") = NC
        object@post.beta = PB
    }
    
    # Nix out things that are no longer needed or valid
    object@grid$.offset. = object@misc$offset.mult =
        object@misc$estHook = object@misc$vcovHook = NULL
    object@model.info$model.matrix = "Submodels are not available with regridded objects"
    if(!missing(predict.type))
        object = update(object, predict.type = predict.type)

    object
}

context("utility functionality")

m1 <- lm(breaks ~ wool*tension, data = warpbreaks)
em1 <- emmeans(m1, ~ wool | tension)

emm_options(pval.digits = 8); invisible(capture.output(joint_tests(em1)))
pdig_1 = get_emm_option("pval.digits") 

emm_options(pval.digits = "eight"); invisible(capture.output(joint_tests(em1)))
pdig_2 = get_emm_option("pval.digits") 

emm_options(pval.digits = 1); invisible(capture.output(joint_tests(em1)))
pdig_3 = get_emm_option("pval.digits") 

emm_options(pval.digits = c(3, 6));  invisible(capture.output(joint_tests(em1)))
pdig_4 = get_emm_option("pval.digits")[1] 

test_that("digits for p-values process", {
  expect_equal(pdig_1, 6)
  expect_equal(pdig_2, 4)
  expect_equal(pdig_3, 2)
  expect_equal(pdig_4, 3)
})

##############################################################################
#    Copyright (c) 2012-2017 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

### Helper functions for emmeans
### Here we have 'recover_data' and 'emm_basis' methods
### For models that this package supports.

#--------------------------------------------------------------
### lm objects (and also aov, rlm, others that inherit) -- but NOT aovList
### Recent additional arhument 'frame' should point to where the model frame 
###   might be available, or NULL otherwise
#' @exportS3Method recover_data lm
#' @export
recover_data.lm = function(object, frame = object$model, ...) {
        fcall = object$call
    recover_data(fcall, delete.response(terms(object)), object$na.action, 
                 frame = frame, pwts = weights(object), ...)
}

#' @export
emm_basis.lm = function(object, trms, xlev, grid, ...) {
    # coef() works right for lm but coef.aov tosses out NAs
    bhat = object$coefficients
    nm = if(is.null(names(bhat))) row.names(bhat) else names(bhat)
    m = suppressWarnings(model.frame(trms, grid, na.action = na.pass, xlev = xlev))
    X = model.matrix(trms, m, contrasts.arg = object$contrasts)
    assign = attr(X, "assign")
    X = X[, nm, drop = FALSE]
    bhat = as.numeric(bhat) 
    # stretches it out if multivariate - see mlm method
    V = .my.vcov(object, ...)
    
    if (sum(is.na(bhat)) > 0)
        nbasis = estimability::nonest.basis(object$qr)
    else
        nbasis = estimability::all.estble
    misc = list()
    if (inherits(object, "glm")) {
        misc = .std.link.labels(object$family, misc)
        dffun = function(k, dfargs) dfargs$df
        dfargs = list(df = ifelse(object$family$family %in% c("gaussian", "Gamma"), 
                                  object$df.residual, Inf))
    }
    else {
        dfargs = list(df = object$df.residual)
        dffun = function(k, dfargs) dfargs$df
    }
    list(X=X, bhat=bhat, nbasis=nbasis, V=V, dffun=dffun, dfargs=dfargs, misc=misc,
         model.matrix = .cmpMM(object$qr, assign = assign))
}



#--------------------------------------------------------------
### mlm objects
# (recover_data.lm works just fine)

#' @export
emm_basis.mlm = function(object, trms, xlev, grid, ...) {
    class(object) = c("mlm", "lm") # avoids error in vcov for "maov" objects
    bas = emm_basis.lm(object, trms, xlev, grid, ...)
    bhat = coef(object)
    k = ncol(bhat)
    bas$X = kronecker(diag(rep(1,k)), bas$X)
    bas$nbasis = kronecker(rep(1,k), bas$nbasis)
    ylevs = dimnames(bhat)[[2]]
    if (is.null(ylevs)) ylevs = seq_len(k)
    bas$misc$ylevs = list(rep.meas = ylevs)
    bas
}

#----------------------------------------------------------
# manova objects
#' @exportS3Method recover_data manova
recover_data.manova = function(object, ...) {
    fcall = match.call(aov, object$call)   # need to borrow arg matching from aov()
    recover_data(fcall, delete.response(terms(object)), object$na.action, 
                 frame = object$model, ...)
}




#--------------------------------------------------------------
### merMod objects (lme4 package)
#' @export
recover_data.merMod = function(object, ...) {
    if(!lme4::isLMM(object) && !lme4::isGLMM(object)) 
        return("Can't handle a nonlinear mixed model")
    fcall = object@call
    recover_data(fcall, delete.response(terms(object)), 
                 attr(object@frame, "na.action"), frame = object@frame, pwts = weights(object), ...)
}

#' @export
emm_basis.merMod = function(object, trms, xlev, grid, 
                            mode = get_emm_option("lmer.df"), lmer.df, 
                            disable.pbkrtest = get_emm_option("disable.pbkrtest"), 
                            pbkrtest.limit = get_emm_option("pbkrtest.limit"), 
                            disable.lmerTest = get_emm_option("disable.lmerTest"), 
                            lmerTest.limit = get_emm_option("lmerTest.limit"), 
                            options, ...) {
    V = .my.vcov(object, ...)
    dfargs = misc = list()
    
    if (lme4::isLMM(object)) {
        # Allow lmer.df in lieu of mode
        if (!missing(lmer.df))
            mode = lmer.df

        mode = match.arg(tolower(mode), c("satterthwaite", "kenward-roger", "asymptotic"))
        # if we're gonna override the df anyway, keep it simple 
        # OTOH, if K-R, documentation promises we'll adjust V
        if (!is.null(options$df) && (mode != "kenward-roger")) 
            mode = "asymptotic"
        
        
        # set flags
        objN = lme4::getME(object, "N")
        tooBig.k = (objN > pbkrtest.limit)
        tooBig.s = (objN > lmerTest.limit)
        
        tooBigMsg = function(pkg, limit) {  
            message("Note: D.f. calculations have been",
                    " disabled because the number of observations exceeds ", limit, ".\n",
                    "To enable adjustments, add the argument '", pkg, ".limit = ", objN, "' (or larger)\n",
                    "[or, globally, 'set emm_options(", pkg, ".limit = ", objN, ")' or larger];\n",
                    "but be warned that this may result in large computation time and memory use.")
        }

        # pick the lowest-hanging apples first
        if (mode == "kenward-roger") {
            if (disable.pbkrtest || tooBig.k || !.requireNS("pbkrtest", 
                    "Cannot use mode = \"kenward-roger\" because *pbkrtest* package is not installed", 
                    fail = message))
                mode = "satterthwaite"
            if (!disable.pbkrtest && tooBig.k)
                tooBigMsg("pbkrtest", pbkrtest.limit)
        }
        if (mode == "satterthwaite") {
            if (disable.lmerTest || tooBig.s || !.requireNS("lmerTest", 
                    "Cannot use mode = \"satterthwaite\" because *lmerTest* package is not installed", 
                    fail = message))
                mode = ifelse(!disable.pbkrtest && !tooBig.k && 
                                  .requireNS("pbkrtest", fail = .nothing), 
                              "kenward-roger", "asymptotic")
            if (!disable.lmerTest && tooBig.s)
                tooBigMsg("lmerTest", lmerTest.limit)
        }
        # if my logic isn't flawed, we are guaranteed that mode is both desired and possible
        
        if (mode == "kenward-roger") {
            if ((\(vcov., ...) missing(vcov.))(...)) {   # if (vcov. not in ...)
                dfargs = list(unadjV = V, 
                              adjV = pbkrtest::vcovAdj.lmerMod(object, 0))
                V = as.matrix(dfargs$adjV)
                tst = try(pbkrtest::Lb_ddf)
                if(!inherits(tst, "try-error"))
                    dffun = function(k, dfargs) pbkrtest::Lb_ddf (k, dfargs$unadjV, dfargs$adjV)
                else {
                    mode = "asymptotic"
                    warning("Failure in loading pbkrtest routines",
                            " - reverted to \"asymptotic\"")
                }
            }
            else {
                message("Kenward-Roger method can't be used with user-supplied covariances")
                mode = "satterthwaite"
            }
        }
        
        if (mode == "satterthwaite") {
            dfargs = list(object = object)
            dffun = function(k, dfargs) 
                suppressMessages(lmerTest::calcSatterth(dfargs$object, k)$denom)
        }
        
        if (mode == "asymptotic") {
            dffun = function(k, dfargs) Inf
        }
        
        attr(dffun, "mesg") = mode
    }
    else if (lme4::isGLMM(object)) {
        dffun = function(k, dfargs) Inf
        misc = .std.link.labels(family(object), misc)
    }
    else 
        stop("Can't handle a nonlinear mixed model")
    
    contrasts = attr(object@pp$X, "contrasts")
    m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
    X = model.matrix(trms, m, contrasts.arg = contrasts)
    bhat = lme4::fixef(object)
    
    if (length(bhat) < ncol(X)) {
        # Newer versions of lmer can handle rank deficiency, but we need to do a couple of
        # backflips to put the pieces together right,
        # First, figure out which columns were retained
        kept = match(names(bhat), dimnames(X)[[2]])
        # Now re-do bhat with NAs in the right places
        bhat = NA * X[1, ]
        bhat[kept] = lme4::fixef(object)
        # we have to reconstruct the model matrix
        modmat = model.matrix(trms, object@frame, contrasts.arg=contrasts)
        nbasis = estimability::nonest.basis(modmat)
    }
    else
        nbasis=estimability::all.estble
    
    mm = .cmpMM(object@pp$X, object@pp$Xwts^2,
                attr(object@pp$X, "assign"))
    
    list(X=X, bhat=bhat, nbasis=nbasis, V=V, dffun=dffun, dfargs=dfargs, misc=misc,
         model.matrix = mm)
}




#--------------------------------------------------------------
### lme objects (nlme package)
#' @export
recover_data.lme = function(object, data, ...) {
    fcall = object$call
    if (!is.null(eval(fcall$weights, environment(terms(object)), parent.frame()))) {  # painful -- we only get weights for complete cases
        if (!is.null(object$na.action)) {
            w = nlme::varWeights(object$modelStruct)
            wts = rep(0, length(w) + length(object$na.action))
            wts[-object$na.action] = w
            fcall$weights = wts
        }
        else
            fcall$weights = nlme::varWeights(object$modelStruct)
    }
    dat = recover_data(fcall, delete.response(object$terms), object$na.action, data = data, ...)
    attr(dat, "pass.it.on") = TRUE
    dat
}

#' @export
emm_basis.lme = function(object, trms, xlev, grid, 
        mode = c("containment", "satterthwaite", "appx-satterthwaite", "auto", "boot-satterthwaite", "asymptotic"), 
        sigmaAdjust = TRUE, options, extra.iter = 0, ...) {
    mode = match.arg(mode)
    if (mode == "boot-satterthwaite") mode = "appx-satterthwaite"  # backward compatibility
    if (mode == "asymptotic")
        options$df = Inf
    if (!is.null(options$df)) # if we're gonna override the df anyway, keep it simple!
        mode = "fixed"
    if (mode == "auto")
        mode = ifelse(is.null(object$apVar), "containment", "appx-satterthwaite")
    if (is.null(object$apVar))
        mode = "containment"
    contrasts = object$contrasts
    m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
    X = model.matrix(trms, m, contrasts.arg = contrasts)
    bhat = nlme::fixef(object)
    V = .my.vcov(object, ...)
    if (sigmaAdjust && object$method == "ML") 
        V = V * object$dims$N / (object$dims$N - nrow(V))
    misc = list()
    if (!is.null(object$family)) {
        misc = .std.link.labels(object$family, misc)
    }
    nbasis = estimability::all.estble
    
    if (mode == "fixed") { # hack to just put in df from options
        dfargs = list(df = options$df)
        dffun = function(k, dfargs) dfargs$df
    }
    else if (mode %in% c("satterthwaite", "appx-satterthwaite")) {
        mode = "appx-satterthwaite"
        G = try(gradV.kludge(object, extra.iter = extra.iter), silent = TRUE)
        ###! not yet, doesn't work G = try(lme_grad(object, object$call, object$data, V))
        if (inherits(G, "try-error"))
            stop("Unable to estimate Satterthwaite parameters")
        dfargs = list(V = V, A = object$apVar, G = G)
        dffun = function(k, dfargs) {
            est = tcrossprod(crossprod(k, dfargs$V), k)
            g = sapply(dfargs$G, function(M) tcrossprod(crossprod(k, M), k))
            varest = tcrossprod(crossprod(g, dfargs$A), g)
            2 * est^2 / varest
        }
    }
    else { # containment df
        dfx = object$fixDF$X
        if (names(bhat[1]) == "(Intercept)")
            dfx[1] = length(levels(object$groups[[1]])) - 1
        ### Correct apparent error in lme containment algorithm
        dffun = function(k, dfargs) {
            idx = which(abs(k) > 1e-4)
            ifelse(length(idx) > 0, min(dfargs$dfx[idx]), NA)
        }
        dfargs = list(dfx = dfx)
    }
    attr(dffun, "mesg") = mode
    
    # submodel support (not great -- omits any weights)
    m = model.frame(trms, attr(object, "data"), na.action = na.pass, xlev = xlev)
    mm = model.matrix(trms, m, contrasts.arg = contrasts)
    mm = .cmpMM(mm, assign = attr(mm, "assign"))
    
    list(X = X, bhat = bhat, nbasis = nbasis, V = V, 
         dffun = dffun, dfargs = dfargs, misc = misc,
         model.matrix = mm)
}

# Here is a total hack, but it works pretty well
# We estimate the gradient of the V matrix by fitting the
# model with a few random perturbations of y, then
# regressing the changes in V against the changes in the 
# covariance parameters
gradV.kludge = function(object, Vname = "varFix", call = formula(object$terms), 
                        data = object$data, extra.iter = 0) {
    # check consistency of contrasts
    #### This code doesn't work with coerced factors. Hardly seems messing with, so I commented it out
    # cnm = names(object$contrasts)
    # cdiff = sapply(cnm, function(.) max(abs(contrasts(data[[.]]) - object$contrasts[[.]])))
    # if (max(cdiff) > 1e-6) {
    #     message("Contrasts don't match those used when the model was fitted. Fix this and re-run")
    #     stop()
    # }
    
    if(is.null(data)) {
        vars = all.vars(eval(object$call[[2]]))
        lst = lapply(vars, get)
        names(lst) = vars
        data = data.frame(lst)
    }
    A = object$apVar
    theta = attr(A, "Pars")
    V = object[[Vname]]
    sig = .01 * object$sigma
    #data = object$data
    yname = all.vars(eval(call))[1]
    y = data[[yname]]
    n = length(y)
    object$call[[2]] = call
    # we're gonna carefully check simulation results to make sure we didn't
    # lose (or gain?) any params so that everything conforms
    niter = 0
    nsim = 2 + extra.iter + length(theta)
    dat = t(replicate(nsim, {
        simt = simv = numeric(0)
        niter = niter + 1
        while((niter < 3 * nsim) && 
              ((length(simv) != length(V)) || (length(simt) != length(theta)))) {
            data[[yname]] = y + sig * rnorm(n)
            mod = update(object, data = data)
            simt = attr(mod$apVar, "Pars")
            simv = mod[[Vname]]
        }
        niter = niter + 1
        if (niter > 3 * nsim) 
            stop("Too many simulations due to inconsistency")
        c(simt - theta, as.numeric(simv - V))
    }))
    dimnames(dat) = c(NULL, NULL)
    xcols = seq_along(theta)
    B = lm.fit(dat[, xcols], dat[,-xcols])$coefficients
    grad = lapply(seq_len(nrow(B)), function(i) matrix(B[i, ], nrow=nrow(V)))
    grad
}

# ### new way to get gradients for lme models
# # (not ready for primetime...)
# lme_grad = function(object, call, data, V) {
#     obj = object$modelStruct
#     conLin = object
#     class(conLin) = class(obj)
#     X = model.matrix(eval(call$fixed), data = data)
#     y = data[[all.vars(call)[1]]]
#     conLin$Xy = cbind(X, y)
#     conLin$fixedSigma = FALSE
#     grps = object$groups # May have to re-order these fancily?
#     MEest = get("MEestimate", getNamespace("nlme")) ## workaround its not being exported
#     func = function(x) {
#         coef(obj) = x
#         tmp = MEest(obj, grps, conLin)
#         crossprod(tmp$sigma * tmp$varFix)
#     }
#     res = numDeriv::jacobian(func, coef(obj))
#     G = lapply(seq_len(ncol(res)), function(j) matrix(res[, j], ncol = ncol(V)))
#     G[[1 + length(G)]] = 2 * V  # gradient wrt log sigma
#     G
# }



#--------------------------------------------------------------

### new way to get jacobians for gls models
gls_grad = function(object, call, data, V) {
    if (is.null(data)) 
        data = environment(eval(call$model))
    obj = object$modelStruct
    conLin = object
    class(conLin) = class(obj)
    X = model.matrix(eval(call$model), data = data)
    y = eval(eval(call$model)[[2]], envir = data)
    conLin$Xy = cbind(X, y)
    conLin$fixedSigma = FALSE
    func = function(x) {
        obj = nlme::`coef<-`(obj, value = x)
        tmp = nlme::glsEstimate(obj, conLin)
        .get.lt(crossprod(tmp$sigma * tmp$varBeta))  # lower triangular form
    }
    res = numDeriv::jacobian(func, coef(obj))
    G = lapply(seq_len(ncol(res)), function(j) .lt2mat(res[, j]))
    G[[1 + length(G)]] = 2 * V  # gradient wrt log sigma
    G
}

### gls objects (nlme package)
#' @exportS3Method recover_data gls
recover_data.gls = function(object, data, ...) {
    fcall = object$call
    if (!is.null(wts <- fcall$weights)) {
        wts = nlme::varWeights(object$modelStruct)
        fcall$weights = NULL
    }
    trms = delete.response(terms(nlme::getCovariateFormula(object)))
    attr(trms, "predvars") = attr(delete.response(terms(object)), "predvars")
    # above copies scaling info for scale() and poly() terms
    
    result = recover_data.call(fcall, trms, object$na.action, data = data, ...)
    if (!is.null(wts))
        result[["(weights)"]] = wts
    if (!missing(data))
        attr(result, "misc") = list(data = data)
    attr(result, "pass.it.on") = TRUE
    result
}

#' @exportS3Method emm_basis gls          
emm_basis.gls = function(object, trms, xlev, grid, 
                         mode = c("auto", "df.error", "satterthwaite", "appx-satterthwaite", "boot-satterthwaite", "asymptotic"), 
                         extra.iter = 0, options, misc, ...) {
    contrasts = object$contrasts
    m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
    X = model.matrix(trms, m, contrasts.arg = contrasts)

    # submodel support (not great because I don't know how to retrieve the weights)
    mf = model.frame(trms, attr(object, "data"), na.action = na.pass, xlev = xlev)
    mm = model.matrix(trms, mf, contrasts.arg = contrasts)
    mm = .cmpMM(mm, assign = attr(mm, "assign"))
    
    tmp = coef(object)
    bhat = rep(NA, ncol(X))
    bhat[match(names(tmp), colnames(X), nomatch = 0)] = tmp
    V = .my.vcov(object, ...)
    if (any(is.na(bhat)))
        nbasis = estimability::nonest.basis(mm)
    else
        nbasis = estimability::all.estble
    mode = match.arg(mode)
    if (mode == "boot-satterthwaite") mode = "appx-satterthwaite"  # backward compatibility
    if (!is.null(options$df)) # if we're gonna override the df anyway, keep it simple!
        mode = "df.error"
    if (mode == "auto")
        mode = ifelse(is.null(object$apVar), "df.error", "satterthwaite")
    if (!is.matrix(object$apVar))
        mode = "df.error"
    if (mode %in% c("satterthwaite", "appx-satterthwaite")) {
        data = if(is.null(misc$data))
            eval(object$call$data, parent.frame(2))
        else
            misc$data
        misc = list()
        chk = attr(object$apVar, "Pars")
        if(max(abs(coef(object$modelStruct) - chk[-length(chk)])) > .001) {
            message("Analytical Satterthwaite method not available; using appx-satterthwaite")
            mode = "appx-satterthwaite"
        }
        if (mode == "appx-satterthwaite") {
            G = try(gradV.kludge(object, "varBeta", call = object$call$model,
                                 data = data, extra.iter = extra.iter),
                    silent = TRUE)
        }
        else
            G = try(gls_grad(object, object$call, data, V))
        if (inherits(G, "try-error")) {
            sugg = ifelse(mode == "satterthwaite", "appx-satterthwaite", "df.error")
            stop("Can't estimate Satterthwaite parameters.\n",
                 "  Try adding the argument 'mode = \"", sugg, "\"'", call. = FALSE)
        }
        dfargs = list(V = V, A = object$apVar, G = G)
        dffun = function(k, dfargs) {
            est = tcrossprod(crossprod(k, dfargs$V), k)
            g = sapply(dfargs$G, function(M) tcrossprod(crossprod(k, M), k))
            varest = tcrossprod(crossprod(g, dfargs$A), g)
            2 * est^2 / varest
        }
    }
    else if (mode %in%  c("df.error", "asymptotic")) {
        df = ifelse(mode == "asymptotic",
                    Inf, 
                    object$dims$N - object$dims$p - length(unlist(object$modelStruct)))
        dfargs = list(df = df)
        dffun = function(k, dfargs) dfargs$df
    }
    attr(dffun, "mesg") = mode
    
    list(X=X, bhat=bhat, nbasis=nbasis, V=V, dffun=dffun, dfargs=dfargs, misc=misc,
         model.matrix = mm)
}



#--------------------------------------------------------------
### polr objects (MASS package)
#' @exportS3Method recover_data polr
recover_data.polr = function(object, ...)
    recover_data.clm(object, ...)

#' @exportS3Method emm_basis polr         
emm_basis.polr = function(object, trms, xlev, grid, 
                          mode = c("latent", "linear.predictor", "cum.prob", "exc.prob", "prob", "mean.class"), 
                          rescale = c(0,1), ...) {
    mode = match.arg(mode)
    contrasts = object$contrasts
    m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
    X = model.matrix(trms, m, contrasts.arg = contrasts)
    # Strip out the intercept (borrowed code from predict.polr)
    xint = match("(Intercept)", colnames(X), nomatch = 0L)
    if (xint > 0L) 
        X = X[, -xint, drop = FALSE]
    bhat = c(coef(object), object$zeta)
    V = .my.vcov(object, ...)
    k = length(object$zeta)
    if (mode == "latent") {
        X = rescale[2] * cbind(X, matrix(- 1/k, nrow = nrow(X), ncol = k))
        bhat = c(coef(object), object$zeta - rescale[1] / rescale[2])
        misc = list(offset.mult = rescale[2])
    }
    else {
        j = matrix(1, nrow=k, ncol=1)
        J = matrix(1, nrow=nrow(X), ncol=1)
        X = cbind(kronecker(-j, X), kronecker(diag(1,k), J))
        link = object$method
        if (link == "logistic") link = "logit"
        misc = list(ylevs = list(cut = names(object$zeta)), 
                    tran = link, inv.lbl = "cumprob", offset.mult = -1)
        if (mode != "linear.predictor") {
            # just use the machinery we already have for the 'ordinal' package
            misc$mode = mode
            misc$postGridHook = ".clm.postGrid"
        }
    }
    misc$respName = as.character.default(terms(object))[2]
    nbasis = estimability::all.estble
    dffun = function(...) Inf
    list(X=X, bhat=bhat, nbasis=nbasis, V=V, dffun=dffun, dfargs=list(), misc=misc)
}




#--------------------------------------------------------------
### survreg objects (survival package)
#' @exportS3Method recover_data survreg
recover_data.survreg = function(object, ...) {
    fcall = object$call
    trms = delete.response(terms(object))
    # I'm gonna delete any terms involving cluster(), frailty(); keep strata()
    mod.elts = dimnames(attr(trms, "factor"))[[2]]
    tmp = grep("cluster\\(|frailty", mod.elts)
    if (length(tmp))
        trms = trms[-tmp]
    recover_data(fcall, trms, object$na.action, pwts = weights(object), ...)
}

# Seems to work right in a little testing.
# However, it fails sometimes if I update the model 
# with a subset argument. Workaround: just fitting a new model
#' @exportS3Method emm_basis survreg      
emm_basis.survreg = function(object, trms, xlev, grid, ...) {
    # Much of this code is adapted from predict.survreg
    bhat = object$coefficients
    k = length(bhat) - sum(is.na(bhat))
    V = .my.vcov(object, ...)[seq_len(k), seq_len(k), drop=FALSE]
    # ??? not used... is.fixeds = (k == ncol(object$var))
    ### zap-out factors in xlev not needed by model.frame
    xlev[setdiff(names(xlev), rownames(attr(trms, "factors")))] = NULL
    m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)    
    # X = model.matrix(object, m) # This is what predict.survreg does
    # But I have manipulated trms, so need to make sure things are consistent
    X = model.matrix(trms, m, contrasts.arg = object$contrasts)
    usecols = intersect(colnames(X), names(bhat))
    bhat = bhat[usecols]  # in case ref_grid code excluded some levels...
    X = X[, usecols, drop=FALSE]
    nbasis = estimability::nonest.basis(model.matrix(object)[, usecols, drop = FALSE])
    dfargs = list(df = object$df.residual)
    dffun = function(k, dfargs) dfargs$df
    if (object$dist %in% c("exponential","weibull","loglogistic","loggaussian","lognormal")) 
        misc = list(tran = "log", inv.lbl = "response")
    else 
        misc = list()
    misc$postGridHook = .notran2   # removes "Surv()" as response transformation
    list(X=X, bhat=bhat, nbasis=nbasis, V=V, dffun=dffun, dfargs=dfargs, misc=misc)
}



#--------------------------------------------------------------
###  coxph objects (survival package)
#' @exportS3Method recover_data coxph
recover_data.coxph = function(object, ...) 
    recover_data.survreg(object, ...)

#' @exportS3Method emm_basis coxph        
emm_basis.coxph = function (object, trms, xlev, grid, ...) 
{
    object$dist = "doesn't matter"
    result = emm_basis.survreg(object, trms, xlev, grid, ...)
    result$dfargs$df = Inf
    nms = colnames(result$X)
    # delete columns for intercept and main effects of strata
    zaps = which(nms %in% setdiff(nms, names(result$bhat)))
    if(length(zaps) > 0)
        result$X = result$X[, -zaps, drop = FALSE]
    ### result$X = result$X - rep(object$means, each = nrow(result$X))
    result$misc$tran = "log"
    result$misc$inv.lbl = "hazard"
    result
}

.notran2 = function(object, ...) {
    for (nm in c("tran", "tran2"))
        if(!is.null(object@misc[[nm]]) && object@misc[[nm]] == "Surv") object@misc[[nm]] = NULL
    object
}

# Note: Very brief experimentation suggests coxph.penal also works.
# This is an extension of coxph


#--------------------------------------------------------------
###  coxme objects ####
### Greatly revised 6-15-15 (after version 2.18)
#' @exportS3Method recover_data coxme
recover_data.coxme = function(object, ...) 
    recover_data.survreg(object, ...)

#' @exportS3Method emm_basis coxme        
emm_basis.coxme = function(object, trms, xlev, grid, ...) {
    bhat = coxme::fixef(object)
    k = length(bhat)
    V = .my.vcov(object, ...)[seq_len(k), seq_len(k), drop = FALSE]
    m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
    X = model.matrix(trms, m)
    X = X[, -1, drop = FALSE] # remove the intercept
    # scale the linear predictor
    for (j in seq_along(X[1, ]))
        X[, j] = (X[, j] - object$means[j]) ### / object$scale[j]
    nbasis = estimability::all.estble
    dffun = function(k, dfargs) Inf
    misc = list(tran = "log", inv.lbl = "hazard")
    misc$postGridHook = .notran2   # removes "Surv()" as response transformation
    list(X = X, bhat = bhat, nbasis = nbasis, V = V, dffun = dffun, 
         dfargs = list(), misc = misc)
}


###  special vcov prototype for cases where there are several vcov options
###  e.g., gee, geeglm, geese
.named.vcov = function(object, method, ...)
    UseMethod(".named.vcov")

# default has optional idx of same length as valid and if so, idx indicating 
#   which elt of valid to use if matched
# Ex: valid = c("mammal", "fish", "rat", "dog", "trout", "perch")
#     idx   = c(   1,        2,     1,     1,       2,       2)
#     -- so ultimately results can only be "mammal" or "fish"
# nonmatches revert to 1st elt.
.named.vcov.default = function(object, method, valid, idx = seq_along(valid), ...) {
    if (!is.character(method)) { # in case vcov. arg was matched by vcov.method {
        V = .my.vcov(object, method)
        method = "user-supplied"
    }
    else {
        i = pmatch(method, valid, 1)
        method = valid[idx[i]]
        V = object[[method]]
    }
    attr(V, "methMesg") = paste("Covariance estimate used:", method)
    V
}

# general-purpose emm_basis function for GEEs
.emmb.geeGP = function(object, trms, xlev, grid, vcov.method, valid, idx = seq_along(valid), ...) {
    m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
    X = model.matrix(trms, m, contrasts.arg = object$contrasts)
    bhat = coef(object)
    V = .named.vcov(object, vcov.method, valid, idx, ...)
    
    if (sum(is.na(bhat)) > 0)
        nbasis = estimability::nonest.basis(object$qr)
    else
        nbasis = estimability::all.estble
    
    misc = .std.link.labels(object$family, list())
    misc$initMesg = attr(V, "methMesg")
    dffun = function(k, dfargs) Inf
    dfargs = list()
    list(X=X, bhat=bhat, nbasis=nbasis, V=V, dffun=dffun, dfargs=dfargs, misc=misc)
}

#---------------------------------------------------------------
###  gee objects  ####


#' @exportS3Method recover_data gee
recover_data.gee = function(object, ...)
    recover_data.lm(object, frame = NULL, ...)

#' @exportS3Method emm_basis gee          
emm_basis.gee = function(object, trms, xlev, grid, vcov.method = "robust.variance", ...)
    .emmb.geeGP(object, trms, xlev, grid, vcov.method, 
                valid = c("robust.variance", "naive.variance"))

###  geepack objects  ####
#' @exportS3Method recover_data geeglm
recover_data.geeglm = function(object, ...)
    recover_data.lm(object, ...)

#' @exportS3Method emm_basis geeglm       
emm_basis.geeglm = function(object, trms, xlev, grid, vcov.method = "vbeta", ...) {
    m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
    X = model.matrix(trms, m, contrasts.arg = object$contrasts)
    bhat = coef(object)
    V = .named.vcov(object$geese, vcov.method, 
                    valid = c("vbeta", "vbeta.naiv","vbeta.j1s","vbeta.fij","robust","naive"), 
                    idx = c(1,2,3,4,1,2))
    
    if (sum(is.na(bhat)) > 0)
        nbasis = estimability::nonest.basis(object$qr)
    else
        nbasis = estimability::all.estble
    
    misc = .std.link.labels(object$family, list())
    misc$initMesg = attr(V, "methMesg")
    dffun = function(k, dfargs) dfargs$df
    dfargs = list(df = df.residual(object))
    list(X=X, bhat=bhat, nbasis=nbasis, V=V, dffun=dffun, dfargs=dfargs, misc=misc)
}


#' @exportS3Method recover_data geese
recover_data.geese = function(object, ...) {
    fcall = object$call
    # what a pain - we need to reconstruct the terms component
    args = as.list(fcall[-1])
    na.action = object$na.action
    #trms = terms.formula(fcall$formula)
    if (!is.null(args$data)) {
        data = eval(args$data, parent.frame())
        trms = terms(model.frame(fcall$formula, data = data))
    } else {
        trms = terms(model.frame(fcall$formula))
    }
    recover_data(fcall, delete.response(trms), na.action, ...)
}

#' @exportS3Method emm_basis geese        
emm_basis.geese = function(object, trms, xlev, grid, vcov.method = "vbeta", ...) {
    m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
    X = model.matrix(trms, m, contrasts.arg = object$contrasts)
    bhat = object$beta
    V = .named.vcov(object, vcov.method, 
                    valid = c("vbeta", "vbeta.naiv","vbeta.j1s","vbeta.fij","robust","naive"), 
                    idx = c(1,2,3,4,1,2))

    # We don't have the qr component - I'm gonna punt for now
     if (sum(is.na(bhat)) > 0)
         warning("There are non-estimable functions, but estimability is NOT being checked")
#         nbasis = estimability::nonest.basis(object$qr)
#     else
        nbasis = estimability::all.estble
    
    misc = list()
    if (!is.null(fam <- object$call$family))
        misc = .std.link.labels(eval(fam)(), misc)
    misc$initMesg = attr(V, "methMesg")
    dffun = function(k, dfargs) Inf
    dfargs = list()
    list(X=X, bhat=bhat, nbasis=nbasis, V=V, dffun=dffun, dfargs=dfargs, misc=misc)
}


### glmtoolbox package -- we just pretend it's lm and it seems to work

#' @exportS3Method recover_data glmgee
recover_data.glmgee = function(object, ...) {
    recover_data.lm(object, ...)
}

#' @exportS3Method emm_basis glmgee
emm_basis.glmgee = function(object, trms, xlev, grid, vcov.method = "robust", ...) {
    vcov. = if(is.character(vcov.method))
        vcov(object, type = vcov.method, ...)
    else
        vcov.method
    class(object) = c("glm", "lm") # pretend it's glm
    rtn = emm_basis(object, trms = trms, xlev = xlev, grid = grid, vcov. = vcov., ...)
    rtn$misc$initMesg = paste("Covariance estimate used:",
        ifelse(is.character(vcov.method), vcov.method, "user-supplied"))
    rtn
}



### survey package
# svyglm class
#' @exportS3Method recover_data svyglm
recover_data.svyglm = function(object, data = NULL, ...) {
    if (is.null(data)) {
        env = environment(terms(object))
        des = eval(object$call$design, envir = env)
        data = eval(des$call$data, envir = env)
    }
    recover_data.lm(object, data = data, frame = object$model, ...)
}

# inherited emm_basis.lm method works fine




### ----- Auxiliary routines -------------------------
# Provide for vcov. argument in ref_grid call, which could be a function or a matrix

.statsvcov = function(object, ...)
    stats::vcov(object, complete = FALSE, ...)

#' @rdname extending-emmeans
#' @order 36
#' @param vcov. Function or matrix that returns a suitable covariance matrix.
#' The default is \code{.statsvcov} which is \code{stats::vcov}. The \code{.my.vcov} 
#' function should be called in place of \code{\link{vcov}}, and it supports the user 
#' being able to specify a different matrix or function via the
#' optional \code{vcov.} argument.
#' @export
.my.vcov = function(object, vcov. = .statsvcov, ...) {
    if (is.function(vcov.))
        vcov. = vcov.(object, ...)
    vcov. = try(as.matrix(vcov.), silent = TRUE)
    if (inherits(vcov., "try-error") || (nrow(vcov.) != ncol(vcov.)))
        stop("vcov. must be a function or a square matrix")
    vcov.
}

#' @rdname extending-emmeans
#' @order 30
#' @param fam Result of call to \code{family(object)}
#' @param misc A \code{list} intended for the \code{@misc} slot of an \code{emmGrid} object
#' @return \code{.std.link.llabels} returns a modified version of \code{misc}
#'   with the appropriate information included corresponding to the information in \code{fam}
#' @export
.std.link.labels = function(fam, misc) {
    if (is.null(fam) || !is.list(fam))
        return(misc)
    if (fam$link == "identity")
        return(misc)
    misc$tran = fam$link
    misc$inv.lbl = "response"
    if (length(grep("binomial", fam$family)) == 1)
        misc$inv.lbl = "prob"
    else if (length(grep("poisson", fam$family)) == 1)
        misc$inv.lbl = "rate"
    if(length(grep("gaussian", fam$family)) == 0)
        misc$sigma = NA
    misc
}

##############################################################################
#    Copyright (c) 2012-2017 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

### Code for an enhancement of 'glht' in 'multcomp' package
### Provides for using 'emm' in similar way to 'mcp'
### This is implemented via the class "emmlf" -- linear functions for emmeans

## NOTE: Registration of S3 methods for glht is done dynamically in zzz.R

# emm(specs) will be used as 'linfct' argument in glht
# all we need to do is class it and save the arguments

#' Support for \code{multcomp::glht}
#' 
#' These functions and methods provide an interface between \pkg{emmeans} and
#' the \code{multcomp::glht} function for simultaneous inference provided
#' by the \pkg{multcomp} package.
#' 
#' 
#' @rdname glht-support
#' @aliases glht-support glht.emmGrid glht.emmlf modelparm.emmwrap
#' @param ... In \code{emm}, the \code{specs}, \code{by}, and \code{contr}
#'   arguments you would normally supply to \code{\link{emmeans}}. Only
#'   \code{specs} is required. Otherwise, arguments are passed to other
#'   methods. You may also include a \code{which} argument; see Details.
#'   
#' @section Details for \code{emm}:  
#' \code{emm} is meant to be called only \emph{from} \code{"glht"} as its second
#' (\code{linfct}) argument. It works similarly to \code{multcomp::mcp},
#' except with \code{specs} (and optionally \code{by} and \code{contr}
#' arguments) provided as in a call to \code{\link{emmeans}}.
#' 
#' If the specifications in \code{...} would result in a list (i.e., an
#' \code{emm_list} object), then by default, only the last element of that list
#' is passed to \code{glht}. However, if \code{...} contains a \code{which}
#' argument consisting of integer values, the list elements with those indexes
#' are selected and combined and passed on to \code{glht}. No checking is done
#' on whether the indexes are valid, and the keyword \code{which} must be spelled-out.
#' 
#'
#' @return \code{emm} returns an object of an intermediate class for which
#'   there is a \code{multcomp::glht} method.
#' @export
emm <- function(...) {
    result <- list(...)
    class(result) <- "emmlf"
    result
}

# New S3 method for emmlf objects
glht.emmlf <- function(model, linfct, ...) {
    # Pass the arguments we should pass to ref_grid:
    args = linfct
    args[[1]] = model
    names(args)[1] = "object"
    # Now pass the ref_grid to emmeans:
    linfct$object <- do.call("ref_grid", args)
    emmo <- do.call("emmeans", linfct)
    if (is.list(emmo)) {
        if(is.null(linfct$which))
            emmo = emmo[[length(emmo)]]
        else {
            emmo = do.call(rbind.emmGrid, emmo[linfct$which])
            emmo@misc$by.vars = linfct$by
        }
    }
    # Then call the method for emmo objject
    glht.emmGrid(model, emmo, ...)
}


# S3 method for an emmGrid object
# Note: model is redundant, really, so can be omitted
# See related roxygen stuff just before glht.emmlf
glht.emmGrid <- function(model, linfct, by, ...) {
    .requireNS("multcomp", sQuote("glht")," requires ", dQuote("multcomp"), 
               " to be installed", call. = FALSE)
    object = linfct # so I don't get confused
    if (missing(model)) 
        model = .cls.list("emmwrap", object = object)
    args = list(model = model, ...)
    # add a df value if not supplied
    if (is.null(args$df)) {
        df = summary(linfct)$df
        df[is.infinite(df)] = NA
        if(any(!is.na(df))) {
            args$df = max(1, as.integer(mean(df, na.rm=TRUE) + .25))
            if (any(args$df != df))
                message("Note: df set to ", args$df)
        }
    }
    if (missing(by)) by = object@misc$by.vars
    
    nms = setdiff(names(object@grid), c(by, ".offset.", ".freq.", ".wgt."))
    if (is.null(object@misc$estHook))
        lf = object@linfct
    else # custom estimation setup - use the grid itself as the parameterization
        lf = diag(1, nrow(object@linfct))
    dimnames(lf)[[1]] = as.character(interaction(object@grid[, nms], sep=", "))
    
    if (is.null(by)) {
        args$linfct = lf
        return(do.call(multcomp::glht, args))
    }
    
    # (else...)
    by.rows = .find.by.rows(object@grid, by)
    result = lapply(by.rows, function(r) {
        args$linfct = lf[r, , drop=FALSE]
        do.call(multcomp::glht, args)
    })
    bylevs = lapply(by, function(byv) unique(object@grid[[byv]]))
    names(bylevs) = by
    bygrid = do.call("expand.grid", bylevs)
    levlbls = unname(lapply(by, function(byv) paste(byv, "=", bygrid[[byv]])))
    levlbls$sep = ", "
    names(result) = do.call("paste", levlbls)
    class(result) = c("glht_list", "list")
    result
}

### as. glht -- convert my object to glht object
#' @rdname glht-support
#' @param object An object of class \code{emmGrid} or \code{emm_list}
#' 
#' @return \code{as.glht} returns an object of class \code{glht} or \code{glht_list}
#'   according to whether \code{object} is of class \code{emmGrid} or \code{emm_list}. 
#'   See Details below for more on \code{glht_list}s.
#'   
#' @section Details for \code{as.glht}:
#' When no \code{by} variable is in force, we obtain a \code{glht} object; otherwise
#' it is a \code{glht_list}. The latter is defined in \pkg{emmeans}, not \pkg{multcomp},
#' and is simply a \code{list} of \code{glht} objects. 
#' Appropriate convenience methods \code{coef},
#' \code{confint}, \code{plot}, \code{summary}, and \code{vcov} are provided,
#' which simply apply the corresponding \code{glht} methods to each member.
#' 
#' @note The multivariate-\eqn{t} routines used by \code{glht} require that all
#'   estimates in the family have the same integer degrees of freedom. In cases
#'   where that is not true, a message is displayed that shows what df is used.
#'   The user may override this via the \code{df} argument.
#' 
#' @examples
#' if(require(multcomp, quietly = TRUE)) 
#'     emm_example("glht-multcomp") 
#'     # Use emm_example("glht-multcomp", list = TRUE) # to see just the code
#'     
#' @export
as.glht <- function(object, ...) {
    UseMethod("as.glht")
}

#' @method as.glht default
#' @export
as.glht.default <- function(object, ...)
    stop("Cannot convert an object of class ", sQuote(class(object)[1]),
         " to a ", sQuote("glht"), " object")

#' @rdname glht-support
#' @method as.glht emmGrid
#' @export
as.glht.emmGrid <- function(object, ...)
    glht.emmGrid( , object, ...)   # 1st arg not necessary

#' @method as.glht emm_list
#' @export
as.glht.emm_list <- function(object, ..., which = 1)
    as.glht(object[[which]], ...)


# S3 modelparm method for emmwrap (S3 wrapper for an emmGrid obj - see glht.emmGrid)
#--- dynamically registered in zzz.R --- #' @export
modelparm.emmwrap <- function(model, coef., vcov., df, ...) {
    object = model$object
    if (is.null(object@misc$estHook)) {
        bhat = object@bhat
        V = object@V
    }
    else { # Have custom vcov and est methods. Use the grid itself as parameterization
        bhat = predict(object)
        V = vcov(object)
    }
    if(missing(df) || is.na(df) || is.infinite(df))
        df = 0
    .cls.list("modelparm", coef = bhat, vcov = V,
                df = df, estimable = !is.na(bhat))
    # This is NOT what we mean by 'estimable', but it is what glht wants...
}


# S3 methods for glht_list

### Doesn't work so excluded...
# cld.glht_list = function(object, ...)
#     lapply(object, cld, ...)

#' @method coef glht_list
#' @export
coef.glht_list = function(object, ...)
    lapply(object, coef, ...)

#' @method confint glht_list
#' @export
confint.glht_list = function(object, ...)
    lapply(object, confint, ...)

#' @method plot glht_list
#' @export
plot.glht_list = function(x, ...)
    lapply(x, plot, ...)

#' @method summary glht_list
#' @export
summary.glht_list = function(object, ...)
    lapply(object, summary, ...)

#' @method vcov glht_list
#' @export
vcov.glht_list = function(object, ...)
    lapply(object, vcov, ...)

### Rudimentary support for brms. 
### Obviously this is way less than is needed, but it does support simpler models

#xxxx' @importFrom brms parse_bf
#' @exportS3Method recover_data brmsfit
recover_data.brmsfit = function(object, data, ...) {
    bt = brms::brmsterms(formula(object))
    if (!inherits(bt, "brmsterms"))
        stop("This model is currently not supported.")
    mt = attr(model.frame(bt$dpars$mu$fe, data = object$data), "terms")
    recover_data(call("brms"), mt, "na.omit", data = object$data, ...)
}

#' @exportS3Method emm_basis brmsfit
emm_basis.brmsfit = function(object, trms, xlev, grid, vcov., ...) {
    m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
    contr = lapply(object$data, function(.) attr(., "contrasts"))
    contr = contr[!sapply(contr, is.null)]
    X = model.matrix(trms, m, contrasts.arg = contr)
    ###nm = gsub("(Intercept)", "Intercept", dimnames(X)[[2]], fixed = TRUE)
    nm = eval(parse(text = "brms:::rename(colnames(X))"))
    V = vcov(object)[nm, nm, drop = FALSE]
    nbasis = estimability::all.estble
    dfargs = list()
    dffun = function(k, dfargs) Inf
    misc = .std.link.labels(brms::parse_bf(formula(object))$dpars$mu$family, list())
    post.beta = as.matrix(object, pars = paste0("b_", nm), exact = TRUE)
    bhat = apply(post.beta, 2, mean)
    
    list(X=X, bhat=bhat, nbasis=nbasis, V=V, dffun=dffun, dfargs=dfargs, 
         misc=misc, post.beta=post.beta)
}

##############################################################################
#    Copyright (c) 2012-2024 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

# Methods for emm_list objects


# First, here is documentation for the emm_list class


#' The \code{emm_list} class
#' 
#' An \code{emm_list} object is simply a list of
#' \code{\link[=emmGrid-class]{emmGrid}} objects. Such a list is returned,
#' for example, by \code{\link{emmeans}} with a two-sided formula or a list as its
#' \code{specs} argument. Several methods for this class are provided, as detailed below.
#' Typically, these methods just quietly do the same thing as their \code{emmGrid}
#' methods, using the first element of the list. You can specify \code{which}
#' to select a different element, or just run the corresponding \code{emmGrid}
#' method on \code{object[[k]]}.
#' 
#' @param object,x an object of class \code{emm_list}
#' @param ... additional arguments passed to corresponding \code{emmGrid}
#'   method. In addition, the user may include a logical argument \code{drop}
#'   that is akin to \code{\link{drop}} and the argument of the same name in
#'   subscripting matrices and data frames. When \code{drop} is \code{TRUE} (the
#'   default), then when the result is a \code{list} of length 1, the
#'   \code{list} structure is removed.
#' @param which integer vector specifying which elements to select;
#'   if \code{NULL},
#'   we try to guess which elements make sense. Usually, this is all elements having 
#'   names that start with \sQuote{em} or \sQuote{ls},
#'   or the first element if no matches are found. However, in \code{coef.emm_list},
#'   these are the ones we \emph{exclude}.
#' 
#' @return a \code{list} of objects returned by the corresponding \code{emmGrid}
#'   method (thus, often, another \code{emm_list} object). However, if
#'   \code{which} has length 1, the one result is not wrapped in a list.
#' 
#' @rdname emm_list-object
#' @name emm_list
#' @order 1
#' @examples
#' mod <- lm(conc ~ source, data = pigs)
#' obj <- emmeans(mod, pairwise ~ source)
#' 
#' linfct(obj)
#' 
#' coef(obj)     # done only for the contrasts
#' 
#' contrast(obj, "consec")  # done only for the means
#' 
#' contrast(obj, "eff", drop = FALSE)   # kept as a list
NULL

# Internal utility to noisily return one of an emm_list
# Call with ... argument so that message is suppressed if we specify 'which'
.chk.list = function(object, which, ...) {
    if (inherits(object, "emm_list")) {
        if (missing(which))
            which = 1
        object = object[[which]]
    }
    object
}

# My own lapply() function that drops when the dimension is 1
.lapply = function(X, ..., drop = TRUE) {
    oldClass(X) = "list"
    rtn = lapply(X, ...)
    if (drop && (length(rtn) == 1))   {
        rtn = rtn[[1]]
    }
    else if(length(rtn) != 0)                   {
        cls = ifelse(class(rtn[[1]])[1] == "emmGrid", "emm_list", "summary_eml")
        class(rtn) = c(cls, "list")
    }
    rtn
}


#' @export
#' @method str emm_list
#' @rdname emm_list-object
#' @order 12
str.emm_list = function(object, ...) {
    for(nm in names(object)) {
        cat(paste("$", nm, "\n", sep=""))
        str(object[[nm]])
        cat("\n")
    }
}


# summary.emm_list et al take an argument 'which' that allows doing a subset
# Each returns a regular 'list'


#' @export
#' @method summary emm_list
#' @return \code{summary.emm_list} returns an object
#' of class \code{summary_eml}, which is a list of \code{summary_emm}
#' objects.
#' @rdname emm_list-object
#' @order 13
summary.emm_list <- function(object, ..., which = seq_along(object)) {
    # .lapply(object[which], function(x) {
    #     if (inherits(x, "summary_emm"))  x
    #     else summary.emmGrid(x, ...)
    # })
    if(length(which) == 1)
        summary.emmGrid(object[[which]], ...)
    else
        .lapply(object[which], \(x) summary.emmGrid(x, ...))
}

#' @export
summary.summary_eml = function(object, ...) object

#' @export
#' @rdname emm_list-object
#' @order 25
#' @param row.names,optional Required arguments of \code{as.data.frame}, ignored
#' @method as.data.frame summary_eml
as.data.frame.summary_eml = function(x, row.names = NULL, optional = FALSE, which, ...) {
    rbind(x, which = which)
}

#' @export
print.summary_eml = function(x, ...) {
    attr(x, "class") = NULL
    print(x)
}

#' @export
#' @method print emm_list
#' @rdname emm_list-object
#' @order 14
#' @note No \code{export} option is provided for printing an \code{emm_list}
#' (see \code{\link{print.emmGrid}}). If you wish to export these objects, you 
#' must do so separately for each element in the list.
#'
print.emm_list = function(x, ...) {
    print(summary(x, ...))
}

### Utility fcn to identify which elements' names start with "em" or "lm"
### this only operates if which == NULL
.guess.which = guess.which = function(object, which) {
    if(!is.null(which))
        return(which)
    rtn = which(substr(paste0(names(object), "xx"), 1, 2) %in% c("em", "ls"))
    if(length(rtn) == 0)
        rtn = 1
    rtn
}

#' @export
#' @method contrast emm_list
#' @rdname emm_list-object
#' @order 3
contrast.emm_list = function(object, ... , which = NULL) {
    which = .guess.which(object, which)
    rtn = .lapply(object[which], contrast, ...)
    if(is.list(rtn))
        names(rtn) = paste("contrasts of", names(rtn))
    rtn
}

#' @export
#' @method pairs emm_list
#' @rdname emm_list-object
#' @order 4
pairs.emm_list = function(x, ..., which = NULL) {
    which = .guess.which(x, which)
    rtn = .lapply(x[which], pairs, ...)
    names(rtn) = paste("comparisons of", names(rtn))
    rtn
}

#' @export
#' @method test emm_list
#' @rdname emm_list-object
#' @order 6
test.emm_list = function(object, ..., which = seq_along(object)) {
    which = .guess.which(object, which)
    .lapply(object[which], test, ...)
}

#' @export
#' @method confint emm_list
#' @rdname emm_list-object
#' @order 7
confint.emm_list = function(object, ..., which = seq_along(object)) {
    which = .guess.which(object, which)
    .lapply(object[which], confint, ...)
}

#' @export
#' @method coef emm_list
#' @rdname emm_list-object
#' @order 9
coef.emm_list = function(object, ..., which = NULL) {
    which = - .guess.which(object, which)
    .lapply(object[which], coef, ...)
}

#' @export
#' @method linfct emm_list
#' @rdname emm_list-object
#' @order 11
linfct.emm_list = function(object, ..., which = seq_along(object)) {
    .lapply(object[which], \(x) attr(x, "linfct"), ...)
}


#' @export
#' @method plot emm_list
#' @rdname emm_list-object
#' @order 8
#' @note The \code{plot} method uses only the first element of \code{which}; the others are ignored.
plot.emm_list = function(x, ..., which = 1) {
    which = .guess.which(x, which)
    plot.emmGrid(x[[which[1]]], ...)
}

#' @rdname rbind.emmGrid
#' @order 23
#' @param which Integer vector of subset of elements to use;
#' if missing, we use all elements. 
#' @return The \code{rbind} method for \code{emm_list} objects simply combines 
#' the \code{emmGrid} objects comprising the first element of \code{...}.
#' Note that the returned object is not yet summarized, so any \code{adjust}
#' parameters apply to the combined \code{emmGrid}.
#' @export
#' @method rbind emm_list
#' @examples
#' 
#' ### Working with 'emm_list' objects
#' mod <- lm(conc ~ source + factor(percent), data = pigs)
#' all <- emmeans(mod, list(src = pairwise ~ source, pct = consec ~ percent))
#' rbind(all, which = c(2, 4), adjust = "mvt")
rbind.emm_list = function(..., which, adjust = "bonferroni") {
    elobj = list(...)[[1]]
    if(!missing(which))
         elobj = elobj[which]
    class(elobj) = c("emm_list", "list")
    update(do.call(rbind.emmGrid, elobj), adjust = adjust)
}

#' @rdname rbind.emmGrid
#' @order 33
#' @return The \code{rbind} method for \code{summary_emm} objects (or a list thereof)
#' returns a single \code{summary_emm} object. This combined object
#' \emph{preserves} any adjusted P values or confidence limits in the
#' original summaries, since those quantities have already been computed.
#' @export
#' @method rbind summary_emm
rbind.summary_emm = function(..., which) {
    slobj = list(...)
    if(!all(sapply(slobj, \(z) inherits(z, "data.frame")))) {
        # workaround to make tern.gee::lsmeans() work
        slobj = lapply(slobj, \(z) if(inherits(z, "data.frame")) data.frame(z) else z)
       return(do.call("rbind", slobj))
    }
    rbind.summary_eml(slobj, which = which)
}

#' 
#' @export
#' @method rbind summary_eml
rbind.summary_eml = function(..., which) {
    x = list(...)[[1]]
    if(!missing(which))
        x = x[which]
    nms.lst = lapply(x, names)
    bys = unique(do.call(c, lapply(x, \(z) attr(z, "by.vars"))))
    pris = unique(do.call(c, lapply(x, \(z) attr(z, "pri.vars"))))
    if (length(x) == 1) {
        attr(x[[1]], "pri.vars") = c(pris, bys)
        attr(x[[1]], "by.vars") = NULL
        return (x[[1]])
    }
    nms = pris = union(bys, pris)
    for (n in nms.lst)
        nms = union(nms, n)
    nums = setdiff(nms, pris)  # numeric columns
    xx = lapply(x, function(df) {
        d = data.frame(matrix(".", nrow = nrow(df), ncol = length(nms),
                       dimnames = list(NULL, nms)))
        d[, nums] = NA
        d[, names(df)] = df
        d
    })
    rtn = do.call("rbind", xx)
    row.names(rtn) = NULL
    class(rtn) = c("summary_emm", "data.frame")
    attr(rtn, "pri.vars") = pris
    attr(rtn, "estName") = attr(x[[1]], "estName")
    mesg = otr.mesg = attr(x[[1]], "mesg")
    for (i in 2:length(x)) {
        mesg = intersect(mesg, attr(x[[i]], "mesg"))
        otr.mesg = union(otr.mesg, attr(x[[i]], "mesg"))
    }
    otr.mesg = setdiff(otr.mesg, mesg)
    if (length(otr.mesg) > 0) 
        mesg = c(mesg, "The following messages apply only to some rows:",
                 paste("*", otr.mesg))
    attr(rtn, "mesg") = mesg
    rtn
}

#' @export
#' @rdname emm_list-object
#' @order 24
#' @return The \code{as.data.frame} methods return a single data frame via
#' \code{as.data.frame(rbind(x))}.
#' See also \code{\link{rbind.emm_list}} and \code{\link{as.data.frame.emmGrid}}
#' @method as.data.frame emm_list
as.data.frame.emm_list = function(x, ...) {
    if (length(x) > 1)
        warning("Note: 'as.data.frame' has combined your ", length(x), " sets of results into one object,\n",
                "and this affects things like adjusted P values. Refer to the annotations.")
    as.data.frame(rbind(x, ..., check.names = FALSE))
}

#' @export
#' @method as.list emm_list
as.list.emm_list = function(x, ...) {
    rtn = list()
    for (nm in names(x))
        rtn[[nm]] = as.list.emmGrid(x[[nm]])
    attr(rtn, "emm_list") = TRUE
    rtn
}

#' @export
#' @return \code{as.emm_list} returns an object of class \code{emm_list}.
#' 
#' @rdname as.emmGrid
#' @order 3
as.emm_list = function(object, ...) {
    if (is.null(attr(object, "emm_list")))
        as.emmGrid(object, ...)
    else
        lapply(object, as.emmGrid, ...)
}


### Others we won't document
#' @exportS3Method update emm_list
update.emm_list = function(object, ...)
    update.emmGrid(object[[1]])

#' @exportS3Method predict emm_list
predict.emm_list = function(object, ...)
    predict.emmGrid(object[[1]], ...)

#' @exportS3Method vcov emm_list
vcov.emm_list = function(object, ...)
    vcov.emmGrid(object[[1]], ...)

xtable.emm_list = function(x, ...)
    xtable.emmGrid(x[[1]], ...)

##############################################################################
#    Copyright (c) 2012-2017 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

### Multinomial modeling

### Example for testing
### From: http://www.ats.ucla.edu/stat/r/dae/mlogit.htm
# library(foreign)
# ml <- read.dta("http://www.ats.ucla.edu/stat/data/hsbdemo.dta")
# library(nnet)
# ml$prog2 <- relevel(ml$prog, ref = "academic")
# test <- multinom(prog2 ~ ses + write, data = ml)
# 

# same as recover_data.lm
#' @exportS3Method recover_data multinom
recover_data.multinom = function(object, ...) {
    fcall = object$call
    recover_data(fcall, delete.response(terms(object)), object$na.action, ...)
}

#' @exportS3Method emm_basis multinom     
emm_basis.multinom = function(object, trms, xlev, grid, 
                              mode = c("prob", "latent"), ...) {
    mode = match.arg(mode)
    bhat = t(coef(object))
    V = .my.vcov(object, ...)
    # NOTE: entries in vcov(object) come out in same order as
    # in as.numeric(bhat), even though latter has been transposed
    k = ifelse(is.matrix(coef(object)), ncol(bhat), 1)
    m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
    X = model.matrix(trms, m, contrasts.arg = object$contrasts)
    # recenter for latent predictions
    pat = (rbind(0, diag(k + 1, k)) - 1) / (k + 1)
    X = kronecker(pat, X)
    nbasis = estimability::all.estble
    nbasis = kronecker(rep(1,k), nbasis)
    misc = list(tran = "clr", inv.lbl = "e^y")   ### misc = list(tran = "log", inv.lbl = "e^y")
    dfargs = list(df = object$edf)
    dffun = function(k, dfargs) dfargs$df
    if(is.null(ylevs <- object$lev))
        ylevs = seq_len(k + 1)
    ylevs = list(class = ylevs)
    if (is.null(ylevs)) ylevs = list(class = seq_len(k))
    names(ylevs) = as.character.default(eval(object$call$formula, environment(trms))[[2]])[1]
    misc$ylevs = ylevs
    if (mode == "prob")
        misc$postGridHook = .multinom.postGrid
    list(X = X, bhat = as.numeric(bhat), nbasis = nbasis, V = V, 
         dffun = dffun, dfargs = dfargs, misc = misc)
}

# post-processing of ref_grid for "prob" mode
# also allows simulated outcomes
## Note - now that we have mvregrid, we could just pass the latent grid
## through with the clrInv transform. But this works so I'm just leaving it here
.multinom.postGrid = function(object, N.sim, ...) {
    linfct = object@linfct
    misc = object@misc
    # grid will have multresp as slowest-varying factor...
    idx = matrix(seq_along(linfct[, 1]), 
                 ncol = length(object@levels[[object@roles$multresp]]))
    bhat = as.numeric(idx) # right length, contents will be replaced
    if(sim <- !missing(N.sim)) {
        message("Simulating a sample of size ", N.sim)
        bsamp = mvtnorm::rmvnorm(N.sim, object@bhat, object@V)
        postb = matrix(0, nrow = N.sim, ncol = length(bhat))
    }
    for (i in 1:nrow(idx)) {
        rows = idx[i, ]
        exp.psi = exp(linfct[rows, , drop = FALSE] %*% object@bhat)
        p = as.numeric(exp.psi / sum(exp.psi))
        bhat[rows] = p
        if (sim) {
            ex = exp(linfct[rows, , drop = FALSE] %*% t(bsamp))  # p x N
            px = t(apply(ex, 2, function(x) x / sum(x)))
            postb[, rows] = px
        }
        A = .diag(p) - outer(p, p)    # partial derivs
        linfct[rows, ] = A %*% linfct[rows, ]
    }
    misc$postGridHook = misc$tran = misc$inv.lbl = NULL
    misc$estName = "prob"
    
    object@bhat = bhat
    object@V = linfct %*% tcrossprod(object@V, linfct)
    object@linfct = diag(1, length(bhat))
    object@misc = misc
    if (sim)
        object@post.beta = postb
    object
}


### Support for mclogit::mblogit models???
#' @exportS3Method recover_data mblogit
recover_data.mblogit = function (object, ...) 
{
    recover_data.multinom(object, ...)
}

#' @exportS3Method emm_basis mblogit      
emm_basis.mblogit = function(object, ..., vcov.) {
    object$coefficients = object$coefmat
    object$lev = levels(object$model[[1]])
    object$edf = Inf
    # we have to arrange the vcov elements in row-major order
    if(missing(vcov.))
        vcov. = vcov(object)
    perm = matrix(seq_along(as.numeric(object$coefmat)), 
                  ncol = ncol(object$coefmat))
    perm = as.numeric(t(perm))
    vcov. = vcov.[perm, perm]
    emm_basis.multinom(object, ..., vcov. = vcov.)
}

##############################################################################
#    Copyright (c) 2018-2024 Russell V. Lenth                                     #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

### Quick-and-dirty support for otherwise unsupported models

#' Quick and dirty reference grid
#' 
#' This function may make it possible to compute a reference grid for a model 
#' object that is otherwise not supported.
#' 
#' Usually, you need to provide either \code{object}; or
#' \code{formula}, \code{coef}, \code{vcov}, \code{data}, and perhaps other
#' parameters. It is often fairly straightforward to figure out how to get
#' these from the model \code{object}; see the documentation for the model class that
#' was fitted. Sometimes one or more of these quantities contains extra parameters,
#' and if so, you may need to subset them to make everything conformable. For a given \code{formula} and \code{data},
#' you can find out what is needed via \code{colnames(model.matrix(formula, data))}.
#' (However, for an ordinal model, we expect the first \code{ordinal.dim - 1} coefficients
#' to replace \code{(Intercept)}. And for a multivariate model, we expect \code{coef} 
#' to be a matrix with these row names, and \code{vcov} to have as many rows and columns as
#' the total number of elements of \code{coef}.)
#' 
#' If \code{object} is specified, this function serves as a generic for dispatching
#' a method for \code{object}'s class. See the arguments for \code{qdrg.default} to
#' see how the arguments are determined by default. For many \code{lm}- or \code{glm}-like
#' models, it may suffice override just one or two of these
#' defaults. The possibility of a custom \code{qdrg} method also provides a
#' minimal way for package developers to provide \pkg{emmeans} support: it doesn't
#' allow directly applying \code{emmeans()} on the model, but at least the user
#' can obtain a reference grid, and then go from there. Note that when using a `qdrg` 
#' method, it is best for the user to specify `object =` explicitly in the
#' call, since `object` is not the first argument of `qdrg()`. However, if the
#' first argument is not a formula, the function is retried with it as \code{object}.
#' 
#' The functions \code{\link{qdrg}} and \code{emmobj} are close cousins, in that
#' they both produce \code{emmGrid} objects. When starting with summary
#' statistics for an existing grid, \code{emmobj} is more useful, while
#' \code{qdrg} is more useful when starting from a fitted model.
#'
#' @param formula Formula for the fixed effects
#' @param data Dataset containing the variables in the model
#' @param coef Fixed-effect regression coefficients (must conform to formula)
#' @param vcov Variance-covariance matrix of the fixed effects
#' @param df Error degrees of freedom
#' @param mcmc Posterior sample of fixed-effect coefficients
#' @param object Optional model object. \emph{This rarely works!}; 
#'        but if provided, we try to set 
#'        other arguments based on an expectation that `object` has a similar
#'        structure to `lm` objects. See Details.
#' @param subset Subset of \code{data} used in fitting the model
#' @param weights Weights used in fitting the model
#' @param contrasts List of contrasts specified in fitting the model
#' @param link Link function (character or list) used, if a generalized linear model.
#'     (Note: response transformations are auto-detected from \code{formula})
#' @param qr QR decomposition of the model matrix; used only if there are \code{NA}s
#'     in \code{coef}.
#' @param ordinal \code{list} with elements \code{dim} and \code{mode}.
#'     \code{ordinal$dim} (integer) is the number of levels in an ordinal response. If 
#'     \code{ordinal} is provided, the intercept terms are modified appropriate to predicting 
#'     an ordinal response, as described in \code{vignette("models")}, Section O,
#'     using \code{ordinal$mode} as the \code{mode} argument (if not
#'     provided, \code{"latent"} is assumed).
#'     (All modes are supported except `scale`)
#'     For this to work, we expect
#'     the first \code{ordinal$dim - 1} elements of \code{coef} to be the
#'     estimated threshold parameters, followed by the coefficients for the
#'     linear predictor. Also, if \code{mode} requires back-transforming (e.g.,
#'     \code{"prob"} or \code{"mean.class"}), the user may need to supply the \code{link}
#'     for it to work correctly.
#' @param ... Optional arguments passed to \code{\link{ref_grid}}
#'
#' @return An \code{emmGrid} object constructed from the arguments
#' 
#' @section Rank deficiencies:
#' Different model-fitting packages take different approaches when the model
#' matrix is singular, but \code{qdrg} tries to reconcile them by comparing the
#' linear functions created by \code{formula} to \code{coefs} and \code{vcov}.
#' We may then use the \pkg{estimability} package to determine what quantities
#' are estimable. For reconciling to work properly, \code{coef} should be named
#' and \code{vcov} should have dimnames. To disable this name-matching
#' action, remove the names from \code{coef}, e.g., by calling \code{unname()}.
#' No reconciliation is attempted in multivariate-response cases. For more
#' details on estimability, see the documentation in the \pkg{estimability}
#' package.
#' 
#' @seealso \code{\link{emmobj}} for an alternative way to construct an \code{emmGrid}.
#' 
#' @note For backwards compatibility, an argument \code{ordinal.dim} is invisibly 
#' supported as part of \code{...}, and if present, sets 
#' \code{ordinal = list(dim = ordinal.dim, mode = "latent")}
#' 
#' @export
#' @examples
#' # In these examples, use emm_example(..., list = TRUE) # to see just the code
#' 
#' if (require(biglm, quietly = TRUE)) 
#'     emm_example("qdrg-biglm")
#'     
#' if(require(coda, quietly = TRUE) && require(lme4, quietly = TRUE)) 
#'     emm_example("qdrg-coda")
#'     
#' if(require(ordinal, quietly = TRUE)) 
#'     emm_example("qdrg-ordinal")
#'
qdrg = function(formula, data, coef, vcov, df, mcmc, object,
                subset, weights, contrasts, link, qr, ordinal, ...) {
    
    if (!missing(object)) {
        UseMethod("qdrg", object = object)
    }
    else { 
        if(!inherits(formula, "formula")) {
            cl = match.call()
            cl$object = formula
            cl$formula = NULL
            ### message("Retrying with first argument taken as 'object'...")
            return(eval(cl))
        }
        # back-compatible access to old ordinal.dim arg...
        od = (\(ordinal, ordinal.dim = NULL, ...) {
            if(!missing(ordinal) && is.numeric(ordinal)) ordinal.dim = ordinal
            ordinal.dim
        })(ordinal, ...)
        if(!is.null(od)) ordinal = list(dim = od, mode = "latent")
        
        result = match.call()
        if(missing(formula))
            stop("When 'object' is missing, must at least provide 'formula'")
        result$formula = formula
        if(missing(data))
            result$data = parent.frame()
        else
            result$data = data
        if (!missing(coef)) result$coef = coef
        if (!missing(vcov)) result$vcov = vcov
        if(!missing(df)) result$df = df
        if(missing(contrasts))
            contrasts = attr(model.matrix(result$formula, data = data), "contrasts")
        
        if(!missing(df)) result$df = df
        if(is.null(result$df))
            result$df = Inf
        if(!missing(mcmc)) result$mcmc = mcmc
        if(!missing(subset)) result$subset = subset
        if(!missing(weights)) result$weights = weights
        if(!missing(contrasts)) result$contrasts = contrasts
        if(!missing(link)) result$link = link
        if(!missing(qr) && any(is.na(result$coef))) result$qr = qr
        if(!missing(ordinal)) result$ordinal = ordinal
        
        # make sure "formula" exists, has a LHS and is is 2nd element so that 
        # response transformation can be found
        if (is.null(result$formula))
            stop("No formula; cannot construct a reference grid")
        if(length(result$formula) < 3)
            result$formula = update.formula(result$formula, response ~ .)
        fpos = grep("formula", names(result))[1]
        result = result[c(1, fpos, seq_along(result)[-c(1, fpos)])]
        
        class(result) = c("qdrg", "call")
        ref_grid(result, ...)
    }
}

#' @rdname qdrg
#' @exportS3Method qdrg default
qdrg.default = function(formula = stats::formula(object), 
                        data = try(recover_data.lm(object), silent = TRUE), 
                        coef = stats::coef(object),
                        vcov = stats::vcov(object), 
                        df = stats::df.residual(object), 
                        mcmc, 
                        object,
                        subset, 
                        weights = stats::weights(object), 
                        contrasts = object$contrasts, 
                        link = ifelse(!is.null(lnk<-object$family$link), lnk, object$link),
                        qr = object$qr, 
                        ordinal, 
                        ...) 
{
    
    if(inherits(data, "try-error")) {
        if(is.null(data <- object$data))
            stop("Unable to recover data. You must specify it explicitly in the 'data' argument.")
    }
    if(missing(mcmc)) mcmc = NULL   # for some weird reason, this is needed
    if(missing(subset)) subset = NULL
    if(missing(ordinal)) ordinal = NULL
    qdrg(formula = formula, data = data, coef = coef, vcov = vcov, df = df, mcmc = mcmc,
         subset = subset, weights = weights, contrasts = contrasts, link = link,
         qr = qr, ordinal = ordinal, ...)
}



#' @exportS3Method recover_data qdrg
recover_data.qdrg = function(object, ...) {
    recover_data.call(object, delete.response(terms(object$formula)), object$na.action, ...)
}

#' @exportS3Method vcov qdrg
vcov.qdrg = function(object, ...) 
    object$vcov

#' @exportS3Method emm_basis qdrg         
emm_basis.qdrg = function(object, trms, xlev, grid, ...) {
    m = suppressWarnings(model.frame(trms, grid, na.action = na.pass, xlev = xlev))
    X = model.matrix(trms, m, contrasts.arg = object$contrasts)
    if (is.null(object$mcmc)) {
        bhat = object$coef
        V = .my.vcov(object, ...)
    }
    else {
        if (is.null(object$coef)) bhat = apply(object$mcmc, 2, mean)
        if (is.null(object$vcov)) V = cov(object$mcmc)
    }
    
    misc = list()
    
    # If ordinal, add extra avgd, subtracted intercepts -- for latent mode
    if(!is.null(ordinal <- object$ordinal)) {
        if(is.null(ordinal$mode)) ordinal$mode = "latent"
        ordinal$mode = match.arg(ordinal$mode, c("latent", "linear.predictor", "cum.prob", "exc.prob", "prob", "mean.class"))
        if(is.null(od <- ordinal$dim)) stop ("'ordinal' MUST have a 'dim' element", call. = FALSE)
        if(ordinal$mode == "latent") {
            intcpt = matrix(-1 / (od - 1), nrow = nrow(X), ncol = od - 1)
            colnames(intcpt) = names(bhat)[1:(od - 1)]
            X = cbind(intcpt, X[, -1, drop = FALSE])
        }
        else {
            levs = seq_len(od - 1)
            misc$ylevs = list(cut = paste(levs, levs+1, sep = "|"))
            misc$inv.lbl = "cumprob"
            misc$offset.mult = -1
            J = matrix(1, nrow = od - 1)
            X = cbind(kronecker(diag(od - 1), matrix(1, nrow = nrow(X))), 
                      kronecker(-J, X[, -1, drop = FALSE]))
            if (ordinal$mode != "linear.predictor") {
                misc$mode = ordinal$mode
                misc$respName = as.character(object$formula)[2]
                misc$postGridHook = ".clm.postGrid"
            }
        }
    }
    
    bhat = .impute.NAs(bhat, X) # make coefs lm-compatible
    nbasis = estimability::all.estble
    if (sum(is.na(bhat)) > 0) {
        if(!is.null(object$qr))
            nbasis = estimability::nonest.basis(object$qr)
        else {
            if (is.name(object$data))
                object$data = eval(object$data)
            mm = suppressWarnings(model.frame(trms, object$data, na.action = na.pass, xlev = xlev))
            XX = model.matrix(trms, mm, contrasts.arg = object$contrasts)
            nbasis = estimability::nonest.basis(XX)
        }
        if (nrow(V) == length(bhat)) {
            ii = which(!is.na(bhat))
            V = V[ii, ii, drop = FALSE]
        }
    }
    
    # check multivariate situation
    if (is.matrix(bhat)) {
        X = kronecker (diag(ncol(bhat)), X)
        nbasis = kronecker(rep(1, ncol(bhat)), nbasis)
        nms = colnames(bhat)
        if (is.null(nms))
            nms = seq_len(ncol(bhat))
        misc$ylevs = list(rep.meas = nms)
        bhat = as.numeric(bhat)
    }
    
    if (!is.null(object$link))
        misc = .std.link.labels(eval(list(link = object$link)), misc)
    dfargs = list(df = object$df)
    dffun = function(k, dfargs) dfargs$df
    
    list(X=X, bhat=bhat, nbasis=nbasis, V=V, dffun=dffun, dfargs=dfargs, 
         misc=misc, post.beta=object$mcmc)
    }
    

## ----echo = FALSE, results = "hide", message = FALSE--------------------------
require("emmeans")
require("ggplot2")
options(show.signif.stars = FALSE) 
knitr::opts_chunk$set(fig.width = 4.5, class.output = "ro") 

## -----------------------------------------------------------------------------
nutr.lm <- lm(gain ~ (age + group + race)^2, data = nutrition) 
car::Anova(nutr.lm)

## -----------------------------------------------------------------------------
emmeans(nutr.lm, ~ group * race, calc = c(n = ".wgt."))

## -----------------------------------------------------------------------------
with(nutrition, table(race, age))

## -----------------------------------------------------------------------------
emmeans(nutr.lm, pairwise ~ group | race, at = list(age = "3")) |>
    summary(by = NULL)

## -----------------------------------------------------------------------------
framing <- mediation::framing 
levels(framing$educ) <- c("NA","Ref","< HS", "HS", "> HS","Coll +") 
framing.glm <- glm(cong_mesg ~ age + income + educ + emo + gender * factor(treat), 
    family = binomial, data = framing)

## ----fig.alt = "Two panels, each showing a decreasing trend with educ except they increase again with educ=Coll+. In the male panel, the curve for treat 1 is higher than that for treat 2, while the reverse is true in the female panel"----
emmip(framing.glm, treat ~ educ | gender, type = "response") 

## ----fig.alt = "Similar to previous figure but now the curves flatten out with >HS about the same as Coll+. For the maile panel, we still have the trace for treat 1 higher than for treat 0; but in the female panel, they are about the same as each other, and a bit lower than treat 0 for males. To see these results in numerical form, call emmeans() with the same arguments *except* replace the second argument with ~treat*educ|gender"----
emmip(framing.glm, treat ~ educ | gender, type = "response", 
    cov.reduce = emo ~ treat*gender + age + educ + income)

## ----eval = FALSE-------------------------------------------------------------
# emo.adj <- resid(lm(emo ~ treat*gender + age + educ + income, data = framing))

## ----eval = FALSE-------------------------------------------------------------
# emmeans(..., cov.reduce = list(x1 ~ trt, x2 ~ trt + x1, x3 ~ trt + x1 + x2))

## ----eval = FALSE-------------------------------------------------------------
# emmeans(model, "A", weights = "outer")
# emmeans(model, c("A", "B"), weights = "prop") |>  emmeans(weights = "prop")

## ----message = FALSE----------------------------------------------------------
sapply(c("equal", "prop", "outer", "cells", "flat"), \(w)
    emmeans(nutr.lm, ~ race, weights = w) |> predict())

## -----------------------------------------------------------------------------
mtcars.lm <- lm(mpg ~ factor(cyl)*am + disp + hp + drat + log(wt) + vs + 
                  factor(gear) + factor(carb), data = mtcars)

## -----------------------------------------------------------------------------
rg.usual <- ref_grid(mtcars.lm)
rg.usual
nrow(linfct(rg.usual))
rg.nuis = ref_grid(mtcars.lm, non.nuisance = "cyl")
rg.nuis
nrow(linfct(rg.nuis))

## -----------------------------------------------------------------------------
emmeans(rg.usual, ~ cyl * am)
emmeans(rg.nuis, ~ cyl * am)

## -----------------------------------------------------------------------------
predict(emmeans(mtcars.lm, ~ cyl * am, non.nuis = c("cyl", "am"), 
                wt.nuis = "prop"))
predict(emmeans(mtcars.lm, ~ cyl * am, weights = "outer"))

## -----------------------------------------------------------------------------
emmeans(mtcars.lm, ~ gear | am, non.nuis = quote(all.vars(specs)))

## ----error = TRUE-------------------------------------------------------------
try({
ref_grid(mtcars.lm, rg.limit = 200)
})

## -----------------------------------------------------------------------------
neuralgia.glm <- glm(Pain ~ Sex + Age + Duration + Treatment,
                     data = neuralgia, family = binomial)
emmeans(neuralgia.glm, "Treatment", counterfactuals = "Treatment",
        vcov. = sandwich::vcovHC)

## -----------------------------------------------------------------------------
emmeans(neuralgia.glm, "Treatment", weights = "prop", type = "response")

## -----------------------------------------------------------------------------
emmeans(nutr.lm, pairwise ~ group | race, submodel = ~ age + group*race) |> 
        summary(by = NULL)

## -----------------------------------------------------------------------------
emmeans(nutr.lm, ~ group * race, submodel = "minimal")

## -----------------------------------------------------------------------------
joint_tests(nutr.lm, submodel = "type2")

## -----------------------------------------------------------------------------
cows <- data.frame (
    route = factor(rep(c("injection", "oral"), c(5, 9))),
    drug = factor(rep(c("Bovineumab", "Charloisazepam", 
              "Angustatin", "Herefordmycin", "Mollycoddle"), c(3,2,  4,2,3))),
    resp = c(34, 35, 34,   44, 43,      36, 33, 36, 32,   26, 25,   25, 24, 24)
)
cows.lm <- lm(resp ~ route + drug, data = cows)

## ----message = FALSE----------------------------------------------------------
cows.rg <- ref_grid(cows.lm)
cows.rg

## -----------------------------------------------------------------------------
route.emm <- emmeans(cows.rg, "route")
route.emm

## -----------------------------------------------------------------------------
drug.emm <- emmeans(cows.rg, "drug")
drug.emm

## -----------------------------------------------------------------------------
pairs(route.emm, reverse = TRUE)

pairs(drug.emm, by = "route", reverse = TRUE)

## ----fig.width = 5.5, fig.alt = "A panel for each route. This interaction plot has a lot of empty space because all 5 drugs are represented in each panel, and the x axis labels all overlap"----
emmip(cows.rg, ~ drug | route)

## ----fig.width = 5.5, fig.alt = "This plot shows the same means as the previous one, but each panel shows only the drugs that are nested in each route"----
require(ggplot2)
emmip(cows.rg, ~ drug) + facet_wrap(~ route, scales = "free_x")

## ----fig.height = 2.5, fig.width = 5.5, fig.alt = "side-by-side CIs and PIs for drugs in each route. Again, with free_y scaling, each panel contains only the drugs involved"----
plot(drug.emm, PIs = TRUE) + 
    facet_wrap(~ route, nrow = 2, scales = "free_y")

##############################################################################
#    Copyright (c) 2012-2017 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

#' Support functions for model extensions
#' 
#' This documents some functions and methods that may be useful to package 
#' developers wishing to add support for \pkg{emmeans} for their model objects.A user
#' or package developer may add \pkg{emmeans} support for a model
#' class by writing \code{recover_data} and \code{emm_basis} methods
#' for that class. (Users in need for a quick way to obtain results for a model
#' that is not supported may be better served by the \code{\link{qdrg}} function.)
#' There are several other exported functions that may be useful. See the
#' "xtending" vignette for more details.
#' 
## #' @rdname extending-emmeans
#' @name extending-emmeans
#' @param object An object of the same class as is supported by a new method.
#' @param ... Additional parameters that may be supported by the method.
#' 
#' @section Details:
#' To create a reference grid, the \code{ref_grid} function needs to reconstruct
#' the data used in fitting the model, and then obtain a matrix of linear
#' functions of the regression coefficients for a given grid of predictor
#' values. These tasks are performed by calls to \code{recover_data} and
#' \code{emm_basis} respectively. A vignette giving details and examples
#' is available via \href{../doc/xtending.html}{vignette("xtending", "emmeans")}
#' 
#' To extend \pkg{emmeans}'s support to additional model types, one need only
#' write S3 methods for these two functions. The existing methods serve as
#' helpful guidance for writing new ones.  Most of the work for
#' \code{recover_data} can be done by its method for class \code{"call"},
#' providing the \code{terms} component and \code{na.action} data as additional
#' arguments. Writing an \code{emm_basis} method is more involved, but the
#' existing methods (e.g., \code{emmeans:::emm_basis.lm}) can serve as models.
#' Certain \code{recover_data} and \code{emm_basis} methods are exported from
#' \pkg{emmeans}. (To find out, do \code{methods("recover_data")}.) If your
#' object is based on another model-fitting object, it
#' may be that all that is needed is to call one of these exported methods and
#' perhaps make modifications to the results. Contact the developer if you need
#' others of these exported.
#' 
#' If the model has a multivariate response, \code{bhat} needs to be
#' \dQuote{flattened} into a single vector, and \code{X} and \code{V} must be
#' constructed consistently.
#' 
#' In models where a non-full-rank result is possible (often, you can tell by
#' seeing if there is a \code{singular.ok} argument in the model-fitting
#' function), \code{\link{summary.emmGrid}} and its relatives check the
#' estimability of each
#' prediction, using the \code{\link[estimability]{nonest.basis}} function in
#' the \pkg{estimability} package.
#' 
#' The models already supported are detailed in \href{../doc/models.html}{the
#' "models" vignette}. Some packages may provide additional \pkg{emmeans}
#' support for its object classes.
#'
#'
#' @return The \code{recover_data} method must return a \code{\link{data.frame}}
#'   containing all the variables that appear as predictors in the model,
#'   and attributes \code{"call"}, \code{"terms"}, \code{"predictors"},
#'   and \code{"responses"}. (\code{recover_data.call} will 
#'   provide these attributes.)
#' 
#' @note Without an explicit \code{data} argument, \code{recover_data} returns
#'    the \emph{current version} of the dataset. If the dataset has changed
#'    since the model was fitted, then this will not be the data used to fit
#'    the model. It is especially important to know this in simulation studies
#'    where the data are randomly generated or permuted, and in cases where
#'    several datasets are processed in one step (e.g., using \code{dplyr}).
#'    In those cases, users should be careful to provide the actual data
#'    used to fit the model in the \code{data} argument.
#'   
#' @seealso \href{../doc/xtending.html}{Vignette on extending emmeans}
#' 
#' @export
#' @order 1
recover_data = function(object, ...) {
    mth = .find_method(object, "recover_data")
    if(!is.null(mth))
        return(mth(object, ...))
    UseMethod("recover_data")  ## This call has to be here to establish recover_data as a generic
    
}

# get classes that are OK for external code to modify
# We don't allow overriding certain anchor classes, 
# nor ones in 3rd place or later in inheritance
.chk.cls = function(cls) {
    sacred = c("default", "call", "lm", "glm", "mlm", "aovlist", "lme", "qdrg")
    setdiff(head(cls, 2), sacred)
}

# Private utility to find a method
#' @importFrom methods getClass
.find_method = function(object, generic) {
    cls = class(object)
    # is it an S4 class? If so, replace cls with its inheritance chain
    if(!inherits(classes <- try(getClass(cls[[1]]), silent = TRUE), "try-error"))
        cls = c(cls[[1]], names(classes@contains))
    
    for (cl in .chk.cls(cls)) { # Look for user-provided method if not buried too deep
        mth <- .get.outside.method(generic, cl)
        if(!is.null(mth))
            return(mth)
    }
    # Now look for our own S3 methods
    for (cl in cls) {
        mth = getS3method(generic, cl, optional = TRUE)
        if(!is.null(mth))
            return(mth)
    }
    NULL
}


### My internal method dispatch -- we prefer outside methods
.get.outside.method = function(generic, cls) {
    mth = utils::getAnywhere(paste(generic, cls, sep = "."))
    from = sapply(strsplit(mth[[3]], "[ :]"), function(x) rev(x)[1])
    if (length(from) == 0)
        return (NULL)
    if(any(outside <- (from != "emmeans")))
        mth[which(outside)[1]]
    else
        NULL
}






#--------------------------------------------------------------
### call' objects
# This recover_data method serves as the workhorse for the others
# For model objects, call this with the object's call and its terms component
# Late addition: if data is non-null, use it in place of recovered data
# Later addition: na.action arg req'd - vector of row indexes removed due to NAs
#    na.action is ignored when data is non-NULL

#' @rdname extending-emmeans
#' @param trms The \code{\link{terms}} component of \code{object} (typically with
#'   the response deleted, e.g. via \code{\link{delete.response}})
#' @param na.action Integer vector of indices of observations to ignore; or
#'   \code{NULL} if none
#' @param data Data frame. Usually, this is \code{NULL}. However, if non-null,
#'   this is used in place of the reconstructed dataset. It must have all of the
#'   predictors used in the model, and any factor levels must match those used
#'   in fitting the model.
#' @param params Character vector giving the names of any variables in the model
#'   formula that are \emph{not} predictors. For example, a spline model may involve
#'   a local variable \code{knots} that is not a predictor, but its value is
#'   needed to fit the model. Names of parameters not actually used are harmless,
#'   and the default value \code{"pi"} (the only numeric constant in base R)
#'   is provided in case the model involves it. An example involving splines
#'   may be found at \url{https://github.com/rvlenth/emmeans/issues/180}.
#' @param frame Optional \code{data.frame}. Many model objects contain the 
#'   model frame used when fitting the model. In cases where there are no 
#'   predictor transformations, this model frame has all the original predictor
#'   values and so is usable for recovering the data. Thus, if \code{frame} is
#'   non-missing and \code{data} is \code{NULL}, a check is made on \code{trms}
#'   and if there are no function calls, we use \code{data = frame}. This
#'   can be helpful because it provides a modicum of security against the
#'   possibility that the original data used when fitting the model has been
#'   altered or removed.
#' @param pwts Optional vector of prior weights. Typically, this may be obtained
#'   from the fitted \code{model} via \code{weights(model)}. If this is provided,
#'   it is used to set weights as long as it is non-\code{NULL} and the same length 
#'   as the number of rows of the data.
#' @param addl.vars Character value or vector specifying additional predictors
#'   to include in the reference grid. These must be names of variables that
#'   exist, or you will get an error. 
#'   This may be useful if you need to do
#'   additional computations later on that depend on these variables; e.g., 
#'   bias adjustments for random slopes of variables not among the fixed predictors.
#' 
#' @exportS3Method recover_data call
#' @export
#' @order 2
recover_data.call = function(object, trms, na.action, data = NULL, 
                             params = "pi", frame, pwts, addl.vars, ...) {
    fcall = object # because I'm easily confused
    vars = setdiff(.all.vars(trms), params)
    offarg = fcall$offset
    if(missing(addl.vars))
        addl.vars = character(0)
    vars = union(vars, addl.vars)
        
    if (!missing(frame)) {
        if(is.null(data) && !.has.fcns(trms))
            data = frame
        if("(offset)" %in% names(data)) 
            vars = union(vars, "(offset)")
    }
    else if(!is.null(offarg))
        vars = union(vars, .all.vars(reformulate(deparse(offarg))))
    
    
    tbl = data
    if (length(vars) == 0 || vars[1] == "1") {
        tbl = data.frame(c(1,1))
        vars = names(tbl) = 1
    }
    if (is.null(tbl)) {
        possibly.random = FALSE
        m = match(c("formula", "data", "subset", "weights"), names(fcall), 0L)
        fcall = fcall[c(1L, m)]
        
        # check to see if there are any function calls to worry about
        # [e.g., subset = sample(1:n, 50) will give us a 
        #    different subset than model used]
        mm = match(c("data", "subset"), names(fcall), 0L)
        if(any(mm > 0)) {
            # Flag cases where there is a function call in data or subset
            # May indicate a situation where data are randomized
            fcns = unlist(lapply(fcall[mm], 
                     function(x) setdiff(all.names(x), 
                                         c("::",":::","[[","]]",all.vars(x)))))
            possibly.random = (max(nchar(c("", fcns))) > 1)
        }
        
        fcall$drop.unused.levels = TRUE
        fcall[[1L]] = quote(stats::model.frame)
        fcall$xlev = NULL # we'll ignore xlev
        
        if(!is.numeric(na.action))   ### In case na.action is not a vector of indices
            na.action = NULL
        
        # If we have an explicit list of cases to exclude, let everything through now
        if (!is.null(na.action))
            fcall$na.action = na.pass
        else  # exclude incomplete cases
            fcall$na.action = na.omit
        form = .reformulate(vars)
        fcall$formula = update(trms, form)
        env = environment(trms)
        if (is.null(env))
            env = parent.frame()
        tbl = try(eval(fcall, env, parent.frame()), silent = TRUE)
        if(inherits(tbl, "try-error"))
            return(.rd.error(vars, fcall))
        if (possibly.random) {
            chk = eval(fcall, env, parent.frame())
            if (!all(chk == tbl))
                stop("Data appear to be randomized -- ", 
                     "cannot consistently recover the data\n",
                     "Move the randomization ",
                     "outside of the model-fitting call.")
        }
        
        # Now we can drop na.action's rows
        if (!is.null(na.action))
            tbl = tbl[-(na.action),  , drop=FALSE]
    }
    
    else {
        tbl = tbl[, vars, drop = FALSE] # consider only the variables actually needed
        tbl = tbl[complete.cases(tbl), , drop=FALSE]
    }
    # if there is a separate offset argument, calculate the static offset
    if(!is.null(offarg)) {
        if ("(offset)" %in% names(tbl)) { # don't have to calculate it
            offval = tbl[["(offset)"]]
            vars = setdiff(vars, "(offset)")
        }
        else
            offval = eval(offarg, tbl, enclos = environment(trms))
        if(!is.null(offval)) {
            tbl$.static.offset. = offval
            addl.vars = c(addl.vars, ".static.offset.")
        }
    }

    
    if(!missing(pwts) && !is.null(pwts)) {
        if ((npw <- length(pwts)) == nrow(tbl))
            tbl[["(weights)"]] = pwts
        else if (length(unique(pwts)) > 1)
            warning("Model has ", npw, " prior weights, but we recovered ",
                    nrow(tbl), " rows of data.\nSo prior weights were ignored.",
                    call. = FALSE)
    }
    
    attr(tbl, "call") = object # the original call
    attr(tbl, "terms") = trms
    attr(tbl, "predictors") = setdiff(.all.vars(delete.response(trms)), params)
    if(!missing(addl.vars)) {
        attr(tbl, "predictors") = union(attr(tbl, "predictors"), addl.vars)
        vars = setdiff(vars, addl.vars)
    }
    attr(tbl, "responses") = setdiff(vars, union(attr(tbl, "predictors"), params))
    tbl
}

# error message for recover_data.call
.rd.error = function(vars, fcall) {
    if ("pi" %in% vars) 
        return("\nTry re-running with 'params = c\"pi\", ...)'")
    if (is.list(fcall$data)) fcall$data = "(raw data structure)"
    dataname = as.character(fcall$data)[1]
    if ((!is.na(dataname)) && (nchar(dataname) > 50))
        dataname = paste(substring(dataname, 1, 50), "...")
    mesg = "We are unable to reconstruct the data.\n"
    mesg = paste0(mesg, "The variables needed are:\n\t",
           paste(vars, collapse = " "), "\n",
           "Are any of these actually constants? (specify via 'params = ')\n")
    if (is.na(dataname))
        mesg = paste(mesg, "Try re-running with 'data = \"<name of dataset>\"'\n")
    else 
        mesg = paste0(mesg, "The dataset name is:\n\t", dataname, "\n",
           "Does the data still exist? Or you can specify a dataset via 'data = '\n")
    mesg
}

#----------------------------------------------------------
### emm_basis methods create a basis for the reference grid
#
# Required args:
#     object - the model object
#     trms   - terms component of object
#     xlev   - named list of factor levels (but not the coerced ones)
#     grid   - reference grid
# All methods must return a list with these elements:
#     X      - basis for linear fcns over reference grid
#     bhat   - regression coefficients for fixed effects (INCLUDING any NAs)
#     nbasis - matrix whose columns for a basis for non-estimable functions of beta; all.estble if none
#     V      - estimated covariance matrix of bhat
#     dffun  - function(k, dfargs) to find df for k'bhat having std error se
#     dfargs - additional arguments, if any, for dffun
#     misc   - Extra info ...
#              -- if extra levels need to be added (e.g. mlm, polr),
#                 put them in misc$ylevs
#              -- For transformations or link fcns, use misc$tran
#                 for name (see 'make.link'), and use misc$inv.lbl
#                 for label to use in 'summary' when tran is inverted
#                 (ref_grid looks at lhs of model for tran if none found)
# Note: if no df exists, set dffun = function(...) NA and dfargs = list()
#--------------------------------------------------------------

#' @rdname extending-emmeans
#' @order 11
#' @param xlev Named list of factor levels (\emph{excluding} ones coerced to 
#'   factors in the model formula)
#' @param grid A \code{data.frame} (provided by \code{ref_grid}) containing 
#'   the predictor settings needed in the reference grid
#'
#' @return The \code{emm_basis} method should return a \code{list} with the
#'   following elements:
#' \describe{
#' \item{X}{The matrix of linear functions over \code{grid}, having the same
#'   number of rows as \code{grid} and the number of columns equal to the length
#'   of \code{bhat}.}
#' \item{bhat}{The vector of regression coefficients for fixed effects. This
#'   should \emph{include} any \code{NA}s that result from rank deficiencies.}
#' \item{nbasis}{A matrix whose columns form a basis for non-estimable functions
#'   of beta, or a 1x1 matrix of \code{NA} if there is no rank deficiency.}
#' \item{V}{The estimated covariance matrix of \code{bhat}.}
#' \item{dffun}{A function of \code{(k, dfargs)} that returns the degrees of
#'   freedom associated with \code{sum(k * bhat)}.}
#' \item{dfargs}{A \code{list} containing additional arguments needed for
#'   \code{dffun}}.
#' } %%% end of describe
#' @export
#' 
#' @section Communication between methods:
#' If the \code{recover_data} method generates information needed by \code{emm_basis},
#' that information may be incorporated by creating a \code{"misc"} attribute in the
#' returned recovered data. That information is then passed as the \code{misc} 
#' argument when \code{ref_grid} calls \code{emm_basis}.
#' 
#' @section Optional hooks:
#' Some models may need something other than standard linear estimates and
#' standard errors. If so, custom functions may be pointed to via the items
#' \code{misc$estHook}, \code{misc$vcovHook} and \code{misc$postGridHook}. If
#' just the name of the hook function is provided as a character string, then it
#' is retrieved using \code{\link{get}}.
#' 
#' The \code{estHook} function should have arguments \samp{(object, do.se, tol,
#' ...)} where \code{object} is the \code{emmGrid} object,
#' \code{do.se} is a logical flag for whether to return the standard error, and
#' \code{tol} is the tolerance for assessing estimability. It should return a
#' matrix with 3 columns: the estimates, standard errors (\code{NA} when
#' \code{do.se==FALSE}), and degrees of freedom (\code{NA} for asymptotic). The
#' number of rows should equal \samp{nrow(linfct(object)}. The
#' \code{vcovHook} function should have arguments \samp{(object, tol, ...)} as
#' described. It should return the covariance matrix for the estimates. Finally,
#' \code{postGridHook}, if present, is called at the very end of
#' \code{ref_grid}; it takes one argument, the constructed \code{object}, and
#' should return a suitably modified \code{emmGrid} object.
emm_basis = function(object, trms, xlev, grid, ...) {
    # look for outside methods first
    mth = .find_method(object, "emm_basis")
    if(!is.null(mth))
        return(mth(object, trms, xlev, grid, ...))
    UseMethod("emm_basis")  ## This call has to be here to establish emm_basis as a generic
}

# Courtesy function that provides access to all recover_data methods
# I don't think we really need this...
# #' @  rdname extending-emmeans
# #' @ order 21
# #' @ export
# .recover_data = function(object, ...)
#     recover_data(object, ...)

# Hidden courtesy function that provides access to all emm_basis methods
# #' @rdname extending-emmeans
# #' @order 22
# #' @return \code{.recover_data} and \code{.emm_basis} are hidden exported versions of 
# #'   \code{recover_data} and \code{emm_basis}, respectively. They run in \pkg{emmeans}'s
# #'   namespace, thus providing access to all existing methods.
# #' @export
# .emm_basis = function(object, trms, xlev, grid, ...)
#     emm_basis(object, trms, xlev, grid, ...)
 



#--------------------------------------------------------------
### DEFAULT METHODS (we hit these when a model is NOT supported)
# I'll have it return the message if we caught the error in this way
# Then caller can use try() to check for other types of errors,
# and just print this message otherwise 
# NOT @exported
#' @exportS3Method recover_data default
recover_data.default = function(object, ...) {
    paste("Can't handle an object of class ", dQuote(class(object)[1]), "\n",
          paste(.show_supported(), collapse=""))
}
# NOT @exported
#' @exportS3Method emm_basis default      
emm_basis.default = function(object, trms, xlev, grid, ...) {
    stop("Can't handle an object of class", dQuote(class(object)[1]), "\n",
         .show_supported())
}

# Private fcn to get a list of supported objects
# does this by looking in namespace [ns] and methods [meth]
# then strips that off leaving extensions
.show_supported = function(ns = "emmeans", meth = "emm_basis") {
    "Use help(\"models\", package = \"emmeans\") for information on supported models."
}

##############################################################################
#    Copyright (c) 2012-2017 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

# Code supporting nested models

# This code replies on nested structures specified in a named list like
#    list(a = "b", c = c("d", "e"))
# ... to denote a %in% b, c %in% d*e


### Note: add_grouping() has been moved to factors.R


### ----- Rest of this file is used only internally ---------

# Internal function to deal with nested structures. 
#   rgobj        -- an emmGrid object
#   specs, ...   -- arguments for emmeans
#   nesting      -- a named list of nesting info
# This function works by subsetting rgobj as needed, and applying emmeans
# to each subsetted object
# This is a servant to emmeans.character.emmGrid, so we can assume specs is character
.nested_emm = function(rgobj, specs, by = NULL, ..., nesting) {
    # # Trap something not supported for these... This doesn't work
    # dots = list(...)
    # if("weights" %in% dots)
    #     if(!is.na(pmatch(dots$weights, "show.levels")))
    #         stop('weights = "show.levels" is not supported for nested models.')

    orig.by = by   # save original 'by' vars
    #### Two issues to worry about....
    # (1) specs contains nested factors. We need to include their grouping factors
    xspecs = intersect(union(specs, by), names(nesting))
    if (length(xspecs) > 0) {
        xgrps = unlist(nesting[xspecs])
        specs = union(union(xspecs, xgrps), specs)  # expanded specs with flagged ones first
        by = setdiff(by, xspecs) # can't use nested factors for grouping
    }
    # (2) If we average over any nested factors, we need to do it separately
    avg.over = setdiff(names(rgobj@levels), union(specs, by))
    afacs = intersect(names(nesting), avg.over) ### DUH!names(nesting)[names(nesting) %in% avg.over]
    rgobj@misc$display = NULL  ## suppress warning messages from emmeans
    
    if (length(afacs) == 0)  { # no nesting issues; just use emmeans
        result = emmeans(rgobj, specs, by = by, ...)
    }
    else { # we need to handle each group separately
        sz = sapply(afacs, function(nm) length(nesting[[nm]]))
        # use highest-order one first: potentially, we end up calling this recursively
        afac = afacs[rev(order(sz))][1] 
        otrs = setdiff(afacs, afac)   # other factors than afac
        grpfacs = union(nesting[[afac]], otrs)
        gspecs = union(specs, union(by, grpfacs))
        grpids = as.character(interaction(rgobj@grid[, grpfacs]))
        grps = do.call(expand.grid, rgobj@levels[grpfacs])  # all combinations of group factors
        result = NULL
        rg = rgobj
        actually.avgd.over = character(0) # keep track of this from emmeans calls
        for (i in seq_len(nrow(grps))) {
            sig = as.character(interaction(grps[i, ]))
            rows = which(grpids == sig)
            grd = rgobj@grid[rows, , drop = FALSE]
            lf = rgobj@linfct[rows, , drop = FALSE]
            # Reduce grid to infacs that actually appear in  this group
            nzg = grd[grd$.wgt. > 0, , drop = FALSE]
            rows = integer(0)
            # focus on levels of afac that exist in this group
            levs = unique(nzg[[afac]])
            rg@levels[[afac]] = levs
            rows = union(rows, which(grd[[afac]] %in% levs))
            rg@grid = grd[rows, , drop = FALSE]
            rg@linfct = lf[rows, , drop = FALSE]
            for (j in seq_along(grpfacs))
                rg@levels[[grpfacs[j]]] = grps[i, j]
            emmGrid = suppressMessages(emmeans(rg, gspecs, ...))
            actually.avgd.over = union(actually.avgd.over, emmGrid@misc$avgd.over)
            if (is.null(result))
                result = emmGrid
            else {
                result@grid = rbind(result@grid, emmGrid@grid)
                result@linfct = rbind(result@linfct, emmGrid@linfct)
            }
        }
        for (j in seq_along(grpfacs))
            result@levels[grpfacs[j]] = rgobj@levels[grpfacs[j]]
        
        result@misc$avgd.over = setdiff(actually.avgd.over, gspecs)
        result@misc$display = NULL
        nkeep = intersect(names(nesting), names(result@levels))
        if (length(nkeep) > 0)
            result@model.info$nesting = nesting[nkeep]
        else
            result@model.info$nesting = NULL
        
        # Note: if any nesting remains, this next call recurs back to this function
        result = emmeans(result, specs, by = by, ...)
    }
    
    if (length(xspecs) > 0)
        result@misc$display = .find.nonempty.nests(result, xspecs, nesting)

    # preserve any nesting that still exists
    nesting = nesting[names(nesting) %in% names(result@levels)]
    result@model.info$nesting =   if (length(nesting) > 0) nesting    else NULL
    
    # resolve 'by'
    by = orig.by
    if (length(xspecs <- intersect(by, names(nesting))))
        by = union(unlist(nesting[xspecs]), by)
    result@misc$by.vars = by
    
    result
}


### contrast function for nested structures
.nested_contrast = function(rgobj, method = "eff", interaction = FALSE, by = NULL, adjust, ...) {
    nesting = rgobj@model.info$nesting
    # Prevent meaningless cases -- if A %in% B, we can't have A in 'by' without B
    # Our remedy will be to EXPAND the by list
    for (nm in intersect(by, names(nesting)))
        if (!all(nesting[[nm]] %in% by)) {
            by = union(by, nesting[[nm]])
            message("Note: Grouping factor(s) for '", nm, "' have been added to the 'by' list.")
        }

    if(!is.character(method))
        stop ("Non-character contrast methods are not supported with nested objects")
    
    testcon = try(get(paste0(method, ".emmc"))(1:3), silent = TRUE)
    if(inherits(testcon, "try-error")) testcon = NULL
    if(missing(adjust)) 
        adjust = attr(testcon, "adjust")
    estType = attr(testcon, "type")

    wkrg = rgobj # working copy
    facs = setdiff(names(wkrg@levels), by)  # these are the factors we'll combine & contrast
    if (length(facs) == 0)
        stop("There are no factor levels left to contrast. Try taking nested factors out of 'by'.")
    if (!is.null(display <- wkrg@misc$display))
        wkrg = wkrg[which(display), drop.levels = TRUE]
    wkrg@model.info$nesting = wkrg@misc$display = NULL
    by.rows = .find.by.rows(wkrg@grid, by)
    if(length(by.rows) == 1)
        result = contrast.emmGrid(wkrg, method = method, interaction = interaction, by = by, adjust = adjust, ...)
    else {
        result = lapply(by.rows, function(rows) {
            contrast.emmGrid(wkrg[rows, drop.levels = TRUE], method = method, 
                             interaction = interaction, by = by, adjust = adjust, ...)
        })
        # set up coef matrix
        comb.nms = unique(do.call(paste, wkrg@grid[facs]))
        ncon = sapply(result, function(x) nrow(x@grid))
        con.idx = rep(seq_along(by.rows), ncon)  ## looks like 1,1, 2,2,2, 3, 4,4 ...
        con.coef = matrix(0, nrow = sum(ncon), ncol = nrow(wkrg@grid))
        # Have to define .wgt. for nested emmGrid. Use average weight - seems most sensible
        for (i in seq_along(by.rows)) {
            result[[i]]@grid$.wgt. = mean(wkrg@grid[[".wgt."]][by.rows[[i]]])
            con.coef[con.idx == i, by.rows[[i]]] = result[[i]]@misc$con.coef
        }
        result$adjust = ifelse(is.null(adjust), "none", adjust)
        result = do.call(rbind.emmGrid, result)
        result@misc$con.coef = con.coef
        result@misc$orig.grid = wkrg@grid[names(wkrg@levels)]
        result = update(result, by = by, 
                        estType = ifelse(is.null(estType), "contrast", estType))
        cname = setdiff(names(result@levels), by)
        if (!is.null(result@model.info$nesting))
            for (nm in cname) 
                result@model.info$nesting[[nm]] = by
    }
##    result@misc$orig.grid = result@misc$con.coef = NULL # we now provide these

    for (nm in by) {
        if (nm %in% names(nesting))
            result@model.info$nesting[[nm]] = intersect(nesting[[nm]], by)
    }
    if(!is.na(ET <- result@misc$estType) && (ET == "pairs")) # internal flag to keep track of original by vars for paired comps
        result@misc$.pairby = paste(c("", by), collapse = ",")
    
    result
}


# Internal function to find nonempty cells in nested structures in rgobj for xfacs
# Returns logical vector, FALSE are rows of the grid we needn't display
.find.nonempty.nests = function(rgobj, xfacs = union(names(nesting), unlist(nesting)), 
                                nesting = rgobj@model.info$nesting) {
    grid = rgobj@grid
    keep = rep(TRUE, nrow(grid))
    for (x in xfacs) {
        facs = union(x, nesting[[x]])
        combs = do.call(expand.grid, rgobj@levels[facs])
        levs = as.character(interaction(combs))
        glevs = as.character(interaction(grid[facs]))
        
        for (lev in levs) {
            idx = which(glevs == lev)
            if (all(grid$.wgt.[idx] == 0)) {
                keep[idx] = FALSE
                levs[levs==lev] = ""
            }
        }
    }
    keep
}

# Fill-in extra elements to make a grid regular
# Note: this will be documented at the end of methods for rbind and []
#' @rdname rbind.emmGrid
#' @order 50
#' @return \code{force_regular} adds extra (invisible) rows to an \code{emmGrid} object
#'   to make it a regular grid (all combinations of factors). This regular structure is 
#'   needed by \code{emmeans}. An object can become irregular by, for example,
#'   subsetting rows, or by obtaining contrasts of a nested structure.
#' @export
#' @examples
#' 
#' ### Irregular object
#' tmp <- warp.rg[-1]
#' ## emmeans(tmp, "tension")   # will fail because tmp is irregular
#' emmeans(force_regular(tmp), "tension")   # will show some results
force_regular = function(object) {
    newgrid = do.call(expand.grid, object@levels)
    newkey = do.call(paste, newgrid)
    newlf = matrix(NA, nrow = nrow(newgrid), ncol = ncol(object@linfct))
    colnames(newlf) = colnames(object@linfct) 
    newdisp = rep(FALSE, nrow(newgrid))
    
    oldgrid = object@grid
    oldkey = do.call(paste, oldgrid[setdiff(names(oldgrid), c(".wgt.", ".offset."))])
    if (wtd <- (".wgt." %in% names(oldgrid))) 
        newgrid$.wgt. = 0
    if (ofs <- (".offset." %in% names(oldgrid))) 
        newgrid$.offset. = NA
    for (j in seq_along(oldkey)) {
        key = oldkey[j]
        i = which(newkey == key)
        newlf[i, ] = object@linfct[j, ]
        newdisp[i] = TRUE
        if(wtd) newgrid$.wgt.[i] = oldgrid$.wgt.[j]
        if(ofs) newgrid$.offset.[i] = oldgrid$.offset.[j]
    }
    object@grid = newgrid
    object@linfct = newlf
    if(is.null(object@model.info$nesting) && all(newdisp))  # remove 'display` if all TRUE`
        newdisp = NULL
    object@misc$display = newdisp
    object
}


# Internal function to find nesting
# We look at two things:
# (1) structural nesting - i.e., any combinations of
#     factors A and B for which each level of A occurs with one and only one
#     level of B. If so, we deem A %in% B.
# (2) Model-term nesting - cases where a factor appears not as a main effect
#     but only in higher-order terms. This is discovered using the 1s and 2s in 
#     trms$factors
# The function returns a named list, e.g., list(A = "B")
# If none found, an empty list is returned.
# 
# Added ver 1.5.3+ : if trms is NULL, we skip that part
.find_nests = function(grid, trms, coerce, levels) {
    result = list()
    
    # only consider cases where levels has length > 1
    lng = sapply(levels, length)
    nms = names(levels[lng > 1])
    if (length(nms) < 2)
        return (result)
    g = grid[grid$.wgt. > 0, nms, drop = FALSE]
    for (nm in nms) {
        x = levels[[nm]]
        # exclude other factors this is already nested in
        excl = sapply(names(result), function(lnm)
            ifelse(nm %in% result[[lnm]], lnm, ""))
        otrs = setdiff(nms[!(nms == nm)], excl)
        max.levs = sapply(otrs, function(n) {
            max(sapply(x, function(lev) length(unique(g[[n]][g[[nm]] == lev]))))
        })
        if (any(max.levs == 1)) 
            result[[nm]] = otrs[max.levs == 1]
    }
    
    if(!is.null(trms)) {
        
        # Now look at factors attribute
        fac = attr(trms, "factors")
        if (length(fac) > 0) {
            if (!is.null(coerce)) for (stg in coerce) {
                subst = paste(.all.vars(stats::reformulate(stg)), collapse = ":")
                for (i in 1:2)
                    dimnames(fac)[[i]] = gsub(stg, subst, dimnames(fac)[[i]], 
                                              fixed = TRUE)
            }
            fac = fac[intersect(nms, row.names(fac)), , drop = FALSE]
            
            ### new code
            nms = row.names(fac)
            cols = dimnames(fac)[[2]]
            pert = setdiff(nms, intersect(nms, cols)) # pertinent - no main effect in model
            for (nm in pert) {
                pfac = fac[, fac[nm, ] == 1, drop = FALSE]  # cols where nm appears
                if (ncol(pfac) == 0) { # case where there is no 1 in a row
                    pfac = fac[, fac[nm, ] == 2, drop = FALSE]
                    pfac[nm, ] = 1 # make own entry 1 so it isn't nested in self
                }
                nst = .strip.supersets(apply(pfac, 2, function(col) nms[col == 2]))
                if (length(nst) > 0)
                    result[[nm]] = union(result[[nm]], nst)
            }
        }
        
    }  # [end if(!is.null(trms))]
    
    # include nesting factors that are themselves nested
    for (nm in names(result))
        result[[nm]] = union(unlist(result[result[[nm]]]), result[[nm]])

    # omit any entries where factors are nested in themselves (we get such in a model like y ~ A : B)
    for (nm in names(result))
        if (nm %in% result[[nm]])
            result[[nm]] = NULL
    
    
    result
}

# strip supersets from a list and condense down to a character vector
# e.g., lst = list("a", c("A", "B")) --> "A"
.strip.supersets = function(lst) {
    if (is.list(lst) && (length(lst) > 1)) {
        lst = lst[order(sapply(lst, length))]  # order by length
        for (i in length(lst):2) {
            tst = sapply(lst[1:(i-1)], function(x) all(x %in% lst[[i]]))
            if (any(tst)) lst[[i]] = NULL
        }
    }
    unique(unlist(lst))
}

# internal function to format a list of nested levels
.fmt.nest = function(nlist) {
    if (length(nlist) == 0)
        "none"
    else {
        tmp = lapply(nlist, function(x) 
            if (length(x) == 1) x
            else                paste0("(", paste(x, collapse = "*"), ")")
        )
        paste(sapply(names(nlist), function (nm) paste0(nm, " %in% ", tmp[[nm]])),
              collapse = ", ")
    }
}

# internal function to parse a nesting string & return a list
# spec can be a named list, character vector    ####, or formula
.parse_nest = function(spec) {
    if (is.null(spec))
        return(NULL)
    if (is.list(spec))
        return (spec)
    result = list()
    # break up any comma delimiters
    spec = trimws(unlist(strsplit(spec, ",")))
    for (s in spec) {
        parts = strsplit(s, "[ ]+%in%[ ]+")[[1]]
        trm = .all.vars(stats::reformulate(parts[1]))
        grp = .all.vars(stats::reformulate(parts[2]))
        result[[trm]] = grp
    }
    if(length(result) == 0)
        result = NULL
    result
}



# ### I'm removing this because I now think it creates more problems than it solves
# #
# # courtesy function to create levels for a nested structure factor %in% nest
# # factor: factor (or interaction() result)
# # ...:    factors in nest
# # SAS:    if (FALSE|TRUE), reference level in each nest is (first|last)
# nested = function(factor, ..., SAS = FALSE) {
#     nfacs = list(...)
#     if (length(nfacs) == 0)
#         return(factor)
#     nfacs$drop = TRUE
#     nest = do.call(interaction, nfacs)
#     result = as.character(interaction(factor, nest, sep = ".in."))
#     ores = unique(sort(result))
#     nlev = levels(nest)
#     flev = levels(factor)
#     refs = lapply(nlev, function(nst) {
#         r = ores[ores %in% paste0(flev, ".in.", nst)]
#         ifelse (SAS, rev(r)[1], r[1])
#     })
#     result[result %in% refs] = ".nref."
#     ores[ores %in% refs] = ".nref."
#     ores = setdiff(ores, ".nref.")
#     if (SAS)
#         factor(result, levels = c(ores, ".nref."))
#     else
#         factor(result, levels = c(".nref.", ores))
# }

##############################################################################
#    Copyright (c) 2012-2020 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

# Reference grid code


# Change to cov.reduce specification: can be...
#     a function: is applied to all covariates
#     named list of functions: applied to those covariates (else mean is used)
#     TRUE - same as mean
#     FALSE - same as function(x) sort(unique(x))

#' Create a reference grid from a fitted model
#'
#' Using a fitted model object, determine a reference grid for which estimated
#' marginal means are defined. The resulting \code{ref_grid} object encapsulates
#' all the information needed to calculate EMMs and make inferences on them.
#'
#' To users, the \code{ref_grid} function itself is important because most of
#' its arguments are in effect arguments of \code{\link{emmeans}} and related
#' functions, in that those functions pass their \code{...} arguments to
#' \code{ref_grid}.
#'
#' The reference grid consists of combinations of independent variables over
#' which predictions are made. 
#' Estimated marginal means are defined as these
#' predictions, or marginal averages thereof. The grid is determined by first
#' reconstructing the data used in fitting the model (see
#' \code{\link{recover_data}}), or by using the \code{data.frame} provided in
#' \code{data}. 
#' 
#' By \dQuote{independent variables,} we mean (in most cases) the results of
#' \code{all.vars()} applied to the fixed-effects part of the right-hand side of
#' the model formula. Any random effects are excluded. Thus, if the model
#' formula in an \code{lme4::lmer} call is \code{yield ~ fert +
#' seed*density + log(rain) + (1|block/plot)}, the independent variables are
#' \code{fert}, \code{seed}, \code{density}, and \code{rain} (not
#' \code{log(rain)}). In multivariate models, the dimension of the multivariate
#' response is also considered an independent variable.
#' 
#' 
#' The default reference grid is determined by the observed levels
#' of any factors, the ordered unique values of character-valued predictors, and
#' the results of \code{cov.reduce} for numeric predictors. These may be
#' overridden using \code{at}. See also the section below on
#' recovering/overriding model information.
#'
#'
#' @param object An object produced by a supported model-fitting function, such
#'   as \code{lm}. Many models are supported. See
#'   \href{../doc/models.html}{\code{vignette("models", "emmeans")}}.
#' @param at Optional named list of levels for the corresponding variables
#' @param cov.reduce A function, logical value, or formula; or a named list of
#'   these. Each covariate \emph{not} specified in \code{cov.keep} or \code{at}
#'   is reduced according to these specifications. See the section below on
#'   \dQuote{Using \code{cov.reduce} and \code{cov.keep}}.
#' @param cov.keep Character vector: names of covariates that are \emph{not}
#'   to be reduced; these are treated as factors and used in weighting calculations.
#'   \code{cov.keep} may also include integer value(s), and if so, the maximum
#'   of these is used to set a threshold such that any covariate having no more
#'   than that many unique values is automatically included in \code{cov.keep}.
#' @param mult.names Character value: the name(s) to give to the
#'   pseudo-factor(s) whose levels delineate the elements of a multivariate
#'   response. If this is provided, it overrides the default name(s) used for
#'   \code{class(object)} when it has a multivariate response (e.g., the default
#'   is \code{"rep.meas"} for \code{"mlm"} objects).
#' @param mult.levs A named list of levels for the dimensions of a multivariate
#'   response. If there is more than one element, the combinations of levels are
#'   used, in \code{\link{expand.grid}} order. The (total) number of levels must
#'   match the number of dimensions. If \code{mult.name} is specified, this
#'   argument is ignored.
#' @param options If non-\code{NULL}, a named \code{list} of arguments to pass
#'   to \code{\link{update.emmGrid}}, just after the object is constructed.
#' @param data A \code{data.frame} to use to obtain information about the
#'   predictors (e.g. factor levels). If missing, then
#'   \code{\link{recover_data}} is used to attempt to reconstruct the data. See
#'   the note with \code{\link{recover_data}} for an important precaution.
#' @param df Numeric value. This is equivalent to specifying \code{options(df =
#'   df)}. See \code{\link{update.emmGrid}}.
#' @param type Character value. If provided, this is saved as the
#'   \code{"predict.type"} setting. See \code{\link{update.emmGrid}} and the
#'   section below on prediction types and transformations.
#' @param regrid Character, logical, or list. If non-missing, the reference
#'   grid is reconstructed via \code{\link{regrid}} with the argument
#'   \code{transform = regrid}. See the section below on prediction types and
#'   transformations. \emph{Note:} This argument was named \code{transform} in
#'   version 1.7.2 and earlier. For compatibility with old code, \code{transform}
#'   is still accepted if found among \code{...}, 
#'   as long as it doesn't match \code{tran}.
#' @param nesting If the model has nested fixed effects, this may be specified
#'   here via a character vector or named \code{list} specifying the nesting
#'   structure. Specifying \code{nesting} overrides any nesting structure that
#'   is automatically detected. See the section below on Recovering or Overriding 
#'   Model Information.
#' @param offset Numeric scalar value (if a vector, only the first element is
#'   used). This may be used to add an offset, or override offsets based on the
#'   model. A common usage would be to specify \code{offset = 0} for a Poisson
#'   regression model, so that predictions from the reference grid become rates
#'   relative to the offset that had been specified in the model.
#' @param sigma Numeric value to use for subsequent predictions or
#'   back-transformation bias adjustments. If not specified, we use
#'   \code{sigma(object)}, if available, and \code{NULL} otherwise.
#'   Note: This applies only when the family is \code{"gaussian"}; for other families,
#'   \code{sigma} is set to \code{NA} and cannot be overridden.
#' @param counterfactuals \code{counterfactuals} specifies character
#'   names of counterfactual factors. If this is non-missing, a reference grid
#'   is created consisting of combinations of counterfactual levels 
#'   and the actual levels of those same factors.
#'   This grid is always converted to the response transformation scale
#'   and averaged over the actual factor levels. See the section below 
#'   on counterfactuals. 
#' @param nuisance,non.nuisance,wt.nuis If \code{nuisance} is a vector of predictor names,
#'   those predictors are omitted from the reference grid. Instead, the result 
#'   will be as if we had averaged over the levels of those factors, with either 
#'   equal or proportional weights as specified in \code{wt.nuis} (see the 
#'   \code{weights} argument in \code{\link{emmeans}}). The factors in 
#'   \code{nuisance} must not interact with other factors, not even other
#'   nuisance factors. Specifying nuisance factors can save considerable
#'   storage and computation time, and help avoid exceeding the maximum
#'   reference-grid size (\code{get_emm_option("rg.limit")}). 
#'   (\emph{Note:} For certain models where the \code{emm_basis} method returns a
#'   re-gridded parameterization, nuisance factors cannot be used, and an error
#'   is thrown.)
#' @param rg.limit Integer limit on the number of reference-grid rows to allow
#'   (checked before any multivariate responses are included).
#' @param ... Optional arguments passed to \code{\link{summary.emmGrid}},
#'   \code{\link{emm_basis}}, and
#'   \code{\link{recover_data}}, such as \code{params}, \code{vcov.} (see
#'   \bold{Covariance matrix} below), or options such as \code{mode} for
#'   specific model types (see \href{../doc/models.html}{vignette("models",
#'   "emmeans")}).
#'
#' @section Using \code{cov.reduce} and \code{cov.keep}: 
#'   The \code{cov.keep} argument was not available in \pkg{emmeans} versions
#'   1.4.1 and earlier. Any covariates named in this list are treated as if they
#'   are factors: all the unique levels are kept in the reference grid. The user
#'   may also specify an integer value, in which case any covariate having no more
#'   than that number of unique values is implicitly included in \code{cov.keep}.
#'   The default for \code{cov.keep} is set and retrieved via the 
#'   \code{\link{emm_options}} framework, and the system default is \code{"2"},
#'   meaning that covariates having only two unique values are automatically
#'   treated as two-level factors. See also the Note below on backward compatibility.
#'   
#'   There is a subtle distinction between including a covariate in \code{cov.keep}
#'   and specifying its values manually in \code{at}: Covariates included in 
#'   \code{cov.keep} are treated as factors for purposes of weighting, while
#'   specifying levels in \code{at} will not include the covariate in weighting.
#'   See the \code{mtcars.lm} example below for an illustration.
#'   
#'   \code{cov.reduce} may be a function,
#'   logical value, formula, or a named list of these.
#'   If a single function, it is applied to each covariate.
#'   If logical and \code{TRUE}, \code{mean} is used. If logical and
#'   \code{FALSE}, it is equivalent to including all covariates in
#'   \code{cov.keep}. Use of \samp{cov.reduce = FALSE} is inadvisable because it
#'   can result in a huge reference grid; it is far better to use
#'   \code{cov.keep}.
#'
#'   If a formula (which must be two-sided), then a model is fitted to that
#'   formula using \code{\link{lm}}; then in the reference grid, its response
#'   variable is set to the results of \code{\link{predict}} for that model,
#'   with the reference grid as \code{newdata}. (This is done \emph{after} the
#'   reference grid is determined.) A formula is appropriate here when you think
#'   experimental conditions affect the covariate as well as the response.
#'   
#'   To allow for situations where a simple \code{lm()} call as described above won't
#'   be adequate, a formula of the form \code{ext ~ fcnname} is also supported,
#'   where the left-hand side may be \code{ext}, \code{extern}, or
#'   \code{external} (and must \emph{not} be a predictor name) and the
#'   right-hand side is the name of an existing function. The function is called
#'   with one argument, a data frame with columns for each variable in the
#'   reference grid. The function is expected to use that frame as new data to
#'   be used to obtain predictions for one or more models; and it should return
#'   a named list or data frame with replacement values for one or more of the
#'   covariates.
#'
#'   If \code{cov.reduce} is a named list, then the above criteria are used to
#'   determine what to do with covariates named in the list. (However, formula
#'   elements do not need to be named, as those names are determined from the
#'   formulas' left-hand sides.) Any unresolved covariates are reduced using
#'   \code{"mean"}.
#'
#'   Any \code{cov.reduce} of \code{cov.keep} specification for a covariate 
#'   also named in \code{at} is ignored.
#'   
#'   
#' @section Interdependent covariates: Care must be taken when covariate values
#'   depend on one another. For example, when a polynomial model was fitted
#'   using predictors \code{x}, \code{x2} (equal to \code{x^2}), and \code{x3}
#'   (equal to \code{x^3}), the reference grid will by default set \code{x2} and
#'   \code{x3} to their means, which is inconsistent. The user should instead
#'   use the \code{at} argument to set these to the square and cube of
#'   \code{mean(x)}. Better yet, fit the model using a formula involving
#'   \code{poly(x, 3)} or \code{I(x^2)} and \code{I(x^3)}; then there is only
#'   \code{x} appearing as a covariate; it will be set to its mean, and the
#'   model matrix will have the correct corresponding quadratic and cubic terms.
#'
#' @section Matrix covariates: Support for covariates that appear in the dataset
#'   as matrices is very limited. If the matrix has but one column, it is
#'   treated like an ordinary covariate. Otherwise, with more than one column,
#'   each column is reduced to a single reference value -- the result of
#'   applying \code{cov.reduce} to each column (averaged together if that
#'   produces more than one value); you may not specify values in \code{at}; and
#'   they are not treated as variables in the reference grid, except for
#'   purposes of obtaining predictions.
#'
#' @section Recovering or overriding model information: Ability to support a
#'   particular class of \code{object} depends on the existence of
#'   \code{recover_data} and \code{emm_basis} methods -- see
#'   \link{extending-emmeans} for details. The call
#'   \code{methods("recover_data")} will help identify these.
#'
#'   \bold{Data.} In certain models, (e.g., results of
#'   \code{\link[lme4]{glmer.nb}}), it is not possible to identify the original
#'   dataset. In such cases, we can work around this by setting \code{data}
#'   equal to the dataset used in fitting the model, or a suitable subset. Only
#'   the complete cases in \code{data} are used, so it may be necessary to
#'   exclude some unused variables. Using \code{data} can also help save
#'   computing, especially when the dataset is large. In any case, \code{data}
#'   must represent all factor levels used in fitting the model. It
#'   \emph{cannot} be used as an alternative to \code{at}. (Note: If there is a
#'   pattern of \code{NAs} that caused one or more factor levels to be excluded
#'   when fitting the model, then \code{data} should also exclude those levels.)
#'
#'   \bold{Covariance matrix.} By default, the variance-covariance matrix for
#'   the fixed effects is obtained from \code{object}, usually via its
#'   \code{\link{vcov}} method. However, the user may override this via a
#'   \code{vcov.} argument, specifying a matrix or a function. If a matrix, it
#'   must be square and of the same dimension and parameter order of the fixed
#'   effects. If a function, must return a suitable matrix when it is called
#'   with arguments \code{(object, ...)}. Be careful with possible 
#'   unintended conflicts with arguments in \code{...}; for example, 
#'   \code{sandwich::vcovHAC()} has optional arguments \code{adjust} and \code{weights}
#'   that may be intended for \code{emmeans()} but will also be passed to \code{vcov.()}.
#'
#'   \bold{Nested factors.} Having a nesting structure affects marginal
#'   averaging in \code{emmeans} in that it is done separately for each level
#'   (or combination thereof) of the grouping factors. \code{ref_grid} tries to
#'   discern which factors are nested in other factors, but it is not always
#'   obvious, and if it misses some, the user must specify this structure via
#'   \code{nesting}; or later using \code{\link{update.emmGrid}}. The
#'   \code{nesting} argument may be a character vector, a named \code{list}, 
#'   or \code{NULL}.
#'   If a \code{list}, each name should be the name of a single factor in the
#'   grid, and its entry a character vector of the name(s) of its grouping
#'   factor(s). \code{nested} may also be a character value of the form
#'   \code{"factor1 \%in\% (factor2*factor3)"} (the parentheses are optional).
#'   If there is more than one such specification, they may be appended
#'   separated by commas, or as separate elements of a character vector. For
#'   example, these specifications are equivalent: \code{nesting = list(state =
#'   "country", city = c("state", "country")}, \code{nesting = "state \%in\%
#'   country, city \%in\% (state*country)"}, and \code{nesting = c("state \%in\%
#'   country", "city \%in\% state*country")}.
#'
#' @section Predictors with subscripts and data-set references: When the fitted
#'   model contains subscripts or explicit references to data sets, the
#'   reference grid may optionally be post-processed to simplify the variable
#'   names, depending on the \code{simplify.names} option (see
#'   \code{\link{emm_options}}), which by default is \code{TRUE}. For example,
#'   if the model formula is \code{data1$resp ~ data1$trt + data2[[3]] +
#'   data2[["cov"]]}, the simplified predictor names (for use, e.g., in the
#'   \code{specs} for \code{\link{emmeans}}) will be \code{trt},
#'   \code{data2[[3]]}, and \code{cov}. Numerical subscripts are not simplified;
#'   nor are variables having simplified names that coincide, such as if
#'   \code{data2$trt} were also in the model.
#'
#'   Please note that this simplification is performed \emph{after} the
#'   reference grid is constructed. Thus, non-simplified names must be used in
#'   the \code{at} argument (e.g., \code{at = list(`data2["cov"]` = 2:4)}.
#'
#'   If you don't want names simplified, use \code{emm_options(simplify.names =
#'   FALSE)}.
#'
#'
#'
#' @section Prediction types and transformations:
#'   Transformations can exist because of a link function in a generalized linear model, 
#'   or as a response transformation, or even both. In many cases, they are auto-detected,
#'   for example a model formula of the form \code{sqrt(y) ~ ...}. Even transformations
#'   containing multiplicative or additive constants, such as \code{2*sqrt(y + pi) ~ ...},
#'   are auto-detected. A response transformation of \code{y + 1 ~ ...} is \emph{not}
#'   auto-detected, but \code{I(y + 1) ~ ...} is interpreted as \code{identity(y + 1) ~ ...}.
#'   A warning is issued if it gets too complicated.
#'   Complex transformations like the Box-Cox transformation are not auto-detected; but see 
#'   the help page for \code{\link{make.tran}} for information on some advanced methods.
#'   
#'   There is a subtle difference
#'   between specifying \samp{type = "response"} and \samp{regrid =
#'   "response"}. While the summary statistics for the grid itself are the same,
#'   subsequent use in \code{\link{emmeans}} will yield different results if
#'   there is a response transformation or link function. With \samp{type =
#'   "response"}, EMMs are computed by averaging together predictions on the
#'   \emph{linear-predictor} scale and then back-transforming to the response
#'   scale; while with \samp{regrid = "response"}, the predictions are
#'   already on the response scale so that the EMMs will be the arithmetic means
#'   of those response-scale predictions. To add further to the possibilities,
#'   \emph{geometric} means of the response-scale predictions are obtainable via
#'   \samp{regrid = "log", type = "response"}. See also the help page for 
#'   \code{\link{regrid}}.
#'   
#'   \emph{Order-of-processing issues:} 
#'   The \code{regrid} argument, if present, is acted on immediately after the reference 
#'   grid is constructed, while some of the \code{...} arguments may be used to
#'   update the object at the very end. Thus, code like
#'   \code{ref_grid(mod, tran = "sqrt", regrid = "response")} will not work correctly
#'   if the intention was to specify the response transformation, because the re-grid 
#'   is done \emph{before} it processes \code{tran = "sqrt"}. To get the intended
#'   result, do
#'   \code{regrid(ref_grid(mod, tran = "sqrt"), transform = "response")}.
#'
#' @section Counterfactuals:
#'   If \code{counterfactuals} is specified, the rows of the entire dataset
#'   become part of the reference grid, and the other reference levels are
#'   confined to those named in \code{counterfactuals}. In this type of analysis
#'   (called G-computation), we substitute (or impute) each combination of counterfactual
#'   levels into the entire dataset. Thus, predictions from this grid are those
#'   of each observation under each of the counterfactual levels. For this to
#'   make sense, we require an assumption of exchangeability of these levels.
#'   
#'   This grid is always converted to the response scale, as G-computation on
#'   the linear-predictor scale produces the same results as ordinary weighted EMMs.
#'   If we have counterfactual factors \code{A, B}, the reference grid also includes
#'   factors \code{actual_A, actual_B} which are used to track which observations
#'   originally had the \code{A, B} levels before they were changed by the 
#'   counterfactuals code. We average the response-scale predictions for each
#'   combination of actual levels and imputed levels (and multivariate levels, 
#'   if any). See additional discussion of how \code{\link{emmeans}} handles
#'   counterfactuals under that documentation.
#'   
#'   Currently, counterfactuals are not supported when the reference grid
#'   requires post-processing (e.g., ordinal models with \code{mode = "prob"}).
#'   Cases where we have nested factor levels can be complicated if mixed-in with counterfactuals, 
#'   and we make no guarantees. 
#'   Note that past implementations included arguments \code{wt.counter} and \code{avg.counter},
#'   which are now deprecated and are just ignored if specified.
#' 
#' @section Optional side effect: If the \code{save.ref_grid} option is set to
#'   \code{TRUE} (see \code{\link{emm_options}}),
#'   The most recent result of \code{ref_grid}, whether
#'   called directly or indirectly via \code{\link{emmeans}},
#'   \code{\link{emtrends}}, or some other function that calls one of these, is
#'   saved in the user's environment as \code{.Last.ref_grid}. This facilitates
#'   checking what reference grid was used, or reusing the same reference grid
#'   for further calculations. This automatic saving is disabled by default, but
#'   may be enabled via \samp{emm_options(save.ref_grid = TRUE)}.
#'   
#' 
#'
#' @return An object of the S4 class \code{"emmGrid"} (see
#'   \code{\link{emmGrid-class}}). These objects encapsulate everything needed
#'   to do calculations and inferences for estimated marginal means, and contain
#'   nothing that depends on the model-fitting procedure.
#'
#' @seealso Reference grids are of class \code{\link[=emmGrid-class]{emmGrid}},
#'   and several methods exist for them -- for example
#'   \code{\link{summary.emmGrid}}. Reference grids are fundamental to
#'   \code{\link{emmeans}}. Supported models are detailed in
#'   \href{../doc/models.html}{\code{vignette("models", "emmeans")}}.
#'   See \code{\link{update.emmGrid}} for details of arguments that can be in 
#'   \code{options} (or in \code{...}).
#'   
#' @note The system default for \code{cov.keep} causes models
#'   containing indicator variables to be handled differently than in
#'   \pkg{emmeans} version 1.4.1 or earlier. To replicate older
#'   analyses, change the default via 
#'   \samp{emm_options(cov.keep = character(0))}.
#'   
#' @note Some earlier versions of \pkg{emmeans} offer a \code{covnest} argument.
#'   This is now obsolete; if \code{covnest} is specified, it is harmlessly
#'   ignored. Cases where it was needed are now handled appropriately via the
#'   code associated with \code{cov.keep}.
#'
#' @export
#'
#' @examples
#' fiber.lm <- lm(strength ~ machine*diameter, data = fiber)
#' ref_grid(fiber.lm)
#'
#' ref_grid(fiber.lm, at = list(diameter = c(15, 25)))
#'
#' \dontrun{
#' # We could substitute the sandwich estimator vcovHAC(fiber.lm)
#' # as follows:
#' summary(ref_grid(fiber.lm, vcov. = sandwich::vcovHAC))
#' }
#'
#' # If we thought that the machines affect the diameters
#' # (admittedly not plausible in this example), then we should use:
#' ref_grid(fiber.lm, cov.reduce = diameter ~ machine)
#' 
#' ### Model with indicator variables as predictors:
#' mtcars.lm <- lm(mpg ~ disp + wt + vs * am, data = mtcars)
#' (rg.default <- ref_grid(mtcars.lm))
#' (rg.nokeep <- ref_grid(mtcars.lm, cov.keep = character(0)))
#' (rg.at <- ref_grid(mtcars.lm, at = list(vs = 0:1, am = 0:1)))
#' 
#' # Two of these have the same grid but different weights:
#' rg.default@grid
#' rg.at@grid
#' 
#' ### Using cov.reduce formulas...
#' # Above suggests we can vary disp indep. of other factors - unrealistic
#' rg.alt <- ref_grid(mtcars.lm, at = list(wt = c(2.5, 3, 3.5)),
#'     cov.reduce = disp ~ vs * wt)
#' rg.alt@grid
#' 
#' # Alternative to above where we model sqrt(disp)
#' disp.mod <- lm(sqrt(disp) ~ vs * wt, data = mtcars)
#' disp.fun <- function(dat)
#'     list(disp = predict(disp.mod, newdata = dat)^2)
#' rg.alt2 <- ref_grid(mtcars.lm, at = list(wt = c(2.5, 3, 3.5)),
#'     cov.reduce = external ~ disp.fun)
#' rg.alt2@grid
#' 
#'
#' # Multivariate example
#' MOats.lm = lm(yield ~ Block + Variety, data = MOats)
#' ref_grid(MOats.lm, mult.names = "nitro")
#' # Silly illustration of how to use 'mult.levs' to make comb's of two factors
#' ref_grid(MOats.lm, mult.levs = list(T=LETTERS[1:2], U=letters[1:2]))
#' 
#' # Comparing estimates with and without counterfactuals
#' neuralgia.glm <- glm(Pain ~ Treatment + Sex + Age + Duration, 
#'                      family = binomial(), data = neuralgia)
#' emmeans(neuralgia.glm, "Treatment", type = "response")
#' 
#' emmeans(neuralgia.glm, "Treatment", counterfactuals = "Treatment")
#' 
#' 
#' # Using 'params'
#' require("splines")
#' my.knots = c(2.5, 3, 3.5)
#' mod = lm(Sepal.Length ~ Species * ns(Sepal.Width, knots = my.knots), data = iris)
#' ## my.knots is not a predictor, so need to name it in 'params'
#' ref_grid(mod, params = "my.knots") 
#' 
ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("cov.keep"),
                     mult.names, mult.levs, 
                     options = get_emm_option("ref_grid"), data, df, type, 
                     regrid, nesting, offset, sigma, 
                     counterfactuals, ## wt.counter, avg.counter = TRUE,
                     nuisance = character(0), non.nuisance, wt.nuis = "equal",
                      rg.limit = get_emm_option("rg.limit"), ...) 
{
    if(!missing(counterfactuals)) { # route this to a different routine
        cl = match.call()
        cl[[1]] = as.name(".cf.refgrid") # internal function for counterfactuals
        return(eval(cl))
    }
    
    # hack to ignore 'tran' in dots arguments and interpret 'transform' as `regrid` :
    .foo = function(t,tr,tra,tran, transform = NULL, ...) transform
    .bar = .foo(...)
    if (!is.null(.bar)) {
        regrid = .bar
        message("In 'ref_grid()', use 'regrid = ...' rather than 'transform = ...' ",
        "to avoid this message.")
    }
    
    if (!missing(df)) {
        if(is.null(options)) options = list()
        options$df = df
    }
    
    # recover the data
    if (missing(data)) {
        data = try(recover_data (object, data = NULL, ...))
        if (inherits(data, "try-error"))
            stop("Perhaps a 'data' or 'params' argument is needed")
    }
    # note if options$delts non-null, we called 2nd time from emtrends and data is ok as-is
    else if (is.null(options$delts)) # attach needed attributes to given data
        data = recover_data(object, data = as.data.frame(data), ...)
    
    if(is.character(data)) # 'data' is in fact an error message
        stop(data)

    ## undocumented hook to use ref_grid as slave to get data
    if (!is.null(options$just.data))
        return(data)
    
    
    trms = attr(data, "terms")
    
    # find out if any variables are coerced to factors or vice versa
    coerced = .find.coerced(trms, data) # now list with members 'factors' and 'covariates'
    
    # convenience function
    sort.unique = function(x) sort(unique(x))
    
    # Get threshold for max #levels to keep a covariate
    if (is.null(cov.keep))
        cov.keep = character(0)
    cov.thresh = max(c(1, suppressWarnings(as.integer(cov.keep))), na.rm = TRUE)
    if (is.logical(cov.reduce)) {
        if (!cov.reduce)
            cov.thresh = 99 # big enough!
        cov.reduce = mean
    }
    
    
    
    
    # Ensure cov.reduce is a function or list thereof
    dep.x = list() # list of formulas to fit later
    fix.cr = function(cvr) {
        if (inherits(cvr, "formula")) {
            if (length(cvr) < 3)
                stop("Formulas in 'cov.reduce' must be two-sided")
            lhs = .all.vars(cvr)[1]
            dep.x[[lhs]] <<- cvr
            cvr = mean 
        }
        else if (!inherits(cvr, c("function","list")))
            stop("Invalid 'cov.reduce' argument")
        cvr
    }
    
    # IMPORTANT: following stmts may also affect x.dep
    if (is.list(cov.reduce))
        cov.reduce = lapply(cov.reduce, fix.cr)
    else
        cov.reduce = fix.cr(cov.reduce)
    
    # zap any formulas that are also in 'at'
    if (!missing(at))
        for (xnm in names(at)) dep.x[[xnm]] = NULL
    
    
    # local cov.reduce function that works with function or named list
    cr = function(x, nm) {
        if (is.function(cov.reduce))
            cov.reduce(x)
        else if (hasName(cov.reduce, nm))
            cov.reduce[[nm]](x)
        else
            mean(x)
    }
    
    # initialize empty lists
    ref.levels = matlevs = xlev = chrlev = list()
    
    for (nm in attr(data, "responses")) {
        y = data[[nm]]
        if (is.matrix(y))
            matlevs[[nm]] = apply(y, 2, mean)
        else
            ref.levels[[nm]] = mean(y)
    }
    
    for (nm in attr(data, "predictors")) {
        x = data[[nm]]
        if (is.matrix(x) && ncol(x) == 1)   # treat 1-column matrices as covariates
            x = as.numeric(x)
        
        # Save the original levels of factors, no matter what
        if (is.factor(x) && !(nm %in% coerced$covariates))
            xlev[[nm]] = levels(.chk.fac(x))
            # (drops any used levels unless allow.na.levs option is TRUE)
        else if (is.character(x)) # just like a factor
            xlev[[nm]] = sort(unique(x))
    
        # Now go thru and find reference levels...
        # mentioned in 'at' list but not coerced factor
        if (!(nm %in% coerced$factors) && !missing(at) && (hasName(at, nm)))
            ref.levels[[nm]] = at[[nm]]
        # factors not in 'at'
        else if (is.factor(x) && !(nm %in% coerced$covariates))
            ref.levels[[nm]] = levels(.chk.fac(x))
        else if (is.character(x) || is.logical(x))
            ref.levels[[nm]] = chrlev[[nm]] = sort.unique(x)
        # matrices
        else if (is.matrix(x)) {
            # Matrices -- reduce columns thereof, but don't add to baselevs
            matlevs[[nm]] = apply(x, 2, cr, nm)
            # if cov.reduce returns a vector, average its columns
            if (is.matrix(matlevs[[nm]]))
                matlevs[[nm]] = apply(matlevs[[nm]], 2, mean)
        }
        # covariate coerced, or not mentioned in 'at'
        else {
            # single numeric pred but coerced to a factor - use unique values
            # even if in 'at' list. We'll fix this up later
            if (nm %in% coerced$factors)            
                ref.levels[[nm]] = sort.unique(x)
            
            # Ordinary covariates - summarize based on cov.keep
            else {
                if ((length(uval <- sort.unique(x)) > cov.thresh) && !(nm %in% cov.keep))
                    ref.levels[[nm]] = cr(as.numeric(x), nm)
                else {
                    ref.levels[[nm]] = uval
                    cov.keep = c(cov.keep, nm)
                }
            }
        }
    }
    

    if (!missing(non.nuisance))
        nuisance = setdiff(names(ref.levels), non.nuisance)

    # Now create the reference grid
    if(no.nuis <- (length(nuisance) == 0)) {
        .check.grid(ref.levels, rg.limit)
        grid = do.call(expand.grid, ref.levels)
    }
    else {
        nuis.info = .setup.nuis(nuisance, ref.levels, trms, rg.limit)
        grid = nuis.info$grid
    }
    
    # undocumented hook to expand grid by increments of 'var' (needed by emtrends)
    if (!is.null(delts <- options$delts)) {
        var = options$var
        n.orig = nrow(grid) # remember how many rows we had
        grid = grid[rep(seq_len(n.orig), length(delts)), , drop = FALSE]
        options$var = options$delts = NULL
        # grid[[var]] = grid[[var]] + rep(delts, each = n.orig)
        #     (we have to wait until after covariate calcs to do this)
    }
    
    # add any matrices
    for (nm in names(matlevs)) {
        tmp = matrix(rep(matlevs[[nm]], each=nrow(grid)), nrow=nrow(grid))
        dimnames(tmp) = list(NULL, names(matlevs[[nm]]))
        grid[[nm]] = tmp
    }

    # resolve any covariate formulas
    for (xnm in names(dep.x)) {
        if ((xnm %in% c("ext", "extern", "external")) && !(xnm %in% names(grid))) { # use external fcn
            fun = get(as.character(dep.x[[xnm]][[3]]), inherits = TRUE)
            rslts = fun(grid)   # should be some thing that supports names() and [[]], e.g., a list or d.f.
            for (nm in  intersect(names(rslts), names(grid))) {
                grid[[nm]] = rslts[[nm]]
                ref.levels[[nm]] = NULL
            }
        }
        else if (!all(.all.vars(dep.x[[xnm]]) %in% names(grid)))
            stop("Formulas in 'cov.reduce' must predict covariates actually in the model")
        else { # Use lm() to predict this covariate
            xmod = lm(dep.x[[xnm]], data = data)
            grid[[xnm]] = predict(xmod, newdata = grid)
            ref.levels[[xnm]] = NULL
        }
    }
    
    # finish-up our hook for expanding the grid
    if (!is.null(delts)) # add increments if any
        grid[[var]] = grid[[var]] + rep(delts, each = n.orig)
    
    if (!is.null(attr(data, "pass.it.on")))   # a hook needed by emm_basis.gamlss et al.
        attr(object, "data") = data
    
    ###!! Prevent a warning like in https://stackoverflow.com/questions/68969384/emmeans-warning-in-model-frame-defaultformula-data-data-variable-gr/68990172#68990172
    xl = xlev
    modnm = rownames(attr(trms, "factors"))
    chk = sapply(modnm, function(mn) mn %in% names(xl))
    for(i in which(!chk)) { # replace names in xl - e.g., as.factor(trt) where trt already a factor
        fn = all.vars(reformulate(modnm[i]))
        if (length(fn) == 1)
            names(xl)[names(xl) == fn] = modnm[i]
    }
    ###!! If we remove this code, also need to change xl back to xlev in 'basis =' call below
    
    # we've added args `misc` and `options` so emm_basis methods can access and use these if they want
    basis = emm_basis(object, trms, xl, grid, misc = attr(data, "misc"), options = options, ...)

    environment(basis$dffun) = baseenv()   # releases unnecessary storage
    if(length(basis$bhat) != ncol(basis$X))
        stop("Something went wrong:\n",
             " Non-conformable elements in reference grid.",
             call. = TRUE)
    
    if(!no.nuis) {
        basis = .basis.nuis(basis, nuis.info, wt.nuis, ref.levels, data, grid, ref.levels)
        grid = basis$grid
        nuisance = ref.levels[nuis.info$nuis] # now nuisance has the levels info
        ref.levels = basis$ref.levels
    }
    
    misc = basis$misc
    
    ### Figure out if there is a response transformation...
    # next stmt assumes that model formula is 1st argument (2nd element) in call.
    # if not, we probably get an error or something that isn't a formula
    # and it is silently ignored
    frm = try(formula(eval(attr(data, "call")[[2]], environment(trms))), silent = TRUE)
    if (inherits(frm, "formula")) { # response may be transformed
        lhs = if(length(frm) == 2) NULL
              else frm[-3]
        tran = setdiff(.all.vars(lhs, functions = TRUE), c(.all.vars(lhs), "~", "cbind", "+", "-", "*", "/", "^", "%%", "%/%"))
        if(length(tran) > 0) {
            # we are now supporting scale() as well as some functions from datawizard
            if (tran[1] %in% c("scale", "center", "centre", "standardize", "standardise")) { # we'll try to set it up based on terms component
                pv = try(attr(terms(object), "predvars"), silent = TRUE)
                if (!inherits(pv, "try-error") && !is.null(pv)) {
                    scal = which(sapply(c(sapply(pv, as.character), "foo"), 
                                        function(x) x[1]) == tran[1])
                    if(length(scal) > 0) {
                        pv = pv[[scal[1]]]
                        ctr = ifelse(is.null(pv$center), 0, ifelse(pv$center, pv$center, 0))
                        scl = ifelse(is.null(pv$scale), 1, ifelse(pv$scale, pv$scale, 1))
                        tran = make.tran("scale", y = 0, center = ctr, scale = scl)
                    }
                }
                if (is.character(tran)) { # didn't manage to find params
                    tran = NULL
                    message("NOTE: Unable to recover scale() parameters. See '? make.tran'")
                }
            }
 
            else if (tran[1] == "linkfun")
                tran = as.list(environment(trms))[c("linkfun","linkinv","mu.eta","valideta","name")]
            else {
                if (tran[1] == "I") 
                    tran = "identity"
                tran = paste(tran, collapse = ".")  
                # length > 1: Almost certainly unsupported, but facilitates a more informative error message
                const.warn = "There are unevaluated constants in the response formula\nAuto-detection of the response transformation may be incorrect"
                # Look for a multiplier, e.g. 2*sqrt(y)
                tst = strsplit(strsplit(as.character(lhs[2]), "\\(")[[1]][1], "\\*")[[1]]
                if(length(tst) > 1) {
                    mul = try(eval(parse(text = tst[1])), silent = TRUE)
                    if(!inherits(mul, "try-error")) {
                        misc$tran.mult = mul
                        tran = gsub("\\*\\.", "", tran)
                    }
                    else
                        warning(const.warn)
                }
                
                # look for added constant, e.g. log(y + 1)
                tst = strsplit(as.character(lhs[2]), "\\(|\\)|\\+")[[1]]
                if (length(tst) > 2) {
                    const = try(eval(parse(text = tst[3])), silent = TRUE)
                    if (!inherits(const, "try-error") && (length(tst) == 3))
                        misc$tran.offset = const
                    else
                        warning(const.warn)
                }
            }
            if(is.null(misc[["tran"]]))
                misc$tran = tran
            else
                misc$tran2 = tran
            misc$inv.lbl = "response"
        }
    }
    
    # Take care of multivariate response
    multresp = character(0) ### ??? was list()
    ylevs = misc$ylevs
    if(!is.null(ylevs)) { # have a multivariate situation
        if (missing(mult.levs))
            mult.levs = ylevs
        if(!missing(mult.names)) {
            k = seq_len(min(length(ylevs), length(mult.names)))
            names(mult.levs)[k] = mult.names[k]
        } 
        if (length(ylevs) > 1)
            ylevs = list(seq_len(prod(sapply(mult.levs, length))))
        
        k = prod(sapply(mult.levs, length))
        if (k != length(ylevs[[1]])) 
            stop("supplied 'mult.levs' is of different length ",
                 "than that of multivariate response")
        for (nm in names(mult.levs))
            ref.levels[[nm]] = mult.levs[[nm]]
        multresp = names(mult.levs)
        MF = do.call("expand.grid", mult.levs)
        grid = merge(grid, MF)
    }
            
    # add any matrices
    for (nm in names(matlevs))
        grid[[nm]] = matrix(rep(matlevs[[nm]], 
                                each=nrow(grid)), nrow=nrow(grid))
    
# Here's a complication. If a numeric predictor was coerced to a factor, we had to
# include all its levels in the reference grid, even if altered in 'at'
# Moreover, whatever levels are in 'at' must be a subset of the unique values
# So we now need to subset the rows of the grid and linfct based on 'at'

    problems = if (!missing(at)) 
        intersect(c(multresp, coerced$factors), names(at)) 
    else character(0)
    if (length(problems) > 0) {
        incl.flags = rep(TRUE, nrow(grid))
        for (nm in problems) {
            if (is.numeric(ref.levels[[nm]])) {
                dig = 3 - log10(max(abs(ref.levels[[nm]])))
                at[[nm]] = round(at[[nm]], digits = dig)
                ref.levels[[nm]] = round(ref.levels[[nm]], digits = dig)
                grid[[nm]] = round(grid[[nm]], digits = dig)
            }
            # get only "legal" levels
            at[[nm]] = ref.levels[[nm]] = at[[nm]][at[[nm]] %in% ref.levels[[nm]]]
            rows = numeric(0)
            for(x in at[[nm]])
                rows = c(rows, which(grid[[nm]] == x))
            grid = grid[rows, , drop = FALSE]
            grid[[nm]] = factor(grid[[nm]], levels = at[[nm]])
            basis$X = basis$X[rows, , drop = FALSE]
        }
    }

    # Any offsets??? (misc$offset.mult might specify removing or reversing the offset)
    om = ifelse(is.null(misc$offset.mult), 1, misc$offset.mult)
    oval = 0
    if (!missing(offset)) {  # For safety, we always treat it as scalar
        if (offset[1] != 0)
            oval = offset[1]
        if(".static.offset." %in% names(grid))
            grid$.static.offset. = ref.levels$.static.offset. = oval
    }
    else {
        if(".static.offset." %in% names(grid)) { # static offset is available
            oval = om * grid[[".static.offset."]]
        }
        #else implied
        if(!is.null(attr(trms,"offset"))) {
            if (any(om != 0))
                oval = om * (oval + .get.offset(trms, grid))
        }
    }
    if(any(oval != 0))
        grid[[".offset."]] = oval
    
    
    ### --- Determine weights for each grid point
    if (!hasName(data, "(weights)"))
        data[["(weights)"]] = 1
    cov.keep = intersect(unique(cov.keep), names(ref.levels))
    nms = union(union(union(names(xlev), names(chrlev)), coerced$factors), cov.keep)
    nms = intersect(nms, names(grid))
    #### Old code...
    # if (!covnest)
    #     nms = union(union(names(xlev), names(chrlev)), coerced$factors) # only factors, no covariates or mult.resp
    # else
    #     nms = setdiff(names(ref.levels)[sapply(ref.levels, length) > 1], multresp) # all names (except multiv) for which there is > 1 level
    if (length(nms) == 0)
        wgt = rep(1, nrow(grid))  # all covariates; give each weight 1
    else {
        id = .my.id(data[, nms, drop = FALSE])
        uid = !duplicated(id)
        key = do.call(paste, unname(data[uid, nms, drop = FALSE]))
        key = key[order(id[uid])]
        tgt = do.call(paste, unname(grid[, nms, drop = FALSE]))
        wgt = rep(0, nrow(grid))
        for (i in seq_along(key))
            wgt[tgt == key[i]] = sum(data[["(weights)"]][id==i])
    }
    grid[[".wgt."]] = wgt
    
    model.info = list(call = attr(data,"call"), terms = trms, xlev = xlev)
    if (!is.null(mm <- basis$model.matrix)) { # submodel support
        attr(mm, "factors") = .smpFT(trms)
        model.info$model.matrix = mm  
    }
    # Detect any nesting structures
    nst = .find_nests(grid, trms, coerced$orig, ref.levels)
    if (length(nst) > 0)
        model.info$nesting = nst

    misc$is.new.rg = TRUE
    misc$ylevs = NULL # No longer needed
    misc$estName = "prediction"
    misc$estType = "prediction"
    misc$infer = c(FALSE,FALSE)
    misc$level = .95
    misc$adjust = "none"
    misc$famSize = nrow(grid)
    if(is.null(misc$avgd.over))
       misc$avgd.over = character(0)
    
    # emm_basis may have provided a sigma. If not, use sigma value from model or args
    if(is.null(misc$sigma) && missing(sigma)) { # Get 'sigma(object)' if available, else NULL
        sigma = suppressWarnings(try(stats::sigma(object), silent = TRUE))
        if (inherits(sigma, "try-error"))
            sigma = NULL
        misc$sigma = sigma
    }
    # ... but override it only if sigma != NA
    if(is.null(misc$sigma) || (length(misc$sigma) == 0) || !is.na(misc$sigma[1]))
        misc$sigma = sigma
    

    post.beta = basis$post.beta
    if (is.null(post.beta))
        post.beta = matrix(NA)
    
    predictors = intersect(attr(data, "predictors"), names(grid))
    # if(!missing(counterfactuals)) predictors = c(predictors, ".obs.no.")
    
    simp.tbl = environment(trms)$.simplify.names.
    if (! is.null(simp.tbl)) {
        names(grid) = .simplify.names(names(grid), simp.tbl)
        predictors = .simplify.names(predictors, simp.tbl)
        names(ref.levels) = .simplify.names(names(ref.levels), simp.tbl)
        if (!is.null(post.beta)) names(post.beta) = .simplify.names(names(post.beta), simp.tbl)
        if (!is.null(model.info$nesting)) {
            model.info$nesting = lapply(model.info$nesting, .simplify.names, simp.tbl)
            names(model.info$nesting) = .simplify.names(names(model.info$nesting), simp.tbl)
        }
        
        environment(trms)$.simplify.names. = NULL
    }
    
    result = new("emmGrid",
         model.info = model.info,
         roles = list(predictors = predictors, 
                      responses = attr(data, "responses"), 
                      multresp = multresp,
                      nuisance = nuisance),
         grid = grid, levels = ref.levels, matlevs = matlevs,
         linfct = basis$X, bhat = basis$bhat, nbasis = basis$nbasis, V = basis$V,
         dffun = basis$dffun, dfargs = basis$dfargs, 
         misc = misc, post.beta = post.beta)
        
    if (!missing(type)) {
        if (is.null(options)) options = list()
        options$predict.type = type
    }
    
    if (!missing(nesting)) {
        result@model.info$nesting = lst = .parse_nest(nesting)
        if(!is.null(lst)) {
            nms = union(names(lst), unlist(lst))
            if(!all(nms %in% names(result@grid)))
                stop("Nonexistent variables specified in 'nesting'")
            result@misc$display = .find.nonempty.nests(result, nms)
        }
    }
        
    else if (!is.null(nst <- result@model.info$nesting)) {
        result@misc$display = .find.nonempty.nests(result)
        if (get_emm_option("msg.nesting"))
            message("NOTE: A nesting structure was detected in the ",
                    "fitted model:\n    ", .fmt.nest(nst))
    }

    result = .update.options(result, options, ...)

    if(!is.null(hook <- misc$postGridHook)) {
        if (is.character(hook))
            hook = get(hook)
        result@misc$postGridHook = NULL
        result = hook(result, ...)
    }
    if(!missing(regrid)) {
        # if(missing(wt.counter)) wt.counter = 1
        result = regrid(result, transform = regrid, sigma = sigma, ...)
    }
    
    .save.ref_grid(result)
    result
}


#### End of ref_grid ------------------------------------------

# local utility to save each newly constructed ref_grid, if enabled
# Goes into global environment unless .Last.ref_grid is found further up
.save.ref_grid = function(object) {
    if (is.logical(isnewrg <- object@misc$is.new.rg))
        if(isnewrg && get_emm_option("save.ref_grid"))
            assign(".Last.ref_grid", object, inherits = TRUE)
}



# This function figures out which covariates in a model 
# have been coerced to factors. And also which factors have been coerced
# to be covariates
.find.coerced = function(trms, data) {
    if (ncol(data) == 0) 
        return(list(factors = integer(0), covariates = integer(0)))
    isfac = sapply(data, function(x) inherits(x, "factor"))
    
    # Character vectors of factors and covariates in the data...
    facs.d = names(data)[isfac]
    covs.d = names(data)[!isfac]
    
    lbls = attr(trms, "term.labels")
    M = model.frame(trms, utils::head(data, 2)) #### just need a couple rows
    isfac = sapply(M, function(x) inherits(x, "factor"))
    
    # Character vector of terms in the model frame that are factors ...
    facs.m = names(M)[as.logical(isfac)]
    covs.m = setdiff(names(M), facs.m)
    
    # Exclude the terms that are already factors
    # What's left will be things like "factor(dose)", "interact(dose,treat)", etc
    # we're saving these in orig
    orig = cfac = setdiff(facs.m, facs.d)
    if(length(cfac) != 0) {
        cvars = lapply(cfac, function(x) .all.vars(stats::reformulate(x))) # Strip off the function calls
        cfac = intersect(unique(unlist(cvars)), covs.d) # Exclude any variables that are already factors
    }
    
    # Do same with covariates
    ccov = setdiff(covs.m, covs.d)
    orig = c(orig, ccov)
    if(length(ccov) > 0) {
        cvars = lapply(ccov, function(x) .all.vars(stats::reformulate(x)))
        ccov = intersect(unique(unlist(cvars)), facs.d)
    }
    
    list(factors = cfac, covariates = ccov, orig = orig)
}

# My replacement for plyr::id(, drop = TRUE)
.my.id = function(data){
    p = do.call(paste, data)
    u = unique(p)
    match(p, u)
}

# Utility to error-out when potential reference grid size is too big
.check.grid = function(levs, limit = get_emm_option("rg.limit")) {
    size = prod(sapply(levs, length))
    if (size > limit)
        stop("The rows of your requested reference grid would be ", size, ", which exceeds\n",
             "the limit of ", limit, " (not including any multivariate responses).\n",
             "Your options are:\n",
             "  1. Specify some (or more) nuisance factors using the 'nuisance' argument\n",
             "     (see ?ref_grid). These must be factors that do not interact with others.\n",
             "  2. Add the argument 'rg.limit = <new limit>' to the call. Be careful,\n",
             "     because this could cause excessive memory use and performance issues.\n",
             "     Or, change the default via 'emm_options(rg.limit = <new limit>)'.\n",
             call. = FALSE)
}

# Utility to set up the grid for nuisance factors. This consists of two or more
# data.frames rbinded together:
#   * the expanded grid for all factors *not* in nuis, with the nuis factors set 
#     at their first levels
#   * for each factor f in nuis, a set of rows for (levs$f), with the other 
#     factors at their first levels (this is arbitrary)
# In addition, we return a character vector 'row.assign' corresponding to the rows 
# in the grid, telling which is what: ".main.grid." for the first part of the grid,
# otherwise factor names from nuis.
# We also return 'nuis' itself - which may be reduced since illegal 
# entries are silently removed
###
# Update July 2024...
# All this trickery assumes that columns of X are associated with model terms!
# So this does not work for anything that's been re-gridded.
# Those (almost?) always have X = I. Unfortunately we won't see X until
# later, but we can still stop it before we inflict damage...
# Note: we also provide a simpler check: misc$regrid.flag is non-NULL
.setup.nuis = function(nuis, levs, trms, rg.limit) {
    firsts =  args = lapply(levs, function(x) x[1])
    nuis = intersect(nuis, names(levs))
    # sanity checks on terms, and term indexes
    fsum = rep(99, length(nuis))
    tbl = attr(trms, "factors")
    rn = row.names(tbl) = sapply(row.names(tbl), function(nm)
        paste(all.vars(reformulate(nm)), collapse = ","))
    for (i in seq_along(nuis)) {
        f = nuis[i]
        if(f %in% rn)
            fsum[i] = sum(tbl[f, ])
    }
    nuis = nuis[fsum == 1]   # silently remove any unfound or interacting factors

    # top part...
    non.nuis = setdiff(names(levs), nuis)
    for (n in non.nuis)
        args[[n]] = levs[[n]]
    .check.grid(args, rg.limit)
    grid = do.call(expand.grid, args)
    ra = rep(".main.grid.", nrow(grid))
    # bottom parts
    for (f in nuis) {
        args = firsts
        args[[f]] = levs[[f]]
        grid = rbind(grid, do.call(expand.grid, args))
        ra = c(ra, rep(f, length(levs[[f]])))
    }
    list(grid = grid, row.assign = ra, nuis = nuis)
}

# Do the required post-processing for nuisance factors...
# After we get the model matrix for this grid, we'll average each set of rows in the
# bottom part, and substitute those averages in the required columns in the top part 
# of the model matrix.
.basis.nuis = function(basis, info, wt, levs, data, grid, ref.levels) {
    X = basis$X
    if(!is.null(basis$misc$regrid.flag) || all(apply(X, 1, \(x) sum(x != 0)) == 1))   # each row has 1 nonzero element
        stop("Sorry, 'nuisance' specs are not allowed for this situation.",
             " Revise the call accordingly.", call. = FALSE)
    ra = info$row.assign
    r. = rep(".", length(ra))  # fillers
    n = sum(ra == ".main.grid.")
    k = nrow(X) / length(ra)   # multivariate dimension
    nuis = info$nuis
    wts = lapply(nuis, function(f) {
        if (wt == "equal")
            w = rep(1, length(levs[[f]]))
        else {
            x = data[[f]]
            w = sapply(levs[[f]], function(lev) sum(x == lev))
        }
        w / sum(w)
    })
    names(wts) = nuis
    
    # In a multivariate case, we have to repeat the same operations for each block of X rows
    for (m in 1:k) {
        RA = c(rep(r., m - 1), ra, rep(r., k - m))
        for (f in nuis) {
            subX = X[RA == f, , drop = FALSE]
            cols = which(apply(subX, 2, function(x) diff(range(x)) > 0))
            subX = sweep(subX[, cols, drop = FALSE], 1, wts[[f]], "*")
            avg = apply(subX, 2, sum)
            avg = matrix(rep(avg, each = n), nrow = n) # now several copies
            X[RA == ".main.grid.", cols] = avg
        }
    }
    basis$misc$nuis = nuis
    basis$misc$avgd.over = paste(length(nuis), "nuisance factors")
    RA = rep(ra, k)
    basis$X = X[RA == ".main.grid.", , drop = FALSE]
    non.nuis = setdiff(names(ref.levels), info$nuis)
    basis$ref.levels = ref.levels[non.nuis]
    basis$grid = grid[ra == ".main.grid.", non.nuis, drop = FALSE]
    basis
}

# Internal function to do reference grid for counterfactuals
.cf.refgrid = function(object, counterfactuals, data, options = list(), ...) {
    if(missing(data))
        data = recover_data(object, ...)
    if(!hasName(data, "(weights)"))
        pwts = rep(1, nrow(data))
    else
        pwts = data[["(weights)"]]
    
    # Start with just the ordinary reference grid
    rg = ref_grid(object, data = data, ...)
    cfac = intersect(counterfactuals, names(rg@levels))
    clevs = rg@levels[cfac]
    cgrid = do.call(expand.grid, clevs)
    # handle nasty fact that character predictors don't act like factors
    for (j in cfac)
        if(is.character(data[[j]])) 
            cgrid[[j]] = as.character(cgrid[[j]])
    
    # Get the stuff we need for each main dataset step
    link = .get.link(rg@misc)
    if(is.null(link))
        link = make.link("identity")
    trms = rg@model.info$terms
    xlev = rg@model.info$xlev
    misc = list()
    offset = .get.offset(object, grid = data)
    k = ifelse(length(mr <- rg@roles$multresp) == 0, 1, length(rg@levels[[mr]])) # grid expansion factor
    
    # Index sets for combinations of factors
    pg = apply(cgrid, 1, paste, collapse = ".")
    pd = apply(data[cfac], 1, paste, collapse = ".")
    cidx = lapply(pg, \(x) which(pd == x))

    # special case for covariates with no matches
    if(all(sapply(cidx, length) == 0))
        cidx = list(seq_len(nrow(data)))
    
    # account for any NAs in bhat
    nonNA = !is.na(rg@bhat)
    # ensure we include all levels of cfacs with data
    all.active = sort(unlist(cidx))
    deadrows = sapply(cidx, function(x) x[1])
    offset = c(offset, rep(mean(offset), length(deadrows)))
    pwts = c(pwts, rep(mean(pwts), length(deadrows)))
    data = rbind(data, data[deadrows, ])
    n = nrow(data)
    mymean = function(x) ifelse(is.null(x), NA, mean(x))
    
    # get means of groups of prior weights
    mpwt = sapply(cidx, \(i) mean(pwts[i]))
    
    ## Compile the averaged results for delta method
    DL = matrix(nrow = 0, ncol = sum(nonNA))
    bh = numeric(0)
    embmeth = .find_method(object, "emm_basis")
    for (i in seq_len(nrow(cgrid))) {
        g = data
        for(j in cfac)
            g[all.active, j] = cgrid[i,j]
        bas = embmeth(object, trms, xlev, g, ...)
        if(!is.null(bas$misc$postGridHook))
            stop("Sorry, we do not support counterfactuals for this situation.")
        X = bas$X[, nonNA, drop = FALSE]
        eta = X %*% bas$bhat[nonNA]
        yhat = link$linkinv(eta + offset)
        d = link$mu.eta(eta) * rep(pwts, k)   # includes prior weights
        X = sweep(X, 1, d, "*")
        
        pos = 0
        XX = matrix(nrow = 0, ncol = ncol(X))
        for(I in 1:k) {
            XX = sapply(cidx, \(i) apply(X[pos + i, , drop = FALSE], 2, mymean)) / mpwt
            DL = rbind(DL, t(XX))
            yy = sapply(cidx, \(i) ifelse(length(i) == 0, NA, mean(yhat[i + pos])))
            bh = c(bh, yy)
            pos = pos + n
        }
    }
    RG = rg
    RG@bhat = bh
    nonNA = !is.na(bh)
    RG@linfct = diag(nrow(DL))
    RG@V = DL %*% rg@V %*% t(DL)
    levs = rg@levels
    levs[cfac] = NULL
    alevs = clevs
    names(alevs) = paste0("actual_", cfac)
    levs = c(alevs, clevs)
    if (k > 1)
        levs = c(levs, rg@levels[length(rg@levels)])
    RG@levels = levs
    wts = sapply(cidx, length) * mpwt
    RG@grid = do.call("expand.grid", levs)
    RG@grid$.wgt. = rep(wts, length(bh)/length(wts))
    misc = rg@misc
    if(!is.null(misc$inv.lbl))
        misc$estName = misc$inv.lbl
    misc[c("tran", "inv.lbl", "sigma")] = NULL
    RG@misc = c(misc, famSize = length(bh), cf.grid = TRUE)
    RG@model.info$model.matrix = NULL
    RG@roles$predictors = c(names(alevs), names(clevs))
    RG@roles$counterfactuals = cfac
    if (all(nonNA))
        RG@nbasis = estimability::all.estble
    else {
        RG@nbasis = matrix(0, ncol = sum(!nonNA), nrow = length(bh))
        idx = which(!nonNA)
        for (j in seq_len(ncol(RG@nbasis)))
            RG@nbasis[idx[j], j] = 1
        RG@V = RG@V[nonNA, nonNA]
    }
    RG
}


## OLD CODE...
# Set up grid for counterfactuals - i.e., copies of whole dataset with
# counterfactual levels substituted, with obs index varying the slowest
# .setup.cf = function(levs, data) {
#     lv = arg = levs[-length(levs)] # omit .obs.no.
#     arg$stringsAsFactors = FALSE
#     g = do.call(expand.grid, arg)
#     idx = rep(seq_len(nrow(data)), each = nrow(g))
#     xdata = data[idx, , drop = FALSE]
#     idx = matrix(seq_len(nrow(xdata)), nrow = nrow(g))
#     for (i in seq_along(g[, 1]))
#         for (j in names(lv))
#             xdata[idx[i, ], j] = g[i, j]
#     xdata
# }

context("head/tail for emmGrid")

# head()/tail() for emmGrid should match base-R semantics, including a
# negative n. Previously the negative-n branches of head and tail were
# effectively swapped.
test_that("head()/tail() honor base-R negative-n semantics", {
    warp.rg <- ref_grid(lm(breaks ~ wool * tension, data = warpbreaks))
    ten <- as.character(warp.rg@grid$tension)   # 6 rows: L L M M H H
    expect_equal(length(ten), 6L)

    gten <- function(x) as.character(x@grid$tension)

    # positive n: first / last n rows
    expect_equal(gten(head(warp.rg, 3)), ten[1:3])
    expect_equal(gten(tail(warp.rg, 3)), ten[4:6])

    # negative n (base R): head(-2) drops the LAST 2; tail(-2) drops the FIRST 2
    expect_equal(gten(head(warp.rg, -2)), ten[1:4])
    expect_equal(gten(tail(warp.rg, -2)), ten[3:6])
})

##############################################################################
#    Copyright (c) 2012-2025 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

# emmip code - interaction plots

#' Interaction-style plots for estimated marginal means
#'
#' Creates an interaction plot of EMMs based on a fitted model and a simple
#' formula specification.
#' 
#' @export
emmip = function(object, formula, ...) {
    UseMethod("emmip")
}

# Our one method
#' @rdname emmip
#' @param object An object of class \code{emmGrid}, or a fitted model of a class
#'   supported by the \pkg{emmeans} package
#' @param formula Formula of the form 
#'   \code{trace.factors ~ x.factors | by.factors}. The EMMs are
#'   plotted against \code{x.factor} for each level of \code{trace.factors}.
#'   \code{by.factors} is optional, but if present, it determines separate
#'   panels. Each element of this formula may be a single factor in the model,
#'   or a combination of factors using the \code{*} operator.
#' @param type As in \code{\link{predict.emmGrid}}, this determines
#'   whether we want to inverse-transform the predictions
#'   (\code{type = "response"}) or not (any other choice). The default is
#'   \code{"link"}, unless the \code{"predict.type"} option is in force; see
#'   \code{\link{emm_options}}.
#'   In addition, the user may specify \code{type = "scale"} to create a
#'   transformed scale for the vertical axis based on \code{object}'s response 
#'   transformation or link function. 
#' @param CIs Logical value. If \code{TRUE}, confidence intervals (or HPD intervals
#'   for Bayesian models) are added to the plot 
#'   (works only with \code{engine = "ggplot"}).
#' @param PIs Logical value. If \code{TRUE}, prediction intervals are added to the plot 
#'   (works only with \code{engine = "ggplot"}). This is allowed only if the
#'   underlying model family is \code{"gaussian"}.
#'   If both \code{CIs} and
#'   \code{PIs} are \code{TRUE}, the prediction intervals will be somewhat
#'   longer, lighter, and thinner than the confidence intervals. Additional
#'   parameters to \code{\link{predict.emmGrid}} (e.g., \code{sigma}) may be passed via
#'   \code{...}. For Bayesian models, PIs require \code{frequentist = TRUE} and 
#'   a value for \code{sigma}.
#' @param style Optional character value. This has an effect only when the
#'   horizontal variable is a single numeric variable. If \code{style} is
#'   unspecified or \code{"numeric"}, the horizontal scale will be numeric and
#'   curves are plotted using lines (and no symbols). With \code{style =
#'   "factor"}, the horizontal variable is treated as the levels of a factor
#'   (equally spaced along the horizontal scale), and curves are plotted using
#'   lines and symbols. When the horizontal variable is character or factor, or
#'   a combination of more than one predictor, \code{"factor"} style is always used.
#' @param engine Character value matching \code{"ggplot"} (default), 
#'   \code{"lattice"}, or \code{"none"}. The graphics engine to be used to produce the plot.
#'   These require, respectively, the \pkg{ggplot2} or \pkg{lattice} package to
#'   be installed. Specifying \code{"none"} is equivalent to setting \code{plotit = FALSE}.
#' @param plotit Logical value. If \code{TRUE}, a graphical object is returned;
#'   if \code{FALSE}, a data.frame is returned containing all the values
#'   used to construct the plot.
#' @param nesting.order Logical value. If \code{TRUE}, factors that are nested
#'   are presented in order according to their nesting factors, even if those nesting
#'   factors are not present in \code{formula}. If \code{FALSE}, only the
#'   variables in \code{formula} are used to order the variables.
#' @param abbr.len Integer vector of length 1 or 2 used to specify that factor levels
#'   be abbreviated to the specified lengths, using \code{\link{abbreviate}}. Any values
#'   less than 1 are ignored. \code{abbr.len[1]} applies to a factor plotted along the 
#'   horizontal axis, and \code{abbr.len[2]} (if present) applies to trace factors (shown in a legend)
#'   or by factors (shown in facet labels). 
#' @param ... Additional arguments passed to \code{\link{emmeans}} (when
#'   \code{object} is not already an \code{emmGrid} object),
#'   \code{predict.emmGrid}, 
#'   \code{emmip_ggplot}, or \code{emmip_lattice}.
#'   
#' @section Details:
#' If \code{object} is a fitted model, \code{\link{emmeans}} is called with an
#' appropriate specification to obtain estimated marginal means for each
#' combination of the factors present in \code{formula} (in addition, any 
#' arguments in \code{\dots} that match \code{at}, \code{trend}, 
#' \code{cov.reduce}, or \code{fac.reduce} are passed to \code{emmeans}). 
#' Otherwise, if \code{object} is an \code{emmGrid} object, its first element is 
#' used, and it must contain one estimate for each combination of the factors
#' present in \code{formula}.
#'
#' @return If \code{plotit = FALSE}, a \code{data.frame} (actually, a
#'   \code{summary_emm} object) with the table of EMMs that would be plotted.
#'   The variables plotted are named \code{xvar} and \code{yvar}, and the trace
#'   factor is named \code{tvar}. This data frame has an added \code{"labs"}
#'   attribute containing the labels \code{xlab}, \code{ylab}, and \code{tlab}
#'   for these respective variables. The confidence limits are also
#'   included, renamed \code{LCL} and \code{UCL}.
#'   
#' @return If \code{plotit = TRUE}, the function
#'   returns an object of class \code{"ggplot"} or a \code{"trellis"}, depending
#'   on \code{engine}.
#'   
#' @note Conceptually, this function is equivalent to 
#'   \code{\link{interaction.plot}} where the summarization function is thought 
#'   to return the EMMs.
#' 
#' @seealso \code{\link{emmeans}}, \code{\link{interaction.plot}}.
#' @export
#' @method emmip default
#'
#' @examples
#' #--- Three-factor example
#' noise.lm = lm(noise ~ size * type * side, data = auto.noise)
#'
#' # Separate interaction plots of size by type, for each side
#' emmip(noise.lm, type ~ size | side)
#' 
#' # Same using the "lattice" engine
#' emmip(noise.lm, type ~ size | side, engine = "lattice")
#'
#' # One interaction plot, using combinations of size and side as the x factor
#' # ... with added confidence intervals and some formatting changes
#' emmip(noise.lm, type ~ side * size, CIs = TRUE,
#'     CIarg = list(linewidth = 1.5, alpha = 1, color = "orange"),
#'     dotarg = list(size = 2, shape = "square", color = "black"))
#'     
#' # Same using legacy theme
#' with_emm_options(gg.theme = 1,
#'     emmip(noise.lm, type ~ side * size, CIs = TRUE,
#'         CIarg = list(linewidth = 1.5, alpha = 1, color = "orange"),
#'         dotarg = list(size = 2, shape = "square", color = "black")))
#' 
#' # Create a black-and-white version of above with different linetypes
#' # (Let the linetypes and symbols default to the palette)
#' emmip(noise.lm, type ~ side * size, CIs = TRUE, col = "black",
#'       linearg = list(), dotarg = list(size = 4), CIarg = list(alpha = 1)) +
#'     ggplot2::theme_bw()
#'
#' # One interaction plot using combinations of type and side as the trace factor
#' emmip(noise.lm, type * side ~ size)
#'
#' # Individual traces in panels
#' emmip(noise.lm, ~ size | type * side)
#' 
#' # Example for the 'style' argument
#' fib.lm = lm(strength ~ machine * sqrt(diameter), data = fiber)
#' fib.rg = ref_grid(fib.lm, at = list(diameter = c(12, 14, 15, 25, 36)))
#' emmip(fib.rg, machine ~ diameter)   # curves (because diameter is numeric)
#' emmip(fib.rg, machine ~ diameter, style = "factor")  # points and lines
#' 
#' # For an example using extra ggplot2 code, see 'vignette("messy-data")',
#' # in the section on nested models.
emmip.default = function(object, formula, type, CIs = FALSE, PIs = FALSE,
                         style,
                         engine = get_emm_option("graphics.engine"),
                         # pch = c(1,2,6,7,9,10,15:20), 
                         # lty = 1, col = NULL, 
                         plotit = TRUE, 
                         nesting.order = FALSE, abbr.len = c(0,0), ...) {
    object = .chk.list(object, ...)
    engine = match.arg(engine, c("ggplot", "lattice", "none"))
    if (engine == "ggplot")
        .requireNS("ggplot2",
                   "The 'ggplot' engine requires the 'ggplot2' package be installed.")
    else if (engine == "lattice")
        .requireNS("lattice", 
                "The 'lattice' engine requires the 'lattice' package be installed.")
    else
        plotit = FALSE
    
    specs = .parse.by.formula(formula) # list of lhs, rhs, by
    
    # Glean the parts of ... to use in emmeans call
    # arguments allowed to be passed
    lsa.allowed = c("at","trend","cov.reduce","fac.reduce")
    xargs = list(...)
    emmopts = list(...)
    for (arg in names(xargs)) {
        idx = pmatch(arg, lsa.allowed)
        if (!is.na(idx)) {
            opt = lsa.allowed[idx]
            emmopts[[opt]] = xargs[[arg]]
            xargs[[arg]] = NULL
        }
    }
    
    emmopts$object = object
    emmopts$specs = .reformulate(unlist(specs))
    emmo = do.call("emmeans", emmopts)
    
    # add possibility of type = "scale". If so, we use "response" and set a flag
    if(missing(type)) {
        type = get_emm_option("summary")$predict.type
        if (is.null(type))
            type = .get.predict.type(emmo@misc)
    }
    # If we say type = "scale", set it to "response" and set a flag
    if (nonlin.scale <- (type %.pin% "scale")) 
        type = "response"
            
    type = .validate.type(type)
    
    # get point.est & frequentist if specified (affects only Bayesian models)
    point.est = (\(point.est = "median", ...) point.est)(...)
    frequentist = (\(frequentist = FALSE, ...) frequentist)(...)
    emms = summary(emmo, type = type, infer = c(CIs, FALSE), 
                   point.est = point.est, frequentist = frequentist)
    if(PIs) {
        prd = predict(emmo, interval = "pred", type = type, ...)
        if(!is.null(prd)) {
            emms$LPL = prd$lower.PL
            emms$UPL = prd$upper.PL
        }
    }
    # Ensure the estimate is named "yvar" and the conf limits are "LCL" and "UCL"
    nm = names(emms)
    tgts = c(attr(emms, "estName"), attr(emms, "clNames"))
    subs = c("yvar", "LCL", "UCL")
    for (i in 1:3)
        names(emms)[nm == tgts[i]] = subs[i] 
    attr(emms, "estName") = "yvar"
    if(!CIs)
        emms$LCL = emms$UCL = NULL
    
    if(!nesting.order) { # re-order by factor levels actually in plot
        snm = intersect(nm, unlist(specs))
        ord = do.call(order, unname(emms[rev(snm)]))
        emms = emms[ord, ]
    }
    
    sep = get_emm_option("sep")
    
    # Set up trace vars and key
    tvars = specs$lhs
    if (one.trace <- (length(tvars) == 0)) {
        tlab = ""
        tvars = ".single."
        emms$.single. = 1
    }
    else 
        tlab = paste(tvars, collapse = sep)
    tv = do.call(paste, c(unname(emms[tvars]), sep = sep))
    emms$tvar = factor(tv, levels=unique(tv))
    
    xvars = specs$rhs
    xv = do.call(paste, c(unname(emms[xvars]), sep = sep))
    ltest = max(apply(table(xv,tv), 2, function(x) sum(x > 0))) # length of longest trace
    if (!missing(style))
        styl = match.arg(style, c("factor", "numeric"))
    if (missing(style) || styl == "numeric")
        styl = ifelse(length(xvars) == 1 && 
                           is.numeric(emms[[xvars]]) &&
                           ltest > 1,
                   "numeric", "factor")
    if (styl == "factor") {
        emms$xvar = factor(xv, levels = unique(xv))
        predicate = "Levels of "
        if (ltest <= 1)
            message("Suggestion: Add 'at = list(", xvars, " = ...)' ",
                    "to call to see > 1 value per group.")
    }
    else {
        emms$xvar = as.numeric(xv)
        predicate = ""
    }
    emms = emms[order(emms$xvar), ]

    byvars = specs$by
    xlab = ifelse(is.null(xargs$xlab),
                  paste0(predicate, paste(xvars, collapse = sep)), xargs$xlab)
    rspLbl = paste("Predicted", 
                   ifelse(is.null(emmo@misc$inv.lbl), "response", emmo@misc$inv.lbl))
    ylab = ifelse(is.null(xargs$ylab),
                  ifelse(type == "response", rspLbl, "Linear prediction"),
                  xargs$ylab)
    
    # remove the unneeded stuff from xlabs
    xargs = xargs[setdiff(names(xargs), c("xlab","ylab"))]
    
    ### any abbreviations?
    if((abbr.len[1] >= 1) && is.factor(emms$xvar))
        levels(emms$xvar) = abbreviate(levels(emms$xvar), abbr.len[1])
    if((length(abbr.len) > 1) && (abbr.len[2] >= 1) && is.factor(emms$tvar)) {
        levels(emms$tvar) = abbreviate(levels(emms$tvar), abbr.len[2])
        if(!is.null(emms[[tlab]]))
            levels(emms[[tlab]]) = abbreviate(levels(emms[[tlab]]), abbr.len[2])
    }
    emms$.single. = NULL   # in case we have that trick column
    attr(emms, "labs") = list(xlab = xlab, ylab = ylab, tlab = tlab)
    attr(emms, "vars") = list(byvars = byvars, tvars = setdiff(tvars, ".single."))
    if (!plotit && nonlin.scale)
        attr(emms, "scale") = .make.scale(emmo@misc)
    
    if (!plotit || engine == "none")
        return (emms)
    
    fcn = paste("emmip", engine, sep = "_")
    args = c(list(emms = emms, style = styl), xargs)
    if (nonlin.scale)
        args = c(args, list(scale = .make.scale(emmo@misc)))
    do.call(fcn, args)
}

### render emmip using ggplot
#' @rdname emmip
#' @param dodge Numerical amount passed to \code{ggplot2::position_dodge} 
#'   by which points and intervals are offset so they do not collide.
#' @param xlab,ylab,tlab Character labels for the horizontal axis, vertical
#'   axis, and traces (the different curves), respectively. The \code{emmip}
#'   function generates these automatically and provides therm via the \code{labs} 
#'   attribute, but the user may override these if desired.
#' @param facetlab Labeller for facets (when by variables are in play). Use
#'   \code{"label_value"} to show just the factor levels, or \code{"label_both"}
#'   to show both the factor names and factor levels; \code{"label_context"}
#'   decides which based on how many \code{by} factors there are. See the
#'   documentation for \code{ggplot2::labellers}.
#' @param scale If not missing, an object of class \code{scales::trans} specifying
#'   a (usually) nonlinear scaling for the vertical axis. For example, 
#'   \code{scales = scales::log_trans()} specifies a logarithmic scale. For
#'   fine-tuning purposes, additional
#'   arguments to \code{ggplot2::scale_y_continuous} may be included in \code{...} .
#' @param dotarg \code{list}
#'   of arguments passed to \code{geom_point} to customize appearance of points.
#'   If this includes \code{shape} and it is of length greater than 1, it is interpreted
#'   as a shape palette instead.
#' @param linearg \code{list}
#'   of arguments passed to \code{geom_line} to customize appearance of lines.
#'   If this includes \code{linetype} and it is of length greater than 1,
#'   it is interpreted as a linetype.
#' @param CIarg,PIarg \code{list}s
#'   of arguments passed to \code{geom_linerange} to customize appearance of intervals.
#'   (Note: the \code{linetype} aesthetic defaults to \code{"solid"} under the hood)
#'   
#' @section Rendering functions:
#' The functions \code{emmip_ggplot} and \code{emmip_lattice}
#' are called when \code{plotit == TRUE} to render the plots; 
#' but they may also be called later on an object saved via \code{plotit = FALSE}
#' (or \code{engine = "none"}). The functions require that \code{emms} contains variables
#' \code{xvar}, \code{yvar}, and \code{tvar}, and attributes \code{"labs"} and \code{"vars"}.
#' Confidence intervals are plotted if variables \code{LCL} and \code{UCL} exist;
#' and prediction intervals are plotted if \code{LPL} and \code{UPL} exist.
#' Finally, it must contain the variables named in \code{attr(emms, "vars")}.
#' 
#' In \code{emmip_ggplot}, colors, linetypes, and shapes are all assigned to
#' groups (according to \code{tvar}) unless overridden. So, for example, one may 
#' have different symbols for each group by simply specifying \code{dotarg = list()}.
#' 
#' 
#' @examples
#' 
#'### Options with transformations or link functions
#' neuralgia.glm <- glm(Pain ~ Treatment * Sex + Age, family = binomial(), 
#'                      data = neuralgia) 
#' 
#' # On link scale:
#' emmip(neuralgia.glm, Treatment ~ Sex)
#' 
#' # On response scale:
#' emmip(neuralgia.glm, Treatment ~ Sex, type = "response")
#' 
#' # With transformed axis scale and custom scale divisions
#' emmip(neuralgia.glm, Treatment ~ Sex, type = "scale",
#'     breaks = seq(0.10, 0.90, by = 0.10))
#' @export
#' @importFrom rlang .data
emmip_ggplot = function(emms, style = "factor", dodge = .2,
                        xlab = labs$xlab, ylab = labs$ylab, tlab = labs$tlab,
                        facetlab = "label_both",
                        scale, 
                        dotarg = list(shape = "circle"), 
                        linearg = list(linetype = "solid"),
                        CIarg = list(alpha = .40),
                        PIarg = list(alpha = .25),
                        col = NULL,
                        ...) {
    
    thm = theme_emm()    # Depends on gg.theme option
    
    labs = attr(emms, "labs")
    vars = attr(emms, "vars")
    ngrps = ifelse(is.null(emms$tvar), 1, length(unique(emms$tvar)))
    
    shape.pal = linetype.pal = NULL   # we use these to store shape and linetype specs in dotarg and linearg
    
    CIs = !is.null(emms$LCL)
    PIs = !is.null(emms$LPL)
    pos = ggplot2::position_dodge(width = ifelse(CIs|PIs, dodge, 0)) # use dodging if CIs
    
    if(missing(scale) && !is.null(attr(emms, "scale")))
        scale = attr(emms, "scale")

    dotarg$position = pos
    if(is.null(dotarg$size)) 
        dotarg$size = 3
    
    if (ngrps > 1) {
        if(!is.null(dotarg$shape) && length(dotarg$shape) > 1) {  # treat as a shape palette
            shape.pal = dotarg$shape
            dotarg$shape = NULL
        }
        
        linearg$mapping = ggplot2::aes(x = .data$xvar, linetype = .data[[tlab]], group = .data[[tlab]])
        dotarg$mapping = ggplot2::aes(shape = .data[[tlab]])
        
        linearg$position = pos
        if(is.null(linearg$linewidth))
            linearg$linewidth = 0.8
        if(!is.null(linearg$linetype) && length(linearg$linetype) > 1) {  # treat as a linetype palette
            linetype.pal = linearg$linetype
            linearg$linetype = NULL
        }
        
        emms[[tlab]] = emms$tvar   # make the trace column have same name as its label
        
        grobj = ggplot2::ggplot(emms, ggplot2::aes(x = .data$xvar, y = .data$yvar, group = .data[[tlab]], 
                    color = .data[[tlab]]))
        
        if (style == "factor") {
            grobj = grobj + do.call(ggplot2::geom_point, dotarg) +
                ggplot2::scale_x_discrete(expand = ggplot2::expansion(mult = 0.1))
        }
        grobj = grobj +
                do.call(ggplot2::geom_line, linearg)
        
        # handle any custom shapes or linetypes
        if(!is.null(shape.pal))
            grobj = grobj + ggplot2::scale_shape_manual(values = shape.pal)
        if(!is.null(linetype.pal))
            grobj = grobj + ggplot2::scale_linetype_manual(values = linetype.pal)
    }
    else { # just one trace per plot
        if(is.null(col))
            col = .emm_palette[1]
        linearg$color = dotarg$color = CIarg$color = PIarg$color = col
        col = NULL
        grobj = ggplot2::ggplot(emms, ggplot2::aes(x = .data$xvar, y = .data$yvar))
        if (style == "factor")
            grobj = grobj + do.call(ggplot2::geom_point, dotarg) + 
                ggplot2::scale_x_discrete(expand = ggplot2::expansion(mult = 0.1))
        linearg$mapping = ggplot2::aes(x = .data$xvar, group = .data$tvar)
        grobj = grobj +
            do.call(ggplot2::geom_line, linearg)
        
    }
    grobj = grobj + ggplot2::labs(x = xlab, y = ylab)
    if (PIs) {
        PIarg$mapping = ggplot2::aes(ymin = .data$LPL, ymax = .data$UPL)
        PIarg$position = pos
        if(is.null(PIarg$linetype))  PIarg$linetype = "solid"
        if(is.null(PIarg$linewidth))  PIarg$linewidth = 1.5
        grobj = grobj + do.call(ggplot2::geom_linerange, PIarg)
    }
    if (CIs) {
        CIarg$mapping = ggplot2::aes(ymin = .data$LCL, ymax = .data$UCL)
        CIarg$position = pos
        if(is.null(CIarg$linetype))  CIarg$linetype = "solid"
        if(is.null(CIarg$linewidth))  CIarg$linewidth = 2.5
        grobj = grobj + do.call(ggplot2::geom_linerange, CIarg)
    }
    if (length(byvars <- vars$byvars) > 0) {  # we have by variables 
        if (length(byvars) > 1) {
            byform = as.formula(paste(byvars[1], " ~ ", paste(byvars[-1], collapse="*")))
            grobj = grobj + ggplot2::facet_grid(byform, labeller = facetlab)
        }
        else
            grobj = grobj + ggplot2::facet_wrap(byvars, labeller = facetlab)
    }
    if (!missing(scale)) {
        args = list(...)
        pass = names(args) %.pin% names(as.list(args(ggplot2::scale_y_continuous)))
        args = c(list(trans = scale), args[pass])
        grobj = grobj + do.call(ggplot2::scale_y_continuous, args)
    }
    if (style == "factor")
        grobj = grobj + do.call(ggplot2::geom_point, dotarg)
    
    if(!is.null(col))
        grobj = grobj + ggplot2::scale_color_manual(values = rep(col, ngrps))
    
    
    grobj + thm
}

#' @importFrom grDevices palette.colors
##### Here is a palette of 45 colors. More than that, SOL
.emm_palette = c(
    grDevices::palette.colors(palette = "Okabe-Ito)")[c(6:9,1:5)], # Start w/ steel blue, yellow is last
    grDevices::palette.colors(palette = "Polychrome 36")[sample(1:36)]
)

theme_emm = function (base_size = 13, base_family = "sans", header_family = "sans", 
                      base_line_size = base_size/22, base_rect_size = base_size/22, 
                      ink ="#0e0033ff", paper = "white", accent = "#FF4000") 
{
    if(inherits(thm <- get_emm_option("gg.theme"), "theme"))
        return(thm)
    
    if(thm == 1) # legacy theme from version 1.x.x
        return(ggplot2::theme_grey())
    
    dark_color = "#4E4369"
    mid_color =  "#79718D"   ###   "#002789"
    lit_color =  "#d2d2d8ff"
  
    ggplot2::theme_light(base_size = base_size, base_family = base_family,
            header_family = header_family, base_line_size = base_line_size,
            base_rect_size = base_rect_size, ink = ink, paper = paper, accent = accent) +
      ggplot2::theme(panel.grid.minor = ggplot2::element_blank(),
                    panel.border = ggplot2::element_rect(color = mid_color, linewidth = 0.85),
                    panel.grid.major = ggplot2::element_line(colour = lit_color),
                    strip.background = ggplot2::element_rect(fill = mid_color, color = mid_color),
                    strip.text = ggplot2::element_text(size = ggplot2::rel(0.8), family = "sans", face = "bold", lineheight = 1.1),
                    plot.title = ggplot2::element_text(colour = ink, family = "serif", size = ggplot2::rel(1.8), margin = ggplot2::margin(12, 0, 8, 0)),
                    plot.subtitle = ggplot2::element_text(size = ggplot2::rel(1.1), family = "serif", margin = ggplot2::margin(4, 0, 0, 0), color = dark_color),
                    legend.justification = "top",
                    palette.colour.discrete = .emm_palette # color-blind friendly, 45 total colors
    )
}


#' @rdname emmip
#' @param emms A \code{data.frame} created by calling \code{emmip} with
#'   \code{plotit = FALSE}. Certain variables and attributes are expected
#'   to exist in this data frame; see the section detailing the rendering functions.
#' @param pch (Lattice only) The plotting characters to use for each group (i.e., levels of
#'   \code{trace.factors}). They are recycled as needed.
#' @param lty (Lattice only) The line types to use for each group. Recycled as needed.
#' @param col With \code{emmip_ggplot}, this adds \code{color = col} (not
#'   \code{colour}) to all of the \code{*arg} lists. This is intended for setting a
#'   common color for everything, such as a black-and-white plot. 
#'   With \code{emmip_lattice}, \code{col} specifies the colors to use
#'   for each group, recycled as needed. If not specified, the default trellis
#'   colors are used.
#' @export
emmip_lattice = function(emms, style = "factor", 
                         xlab = labs$xlab, ylab = labs$ylab, tlab = labs$tlab, 
                         pch = c(1,2,6,7,9,10,15:20), 
                         lty = 1, col = NULL, ...) {
    labs = attr(emm, "labs")
    vars = attr(emms, "vars")
    
    # The strips the way I want them
    my.strip = lattice::strip.custom(strip.names = c(TRUE,TRUE), strip.levels = c(TRUE,TRUE), sep = " = ")
    
    if (length(vars$byvars) == 0)
        plotform = yvar ~ xvar
    else
        plotform = as.formula(paste("yvar ~ xvar |", paste(vars$byvars, collapse="*")))
    
    sep = get_emm_option("sep")
    my.key = function(tvars) 
        list(space="right", 
             title = paste(tvars, collapse = sep), 
             points = TRUE, 
             lines=length(lty) > 1,
             cex.title=1)
    
    TP = TP.orig = lattice::trellis.par.get()
    TP$superpose.symbol$pch = pch
    TP$superpose.line$lty = lty
    if (!is.null(col)) TP$superpose.symbol$col = TP$superpose.line$col = col
    lattice::trellis.par.set(TP)
    
    plty = if(style=="factor") c("p","l")   else "l"
    plotspecs = list(x = plotform, data = emms, groups = ~ tvar, 
                     xlab = xlab, ylab = ylab,
                     strip = my.strip, auto.key = my.key(vars$tvars), 
                     type = plty)
    if(length(vars$tvars) == 0)
        plotspecs$auto.key = NULL # no key when single trace
    grobj = do.call(lattice::xyplot, c(plotspecs, list(...)))
    lattice::trellis.par.set(TP.orig)
    
    grobj
}

##############################################################################
#    Copyright (c) 2012-2025 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

#' Estimated marginal means (aka Least-squares means)
#' 
#' This package provides methods for obtaining estimated marginal means (EMMs, also 
#' known as least-squares means) for factor combinations in a variety of models.
#' Supported models include [generalized linear] models, models for counts,
#' multivariate, multinomial and ordinal responses, survival models, GEEs, and
#' Bayesian models. For the latter, posterior samples of EMMs are provided.
#' The package can compute contrasts or linear
#' combinations of these marginal means with various multiplicity adjustments.
#' One can also estimate and contrast slopes of trend lines.
#' Some graphical displays of these results are provided.
#' 
#' @section Overview:
#' \describe{
#' \item{Vignettes}{A number of vignettes are provided to help the user get
#' acquainted with the \pkg{emmeans} package and see some examples.}
#' 
#' \item{Concept}{Estimated marginal means (see Searle \emph{et al.} 1980 are
#' popular for summarizing linear models that include factors. For balanced
#' experimental designs, they are just the marginal means. For unbalanced data,
#' they in essence estimate the marginal means you \emph{would} have observed
#' that the data arisen from a balanced experiment. Earlier developments
#' regarding these techniques were developed in a least-squares context and are
#' sometimes referred to as \dQuote{least-squares means}. Since its early
#' development, the concept has expanded far beyond least-squares settings.}
#' 
#' \item{Reference grids}{ The implementation in \pkg{emmeans} relies on our own
#' concept of a \emph{reference grid}, which is an array of factor and predictor
#' levels. Predictions are made on this grid, and estimated marginal means (or
#' EMMs) are defined as averages of these predictions over zero or more
#' dimensions of the grid. The function \code{\link{ref_grid}} explicitly
#' creates a reference grid that can subsequently be used to obtain
#' least-squares means. The object returned by \code{ref_grid} is of class
#' \code{"emmGrid"}, the same class as is used for estimated marginal means (see
#' below).
#' 
#' Our reference-grid framework expands slightly upon Searle \emph{et al.}'s
#' definitions of EMMs, in that it is possible to include multiple levels of
#' covariates in the grid. }
#' 
#' \item{Models supported}{As is mentioned in the package description, many
#' types of models are supported by the package. 
#' See \href{../doc/models.html}{vignette("models", "emmeans")} for full details. 
#' Some models may require other packages be
#' installed in order to  access all of the available features.
#' For models not explicitly supported, it may still be possible to do basic
#' post hoc analyses of them via the \code{\link{qdrg}} function.}
#' 
#' \item{Estimated marginal means}{
#' The \code{\link{emmeans}} function computes EMMs given a fitted model (or a
#' previously constructed \code{emmGrid} object), using a specification indicating
#' what factors to include. The \code{\link{emtrends}} function creates the same
#' sort of results for estimating and comparing slopes of fitted lines. Both
#' return an \code{emmGrid} object.}
#' 
#' \item{Summaries and analysis}{
#' The \code{\link{summary.emmGrid}}  method may be used to display an \code{emmGrid}
#' object. Special-purpose summaries are available via \code{\link{confint.emmGrid}} and
#' \code{\link{test.emmGrid}}, the latter of which can also do a joint test of several
#' estimates. The user may specify by variables, multiplicity-adjustment
#' methods, confidence levels, etc., and if a transformation or link function is
#' involved, may reverse-transform the results to the response scale.}
#' 
#' \item{Contrasts and comparisons}{
#' The \code{\link{contrast}} method for \code{emmGrid} objects is used to obtain
#' contrasts among the estimates; several standard contrast families are
#' available such as deviations from the mean, polynomial contrasts, and
#' comparisons with one or more controls. Another \code{emmGrid} object is returned,
#' which can be summarized or further analyzed. For convenience, a \code{pairs.emmGrid}
#' method is provided for the case of pairwise comparisons. 
#' }
#' \item{Graphs}{The \code{\link{plot.emmGrid}} method will display
#' side-by-side confidence intervals for the estimates, and/or
#' \dQuote{comparison arrows} whereby the *P* values of pairwise differences
#' can be observed by how much the arrows overlap. The \code{\link{emmip}} function
#' displays estimates like an interaction plot, multi-paneled if there are by
#' variables. These graphics capabilities require the \pkg{lattice} package be
#' installed.}
#' 
#' \item{MCMC support}{When a model is fitted using MCMC methods, the posterior
#' chains(s) of parameter estimates are retained and converted into posterior
#' samples of EMMs or contrasts thereof. These may then be summarized or plotted
#' like any other MCMC results, using tools in, say \pkg{coda} or
#' \pkg{bayesplot}.}
#' 
#' \item{\pkg{multcomp} interface}{The \code{\link{as.glht}} function and
#' \code{glht} method for \code{emmGrid}s provide an interface to the
#' \code{glht} function in the \pkg{multcomp} package, thus
#' providing for more exacting simultaneous estimation or testing. The package
#' also provides an \code{\link{emm}} function that works as an alternative to
#' \code{mcp} in a call to \code{glht}.
#' }
#' } %%% end describe
#' 
#' @import estimability
#' @import mvtnorm
#' @import stats
#' @importFrom graphics pairs plot
#' @importFrom methods as is new show slot slot<- slotNames 
#' @importFrom utils getS3method hasName installed.packages methods str
#' @name emmeans-package
NULL

##############################################################################
#    Copyright (c) 2012-2017 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

### xtable method
# Modified from xtableLSMeans function provided by David Scott

#' Using \code{xtable} for EMMs
#' 
#' These methods provide support for the \pkg{xtable} package, enabling 
#' polished presentations of tabular output from \code{\link{emmeans}}
#' and other functions.
#' 
#' The methods actually use \code{\link[xtable]{xtableList}},
#' because of its ability to display messages such as those for P-value
#' adjustments. These methods return an object of class \code{"xtable_emm"} --
#' an extension of \code{"xtableList"}. Unlike other \code{xtable} methods, the
#' number of digits defaults to 4; and degrees of freedom and \emph{t} ratios
#' are always formatted independently of \code{digits}. The \code{print} method
#' uses \code{\link[xtable:xtableList]{print.xtableList}}, and any \code{\dots} arguments are
#' passed there.
#'
#' @param x Object of class \code{emmGrid}
#' @param caption Passed to \code{\link[xtable]{xtableList}}
#' @param label Passed to \code{xtableList}
#' @param align Passed to \code{xtableList}
#' @param digits Passed to \code{xtableList}
#' @param display Passed to \code{xtableList}
#' @param auto Passed to \code{xtableList}
#' @param ... Arguments passed to \code{\link{summary.emmGrid}}
#'
#' @return The \code{xtable} methods return an \code{xtable_emm}
#' object, for which its print method is \code{print.xtable_emm} .
#' 
#' @method xtable emmGrid
#' @examples
#' if(requireNamespace("xtable"))
#'     emm_example("xtable")
#'     # Use emm_example("xtable", list = TRUE) # to just list the code
xtable.emmGrid = function(x, caption = NULL, label = NULL, align = NULL, digits = 4, 
    display = NULL, auto = FALSE, ...) 
{
    xtable.summary_emm(summary(x, ...), caption = caption, label = label, align = align, digits = digits, 
           display = display, auto = auto)
}

#' @rdname xtable.emmGrid
#' @method xtable summary_emm
xtable.summary_emm = function (x, caption = NULL, label = NULL, 
          align = NULL, digits = 4, 
          display = NULL, auto = FALSE, ...) 
{
    if (!is.null(x$df)) x$df = round(x$df, 2)
    if (!is.null(x$t.ratio)) x$t.ratio = round(x$t.ratio, 3)
    if (!is.null(x$z.ratio)) x$z.ratio = round(x$z.ratio, 3)
    if (!is.null(x$p.value)) {
        fp = x$p.value = format(round(x$p.value,4), nsmall=4, sci=FALSE)
        x$p.value[fp=="0.0000"] = "<.0001"
    }
    if (!is.null(byv <- attr(x, "by.vars"))) {
        byc = which(names(x) %in% byv)
        xList = split(as.data.frame(x), f = x[, byc])
        labs = rep("", length(xList))
        for (i in 1:length(xList)) {
            levs = sapply(xList[[i]][1, byc], as.character)
            labs[i] = paste(paste(byv, levs, sep = " = "), collapse = ", ")
            xList[[i]] = as.data.frame(xList[[i]][, -byc, drop = FALSE])
        }
        attr(xList, "subheadings") = labs
    }
    else {
        xList = list(data.frame(x))
    }
    attr(xList, "message") = attr(x, "mesg")
    result = xtable::xtableList(xList, caption = caption, label = label, 
       align = align, digits = digits, display = display, 
       auto = auto, ...)
    digits = xtable::digits(result[[1]])
    
    # format df and t ratios
    digits = xtable::digits(result[[1]])
    i = which(names(x) == "df")
    if (length(i) > 0) {
        dfd = ifelse(all(zapsmall(x$df - round(x$df)) == 0), 0, 2)
        digits[i + 1 - length(byv)] = ifelse(is.na(dfd), 0, dfd)
    }
    i = which(names(x) %in% c("t.ratio", "z.ratio"))
    if (length(i) > 0) digits[i + 1 - length(byv)] = 3
    for (i in seq_along(result))
        xtable::digits(result[[i]]) = digits
    
    class(result) = c("xtable_emm", "xtableList")
    result
}

#' @rdname xtable.emmGrid
#' @param type Passed to \code{\link[xtable]{print.xtable}}
#' @param include.rownames Passed to \code{print.xtable}
#' @param sanitize.message.function Passed to \code{print.xtable}
#' @method print xtable_emm
print.xtable_emm = function(x, type = getOption("xtable.type", "latex"),
                            include.rownames = FALSE, 
                            sanitize.message.function = footnotesize,
                            ...)
{
    footnotesize = switch(type,
        html = function(x) paste0("<font size = -1>", x, "</font>"),
        latex = function(x) paste0("{\\footnotesize ", x, "}"),
        function(x) x )
    invisible(xtable::print.xtableList(x, type = type, include.rownames = include.rownames, 
        sanitize.message.function = sanitize.message.function, ...))
}

##############################################################################
#    Copyright (c) 2012-2017 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

# Support for zeroinfl and hurdle models (pscl)

# We'll support two optional arguments:
# mode     -- type of result required
# lin.pred -- TRUE:  keep linear predictor and link
#             FALSE: back-transform (default)
#
# With lin.pred = FALSE and mode %in% c("response", "count", "zero"), we
# will return comparable results to predict(..., type = mode)
# with mode = "prob0", same results as predict(..., type = "prob")[, 1]
#
# lin.pred only affects results for mode %in% c("count", "zero"). 
# When lin.pred = TRUE, we get the actual linear predictor and link function
# for that part of the model.



# ----- zeroinfl objects -----

#' @exportS3Method recover_data zeroinfl
recover_data.zeroinfl = function(object, mode = c("response", "count", "zero", "prob0"), ...) {
    fcall = object$call
    mode = match.arg(mode)
    if (mode %in% c("count", "zero"))
        trms = delete.response(terms(object, model = mode))
    else ### mode = %in% c("response", "prob0")
        trms = delete.response(object$terms$full)
    # Make sure there's an offset function available
    env = new.env(parent = attr(trms, ".Environment"))
    env$offset = function(x) x
    attr(trms, ".Environment") = env
    recover_data(fcall, trms, object$na.action, ...)
}


#' @exportS3Method emm_basis zeroinfl     
emm_basis.zeroinfl = function(object, trms, xlev, grid, 
        mode = c("response", "count", "zero", "prob0"), lin.pred = FALSE, ...) 
{
    mode = match.arg(mode)
    m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
    if (mode %in% c("count", "zero")) {
        contr = object$contrasts[[mode]]
        X = model.matrix(trms, m, contrasts.arg = contr)
        bhat = coef(object, model = mode)
        V = .pscl.vcov(object, model = mode, ...)
        if (mode == "count")
            misc = list(tran = "log", inv.lbl = "count")
        else
            misc = list(tran = object$link, inv.lbl = "prob")
        if (!lin.pred) { # back-transform the results
            lp = as.numeric(X %*% bhat + .get.offset(trms, grid))
            lnk = make.link(misc$tran)
            bhat = lnk$linkinv(lp)
#            delta = .diag(lnk$mu.eta(lp)) %*% X
            delta = sweep(X, 1, lnk$mu.eta(lp), "*")
            V = delta %*% tcrossprod(V, delta)
            X = diag(1, length(bhat))
            misc = list(offset.mult = 0)
        }
    }
    else { ## "response", "prob0"
        trms1 = delete.response(terms(object, model = "count"))
        off1 = .get.offset(trms1, grid)
        contr1 = object$contrasts[["count"]]
        X1 = model.matrix(trms1, m, contrasts.arg = contr1)
        b1 = coef(object, model = "count")
        lp1 = as.numeric(X1 %*% b1 + off1)
        mu1 = exp(lp1)
        
        trms2 = delete.response(terms(object, model = "zero"))
        off2 = .get.offset(trms2, grid)
        contr2 = object$contrasts[["zero"]]
        X2 = model.matrix(trms2, m, contrasts.arg = contr2)
        b2 = coef(object, model = "zero")
        lp2 = as.numeric(X2 %*% b2) + off2
        mu2 = object$linkinv(lp2)
        mu2prime = stats::make.link(object$link)$mu.eta(lp2)
        
        if(mode == "response") {
            delta = cbind(sweep(X1, 1, mu1 * (1 - mu2), "*"), 
                          sweep(X2, 1, -mu1 * mu2prime, "*"))
            bhat = (1 - mu2) * mu1
        }
        else { # mode = "prob0"
            tmp = .zi.support(mu1, object$theta, .make.p0(object$dist))
            p0 = tmp[1, ]; dp0 = tmp[2, ]
            bhat = (1 - mu2) * p0 + mu2
            delta = cbind(sweep(X1, 1, (1 - mu2) * dp0, "*"),
                          sweep(X2, 1, mu2prime * (1 - p0), "*"))
        }
        V = delta %*% tcrossprod(.pscl.vcov(object, model = "full", ...), delta)
        X = diag(1, length(bhat))
        misc = list(offset.mult = 0)
    }
    nbasis = estimability::all.estble
    dffun = function(k, dfargs) Inf
    dfargs = list()
    list(X = X, bhat = bhat, nbasis = nbasis, V = V, 
         dffun = dffun, dfargs = dfargs, misc = misc)
}



#### Support for hurdle models

#' @exportS3Method recover_data hurdle
recover_data.hurdle = function(object, mode = c("response", "count", "zero", "prob0"), ...) {
    fcall = object$call
    mode = match.arg(mode)
    if (mode %in% c("count", "zero"))
        trms = delete.response(terms(object, model = mode))
    else ### mode = "mean" or "prob.ratio"
        trms = delete.response(object$terms$full)
    # Make sure there's an offset function available
    env = new.env(parent = attr(trms, ".Environment"))
    env$offset = function(x) x
    attr(trms, ".Environment") = env
    recover_data(fcall, trms, object$na.action, ...)
}

# see expl notes afterward for notations in some of this
#' @exportS3Method emm_basis hurdle       
emm_basis.hurdle = function(object, trms, xlev, grid, 
                            mode = c("response", "count", "zero", "prob0"), 
                            lin.pred = FALSE, ...) 
{
    mode = match.arg(mode)
    m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
    if ((lin.pred && mode %in% c("count", "zero")) || (!lin.pred && mode %in% c("count", "prob0"))) {
        model = ifelse(mode == "count", "count", "zero")
        contr = object$contrasts[[model]]
        X = model.matrix(trms, m, contrasts.arg = contr)
        bhat = coef(object, model = model)
        V = .pscl.vcov(object, model = model, ...)
        misc = switch(object$dist[[model]],
                        binomial = list(tran = object$link, inv.lbl = "prob"),
                        list(tran = "log", inv.lbl = "count"))
        if (!lin.pred) { # back-transform
            lp = as.numeric(X %*% bhat + .get.offset(trms, grid))
            lnk = make.link(misc$tran)
            bhat = lnk$linkinv(lp)
            mult = lnk$mu.eta(lp)
            if(mode == "prob0") {
                shape = 
                tmp = .zi.support(bhat, object$theta["zero"], .make.p0(object$dist$zero))
                bhat = tmp[1, ]
                mult = mult * tmp[2, ]
            }
            delta = sweep(X, 1, mult, "*")
            V = delta %*% tcrossprod(V, delta)
            X = diag(1, length(bhat))
            misc = list(offset.mult = 0)
        }
    }
    else {   ### "zero" or "response" with implied lin.pred = FALSE
        trms1 = delete.response(terms(object, model = "count"))
        off1 = .get.offset(trms1, grid)
        contr1 = object$contrasts[["count"]]
        X1 = model.matrix(trms1, m, contrasts.arg = contr1)
        b1 = coef(object, model = "count")
        mu1 = mu1prime = as.numeric(exp(X1 %*% b1 + off1))
        theta1 = object$theta["count"]

        trms2 = delete.response(terms(object, model = "zero"))
        off2 = .get.offset(trms2, grid)
        contr2 = object$contrasts[["zero"]]
        X2 = model.matrix(trms2, m, contrasts.arg = contr2)
        b2 = coef(object, model = "zero")
        lp2 = as.numeric(X2 %*% b2 + off2)
        mu2 = switch(object$dist$zero, 
                     binomial = object$linkinv(lp2),
                     exp(lp2)  )
        theta2 = object$theta["zero"]
        mu2prime = switch(object$dist$zero, 
                          binomial = stats::make.link("logit")$mu.eta(lp2),
                          exp(lp2) )
        if (mode == "response") {
            tmp = .hurdle.support(mu1, theta1, .make.p0(object$dist$count), \(mu, shape) mu, 
                                  mu2, theta2, .make.p0(object$dist$zero))
            bhat = tmp[1, ]
            delta = cbind(sweep(X1, 1, tmp[2, ] * mu1prime, "*"),
                          sweep(X2, 1, tmp[3, ] * mu2prime, "*"))
        }
        else {  ## mode == "zero"
            tmp1 = .zi.support(mu1, theta1, .make.p0(object$dist$count))
            tmp2 = .zi.support(mu2, theta2, .make.p0(object$dist$zero))
            p1 = 1 - tmp1[1, ]; dp1 = -tmp1[2, ]
            p2 = 1 - tmp2[1, ]; dp2 = -tmp2[2, ]
            bhat = p2 / p1
            # delta = cbind(.diag(-p2 * dp1 / p1^2) %*% X1,
            #               .diag(dp2 / p1) %*% X2)
            delta = cbind(sweep(X1, 1, -p2 * dp1, "*"),
                          sweep(X2, 1, dp2 / p1, "*"))
        }
        V = delta %*% tcrossprod(.pscl.vcov(object, model = "full", ...), delta)
        X = .diag(1, length(bhat))
        
        misc = list(estName = mode, offset.mult = 0)
    }
    nbasis = estimability::all.estble
    dffun = function(k, dfargs) dfargs$df
    dfargs = list(df = object$df.residual)
    list(X = X, bhat = bhat, nbasis = nbasis, V = V, 
         dffun = dffun, dfargs = dfargs, misc = misc)
}


#' @rdname extending-emmeans
#' @order 93
#'
#' @param cmu,zmu In \code{.hurdle.support} and \code{.zi.support}, 
#'   these specify a vector of back-transformed 
#'   estimates for the count and zero model, respectively
#' @param cshape,zshape Shape parameter for the count and zero model, respectively 
#' @param cp0,zp0 Function of \code{(mu, shape)} for computing Prob(Y = 0)
#'        for the count and zero model, respectively
#' @param cmean Function of \code{(mu, shape)} for computing the mean of the
#'        count model. Typically, this just returns \code{mu}
#'
#' @return \code{.hurdle.support} returns a matrix with 3 rows containing the
#'       estimated mean responses and the differentials wrt \code{cmu} and \code{zmu},
#'       resp. 
#' @export
.hurdle.support = function(cmu, cshape, cp0, cmean, 
                           zmu, zshape, zp0) {
    if (is.null(cshape) || is.na(cshape)) cshape = 1
    if (is.null(zshape) || is.na(zshape)) zshape = 1
    mfcn = function(x) (1 - zp0(x[2], zshape)) * cmean(x[1], cshape) / 
        (1 - cp0(x[1], cshape))
    result = sapply(seq_along(cmu), function(i) {
        x = c(cmu[i], zmu[i])
        d = numDeriv::jacobian(mfcn, x)
        c(mfcn(x), d)
    })
    rownames(result) = c("mean", "dcmu", "dzmu")
    result
}

#' @rdname extending-emmeans
#' @order 94
#' @return \code{.zi.support} returns a matrix with 2 rows containing the
#'       estimated probabilities of 0 and the differentials wrt \code{mu}.
#'       See the section on hurdle and zero-inflated models.
#' @section Support for Hurdle and Zero-inflated models:
#'   The functions \code{.hurdle.support} and \code{.zi.support} help facilitate
#'   calculations needed to estimate the mean response (count model and zero model
#'   combined) of these models. \code{.hurdle.support} returns a matrix of three rows.
#'   The first is the estimated mean for a hurdle model, and the 2nd and 3rd rows are
#'   differentials for the count and zero models, which needed for delta-method
#'   calculations. To use these, regard the \code{@linfct} slot as comprising
#'   two sets of columns, for the count and zero models respectively. To do
#'   the delta method calculations, multiply the rows of the count part by its 
#'   differentials times \code{link$mu.eta} evcaluated at that part of the linear predictor.
#'   Do the same for the zero part, using its differentials and \code{mu.eta}.
#'   If the resulting matrix is \bold{A}, then the covariance of the mean response
#'   is \bold{AVA'} where \bold{V}is the \code{@V} slot of the object.
#'   
#'   The function \code{zi.support} works the same way, only it is much simpler,
#'   and is used to estimate the probability of 0 and its differential for either 
#'   part of a zero-inflated model or hurdle model.
#'   
#'   See the code for \code{emm_basis.zeroinfl} and \code{emm_basis.hurdle}
#'   for how these are used with models fitted by the \pkg{pscl} package.
#' @export
#'
.zi.support = function(zmu, zshape, zp0) {
    if (is.null(zshape) || is.na(zshape)) zshape = 1
    pfcn = function(x) zp0(x, zshape)
    result = sapply(seq_along(zmu), function(i) {
        c(pfcn(zmu[i]), numDeriv::jacobian(pfcn, zmu[i]))
    })
    rownames(result) = c("p0", "dmu")
    result
}


.make.p0 = function(dist) {
    switch(dist,
           bernoulli = \(mu, shape) 1 - mu,
           binomial = \(mu, shape) (1 - mu)^shape,
           poisson = \(mu, shape) exp(-mu),
           negbin = \(mu, shape) (shape / (mu + shape))^shape,
           geometric = \(mu, shape) 1 / (1 + mu)
    )
}



# special version of .my.vcov that accepts (and requires!) model argument
.pscl.vcov = function(object, model, vcov. = stats::vcov, ...) {
    if (is.function(vcov.))
        vcov. = vcov.(object, model = model)
    else if (!is.matrix(vcov.))
        stop("vcov. must be a function or a square matrix")
    vcov.
}

##############################################################################
#    Copyright (c) 2012-2021 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

# experimental support for nls, nlme objects

#' @exportS3Method recover_data nls
recover_data.nls = function(object, ...) {
    fcall = object$call
    trms = terms(.reformulate(names(object$dataClasses)))
    recover_data(fcall, trms, object$na.action, ...)
}

#' @exportS3Method emm_basis nls          
emm_basis.nls = function(object, trms, xlev, grid, ...) {
    Vbeta = .my.vcov(object, ...)
    env = object$m$getEnv()
    for (nm in names(grid)) env[[nm]] = grid[[nm]]
    pars = object$m$getAllPars()
    DD = deriv(object$m$formula(), names(pars))
    ests = eval(DD, env)
    bhat = as.numeric(ests)
    grad = attr(ests, "gradient")
    V = grad %*% Vbeta %*% t(grad)
    X = diag(1, nrow(grid))
    list(X=X, bhat=bhat, nbasis=all.estble, V=V, 
         dffun=function(k, dfargs) Inf, dfargs=list(), 
         misc=list())
}
    

### For nlme objects, we can do stuff with the fixed part of the model
### Additional REQUIRED argument is 'param' - parameter name to explore
#' @exportS3Method recover_data nlme
recover_data.nlme = function(object, param, ...) {
    if(missing(param))
        return("'param' argument is required for nlme objects")
    fcall = object$call
    if (!is.null(fcall$weights))
        fcall$weights = nlme::varWeights(object$modelStruct)
    fixed = fcall$fixed
    if (is.call(fixed))
        fixed = eval(fixed, envir = parent.frame())
    if(!is.list(fixed))
        fixed = list(fixed)
    form = NULL
    for (x in fixed)
        if (param %in% all.names(x)) form = x
    if (is.null(form))
        return(paste("Can't find '", param, "' among the fixed parameters", sep = ""))
    fcall$weights = NULL
    trms = delete.response(terms(form))
    if (length(.all.vars(trms)) == 0)
        return(paste("No predictors for '", param, "' in fixed model", sep = ""))
    recover_data(fcall, trms, object$na.action, ...)
}

#' @exportS3Method emm_basis nlme         
emm_basis.nlme = function(object, trms, xlev, grid, param, ...) {
    idx = object$map$fmap[[param]]
    V = object$varFix[idx, idx, drop = FALSE]
    bhat = object$coefficients$fixed[idx]
    contr = attr(object$plist[[param]]$fixed, "contrasts")
    m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
    X = model.matrix(trms, m, contrasts.arg = contr)
    dfx = object$fixDF$X[idx]
    dfx[1] = min(dfx) # I'm assuming 1st one is intercept
    dffun = function(k, dfargs) { # containment df
        idx = which(abs(k) > 1e-6)
        ifelse(length(idx) > 0, min(dfargs$dfx[idx]), NA)
    }
    list(X = X, bhat = bhat, nbasis = estimability::all.estble, 
         V = V, dffun = dffun, dfargs = list(dfx = dfx), 
         misc = list(estName = param))
}


# Support for gnls - contributed by Fernando Miguez <femiguez@iastate.edu>
#' @exportS3Method recover_data gnls
recover_data.gnls = function(object, param, data, ...) {
    fcall = object$call$params
    if (is.null(fcall))
        return("Models fitted without a 'params' specification are not supported.")
    if(missing(param))
        return("'param' argument is required for gnls objects")

    plist = object$plist
    pnames = names(plist)
    params = eval(object$call$params)
    if (!is.list(params)) params = list(params)
    
    params = unlist(lapply(params, function(pp) {
        if (is.name(pp[[2]])){
            list(pp)
        }
        else { ## multiple parameters on left hand side
            eval(parse(text = paste("list(",
                                    paste(paste(all.vars(pp[[2]]), deparse(pp[[3]]), sep = "~"),
                                          collapse = ","),
                                    ")")))
        }
    }), recursive = FALSE)

    names(params) = pnames
    form = params[[param]]
    
    trms = delete.response(terms(eval(form, envir = environment(formula(object)))))
    if(is.null(data))
        data = eval(object$call$data, envir = environment(formula(object)))
    recover_data(fcall, trms, object$na.action, data = data, ...)
}

#' @exportS3Method emm_basis gnls         
emm_basis.gnls = function(object, trms, xlev, grid, param, ...) {
    idx = object$pmap[[param]]
    V = object$varBeta[idx, idx, drop = FALSE]
    bhat = object$coefficients[idx]
    contr = attr(object$plist[[param]], "contrasts")
    m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
    X = model.matrix(trms, m, contrasts.arg = contr)
    dfx = with(attributes(logLik(object)), nobs - df)
    dffun = function(k, dfargs) dfargs$dfx
    list(X = X, bhat = bhat, nbasis = estimability::all.estble, 
         V = V, dffun = dffun, dfargs = list(dfx = dfx), 
         misc = list(estName = param))
}

##############################################################################
#    Copyright (c) 2012-2016 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

# Code to implement transformations my way

# Implementation of additional transformations, typically ones with parameters
# Returns a list like stats::make.link


#' Response-transformation extensions
#' 
#' The \code{make.tran} function creates the needed information to perform
#' transformations of the response
#' variable, including inverting the transformation and estimating variances of
#' back-transformed predictions via the delta method. \code{make.tran} is
#' similar to \code{\link{make.link}}, but it covers additional transformations.
#' The result can be used as an environment in which the model is fitted, or as
#' the \code{tran} argument in \code{\link{update.emmGrid}} (when the given
#' transformation was already applied in an existing model).
#' 
#'
#' @param type The name of a standard transformation supported by \code{stat::make.link},
#' or of a special transformation described under Details.
#' @param alpha,beta Numeric parameters needed for special transformations.
#' @param param If non-missing, this specifies either
#'   \code{alpha} or \code{c(alpha, beta)} (provided for backward compatibility).
#'   Also, for the same reason, if \code{alpha} is of length more than 1,
#'   it is taken as \code{param}.
#' @param y A numeric response variable used (\emph{and required}) with \code{type = "scale"}, 
#'   where \code{scale(y)} determines \code{alpha} and \code{beta}.
#' @param inner another transformation. See the section on compound transformations
#' @param ... Additional arguments passed to other functions/methods
#'
#'
#' @section Details:
#' The \code{make.tran} function returns a
#' suitable list of functions for several popular transformations. Besides being
#' usable with \code{update}, the user may use this list as an enclosing
#' environment in fitting the model itself, in which case the transformation is
#' auto-detected when the special name \code{linkfun} (the transformation
#' itself) is used as the response transformation in the call. See the examples
#' below.
#' 
#' The primary purpose of \code{make.tran} is to support transformations that
#' require additional parameters, specified as \code{alpha} and \code{beta};
#' these are the onse shown in the argument-matching list. However, standard
#' transformations supported by \code{stats::make.link} are also supported.
#' In the following discussion of ones requiring parameters, 
#' we use \eqn{\alpha} and \eqn{\beta} to
#' denote \code{alpha} and \code{beta}, and \eqn{y} to denote the response variable.
#' The \code{type} argument specifies the following transformations:
#' \describe{
#' \item{\code{"genlog"}}{Generalized logarithmic transformation: \eqn{\log_\beta(y +
#'   \alpha)}, where \eqn{y > -\alpha}.
#'   When \eqn{\beta = 0} (the default), we use \eqn{\log_e(y + \alpha)}}
#' \item{\code{"power"}}{Power transformation: \eqn{(y-\beta)^\alpha}, where \eqn{y > \beta}.
#'   When \eqn{\alpha = 0}, \eqn{\log(y-\beta)} is used instead.}
#' \item{\code{"boxcox"}}{The Box-Cox transformation (unscaled by the geometric
#'   mean): \eqn{((y - \beta)^\alpha - 1) / \alpha}, where \eqn{y > \beta}. 
#'   When \eqn{\alpha = 0}, \eqn{\log(y - \beta)}
#'   is used.}
#' \item{\code{"sympower"}}{A symmetrized power transformation on the whole real
#'   line:
#'   \eqn{|y - \beta|^\alpha\cdot sign(y - \beta)}. There are no restrictions on \eqn{y}, but we
#'   require \eqn{\alpha > 0} in order for the transformation to be monotone and
#'   continuous.}
#' \item{\code{"asin.sqrt"}}{Arcsin-square-root transformation:
#'   \eqn{\sin^{-1}(y/\alpha)^{1/2}}. Typically, \code{alpha} will be either 1 (default) or 100.}
#' \item{\code{"atanh"}}{Arctanh transformation:
#'   \eqn{\tanh^{-1}(y/\alpha)}. Typically, \code{alpha} will be either 1 (default) or 100.}
#' \item{\code{"bcnPower"}}{Box-Cox with negatives allowed, as described for the 
#'   \code{bcnPower} function in the \pkg{car} package. It is defined as the Box-Cox
#'   transformation \eqn{(z^\alpha - 1) / \alpha} of the variable \eqn{z = y + (y^2+\beta^2)^{1/2}}. 
#'   Note that this requires both parameters and that \code{beta > 0}.}
#' \item{\code{"scale"}}{This one is a little different than the others, in that
#'   \code{alpha} and \code{beta} are ignored; instead, they are determined by calling 
#'   \code{scale(y, ...)}. The user should give as \code{y} the response variable in the
#'   model to be fitted to its scaled version.}
#' 
#' }
#' 
#' Note that with the \code{"power"}, \code{"boxcox"}, or \code{"sympower"} transformations, 
#' the argument \code{beta} specifies a location shift. 
#' In the \code{"genpower"} transformation, \code{beta} specifies
#' the base of the logarithm -- however, quirkily, the default of \code{beta = 0}
#' is taken to be the natural logarithm. For example,
#' \code{make.tran(0.5, 10)} sets up the \eqn{\log_{10}(y + \frac12)}
#' transformation. In the \code{"bcnPower"} transformation, \code{beta}
#' must be specified as a positive value.
#' 
#' For purposes of back-transformation, the \samp{sqrt(y) + sqrt(y+1)}
#' transformation is treated exactly the same way as \samp{2*sqrt(y)}, because
#' both are regarded as estimates of \eqn{2\sqrt\mu}.
#' 
#' @section Cases where \code{make.tran} may not be needed:
#' For standard transformations with no parameters, we usually don't need to use
#' \code{make.tran}; just the name of the transformation is all that is needed.
#' The functions \code{\link{emmeans}}, \code{\link{ref_grid}}, and related ones
#' automatically detect response transformations that are recognized by
#' examining the model formula. These are \code{log}, \code{log2}, \code{log10},
#' \code{log1p},
#' \code{sqrt}, \code{logit}, \code{probit}, \code{cauchit}, \code{cloglog}; as
#' well as (for a response variable \code{y}) \code{asin(sqrt(y))},
#' \code{asinh(sqrt(y))}, \code{atanh(y)}, and \code{sqrt(y) + sqrt(y+1)}. 
#' In addition, any
#' constant multiple of these (e.g., \code{2*sqrt(y)}) is auto-detected and
#' appropriately scaled (see also the \code{tran.mult} argument in
#' \code{\link{update.emmGrid}}).
#' 
#' A few additional transformations may be specified as character strings and
#' are auto-detected: \code{"identity"}, \code{"1/mu^2"},
#' \code{"inverse"}, \code{"reciprocal"}, \code{"log10"}, \code{"log2"},
#' \code{"asin.sqrt"}, \code{"asinh.sqrt"}, and \code{"atanh"}.
#' 
#'
#' @section Compound transformations:
#' A transformation that is a function of another function can be created by
#' specifying \code{inner} for the other function. For example, the
#' transformation \eqn{1/\sqrt{y}} can be created either by
#' \code{make.tran("inverse", inner = "sqrt")} or by \code{make.tran("power",
#' -0.5)}. In principle, transformations can be compounded to any depth.
#' Also, if \code{type} is \code{"scale"}, \code{y} is replaced by 
#' \code{inner$linkfun(y)}, because that will be the variable that is scaled.
#' 
#' @return A \code{list} having at least the same elements as those returned by
#'   \code{\link{make.link}}. The \code{linkfun} component is the transformation
#'   itself. Each of the functions is associated with an environment where any 
#'   parameter values are defined.
#' 
#' @note The \code{genlog} transformation is technically unneeded, because
#'   a response transformation of the form \code{log(y + c)} is now auto-detected 
#'   by \code{\link{ref_grid}}.
#' @note We modify certain \code{\link{make.link}} results in transformations
#'   where there is a restriction on valid prediction values, so that reasonable
#'   inverse predictions are obtained, no matter what. For example, if a
#'   \code{sqrt} transformation was used but a predicted value is negative, the
#'   inverse transformation is zero rather than the square of the prediction. A
#'   side effect of this is that it is possible for one or both confidence
#'   limits, or even a standard error, to be zero.
#' @export
#'
#' @examples
#' # Fit a model using an oddball transformation:
#' bctran <- make.tran("boxcox", 0.368)
#' warp.bc <- with(bctran, 
#'     lm(linkfun(breaks) ~ wool * tension, data = warpbreaks))
#' # Obtain back-transformed LS means:    
#' emmeans(warp.bc, ~ tension | wool, type = "response")
#' 
#' ### Using a scaled response...
#' # Case where it is auto-detected:
#' mod <- lm(scale(yield[, 1]) ~ Variety, data = MOats)
#' emmeans(mod, "Variety", type = "response")
#' 
#' # Case where scaling is not auto-detected -- and what to do about it:
#' copt <- options(contrasts = c("contr.sum", "contr.poly"))
#' mod.aov <- aov(scale(yield[, 1]) ~ Variety + Error(Block), data = MOats)
#' emm.aov <- suppressWarnings(emmeans(mod.aov, "Variety", type = "response"))
#' 
#' # Scaling was not retrieved, but we can do:
#' emm.aov <- update(emm.aov, tran = make.tran("scale", y = MOats$yield[, 1]))
#' emmeans(emm.aov, "Variety", type = "response")
#' 
#' ### Compound transformations
#' # The following amount to the same thing:
#' t1 <- make.tran("inverse", inner = "sqrt")
#' t2 <- make.tran("power", -0.5)
#' 
#' options(copt)
#'
#' 
#' \dontrun{
#' ### An existing model 'mod' was fitted with a y^(2/3) transformation...
#'   ptran = make.tran("power", 2/3)
#'   emmeans(mod, "treatment", tran = ptran)
#' }
make.tran = function(type = c("genlog", "power", "boxcox", "sympower", 
                              "asin.sqrt", "atanh", "bcnPower", "scale"), 
                              alpha = 1, beta = 0, param, y, inner, ...) {
    if(!missing(inner)){
        if(type == "scale") {
            if(is.character(inner))
                inner = make.tran(inner)
            y = inner$linkfun(y)
        }
        return(.make.compound.link(make.tran(type, alpha, beta, param, y), inner))
    }
    
    txt = type
    type = try(match.arg(type), silent = TRUE)
    if(inherits(type, "try-error"))
        return(.make.link(txt))
    remove(list = "txt")
    
    # backward compat
    if(length(alpha) > 1) param = alpha  # unnamed parem
    if (!missing(param)) {
        if (length(param) > 1) beta = param[2]
        alpha = param[1]
     }
    mu.lbl = "mu"
    if (beta != 0)
        mu.lbl = paste0("(mu - ", round(beta, 3), ")")
    if(type == "scale") {
        sy = scale(y, ...)
        if(is.null(alpha <- attr(sy, "scaled:center")))
            alpha = 0
        if(is.null(beta <- attr(sy, "scaled:scale")))
            beta = 1
        remove(list = c("y", "sy")) # remove baggage from env
    }
    switch(type,
           genlog = { # beta serves in the role of the base of the log
               if((beta < 0) || (beta == 1))
                   stop('"genlog" transformation must have a positive base != 1')
               logbase = ifelse(beta == 0, 1, log(beta))
               xlab = ifelse(beta == 0, "", paste0(" (base ", round(beta, 3), ")"))
               list(linkfun = function(mu) log(pmax(mu + alpha, 0)) / logbase,
                    linkinv = function(eta) pmax(exp(logbase * eta), .Machine$double.eps) - alpha,
                    mu.eta = function(eta) logbase * pmax(exp(logbase * eta), .Machine$double.eps),
                    valideta = function(eta) TRUE,
                    alpha = alpha, logbase = logbase,
                    name = paste0("log(mu + ", round(alpha,3), ")", xlab)
               )
           },
           power = {
               if (alpha == 0) {
                   if(beta == 0) make.link("log")
                   else make.tran("genlog", -beta)
               }
               else list(
                   linkfun = function(mu) pmax(mu - beta, 0)^alpha,
                   linkinv = function(eta) beta + pmax(eta, 0)^(1/alpha),
                   mu.eta = function(eta) pmax(eta, 0)^(1/alpha - 1) / alpha,
                   valideta = function(eta) all(eta > 0),
                   alpha = alpha, beta = beta, 
                   name = ifelse(alpha > 0, 
                                 paste0(mu.lbl, "^", round(alpha,3)),
                                 paste0(mu.lbl, "^(", round(alpha,3), ")"))
               )
           },
           boxcox = {
               if (alpha == 0) {
                   result = if(beta == 0) make.link("log")
                   else make.tran("genlog", -beta)
                   return (result)
               }
               min.eta = ifelse(alpha > 0, -1 / alpha, -Inf)
               xlab = ifelse(beta == 0, "", paste0(" of (y - ", round(beta, 3), ")"))
               list(
                   linkfun = function(mu) ((mu - beta)^alpha - 1) / alpha,
                   linkinv = function(eta) beta + (1 + alpha * pmax(eta, min.eta))^(1/alpha),
                   mu.eta = function(eta) (1 + alpha * pmax(eta, min.eta))^(1/alpha - 1),
                   valideta = function(eta) all(eta > min.eta),
                   alpha = alpha, beta = beta, 
                   name = paste0("Box-Cox (lambda = ", round(alpha, 3), ")", xlab)
               )
           },
           sympower = {
               if (alpha <= 0) 
                   stop('"sympower" transformation requires positive alpha')
               absmu.lbl = gsub("\\(|\\)", "|", mu.lbl)
               if (beta == 0) mu.lbl = paste0("(", mu.lbl, ")")
               list(linkfun = function(mu) sign(mu - beta) * abs(mu - beta)^alpha,
                    linkinv = function(eta) beta + sign(eta) * abs(eta)^(1/alpha),
                    mu.eta = function(eta) (abs(eta))^(1/alpha - 1),
                    valideta = function(eta) all(eta > min.eta),
                    alpha = alpha, beta = beta, 
                    name = paste0(absmu.lbl, "^", round(alpha,3), " * sign", mu.lbl)
               )
           },
           asin.sqrt = {
               mu.lbl = ifelse(alpha == 1, "mu", paste0("mu/", round(alpha,3)))
               list(linkfun = function(mu) asin(sqrt(mu/alpha)),
                    linkinv = function(eta) alpha * sin(pmax(pmin(eta, pi/2), 0))^2,
                    mu.eta = function(eta) alpha * sin(2*pmax(pmin(eta, pi/2), 0)),
                    valideta = function(eta) all(eta <= pi/2) && all(eta >= 0),
                    alpha = alpha,
                    name = paste0("asin(sqrt(", mu.lbl, "))")
               )
           },
           atanh = {
               mu.lbl = ifelse(alpha == 1, "mu", paste0("mu/", round(alpha,3)))
               list(linkfun = function(mu) atanh(mu/alpha),
                    linkinv = function(eta) alpha * tanh(eta),
                    mu.eta = function(eta) alpha * (1 - tanh^2(eta)),
                    valideta = function (eta) all(is.finite(eta)) && all(eta > -1) && all(eta < 1),
                    alpha = alpha,
                    name = paste0("atanh(", mu.lbl, ")")
               )
           },
           bcnPower = {
               if(beta <= 0)
                   stop ("The value of 'beta' must be strictly positive.")
               list(
                   linkfun = function(mu) {
                       s = sqrt(mu^2 + beta^2)
                       if (abs(alpha) < 1e-10) log(.5*(mu + s))
                       else ((0.5 * (mu + s))^alpha - 1) / alpha  },
                   linkinv = function(eta) {
                       q = if (abs(alpha) < 1e-10) 2 * exp(eta)
                           else 2 * (alpha * eta + 1) ^ (1/alpha)
                       (q^2 - beta^2) / (2 * q) },
                   mu.eta = function(eta) {
                       if (abs(alpha) < 1e-10) { q = 2 * exp(eta); dq = q }
                       else { q = 2 * (alpha * eta + 1) ^ (1/alpha)
                           dq = 2 * (alpha * eta + 1)^(1/alpha - 1) }
                       0.5 * (1 + (beta/q)^2) * dq },
                   valideta = function(eta) all(eta > 0),
                   alpha = alpha, beta = beta, 
                   name = paste0("bcnPower(", signif(alpha,3), ", ", signif(beta,3), ")")
               )
           },
           scale = list(
               linkfun = function(mu) (mu - alpha) / beta,
               linkinv = function(eta) beta * eta + alpha,
               mu.eta = function(eta) rep(beta, length(eta)),
               valideta = function(eta) TRUE,
               alpha = alpha, beta = beta, 
               name = paste0("scale(", signif(alpha, 3), ", ", signif(beta, 3), ")")
           )
    )
}

# Create a compound link function f(g(y)) where outer and inner are make.link results
.make.compound.link = function(outer, inner, ...) {
    if(is.character(inner))
        inner = make.tran(inner)
    name = outer$name
    if(length(grep("mu", name)) == 0) name = paste0(name, "(mu)")
    name = gsub("mu", inner$name, name)
    list(
        linkfun = function(mu) outer$linkfun(inner$linkfun(mu)),
        linkinv = function(eta) inner$linkinv(outer$linkinv(eta)),
        mu.eta = function(eta) inner$mu.eta(outer$linkinv(eta))*outer$mu.eta(eta),
        valideta = function(eta) inner$valideta(outer$linkinv(eta)),
        name = name
    )
}


#' @rdname make.tran
#' @export
#' @return \code{inverse} returns the reciprocal of its argument. It allows
#'   the \code{"inverse"} link to be auto-detected as a response transformation.
#' @examples
#' 
#' pigs.lm <- lm(inverse(conc) ~ source + factor(percent), data = pigs)
#' emmeans(pigs.lm, "source", type = "response")
inverse = function(y) 1/y


### My modification/expansion of stats:make.link()
### Also, if not found, returns make.link("identity") modified with
##     unknown = TRUE, name = link
## In addition, I make all links truly monotone on (-Inf, Inf) in
##     lieu of valideta
##
## Extensions to make.link results:
##     unknown: set to TRUE if link is unknown
##     mult: scalar multiple of transformation
##
.make.link = function(link) {
    if (link %in% c("logit", "probit", "cauchit", "cloglog", "identity", "log"))
        result = stats::make.link(link)
    else result = switch(link,
         sqrt = { 
             tmp = make.link("sqrt") 
             tmp$linkinv = function(eta) pmax(0, eta)^2
             tmp$mu.eta = function(eta) 2*pmax(0, eta)
             tmp },
         `1/mu^2` = { 
             tmp = make.link("1/mu^2") 
             tmp$linkinv = function(eta) 1/sqrt(pmax(0, eta))
             tmp$mu.eta = function(eta) -1/(2*pmax(0, eta)^1.5)
             tmp },
         inverse = { 
             tmp = make.link("inverse") 
             tmp$linkinv = function(eta) 1/pmax(0, eta)
             tmp$mu.eta = function(eta) -1/pmax(0, eta)^2
             tmp },
         `/` = .make.link("inverse"),
         reciprocal = .make.link("inverse"),
         log10 = list(
             linkfun = log10,
             linkinv = function(eta) 10^eta,
             mu.eta = function(eta) 10^eta * log(10),
             name = "log10"
         ),
         log2 = list(
             linkfun = log2,
             linkinv = function(eta) 2^eta,
             mu.eta = function(eta) 2^eta * log(2),
             name = "log2"
         ),
         log1p = list(
             linkfun = log1p,
             linkinv = expm1,
             mu.eta = exp,
             name = "log1p"
         ),
         asin.sqrt = make.tran("asin.sqrt"),
         `asin.sqrt./` = make.tran("asin.sqrt", 100),
         asinh.sqrt = list(
             linkinv = function(eta) sinh(eta)^2,
             mu.eta = function(eta) sinh(2 * eta),
             name = "asinh(sqrt(mu))"
         ),
         atanh = make.tran("atanh"),
         exp = list(
           linkinv = function(eta) log(eta),
           mu.eta = function(eta) 1/eta,
           name = "exp"
         ),
         `+.sqrt` = {
             tmp = .make.link("sqrt")
             tmp$mult = 2
             tmp
         },
         log.o.r. = {
             tmp = make.link("log")
             tmp$name = "log odds ratio"
             tmp
         },
         
         { # default if not included, flags it as unknown
             tmp = stats::make.link("identity")
             tmp$unknown = TRUE
             tmp$name = link
             tmp
         }
    )
    result
}


### Internal routine to make a scales::trans object
.make.scale = function(misc) {
    if (!requireNamespace("scales", quietly = TRUE)) 
        stop("type = \"scale\" requires the 'scales' package to be installed")
    tran = misc$tran
    if (is.null(tran))
        return(scales::identity_trans())
    if (is.character(tran)) {
        # is it a canned scale?
        if ((length(intersect(names(misc), c("tran.mult", "tran.offset"))) == 0) && 
            tran %in% c("log", "log1p", "log2", "log10", "sqrt", "logit", "probit", 
                        "exp", "identity"))
            return(get(paste(tran, "trans", sep = "_"), envir = asNamespace("scales"))())
        # not built-in, so let's get a list
        tran = .make.link(tran)
    }
    # tran is a list. we'll incorporate any scaling
    tran$mult = ifelse(is.null(misc$tran.mult), 1, misc$tran.mult)
    tran$offset = ifelse(is.null(misc$tran.offset), 0, misc$tran.offset)
    with(tran, 
         scales::trans_new(name, 
                           function(x) mult * linkfun(x + offset), 
                           function(z) linkinv(z / mult) - offset))
}

##############################################################################
#    Copyright (c) 2012-2019 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

# contrast() and related functions (previously in with emmeans code)

### 'contrast' S3 generic and method
#' Contrasts and linear functions of EMMs
#' 
#' These methods provide for follow-up analyses of \code{emmGrid} objects:
#' Contrasts, pairwise comparisons, tests, and confidence intervals. They may
#' also be used to compute arbitrary linear functions of predictions or EMMs.
#'
#' @export
contrast = function(object, ...)
    UseMethod("contrast")

#' @rdname contrast
#' @param object An object of class \code{emmGrid}
#' @param method Character value giving the root name of a contrast method (e.g.
#'   \code{"pairwise"} -- see \link{emmc-functions}). Alternatively, a function
#'   of the same form, or a named \code{list} of coefficients (for a contrast or
#'   linear function) that must each conform to the number of results in each
#'   \code{by} group. In a multi-factor situation, the factor levels are
#'   combined and treated like a single factor.
#' @param interaction Character vector, logical value, or list. If this is specified,
#'   \code{method} is ignored. See the \dQuote{Interaction contrasts} section
#'   below for details.
#' @param by Character names of variable(s) to be used for ``by'' groups. The
#'   contrasts or joint tests will be evaluated separately for each combination
#'   of these variables. If \code{object} was created with by groups, those are
#'   used unless overridden. Use \code{by = NULL} to use no by groups at all.
#' @param offset,scale Numeric vectors of the same length as each \code{by} group.
#'   The \code{scale} values, if supplied, multiply their respective linear estimates, and
#'   any \code{offset} values are added. Scalar values are also allowed. 
#'   (These arguments are ignored when \code{interaction} is specified.)
#' @param name Character name to use to override the default label for contrasts
#'   used in table headings or subsequent contrasts of the returned object.
#' @param options If non-\code{NULL}, a named \code{list} of arguments to pass
#'   to \code{\link{update.emmGrid}}, just after the object is constructed.
#' @param type Character: prediction type (e.g., \code{"response"}) -- added to
#'   \code{options}
#' @param adjust Character: adjustment method (e.g., \code{"bonferroni"}) --
#'   added to \code{options}
#' @param simple Character vector or list: Specify the factor(s) \emph{not} in
#'   \code{by}, or a list thereof. See the section below on simple contrasts.
#' @param combine Logical value that determines what is returned when
#'   \code{simple} is a list. See the section on simple contrasts.
#' @param ratios Logical value determining how log and logit transforms are
#'   handled. These transformations are exceptional cases in that there is a
#'   valid way to back-transform contrasts: differences of logs are logs of
#'   ratios, and differences of logits are odds ratios. If \code{ratios = TRUE}
#'   and summarized with \code{type = "response"}, \code{contrast} results are
#'   back-transformed to ratios whenever we have true contrasts (coefficients
#'   sum to zero). For other transformations, there is no natural way to
#'   back-transform contrasts, so even when summarized with \code{type = "response"},
#'   contrasts are computed and displayed on the linear-predictor scale. Similarly, 
#'   if \code{ratios = FALSE}, log and logit transforms are treated in the same way as
#'   any other transformation.
#' @param parens character or \code{NULL}. If a character value, the labels for levels
#'   being contrasted are parenthesized if they match the regular expression in 
#'   \code{parens[1]} (via \code{\link{grep}}). The default is \code{emm_option("parens")}.
#'   Optionally, \code{parens} may contain second and third elements specifying
#'   what to use for left and right parentheses (default \code{"("} and \code{")"}).
#'   Specify \code{parens = NULL} or \code{parens = "a^"} (which won't match anything)
#'   to disable all parenthesization.
#' @param enhance.levels character or logical. 
#'   If character, the levels of the named factors that are contrasted are enhanced 
#'   by appending the name of the factor; e.g., if a factor named \code{"trt"} has
#'   levels \code{A} and \code{B}, a \code{trt} comparison is labeled \code{trtA - trtB}.
#'   If \code{enhance.levels} is logical, then if \code{TRUE} (the default), 
#'   only factors with numeric levels are enhanced; and of
#'   course if \code{FALSE}, no levels are enhanced.
#'   The levels of \code{by} variables are not enhanced, and any 
#'   names of factors that don't exist are silently ignored. 
#'   To enhance the labels beyond what is done here, change them
#'   directly via \code{\link[=update.emmGrid]{levels<-}}.
#' @param wts The \code{wts} argument for some contrast methods. You should omit
#'   this argument unless you want unequal \code{wts}. Otherwise we recommend
#'   specifying \code{wts = NA} which instructs that \code{wts} be obtained from
#'   \code{object}, \emph{separately} for each \code{by} group. If numerical
#'   \code{wts} are specified, they must
#'   conform to the number of levels in each \code{by} group, and those same weights
#'   are used in each group. 
#'
#' @param ... Additional arguments passed to other methods
#'
#' @return \code{contrast} and \code{pairs} return an object of class
#'   \code{emmGrid}. Its grid will correspond to the levels of the contrasts and
#'   any \code{by} variables. The exception is that an \code{\link{emm_list}}
#'   object is returned if \code{simple} is a list and \code{combine} is
#'   \code{FALSE}.
#'
#' @section Pairs method: The call \code{pairs(object)} is equivalent to
#'   \code{contrast(object, method = "pairwise")}; and \code{pairs(object,
#'   reverse = TRUE)} is the same as \code{contrast(object, method =
#'   "revpairwise")}.
#'
#' @section Interaction contrasts: When \code{interaction} is specified,
#'   interaction contrasts are computed. Specifically contrasts are generated
#'   for each factor separately, one at a time; and these contrasts are applied
#'   to the object (the first time around) or to the previous result
#'   (subsequently). (Any factors specified in \code{by} are skipped.) The final
#'   result comprises contrasts of contrasts, or, equivalently, products of
#'   contrasts for the factors involved. Any named elements of \code{interaction}
#'   are assigned to contrast methods; others are assigned in order of
#'   appearance in \code{object@levels}. The contrast factors in the resulting 
#'   \code{emmGrid} object are ordered the same as in \code{interaction}.
#'   
#'   \code{interaction} may be a character vector or list of valid contrast
#'   methods (as documented for the \code{method} argument). If the vector or
#'   list is shorter than the number needed, it is recycled. Alternatively, if
#'   the user specifies \code{contrast = TRUE}, the contrast specified in
#'   \code{method} is used for all factors involved.
#'   
#' @section Simple contrasts:
#'   \code{simple} is essentially the complement of \code{by}: When
#'   \code{simple} is a character vector, \code{by} is set to all the factors in
#'   the grid \emph{except} those in \code{simple}. If \code{simple} is a list,
#'   each element is used in turn as \code{simple}, and assembled in an
#'   \code{"emm_list"}. To generate \emph{all} simple main effects, use
#'   \code{simple = "each"} (this works unless there actually is a factor named
#'   \code{"each"}). Note that a non-missing \code{simple} will cause \code{by}
#'   to be ignored.
#'   
#'   Ordinarily, when \code{simple} is a list or \code{"each"}, the return value
#'   is an \code{\link{emm_list}} object with each entry in correspondence with
#'   the entries of \code{simple}. However, with \code{combine = TRUE}, the
#'   elements are all combined into one family of contrasts in a single
#'   \code{\link[=emmGrid-class]{emmGrid}} object using
#'   \code{\link{rbind.emmGrid}}.. In that case, the \code{adjust} argument sets
#'   the adjustment method for the combined set of contrasts.
#'
#' @note When \code{object} has a nesting structure (this can be seen via
#'   \code{str(object)}), then any grouping factors involved are forced into
#'   service as \code{by} variables, and the contrasts are thus computed
#'   separately in each nest. This in turn may lead to an irregular grid in the
#'   returned \code{emmGrid} object, which may not be valid for subsequent
#'   \code{emmeans} calls.
#'
#' @method contrast emmGrid
#' @export
#'
#' @examples
#' warp.lm <- lm(breaks ~ wool*tension, data = warpbreaks)
#' (warp.emm <- emmeans(warp.lm, ~ tension | wool))
#' 
#' contrast(warp.emm, "poly")    # inherits 'by = "wool"' from warp.emm
#' 
#' ### Custom contrast coefs (we already have wool as 'by' thus 3 means to contrast)
#' contrast(warp.emm, list(mid.vs.ends = c(-1,2,-1)/2, lo.vs.hi = c(1,0,-1)))
#' 
#' pairs(warp.emm)
#' 
#' # Effects (dev from mean) of the 6 factor combs, with enhanced levels:
#' contrast(warp.emm, "eff", by = NULL, 
#'     enhance.levels = c("wool", "tension"))  
#'     
#' pairs(warp.emm, simple = "wool") # same as pairs(warp.emm, by = "tension")
#' 
#' # Do all "simple" comparisons, combined into one family
#' pairs(warp.emm, simple = "each", combine = TRUE)
#' 
#' \dontrun{
#' 
#' ## Note that the following are NOT the same:
#' contrast(warp.emm, simple = c("wool", "tension"))
#' contrast(warp.emm, simple = list("wool", "tension"))
#' ## The first generates contrasts for combinations of wool and tension
#' ##   (same as by = NULL)
#' ## The second generates contrasts for wool by tension, and for 
#' ##   tension by wool, respectively.
#' }
#'
#' # An interaction contrast for tension:wool
#' tw.emm <- contrast(warp.emm, interaction = c(tension = "poly", wool = "consec"), 
#'                    by = NULL)
#' tw.emm          # see the estimates
#' coef(tw.emm)    # see the contrast coefficients
#' 
#' # Use of scale and offset
#' #   an unusual use of the famous stack-loss data...
#' mod <- lm(Water.Temp ~ poly(stack.loss, degree = 2), data = stackloss)
#' (emm <- emmeans(mod, "stack.loss", at = list(stack.loss = 10 * (1:4))))
#' # Convert results from Celsius to Fahrenheit:
#' confint(contrast(emm, "identity", scale = 9/5, offset = 32))
#' 
contrast.emmGrid = function(object, method = "eff", interaction = FALSE, 
                        by, offset = NULL, scale = NULL, name = "contrast", 
                        options = get_emm_option("contrast"), 
                        type, adjust, simple, combine = FALSE, ratios = TRUE, 
                        parens, enhance.levels = TRUE, wts, ...) 
{
    if (!missing(type))
        options = as.list(c(options, predict.type = type))
    
    if(!missing(simple))
        return(.simcon(object, method = method, interaction = interaction,
                      offset = offset, scale = scale, name = name, options = options,
                      type = type, simple = simple, combine = combine, 
                      adjust = adjust, parens = parens, ...))
    if(missing(by)) 
        by = object@misc$by.vars
    if(length(by) == 0) # character(0) --> NULL
        by = NULL
    
    nesting = object@model.info$nesting
    if (!is.null(nesting) || !is.null(object@misc$display))
        return (.nested_contrast(rgobj = object, method = method, interaction = interaction, by = by, adjust = adjust, type = type, offset = offset, ...))
    
    orig.grid = object@grid[, , drop = FALSE]
    orig.grid[[".wgt."]] = orig.grid[[".offset."]] = NULL
    
    if (is.logical(interaction) && interaction)
        interaction = method
    if (!is.logical(interaction)) { # i.e., interaction is not FALSE
        if(missing(adjust))
            adjust = "none"
        vars = names(object@levels)
        k = length(vars)
        if(!is.null(by)) {
            vars = c(setdiff(vars, by), by)
            k = k - length(by)
        }
        nms = names(interaction)
        interaction = as.list(rep(interaction, k)[1:k])
        names(interaction) = c(nms, rep("", k))[1:k]
        nms = vars[1:k]
        if (is.null(names(interaction)))
            names(interaction) = nms
        else {
            unnamed = which(!(names(interaction) %in% nms))
            names(interaction)[unnamed] = setdiff(nms, names(interaction))
            nms = names(interaction)
        }
        
        
        # if (!is.character(interaction))
        #     stop("interaction requires named contrast function(s)")
        ### by = NULL why was this here before ???
        tcm = NULL
        for (i in k:1) {
            if (is.character(interaction[[i]]))
                nm = paste(nms[i], interaction[[i]], sep = "_")
            else
                nm = paste(nms[i], "custom", sep = "_")
            pos = which(vars == nms[i])
            object = contrast.emmGrid(object, interaction[[i]], by = vars[-pos], 
                                      name = nm, enhance.levels = FALSE, ...)
            if(is.null(tcm))
                tcm = object@misc$con.coef
            else
                tcm = object@misc$con.coef %*% tcm
            vars[pos] = nm
        }
        object = update(object, by = by, adjust = adjust, silent = TRUE) 
          ### removed `...` here Nov 2019 because a `mode` arg gets matched with `model.info`
          ### when passed via formula lhs in `emmeans()`
        object@misc$is.new.rg = NULL
        object@misc$orig.grid = orig.grid
        object@misc$con.coef = tcm
        object = .update.options(object, options, ...)
        return(object)
    }
    
    # else we have a regular contrast (not interaction)
    linfct = object@linfct[, , drop = FALSE]
    args = g = object@grid[, , drop = FALSE]
    args[[".offset."]] = NULL 
    args[[".wgt."]] = NULL # ignore auxiliary stuff in labels, etc.
    
    # figure out any enhancement of factor levels
    if (is.logical(enhance.levels)) { # convert to character
        if(enhance.levels)
            enhance.levels = names(args[sapply(args, .is.num)])               
        else # FALSE
            enhance.levels = character(0)
    }
    enhance.levels = intersect(setdiff(enhance.levels, by), names(args))
    for (nm in enhance.levels)
        args[[nm]] = paste0(nm, args[[nm]])
    
    if (!is.null(by)) {
        by.rows = .find.by.rows(args, by)
        ulen = unique(sapply(by.rows, length))
        if (length(ulen) > 1)
            stop ("`by` groups are of irregular size;\n  currently not supported except with nested structures")
        bylevs = args[, by, drop=FALSE]
        all.args = args
        args = args[by.rows[[1]], , drop=FALSE]
        for (nm in by) {
            args[[nm]] = NULL
            all.args[[nm]] = NULL
        }
        all.levs = do.call("paste", c(unname(all.args), sep = get_emm_option("sep")))   # keep all levels in case we have permutations of them
    }
    args = unname(args)
    args$sep = get_emm_option("sep")
    levs = do.call("paste", args)  # NOTE - these are levels for the first (or only) by-group
    if (length(levs) == 0)   # prevent error when there are no levels to contrast
        method = "eff"
    if(is.null(by))
        all.levs = levs
    
    # parenthesize levels if they contain spaces or operators
    rawlevs = levs  # save orig ones
    if(missing(parens))
        parens = get_emm_option("parens")
    if(is.character(parens) && length(idx <- grep(parens[1], all.levs)) > 0) {
        if(length(parens) < 3)
            parens = c(parens, "(", ")")
        all.levs[idx] = paste0(parens[2], all.levs[idx], parens[3])
        idx = grep(parens[1], levs)
        if (length(idx) > 0)
            levs[idx] = paste0(parens[2], levs[idx], parens[3])
    }
    attr(levs, "raw") = rawlevs # so we can recover original levels when needed
    
    variable.cmat = FALSE
    if (is.list(method)) {
        cmat = as.data.frame(method, optional = TRUE)
        # I have no clue why they named that argument 'optional',
        # but setting it to TRUE keeps it from messing up the names
        method = function(levs, ...) cmat
    }
    else if (is.character(method)) {
        fn = paste(method, "emmc", sep=".")
        if (exists(fn, mode = "function")) 
            method = get(fn) 
        else if (exists(fn, mode = "function", envir = parent.frame()))
            method = get(fn, envir = parent.frame()) 
        else 
            stop(paste("Contrast function '", fn, "' not found", sep=""))
        if(!missing(wts) && any(is.na(wts))) {
            variable.cmat = TRUE
            all.wts = weights(object)
            wts = if(!is.null(by))   rep(1, length(levs))
                  else               all.wts
        }
        if(missing(wts))
            wts = rep(1, length(levs))
    }

    # case like in old lsmeans, contr = list
    else if (!is.function(method))
        stop("'method' must be a list, function, or the basename of an '.emmc' function")
    
    # Get the contrasts; this should be a data.frame
    cmat = method(levs, wts = wts, ...)
    if (!is.data.frame(cmat))
        stop("Contrast function must provide a data.frame")
    else if(ncol(cmat) == 0) {
        cmat = data.frame(`(nothing)` = rep(NA, nrow(args)), check.names = FALSE)
        adjust = "none"
    }
    # warning("No contrasts were generated! Perhaps only one emmean is involved.\n",
    #      "  This can happen, for example, when your predictors are not factors.")
    else if (nrow(cmat) != nrow(args))
        stop("Nonconforming number of contrast coefficients")
    tcmat = t(cmat)
    if (!is.null(scale)) {
        if (length(scale) %in% c(1, nrow(tcmat)))
            tcmat = tcmat * scale
        else
            stop("'scale' length of ", length(scale), 
                 " does not conform with ", nrow(tcmat), " contrasts", call. = FALSE)
    }
    else
        scale = 1
    
    # do some bookkeeping whereby we get NAs only when NA rows get nonzero weight
    NAflag = linfct[, 1] * 0
    NAflag[is.na(linfct[, 1])] = 1
    linfct[is.na(linfct)] = 0 # now we don't have any NAs but we know where to put them back later
    .NArows = function(mat, flag) # utility for flagging bad rows (any nonzero coef applied to an NA)
        apply(mat, 1, function(x, f) any(x * f != 0), flag)
    
    if (is.null(by)) {
        linfct = tcmat %*% linfct
        linfct[.NArows(tcmat, NAflag), ] = NA # put back the NAs where they belong
        grid = data.frame(.contrast.=names(cmat))
        if (hasName(object@grid, ".offset."))
            grid[[".offset."]] = t(cmat) %*% object@grid[[".offset."]]
        by.rows = list(seq_along(object@linfct[ , 1]))
    }
    
    # NOTE: The kronecker thing here depends on the grid being regular.
    # Irregular grids are handled by .nested_contrast
    else {
        tcmat = kronecker(.diag(rep(1,length(by.rows))), tcmat)
        if(variable.cmat) { # we have to replace each block
            rowidx = seq_len(ncol(cmat))  # note we're working with transpose
            colidx = seq_len(nrow(cmat))
            for(i in seq_along(by.rows)) {
                wts = all.wts[by.rows[[i]]]
                tcmat[rowidx, colidx] = t(method(levs, wts = wts, ...)) * scale
                rowidx = rowidx + ncol(cmat)
                colidx = colidx + nrow(cmat)
            }
        }
        linfct = tcmat %*% linfct[unlist(by.rows), , drop = FALSE]
        linfct[.NArows(tcmat, NAflag[unlist(by.rows)]), ] = NA
        tmp = expand.grid(con = names(cmat), by = seq_len(length(by.rows)), stringsAsFactors = FALSE)###unique(by.id))
        # check if levs have different orderings in subsequent by groups
        for (i in 1 + seq_along(by.rows[-1])) { 
            j = by.rows[[i]]
            if (any(all.levs[j] != levs)) {
                cm = method(all.levs[j], wts = wts, ...)
                tmp$con[seq_along(cm) + length(cm)*(i-1)] = names(cm)
            }
        }
        grid = data.frame(.contrast. = factor(tmp$con, levels = unique(tmp$con)))
        n.each = ncol(cmat)
        row.1st = sapply(by.rows, function(x) x[1])
        xlevs = list()
        for (v in by)
            xlevs[[v]] = rep(bylevs[row.1st, v], each=n.each)
        grid = cbind(grid, data.frame(xlevs, check.names = FALSE))
        if (hasName(object@grid, ".offset."))
            grid[[".offset."]] = tcmat %*% object@grid[unlist(by.rows), ".offset."]
    }
    
    # Rename the .contrast. column -- ordinarily to "contrast",
    # but otherwise a unique variation thereof
    con.pat = paste("^", name, "[0-p]?", sep = "")
    n.prev.con = length(grep(con.pat, names(grid)))
    con.col = grep("\\.contrast\\.", names(grid))
    con.name = paste(name, 
                     ifelse(n.prev.con == 0, "", n.prev.con), sep="")
    names(grid)[con.col] = con.name
    
    row.names(linfct) = NULL
    misc = object@misc
    misc$initMesg = NULL
    misc$estName = "estimate"
    if (!is.null(et <- attr(cmat, "type")))
        misc$estType = et
    else {
        is.con = all(abs(sapply(cmat, sum)) < .001)
        misc$estType = ifelse(is.con, "contrast", "prediction")
    }
    misc$methDesc = attr(cmat, "desc")
    misc$famSize = ifelse(is.null(fs <- attr(cmat, "famSize")), length(by.rows[[1]]), fs)
    misc$pri.vars = setdiff(names(grid), c(by, ".offset.",".wgt."))
    if (missing(adjust)) adjust = attr(cmat, "adjust")
    if (is.null(adjust)) adjust = "none"
    if (!is.null(attr(cmat, "offset")))
        offset = attr(cmat, "offset")
    if (!is.null(offset)) {
        if(!hasName(grid, ".offset."))
            grid[[".offset."]] = 0
        grid[[".offset."]] = grid[[".offset."]] + rep(offset, length(by.rows))
    }
    if (!is.null(fs <- attr(cmat, "famSize")))
        misc$famSize = fs
    misc$is.new.rg = FALSE
    misc$adjust = adjust
    misc$infer = c(FALSE, TRUE)
    misc$by.vars = by
    if(!is.na(misc$estType) && misc$estType == "pairs") # internal flag to keep track of original by vars for paired comps
        misc$.pairby = paste(c("", by), collapse = ",")
    # save contrast coefs
    by.cols = seq_len(ncol(tcmat))
    if(!is.null(by.rows))
        by.cols[unlist(by.rows)] = by.cols # gives us inverse of by.rows order
    misc$orig.grid = orig.grid  # save original grid
    misc$con.coef = tcmat[ , by.cols, drop = FALSE] # save contrast coefs
    # test that each set of coefs sums to 0
    true.con = all(abs(apply(cmat, 2, function(.) sum(.) / (1.0e-6 + max(abs(.))))) < 1.0e-6) 
    if(is.na(true.con)) # prevent error when there are no contrasts
        true.con = FALSE
    if(true.con)
        misc$sigma = NULL   # sigma surely no longer makes sense

    # zap the transformation info except in special cases
    if (!is.null(misc$tran)) {
        misc$orig.tran = .fmt.tran(misc)
        if (ratios && true.con && misc$orig.tran %in% c("log", "log2", "log10", "logit", "log.o.r.", log.mult.trans)) {
            misc$log.contrast = TRUE      # remember how we got here; used by summary
            misc$orig.inv.lbl = misc$inv.lbl
            if (misc$tran %in% c("logit", "alr", "clr", "ilr")) {
                misc$inv.lbl = "odds.ratio"
                misc$tran = "log.o.r."
                misc$tran.mult = misc$tran.offset = NULL
            }
            else if (misc$tran == "log.o.r.") {
                # leave everything as-is. Once an odds ratio, always an odds ratio
            }
            else {
                misc$inv.lbl = "ratio"
                ### (stays at log, log2, log10)   misc$tran = "log"
                misc$tran.mult = misc$tran.offset = NULL
            }
        }
        else {
            misc$initMesg = c(misc$initMesg, 
                              paste("Note: contrasts are still on the", misc$orig.tran, "scale. Consider using\n",
                              "     regrid() if you want contrasts of back-transformed estimates."))
             misc$tran = misc$tran.mult = misc$tran.offset = NULL
        }
    }
    
    # ensure we don't inherit inappropriate settings
    misc$null = misc$delta = misc$side = misc$calc = NULL
    
    object@roles$predictors = "contrast"
    levels = list()
    for (nm in setdiff(names(grid), ".offset."))
        levels[[nm]] = unique(grid[[nm]])
    
    ### bypass new as we're not re-classing    result = new("emmGrid", object, linfct = linfct, levels = levels, grid = grid, misc = misc)
    result = as(object, "emmGrid")
    result@linfct = linfct
    result@levels = levels
    result@grid = grid
    result@misc = misc
    result@roles$predictors = setdiff(names(result@levels), result@roles$multresp)
    
    .update.options(result, options, ...)
}


# Add-on to support `simple` argument - the complement of `by`
# e.g., with factors A, B, C, simple = "A" <==> by = c("B", "C")
# Note `by` is an argument so that it can be ignored and never duplicated
# If `simple` is a list, we run this on each element
# If simple = "each", we run it on each factor in the grid
# We handle `adjust`` ourselves rather than passing it to `contrast``
.simcon = function(object, ..., simple, by, combine = FALSE, adjust) {
    if (is.list(simple)) {
        if(is.null(names(simple)))
            names(simple) = sapply(simple, function(.) 
                paste("simple contrasts for", paste0(., collapse = "*")))
        result = lapply(simple, function(.) 
            .simcon(object, ..., simple = .))
        class(result) = c("emm_list", "list")
        if(combine)
            result = do.call(rbind.emmGrid, result)
    }
    else if ((length(simple) == 1) && (simple == "each") && !("each" %in% names(object@levels))) {
        result = .simcon(object, ..., 
            simple = as.list(names(object@levels)), combine = combine)
    }
    else {
        facs = names(object@levels)
        by = setdiff(facs, simple)
        if (length(by) == 0) by = NULL
        result = contrast.emmGrid(object, ..., by = by)
    }
    if (!missing(adjust)) {
        if (is.list(result)) {
            for (i in seq_along(result))
                result[[i]] = update(result[[i]], adjust = adjust)
        }
        else
            result = update(result, adjust = adjust)
    }
    result
}


# pairs method

#' @rdname contrast 
#' @param x An \code{emmGrid} object
#' @param reverse Logical value - determines whether to use \code{"pairwise"} (if \code{TRUE}) or \code{"revpairwise"} (if \code{FALSE}).
#' @importFrom graphics pairs
#' @export
pairs.emmGrid = function(x, reverse = FALSE, ...) {
    object = x # for my sanity
    if (reverse)
        contrast(object, method = "revpairwise", ...)
    else
        contrast(object, method = "pairwise", ...)
}


# coef method - returns contrast coefficients
#' @rdname contrast 
#' @return \code{coef} returns a \code{data.frame} containing the "parent" object's grid, 
#' along with columns named \code{c.1, c.2, ...} containing the contrast coefficients
#' used to produce the linear functions embodied in the object. \code{coef()} only
#' returns coefficients if \code{object} is the result of a call to \code{contrast()},
#' and the parent object is the object that was handed to \code{contrast}. This
#' is most useful for understanding interaction contrasts.
#'
#' @export
#' @importFrom stats coef
#' @method coef emmGrid
coef.emmGrid = function(object, ...) {
    if (is.null(cc <- object@misc$con.coef)) {
        message("No contrast coefficients are available")
        return (NULL)
    }
    cc = as.data.frame(t(cc))
    rownames(cc) = NULL
    names(cc) = paste("c", seq_len(ncol(cc)), sep = ".")
    cbind(object@misc$orig.grid, cc)
}

#' @rdname contrast
#' @importFrom stats weights
#' @return \code{weights} returns the weights stored for each row of \code{object},
#'    or a vector of 1s if no weights are saved.
#' @method weights emmGrid
#' @export
weights.emmGrid = function(object, ...) {
    if(is.null(rtn <- object@grid[[".wgt."]]))
        rtn = rep(1, nrow(object@grid))
    if(!is.null(object@misc$display))
        rtn = rtn[object@misc$display]
    rtn
}

##############################################################################
#    Copyright (c) 2012-2017 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

### Code for emtrends


### emtrends function
#' Estimated marginal means of linear trends
#'
#' The \code{emtrends} function is useful when a fitted model involves a
#' numerical predictor \eqn{x}  interacting with another predictor \code{a}
#' (typically a factor). Such models specify that \eqn{x} has a different trend
#' depending on \eqn{a}; thus, it may be of interest to estimate and compare
#' those trends. Analogous to the \code{\link{emmeans}} setting, we construct a
#' reference grid of these predicted trends, and then possibly average them over
#' some of the predictors in the grid.
#' 
#' The function works by constructing reference grids for \code{object} with 
#' various values of \code{var}, and then calculating difference quotients of predictions
#' from those reference grids. Finally, \code{\link{emmeans}} is called with
#' the given \code{specs}, thus computing marginal averages as needed of
#' the difference quotients. Any \code{...} arguments are passed to the
#' \code{ref_grid} and \code{\link{emmeans}}; examples of such optional
#' arguments include optional arguments (often \code{mode}) that apply to
#' specific models; \code{ref_grid} options such as \code{data}, \code{at},
#' \code{cov.reduce}, \code{mult.names}, \code{nesting}, or \code{transform};
#' and \code{emmeans} options such as \code{weights} (but please avoid
#' \code{trend} or \code{offset}.
#' 
#' 
#' @param object A supported model object (\emph{not} a reference grid)
#' @param specs Specifications for what marginal trends are desired -- as in
#'   \code{\link{emmeans}}. If \code{specs} is missing or \code{NULL},
#'   \code{emmeans} is not run and the reference grid for specified trends
#'   is returned.
#' @param var Character value giving the name of a variable with respect to 
#'   which a difference quotient of the linear predictors is computed. In order
#'   for this to be useful, \code{var} should be a numeric predictor that
#'   interacts with at least one factor in \code{specs}. Then instead of
#'   computing EMMs, we compute and compare the slopes of the \code{var} trend
#'   over levels of the specified other predictor(s). As in EMMs, marginal
#'   averages are computed for the predictors in \code{specs} and \code{by}.
#'   See also the \dQuote{Generalizations} section below.
#' @param delta.var The value of \emph{h} to use in forming the difference
#'   quotient \eqn{(f(x+h) - f(x))/h}. Changing it (especially changing its
#'   sign) may be necessary to avoid numerical problems such as logs of negative
#'   numbers. The default value is 1/1000 of the range of \code{var} over the
#'   dataset.
#' @param max.degree Integer value. The maximum degree of trends to compute (this
#'   is capped at 5). If greater than 1, an additional factor \code{degree} is
#'   added to the grid, with corresponding numerical derivatives of orders
#'   \code{1, 2, ..., max.degree} as the estimates.
#' @param ... Additional arguments passed to \code{\link{ref_grid}} or 
#'   \code{\link{emmeans}} as appropriate. See Details.
#'
#' @section Generalizations:
#' Instead of a single predictor, the user may specify some monotone function of
#' one variable, e.g., \code{var = "log(dose)"}. If so, the chain rule is
#' applied. Note that, in this example, if \code{object} contains
#' \code{log(dose)} as a predictor, we will be comparing the slopes estimated by
#' that model, whereas specifying \code{var = "dose"} would perform a
#' transformation of those slopes, making the predicted trends vary depending on
#' \code{dose}.
#' 
#' @return An \code{emmGrid} or \code{emm_list} object, according to \code{specs}.
#' See \code{\link{emmeans}} for more details on when a list is returned.
#' 
#' @seealso \code{\link{emmeans}}, \code{\link{ref_grid}}
#' 
#' @note
#' In earlier versions of \code{emtrends}, the first argument was named
#' \code{model} rather than \code{object}. (The name was changed because of
#' potential mis-matching with a \code{mode} argument, which is an option for
#' several types of models.) For backward compatibility, \code{model} still works
#' \emph{provided all arguments are named}.
#' 
#' @note
#' It is important to understand that trends computed by \code{emtrends} are
#' \emph{not} equivalent to polynomial contrasts in a parallel model where
#' \code{var} is regarded as a factor. That is because the model \code{object}
#' here is assumed to fit a smooth function of \code{var}, and the estimated
#' trends reflect \emph{local} behavior at particular value(s) of \code{var};
#' whereas when \code{var} is modeled as a factor and polynomial contrasts are
#' computed, those contrasts represent the \emph{global} pattern of changes over
#' \emph{all} levels of \code{var}. 
#' 
#' See the \code{pigs.poly} and \code{pigs.fact} examples below for an
#' illustration. The linear and quadratic trends depend on the value of 
#' \code{percent}, but the cubic trend is constant (because that is true of
#' a cubic polynomial, which is the underlying model). The cubic contrast
#' in the factorial model has the same P value as for the cubic trend,
#' again because the cubic trend is the same everywhere.
#' 
#' @export
#'
#' @examples
#' fiber.lm <- lm(strength ~ diameter*machine, data=fiber)
#' # Obtain slopes for each machine ...
#' ( fiber.emt <- emtrends(fiber.lm, "machine", var = "diameter") )
#' # ... and pairwise comparisons thereof
#' pairs(fiber.emt)
#' 
#' # Suppose we want trends relative to sqrt(diameter)...
#' emtrends(fiber.lm, ~ machine | diameter, var = "sqrt(diameter)", 
#'          at = list(diameter = c(20, 30)))
#'
#' # Obtaining a reference grid
#' mtcars.lm <- lm(mpg ~ poly(disp, degree = 2) * (factor(cyl) + factor(am)), data = mtcars)
#' 
#' # Center trends at mean disp for each no. of cylinders
#' mtcTrends.rg <- emtrends(mtcars.lm, var = "disp", 
#'                           cov.reduce = disp ~ factor(cyl))
#' summary(mtcTrends.rg)  # estimated trends at grid nodes
#' emmeans(mtcTrends.rg, "am", weights = "prop")
#' 
#' 
#' ### Higher-degree trends ...
#' 
#' pigs.poly <- lm(conc ~ poly(percent, degree = 3), data = pigs)
#' emt <- emtrends(pigs.poly, ~ degree | percent, "percent", max.degree = 3,
#'                 at = list(percent = c(9, 13.5, 18)))
#'        # note: 'degree' is an extra factor created by 'emtrends'
#'        
#' summary(emt, infer = c(TRUE, TRUE))
#' 
#' # Compare above results with poly contrasts when 'percent' is modeled as a factor ...
#' pigs.fact <- lm(conc ~ factor(percent), data = pigs)
#' emm <- emmeans(pigs.fact, "percent")
#' 
#' contrast(emm, "poly")
#' # Some P values are comparable, some aren't! See Note in documentation
emtrends = function(object, specs, var, delta.var=.001*rng,
                    max.degree = 1, ...) {
    estName = paste(var, "trend", sep=".") # Do now as I may replace var later
    
    # construct our first call to ref_grid() to get the data...
    rgargs = list(object = object, ...)
    if (is.null(rgargs$options))
        rgargs$options = list()
    
    # backward compatibility for when 1st argument was "model"
    if (missing(object) && ("model" %in% names(rgargs))) {
        names(rgargs)[names(rgargs) == "model"] = "object"
    }
    
    rgargs$options$just.data = TRUE
    data = do.call("ref_grid", c(rgargs))
    rgargs$options$just.data = rgargs$data = NULL
    
    x = data[[var]]
    fcn = NULL   # differential
    if (is.null(x)) {
        fcn = var
        var = .all.vars(as.formula(paste("~", var)))
        if (length(var) > 1)
            stop("Can only support a function of one variable")
        else {
            x = data[[var]]
            if (is.null(x)) stop("Variable '", var, "' is not in the dataset")      
        }
    }
    rng = diff(range(x))
    if (delta.var == 0)  stop("Provide a nonzero value of 'delta.var'")
    
    max.degree = max(1, min(5, as.integer(max.degree + .1)))

        # create a vector of delta values, such that a middle one has value 0
    delts = delta.var * (0:max.degree)
    idx.base = as.integer((2 + max.degree)/2)
    delts = delts - delts[idx.base]
    
    # set up call for ref_grid
    # ref_grid hook -- expand grid by these increments of var
    rgargs$options = c(rgargs$options, list(var = var, delts = delts))
    rgargs$data = data
    bigRG = do.call("ref_grid", rgargs)
    
    ### create var.subs -- list of indexes for each value of delts
    ## Since var was created as slowest-varying, only multivariate factors will vary slower
    gdim = nrow(bigRG@linfct) / length(delts)
    mdim = 1
    # added code: only consider mult levels that come AFTER var (post-processing could affect sort order)
    levnms = names(bigRG@levels)
    vi = which(levnms == var)
    for (v in bigRG@roles$multresp)
        if (which(levnms == v) > vi)
            mdim = mdim * length(bigRG@levels[[v]])
    arr = array(seq_len(nrow(bigRG@linfct)), c(gdim / mdim, length(delts), mdim))
    var.subs = lapply(seq_along(delts), function(i) as.numeric(arr[,i,]))
    
    RG = orig.rg = bigRG[var.subs[[idx.base]]]  # the subset that corresponds to reference values

    row.names(RG@grid) = seq_along(RG@grid[[1]])

    linfct = lapply(seq_along(delts), function(i) bigRG@linfct[var.subs[[i]], , drop = FALSE])
    
    if (!is.null(fcn)) { # need a different "h" when diff wrt a function
        tmp = sapply(seq_along(delts), function(i)
            eval(parse(text = fcn), envir = bigRG@grid[var.subs[[i]], , drop = FALSE]))
        if(!is.matrix(tmp)) tmp = matrix(tmp, nrow = 1)
        delta.var = apply(tmp, 1, function(.) mean(diff(.)))
    }
    
    newlf = numeric(0)
    h = 1
    for (i in 1:max.degree) { # successively difference linfct
        linfct = lapply(seq_along(linfct)[-1], function(j) linfct[[j]] - linfct[[j-1]])
        h = h * delta.var * i
        what = as.integer((length(linfct) + 1) / 2) # pick out one in the middle
        newlf = rbind(newlf, linfct[[what]] / h)
    }
    
    # Now replace linfct w/ difference quotient(s)
    RG@linfct = newlf
    RG@roles$trend = var
    
    if (max.degree > 1) {
        degnms = c("linear", "quadratic", "cubic", "quartic", "quintic")
        RG@grid$degree = degnms[1]
        g = RG@grid
        for (j in 2:max.degree) {
            g$degree = degnms[j]
            RG@grid = rbind(RG@grid, g)
        }
        RG@roles$predictors = c(RG@roles$predictors, "degree")
        RG@levels$degree = degnms[1:max.degree]
    }
    RG@grid$.offset. = NULL   # offset never applies after differencing
    RG@misc$tran = RG@misc$tran.mult = NULL
    RG@misc$estName = estName
    RG@misc$methDesc = "emtrends"
    
    .save.ref_grid(RG)  # save in .Last.ref_grid, if enabled
    
    if (missing(specs) || is.null(specs))
        return (RG)
    
    # args for emmeans calls
    args = list(object = NULL, specs = specs, ...)
    args$at = args$cov.reduce = args$mult.levs = args$vcov. = args$data = args$trend =
              args$regrid = args$tran = NULL
    if(max.degree > 1) {
        chk = union(all.vars(specs), args$by)
        if (!("degree" %in% chk))
            args$by = c("degree", args$by)
    }
    
    
    args$object = RG
    do.call("emmeans", args)
}

context("Emtrends function")

pigs.lm = lm(log(conc) ~ source * poly(percent, 3), data = pigs)

test_that("emtrends works", {
    emt = emtrends(pigs.lm, ~ source, "percent")
    expect_equal(nrow(semt <- summary(emt)), 3)
    expect_equal(semt$percent.trend[1], .00429, tol = 0.0001)
    
    emtt = emtrends(pigs.lm, ~ source, "sqrt(percent)")
    expect_equal(summary(emtt)[["sqrt(percent).trend"]][1], .0309, tol = 0.001)
    
    emtp = emtrends(pigs.lm, ~ source, "percent", max.degree = 3)
    expect_equal(nrow(semtp <- summary(emtp)), 9)
    expect_equal(semtp$percent.trend[7], .001337, tol = 0.0001)
    
    emtpa = emtrends(pigs.lm, ~ source | percent, "percent", max.degree = 2,
                     at = list(percent = c(9,12,15,18)))
    expect_equal(nrow(summary(emtpa)), 24) # 3 sources * 2 degrees * 4 percents
})

##############################################################################
#    Copyright (c) 2012-2022 Russell V. Lenth                                #
#                                                                            #
#    This file is part of the emmeans package for R (*emmeans*)              #
#                                                                            #
#    *emmeans* is free software: you can redistribute it and/or modify       #
#    it under the terms of the GNU General Public License as published by    #
#    the Free Software Foundation, either version 2 of the License, or       #
#    (at your option) any later version.                                     #
#                                                                            #
#    *emmeans* is distributed in the hope that it will be useful,            #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
#    GNU General Public License for more details.                            #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with R and *emmeans*.  If not, see                                #
#    <https://www.r-project.org/Licenses/> and/or                            #
#    <http://www.gnu.org/licenses/>.                                         #
##############################################################################

### Support for 'gam' objects
# Note: This is a mess, because both packages 'gam' and 'mgcv' produce these,
# and they are different. Both inherit from glm and lm, though, so recover_data.lm
# still serves for these (I hope)


# gam::Gam objects...

# We have one args:
#   nboot   # of bootstrap reps to get variances of smooths
#' @exportS3Method emm_basis Gam
emm_basis.Gam = function(object, trms, xlev, grid, nboot = 800, ...) {
    result = emm_basis.lm(object, trms, xlev, grid, ...)
    old.smooth = object$smooth
    if (is.null(old.smooth))  # "just an ordinary glm" (My Fair Lady)
        return(result)
    # else we need to add-in some smoothers
    smooth.frame = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
    data = object$smooth.frame
    labs = names(data)
    w = object$weights
    resid = object$residuals
    for (i in seq_along(labs)) {
        lab = labs[i]
        sig = apply(smooth.frame[, i, drop=FALSE], 1, paste, collapse = ":")
        usig = unique(sig)
        rows = lapply(usig, function(s) which(sig == s))
        xeval = smooth.frame[sapply(rows, "[", 1), lab]
        bsel = matrix(0, nrow = length(sig), ncol = length(usig))
        for (j in seq_along(rows))
            bsel[rows[[j]], j] = 1
        
        cl = attr(data[[i]], "call")
        cl$xeval = substitute(xeval)
        z = resid + old.smooth[, lab]
        bh = as.numeric(eval(cl))
        m = length(bh)
        n = length(result$bhat)
        result$bhat = c(result$bhat, bh)
        result$X = cbind(result$X, bsel)
        boot = replicate(nboot, {
                z = sample(resid, replace = TRUE) + old.smooth[, lab]
                as.numeric(eval(cl))
            })
        covar = if(m == 1) var(boot) 
                else       cov(t(boot))
        result$V = rbind(cbind(result$V, matrix(0, nrow = n, ncol = m)),
                         cbind(matrix(0, nrow = m, ncol = n), covar))
    }
    result
}


### This addition contributed by Hannes Riebl (#303)
.emm_basis.gam_multinom = function(object, trms, xlev, grid,
                                   freq = FALSE, unconditional = FALSE,
                                   mode = c("prob", "latent"), ...) {
    mode = match.arg(mode)

    X = mgcv::predict.gam(object, newdata = grid, type = "lpmatrix",
                          newdata.guaranteed = TRUE)

    k = length(attr(X, "lpi"))
    nbhat = vapply(attr(X, "lpi"), length, FUN.VALUE = integer(1))
    pat = (rbind(0, diag(k + 1, k)) - 1) / (k + 1)

    X = apply(pat, 1, function(row) {
        y = rep.int(row, times = nbhat)
        out = apply(X, 1, "*", y = y, simplify = FALSE)
        do.call(rbind, out)
    }, simplify = FALSE)

    X = do.call(rbind, X)

    bhat = as.numeric(coef(object))
    V = .my.vcov(object, freq = freq, unconditional = unconditional, ...)
    nbasis = kronecker(rep.int(1, times = k), estimability::all.estble)

    dfargs = list(df = sum(object$edf))
    dffun = function(k, dfargs) dfargs$df

    misc = list(tran = "log", inv.lbl = "e^y")

    ylevs = list(class = seq.int(0, k))
    names(ylevs) = as.character(object$formula[[1]][[2]])
    misc$ylevs = ylevs

    if (mode == "prob") {
        misc$postGridHook = .multinom.postGrid
    }

    list(X = X, bhat = bhat, nbasis = nbasis, V = V, dffun = dffun,
         dfargs = dfargs, misc = misc)
}

# Local utility for identifying random smooths
.smooth.is.random = function(s) {
    rcls = c("random.effect", "fs.interaction") ### what to look for
    cls = c(class(s), unname(unlist(lapply(s$margin, class))))
    any(cls %in% rcls)
}

### for mgcv::gam objects
### Many thanks to Maarten Jung for help on sorting-out fixed and random effects
#' @exportS3Method recover_data gam
recover_data.gam = function(object, ...) {
    if (length(object$smooth) > 0) { # get rid of random terms
        fixnm = unlist(lapply(object$smooth, function(s) {
                              if(.smooth.is.random(s)) ""
                              else c(s$term, s$by)
            }))
        fixnm = union(.all.vars(delete.response(object$pterms)), fixnm)
        fixnm = setdiff(fixnm, c("1", "", "NA"))
        object$terms = terms(.reformulate(fixnm, env = environment(terms(object))))
    }
    recover_data.lm(object, ...)
}


### emm_basis method for mgcv::gam objects
### extra arg `unconditional` and `freq` as in `vcov.gam`
#' @exportS3Method emm_basis gam          
emm_basis.gam = function(object, trms, xlev, grid,
                         freq = FALSE, unconditional = FALSE,
                         what = c("location", "scale", "shape", "rate", "prob.gt.0"), 
                         ...) {
    if (length(object$smooth) > 0) { # get rid of random terms 
        rand = sapply(object$smooth, function(s) {ifelse(.smooth.is.random(s), s$label, NA)})
        rand = if (all(is.na(rand))) NULL else rand[!is.na(rand)]
    }
    else
        rand = NULL
    X = mgcv::predict.gam(object, newdata = grid, type = "lpmatrix",
                          exclude = rand, newdata.guaranteed = TRUE)
    keep = if (is.null(rand)) rep(TRUE, ncol(X)) else apply(X, 2, function(x) !all(x == 0))
    X = X[, keep, drop = FALSE]
    bhat = as.numeric(object$coefficients[keep])
    V = .my.vcov(object, freq = freq, unconditional = unconditional, ...)[keep, keep]

    fam_name = object$family$family
    what_num = what
    
    if (fam_name == "multinom") {
        return(.emm_basis.gam_multinom(object, trms, xlev, grid, freq,
                                       unconditional, ...))
    }
    else if (fam_name == "mvn") {
        if (!is.numeric(what)) {
            stop("Family 'mvn' requires a numeric argument 'what'")
        }
    } 
    else if (is.character(what)) {
        what = match.arg(what)
        if (fam_name == "ziplss") {
            what_num = switch(what, location = 1, rate = 1, prob.gt.0 = 2)
        } 
        else {
            what_num = switch(what, location = 1, scale = 2, shape = 3)
        }
    }
    
    select = attr(X, "lpi")
    if (is.null(select)) select = list(seq_along(bhat))
    select = try(select[[what_num]], silent = TRUE)
    
    if (inherits(select, "try-error")) {
        stop("Model does not have a linear predictor 'what = ", what, "'")
    }
    
    bhat = bhat[select]
    X = X[, select, drop = FALSE]
    V = V[select, select, drop = FALSE]
    
    nbasis = estimability::all.estble
    link = object$family$link[what_num]
    if(link == "identity") # they may be lying
        link = switch(fam_name,
                      ocat = "logit",
                      ziP = "log",
                      cox.ph = "log", ## ???
                      ziplss = c("log", "cloglog")[what_num],
                      gevlss = c("identity", "log", "logit")[what_num],
                      "identity")
    misc = .std.link.labels(list(link = link, family = fam_name), list())
    
    if (!is.null(misc$tran) && misc$tran == "logb")  # the way this is documented is truly bizarre but I think this is right
        misc$tran = make.tran("genlog", - environment(object$family$linfo[[what_num]]$linkfun)$b)

    dfargs = list(df = object$df.residual)
    dffun = function(k, dfargs) dfargs$df
    list(X=X, bhat=bhat, nbasis=nbasis, V=V, dffun=dffun, dfargs=dfargs, misc=misc)
}


### mgcv::gamm objects...
#' @exportS3Method recover_data gamm
recover_data.gamm = function(object, data = NULL, call = object$gam$call, ...) {
    gam = object$gam
    class(gam) = c("gam", "glm", "lm")
    if (!is.null(data)) {
        gam$call = quote(gamm())
        return(recover_data(gam, data = data, ...))
    }
    else {
        if (is.null(call))
            return("Must supply either 'data' or 'call' with gamm objects")
        gam$call = call
        recover_data(gam, ...)
    }
}

#' @exportS3Method emm_basis gamm         
emm_basis.gamm = function(object, ...)
    emm_basis(object$gam, ...)


###===================================================================
# Support for gamlss objects
# 'what' parameter mimics predict.gamlss

#' @exportS3Method recover_data gamlss
recover_data.gamlss = function(object, what = c("mu", "sigma", "nu", "tau"), ...) {
    fcall = object$call
    what = match.arg(what)
    trms = terms(formula(object, what = what))
    recover_data(fcall, delete.response(trms), object$na.action, ...)
}

#' @exportS3Method emm_basis gamlss       
emm_basis.gamlss = function(object, trms, xlev, grid, 
                            what = c("mu", "sigma", "nu", "tau"), vcov., ...) {
    what = match.arg(what)
    smo.mat = object[[paste0(what, ".s")]]
    if (!is.null(smo.mat))
        stop("gamlss models with smoothing are not yet supported in 'emmeans'",
             call. = NULL)
    
    object$coefficients = object[[paste0(what, ".coefficients")]]
    if (missing(vcov.)) {
        # tedious code to pull needed vcov elements
        # Gotta do this before messing up the object
        V = suppressWarnings(vcov(object))
        len = sapply(object$parameters, function(p) length(object[[paste0(p, ".coefficients")]]))
        before = which(object$parameters == what) - 1
        if (before > 0)  before = sum(len[seq_len(before)])
        idx = before + seq_along(object$coefficients)
        vcov. = V[idx, idx, drop = FALSE]
    }
    if (!is.null(link <- object[[paste0(what, ".link")]])) {
        # Decide whether to use d.f. or not
        use.df = c("BCCG", "BCPE", "BCT", "GA", "GT", "NO", "NOF", "TF")
        fam = ifelse((what == "mu") && (object$family[1] %in% use.df), 
                     "gaussian", "other")
        object$family = list(family = fam, link = link)
    }
    ###object$qr = object[[paste0(what, ".qr")]]
    ###NextMethod("emm_basis", vcov. = vcov., ...)
    emm_basis.lm(object, trms, xlev, grid, vcov. = vcov., ...)
}

---
title: "Prediction in emmeans"
author: "emmeans package, Version `r packageVersion('emmeans')`"
output: emmeans::.emm_vignette
vignette: >
  %\VignetteIndexEntry{Prediction in emmeans}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
```{r, echo = FALSE, results = "hide", message = FALSE} 
require("emmeans") 
options(show.signif.stars = FALSE, width = 100) 
knitr::opts_chunk$set(fig.width = 4.5, class.output = "ro") 
```

<!-- @index Vignettes!Predictions -->
In this vignette, we discuss **emmeans**'s rudimentary capabilities for constructing prediction intervals. 

## Contents {#contents}

 1. [Focus on reference grids](#ref-grid)
 2. [Need for an SD estimate](#sd-estimate)
 3. [Feedlot example](#feedlot)
 4. [Predictions on particular strata](#strata)
 5. [Predictions with Bayesian models](#bayes)

[Index of all vignette topics](vignette-topics.html) 

##  Focus on reference grids {#ref-grid}
<!-- @index Predictions!Reference grid; Reference grids!Prediction on -->
Prediction is not the central purpose of the **emmeans** package. Even its name
refers to the idea of obtaining marginal averages of fitted values; and it is a
rare situation where one would want to make a prediction of the average of
several observations. We can certainly do that if it is truly desired,
but almost always, predictions should be based on the reference grid itself 
(i.e., *not* the result of an `emmeans()` call),
inasmuch as a reference grid comprises combinations of model predictors. 


## Need for an SD estimate {#sd-estimate}
<!-- @index Predictions!Error SD; -->
A prediction interval requires an estimate of the error standard deviation,
because we need to account for both the uncertainty of our point predictions
and the uncertainty of outcomes centered on those estimates. By its current design,
we save the value (if any) returned by `stats::sigma(object)` when a reference grid is constructed for a model `object`. Not all models provide a `sigma()` method,
in which case an error is thrown if the error SD is not manually specified.
Also, in many cases, there may be a `sigma()` method, but it does not
return the appropriate value(s) in the context of the needed predictions.
(In an object returned by `lme4::glmer(), for example, `sigma()` seems to always 
returns 1.0.) Indeed, as will be seen in the example that follows,
one usually needs to construct a manual SD estimate when the model is a mixed-effects model.

So it is essentially always important to think very specifically about
whether we are using an appropriate value. You may check the value being
assumed by looking at the `misc` slot in the reference grid:
```{r eval = FALSE}
rg <- ref_grid(model)
rg@misc$sigma
```

Finally, `sigma` may be a vector, as long as it is conformable with the estimates
in the reference grid. This would be appropriate, for example, with a model
fitted by `nlme::gls()` with some kind of non-homogeneous error structure. It may take
some effort, as well as a clear understanding of the model and its structure, to
obtain suitable SD estimates. It was suggested to me that the function
`insight::get_variance()` may be helpful -- especially when working with
an unfamiliar model class. Personally, I prefer to make sure I understand the 
structure of the model object and/or its summary to ensure I am not going astray.

[Back to Contents](#contents)

## Feedlot example {#feedlot}
<!-- @index Examples!`feedlot`; Predictions!Total SD; Predictions!graphics -->
To illustrate, consider the `feedlot` dataset provided with the package.
Here we have several herds of feeder cattle that are sent to feed lots and given
one of three diets. The weights of the cattle are measured at time of entry
(`ewt`) and at time of slaughter (`swt`). Different herds have possibly
different entry weights, based on breed and ranching practices, so we will
center each herd's `ewt` measurements, then use that as a covariate in a mixed
model:
```{r, message = FALSE}
feedlot = transform(feedlot, adj.ewt = ewt - predict(lm(ewt ~ herd)))
require(lme4)
feedlot.lmer <- lmer(swt ~ adj.ewt + diet + (1|herd), data = feedlot)
feedlot.rg <- ref_grid(feedlot.lmer, at = list(adj.ewt = 0))
summary(feedlot.rg)  ## point predictions
```

Now, as advised, let's look at the SDs involved in this model:
```{r}
lme4::VarCorr(feedlot.lmer)  ## for the model
feedlot.rg@misc$sigma  ## default in the ref. grid
```
So the residual SD will be assumed in our prediction intervals 
if we don't specify something else. And we *do* want something else,
because in order to predict the slaughter weight of an arbitrary animal,
without regard to its herd, we need to account for the variation among
herds too, which is seen to be considerable. The two SDs reported by `VarCorr()`
are assumed to represent independent sources of variation, so they may be
combined into a total SD using the Pythagorean Theorem.
We will update the reference grid with the new value:
```{r}
feedlot.rg <- update(feedlot.rg, sigma = sqrt(77.087^2 + 57.832^2))
```

We are now ready to form prediction intervals. To do so, simply call the 
`predict()` function with an `interval` argument:
```{r}
predict(feedlot.rg, interval = "prediction")
```
These results may also be displayed graphically:
```{r, fig.height = 2, fig.alt = "Side-by-side CIs and PIs. The PIs are much wider, and have the endpoints found in the preceding predit() call"}
plot(feedlot.rg, PIs = TRUE)
```

The inner intervals are confidence intervals, and the outer ones are the 
prediction intervals.

Note that the SEs for prediction are considerably greater than the SEs for
estimation in the original summary of `feedlot.rg`. Also, as a sanity check,
observe that these
prediction intervals cover about the same ground as the original data:
```{r}
range(feedlot$swt)
```

By the way, we could have specified the desired `sigma` value as an additional
`sigma` argument in the `predict()` call, rather than updating the `feedlot.rg`
object.

[Back to Contents](#contents)


## Predictions on particular strata {#strata}
<!-- @index Predictions!on Particular strata -->
Suppose, in our example, we want to predict `swt` for one or more particular herds.
Then the total SD we computed is not appropriate for that purpose, because that includes variation among herds.

But more to the point, if we are talking about particular herds, then we are really regarding `herd` as a fixed effect of interest; so the expedient thing to do is
to fit a different model where `herd` is a fixed effect:
```{r}
feedlot.lm <- lm(swt ~ adj.ewt + diet + herd, data = feedlot)
```
So to predict slaughter weight for herds `9` and `19`:
```{r}
newrg <- ref_grid(feedlot.lm, at = list(adj.ewt = 0, herd = c("9", "19")))
predict(newrg, interval = "prediction", by = "herd")
```
This is an instance where the default `sigma` was already correct (being the only error SD we have available). The SD value is comparable to the residual SD in the previous model, and the prediction SEs are smaller than those for predicting over all herds.

[Back to Contents](#contents)

## Predictions with Bayesian models {#bayes}
<!-- @index Predictions!Bayesian models -->
For models fitted using Bayesian methods, these kinds of prediction intervals are
available only by forcing a frequentist analysis (`frequentist = TRUE`).

However, a better and more flexible approach with Bayesian models is to simulate
observations from the posterior predictive distribution. This is done via `as.mcmc()` and specifying a `likelihood` argument. An example is given in the ["sophisticated models" vignette](sophisticated.html#predict-mcmc).

[Back to Contents](#contents)

[Index of all vignette topics](vignette-topics.html) 

---
title: "Index of vignette topics"
author: "emmeans package, Version `r packageVersion('emmeans')`"
output: emmeans::.emm_vignette
vignette: >
  %\VignetteIndexEntry{Index of vignette topics}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<div class="vigindex" id="Index follows this line">



### Jump to:  [A](#a) [B](#b) [C](#c) [D](#d) [E](#e) [F](#f) [G](#g) [H](#h) [I](#i) [J](#j) [K](#k) [L](#l) [M](#m) [N](#n) [O](#o) [P](#p) [Q](#q) [R](#r) [S](#s) [T](#t) [U](#u) [V](#v) [W](#w) [X](#x) [Z](#z)  {#topnav}




### A {#a}


  * [`add_grouping()`](utilities.html#groups)
  * [`addl.vars`](sophisticated.html#random-slopes)
  * `adjust`
      * [in *comparisons: pairwise*](comparisons.html#pairwise)
      * [in *confidence-intervals: adjust*](confidence-intervals.html#adjust)
      * [vs. `cross.adjust`](confidence-intervals.html#cross-adjust)
  * [Adjusted means](basics.html#RG)
  * [Adjusted R-squared](basics.html#pigs)
  * [`afex_aov` objects](models.html#V)
  * [Alias matrix](xplanations.html#submodels)
  * [Analysis of subsets of data](FAQs.html#model)
  * Analysis of variance
      * [versus *post hoc* comparisons](FAQs.html#anova)
      * [Type III](confidence-intervals.html#joint_tests)
  * [`aovList` objects](models.html#V)
  * [`appx-satterthwaite` method](models.html#K)
  * Arguments
      * [`...`](basics.html#arguments)
      * [Passed to `ref_grid()`](basics.html#arguments)
  * `as.mcmc()`
      * [in *models: S*](models.html#S)
      * [in *sophisticated: bayesxtra*](sophisticated.html#bayesxtra)
  * [ASA Statement on *P* values](basics.html#pvalues)
  * [Asymptotic tests](sophisticated.html#dfoptions)
  * [ATOM](basics.html#pvalues)
  * [`averaging` models](models.html#I)


[Back to top](#topnav)

### B {#b}


  * [Back-transforming](basics.html#transf)
  * [Balanced allocation](basics.html#exper)
  * [Bayes factor](sophisticated.html#bayesxtra)
  * Bayesian models
      * [in *models: S*](models.html#S)
      * [in *sophisticated: mcmc*](sophisticated.html#mcmc)
  * [**bayesplot** package](sophisticated.html#bayesxtra)
  * [**bayestestR** package](sophisticated.html#bayesxtra)
  * [Beta regression](models.html#B)
  * [`betareg` models](models.html#B)
  * Bias adjustment
      * [For link functions vs. response transformations](transformations.html#link-bias)
      * [in Bayesian models](sophisticated.html#bias-adj-mcmc)
      * [In GLMMs and GEE models](transformations.html#cbpp)
      * [When back-transforming](transformations.html#bias-adj)
      * [When *not* to use](transformations.html#insects)
      * [with random slopes](sophisticated.html#random-slopes)
  * [Bonferroni adjustment](confidence-intervals.html#adjmore)
  * [`boot-satterthwaite` method](models.html#K)
  * [Brackets (`[ ]` and `[[ ]]` operators)](utilities.html#brackets)
  * [`brmsfit` objects](models.html#S)
  * [`by` groups](confidence-intervals.html#byvars)
      * [Adjusting across groups](confidence-intervals.html#cross-adjust)
      * [Identical comparisons](FAQs.html#additive)


[Back to top](#topnav)

### C {#c}


  * [Causal inference](messy-data.html#counterfact)
  * [`cld()`](comparisons.html#CLD)
  * [`clm` models](models.html#O)
  * [**coda** package](sophisticated.html#bayesxtra)
  * [`coef()`](comparisons.html#contrasts)
  * [Cohen's *d*](comparisons.html#pwpm)
  * [Compact letter displays](comparisons.html#CLD)
  * Comparison arrows
      * [Derivation](xplanations.html#arrows)
  * Comparisons
      * [Back-transforming](comparisons.html#logs)
      * [Displaying as groups](comparisons.html#CLD)
      * [Displaying *P* values](comparisons.html#pwpp)
      * [Graphical](comparisons.html#graphical)
          * [How arrows are determined](xplanations.html#arrows)
      * [with logs](comparisons.html#logs)
      * [Obtaining differences rather than ratios](FAQs.html#ratios)
      * [with overlapping CIs](FAQs.html#CIerror)
  * [Comparisons result in `(nothing)`](FAQs.html#nopairs)
  * [Confidence intervals](confidence-intervals.html#summary)
      * [Overlapping](FAQs.html#CIerror)
  * [`confint()`](confidence-intervals.html#summary)
  * [Confounded effects](xplanations.html#joint_tests)
  * [Confounding](basics.html#emms)
  * [`consec` contrasts](comparisons.html#contrasts)
  * [Constrained marginal means](xplanations.html#submodels)
  * [Consultants](basics.html#recs3)
  * [Containment d.f.](models.html#K)
  * [`contrast()`](comparisons.html#contrasts)
      * [`adjust`](comparisons.html#linfcns)
      * [Changing defaults](utilities.html#defaults)
      * [`combine`](interactions.html#simple)
      * [`interaction`](interactions.html#contrasts)
      * [Linear functions](comparisons.html#linfct)
      * `simple`
          * [in *confidence-intervals: simple*](confidence-intervals.html#simple)
          * [in *interactions: simple*](interactions.html#simple)
  * Contrasts
      * [of other contrasts](interactions.html#contrasts)
      * [Custom](comparisons.html#linfcns)
      * [Formula](comparisons.html#formulas)
      * [Multivariate](interactions.html#multiv)
      * [Normalized](comparisons.html#nrmlze)
      * [Pairwise](comparisons.html#pairwise)
      * [Polynomial](comparisons.html#contrasts)
      * Tests of
          * [with transformations](comparisons.html#logs)
  * [Controlled experiments](basics.html#exper)
  * [Count regression](models.html#C)
  * [Counterfactuals](messy-data.html#counterfact)
  * [`cov.reduce`](messy-data.html#med.covred)
  * Covariates
      * [Adjusted](messy-data.html#adjcov)
      * [`cov.keep`](FAQs.html#numeric)
      * [`cov.reduce`](FAQs.html#numeric)
      * [Derived](basics.html#depcovs)
      * [`emmeans()` doesn't work](FAQs.html#numeric)
      * [Interacting with factors](FAQs.html#trends)
      * [Mediating](messy-data.html#mediators)
  * [cross-group comparisons](confidence-intervals.html#cross-adjust)
  * [`cross.adjust`](confidence-intervals.html#cross-adjust)


[Back to top](#topnav)

### D {#d}


  * [Degrees of freedom](sophisticated.html#dfoptions)
      * [Infinite](FAQs.html#asymp)
  * Digits
      * [Displaying more digits](utilities.html#data)
  * [Digits, optimizing](utilities.html#digits)
  * [Dunnett method](comparisons.html#contrasts)


[Back to top](#topnav)

### E {#e}


  * [`eff` contrasts](comparisons.html#contrasts)
  * [`eff_size()`](comparisons.html#pwpm)
  * [Effect size](comparisons.html#pwpm)
  * [`emm_basis()`](xtending.html#intro)
      * [Arguments and returned value](xtending.html#ebreqs)
      * [Communicating with `recover_data()`](xtending.html#communic)
      * [Dispatching](xtending.html#dispatch)
      * [Hook functions](xtending.html#hooks)
      * [for `lqs` objects](xtending.html#eblqs)
      * [Re-gridded](xtending.html#regridded)
      * [for `rsm` objects](xtending.html#ebrsm)
  * [`emm_list` object](comparisons.html#formulas)
  * [`emm_options()`](utilities.html#defaults)
  * [`.emmc` functions](comparisons.html#linfcns)
  * **emmeans** package
      * [Exporting extensions to](xtending.html#exporting)
      * [Quick-start guide](AQuickStart.html)
  * [`emmeans()`](basics.html#emms)
      * [And the underlying model](FAQs.html#nowelch)
      * [Arguments passed to `ref_grid()`](basics.html#arguments)
      * [Changing defaults](utilities.html#defaults)
      * [Fastest way to get wrong answers](FAQs.html#fastest)
      * [Surprising results from](FAQs.html#transformations)
      * `weights`
          * [in *basics: weights*](basics.html#weights)
          * [in *messy-data: weights*](messy-data.html#weights)
      * [With transformations](transformations.html#regrid)
  * [`emmGrid` objects](basics.html#emmobj)
      * [Accessing data](utilities.html#data)
      * [Combining and subsetting](utilities.html#rbind)
      * [Modifying](utilities.html#update)
      * [Setting defaults for](utilities.html#defaults)
  * `emmip()`
      * [abbreviated labels](messy-data.html#cows)
      * [in *basics: emmip*](basics.html#emmip)
      * [in *basics: plots*](basics.html#plots)
      * [in *interactions: factors*](interactions.html#factors)
      * [nested factors](messy-data.html#cows)
  * EMMs
      * [Projecting to a submodel](xplanations.html#submodels)
      * [What are they?](FAQs.html#what)
  * `emtrends()`
      * [in *interactions: covariates*](interactions.html#covariates)
      * [in *interactions: oranges*](interactions.html#oranges)
  * [`estHook`](xtending.html#hooks)
  * [Estimability](messy-data.html#nonest)
  * [Estimability issues](FAQs.html#NAs)
  * [Estimable functions](xplanations.html#joint_tests)
  * Estimated marginal means
      * [Definition of](basics.html#refgrid)
      * [Demonstrated](basics.html#emms)
      * [Manually computed](basics.html#refgrid)
      * [Reference grid](basics.html#emms)
      * [vs. ordinary marginal means](basics.html#emms)
  * Examples
      * [`auto.noise`](interactions.html#factors)
      * [Bayesian model](sophisticated.html#mcmc)
      * `cbpp`
          * [in *sophisticated: mcmc*](sophisticated.html#mcmc)
          * [in *transformations: cbpp*](transformations.html#cbpp)
      * [`ChickWeight`](sophisticated.html#random-slopes)
      * [`cows`](messy-data.html#cows)
      * [`feedlot`](predictions.html#feedlot)
      * `fiber`
          * [in *interactions: covariates*](interactions.html#covariates)
          * [in *transformations: stdize*](transformations.html#stdize)
      * [`framing`](messy-data.html#mediators)
      * [Gamma regression](transformations.html#tranlink)
      * [`InsectSprays`](transformations.html#insects)
      * [`Insurance`](xplanations.html#offsets)
      * [Insurance claims (SAS)](sophisticated.html#offsets)
      * [Logistic regression](transformations.html#links)
      * [`lqs` objects](xtending.html#lqs)
      * [`MOats`](basics.html#multiv)
      * `mtcars`
          * [in *basics: depcovs*](basics.html#depcovs)
          * [in *messy-data: nuis.example*](messy-data.html#nuis.example)
      * [Multivariate](basics.html#multiv)
      * [Nested fixed effects](messy-data.html#cows)
      * `neuralgia`
          * [`counterfactuals`](messy-data.html#neuralgia)
          * [in *transformations: links*](transformations.html#links)
          * [in *transformations: trangraph*](transformations.html#trangraph)
      * [`nutrition`](messy-data.html#nutrex)
          * [`submodel`](messy-data.html#submodels)
          * [`weights`](messy-data.html#weights)
      * [`Oats`](sophisticated.html#lmer)
      * `oranges`
          * [in *comparisons: formulas*](comparisons.html#formulas)
          * [in *interactions: oranges*](interactions.html#oranges)
      * [Ordinal model](sophisticated.html#ordinal)
      * `pigs`
          * [in *basics: pigs*](basics.html#pigs)
          * [in *confidence-intervals: summary*](confidence-intervals.html#summary)
          * [in *transformations: altscale*](transformations.html#altscale)
          * [in *transformations: overview*](transformations.html#overview)
          * [in *transformations: pigs-biasadj*](transformations.html#pigs-biasadj)
      * [`rlm` objects](xtending.html#rlm)
      * [Robust regression](xtending.html#rlm)
      * [Split-plot experiment](sophisticated.html#lmer)
      * [Unbalanced data](basics.html#pigs)
      * `warpbreaks`
          * [in *confidence-intervals: cross-adjust*](confidence-intervals.html#cross-adjust)
          * [in *transformations: tranlink*](transformations.html#tranlink)
          * [in *utilities: relevel*](utilities.html#relevel)
      * [Welch's *t* comparisons](utilities.html#relevel)
      * [`wine`](sophisticated.html#ordinal)
  * Expected marginal means
      * [Foundations of](basics.html#found)
      * [Reliance on a model](basics.html#models)
  * [Experimental versus observational data](basics.html#exper)
  * [Exporting output](basics.html#formatting)
  * Extending **emmeans**
      * [Exports useful to developers](xtending.html#exported)
      * [Restrictions](xtending.html#dispatch)


[Back to top](#topnav)

### F {#f}


  * *F* test
      * [vs. pairwise comparisons](FAQs.html#anova)
      * [Role in *post hoc* tests](basics.html#recs1)
  * Factors
      * [Mediating](messy-data.html#weights)
  * [Formatting results](basics.html#formatting)
  * [Foundations of EMMs](basics.html#found)
  * [Frequently asked questions](FAQs.html)


[Back to top](#topnav)

### G {#g}


  * [G-Computation](messy-data.html#counterfact)
  * [`gam` models](models.html#G)
  * [`gamlss` models](models.html#H)
  * [GEE models](models.html#E)
  * [Generalized additive models](models.html#G)
  * [Generalized linear models](models.html#G)
  * [Geometric means](transformations.html#bias-adj)
  * [Get the model right first](FAQs.html#nowelch)
  * [`get_emm_option()`](utilities.html#options)
  * **ggplot2** package
      * [in *basics: ggplot*](basics.html#ggplot)
      * [in *messy-data: cows*](messy-data.html#cows)
  * [GIGO (garbage in, garbage out)](basics.html#models)
  * [`glm`*xxx* models](models.html#G)
  * [`gls` models](models.html#K)
  * [Graphical displays](basics.html#plots)
  * [Grouping factors](utilities.html#groups)
  * [Grouping into separate sets](confidence-intervals.html#byvars)


[Back to top](#topnav)

### H {#h}


  * [Hook functions](xtending.html#hooks)
  * [Hotelling's $T^2$](interactions.html#multiv)
  * [`hpd.summary()`](sophisticated.html#mcmc)
  * [`hurdle` models](models.html#C)


[Back to top](#topnav)

### I {#i}


  * [Indicator variables](basics.html#RG)
  * [Infinite degrees of freedom](FAQs.html#asymp)
  * [Interaction-style plots](basics.html#emmip)
  * Interactions
      * [Analysis](interactions.html)
      * [Contrasts](interactions.html#contrasts)
      * [Covariate with factors](interactions.html#covariates)
      * [Implied](interactions.html#oranges)
      * [Plotting](interactions.html#factors)
      * [Possible inappropriateness of marginal means](interactions.html#factors)
  * [`inverse()`](basics.html#pigs)


[Back to top](#topnav)

### J {#j}


  * [`joint`](confidence-intervals.html#joint)
  * `joint_tests()`
      * [in *confidence-intervals: joint_tests*](confidence-intervals.html#joint_tests)
      * [Confounded effects](xplanations.html#joint_tests)
      * [in *interactions: contrasts*](interactions.html#contrasts)
      * [with `submodel = "type2"`](messy-data.html#type2submodel)


[Back to top](#topnav)

### K {#k}


  * [`kable`](basics.html#formatting)
  * [Kenward-Roger d.f.](models.html#L)


[Back to top](#topnav)

### L {#l}


  * Labels
      * [Changing](utilities.html#relevel)
  * [Large models](messy-data.html#nuisance)
  * [Latin squares](basics.html#exper)
  * [Least-squares means](FAQs.html#what)
  * Levels
      * [Changing](utilities.html#relevel)
  * [Linear functions](comparisons.html#linfct)
  * [Link functions](transformations.html#links)
  * [`lme` models](models.html#K)
  * `lmerMod` models
      * [in *models: L*](models.html#L)
      * [in *sophisticated: lmer*](sophisticated.html#lmer)
      * [System options for](sophisticated.html#lmerOpts)
  * Logistic regression
      * [Odds ratios](transformations.html#oddsrats)
      * [Risk ratios](transformations.html#riskrats)
      * [Surprising results](FAQs.html#transformations)
  * Logistic-like regression
      * [Non-logit links](transformations.html#not-logit)
  * LSD
      * [protected](basics.html#recs2)


[Back to top](#topnav)

### M {#m}


  * [`make.tran()`](transformations.html#special)
  * [`mcmc` objects](models.html#S)
  * Means
      * [Generalized](transformations.html#bias-adj)
      * [Weighted](basics.html#weights)
  * [Mediating covariates](messy-data.html#mediators)
  * [Memory usage](messy-data.html#nuisance)
      * [Limiting](messy-data.html#rg.limit)
  * [`mira` models](models.html#I)
  * [`misc` attribute and argument](xtending.html#communic)
  * [`misc$regrid.flag`](xtending.html#regridded)
  * [Missing cells](xplanations.html#joint_tests)
  * [`mlm` models](models.html#N)
  * [`mmer` models](models.html#G)
  * `mode` argument
      * [implied re-gridding](transformations.html#regrid3)
  * Model
      * [Get it right first](basics.html#recs2)
      * Importance of
          * [in *basics: models*](basics.html#models)
          * [in *FAQs: fastest*](FAQs.html#fastest)
      * [Importance of getting it right](FAQs.html#nowelch)
  * [Model averaging](models.html#I)
  * Modeling
      * [`pigs` example](basics.html#pigs)
      * [Response transformation](basics.html#pigs)
  * [Modelling](basics.html#models)
  * Models
      * [Constrained](messy-data.html#submodels)
      * [Large](messy-data.html#nuisance)
      * [Quick reference](models.html#quickref)
      * [Unsupported](FAQs.html#qdrg)
  * [Multi-factor studies](FAQs.html#interactions)
  * [Multinomial models](models.html#N)
  * [Multiple imputation](models.html#I)
  * [Multiplicity adjustments](confidence-intervals.html#adjust)
  * [Multivariate contrasts](interactions.html#multiv)
  * Multivariate models
      * [in *basics: multiv*](basics.html#multiv)
      * [in *interactions: oranges*](interactions.html#oranges)
      * [in *models: M*](models.html#M)
      * [with `submodel`](xplanations.html#mult.submodel)
  * [Multivariate *t* (`"mvt"`) adjustment](confidence-intervals.html#adjmore)
  * [`mvcontrast()`](interactions.html#multiv)
  * [**mvtnorm** package](confidence-intervals.html#adjmore)


[Back to top](#topnav)

### N {#n}


  * [`NA`s in the output](FAQs.html#NAs)
  * [Nesting](messy-data.html#nesting)
      * [Auto-detection](messy-data.html#nest-trap)
  * Nesting factors
      * [Creating](utilities.html#groups)
  * [Non-estimability](messy-data.html#nonest)
  * [`non.nuisance`](messy-data.html#nuisance)
  * [`NonEst` values](FAQs.html#NAs)
  * [Normalized contrasts](comparisons.html#nrmlze)
  * [`(nothing)` in output](FAQs.html#nopairs)
  * [`nrmlz.emmc`](comparisons.html#nrmlze)
  * [`nuisance`](messy-data.html#nuisance)
  * [Nuisance factors](messy-data.html#nuisance)


[Back to top](#topnav)

### O {#o}


  * Observational data
      * [in *basics: weights*](basics.html#weights)
      * [in *messy-data: issues*](messy-data.html#issues)
  * [Observational versus experimental data](basics.html#exper)
  * [Odds ratios](transformations.html#oddsrats)
      * [but I wanted differences](FAQs.html#ratios)
  * Offsets
      * [Models with offsets](sophisticated.html#offsets)
      * [`offset()` term vs. `offset` argument in model](xplanations.html#offsets)
  * [Only one mean](AQuickStart.html#covar)
  * [`opt.digits` option](utilities.html#digits)
  * [Options](utilities.html#options)
      * [Startup](utilities.html#startup)
  * Ordinal models
      * [Latent scale](sophisticated.html#ordinal)
      * [Linear-predictor scale](sophisticated.html#ordlp)
      * [in *models: O*](models.html#O)
      * [`prob` and `mean.class`](sophisticated.html#ordprob)
      * [in *sophisticated: ordinal*](sophisticated.html#ordinal)


[Back to top](#topnav)

### P {#p}


  * *P* values
      * [Adjusted](basics.html#recs1)
      * [Adjustment is ignored](FAQs.html#noadjust)
      * [Interpreting](basics.html#pvalues)
  * [`pairs()`](comparisons.html#pairwise)
  * `pairwise ~ factors`
      * [Why you should avoid this](AQuickStart.html#pairwise)
  * [Pairwise comparisons](comparisons.html#pairwise)
      * [Matrix displays](comparisons.html#pwpm)
  * [`pairwise` contrasts](comparisons.html#contrasts)
  * [Pairwise *P*-value plots](comparisons.html#pwpp)
  * [`params`](basics.html#params)
  * [Percentage differences](transformations.html#altscale)
  * [`plot.emmGrid()`](basics.html#plot.emmGrid)
  * [`plot()`](basics.html#plots)
      * [nested factors](messy-data.html#cows)
  * Plots
      * [of confidence intervals](basics.html#plot.emmGrid)
      * [of EMMs](basics.html#plots)
      * [Enhancing with **ggplot2** functions](basics.html#ggplot)
      * [Interaction-style](basics.html#plots)
  * [`+` operator](utilities.html#rbind)
  * Poisson regression
      * [Surprising results](FAQs.html#transformations)
  * [`polreg` models](models.html#O)
  * [Polynomial regression](basics.html#depcovs)
  * Pooled *t*
      * [Instead of Welch's *t*](FAQs.html#nowelch)
  * [`postGridHook`](xtending.html#hooks)
  * [Practices, recommended](basics.html#recs1)
  * Precision
      * [Displaying results with more digits](utilities.html#data)
  * Predictions
      * Bayesian models
          * [in *predictions: bayes*](predictions.html#bayes)
          * [in *sophisticated: predict-mcmc*](sophisticated.html#predict-mcmc)
      * [Error SD](predictions.html#sd-estimate)
      * [graphics](predictions.html#feedlot)
      * [on Particular strata](predictions.html#strata)
      * [Posterior predictive distribution](sophisticated.html#predict-mcmc)
      * [Reference grid](predictions.html#ref-grid)
      * [Total SD](predictions.html#feedlot)
  * [`print.summary_emm()`](basics.html#emmobj)
  * Probit regression
      * [Odds or risk ratios](transformations.html#not-logit)
  * [`pwpm()`](comparisons.html#pwpm)
  * [`pwpp()`](comparisons.html#pwpp)


[Back to top](#topnav)

### Q {#q}


  * [`qdrg()`](FAQs.html#qdrg)
      * [S3 methods](xtending.html#qdrg)
  * [Quadratic terms](basics.html#depcovs)
  * [Quick-start guide](AQuickStart.html)


[Back to top](#topnav)

### R {#r}


  * [R-squared](basics.html#pigs)
  * Random predictors
      * [Accessing levels](sophisticated.html#random-slopes)
  * Random slopes
      * [Bias adjustment](sophisticated.html#random-slopes)
  * [Rank deficiency](xplanations.html#joint_tests)
  * Ratios
      * [but I wanted differences](FAQs.html#ratios)
  * [`rbind()`](utilities.html#rbind)
  * Re-gridding
      * [Implied](transformations.html#regrid3)
  * [Re-labeling](utilities.html#relevel)
  * [Recommended practices](basics.html#recs1)
  * `recover_data.call()`
      * [`frame` argument](xtending.html#rdargs)
  * [`recover_data()`](xtending.html#intro)
      * [Communicating with `emm_basis()`](xtending.html#communic)
      * [`data` and `params` arguments](xtending.html#rdargs)
      * [Dispatching](xtending.html#dispatch)
      * [Error handling](xtending.html#rderrs)
      * [for `lqs` objects](xtending.html#rd.lqs)
      * [for `rsm` objects](xtending.html#rdrsm)
  * [`ref_grid()`](basics.html#RG)
      * [`at`](basics.html#RG)
      * `cov.reduce`
          * [in *basics: plot.emmGrid*](basics.html#plot.emmGrid)
          * [in *basics: RG*](basics.html#RG)
      * [`mult.name`](basics.html#multiv)
      * [`nesting`](messy-data.html#nest-trap)
      * [`offset`](sophisticated.html#offsets)
      * [`regrid` argument](transformations.html#regrid2)
  * [Reference grid](basics.html#emms)
      * [Covariates](basics.html#RG)
      * [Defined](basics.html#RG)
      * [Indicator variables](basics.html#RG)
      * [Predictions on](basics.html#refgrid)
  * Reference grids
      * [Prediction on](predictions.html#ref-grid)
  * [Region of practical equivalence](sophisticated.html#bayesxtra)
  * [Registering `qdrg` methods](xtending.html#exporting)
  * [Registering `recover_data` and `emm_basis` methods](xtending.html#exporting)
  * [`regrid` argument](transformations.html#stdize)
  * [`regrid.flag`](xtending.html#regridded)
  * [`regrid()`](transformations.html#regrid)
      * [probit or other non-logit models](transformations.html#not-logit)
      * [`regrid` vs. `type`](transformations.html#regrid2)
      * [to obtain risk ratios](transformations.html#riskrats)
      * [`transform = "log"`](transformations.html#logs)
  * [Residual plots](basics.html#pigs)
  * [Response scale](confidence-intervals.html#tran)
  * [Response transformations](basics.html#transf)
  * [`revpairwise` contrasts](comparisons.html#contrasts)
  * [`rg.limit` option](messy-data.html#rg.limit)
  * [Risk ratios](transformations.html#riskrats)
  * [RMarkdown](basics.html#formatting)
  * [**robmixglm** example](xtending.html#qdrg)
  * [ROPE](sophisticated.html#bayesxtra)
  * [**rsm** package](xtending.html#rsm)
  * [`rstanarm`](sophisticated.html#mcmc)


[Back to top](#topnav)

### S {#s}


  * [Sample size, displaying](confidence-intervals.html#summary)
  * [Sandwich estimators](messy-data.html#neuralgia)
  * Satterthwaite d.f.
      * [in *models: K*](models.html#K)
      * [in *models: L*](models.html#L)
  * [`"scale"` type](transformations.html#trangraph)
  * [`scale()`](transformations.html#stdize)
  * [Selecting results](utilities.html#brackets)
  * [Sidak adjustment](confidence-intervals.html#adjust)
  * Significance
      * [Assessing](basics.html#pvalues)
  * [`simple = "each"`](confidence-intervals.html#simple)
  * Simple comparisons
      * [in *confidence-intervals: simple*](confidence-intervals.html#simple)
      * [in *FAQs: interactions*](FAQs.html#interactions)
      * [in *interactions: simple*](interactions.html#simple)
  * [Simpson's paradox](basics.html#emms)
  * `specs`
      * [Formula](comparisons.html#formulas)
  * [Standardized response](transformations.html#stdize)
  * [`stanreg` objects](models.html#S)
  * [* gazing (star gazing)](interactions.html#factors)
  * [Startup options](utilities.html#startup)
  * [Statistical consultants](basics.html#recs3)
  * [Statistics is hard](basics.html#recs3)
  * [`str()`](basics.html#emmobj)
  * `submodel`
      * [`"minimal"` and `"type2"`](messy-data.html#type2submodel)
      * [in a multivariate model](xplanations.html#mult.submodel)
      * [in *messy-data: submodels*](messy-data.html#submodels)
      * [in *xplanations: submodels*](xplanations.html#submodels)
  * [Subsets of data](FAQs.html#model)
  * [`summary_emm` object](basics.html#emmobj)
      * [As a data frame](utilities.html#data)
  * `summary()`
      * [`adjust`](comparisons.html#pairwise)
      * [in *basics: emmobj*](basics.html#emmobj)
      * Bayesian models
          * [in *confidence-intervals: summary*](confidence-intervals.html#summary)
          * [in *models: S*](models.html#S)
      * [Calculated columns](confidence-intervals.html#summary)
      * [in *confidence-intervals: summary*](confidence-intervals.html#summary)
      * [HPD intervals](sophisticated.html#mcmc)
      * [`hpd.summary()`](confidence-intervals.html#summary)
      * `infer`
          * [in *comparisons: pairwise*](comparisons.html#pairwise)
          * [in *confidence-intervals: summary*](confidence-intervals.html#summary)
      * [Show sample size](confidence-intervals.html#summary)
      * [`type = "unlink"`](transformations.html#tranlink)


[Back to top](#topnav)

### T {#t}


  * [*t* tests vs. *z* tests](FAQs.html#asymp)
  * [`test()`](confidence-intervals.html#summary)
      * [`delta`](confidence-intervals.html#equiv)
      * [`joint = TRUE`](confidence-intervals.html#joint)
  * Tests
      * [Equivalence](confidence-intervals.html#equiv)
      * [Non-inferiority](confidence-intervals.html#equiv)
      * [Nonzero null](confidence-intervals.html#summary)
      * [One- and two-sided](confidence-intervals.html#summary)
  * [Too few means](AQuickStart.html#covar)
  * [Transformations](basics.html#transf)
      * [Adding after the fact](transformations.html#after)
      * [Auto-detected](transformations.html#auto)
      * [Back-transforming](confidence-intervals.html#tran)
      * [Bias adjustment](transformations.html#bias-adj)
      * [Custom](transformations.html#special)
      * [faking](transformations.html#faking)
      * [Faking a log transformation](transformations.html#logs)
      * [Graphical display](transformations.html#trangraph)
      * [with link function](transformations.html#tranlink)
      * [Log](comparisons.html#logs)
      * [Overview](transformations.html#overview)
      * [Percent difference](transformations.html#altscale)
      * [Re-gridding](transformations.html#regrid)
      * [Response versus link functions](transformations.html#link-bias)
      * [`scale()`](transformations.html#stdize)
      * [Standardizing](transformations.html#stdize)
      * [Timing is everything](transformations.html#timing)
  * [Transformations|Implied re-gridding](transformations.html#regrid3)
  * [Transformations|Special modes](transformations.html#regrid3)
  * Trends
      * [Estimating and comparing](interactions.html#oranges)
  * [`trt.vs.ctrl` contrasts](comparisons.html#contrasts)
  * [Tukey adjustment](confidence-intervals.html#adjust)
      * [Ignored or changed](FAQs.html#notukey)
  * [`type`](confidence-intervals.html#tran)
  * [`type = "response"`](basics.html#transf)
  * [`type = "scale"`](transformations.html#trangraph)
  * [Type II analysis](messy-data.html#type2submodel)
  * Type III tests
      * [in *confidence-intervals: joint*](confidence-intervals.html#joint)
      * [in *confidence-intervals: joint_tests*](confidence-intervals.html#joint_tests)


[Back to top](#topnav)

### U {#u}


  * [Unadjusted tests](confidence-intervals.html#adjmore)
  * [`update()`](utilities.html#update)
      * [`tran`](transformations.html#after)
  * [Using results](utilities.html#data)


[Back to top](#topnav)

### V {#v}


  * [Variables that are not predictors](basics.html#params)
  * [`vcov.`](messy-data.html#neuralgia)
  * [`vcovHook`](xtending.html#hooks)
  * Vignettes
      * [Comparisons](comparisons.html)
      * [Confidence intervals and tests](confidence-intervals.html)
      * [Explanations supplement](xplanations.html)
      * [Extending **emmeans**](xtending.html)
      * [FAQS](FAQs.html)
      * [Interactions](interactions.html)
      * [Messy data](messy-data.html)
      * [Predictions](predictions.html)
      * [Sophisticated models](sophisticated.html)
      * [Transformations and link functions](transformations.html)
      * [Utilities and options](utilities.html)


[Back to top](#topnav)

### W {#w}


  * Weighting
      * [Equal weights](basics.html#exper)
  * [`weights`](messy-data.html#weights)
  * [Welch's *t* comparisons](FAQs.html#nowelch)
      * [Example](utilities.html#relevel)
  * [`wt.nuis`](messy-data.html#nuisance)


[Back to top](#topnav)

### X {#x}


  * [`xtable` method](basics.html#formatting)


[Back to top](#topnav)

### Z {#z}


  * [*z* tests](sophisticated.html#dfoptions)
      * [vs. *t* tests](FAQs.html#asymp)
  * [`zeroinfl` models](models.html#C)


[Back to top](#topnav)


*Index generated by the [vigindex](https://github.com/rvlenth/vigindex) package.*

</div> <!-- closes div class = "vigindex" -->

---
title: "FAQs for emmeans"
author: "emmeans package, Version `r packageVersion('emmeans')`"
output: emmeans::.emm_vignette
vignette: >
  %\VignetteIndexEntry{FAQs for emmeans}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
```{r, echo = FALSE, results = "hide", message = FALSE}
require("emmeans")
knitr::opts_chunk$set(fig.width = 4.5, class.output = "ro")
options(show.signif.stars = FALSE)
```

<!-- @index Vignettes!FAQS; Frequently asked questions -->
This vignette contains answers to questions received from users or posted on discussion boards like [Cross Validated](https://stats.stackexchange.com) and 
[Stack Overflow](https://stackoverflow.com/)

## Contents {#contents}

 1. [What are EMMs/lsmeans?](#what)
 2. [What is the fastest way to obtain EMMs and pairwise comparisons?](#fastest)
 2. [I wanted comparisons, but all I get is (nothing)](#nopairs)
 2. [The model I fitted is not supported by **emmeans**](#qdrg)
 2. [I have three (or two or four) factors that interact](#interactions)
 3. [I have covariate(s) that interact(s) with factor(s)](#trends)
 3. [I have covariate(s) and am fitting a polynomial model](#polys)
 3. [Some "significant" comparisons have overlapping confidence intervals](#CIerror)
 4. [All my pairwise comparisons have the same *P* value](#notfactor)
 5. [emmeans() doesn't work as expected](#numeric)
 6. [All or some of the results are NA](#NAs)
 6. [If I analyze subsets of the data separately, I get different results](#model)
 7. [My lsmeans/EMMs are way off from what I expected](#transformations)
 8. [Why do I get `Inf` for the degrees of freedom?](#asymp)
 10. [I get exactly the same comparisons for each "by" group](#additive)
 11. [My ANOVA *F* is significant, but no pairwise comparisons are](#anova)
 11. [I wanted differences, but instead I got ratios (or odds ratios)](#ratios)
 12. [I asked for a Tukey adjustments, but that's not what I got](#notukey)
 12. [`emmeans()` completely ignores my P-value adjustments](#noadjust)
 13. [`emmeans()` gives me pooled *t* tests, but I expected Welch's *t*](#nowelch)

[Index of all vignette topics](vignette-topics.html)

## What are EMMs/lsmeans? {#what}
<!-- @index EMMs!What are they?; Least-squares means -->
Estimated marginal means (EMMs), a.k.a. least-squares means, are
predictions on a reference grid of predictor settings, or marginal
averages thereof. See details in [the "basics" vignette](basics.html).

## What is the fastest way to obtain EMMs and pairwise comparisons? {#fastest}
<!-- @index Model!Importance of; `emmeans()`!Fastest way to get wrong answers -->
There are two answers to this (i.e., be careful what you wish for):

  1. Don't think; just fit the first model that comes to mind and run 
    `emmeans(model, pairwise ~ treatment)`. This is the fastest way; however,
    the results have a good chance of being invalid.
  2. *Do* think: Make sure you fit a model that really explains the responses.
     Do diagnostic residual plots, include appropriate interactions, account
     for heteroscadesticity if necessary, etc. This is the fastest way
     to obtain *appropriate* estimates and comparisons.

The point here is that `emmeans()` summarizes the *model*, not the data directly.
If you use a bad model, you will get bad results. And if you use a good model,
you will get appropriate results. It's up to you: it's your research---is it important?

[Back to Contents](#contents)

## I wanted comparisons, but all I get is (nothing) {#nopairs}
<!-- @index Comparisons result in `(nothing)`; `(nothing)` in output@nothing -->
This happens when you have only one estimate; and you can't compare it
with itself! This is turn can happen when you have a situation like this:
you have fitted 
```
mod <- lm(RT ~ treat, data = mydata)
```
and `treat` is coded in
your dataset with numbers 1, 2, 3, ... . Since `treat` is a numeric predictor,
`emmeans()` just reduces it to a single number, its mean, rather than separate values for
each treatment. Also, please note that this is almost certainly NOT the model you want,
because it forces an assumption that the treatment effects all fall on a straight line.
You should fit a model like 
```
mod <- lm(RT ~ factor(treat), data = mydata)
```
then you
will have much better luck with comparisons.

## The model I fitted is not supported by **emmeans** {#qdrg}
<!-- @index Models!Unsupported; `qdrg()` -->
You may still be able to get results using `qdrg()` (quick and dirty reference grid). 
See `?qdrg` for details and examples.

## I have three (or two or four) factors that interact {#interactions}
<!-- @index Multi-factor studies; Simple comparisons -->
Perhaps your question has to do with interacting factors, and you want to do
some kind of *post hoc* analysis comparing levels of one (or more) of the 
factors on the response. Some specific versions of this question...

  * Perhaps you tried to do a simple comparison for one treatment and 
    got a warning message you don't understand
  * You do pairwise comparisons of factor combinations and it's 
    just too much -- want just some of them
  * How do I even approach this?
  
My first answer is: plots almost always help. If you have factors A, B, and C,
try something like `emmip(model, A ~ B | C)`, which creates an interaction-style
plot of the predictions against B, for each A, with separate panels for each C.
This will help visualize what effects stand out in a practical way. This
can guide you in what post-hoc tests would make sense. See the
["interactions" vignette](interactions.html) for more discussion and examples.

[Back to Contents](#contents)

## I have covariate(s) that interact(s) with factor(s) {#trends}
<!-- @index Covariates!Interacting with factors -->
This is a situation where it may well be appropriate to compare the
slopes of trend lines, rather than the EMMs. See the 
`help("emtrends"()`")` and the discussion of this
topic in [the "interactions" vignette](interactions.html#covariates)

## I have covariate(s) and am fitting a polynomial model {#polys}
You need to be careful to define the reference grid consistently. For example,
if you use covariates `x` and `xsq` (equal to `x^2`) to fit a quadratic curve,
the default reference grid uses the mean of each covariate -- and `mean(xsq)` is
usually not the same as `mean(x)^2`. So you need to use `at` to ensure that the
covariates are set consistently with respect to the model. See [this subsection
of the "basics" vignette](basics.html#depcovs) for an example.

## Some "significant" comparisons have overlapping confidence intervals {#CIerror}
<!-- @index Confidence intervals!Overlapping; Comparisons!with overlapping CIs@over -->
That can happen because 
*it is just plain wrong to use [non-]overlapping CIs for individual means to do comparisons*. 
Look at the printed results from something like `emmeans(mymodel, pairwise ~
treatment)`. In particular, note that the `SE` values are *not* the same*, and
may even have different degrees of freedom. Means are one thing statistically,
and differences of means are quite another thing. Don't ever mix them up, and
don't ever use a CI display for comparing means.

I'll add that making hard-line decisions about "significant" and
"non-significant" is in itself a poor practice. 
See [the discussion in the "basics" vignette](basics.html#pvalues) 


## All my pairwise comparisons have the same *P* value {#notfactor}
This will happen if you fitted a model where the treatments you want to 
compare were put in as a numeric predictor; for example `dose`, with 
values of 1, 2, and 3. If `dose` is modeled as numeric, you will be
fitting a linear trend in those dose values, rather than a model
that allows those doses to differ in arbitrary ways. Go back and fit
a different model using `factor(dose)` instead; it will make all the
difference. This is closely related to the next topic.



## emmeans() doesn't work as expected {#numeric}
<!-- @index Covariates!`emmeans()` doesn't work; Covariates!`cov.keep`; Covariates!`cov.reduce`;  -->
Equivalently, users ask how to get *post hoc* comparisons when we have
covariates rather than factors.
Yes, it does work, but you have to tell it the appropriate reference grid.

But before saying more, I have a question for you: *Are you sure your model
is meaningful?*

  * If your question concerns *only* two-level predictors such as `sex` 
    (coded 1 for female, 2 for male), no problem. The model will produce
    the same predictions as you'd get if you'd used these as factors.
  * If *any* of the predictors has 3 or more levels, you may have fitted
    a nonsense model, in which case you need to fit a different model that
    does make sense before doing any kind of *post hoc* analysis. For instance,
    the model contains a covariate `brand` (coded 1 for Acme, 2 for Ajax, and
    3 for Al's), this model is implying that the difference between 
    Acme and Ajax is exactly equal to the difference between Ajax and Al's,
    owing to the fact that a linear trend in `brand` has been fitted. 
    If you had instead coded 1 for Ajax, 2 for Al's, and 3 for Acme, the model
    would produce different fitted values. Ask yourself if it makes sense
    to have `brand = 2.319`. If not, you need to fit another model using 
    `factor(brand)` in place of `brand`.

Assuming that the appropriateness of the model is settled, the current
version of **emmeans** automatically casts two-value covariates as factors,
but not covariates having higher numbers of unique values. Suppose your model has
a covariate `dose` which was experimentally varied over four levels, but
can sensibly be interpreted as a numerical predictor. If you want to include the
separate values of `dose` rather than the mean `dose`, you can do that using
something like `emmeans(model, "dose", at = list(dose = 1:4))`, or 
`emmeans(model, "dose", cov.keep = "dose")`, or `emmeans(model, "dose", cov.keep = "4")`.
There are small differences between these. The last one regards any covariate
having 4 or fewer unique values as a factor.

See "altering the reference grid" in the ["basics" vignette](basics.html#altering)
for more discussion.

[Back to Contents](#contents)


## All or some of the results are NA {#NAs}
<!-- @index `NA`s in the output; `NonEst` values; Estimability issues -->
The **emmeans** package uses tools in the **estimability** package to determine
whether its results are uniquely estimable. For example, in a two-way model
with interactions included, if there are no observations in a particular cell
(factor combination), then we cannot estimate the mean of that cell.

When *some* of the EMMs are estimable and others are not, that is information
about missing information in the data. If it's possible to remove some terms
from the model (particularly interactions), that may make more things estimable 
if you re-fit with those terms excluded; 
but don't delete terms that are really needed for the model to fit well.

When *all* of the estimates are non-estimable, it could be symptomatic of
something else. Some possibilities include:

  * An overly ambitious model; for example, in a Latin square design, interaction
    effects are confounded with main effects; so if any interactions are included
    in the model, you will render main effects inestimable.
  * Possibly you have a nested structure that needs to be included in the
    model or specified via the `nesting` argument. Perhaps the levels that B
    can have depend on which level of A is in force. Then B is nested in A and
    the model should specify `A + A:B`, with no main effect for `B`.
  * Modeling factors as numeric predictors (see also the [related section on
    covariates](#numeric)). To illustrate, suppose you have data on particular
    state legislatures, and the model includes the predictors `state_name` as well
    as `dem_gov` which is coded 1 if the governor is a Democrat and 0 otherwise.
    If the model was fitted with `state_name` as a factor or character variable,
    but `dem_gov` as a numeric predictor, then, chances are, `emmeans()` will
    return non-estimable results. If instead, you use `factor(dem_gov)` in the
    model, then the fact that `state_name` is nested in `dem_gov` will be
    detected, causing EMMs to be computed separately for each party's
    states, thus making things estimable.
  * Some other things may in fact be estimable. For illustration, it's easy
    to construct an example where all the EMMs are non-estimable, but
    pairwise comparisons are estimable:
```{r}
pg <- transform(pigs, x = rep(1:3, c(10, 10, 9)))
pg.lm <- lm(log(conc) ~ x + source + factor(percent), data = pg)
emmeans(pg.lm, consec ~ percent)
```

The ["messy-data" vignette](messy-data.html) has more examples and discussion.


[Back to Contents](#contents)

## If I analyze subsets of the data separately, I get different results {#model}
<!-- @index Analysis of subsets of data; Subsets of data -->
Estimated marginal means summarize the *model* that you fitted to the data
-- not the data themselves. Many of the most common models rely on
several simplifying assumptions -- that certain effects are linear, that the
error variance is constant, etc. -- and those assumptions are passed forward
into the `emmeans()` results. Doing separate analyses on subsets usually
comprises departing from that overall model, so of course the results are 
different.


## My lsmeans/EMMs are way off from what I expected {#transformations}
<!-- @index `emmeans()`!Surprising results from; Poisson regression!Surprising results
     Logistic regression!Surprising results -->
First step: Carefully read the annotations below the output. Do they say
something like "results are on the log scale, not the response scale"?
If so, that explains it. A Poisson or logistic model involves a link function,
and by default, `emmeans()` produces its results on that same scale.
You can add `type = "response"` to the `emmeans()` call and it will
put the results of the scale you expect. But that is not always
the best approach. The ["transformations" vignette](transformations.html)
has examples and discussion.


## Why do I get `Inf` for the degrees of freedom? {#asymp}
<!-- @index Infinite degrees of freedom; Degrees of freedom!Infinite 
     *z* tests!vs. *t* tests; *t* tests vs. *z* tests -->
This is simply the way that **emmeans** labels asymptotic results
(that is, estimates that are tested against the standard normal distribution
-- *z* tests -- rather than the *t* distribution). Note that obtaining quantiles 
or probabilities from the *t* distribution with infinite degrees of freedom
is the same as obtaining the corresponding values from the standard normal.
For example:
```{r}
qt(c(.9, .95, .975), df = Inf)
qnorm(c(.9, .95, .975))
```
so when you see infinite d.f., that just means its a *z* test or a *z* confidence
interval.

[Back to Contents](#contents)



## I get exactly the same comparisons for each "by" group {#additive}
<!-- @index `by` groups!Identical comparisons -->
As mentioned elsewhere, EMMs summarize a *model*, not the data.
If your model does not include any interactions between the `by` variables
and the factors for which you want EMMs, then by definition, the
effects for the latter will be exactly the same regardless of the `by`
variable settings. So of course the comparisons will all be the same.
If you think they should be different, then you are saying that your model
should include interactions between the factors of interest and the 
`by` factors.

## My ANOVA *F* is significant, but no pairwise comparisons are {#anova}
<!-- @index *F* test!vs. pairwise comparisons@pairwz
        Analysis of variance!versus *post hoc* comparisons@post -->
First of all, you should not be making binary decisions of "significant" or
"nonsignificant." This is a simplistic view of *P* values that assigns an 
unmerited magical quality to the value 0.05. It is suggested
that you just report the *P* values actually obtained, and let your readers
decide how significant your findings are in the context of the scientific findings.


But to answer the question: This is a common misunderstanding of ANOVA. If *F*
has a particular *P* value, this implies only that *some contrast* among the
means (or effects) has the same *P* value, after applying the Scheffe
adjustment. That contrast may be very much unlike a pairwise comparison,
especially when there are several means being compared. Having an *F* statistic
with a *P* value of, say, 0.06, does *not* imply that any pairwise comparison
will have a *P* value of 0.06 or smaller. Again referring to the paragraph above,
just report the *P* value for each pairwise comparison, and don't try to
relate them to the *F* statistic.

Another consideration is that by default, *P* values for pairwise comparisons
are adjusted using the Tukey method, and the adjusted *P* values can be quite a
bit larger than the unadjusted ones. (But I definitely do *not* advocate using
no adjustment to "repair" this problem.)


## I wanted differences, but instead I got ratios (or odds ratios) {#ratios}
<!-- @index Comparisons!Obtaining differences rather than ratios;
            Ratios!but I wanted differences; Odds ratios!but I wanted differences  -->
When a transformation or link involves logs, then, unlike other transformations,
comparisons can be back-transformed into ratios -- and that is the default
behavior. If you really want differences and not ratios, you can re-grid the
means first. Re-gridding starts anew with everything on the response scale,
and no memory of the transformation.
```r
EMM <- emmeans(...)
pairs(regrid(EMM))   # or contrast(regrid(EMM), ...)
```
PS -- A side effect is that this causes the tests to be done using SEs obtained
by the delta method on the re-gridded scale, rather than on the link scale.
Re-gridding can be used with any transformation, not just logs, and it has the
same side effect.


## I asked for Tukey adjustments, but that's not what I got {#notukey}
<!-- @index Tukey adjustment!Ignored or changed -->
There are two reasons this could happen:

  1. There is only one comparison in each `by` group (see next topic).
  2. A Tukey adjustment is inappropriate. The Tukey adjustment is appropriate
  for pairwise comparisons of means. When you have some other set of contrasts,
  the Tukey method is deemed unsuitable and the Sidak method is used instead. A
  suggestion is to use `"mvt"` adjustment (which is exact); we don't default to
  this because it can require a lot of computing time for a large set of
  contrasts or comparisons.
  
  
## `emmeans()` completely ignores my P-value adjustments  {#noadjust}
<!-- @index *P* values!Adjustment is ignored -->
This happens when there are only two means (or only two in each `by` group).
Thus there is only one comparison. When there is only one thing to test, there
is no multiplicity issue, and hence no multiplicity adjustment to the *P*
values.

If you wish to apply a *P*-value adjustment to all tests across all groups,
you need to null-out the `by` variable and summarize, as in the following:
```r
EMM <- emmeans(model, ~ treat | group)   # where treat has 2 levels
pairs(EMM, adjust = "sidak")   # adjustment is ignored - only 1 test per group
summary(pairs(EMM), by = NULL, adjust = "sidak")   # all are in one group now
```
Note that if you put `by = NULL` *inside* the call to `pairs()`, then this
causes all `treat`,`group` combinations to be compared.
     
     
[Back to Contents](#contents)


## `emmeans()` gives me pooled *t* tests, but I expected Welch's *t* {#nowelch}
<!-- @index Welch's *t* comparisons; Pooled *t*!Instead of Welch's *t*;
            Model!Importance of getting it right; `emmeans()`!And the underlying model;
            Get the model right first -->
It is important to note that `emmeans()` and its relatives produce results based
on the *model object* that you provide -- not the data. So if your sample SDs
are wildly different, a model fitted using `lm()` or `aov()` is not a good model,
because those R functions use a statistical model that presumes that the errors
have constant variance. That is, the problem isn't in `emmeans()`, it's in
handing it an inadequate model object.

Here is a simple illustrative example. Consider a simple one-way experiment
and the following model:
```
mod1 <- aov(response ~ treat, data = mydata)
emmeans(mod1, pairwise ~ treat)
```
This code will estimate means and comparisons among treatments. All standard errors,
confidence intervals, and *t* statistics are based on the pooled residual SD
with *N - k* degrees of freedom (assuming *N* observations and *k* treatments).
These results are useful *only* if the underlying assumptions of `mod1` are correct -- 
including the assumption that the error SD is the same for all treatments.

Alternatively, you could fit the following model using generalized least-squares:
```
mod2 = nlme::gls(response ~ treat, data = mydata,
                 weights = varIdent(form = ~1 | treat))
emmeans(mod2, pairwise ~ treat)
```
This model specifies that the error variance depends on the levels of `treat`.
This would be a much better model to use when you have wildly different
sample SDs. The results of the `emmeans()` call will reflect this improvement
in the modeling. The standard errors of the EMMs will depend on the individual
sample variances, and the *t* tests of the comparisons will be in essence
the Welch *t* statistics with Satterthwaite degrees of freedom.

To obtain appropriate *post hoc* estimates, contrasts, and comparisons,
one must first find a model that successfully explains the peculiarities in
the data. This point cannot be emphasized enough. If you give `emmeans()` a
good model, you will obtain correct results; if you give it a bad model, 
you will obtain incorrect results. Get the model right *first*.


[Back to Contents](#contents)

[Index of all vignette topics](vignette-topics.html)

---
title: "Basics of estimated marginal means"
author: "emmeans package, Version `r packageVersion('emmeans')`"
output: emmeans::.emm_vignette
vignette: >
  %\VignetteIndexEntry{Basics of EMMs}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
```{r, echo = FALSE, results = "hide", message = FALSE}
require("emmeans")
require("ggplot2")
knitr::opts_chunk$set(fig.width = 4.5, class.output = "ro")
```



## [Contents](#contents)

  1. [Foundations](#found)
     a. [Emphasis on experimental data](#exper)
     b. [Emphasis on models](#models)
     c. [Illustration: `pigs` experiment](#pigs)
     d. [Estimated marginal means](#emms)
     e. [The reference grid, and definition of EMMs](#refgrid)
     f. [More on the reference grid](#RG)
  2. [Other topics](#othertopics)
     a. [Passing arguments](#arguments)
     b. [Transformations](#transf)
     c. [Derived covariates](#depcovs)
     d. [Non-predictor variables](#params)
     e. [Graphical displays](#plots)
     f. [Formatting results](#formatting)
     g. [Using weights](#weights)
     h. [Multivariate responses](#multiv)
  3. [Objects, structures, and methods](#emmobj)
  4. [P values, "significance", and recommendations](#pvalues)
  5. [Summary](#summary)
  6. [Further reading](#more)
  
[Index of all vignette topics](vignette-topics.html)

## Foundations {#found}
<!-- @index Foundations of EMMs; Expected marginal means!Foundations of -->

### Emphasis on experimental data {#exper}
<!-- @index Experimental versus observational data;
            Observational versus experimental data;  
            Controlled experiments; Latin squares; Balanced allocation;
            Weighting!Equal weights; -->
To start off with, we should emphasize that the underpinnings of estimated marginal 
means -- and much of what the **emmeans** package offers -- relate more to
*experimental* data than to *observational* data. In observational data,
we sample from some population, and the goal of statistical analysis is to
characterize that population in some way. In contrast, with experimental data,
the experimenter controls the environment under which test runs are conducted,
and in which responses are observed and recorded. Thus with experimentation,
the population is an abstract entity consisting of potential outcomes of test
runs made under conditions we enforce, rather than a physical entity that we 
observe without changing it.

We say this because the default behavior of the `emmeans()` function is to average
groups together with equal weights; this is common in analysis of experiments,
but not common in analysis of observational data; and I think that misunderstandings
about this underlie some criticisms such as [are found here](https://stats.stackexchange.com/questions/332167/what-are-ls-means-useful-for) and [here](https://stats.stackexchange.com/questions/510862/is-least-squares-means-lsmeans-statistical-nonsense/510923#510923).

Consider, for example, a classic Latin square experimental design. RA Fisher and
others expounded on such designs. Suppose we want to compare four treatments,
say fertilizers, in an agricultural experiment. A Latin square plan would
involve dividing a parcel of land into four rows and four columns, defining 16
plots. Then we apply one of the fertilizers to each plot in such a way that each
fertilizer appears once in each row and once in each column (and thus, each
row and each column contains all four fertilizers). This scheme, to some extent,
controls for possible spatial effects within the land parcel. To compare
the fertilizer, we average together the response values (say, yield of a crop)
observed on the four plots where each fertilizer was used. It seems right to
average these together with equal weight, because each experimental condition
seems equally valid and there is no reason to give one more weight than another.
In this illustration, the fertilizer means are not marginal means of some
physical population; they are simply the means obtained under the four test
conditions defined by the experiment.

### Emphasis on models {#models}
<!-- @index Modelling; Expected marginal means!Reliance on a model;
            Model!Importance of; GIGO (garbage in, garbage out); -->
The **emmeans** package requires you to fit a model to your data. All the
results obtained in **emmeans** rely on this model. So, really, the analysis
obtained is really an analysis of the model, not the data. This analysis does
depend on the data, but only insofar as the fitted model depends on the data. We
use predictions from this model to compute estimated marginal means (EMMs),
which will be defined more explicitly below. For now, there are two things to
know:

  1. If you change the model, that changes the EMMs
  2. If the model fits poorly, the EMMs represent the data poorly
     (the garbage in, garbage out principle)

So to use this package to analyze your data, the most important first step is to
fit a good model.

[Back to Contents](#contents)


### Illustration: `pigs` experiment {#pigs}
<!-- @index Examples!`pigs`; Examples!Unbalanced data; 
            Modeling!Response transformation; Modeling!`pigs` example; 
            `inverse()`; Residual plots; R-squared; Adjusted R-squared -->
Consider the `pigs` dataset provided with the package (`help("pigs")` provides
details). These data come from an experiment where pigs are given
different percentages of protein (`percent`) from different sources (`source`)
in their diet, and later we measured the concentration (`conc`) of leucine.
The `percent` values are quantitative, but we chose those particular values
deliberately, and (at least initially) we want separate estimates at each `percent` 
level; that is, we want to view `percent` as a factor, not a quantitative predictor.

As discussed, our first task is to come up with a good model. Doing so requires
a lot of skill, and we don't want to labor too much over the details; you really 
need other references to deal with this aspect adequately. But we will briefly 
discuss five models and settle on one of them:
```{r}
mod1 <- lm(conc ~ source * factor(percent), data = pigs)
mod2 <- update(mod1, . ~ source + factor(percent))   # no interaction
```
These models have $R^2$ values of 0.808 and 0.700, and adjusted $R^2$ values of
0.684 and 0.634. `mod1` is preferable to `mod2`, suggesting we need the
interaction term. However, a residual-vs-predicted plot of `mod2` has a classic
"horn" shape (curving and fanning out), indicating a situation where a response
transformation might help better than including the interaction. 

It turns out that an inverse transformation, (`1/conc`) really serves us well.
(Perhaps this isn't too surprising, as concentrations are typically determined
by titration, in which the actual measurements are volumes; and these are
reciprocally related to concentrations, i.e., amounts *per* unit volume.)

So here are three more models:
```{r}
mod3 <- update(mod1, inverse(conc) ~ .)
mod4 <- update(mod2, inverse(conc) ~ .)     # no interaction
mod5 <- update(mod4, . ~ source + percent)  # linear term for percent
```
(Note: We could have used `1/conc` as the response variable, but **emmeans**
provides an equivalent `inverse()` function that will prove more advantageous
later.) The residual plots for these models look a lot more like a random scatter
of points (and that is good). The $R^2$ values for these models are 0.818, 0.787,
and 0.749, respectively; and the adjusted $R^2$s are 0.700, 0.740, and 0.719.
`mod4` has the best adjusted $R^2$ and will be our choice.

[Back to Contents](#contents)


### Estimated marginal means {#emms}
<!-- @index Estimated marginal means!Demonstrated; 
            Estimated marginal means!Reference grid; Reference grid;
            Estimated marginal means!vs. ordinary marginal means;
            Simpson's paradox; Confounding;  `emmeans()`; -->
Now that we have a good model, let's use the `emmeans()` function to obtain
estimated marginal means (EMMs). We'll explain them later.
```{r}
(EMM.source <- emmeans(mod4, "source"))
(EMM.percent <- emmeans(mod4, "percent"))
```
Let's compare these with the ordinary marginal means (OMMs) on `inverse(conc)`:
```{r}
with(pigs, tapply(inverse(conc), source, mean))
with(pigs, tapply(inverse(conc), percent, mean))
```
Both sets of OMMs are vaguely similar to the corresponding EMMs. However,
please note that the EMMs for `percent` form a decreasing sequence, while
the the OMMs decrease but then increase at the end.

### The reference grid, and definition of EMMs {#refgrid}
<!-- @index Estimated marginal means!Definition of; 
            Reference grid!Predictions on;
            Estimated marginal means!Manually computed; -->
Estimated marginal means are defined as marginal means of model predictions
over the grid comprising all factor combinations -- called the *reference grid*.
For the example at hand, the reference grid is
```{r}
(RG <- expand.grid(source = levels(pigs$source), percent = unique(pigs$percent)))
```
To get the EMMs, we first need to obtain predictions on this grid:
```{r}
(preds <- matrix(predict(mod4, newdata = RG), nrow = 3))
```
then obtain the marginal means of these predictions:
```{r}
apply(preds, 1, mean)   # row means -- for source
apply(preds, 2, mean)   # column means -- for percent
```
These marginal averages match the EMMs obtained earlier via `emmeans()`.

Now let's go back to the comparison with the ordinary marginal means. The
`source` levels are represented by the columns of `pred`; and note that each row
of `pred` is a decreasing set of values. So it is no wonder that the marginal
means -- the EMMs for `source` -- are decreasing. That the OMMs for `percent` do
not behave this way is due to the imbalance in sample sizes:
```{r}
with(pigs, table(source, percent))
```
This shows that the OMMs of the last column give most of the weight (3/5) to the
first source, which tends to have higher `inverse(conc)`, making the OMM for 18
percent higher than that for 15 percent, even though the reverse is true with
every level of `source`. This kind of disconnect is an example of *Simpson's
paradox,* in which a confounding factor can distort your findings. The EMMs are
not subject to this paradox, but the OMMs are, when the sample sizes are
correlated with the expected values.

In summary, we obtain a references grid of all factor combinations, obtain model
predictions on that grid, and then the expected marginal means are estimated as
equally-weighted marginal averages of those predictions. Those EMMs are not
subject to confounding by other factors, such as might happen with ordinary
marginal means of the data. Moreover, unlike OMMs, EMMs are based on a model
that is fitted to the data.

[Back to Contents](#contents)


### More on the reference grid {#RG}
<!-- @index Reference grid!Defined; `ref_grid()`; 
            `ref_grid()`!`at`; `ref_grid()`!`cov.reduce`;
            Reference grid!Covariates; Adjusted means; 
            Reference grid!Indicator variables; Indicator variables;  -->
In the previous section, we discussed the reference grid as being the set of all
factor combinations. It is slightly more complicated than that when we have
numerical predictors (AKA covariates) in the model. By default, we use the
average of each covariate -- thus not enlarging the number of combinations
comprising the grid. Using the covariate average(s) yields what are often called
*adjusted means*. There is one exception, though: if a covariate has only two
different values, we treat it as a factor having those two levels. For example,
a model could include an indicator variable `male` that is `1` if the subject is
male, and `0` otherwise. Then `male` would be viewed as a factor with levels `0`
and `1`. Note, again, that the reference grid is formulated from the model
we are using.

We can see a snapshot of the reference grid via the `ref_grid` function; for example
```{r}
(RG4 <- ref_grid(mod4))
ref_grid(mod5)
```
The reference grid for `mod5` is different from that for `mod4` because in those models, `percent` is a factor in `mod4` and a covariate in `mod5`.

It is possible to modify the reference grid. In the context of the present example,
it might be interesting to compare EMMs based on `mod4` and `mod5`, and we can put
them on an equal footing by using the same `percent` values as reference levels:
```{r}
(RG5 <- ref_grid(mod5, at = list(percent = c(9, 12, 15, 18))))
```
We could also have done this using
```{r eval = FALSE}
(RG5 <- ref_grid(mod5, cov.reduce = FALSE)
```
... which tells `ref_grid()` to set covariate levels using unique values.
It's safer to use `at` because `cov.reduce` affects *all* covariates
instead of specific ones.
###### {#emmip}
<!-- @index `emmip()`; Interaction-style plots; -->
The two models' predictions can be compared using interaction-style plots
via the `emmip()` function
```{r fig.alt = "interaction-style plots of 'RG4' and 'RG5'. These show parallel trends along 'percent' for each 'source'. The one for 'RG5' consists of parallel straigt lines. The values plotted here can be obtained via 'summary(RG4)' and 'summary(RG5)'"}
emmip(RG4, source ~ percent, style = "factor")
emmip(RG5, source ~ percent, style = "factor")
```
Both plots show three parallel trends, because neither model includes an 
interaction term; but of course for `mod5`, those trends are straight lines.


[Back to Contents](#contents)


## Other topics {#othertopics}


### Passing arguments {#arguments}
<!-- @index Arguments!Passed to `ref_grid()`; Arguments!`...`
     `emmeans()`!Arguments passed to `ref_grid()`; -->
Quite a few functions in the **emmeans** package, including `emmeans()` and `emmip()`,
can take either a model object or a reference-grid object as their first argument.
Thus we can obtain EMMs for `mod5` directly from `RG5`, e.g.
```{r}
emmeans(RG5, "source")
```
These are slightly different results than we had earlier for `mod4`.
In these functions where the model and the reference grid are interchangeable, 
the first thing the function does is to check which it is; and if it is a model
object, it constructs the reference grid. When it does that, it passes
its arguments to `ref_grid()` in case they are needed. For instance, the
above EMMs could have been obtained using
```{r, eval = FALSE}
emmeans(mod5, "source", at = list(percent = c(9, 12, 15, 18)))
## (same results as above)
```
It is a great convenience to be able to pass arguments to `ref_grid()`, but it
also can confuse new users, because if we look at the help page for `emmeans()`,
it does not list `at` as a possible argument. It is mentioned, though, if you
look at the `...` argument. So develop a habit of looking at documentation for
other functions, especially `ref_grid()`, for other arguments that may affect 
your results.


[Back to Contents](#contents)


### Transformations {#transf}
<!-- @index Transformations; Response transformations; Back-transforming;
            `type = "response"`; --> 
In our running example with `pigs`, by now you are surely tired of seeing
all the answers on the `inverse(conc)` scale. What about estimating things on the
`conc` scale? You may have noticed that the `inverse` transformation has not
been forgotten; it is mentioned in the annotations below the `emmeans()` output.
[I'd also comment that having used `inverse(conc)` rather than `1/conc` as the
response variable in the model has made it easier to sort things out, because
`inverse()` is a named transformation that `emmeans()` can work with.]
We can back-transform the results by specifying `type = "response"` in any
function call where it makes sense. For instance,
```{r fig.alt = "interaction-style plots for 'RG4' after back-transforming. Compared to the plots of 'RG4' without back-transforming, these trends increase rather than decrease (due to the inverse transformation) and they fan-out somewhat as 'percent' increases. The values plotted here are obtainable via 'summary(RG4, type = \"response\")'"}
emmeans(RG4, "source", type = "response")
emmip(RG4, source ~ percent, type = "response")
```
We are now on the `conc` scale, and that will likely be less confusing. Compared with the earlier plots in which the trends were decreasing and parallel, this plot has them increasing (because of the inverse relationship) and non-parallel. An interaction that
occurs on the response scale is pretty well explained by a model with no interactions on the inverse scale.

Transformations have a lot of nuances, and we refer you to the [vignette of transformations](transformations.html) for more details.

[Back to Contents](#contents)



### Derived covariates {#depcovs}
<!-- @index Covariates!Derived; Quadratic terms; Polynomial regression;
            Examples!`mtcars`;        -->

You need to be careful when one covariate depends on the value of another. To
illustrate using the `datasets::mtcars` data, suppose we want to predict `mpg` using `cyl` (number of cylinders) as a factor `disp` (displacement) as a covariate, and
include a quadratic term for `disp`. Here are two equivalent models:
```{r}
mcmod1 <- lm(mpg ~ factor(cyl) + disp + I(disp^2), data = mtcars)
mtcars <- transform(mtcars, 
                    dispsq = disp^2)
mcmod2 <- lm(mpg ~ factor(cyl) + disp + dispsq, data = mtcars)
```
These two models have exactly the same predicted values. But look at the EMMs:
```{r}
emmeans(mcmod1, "cyl")
emmeans(mcmod2, "cyl")
```
Wow! Those are really different results -- even though the models are
equivalent. Why is this -- and which (if either) is right? To understand, look
at the reference grids:
```{r}
ref_grid(mcmod1)
ref_grid(mcmod2)
```
For both models, the reference grid uses the `disp` mean of 230.72. But for
`mcmod2`, `dispsq` is a separate covariate, and it is set to its mean of 68113. 
This is not right, because it is impossible to have `disp` equal to 230.72 and its
square equal to 68113 at the same time!
If we use consistent values of `disp` and`dispsq`, we get the same results as for `mcmod1`:
```{r}
emmeans(mcmod2, "cyl", at = list(disp = 230.72, dispsq = 230.72^2))
```

In summary, for polynomial models and others where some covariates depend on
others in nonlinear ways, it is definitely best to include that dependence in
the model formula (as in `mcmod1`) using `I()` or `poly()` expressions, or
alter the reference grid so that the dependency among covariates is correct.

[Back to Contents](#contents)



### Non-predictor variables {#params}
<!-- @index `params`; Variables that are not predictors; -->
Reference grids are derived using the variables in the right-hand side of the model
formula. But sometimes, these variables are not actually predictors. For example:
```{r, eval = FALSE}
deg <- 2
mod <- lm(y ~ treat * poly(x, degree = deg), data = mydata)
```
If we call `ref_grid()` or `emmeans()` with this model, it will try to construct
a grid of values of `treat`, `x`, and `deg` -- causing an error because `deg` is
not a predictor in this model. To get things to work correctly, you need to name
`deg` in a `params` argument, e.g.,
```{r, eval = FALSE}
emmeans(mod, ~ treat | x, at = list(x = 1:3), params = "deg")
```


[Back to Contents](#contents)

### Graphical displays {#plots}
<!-- @index Graphical displays; Plots!of EMMs@emms; Plots!Interaction-style; 
            `emmip()`; `plot()`;  -->
The results of `ref_grid()` or `emmeans()` (these are objects of class `emmGrid`)
may be plotted in two different 
ways. One we have already seen is an interaction-style plot, using `emmip()`. 

The formula
specification we used in `emmip(RG4, source ~ percent)` sets the *x* variable to be the one on the right-hand side and the "trace"
factor (what is used to define the different curves) on the left.

###### {#plot.emmGrid}
<!-- @index Plots!of confidence intervals@conf; `plot.emmGrid()`;
            `ref_grid()`!`cov.reduce`; -->
The other graphics option offered is the `plot()` method for `emmGrid` objects. 
Let's consider a different model for the `mtcars` data with both `cyl` and `disp`
as covariates
```{r}
mcmod3 <- lm(mpg ~ disp * cyl, data = mtcars)
```
In
the following, we display the estimates and 95% confidence intervals for
`RG4` in separate panels for each `source`.


```{r fig.alt = "Plot of side-by-side confidence intervals for 'cyl' means, in 3 panels corresponding to 'disp' values of 100, 200, and 300. The values plotted here are those in 'summary(EMM3)'"}
EMM3 <- emmeans(mcmod3, ~ cyl | disp, 
                at = list(cyl = c(4,6,8), disp = c(100,200,300)))
plot(EMM3)
```

This plot illustrates, as much as anything else, how silly it is to try to
predict mileage for a 4-cylinder car having high displacement, or an 8-cylinder
car having low displacement. The widths of the intervals give us a clue that we
are extrapolating. A better idea is to acknowledge that displacement largely
depends on the number of cylinders. So here is yet another way to 
use `cov.reduce` to modify the reference grid:
```{r}
mcrg <- ref_grid(mcmod3, at = list(cyl = c(4,6,8)),
                         cov.reduce = disp ~ cyl)
mcrg @ grid
```
The `ref_grid` call specifies that `disp` depends on `cyl`; so a linear model 
is fitted with the given formula and its fitted values are used as the `disp`
values -- only one for each `cyl`. If we plot this grid, the results are 
sensible, reflecting what the model predicts for typical cars with each 
number of cylinders:
```{r fig.height = 1.5, fig.alt = "Side-by-side CIs for cyl marginal means. The values plotted are obtainable via 'summary(mcrg)'"}
plot(mcrg)
```

###### {#ggplot}
<!-- @index **ggplot2** package; Plots!Enhancing with **ggplot2** functions -->
Wizards with the **ggplot2** package can further enhance these plots if 
they like. For example, we can add the data to an interaction plot -- this
time we opt to include confidence intervals and put the three sources 
in separate panels:
```{r fig.alt = "Enhanced interaction plot with CIs and observed data added; we have separate panels for the 3 diets, and the 4 percent conentrations in each panel"}
require("ggplot2")
emmip(mod4, ~ percent | source, CIs = TRUE, type = "response") +
    geom_point(aes(x = percent, y = conc), data = pigs, pch = 2, color = "blue")
```

[Back to Contents](#contents)


### Formatting results {#formatting}
<!-- @index Formatting results; `kable`; Exporting output; `xtable` method; 
  RMarkdown; -->
If you want to include `emmeans()` results in a report, you might want to have it
in a nicer format than just the printed output. We provide a little bit of help for this,
especially if you are using RMarkdown or SWeave to prepare the report. 
There is an `xtable` method for exporting these results, which we do not illustrate
here but it works similarly to `xtable()` in other contexts. Also, the `export` option 
the `print()` method allows the user to save exactly what is seen in the printed
output as text, to be saved or formatted as the user likes (see the documentation for `print.emmGrid` for details).
Here is an example using one of the objects above:
```{r, eval = FALSE}
ci <- confint(mcrg, level = 0.90, adjust = "scheffe")
xport <- print(ci, export = TRUE)
cat("<font color = 'blue'>\n")
knitr::kable(xport$summary, align = "r")
for (a in xport$annotations) cat(paste(a, "<br>"))
cat("</font>\n")
```
```{r, results = "asis", echo = FALSE}
ci <- confint(mcrg, level = 0.90, adjust = "scheffe")
xport <- print(ci, export = TRUE)
cat("<font color = 'blue'>\n")
knitr::kable(xport$summary, align = "r")
for (a in xport$annotations) cat(paste(a, "<br>"))
cat("</font>\n")
```


[Back to Contents](#contents)

### Using weights {#weights}
<!-- @index Means!Weighted; `emmeans()`!`weights`; Observational data; -->
As we have mentioned, `emmeans()` uses equal weighting by default, based on its foundations in experimental situations. When you have observational data, you are more likely to use unequal weights that more accurately characterize the population.
Accordingly, a `weights` argument is provided in `emmeans()`. For example, using
`weights = "cells"` in the call will weight the predictions according to their
cell frequencies (recall this information is retained in the reference grid).
This produces results comparable to ordinary marginal means:
```{r}
emmeans(mod4, "percent", weights = "cells")
```
Note that, as in the ordinary marginal means we obtained long ago,
the highest estimate is for `percent = 15` rather than `percent = 18`. It is
interesting to compare this with the results for a model that includes only
`percent` as a predictor.
```{r}
mod6 <- lm(inverse(conc) ~ factor(percent), data = pigs)
emmeans(mod6, "percent")
```
The EMMs in these two tables are identical, so in some sense, 
`weights = "cells"` amounts to throwing-out the uninvolved factors. 
However, note that these
outputs show markedly different standard errors. That is because the model
`mod4` accounts for variations due to `source` while `mod6` does not. The lesson
here is that it is possible to obtain statistics comparable to ordinary marginal
means, while still accounting for variations due to the factors that are being
averaged over.

[Back to Contents](#contents)

### Multivariate responses {#multiv}
<!-- @index Multivariate models; Examples!`MOats`
     Examples!Multivariate; `ref_grid()`!`mult.name` -->
The **emmeans** package supports various multivariate models. When there
is a multivariate response, the dimensions of that response are treated as if
they were levels of a factor. For example, the `MOats` dataset provided in the
package has predictors `Block` and `Variety`, and a four-dimensional response
`yield` giving yields observed with varying amounts of nitrogen added to the soil.
Here is a model and reference grid:
```{r}
MOats.lm <- lm (yield ~ Block + Variety, data = MOats)
ref_grid (MOats.lm, mult.name = "nitro")
```
So, `nitro` is regarded as a factor having 4 levels corresponding to the 4
dimensions of `yield`. We can subsequently obtain EMMs for any of the factors
`Block`, `Variety`, `nitro`, or combinations thereof. The argument `mult.name =
"nitro"` is optional; if it had been excluded, the multivariate levels would
have been named `rep.meas`.

[Back to Contents](#contents)





## Objects, structures, and methods {#emmobj}
<!-- @index `emmGrid` objects -->
The `ref_grid()` and `emmeans()` functions are introduced previously.
These functions, and a few related ones,
return an object of class `emmGrid`. From previously defined objects:
```{r}
class(RG4)
class(EMM.source)
```
If you simply show these objects, you get different-looking results:
```{r}
RG4

EMM.source
```
This is based on guessing what users most need to see when displaying the object.
You can override these defaults; for example to just see a quick summary of
what is there, do
```{r}
str(EMM.source)
```
<!-- @index `str()`; `summary()`; `summary_emm` object; `print.summary_emm()` -->
The most important method for `emmGrid` objects is `summary()`. It is used as the
print method for displaying an `emmeans()` result. For this reason, arguments for 
`summary()` may also be specified within most functions that produce `these kinds of
results.`emmGrid` objects. For example:
```{r}
# equivalent to summary(emmeans(mod4, "percent"), level = 0.90, infer = TRUE))
emmeans(mod4, "percent", level = 0.90, infer = TRUE)
```


This `summary()`
method for `emmGrid` objects) actually produces a `data.frame`, but with extra bells 
and whistles:
```{r}
class(summary(EMM.source))
```
This can be useful to know because if you want to actually *use* `emmeans()` results
in other computations, you should save its summary, and then you can access those
results just like you would access data in a data frame. The `emmGrid` object itself
is not so accessible. There is a `print.summary_emm()` function that is what
actually produces the output you see above -- a data frame with extra
annotations.



[Back to Contents](#contents)




## P values, "significance", and recommendations {#pvalues}
<!-- @index Significance!Assessing; 
    *P* values!Interpreting; ATOM; 
    ASA Statement on *P* values -->
There is some debate among statisticians and researchers about the appropriateness
of *P* values, and that the term "statistical significance" can be misleading. 
If you have a small *P* value, it *only* means that the
effect being tested is unlikely to be explained by chance variation alone, in
the context of the current study and the current statistical model underlying
the test. If you have a large *P* value, it *only* means that the observed
effect could plausibly be due to chance alone: it is *wrong* to conclude that
there is no effect.

The American Statistical Association has for some time been advocating very
cautious use of *P* values (Wasserstein *et al.* 2014) because it is
too often misinterpreted, and too often used carelessly. Wasserstein *et al.* (2019) 
even goes so far as to advise against *ever* using the term "statistically significant".
The 43 articles it accompanies in the same issue of *TAS*, recommend a
number of alternatives. I do not agree with all that is said in the main
article, and there are portions that are too cutesy or wander off-topic.
Further, it is quite dizzying to try to digest all the accompanying articles,
and to reconcile their disagreeing viewpoints. I do agree with one frequent point: that
there is really no substantive difference between $P=.051$ and $P=.049$,
and that one should avoid making sweeping statements based on a hard cutoff
at $P=.05$ or some other value.

For some time I included a summary of Wasserstein *et al.*'s
recommendations and their *ATOM* paradigm (Acceptance of uncertainty, Thoughtfulness, Openness,
Modesty). But in the meantime, I have handled a large number of user questions, and many of
those have made it clear to me that there are more important fish to fry in  a vignette 
section like this. It is just a fact that *P* values are used, and are useful. So I have my own set of recommendations regarding them.

#### A set of comparisons or well-chosen contrasts is more useful and interpretable than an omnibus *F* test {#recs1}
<!-- @index Recommended practices; Practices, recommended;
     *P* values!Adjusted; *F* test!Role in *post hoc* tests; -->
*F* tests are useful for model selection, but don't tell you anything specific about the nature of an effect. If *F* has a small *P* value, it suggests that there is some effect, somewhere. It doesn't
even necessarily imply that any two means differ statistically.

#### Use *adjusted* *P* values
When you run a bunch of tests, there is a risk of
making too many type-I errors, and adjusted *P* values (e.g., the Tukey adjustment 
for pairwise comparisons) keep you from making too many mistakes. That said,
it is possible to go overboard; and it's usually reasonable to regard each
"by" group as a separate family of tests for purposes of adjustment.
    
#### It is *not* necessary to have a significant *F* test as a prerequisite to doing comparisons or contrasts {#recs2}
<!-- @index LSD!protected; Model!Get it right first -->

... as long as an appropriate adjustment is used. There do
exist rules such as the "protected LSD" by which one is given license to do unadjusted
comparisons provided the $F$ statistic is "significant." However, this is a very weak
form of protection for which the justification is, basically, "if $F$ is significant,
you can say absolutely anything you want."

#### Get the model right first
Everything the **emmeans** package does is an interpretation of the model that you
fitted to the data. If the model is bad, you will get bad results from `emmeans()` and other
functions. Every single limitation of your model, be it presuming constant error variance,
omitting interaction terms, etc., becomes a limitation of the results `emmeans()` produces.
So do a responsible job of fitting the model. And if you don't know what's meant by that...

#### Consider seeking the advice of a statistical consultant {#recs3}
<!-- @index Statistical consultants; Consultants; Statistics is hard -->
Statistics is *hard*. It is a lot more than just running programs and copying output.
We began this vignette by emphasizing we need to start with a good model; that
is an artful task, and certainly what is shown here only hints at
what is required; you may need help with it.
It is *your* research; is it important that it be done right?
Many academic statistics and biostatistics departments can refer you to someone 
who can help.



[Back to Contents](#contents)


## Summary of main points {#summary}
  * EMMs are derived from a *model*. A different model for the same data may lead
    to different EMMs.
  * EMMs are based on a *reference grid* consisting of all combinations
    of factor levels, with each covariate set to its average (by default).
  * For purposes of defining the reference grid, dimensions of
    a multivariate response are treated as levels of a factor.
  * EMMs are then predictions on this reference grid, or marginal
    averages thereof (equally weighted by default).
  * Reference grids may be modified using `at` or other arguments
    for `ref_grid()`
  * Reference grids and `emmeans()` results may be plotted via `plot()`
    (for parallel confidence intervals) or `emmip()` (for an interaction-style
    plot).
  * Be cautious with the terms "significant" and "nonsignificant", and don't ever
    interpret a "nonsignificant" result as saying that there is no effect.
  * Follow good statistical practices such as getting the model right first, 
    and using adjusted *P* values
    for appropriately chosen families of comparisons or contrasts.
    
[Back to Contents](#contents)

### References
 Wasserstein RL, Lazar NA (2016)
   "The ASA's Statement on *p*-Values: Context, Process, and Purpose,"
   *The American Statistician*, **70**, 129--133,
   https://doi.org/10.1080/00031305.2016.1154108
 
 Wasserstein RL, Schirm AL, Lazar, NA (2019)
   "Moving to a World Beyond 'p < 0.05',"
   *The American Statistician*, **73**, 1--19,
   https://doi.org/10.1080/00031305.2019.1583913



## Further reading {#more}
The reader is referred to other vignettes for more details and advanced use.
The strings linked below are the names of the vignettes; i.e., they can
also be accessed via `vignette("`*name*`", "emmeans")`

  * Models that are supported in **emmeans** (there are lots of them)
    ["models"](models.html)
  * Confidence intervals and tests: 
    ["confidence-intervals"](confidence-intervals.html)
  * Often, users want to compare or contrast EMMs: ["comparisons"](comparisons.html)
  * Working with response transformations and link functions:
    ["transformations"](transformations.html)
  * Multi-factor models with interactions: ["interactions"](interactions.html)
  * Working with messy data and nested effects: ["messy-data"](messy-data.html)
  * Making predictions from your model: ["predictions"](predictions.html)
  * Examples of more sophisticated models (e.g., mixed, ordinal, MCMC)
    ["sophisticated"](sophisticated.html)
  * Utilities for working with `emmGrid` objects: ["utilities"](utilities.html)
  * Frequently asked questions: ["FAQs"](FAQs.html)
  * Adding **emmeans** support to your package: ["xtending"](xtending.html)

[Back to Contents](#contents)

[Index of all vignette topics](vignette-topics.html)

---
title: "Re-engineering CLDs"
author: "emmeans package, Version `r packageVersion('emmeans')`"
output: emmeans::.emm_vignette
vignette: >
  %\VignetteIndexEntry{Re-engineering CLDs}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, echo = FALSE, results = "hide", message = FALSE}
knitr::opts_chunk$set(echo = TRUE)
require(emmeans)
require(multcomp, quietly = TRUE)
```

<!-- @index Vignettes!Re-engineering CLDs; Compact letter displays (CLDs) -->

## Contents {#contents}
  1. [Introduction](#intro)
  2. [Grouping by underlining](#underlining)
  3. [Grouping using letters or symbols](#CLDs)
  4. [Simulated example](#CLD-example)
  5. [Alternative CLDs](#alt-CLDs)
     a. [Equivalence sets](#equiv-CLDs)
     b. [Significance sets](#signif-CLDs)
  6. [Conclusions](#concl)
  7. [References](#refs)

[Index of all vignette topics](vignette-topics.html)


### Introduction {#intro}
Compact letter displays (CLDs) are a popular way to display multiple comparisons,
especially when there are more than a few means to compare.
They are problematic, however, because they are prone to misinterpretation (more
details later). Here we present some background on CLDs, and show some adaptations
and alternatives that may be less prone to misinterpretation.

### Grouping by underlining {#underlining}
<!-- @index CLDs!Relation to underlining; Underlining, grouping by -->
CLDs generalize an "underlining" technique shown in some old experimental 
design and analysis textbooks, where results may be displayed something like this:
```
    trt1  ctrl   trt3   trt2   trt4
    ----------
          ------------------
```
The observed means are sorted in increasing order, so in this illustration,
`trt1` has the lowest mean, `ctrl` has the next lowest, and `trt4` has the
highest. The underlines group the means such that the extremes of each group
are *not* significantly different according to a statistical test conducted at a
specified alpha level. So in this illustration, `trt1` is significantly less
than `trt3`, `trt2`, and `trt4`, but not `ctrl`; and in fact `trt4` is
significantly greater than all the others.

This grouping also illustrates the dangers created by careless interpretations.
Some observers of this chart might say that "`trt1` and `ctrl` are equal" and
that "`ctrl`, `trt3`, and `trt2` are equal" -- when in fact we have merely
failed to show they are different. And further confusion results because
mathematical equality is transitive -- that is, these two statements of equality
would imply that `trt1` and `trt2` must be equal, seemingly contradicting the
finding that they are significantly different. Statistical nonsignificance does
*not* have the transitivity property!

### Grouping using letters or symbols {#CLDs}
<!-- @index CLDs!Principles; CLDs!**multcompView** package -->
The underlining method becomes problematic in any case where the standard errors
(SEs) of the comparisons are unequal -- for example if we have unequal sample
sizes, or a model with non-homogeneous variances. When the SEs are unequal, it
is possible, for example, for two adjacent means to be significantly different,
while two more distant ones do not differ significantly. If that happens, we
can't use underlines to group the means. The problem here is that lines are
continuous, and that continuity forces a continuum of groupings.

However, Piepho (2004) solved this problem by using *symbols* instead of lines,
and creating a display where any two means associated with the same symbol are
deemed to not be statistically different. Using symbols, it is possible to have
non-contiguous groupings, e.g., it is possible for two means to share a symbol
while an intervening one does not share the same symbol. Such a display is
called a *compact letter display*. We do not absolutely require actual letters,
just symbols that can be distinguished from one another. In the case where 
all the differences have equal SEs, the CLD will be the "same" as the result
of grouping lines, in that each distinct symbol will span a contiguous
range of means that can be interpreted as a grouping line.


The R package
**multcompView** (Graves *et al.*, 2019) provides an implementation of the
Piepho algorithm.  The multcomp package (Hothorn *et al.* 2008) provides a
generic `cld()` function, and the **emmeans** package provides a
`cld()` method for `emmGrid` objects. 

[Back to Contents](#contents)


### Simulated example {#CLD-example}
<!-- @index Examples!CLDs (simulated); CLDs!Simulated example -->
As a moving example, we simulate some data from an unbalanced design with 7 
treatments labeled A, B, ..., G; and fit a model to those
```{r}
set.seed(22.10)
mu = c(16, 15, 19, 15, 15, 17, 16)  # true means
n =  c(19, 15, 16, 18, 29,  2, 14)  # sample sizes
foo = data.frame(trt = factor(rep(LETTERS[1:7], n)))
foo$y = rnorm(sum(n), mean = mu[as.numeric(foo$trt)], sd = 1.0)

foo.lm = lm(y ~ trt, data = foo)
```
There are only four distinct true means underlying these seven treatments: Treatments `B`, `D`, and `E` have mean 15, treatments `A` and `G` have mean 16, and treatments `F` and `C`
are solo players with means 17 and 19 respectively.


#### Default CLD
Let's see a compact letter display for the marginal means.
(Call this **CLD #1**)
```{r}
foo.emm = emmeans(foo.lm, "trt")

library(multcomp)
cld(foo.emm)
```
The default "letters" for the **emmeans** implementation are actually numbers,
and we have three groupings indicated by the symbols `1`, `2`, and `3`. This
illustrates a case where grouping lines would not have worked, as we see in the
fact that group `1` is not contiguous. We have  (among other results) that
treatment `A` differs significantly from treatments `B`, `E`, `D`, `G`, and `C`
(at the default 0.05 significance level, with Tukey adjustment for multiple
testing). and that `C` is significantly greater than all the other means since
it is the only mean in group `3`.

An annotation warns that two means in the same group are not necessarily
the same; yet CLDs present a strong visual message that they are. The careless reader who makes this mistake will have trouble with the gap in group `1`, asking how `A` can differ
from `G` and yet `G` and `F`, are "the same." The explanation is that the SE of
`F` is huge, owing to its very small sample size, so it is hard for it to be
*statistically* different from other means. It is almost a gift to obtain a
non-contiguous grouping like this, as it forces the user to think more carefully
about what these grouping do and do not imply.

[Back to Contents](#contents)


### Alternative CLDs {#alt-CLDs}
<!-- @index CLDs!Alternative; CLDs!Re-engineered -->
Given the discussion above, one might wonder if it is possible to construct a CLD
in such a way that means sharing the same symbol *are* actually shown to be the same?
The answer is yes (otherwise we wouldn't have asked the question!) -- and it is quite easy to do,
thanks to two things:

  1. The algorithm for making grouping letters is based on a matrix of Boolean
     values associated with each pair of means, where we set the value `TRUE` for
     any pair that is statistically different (those means must receive different
     grouping letters), and `FALSE` otherwise; and the algorithm works for *any*
     such Boolean matrix
  2. There is such a thing as *equivalence testing*, by which we can establish
     with specified confidence that two means *do not* differ by more than a
     specified threshold $\delta$. One simple way to do this is to conduct *two
     one-sided tests* (TOST) whereby we can conclude that two means are equivalent
     if we show both that the difference exceeds $-\delta$ and is less than
     $+\delta$. We can use this TOST method to set each Boolean pair to `FALSE` is
     they are shown to be equivalent and `TRUE` if not shown to be equivalent.


#### Equivalence sets {#equiv-CLDs}
<!-- @index CLDs!For equivalence sets; Equivalence sets (with CLDs) -->
For our example, suppose, based on subject-matter considerations, that two means
that differ by less than 1.0 can be considered equivalent. In the **emmeans**
setup, we specify that we want equivalence testing simply by providing this
nonzero threshold value as a `delta` argument. In addition, we typically will
not make multiplicity adjustments to equivalence tests. Here is the result we
obtain (call this **CLD #2**)
```{r}
cld(foo.emm, delta = 1, adjust = "none")

```
So we obtain five groupings -- but only two if we ignore those that apply to
only one mean. We have that treatments `B` and `E` can be considered equivalent,
and treatments `E`, `D`, and `G` are considered equivalent. It is also important
to know that we *cannot* say that means in different groups are significantly
different.

Unlike CLD #1, we are showing only groupings of means that
we can *show* to be the same. The first four means, which were grouped together 
earlier, are now assigned to two equivalence groupings. And treatment `F` is not
grouped with any other mean -- which makes sense because we have so little
data on that treatment that we can hardly say anything.


#### Significance sets {#signif-CLDs}
<!-- @index CLDs!For significance sets; Significance sets (with CLDs) -->
Another variation is to simply reverse all the Boolean flags we used in
constructing CLD #1. Then two means will receive the same letter only if they
are significantly different. Thus, we really obtain *un*grouping letters.
We label these groupings "significance sets." The resulting display has
a distinctively different appearance, because common symbols tend to be
far apart rather than contiguous.
(Call this **CLD #3**)

```{r}
cld(foo.emm, signif = TRUE)
```
Here we have five significance sets. By comparing with CLD #1, you can
confirm that each significant difference shown explicitly here corresponds
to one shown implicitly (by *not* sharing a group) in CLD #1.

[Back to Contents](#contents)


### Conclusions {#concl}
Compact letter displays show symbols based on statistical testing results. In
such tests, we have strong conclusions or *findings* -- those that have small
*P* values, and weak conclusions or *non-findings* -- those where the *P* value
is not less than some $\alpha$. When we create visual flags such as grouping
lines or symbols, those come across visually as findings, and the problem with
standard CLDs is that those are the non-findings. We show two simple ways to use
software that creates CLDs so that actual findings are flagged with symbols. It
is hoped that people will find these modifications useful in visually displaying
comparisons among means.

### References {#refs}
<!-- @index CLDs!References -->
Graves, Spencer, Piepho Hans-Pieter, Selzer, Luciano, and Dorai-Raj, Sundar
(2019). **multcompView**: Visualizations of Paired Comparisons. R package
version 0.1-8, `https://CRAN.R-project.org/package=multcompView`
 
Hothorn, Torsten, Bretz, Frank, and Westfall, Peter (2008). Simultaneous
 Inference in General Parametric Models. *Biometrical Journal* **50(3)**,
 346--363.
  
Piepho, Hans-Peter (2004). An algorithm for a letter-based representation of all
pairwise comparisons, *Journal of Computational and Graphical Statistics*
**13(2)** 456--466.

[Back to Contents](#contents)

[Index of all vignette topics](vignette-topics.html)

---
title: "FAQs for emmeans"
author: "emmeans package, Version `r packageVersion('emmeans')`"
output: emmeans::.emm_vignette
vignette: >
  %\VignetteIndexEntry{FAQs for emmeans}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
```{r, echo = FALSE, results = "hide", message = FALSE}
require("emmeans")
knitr::opts_chunk$set(fig.width = 4.5, class.output = "ro")
options(show.signif.stars = FALSE)
```

<!-- @index Vignettes!FAQS; Frequently asked questions -->
This vignette contains answers to questions received from users or posted on discussion boards like [Cross Validated](https://stats.stackexchange.com) and 
[Stack Overflow](https://stackoverflow.com/)

## Contents {#contents}

 1. [What are EMMs/lsmeans?](#what)
 2. [What is the fastest way to obtain EMMs and pairwise comparisons?](#fastest)
 2. [I wanted comparisons, but all I get is (nothing)](#nopairs)
 2. [The model I fitted is not supported by **emmeans**](#qdrg)
 2. [I have three (or two or four) factors that interact](#interactions)
 3. [I have covariate(s) that interact(s) with factor(s)](#trends)
 3. [I have covariate(s) and am fitting a polynomial model](#polys)
 3. [I have transformed covariate(s)](#trancovs)
 3. [Some "significant" comparisons have overlapping confidence intervals](#CIerror)
 4. [All my pairwise comparisons have the same *P* value](#notfactor)
 5. [emmeans() doesn't work as expected](#numeric)
 6. [All or some of the results are NA](#NAs)
 6. [If I analyze subsets of the data separately, I get different results](#model)
 7. [My lsmeans/EMMs are way off from what I expected](#transformations)
 8. [Why do I get `Inf` for the degrees of freedom?](#asymp)
 10. [I get exactly the same comparisons for each "by" group](#additive)
 11. [My ANOVA *F* is significant, but no pairwise comparisons are](#anova)
 11. [I wanted differences, but instead I got ratios (or odds ratios)](#ratios)
 12. [I asked for a Tukey adjustments, but that's not what I got](#notukey)
 12. [`emmeans()` completely ignores my P-value adjustments](#noadjust)
 13. [`emmeans()` gives me pooled *t* tests, but I expected Welch's *t*](#nowelch)
 14. [I want to see more digits](#rounding)
 15. [I want to reproduce results from Stata's `margins` command](#stata)


[Index of all vignette topics](vignette-topics.html)

## What are EMMs/lsmeans? {#what}
<!-- @index EMMs!What are they?; Least-squares means -->
Estimated marginal means (EMMs), a.k.a. least-squares means, are
predictions on a reference grid of predictor settings, or marginal
averages thereof. See details in [the "basics" vignette](basics.html).

## What is the fastest way to obtain EMMs and pairwise comparisons? {#fastest}
<!-- @index Model!Importance of; `emmeans()`!Fastest way to get wrong answers -->
There are two answers to this (i.e., be careful what you wish for):

  1. Don't think; just fit the first model that comes to mind and run 
    `emmeans(model, pairwise ~ treatment)`. This is the fastest way; however,
    the results have a good chance of being invalid.
  2. *Do* think: Make sure you fit a model that really explains the responses.
     Do diagnostic residual plots, include appropriate interactions, account
     for heteroscadesticity if necessary, etc. This is the fastest way
     to obtain *appropriate* estimates and comparisons.

The point here is that `emmeans()` summarizes the *model*, not the data directly.
If you use a bad model, you will get bad results. And if you use a good model,
you will get appropriate results. It's up to you: it's your research---is it important?

[Back to Contents](#contents)

## I wanted comparisons, but all I get is (nothing) {#nopairs}
<!-- @index Comparisons result in `(nothing)`; `(nothing)` in output@nothing -->
This happens when you have only one estimate; and you can't compare it
with itself! This is turn can happen when you have a situation like this:
you have fitted 
```
mod <- lm(RT ~ treat, data = mydata)
```
and `treat` is coded in
your dataset with numbers 1, 2, 3, ... . Since `treat` is a numeric predictor,
`emmeans()` just reduces it to a single number, its mean, rather than separate values for
each treatment. Also, please note that this is almost certainly NOT the model you want,
because it forces an assumption that the treatment effects all fall on a straight line.
You should fit a model like 
```
mod <- lm(RT ~ factor(treat), data = mydata)
```
then you
will have much better luck with comparisons.

## The model I fitted is not supported by **emmeans** {#qdrg}
<!-- @index Models!Unsupported; `qdrg()` -->
You may still be able to get results using `qdrg()` (quick and dirty reference grid). 
See `?qdrg` for details and examples.

## I have three (or two or four) factors that interact {#interactions}
<!-- @index Multi-factor studies; Simple comparisons -->
Perhaps your question has to do with interacting factors, and you want to do
some kind of *post hoc* analysis comparing levels of one (or more) of the 
factors on the response. Some specific versions of this question...

  * Perhaps you tried to do a simple comparison for one treatment and 
    got a warning message you don't understand
  * You do pairwise comparisons of factor combinations and it's 
    just too much -- want just some of them
  * How do I even approach this?
  
My first answer is: plots almost always help. If you have factors A, B, and C,
try something like `emmip(model, A ~ B | C)`, which creates an interaction-style
plot of the predictions against B, for each A, with separate panels for each C.
This will help visualize what effects stand out in a practical way. This
can guide you in what post-hoc tests would make sense. See the
["interactions" vignette](interactions.html) for more discussion and examples.

[Back to Contents](#contents)

## I have covariate(s) that interact(s) with factor(s) {#trends}
<!-- @index Covariates!Interacting with factors -->
This is a situation where it may well be appropriate to compare the
slopes of trend lines, rather than the EMMs. See the 
`help("emtrends"()`")` and the discussion of this
topic in [the "interactions" vignette](interactions.html#covariates)

## I have covariate(s) and am fitting a polynomial model {#polys}
You need to be careful to define the reference grid consistently. For example,
if you use covariates `x` and `xsq` (equal to `x^2`) to fit a quadratic curve,
the default reference grid uses the mean of each covariate -- and `mean(xsq)` is
usually not the same as `mean(x)^2`. So you need to use `at` to ensure that the
covariates are set consistently with respect to the model. See [this subsection
of the "basics" vignette](basics.html#depcovs) for an example.

## I have transformed covariate(s) {#trancovs}
Suppose you have this code:
```
mod <- lm(log(strength) ~ machine + sqrt(diameter), data = fiber)
emmeans(mod, "machine")
```
This yields adjusted means of `log(strength)` for each machine. But be aware that
these results are predictions at the mean `diameter`, not at the mean `sqrt(diameter)`.
That is because **emmeans** determines the variables in a model via `all.vars()`:
```
all.vars(formula(mod))
## [1] "strength" "machine"  "diameter"
```
... and note that this includes `diameter`, not `sqrt(diameter)`.

It is quite reasonable to want the adjusted means computed at `mean(sqrt(diameter))`,
and if you want that to be done, your options are:

  1. Create a separate variable for `sqrt(diameter)` and use that in fitting the model
  2. Use `at = list(diameter = mean(sqrt(fiber$diameter))^2)`
  3. Use `cov.reduce = \(x) mean(sqrt(x))^2` (but this will be used for *all* covariates)

In (2) and (3), be careful that the transformed means be back-transformed.

Why can't we just use the means of the transformed predictors automatically?
Because in **R**, lots of model formulas include function calls for lots of
different reasons, and it would be very difficult to identify which are intended
to be treated as transformed predictors.

[Back to Contents](#contents)


## Some "significant" comparisons have overlapping confidence intervals {#CIerror}
<!-- @index Confidence intervals!Overlapping; Comparisons!with overlapping CIs@over -->
That can happen because 
*it is just plain wrong to use [non-]overlapping CIs for individual means to do comparisons*. 
Look at the printed results from something like `emmeans(mymodel, pairwise ~
treatment)`. In particular, note that the `SE` values are *not* the same*, and
may even have different degrees of freedom. Means are one thing statistically,
and differences of means are quite another thing. Don't ever mix them up, and
don't ever use a CI display for comparing means.

I'll add that making hard-line decisions about "significant" and
"non-significant" is in itself a poor practice. 
See [the discussion in the "basics" vignette](basics.html#pvalues) 


## All my pairwise comparisons have the same *P* value {#notfactor}
This will happen if you fitted a model where the treatments you want to 
compare were put in as a numeric predictor; for example `dose`, with 
values of 1, 2, and 3. If `dose` is modeled as numeric, you will be
fitting a linear trend in those dose values, rather than a model
that allows those doses to differ in arbitrary ways. Go back and fit
a different model using `factor(dose)` instead; it will make all the
difference. This is closely related to the next topic.



## emmeans() doesn't work as expected {#numeric}
<!-- @index Covariates!`emmeans()` doesn't work; Covariates!`cov.keep`; Covariates!`cov.reduce`;  -->
Equivalently, users ask how to get *post hoc* comparisons when we have
covariates rather than factors.
Yes, it does work, but you have to tell it the appropriate reference grid.

But before saying more, I have a question for you: *Are you sure your model
is meaningful?*

  * If your question concerns *only* two-level predictors such as `sex` 
    (coded 1 for female, 2 for male), no problem. The model will produce
    the same predictions as you'd get if you'd used these as factors.
  * If *any* of the predictors has 3 or more levels, you may have fitted
    a nonsense model, in which case you need to fit a different model that
    does make sense before doing any kind of *post hoc* analysis. For instance,
    the model contains a covariate `brand` (coded 1 for Acme, 2 for Ajax, and
    3 for Al's), this model is implying that the difference between 
    Acme and Ajax is exactly equal to the difference between Ajax and Al's,
    owing to the fact that a linear trend in `brand` has been fitted. 
    If you had instead coded 1 for Ajax, 2 for Al's, and 3 for Acme, the model
    would produce different fitted values. Ask yourself if it makes sense
    to have `brand = 2.319`. If not, you need to fit another model using 
    `factor(brand)` in place of `brand`.

Assuming that the appropriateness of the model is settled, the current
version of **emmeans** automatically casts two-value covariates as factors,
but not covariates having higher numbers of unique values. Suppose your model has
a covariate `dose` which was experimentally varied over four levels, but
can sensibly be interpreted as a numerical predictor. If you want to include the
separate values of `dose` rather than the mean `dose`, you can do that using
something like `emmeans(model, "dose", at = list(dose = 1:4))`, or 
`emmeans(model, "dose", cov.keep = "dose")`, or `emmeans(model, "dose", cov.keep = "4")`.
There are small differences between these. The last one regards any covariate
having 4 or fewer unique values as a factor.

See "altering the reference grid" in the ["basics" vignette](basics.html#altering)
for more discussion.

[Back to Contents](#contents)


## All or some of the results are NA {#NAs}
<!-- @index `NA`s in the output; `NonEst` values; Estimability issues -->
The **emmeans** package uses tools in the **estimability** package to determine
whether its results are uniquely estimable. For example, in a two-way model
with interactions included, if there are no observations in a particular cell
(factor combination), then we cannot estimate the mean of that cell.

When *some* of the EMMs are estimable and others are not, that is information
about missing information in the data. If it's possible to remove some terms
from the model (particularly interactions), that may make more things estimable 
if you re-fit with those terms excluded; 
but don't delete terms that are really needed for the model to fit well.

When *all* of the estimates are non-estimable, it could be symptomatic of
something else. Some possibilities include:

  * An overly ambitious model; for example, in a Latin square design, interaction
    effects are confounded with main effects; so if any interactions are included
    in the model, you will render main effects inestimable.
  * Possibly you have a nested structure that needs to be included in the
    model or specified via the `nesting` argument. Perhaps the levels that B
    can have depend on which level of A is in force. Then B is nested in A and
    the model should specify `A + A:B`, with no main effect for `B`.
  * Modeling factors as numeric predictors (see also the [related section on
    covariates](#numeric)). To illustrate, suppose you have data on particular
    state legislatures, and the model includes the predictors `state_name` as well
    as `dem_gov` which is coded 1 if the governor is a Democrat and 0 otherwise.
    If the model was fitted with `state_name` as a factor or character variable,
    but `dem_gov` as a numeric predictor, then, chances are, `emmeans()` will
    return non-estimable results. If instead, you use `factor(dem_gov)` in the
    model, then the fact that `state_name` is nested in `dem_gov` will be
    detected, causing EMMs to be computed separately for each party's
    states, thus making things estimable.
  * Some other things may in fact be estimable. For illustration, it's easy
    to construct an example where all the EMMs are non-estimable, but
    pairwise comparisons are estimable:
```{r}
pg <- transform(pigs, x = rep(1:3, c(10, 10, 9)))
pg.lm <- lm(log(conc) ~ x + source + factor(percent), data = pg)
emmeans(pg.lm, consec ~ percent)
```

The ["messy-data" vignette](messy-data.html) has more examples and discussion.


[Back to Contents](#contents)

## If I analyze subsets of the data separately, I get different results {#model}
<!-- @index Analysis of subsets of data; Subsets of data -->
Estimated marginal means summarize the *model* that you fitted to the data
-- not the data themselves. Many of the most common models rely on
several simplifying assumptions -- that certain effects are linear, that the
error variance is constant, etc. -- and those assumptions are passed forward
into the `emmeans()` results. Doing separate analyses on subsets usually
comprises departing from that overall model, so of course the results are 
different.


## My lsmeans/EMMs are way off from what I expected {#transformations}
<!-- @index `emmeans()`!Surprising results from; Poisson regression!Surprising results
     Logistic regression!Surprising results -->
First step: Carefully read the annotations below the output. Do they say
something like "results are on the log scale, not the response scale"?
If so, that explains it. A Poisson or logistic model involves a link function,
and by default, `emmeans()` produces its results on that same scale.
You can add `type = "response"` to the `emmeans()` call and it will
put the results of the scale you expect. But that is not always
the best approach. The ["transformations" vignette](transformations.html)
has examples and discussion.


## Why do I get `Inf` for the degrees of freedom? {#asymp}
<!-- @index Infinite degrees of freedom; Degrees of freedom!Infinite 
     *z* tests!vs. *t* tests; *t* tests vs. *z* tests -->
This is simply the way that **emmeans** labels asymptotic results
(that is, estimates that are tested against the standard normal distribution
-- *z* tests -- rather than the *t* distribution). Note that obtaining quantiles 
or probabilities from the *t* distribution with infinite degrees of freedom
is the same as obtaining the corresponding values from the standard normal.
For example:
```{r}
qt(c(.9, .95, .975), df = Inf)
qnorm(c(.9, .95, .975))
```
so when you see infinite d.f., that just means its a *z* test or a *z* confidence
interval.

[Back to Contents](#contents)



## I get exactly the same comparisons for each "by" group {#additive}
<!-- @index `by` groups!Identical comparisons -->
As mentioned elsewhere, EMMs summarize a *model*, not the data.
If your model does not include any interactions between the `by` variables
and the factors for which you want EMMs, then by definition, the
effects for the latter will be exactly the same regardless of the `by`
variable settings. So of course the comparisons will all be the same.
If you think they should be different, then you are saying that your model
should include interactions between the factors of interest and the 
`by` factors.

## My ANOVA *F* is significant, but no pairwise comparisons are {#anova}
<!-- @index *F* test!vs. pairwise comparisons@pairwz
        Analysis of variance!versus *post hoc* comparisons@post -->
First of all, you should not be making binary decisions of "significant" or
"nonsignificant." This is a simplistic view of *P* values that assigns an 
unmerited magical quality to the value 0.05. It is suggested
that you just report the *P* values actually obtained, and let your readers
decide how significant your findings are in the context of the scientific findings.


But to answer the question: This is a common misunderstanding of ANOVA. If *F*
has a particular *P* value, this implies only that *some contrast* among the
means (or effects) has the same *P* value, after applying the Scheffe
adjustment. That contrast may be very much unlike a pairwise comparison,
especially when there are several means being compared. Having an *F* statistic
with a *P* value of, say, 0.06, does *not* imply that any pairwise comparison
will have a *P* value of 0.06 or smaller. Again referring to the paragraph above,
just report the *P* value for each pairwise comparison, and don't try to
relate them to the *F* statistic.

Another consideration is that by default, *P* values for pairwise comparisons
are adjusted using the Tukey method, and the adjusted *P* values can be quite a
bit larger than the unadjusted ones. (But I definitely do *not* advocate using
no adjustment to "repair" this problem.)


## I wanted differences, but instead I got ratios (or odds ratios) {#ratios}
<!-- @index Comparisons!Obtaining differences rather than ratios;
            Ratios!but I wanted differences; Odds ratios!but I wanted differences  -->
When a transformation or link involves logs, then, unlike other transformations,
comparisons can be back-transformed into ratios -- and that is the default
behavior. If you really want differences and not ratios, you can re-grid the
means first. Re-gridding starts anew with everything on the response scale,
and no memory of the transformation.
```r
EMM <- emmeans(...)
pairs(regrid(EMM))   # or contrast(regrid(EMM), ...)
```
PS -- A side effect is that this causes the tests to be done using SEs obtained
by the delta method on the re-gridded scale, rather than on the link scale.
Re-gridding can be used with any transformation, not just logs, and it has the
same side effect.


## I asked for Tukey adjustments, but that's not what I got {#notukey}
<!-- @index Tukey adjustment!Ignored or changed -->
There are two reasons this could happen:

  1. There is only one comparison in each `by` group (see next topic).
  2. A Tukey adjustment is inappropriate. The Tukey adjustment is appropriate
  for pairwise comparisons of means. When you have some other set of contrasts,
  the Tukey method is deemed unsuitable and the Sidak method is used instead. A
  suggestion is to use `"mvt"` adjustment (which is exact); we don't default to
  this because it can require a lot of computing time for a large set of
  contrasts or comparisons.
  
  
## `emmeans()` completely ignores my P-value adjustments  {#noadjust}
<!-- @index *P* values!Adjustment is ignored -->
This happens when there are only two means (or only two in each `by` group).
Thus there is only one comparison. When there is only one thing to test, there
is no multiplicity issue, and hence no multiplicity adjustment to the *P*
values.

If you wish to apply a *P*-value adjustment to all tests across all groups,
you need to null-out the `by` variable and summarize, as in the following:
```r
EMM <- emmeans(model, ~ treat | group)   # where treat has 2 levels
pairs(EMM, adjust = "sidak")   # adjustment is ignored - only 1 test per group
summary(pairs(EMM), by = NULL, adjust = "sidak")   # all are in one group now
```
Note that if you put `by = NULL` *inside* the call to `pairs()`, then this
causes all `treat`,`group` combinations to be compared.
     
     
[Back to Contents](#contents)


## `emmeans()` gives me pooled *t* tests, but I expected Welch's *t* {#nowelch}
<!-- @index Welch's *t* comparisons; Pooled *t*!Instead of Welch's *t*;
            Model!Importance of getting it right; `emmeans()`!And the underlying model;
            Get the model right first -->
It is important to note that `emmeans()` and its relatives produce results based
on the *model object* that you provide -- not the data. So if your sample SDs
are wildly different, a model fitted using `lm()` or `aov()` is not a good model,
because those R functions use a statistical model that presumes that the errors
have constant variance. That is, the problem isn't in `emmeans()`, it's in
handing it an inadequate model object.

Here is a simple illustrative example. Consider a simple one-way experiment
and the following model:
```
mod1 <- aov(response ~ treat, data = mydata)
emmeans(mod1, pairwise ~ treat)
```
This code will estimate means and comparisons among treatments. All standard errors,
confidence intervals, and *t* statistics are based on the pooled residual SD
with *N - k* degrees of freedom (assuming *N* observations and *k* treatments).
These results are useful *only* if the underlying assumptions of `mod1` are correct -- 
including the assumption that the error SD is the same for all treatments.

Alternatively, you could fit the following model using generalized least-squares:
```
mod2 = nlme::gls(response ~ treat, data = mydata,
                 weights = varIdent(form = ~1 | treat))
emmeans(mod2, pairwise ~ treat)
```
This model specifies that the error variance depends on the levels of `treat`.
This would be a much better model to use when you have wildly different
sample SDs. The results of the `emmeans()` call will reflect this improvement
in the modeling. The standard errors of the EMMs will depend on the individual
sample variances, and the *t* tests of the comparisons will be in essence
the Welch *t* statistics with Satterthwaite degrees of freedom.

To obtain appropriate *post hoc* estimates, contrasts, and comparisons,
one must first find a model that successfully explains the peculiarities in
the data. This point cannot be emphasized enough. If you give `emmeans()` a
good model, you will obtain correct results; if you give it a bad model, 
you will obtain incorrect results. Get the model right *first*.


[Back to Contents](#contents)

## I want to see more digits {#rounding}
The summary display of `emmeans()` and other functions uses an optimal-digits
routine that shows only relevant digits. If you temporarily want to see more
digits, add `|> as.data.frame()` to the code line. In addition, piping
`|> data.frame()` will disable formatting of test statistics and P values (but 
unfortunately will also suppress messages below the tabular output).


## I want to reproduce results from Stata's `margins` command {#stata}
**Stata** is oriented towards observational data, and its `margins` command
takes a counterfactuals approach. Thus, try adding a `counterfactuals`
argument (see help for `ref_grid`). For example, to reproduce the results
in Stata's Example 6 for `margins`, do:
```r
margex <- haven::read_dta("https://www.stata-press.com/data/r18/margex.dta")
margex.glm <- glm(outcome ~ sex * factor(group) + age, data = margex,
                  family = binomial)
emmeans(margex.glm, "sex", counterfactuals = "sex")
```


[Index of all vignette topics](vignette-topics.html)

---
title: "Sophisticated models in emmeans"
author: "emmeans package, Version `r packageVersion('emmeans')`"
output: emmeans::.emm_vignette
vignette: >
  %\VignetteIndexEntry{Sophisticated models in emmeans}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
```{r, echo = FALSE, results = "hide", message = FALSE} 
require("emmeans")
require("lme4")
options(show.signif.stars = FALSE, width = 100) 
knitr::opts_chunk$set(fig.width = 4.5, class.output = "ro") 
```

<!-- @index Vignettes!Sophisticated models -->
This vignette gives a few examples of the use of the **emmeans** package to
analyze other than the basic types of models provided by the **stats**
package. Emphasis here is placed on accessing the optional capabilities
that are typically not needed for the more basic models. A reference for
all supported models is provided in the ["models" vignette](models.html).

## Contents {#contents}

 1. [Linear mixed models (lmer)](#lmer)
    a. [System options for lmerMod models](#lmerOpts)
    b. [Bias adjustment with random slopes](#random-slopes)
 2. [Models with offsets](#offsets)
 3. [Ordinal models](#ordinal)
 4. [Models fitted using MCMC methods](#mcmc)

 
[Index of all vignette topics](vignette-topics.html) 


## Linear mixed models (lmer) {#lmer}
<!-- @index `lmerMod` models; Examples!`Oats`; Examples!Split-plot experiment -->
Linear mixed models are really important in statistics. Emphasis here is placed
on those fitted using `lme4::lmer()`, but **emmeans** also supports other
mixed-model packages such as **nlme**.

To illustrate, consider the `Oats` dataset in the **nlme** package. It has the
results of a balanced split-plot experiment: experimental blocks are divided 
into plots that are randomly assigned to oat varieties, and the plots are
subdivided into subplots that are randomly assigned to amounts of nitrogen
within each plot. We will consider a linear mixed model for these data, 
excluding interaction (which is justified in this case). For sake of 
illustration, we will exclude a few observations.
```{r}
library(lme4)
Oats.lmer <- lmer(yield ~ Variety + factor(nitro) + (1|Block/Variety),
                        data = nlme::Oats, subset = -c(1,2,3,5,8,13,21,34,55))
```
Let's look at the EMMs for `nitro`:
```{r}
Oats.emm.n <- emmeans(Oats.lmer, "nitro")
Oats.emm.n
```
You will notice that the degrees of freedom are fractional: that is due to the
fact that whole-plot and subplot variations are combined when standard errors
are estimated. Different degrees-of-freedom methods are available. By default, 
the Kenward-Roger method is used, and that's why you see a message about the
**pbkrtest** package being loaded, as it implements that method. We may specify
a different degrees-of-freedom method via the optional argument `lmer.df`:
```{r}
emmeans(Oats.lmer, "nitro", lmer.df = "satterthwaite")
```

###### {#dfoptions}
<!-- @index Degrees of freedom; *z* tests; Asymptotic tests -->
This latest result uses the Satterthwaite method, which is implemented in the
**lmerTest** package. Note that, with this method, not only are the degrees of
freedom slightly different, but so are the standard errors. That is because the 
Kenward-Roger method also entails making a bias adjustment to the covariance
matrix of the fixed effects; that is the principal difference between the
methods. A third possibility is `"asymptotic"`:
```{r}
emmeans(Oats.lmer, "nitro", lmer.df = "asymptotic")
```
This just sets all the degrees of freedom to `Inf` -- that's **emmeans**'s way of
using *z* statistics rather than *t* statistics. The asymptotic methods tend to
make confidence intervals a bit too narrow and P values a bit too low; but they
involve much, much less computation. Note that the SEs are the same as obtained
using the Satterthwaite method.

Comparisons and contrasts are pretty much the same as with other models. As
`nitro` has quantitative levels, we might want to test polynomial contrasts:
```{r}
contrast(Oats.emm.n, "poly")
```
The interesting thing here is that the degrees of freedom are much larger than
they are for the EMMs. The reason is because `nitro` within-plot factor, so
inter-plot variations have little role in estimating contrasts among `nitro`
levels. On the other hand, `Variety` is a whole-plot factor, and there is not 
much of a bump in degrees of freedom for comparisons:
```{r}
emmeans(Oats.lmer, pairwise ~ Variety)
```

### System options for lmerMod models {#lmerOpts}
<!-- @index `lmerMod` models!System options for -->
The computation required to compute the adjusted covariance matrix and degrees 
of freedom may become cumbersome. Some user options (i.e., `emm_options()` 
calls) make it possible to streamline these computations through default methods
and limitations on them. First, the option `lmer.df`, which may have values of 
`"kenward-roger"`, `"satterthwaite"`, or `"asymptotic"` (partial matches are 
OK!) specifies the default degrees-of-freedom method.

The options `disable.pbkrtest` and `disable.lmerTest` may be `TRUE` or `FALSE`,
and comprise another way of controlling which method is used (e.g., the
Kenward-Roger method will not be used if `get_emm_option("disable.pbkrtest") ==
TRUE`). Finally, the options `pbkrtest.limit` and `lmerTest.limit`, which should
be set to numeric values, enable the given package conditionally on whether the
number of data rows does not exceed the given limit. The factory default is 3000
for both limits.


### Bias adjustment with random slopes {#random-slopes}
<!-- @index Bias adjustment!with random slopes; Random slopes!Bias adjustment;
    Examples!`ChickWeight` -->
In the [`cbpp` example](transformations.html#cbpp), we saw an example
where we applied a bias adjustment to the inverse transformation. To do that 
adjustment, we required an estimate of the total SD of the response.
That computation was (relatively) simple because the model had only random 
intercepts. But what if we have random slopes as well? The short answer is 
"it gets more complicated;" but here is an example of how we can muddle through it.

Our illustration is a model fitted to the `ChickWeight` data in the R **datasets** package. The data comprise weight determinations, over time (in days since birth), of chicks who are
randomized to different diets. Our model fits a trend in the square root of weight 
for each diet, with random intercepts and slopes for each chick (this is likely not the best model, but it's not totally stupid and it serves an an illustration).
```{r}
cw.lmer <- lmer(sqrt(weight) ~ Time*Diet + (1+Time|Chick), data = ChickWeight)
```
Our goal is to use this model to estimate the mean `weight` at times 5, 10, 15, and 20,
for each diet. Accordingly, let's get the estimates needed:
```{r}
cw.emm <- emmeans(cw.lmer, ~ Time|Diet, at = list(Time = c(5, 10, 15, 20)))
```
If we just summarize this with `type = "response", we will under-estimate the
mean weights. We need to apply a bias adjustment; but that involves providing an
estimate of the SD of each transformed response. The problem is that since
random slopes are involved, that SD depends on time. In particular, 
the model states that at time $t$,
$$ \sqrt Y_t = \mu_t + E + C + S\times t $$
where $\mu_t$ is the mean at time $t$, $E$ is the residual error, $C$ is the random intercept for chicks, and $S$ is the random slope for chicks. For purposes of bias correction, we need an estimate of $SD(E + C + S\times t)$ for each $t$.

The first step is to obtain an estimated covariance matrix for $(E, C, S)$:
```{r}
V <- matrix(0, nrow = 3, ncol = 3, dimnames = list(c("E","C","S"), c("E","C","S")))
V[1, 1] <- sigma(cw.lmer)^2              # Var(E)
V[2:3, 2:3] <- VarCorr(cw.lmer)$Chick    # Cov(C, S)
V
```
Now, using the matrix expression $Var(a'X) = a'Va$ for $X=c(E,C,S)$ and a given vector $a$, we can obtain the needed SDs:
```{r}
sig <- sapply(c(5, 10, 15, 20), function(t) {
    a <- c(1, 1, t)
    sqrt(sum(a * V %*% a))
})
sig  
```
As expected, these values increase with $t$.
Finally, we obtain the bias-adjusted estimated weights. We can use `sigma = sig`
as-is since the values follow the same ordering as `cw.emm@grid`.
```{r}
confint(cw.emm, type = "response", bias.adj = TRUE, sigma = sig)
```
This example illustrates that it is possible to deal with random slopes in
bias corrections. However it does require some fairly careful attention to
technical details and familiarity with matrix calculations; so if you don't have
a comfort level with those, it is best to get outside help.

#### Adding variables not in fixed model {addl.vars}
<!-- @index `addl.vars`; Random predictors!Accessing levels -->
Consider a model like
```
mod <- lmer(log(response) ~ treatment + (1 + x | subject), data = mydata)
```
Ordinarily, the reference grid will not include the variable `x` because it
is not part of the fixed-effects formula. However, you can include it via the
`addl.vars` argument:
```
EMM <- emmeans(mod, ~ x|treatment, addl.vars = "x", at = list(x = -1:1))
```
We will then obtain EMMs for combinations of `treatment` and `x`. 
(For a given `treatment`, all those means will be equal for every `x`.)
But the bias adjustments *will* depend on `x`.


[Back to Contents](#contents)


## Models with offsets {#offsets}
<!-- @index Offsets!Models with offsets; Examples!Insurance claims (SAS); `ref_grid()`!`offset` -->
If a model is fitted and its formula includes an `offset()` term, then by
default, the offset is computed and included in the reference grid. To
illustrate, consider a hypothetical dataset on insurance claims (used as an
[example in SAS's
documentation](https://support.sas.com/documentation/cdl/en/statug/63033/HTML/default/viewer.htm#statug_genmod_sect006.htm)).
There are classes of cars of varying counts (`n`), sizes (`size`), and age
(`age`), and we record the number of insurance claims (`claims`). We fit a
Poisson model to `claims` as a function of `size` and `age`. An offset of
`log(n)` is included so that `n` functions as an "exposure" variable.
```{r}
ins <- data.frame(
    n = c(500, 1200, 100, 400, 500, 300),
    size = factor(rep(1:3,2), labels = c("S","M","L")),
    age = factor(rep(1:2, each = 3)),
    claims = c(42, 37, 1, 101, 73, 14))
ins.glm <- glm(claims ~ size + age + offset(log(n)), 
               data = ins, family = "poisson")
```
First, let's look at the reference grid obtained by default:
```{r}
ref_grid(ins.glm)
```
Note that `n` is included in the reference grid and that its average value of 500
is displayed. But let's look at the EMMs:
```{r}
(EMM <- emmeans(ins.glm, "size", type = "response"))
```
We can see more explicitly what is happening by examining the internal structure of `EMM`:
```{r}
EMM@grid
```
and note that $\log(500) \approx 6.215$ is used as the offset value in calculating these estimates.

All this said, many users would like to ignore that average offset for this kind of model,
and instead use one corresponding to `n = 1`, because then the estimates we obtain are estimated rates per unit `n`. This may be accomplished by specifying an `offset` parameter in the
call:
```{r}
emmeans(ins.glm, "size", type = "response", offset = log(1))
```
An alternative way to achieve the same results is to set `n` equal to 1 in the reference grid itself (output not shown, because it is identical):
```{r eval = FALSE}
emmeans(ins.glm, "size", type = "response", at = list(n = 1))
```

By the way, you may set some other reference value for the rates. For example, if you want estimates of claims per 100 cars, simply use (results not shown):
```{r eval = FALSE}
emmeans(ins.glm, "size", type = "response", offset = log(100))
```
For more details on how offsets are handled, and how and why an `offset()` *model term*
is treated differently than an `offset` *argument* in model fitting, see
[the "xplanations" vignette](xplanations.html#offsets).

An additional complication may come up in models zero-inflated or hurdle models.
In those cases, it is somewhat ambiguous what one might mean by a "rate", but one interpretation
would be to just go with the above techniques with estimates for just the Poisson
component of the model. Another approach would be to estimate the response mean with
the zero-inflation included, and divide by the appropriate offset. This can be done,
but it is messy. An example is given on the 
[Cross-Validated site](https://stats.stackexchange.com/questions/620842/help-interpreting-zeroinfl-results-from-emmeans/621108?noredirect=1#comment1155903_621108).

[Back to Contents](#contents)



## Ordinal models {#ordinal}
<!-- @index Ordinal models; Examples!`wine`; Examples!Ordinal model
  Ordinal models!Latent scale -->
Ordinal-response models comprise an example where several options are 
available for obtaining EMMs. To illustrate, consider the `wine` data
in the **ordinal** package. The response is a rating of bitterness on
a five-point scale. we will consider a probit model in two factors
during fermentation: `temp` (temperature) and `contact` (contact with 
grape skins), with the judge making the rating as a scale predictor:
```{r}
require("ordinal")
wine.clm <- clm(rating ~ temp + contact, scale = ~ judge,
                data = wine, link = "probit")
```
(in earlier modeling, we found little interaction between the factors.)
Here are the EMMs for each factor using default options:
```{r}
emmeans(wine.clm, list(pairwise ~ temp, pairwise ~ contact))
```
These results are on the "latent" scale; the idea is that there is a continuous 
random variable (in this case normal, due to the probit link) having a mean that
depends on the predictors; and that the ratings are a discretization of the 
latent variable based on a fixed set of cut points (which are estimated). In 
this particular example, we also have a scale model that says that the variance 
of the latent variable depends on the judges. The latent results are quite a bit
like those for measurement data, making them easy to interpret. The only catch 
is that they are not uniquely defined: we could apply a linear transformation to
them, and the same linear transformation to the cut points, and the results
would be the same.

###### {#ordlp}
<!-- @index Ordinal models!Linear-predictor scale -->
The `clm` function actually fits the model using an ordinary probit model
but with different intercepts for each cut point. We can get detailed 
information for this model by specifying `mode = "linear.predictor"`:
```{r}
tmp <- ref_grid(wine.clm, mode = "lin")
tmp
```
Note that this reference grid involves an additional constructed predictor named
`cut` that accounts for the different intercepts in the model. Let's obtain EMMs
for `temp` on the linear-predictor scale:
```{r}
emmeans(tmp, "temp")
```
These are just the negatives of the latent results obtained earlier (the sign is
changed to make the comparisons go the right direction). Closely related to this
is `mode = "cum.prob"` and `mode = "exc.prob"`, which simply transform the 
linear predictor to cumulative probabilities and exceedance (1 - cumulative) 
probabilities. These modes give us access to the details of the fitted model but
are cumbersome to use for describing results. When they can become useful is
when you want to work in terms of a particular cut point. Let's look at `temp`
again in terms of the probability that the rating will be at least 4:
```{r}
emmeans(wine.clm, ~ temp, mode = "exc.prob", at = list(cut = "3|4"))
```

###### {#ordprob}
<!-- @index Ordinal models!`prob` and `mean.class` -->
There are yet more modes! With `mode = "prob"`, we obtain estimates of the probability distribution of each rating. Its reference grid includes a factor
with the same name as the model response -- in this case `rating`. We usually 
want to use that as the primary factor, and the factors of interest as `by` variables:
```{r}
emmeans(wine.clm, ~ rating | temp, mode = "prob")
```
Using `mode = "mean.class"` obtains the average of these probability 
distributions as probabilities of the integers 1--5:
```{r}
emmeans(wine.clm, "temp", mode = "mean.class")
```

And there is a mode for the scale model too. In this example, the scale model
involves only judges, and that is the only factor in the grid:
```{r}
summary(ref_grid(wine.clm, mode = "scale"), type = "response")
```
Judge 8's ratings don't vary much, relative to the others.
The scale model is in terms of log(SD). Again, these are not uniquely
identifiable, and the first level's estimate is set to log(1) = 0. so,
actually, each estimate shown is a comparison with judge 1.

[Back to Contents](#contents)


## Models fitted using MCMC methods {#mcmc}
<!-- @index Bayesian models; Examples!Bayesian model; Examples!`cbpp`
    `rstanarm`; `hpd.summary()`; `summary()`!HPD intervals -->
To illustrate **emmeans**'s support for models fitted using MCMC methods,
consider the `example_model` available in the **rstanarm** package. 
The example concerns CBPP, a serious disease of cattle in Ethiopia.
A generalized linear mixed model was fitted to the data using the
code below. (This is a Bayesian equivalent of the frequentist model we
considered in the ["Transformations" vignette](transformations.html#cbpp).)
In fitting the model, we first set the contrast coding to `bayestestR::contr.bayes`
because this equalizes the priors across different treatment levels 
(a correction from an earlier version of this vignette.)
We subsequently obtain the reference grids for these models in the usual way.
For later use, we also fit the same model with just the prior information.
<!--- I'm faking this; I actually saved the ref_grid in a system file --->
```{r eval = FALSE}
cbpp <- transform(lme4::cbpp, unit = 1:56)
require("bayestestR")
options(contrasts = c("contr.bayes", "contr.poly"))
cbpp.rstan <- rstanarm::stan_glmer(
    cbind(incidence, size - incidence) ~ period + (1|herd) + (1|unit),
    data = cbpp, family = binomial,
    prior = student_t(df = 5, location = 0, scale = 2, autoscale = FALSE),
    chains = 2, cores = 1, seed = 2021.0120, iter = 1000)
cbpp_prior.rstan <- update(cbpp.rstan, prior_PD = TRUE)
cbpp.rg <- ref_grid(cbpp.rstan)
cbpp_prior.rg <- ref_grid(cbpp_prior.rstan)
```
<!--- here's the system file with the ref_grid --->
```{r echo = FALSE}
cbpp.rg <- do.call(emmobj, 
    readRDS(system.file("extdata", "cbpprglist", package = "emmeans")))
cbpp_prior.rg <- do.call(emmobj, 
    readRDS(system.file("extdata", "cbpppriorrglist", package = "emmeans")))
cbpp.sigma <- readRDS(system.file("extdata", "cbppsigma", package = "emmeans"))
```
Here is the structure of the reference grid:
```{r}
cbpp.rg
```
So here are the EMMs (no averaging needed in this simple model):
```{r}
summary(cbpp.rg)
```
The summary for EMMs of Bayesian models shows the median of the posterior
distribution of each estimate, along with highest posterior density (HPD)
intervals. Under the hood, the posterior sample of parameter estimates is used
to compute a corresponding sample of posterior EMMs, and it is those that are
summarized. (Technical note: the summary is actually rerouted to the
`hpd.summary()` function.

###### {#bayesxtra}
<!-- @index `as.mcmc()`; **coda** package; **bayesplot** package; 
    **bayestestR** package; Bayes factor; Region of practical equivalence; ROPE -->
We can access the posterior EMMs via the `as.mcmc` method for `emmGrid` objects.
This gives us an object of class `mcmc` (defined in the **coda** package), which
can be summarized and explored as we please.
```{r}
require("coda")
summary(as.mcmc(cbpp.rg))
```
Note that `as.mcmc` will actually produce an `mcmc.list` when there is more than
one chain present, as in this example.
The 2.5th and 97.5th quantiles are similar, but not identical, to the
95% confidence intervals in the frequentist summary.

The **bayestestR** package provides `emmGrid` methods for most of its description
and testing functions. For example:
```{r}
bayestestR::bayesfactor_parameters(pairs(cbpp.rg), prior = pairs(cbpp_prior.rg))
bayestestR::p_rope(pairs(cbpp.rg), range = c(-0.25, 0.25))
```
Both of these sets of results suggest that period 1 is different from the others. 
For more information on these methods, refer to [the CRAN page for **bayestestR**](https://cran.r-project.org/package=bayestestR) 
and its vignettes,
e.g., the one on Bayes factors.


### Bias-adjusted incidence probabilities {#bias-adj-mcmc}
<!-- @index Bias adjustment!in Bayesian models -->
Next, let us consider the back-transformed results. As is discussed with the
[frequentist model](transformations.html#cbpp), there are random effects present,
and if wee want to think in terms of marginal probabilities across all herds and units,
we should correct for bias; and to do that, we need the standard deviations of
the random effects. The model object has MCMC results for the random effects of
each herd and each unit, but after those, there are also summary results for the
posterior SDs of the two random effects. (I used the `colnames` function to find
that they are in the 78th and 79th columns.)
<!-- We already read this in earlier, and here's the faked code -->
```{r eval = FALSE}
cbpp.sigma = as.matrix(cbpp.rstan$stanfit)[, 78:79]
```
Here are the first few:
```{r}
head(cbpp.sigma)
```

So to obtain bias-adjusted marginal probabilities, obtain the resultant SD
and regrid with bias correction:
```{r}
totSD <- sqrt(apply(cbpp.sigma^2, 1, sum))
cbpp.rgrd <- regrid(cbpp.rg, bias.adjust = TRUE, sigma = totSD)
summary(cbpp.rgrd)
```
Here is a plot of
the posterior incidence probabilities, back-transformed:
```{r, fig.alt = "kernel denity estimates for each of the 4 periods. Their medians and spreads decrease with period, and period 1 is especially different. See the previous summary table for the numerical values of the estimated means"}
bayesplot::mcmc_areas(as.mcmc(cbpp.rgrd))
```

... and here are intervals for each period compared with its neighbor:
```{r}
contrast(cbpp.rgrd, "consec", reverse = TRUE)
```
The only interval that excludes zero is the one that compares periods 1 and 2.

### Bayesian prediction {#predict-mcmc}
<!-- @index Predictions!Bayesian models; 
            Predictions!Posterior predictive distribution -->
To predict from an MCMC model, just specify the
`likelihood` argument in `as.mcmc`. Doing so causes the function to
simulate data from the posterior predictive distribution. 
For example, if we want to predict the CBPP incidence in future herds of 
25 cattle, we can do:
```{r, fig.alt = "Histograms of the predictive distributions for each period. The one for period 1 has bins from 0 to 15; the number of bins decreases until period 4 has only bins for 0 through 5."}
set.seed(2019.0605)
cbpp.preds <- as.mcmc(cbpp.rgrd, likelihood = "binomial", trials = 25)
bayesplot::mcmc_hist(cbpp.preds, binwidth = 1)
```

[Back to Contents](#contents)

[Index of all vignette topics](vignette-topics.html)

---
title: "Prediction in **emmeans**"
author: "emmeans package, Version `r packageVersion('emmeans')`"
output: emmeans::.emm_vignette
vignette: >
  %\VignetteIndexEntry{Prediction in emmeans}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
```{r, echo = FALSE, results = "hide", message = FALSE} 
require("emmeans") 
options(show.signif.stars = FALSE, width = 100) 
knitr::opts_chunk$set(fig.width = 4.5, class.output = "ro") 
```

<!-- @index Vignettes!Predictions -->
In this vignette, we discuss **emmeans**'s rudimentary capabilities for constructing prediction intervals. 

## Contents {#contents}

 1. [Focus on reference grids](#ref-grid)
 2. [Need for an SD estimate](#sd-estimate)
 3. [Feedlot example](#feedlot)
 4. [Predictions on particular strata](#strata)
 5. [Predictions with Bayesian models](#bayes)

[Index of all vignette topics](vignette-topics.html) 

##  Focus on reference grids {#ref-grid}
<!-- @index Predictions!Reference grid; Reference grids!Prediction on -->
Prediction is not the central purpose of the **emmeans** package. Even its name
refers to the idea of obtaining marginal averages of fitted values; and it is a
rare situation where one would want to make a prediction of the average of
several observations. We can certainly do that if it is truly desired,
but almost always, predictions should be based on the reference grid itself 
(i.e., *not* the result of an `emmeans()` call),
inasmuch as a reference grid comprises combinations of model predictors. 


## Need for an SD estimate {#sd-estimate}
<!-- @index Predictions!Error SD; -->
A prediction interval requires an estimate of the error standard deviation,
because we need to account for both the uncertainty of our point predictions
and the uncertainty of outcomes centered on those estimates. By its current design,
we save the value (if any) returned by `stats::sigma(object)` when a reference grid is constructed for a model `object`. Not all models provide a `sigma()` method,
in which case an error is thrown if the error SD is not manually specified.
Also, in many cases, there may be a `sigma()` method, but it does not
return the appropriate value(s) in the context of the needed predictions.
(In an object returned by `lme4::glmer(), for example, `sigma()` seems to always 
returns 1.0.) Indeed, as will be seen in the example that follows,
one usually needs to construct a manual SD estimate when the model is a mixed-effects model.

So it is essentially always important to think very specifically about
whether we are using an appropriate value. You may check the value being
assumed by looking at the `misc` slot in the reference grid:
```{r eval = FALSE}
rg <- ref_grid(model)
rg@misc$sigma
```

Finally, `sigma` may be a vector, as long as it is conformable with the estimates
in the reference grid. This would be appropriate, for example, with a model
fitted by `nlme::gls()` with some kind of non-homogeneous error structure. It may take
some effort, as well as a clear understanding of the model and its structure, to
obtain suitable SD estimates. It was suggested to me that the function
`insight::get_variance()` may be helpful -- especially when working with
an unfamiliar model class. Personally, I prefer to make sure I understand the 
structure of the model object and/or its summary to ensure I am not going astray.

[Back to Contents](#contents)

## Feedlot example {#feedlot}
<!-- @index Examples!`feedlot`; Predictions!Total SD; Predictions!graphics -->
To illustrate, consider the `feedlot` dataset provided with the package.
Here we have several herds of feeder cattle that are sent to feed lots and given
one of three diets. The weights of the cattle are measured at time of entry
(`ewt`) and at time of slaughter (`swt`). Different herds have possibly
different entry weights, based on breed and ranching practices, so we will
center each herd's `ewt` measurements, then use that as a covariate in a mixed
model:
```{r, message = FALSE}
feedlot = transform(feedlot, adj.ewt = ewt - predict(lm(ewt ~ herd)))
require(lme4)
feedlot.lmer <- lmer(swt ~ adj.ewt + diet + (1|herd), data = feedlot)
feedlot.rg <- ref_grid(feedlot.lmer, at = list(adj.ewt = 0))
summary(feedlot.rg)  ## point predictions
```

Now, as advised, let's look at the SDs involved in this model:
```{r}
lme4::VarCorr(feedlot.lmer)  ## for the model
feedlot.rg@misc$sigma  ## default in the ref. grid
```
So the residual SD will be assumed in our prediction intervals 
if we don't specify something else. And we *do* want something else,
because in order to predict the slaughter weight of an arbitrary animal,
without regard to its herd, we need to account for the variation among
herds too, which is seen to be considerable. The two SDs reported by `VarCorr()`
are assumed to represent independent sources of variation, so they may be
combined into a total SD using the Pythagorean Theorem.
We will update the reference grid with the new value:
```{r}
feedlot.rg <- update(feedlot.rg, sigma = sqrt(77.087^2 + 57.832^2))
```

We are now ready to form prediction intervals. To do so, simply call the 
`predict()` function with an `interval` argument:
```{r}
predict(feedlot.rg, interval = "prediction")
```
These results may also be displayed graphically:
```{r, fig.height = 2, fig.alt = "Side-by-side CIs and PIs. The PIs are much wider, and have the endpoints found in the preceding predit() call"}
plot(feedlot.rg, PIs = TRUE)
```

The inner intervals are confidence intervals, and the outer ones are the 
prediction intervals.

Note that the SEs for prediction are considerably greater than the SEs for
estimation in the original summary of `feedlot.rg`. Also, as a sanity check,
observe that these
prediction intervals cover about the same ground as the original data:
```{r}
range(feedlot$swt)
```

By the way, we could have specified the desired `sigma` value as an additional
`sigma` argument in the `predict()` call, rather than updating the `feedlot.rg`
object.

[Back to Contents](#contents)


## Predictions on particular strata {#strata}
<!-- @index Predictions!on Particular strata -->
Suppose, in our example, we want to predict `swt` for one or more particular herds.
Then the total SD we computed is not appropriate for that purpose, because that includes variation among herds.

But more to the point, if we are talking about particular herds, then we are really regarding `herd` as a fixed effect of interest; so the expedient thing to do is
to fit a different model where `herd` is a fixed effect:
```{r}
feedlot.lm <- lm(swt ~ adj.ewt + diet + herd, data = feedlot)
```
So to predict slaughter weight for herds `9` and `19`:
```{r}
newrg <- ref_grid(feedlot.lm, at = list(adj.ewt = 0, herd = c("9", "19")))
predict(newrg, interval = "prediction", by = "herd")
```
This is an instance where the default `sigma` was already correct (being the only error SD we have available). The SD value is comparable to the residual SD in the previous model, and the prediction SEs are smaller than those for predicting over all herds.

[Back to Contents](#contents)

## Predictions with Bayesian models {#bayes}
<!-- @index Predictions!Bayesian models -->
For models fitted using Bayesian methods, these kinds of prediction intervals are
available only by forcing a frequentist analysis (`frequentist = TRUE`).

However, a better and more flexible approach with Bayesian models is to simulate
observations from the posterior predictive distribution. This is done via `as.mcmc()` and specifying a `likelihood` argument. An example is given in the ["sophisticated models" vignette](sophisticated.html#predict-mcmc).

[Back to Contents](#contents)

[Index of all vignette topics](vignette-topics.html) 

---
title: "For developers: Extending **emmeans**"
author: "emmeans package, Version `r packageVersion('emmeans')`"
output: emmeans::.emm_vignette
vignette: >
  %\VignetteIndexEntry{For developers: Extending emmeans}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
```{r, echo = FALSE, results = "hide", message = FALSE}
require("emmeans")
knitr::opts_chunk$set(fig.width = 4.5, class.output = "ro")
set.seed(271828)
```

<!-- @index Vignettes!Extending **emmeans** -->

## Contents {#contents}
This vignette explains how developers may incorporate **emmeans** support in their packages.
If you are a user looking for a quick way to obtain results for an unsupported model,
you are probably better off trying to use the `qdrg()` function.

  1. [Introduction](#intro)
  2. [Data example](#dataex)
  3. [Supporting `rlm` objects](#rlm)
  4. [Supporting `lqs` objects](#lqs)
  5. [Communication between methods](#communic)
  5. [Hook functions](#hooks)
  6. [Re-gridded basis](#regridded)
  6. [Exported methods from **emmeans**](#exported)
  7. [Existing support for `rsm` objects](#rsm)
  7. [Dispatching and restrictions](#dispatch)
  8. [Exporting and registering your methods](#exporting)
  9. [Conclusions](#concl)
  
[Index of all vignette topics](vignette-topics.html)


## Introduction {#intro}
<!-- @index `recover_data()`; `emm_basis()` -->
Suppose you want to use **emmeans** for some type of model that it doesn't (yet) support. Or, suppose you have developed a new package with a fancy model-fitting function, and you'd like it to work with **emmeans**. What can you do? Well, there is hope because **emmeans** is designed to be extended. 

The first thing to do is to look at the help page for extending the package:
```{r eval=FALSE}
help("extending-emmeans", package="emmeans")
```
It gives details about the fact that you need to write two S3 methods, `recover_data` and `emm_basis`, for the class of object that your model-fitting function returns. The `recover_data`  method is needed to recreate the dataset so that the reference grid can be identified. The `emm_basis` method then determines the linear functions needed to evaluate each point in the reference grid and to obtain associated information---such as the variance-covariance matrix---needed to do estimation and testing.

These methods must also be exported from your package so that they are available
to users. See the section on [exporting the methods](#exporting) for details and
suggestions.

This vignette presents an example where suitable methods are developed, and discusses a few issues that arise.

[Back to Contents](#contents)


## Data example {#dataex}
The **MASS** package contains various functions that do robust or outlier-resistant model fitting. We will cobble together some **emmeans** support for these. But first, let's create a suitable dataset (a simulated two-factor experiment) for testing.
```{r}
fake = expand.grid(rep = 1:5, A = c("a1","a2"), B = c("b1","b2","b3"))
fake$y = c(11.46,12.93,11.87,11.01,11.92,17.80,13.41,13.96,14.27,15.82,
           23.14,23.75,-2.09,28.43,23.01,24.11,25.51,24.11,23.95,30.37,
           17.75,18.28,17.82,18.52,16.33,20.58,20.55,20.77,21.21,20.10)
```
The `y` values were generated using predetermined means and Cauchy-distributed errors. There are some serious outliers in these data.

## Supporting `rlm` {#rlm}
<!-- @index Examples!Robust regression; Examples!`rlm` objects -->
The **MASS** package provides an `rlm` function that fits robust-regression models using *M* estimation. We'll fit a model using the default settings for all tuning parameters:
```{r}
library(MASS)
fake.rlm = rlm(y ~ A * B, data = fake)

library(emmeans)
emmeans(fake.rlm, ~ B | A)
```
The first lesson to learn about extending **emmeans** is that sometimes, it already works! It works here because `rlm` objects inherit from `lm`, which is supported by the **emmeans** package, and `rlm` objects aren't enough different to create any problems.

[Back to Contents](#contents)


## Supporting `lqs` objects {#lqs}
<!-- @index Examples!`lqs` objects -->
The **MASS** resistant-regression functions `lqs`, `lmsreg`, and `ltsreg` are another story, however. They create `lqs` objects that are not extensions of any other class, and have other issues, including not even having a `vcov` method. So for these, we really do need to write new methods for `lqs` objects. First, let's fit a model.
```{r}
fake.lts = ltsreg(y ~ A * B, data = fake)
```

### The `recover_data` method {#rd.lqs}
<!-- @index `recover_data()!for `lqs` objects@lqs -->
It is usually an easy matter to write a `recover_data` method. Look at the one for `lm` objects:
```{r}
emmeans:::recover_data.lm
```
Note that all it does is obtain the `call` component and call the method for class `call`, with additional arguments for its `terms` component and `na.action`. It happens that we can access these attributes in exactly the same way as for `lm` objects; so:
```{r}
recover_data.lqs = emmeans:::recover_data.lm
```
Let's test it:
```{r}
rec.fake = recover_data(fake.lts)
head(rec.fake)
```
Our recovered data excludes the response variable `y` (owing to the `delete.response` call), and this is fine. 

#### Special arguments {#rdargs}
<!-- @index `recover_data()!`data` and `params` arguments;
    `recover_data.call()`!`frame` argument -->
By the way, there are two special arguments `data` and `params` that may be handed to `recover_data` via `ref_grid` or `emmeans` or a related function; and you may need to provide for if you don't use the `recover_data.call` function. The `data` argument is needed to cover a desperate situation that occurs with certain kinds of models where the underlying data information is not saved with the object---e.g., models that are fitted by iteratively modifying the data. In those cases, the only way to recover the data is to for the user to give it explicitly, and `recover_data` just adds a few needed attributes to it. 

The `params` argument is needed when the model formula refers to variables besides predictors. For example, a model may include a spline term, and the knots are saved in the user's environment as a vector and referred to in the call to fit the model. In trying to recover the data, we try to construct a data frame containing all the variables present on the right-hand side of the model, but if some of those are scalars or of different lengths than the number of observations, an error occurs. So you need to exclude any names in `params` when reconstructing the data.

Many model objects contain the model frame as a slot; for example, a model fitted with
`lm(..., model = TRUE)` has a member `$model` containing the model frame. This can
be useful for recovering the data, provided none of the predictors are transformed
(when predictors are transformed, the original predictor values are not in the model
frame so it's harder to recover them). Therefore, when the model frame is
available in the model object, it should be provided in the `frame` argument of `recover_data.call()`; then when `data = NULL`, a check is made on `trms`, and if it
has no function calls, then `data` is set to `frame`. Of course, in the rarer case 
where the original data are available in the model object, specify that as `data`.

#### Error handling {#rderrs}
<!-- @index `recover_data()!Error handling -->
If you check for any error conditions in `recover_data`, simply have it return a character string with the desired message, rather than invoking `stop`. This provides a cleaner exit. The reason is that whenever `recover_data` throws an error, an informative message suggesting that `data` or `params` be provided is displayed. But a character return value is tested for and throws a different error with your string as the message.

### The `emm_basis` method {#ebreqs}
<!-- @index `emm_basis()`!Arguments and returned value -->
The `emm_basis` method has four required arguments:
```{r}
args(emmeans:::emm_basis.lm)
```
These are, respectively, the model object, its `terms` component (at least for the right-hand side of the model), a `list` of levels of the factors, and the grid of predictor combinations that specify the reference grid. 

The function must obtain six things and return them in a named `list`. They are
the matrix `X` of linear functions for each point in the reference grid, the
regression coefficients `bhat`; the variance-covariance matrix `V`; a matrix
`nbasis` for non-estimable functions; a function `dffun(k,dfargs)` for computing
degrees of freedom for the linear function `sum(k*bhat)`; and a list `dfargs` of
arguments to pass to `dffun`. Optionally, the returned list may include a
`model.matrix` element (the model matrix for the data or a compact version
thereof obtained via `.cmpMM()`), which, if included, enables the `submodel`
option.

To write your own `emm_basis` function, examining some of the existing methods can help; but the best resource is the `predict` method for the object in question, looking carefully to see what it does to predict values for a new set of predictors (e.g., `newdata` in `predict.lm`). Following this advice, let's take a look at it:
```{r}
MASS:::predict.lqs
```
###### {#eblqs}
<!-- @index `emm_basis()`!for `lqs` objects@lqs -->
Based on this, here is a listing of an `emm_basis` method for `lqs` objects:
```{r}
emm_basis.lqs = function(object, trms, xlev, grid, ...) { 
    m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
    X = model.matrix(trms, m, contrasts.arg = object$contrasts) 
    bhat = coef(object) 
    Xmat = model.matrix(trms, data=object$model)                      # 5
    V = rev(object$scale)[1]^2 * solve(t(Xmat) %*% Xmat)
    nbasis = matrix(NA) 
    dfargs = list(df = nrow(Xmat) - ncol(Xmat))
    dffun = function(k, dfargs) dfargs$df
    list(X = X, bhat = bhat, nbasis = nbasis, V = V,                  #10
         dffun = dffun, dfargs = dfargs)
}
```
Before explaining it, let's verify that it works:
```{r}
emmeans(fake.lts, ~ B | A)
```
Hooray! Note the results are comparable to those we had for `fake.rlm`, albeit the standard errors are quite a bit smaller. (In fact, the SEs could be misleading; a better method for estimating covariances should probably be implemented, but that is beyond the scope of this vignette.)

[Back to Contents](#contents)

### Dissecting `emm_basis.lqs`
Let's go through the listing of this method, line-by-line:

  * Lines 2--3: Construct the linear functions, `X`. This is a pretty standard 
    two-step process: First obtain a model frame, `m`, for the grid of
    predictors, then pass it as data to `model.matrix` to create the associated
    design matrix. As promised, this code is essentially identical to what you
    find in `predict.lqs`.
  * Line 4: Obtain the coefficients, `bhat`. Most model objects have a `coef` method.
  * Lines 5--6: Obtain the covariance matrix, `V`, of `bhat`. In many models,
    this can be obtained using the object's `vcov` method. But not in this case.
    Instead, I cobbled one together using the inverse of the **X'X** matrix
    as in ordinary regression, and the variance estimate found in the last
    element of the `scale` element of the object. This probably under-estimates
    the variances and distorts the covariances, because robust estimators have
    some efficiency loss.
  * Line 7: Compute the basis for non-estimable functions. This applies only
    when there is a possibility of rank deficiency in the model. But `lqs` methods
    don't allow rank deficiencies, so it we have fitted such a model, we can be sure that
    all linear functions are estimable; we signal that by
    setting `nbasis` equal to a 1 x 1 matrix of `NA`. If rank deficiency were
    possible, the **estimability** package (which is required by **emmeans**)
    provides a `nonest.basis` function that makes this fairly painless---I would
    have coded `nbasis = estimability::nonest.basis(Xmat)`. 
    
    There some subtleties you need to know regarding estimability. Suppose the model
    is rank-deficient, so that the design matrix **X** has *p* columns but
    rank *r* < *p*. In that case, `bhat` should be of length *p* (not *r*), and there
    should be *p* - *r* elements equal to `NA`, corresponding to columns of **X**
    that were excluded from the fit. Also, `X` should have all *p* columns. In other
    words, do not alter or throw-out columns of `X` or their corresponding elements
    of `bhat`---even those with `NA` coefficients---as they are essential for
    assessing estimability. `V` should be *r* x *r*, however---the covariance
    matrix for the non-excluded predictors.

  * Lines 8--9: Obtain `dffun` and `dfargs`. This is a little awkward because it is     designed to allow support for mixed models, where approximate methods may be
    used to obtain degrees of freedom. The function `dffun` is expected to have two
    arguments: `k`, the vector of coefficients of `bhat`, and `dfargs`, a list
    containing any additional arguments. In this case (and in many other models),
    the degrees of freedom are the same regardless of `k`. We put the required
    degrees of freedom in `dfargs` and write `dffun` so that it simply returns that
    value. (Note: If asymptotic tests and CIs are desired, return `Inf`
    degrees of freedom.)
  * Line 10: Return these results in a named list.

[Back to Contents](#contents)


## Communication between methods {#communic}
<!-- @index `emm_basis()`!Communicating with `recover_data()`;
   `recover_data()!Communicating with `emm_basis()`; `misc` attribute and argument -->
If you need to pass information obtained in `recover_data()` to the `emm_basis()` method,
simply incorporate it as `attr(data, "misc")` where `data` is the dataset returned by
`recover_data()`. Subsequently, that attribute is available in `emm_grid()` by adding a
`misc` argument.


## Hook functions {#hooks}
<!-- @index Hook functions; `emm_basis()`!Hook functions 
     `estHook`; `vcovHook`; `postGridHook` -->
Most linear models supported by **emmeans** have straightforward structure: Regression coefficients, their covariance matrix, and a set of linear functions that define the reference grid. However, a few are more complex. An example is the `clm` class in the **ordinal** package, which allows a scale model in addition to the location model. When a scale model is used, the scale parameters are included in the model matrix, regression coefficients, and covariance matrix, and we can't just use the usual matrix operations to obtain estimates and standard errors. To facilitate using custom routines for these tasks, the `emm_basis.clm` function function provided in **emmeans** includes, in its `misc` part, the names (as character constants) of two "hook" functions: `misc$estHook` has the name of the function to call when computing estimates, standard errors, and degrees of freedom (for the `summary` method); and `misc$vcovHook` has the name of the function to call to obtain the covariance matrix of the grid values (used by the `vcov` method). These functions are called in lieu of the usual built-in routines for these purposes, and return the appropriately sized matrices. 

In addition, you may want to apply some form of special post-processing after the reference grid is constructed. To provide for this, give the name of your function to post-process the object in `misc$postGridHook`. Again, `clm` objects (as well as `polr` in the **MASS** package) serve as an example. They allow a `mode` specification that in two cases, calls for post-processing. The `"cum.prob"` mode uses the `regrid` function to transform the linear predictor to the cumulative-probability scale. And the `"prob"` mode performs this, as well as applying the contrasts necessary to convert the cumulative probabilities into the class probabilities.

## Re-gridded basis {#regridded}
<!-- @index `emm_basis()`!Re-gridded; `misc$regrid.flag`; `regrid.flag` -->
Sometimes your `emm_basis` method may essentially create a re-gridded basis, where
`X` and `bhat` are not actually a model matrix and regression coefficients, 
but instead, `X` is the identity, `bhat` comprises the predictions at each grid point, 
and `V` is the covariance matrix of those predictions. In those cases, we recommend also
setting `misc$regrid.flag = TRUE`. Currently, this flag is used only for checking
whether the `nuisance` argument can be used in `ref_grid()`, and it is not 
absolutely necessary because we also check to see if `X` is the identity. But
it provides a more efficient and reliable check. The code for nuisance factors relies
on the structure of model matrices where columns are associated with model terms.
So it is not possible to process nuisance factors with a re-gridded basis.


[Back to Contents](#contents)

## Exported methods from **emmeans** {#exported}
<!-- @index Extending **emmeans**!Exports useful to developers -->
For package developers' convenience, **emmeans** exports some of its S3 methods
for `recover_data` and/or `emm_basis`---use `methods("recover_data")` and
`methods("emm_basis")` to discover which ones. It may be that all you need is to
invoke one of those methods and perhaps make some small changes---especially if
your model-fitting algorithm makes heavy use of an existing model type supported
by **emmeans**. For those methods that are not exported, use `recover_data()`
and `.emm_basis()`, which run in **emmeans**'s namespace, thus providing access
to all available methods..

A few additional functions are exported because they may be useful to developers. They are as follows:

  * `emmeans::.all.vars(expr, retain)` Some users of your package may include `$` or
    `[[]]` operators in their model formulas. If you need to get the variable
    names, `base::all.vars` will probably not give you what you need. For
    example, if `form = ~ data$x + data[[5]]`, then `base::all.vars(form)`
    returns the names `"data"` and `"x"`, whereas `emmeans::.all.vars(form)`
    returns the names `"data$x"` and `"data[[5]]"`. The `retain` argument may be
    used to specify regular expressions for patterns to retain as parts of
    variable names.

  * `emmeans::.diag(x, nrow, ncol)` The base `diag` function has a booby trap whereby,
    for example, `diag(57.6)` returns a 57 x 57 identity matrix rather than
    a 1 x 1 matrix with 57.6 as its only element. But
    `emmeans::.diag(57.6)` will return the latter. The function works identically
    to `diag` except for its tail run around the identity-matrix trap.
  * `emmeans::.aovlist.dffun(k, dfargs)` This function is exported because it is needed
    for computing degrees of freedom for models fitted using `aov`, but it may be
    useful for other cases where Satterthwaite degrees-of-freedom calculations are
    needed. It requires the `dfargs` slot to contain analogous contents.
  * `emmeans::.get.offset(terms, grid)` If `terms` is a model formula containing an
    `offset` call, this is will compute that offset in the context of `grid` (a
    `data.frame`).
  * `emmeans::.my.vcov(object, ...)` In a call to `ref_grid`, `emmeans`, etc., the user may
    use `vcov.` to specify an alternative function or matrix to use as the
    covariance matrix of the fixed-effects coefficients. This function supports
    that feature. Calling `.my.vcov` in place of the `vcov` method will
    substitute the user's `vcov.` when it is specified.
  * `emmeans::.std.link.labels(fam, misc)` This is useful in `emm_basis` methods for
    generalized linear models. Call it with `fam` equal to the `family` object for
    your model, and `misc` either an existing list, or just `list()` if none. It
    returns a new `misc` list containing the link function and, in some cases,
    extra features that are used for certain types of link functions (e.g., for a
    log link, the setups for returning ratio comparisons with `type = "response"`).
  * `emmeans::.num.key(levs, key)` Returns integer indices of elements of 
    `key` in `levs` when `key` is a character vector; or just returns integer values
    if already integer. Also throws an error if levels are mismatched or indices exceed 
    legal range. This is useful in custom contrast functions (`.emmc` functions).
  * `emmeans::.get.excl(levs, exclude, include)` This is support for the `exclude` and 
    `include` arguments of contrast functions. It checks legality and returns an integer 
    vector of `exclude` indices in `levs`, given specified integer or character
    arguments `exclude` and `include`. In your `.emmc` function, 
    `exclude` should default to `integer(0)` and `include` should have no default.
  * `emmeans::.cmpMM(X, weights, assign)` creates a compact version of the
    model matrix `X` (or, preferably, its QR decomposition). 
    This is useful if we want an `emm_basis()` method to return a `model.matrix` element.
    The returned result is
    just the R portion of the QR decomposition of `diag(sqrt(weights)) %*% X`, with the 
    `assign` attribute added. If `X` is a `qr` object, we assume the weights are
    already incorporated, as is true of the `qr` slot of a `lm` object. 

[Back to Contents](#contents)


## Existing support for `rsm` objects {#rsm}
<!-- @index **rsm** package -->
As a nontrivial example of how an existing package supports **emmeans**, we show the support offered by the **rsm** package. Its `rsm` function  returns an `rsm` object which is an extension of the `lm` class. Part of that extension has to do with `coded.data` structures whereby, as is typical in response-surface analysis, models are fitted to variables that have been linearly transformed (coded) so that the scope of each predictor is
represented by plus or minus 1 on the coded scale. 

Without any extra support in **rsm**, `emmeans` will work just fine with `rsm` objects; but if the data are coded, it becomes awkward to present results in terms of the original predictors on their original, uncoded scale. The `emmeans`-related methods in **rsm** provide a `mode` argument that may be used to specify whether we want to work with coded or uncoded data. The possible values for `mode` are `"asis"` (ignore any codings, if present), `"coded"` (use the coded scale), and `"decoded"` (use the decoded scale). The first two are actually the same in that no decoding is done; but it seems clearer to provide separate options because they represent two different situations.

### The `recover_data` method {#rdrsm}
<!-- @index `recover_data()`!for `rsm` objects@rsm -->
Note that coding is a *predictor* transformation, not a response transformation (we could have that, too, as it's already supported by the **emmeans** infrastructure). So, to handle the `"decode"` mode, we will need to actually decode the predictors used to construct he reference grid. That means we need to make `recover_data` a lot fancier! Here it is:

```{r}
recover_data.rsm = function(object, data, mode = c("asis", "coded", "decoded"), ...) {
    mode = match.arg(mode)
    cod = rsm::codings(object)
    fcall = object$call
    if(is.null(data))                                                 # 5
        data = emmeans::recover_data(fcall, 
                   delete.response(terms(object)), object$na.action, 
                   weights = weights(object), ...)
    if (!is.null(cod) && (mode == "decoded")) {
        pred = cpred = attr(data, "predictors")
        trms = attr(data, "terms")                                    #10
        data = rsm::decode.data(rsm::as.coded.data(data, formulas = cod))
        for (form in cod) {
            vn = all.vars(form)
            if (!is.na(idx <- grep(vn[1], pred))) { 
                pred[idx] = vn[2]                                     #15
                cpred = setdiff(cpred, vn[1])
            }
        }
        attr(data, "predictors") = pred
        new.trms = update(trms, reformulate(c("1", cpred)))           #20
        attr(new.trms, "orig") = trms
        attr(data, "terms") = new.trms
        attr(data, "misc") = cod
    }
    data
}
```

Lines 2--7 ensure that `mode` is legal, retrieves the codings from the object, and obtain the results we would get from `recover_data` had it been an `lm` object. If `mode` is not `"decoded"`, *or* if no codings were used, that's all we need. Otherwise, we need to return the decoded data. However, it isn't quite that simple, because the model equation is still defined on the coded scale. Rather than to try to translate the model coefficients and covariance matrix to the decoded scale, we elected to remember what we will need to do later to put things back on the coded scale. In lines 9--10, we retrieve the attributes of the recovered data that provide the predictor names and `terms` object on the coded scale. In line 11, we replace the recovered data with the decoded data. 

By the way, the codings comprise a list of formulas with the coded name on the left and the original variable name on the right. It is possible that only some of the predictors are coded (for example, blocking factors will not be). In the `for` loop in lines 12--18, the coded predictor names are replaced with their decoded names. For technical reasons to be discussed later, we also remove these coded predictor names from a copy, `cpred`, of the list of all predictors in the coded model. In line 19, the `"predictors"` attribute of `data` is replaced with the modified version.

Now, there is a nasty technicality. The `ref_grid` function in **emmeans** has a few lines of code after `recover_data` is called that determine if any terms in the model convert covariates to factors or vice versa; and this code uses the model formula. That formula involves variables on the coded scale, and those variables are no longer present in the data, so an error will occur if it tries to access them. Luckily, if we simply take those terms out of the formula, it won't hurt because those coded predictors would not have been converted in that way. So in line 20, we update `trms` with a simpler model with the coded variables excluded (the intercept is explicitly included to ensure there will be a right-hand side even is `cpred` is empty). We save that as the `terms` attribute, and the original terms as a new `"orig"` attribute to be retrieved later. The `data` object, modified or not, is returned. If data have been decoded, `ref_grid` will construct its grid using decoded variables.

In line 23, we save the codings as the `"misc"` attribute, to be accessed later by `emm_basis()`.

### The `emm_basis` method {#ebrsm}
<!-- @index `emm_basis()`!for `rsm` objects@rsm -->
Now comes the `emm_basis` method that will be called after the grid is defined. 
It is listed below:
```{r}
emm_basis.rsm = function(object, trms, xlev, grid, 
                         mode = c("asis", "coded", "decoded"), misc, ...) {
    mode = match.arg(mode)
    cod = misc
    if(!is.null(cod) && mode == "decoded") {                          # 5
        grid = rsm::coded.data(grid, formulas = cod)
        trms = attr(trms, "orig")
    }
    
    m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)     #10
    X = model.matrix(trms, m, contrasts.arg = object$contrasts)
    bhat = as.numeric(object$coefficients) 
    V = emmeans::.my.vcov(object, ...)
    
    if (sum(is.na(bhat)) > 0)                                         #15
        nbasis = estimability::nonest.basis(object$qr)
    else
        nbasis = estimability::all.estble
    dfargs = list(df = object$df.residual)
    dffun = function(k, dfargs) dfargs$df                             #20

    list(X = X, bhat = bhat, nbasis = nbasis, V = V, 
         dffun = dffun, dfargs = dfargs, misc = list())
}
```

This is much simpler. The coding formulas are obtained from `misc` (line 4) so that we don't have to re-obtain them from the object. All we have to do is determine if decoding was done (line 5); and, if so, convert the grid back to the coded scale (line 6) and recover the original `terms` attribute (line 7). The rest is borrowed directly from the `emm_basis.lm` method in **emmeans**. Note that line 13 uses one of the exported functions we described in the preceding section. Lines 15--18 use functions from the **estimability** package to handle the possibility that the model is rank-deficient.


### A demonstration {#demo}
Here's a demonstration of this **rsm** support. The standard example for `rsm` fits a second-order model `CR.rs2` to a dataset organized in two blocks and with two coded predictors.
```{r results = "hide", warning = FALSE, message = FALSE}
library("rsm")
example("rsm")   ### (output is not shown) ###
```

First, let's look at some results on the coded scale---which are the same as for an ordinary `lm` object.
```{r}
emmeans(CR.rs2, ~ x1 * x2, mode = "coded", 
        at = list(x1 = c(-1, 0, 1), x2 = c(-2, 2)))
```
Now, the coded variables `x1` and `x2` are derived from these coding formulas for predictors `Time` and `Temp`:
```{r}
codings(CR.rs1)
```
Thus, for example, a coded value of `x1 = 1` corresponds to a time of 85 + 1 x 5 = 90. Here are some results working with decoded predictors. Note that the `at` list must now be given in terms of `Time` and `Temp`:
```{r}
emmeans(CR.rs2, ~ Time * Temp, mode = "decoded", 
        at = list(Time = c(80, 85, 90), Temp = c(165, 185)))
```
Since the supplied settings are the same on the decoded scale as were used on the coded scale, the EMMs are identical to those in the previous output.


## Dispatching and restrictions {#dispatch}
<!-- @index Extending **emmeans**!Restrictions; 
    `recover_data()`!Dispatching; `emm_basis()`!Dispatching -->
The **emmeans** package has internal support for a number of model classes.
When `recover_data()` and `emm_basis()` are dispatched, a search is made
for external methods for a given class; and if found, those methods are
used instead of the internal ones. However, certain restrictions apply
when you aim to override an existing internal method:

  1. The class name being extended must appear in the first or second
     position in the results of `class(object)`. That is, you may have a
     base class for which you provide `recover_data()` and `emm_basis()` 
     methods, and those will also work for *direct* descendants thereof;
     but any class in third place or later in the inheritance is ignored.
  2. Certain classes vital to the correct operation of the package, e.g.,
     `"lm"`, `"glm"`, etc., may not be overridden.
     
If there are no existing internal methods for the class(es) you provide
methods for, there are no restrictions on them.


## Exporting and registering your methods {#exporting}
<!-- @index **emmeans** package!Exporting extensions to 
     Registering `recover_data` and `emm_basis` methods -->
To make the methods available to users of your package, the methods must be exported.
R and CRAN are evolving in a way that having S3 methods in the registry is
increasingly important; so it is a good idea to provide for that.
The problem is not all of your package users will have **emmeans** installed.

Thus, registering the methods must be done conditionally. We provide a
courtesy function `.emm_register()` to make this simple. 
Suppose that your package offers two model classes `foo` and `bar`, and it
includes the corresponding functions `recover_data.foo`, `recover_data.bar`,
`emm_basis.foo`, and `emm_basis.bar`.
Then to register these methods, add or modify the
`.onLoad` function in your package (traditionally saved in the source file `zzz.R`):
```r
.onLoad <- function(libname, pkgname) {
    if (requireNamespace("emmeans", quietly = TRUE))
        emmeans::.emm_register(c("foo", "bar"), pkgname)
}

```
You should also add `emmeans (>= 1.4)` and `estimability` (which is required by **emmeans**)
to the `Suggests` field of your `DESCRIPTION` file.


[Back to Contents](#contents)


## Conclusions {#concl}
It is relatively simple to write appropriate methods that work with **emmeans** for model objects it does not support. I hope this vignette is helpful for understanding how. Furthermore, if you are the developer of a package that fits linear models, I encourage you to include `recover_data` and `emm_basis` methods for those classes of objects, so that users have access to **emmeans** support.

[Back to Contents](#contents)


[Index of all vignette topics](vignette-topics.html)

---
title: "Transformations and link functions in emmeans"
author: "emmeans package, Version `r packageVersion('emmeans')`"
output: emmeans::.emm_vignette
vignette: >
  %\VignetteIndexEntry{Transformations and link functions}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
```{r, echo = FALSE, results = "hide", message = FALSE}
require("emmeans")
knitr::opts_chunk$set(fig.width = 4.5, class.output = "ro")
```

<!-- @index Vignettes!Transformations and link functions -->
## Contents {#contents}
This vignette covers the intricacies of transformations and link functions in **emmeans**.

  1. [Overview](#overview)
  2. [Re-gridding](#regrid)
  3. [Link functions](#links)
  3. [Graphing transformations and links](#trangraph)
  4. [Both a response transformation and a link](#tranlink)
  5. [Special transformations](#special)
  6. [Specifying a transformation after the fact](#after)
  6. [Auto-detected transformations](#auto)
  6. [Standardized response](#stdize)
  7. [Faking a log transformation](#logs)
     a. [Faking other transformations](#faking)
     b. [Alternative scale](#altscale)
  8. [Bias adjustment](#bias-adj)
  
[Index of all vignette topics](vignette-topics.html) 

## Overview {#overview}
<!-- @index Transformations!Overview; Examples!`pigs` -->
Consider an example with the `pigs` dataset that is used in many of 
these vignettes; but this time, we use a `log` transformation of the response:
```{r}
pigs.lm <- lm(log(conc) ~ source + factor(percent), data = pigs)
```
This model has two factors, `source` and `percent` (coerced to a factor), as 
predictors; and log-transformed `conc` as the response. Here we obtain the EMMs
for `source`, examine its structure, and finally produce a summary, including a
test against a null value of log(35):
```{r}
emm.src <- emmeans(pigs.lm, "source")
str(emm.src)
```
```{r}
summary(emm.src, infer = TRUE, null = log(35))
```
Now suppose that we want the EMMs expressed on the same scale as `conc`. This 
can be done by adding `type = "response"` to the `summary()` call: 
```{r}
summary(emm.src, infer = TRUE, null = log(35), type = "response")
```

Note: Looking ahead, this output is compared later in this vignette with a 
[bias-adjusted version](#pigs-biasadj).

### Timing is everything {#timing}
<!-- @index Transformations!Timing is everything -->
Dealing with transformations in **emmeans** is somewhat complex, due to the 
large number of possibilities. But the key is understanding what happens, when.
These results come from a sequence of steps. Here is what happens (and doesn't
happen) at each step:

  1. The reference grid is constructed for the `log(conc)` model. The
     fact that a log transformation is used is recorded, but nothing else
     is done with that information.
  2. The predictions on the reference grid are averaged
     over the four `percent` levels, for each `source`, to obtain the EMMs
     for `source` -- *still* on the `log(conc)` scale.
  3. The standard errors and confidence intervals for these EMMs are
     computed -- *still* on the `log(conc)` scale.
  4. Only now do we do back-transformation...
     a. The EMMs are back-transformed to the `conc` scale.
     b. The endpoints of the confidence intervals are back-transformed.
     c. The *t* tests and *P* values are left as-is.
     d. The standard errors are converted to the `conc` scale using the
        delta method. These SEs were *not* used in constructing the 
        tests and confidence intervals.

### The model is our best guide
This choice of timing is based on the idea that *the model is right*. In
particular, the fact that the response is transformed suggests that the
transformed scale is the best scale to be working with. In addition, the model
specifies that the effects of `source` and `percent` are *linear* on the
transformed scale; inasmuch as marginal averaging to obtain EMMs is a linear
operation, that averaging is best done on the transformed scale. For those two
good reasons, back-transforming to the response scale is delayed until the very
end by default.

[Back to Contents](#contents)

## Re-gridding {#regrid}
<!-- @index `regrid()`; Transformations!Re-gridding; `emmeans()`!With transformations -->

As well-advised as it is, some users may not want the default timing of things.
The tool for changing when back-transformation is performed is the `regrid()`
function -- which, with default settings of its arguments, back-transforms an
`emmGrid` object and adjusts everything in it appropriately. For example:
```{r}
str(regrid(emm.src))

summary(regrid(emm.src), infer = TRUE, null = 35)
```
Notice that the structure no longer includes the transformation. That's because
it is no longer relevant; the reference grid is on the `conc` scale, and how we got there is now forgotten. Compare this `summary()` result with the preceding one, and note the following:

  * It no longer has annotations concerning transformations.
  * The estimates and SEs are identical.
  * The confidence intervals, *t* ratios, and *P* values are *not*
    identical. This is because, this time, the SEs shown in the table
    are the ones actually used to construct the tests and intervals.

Understood, right? But think carefully about how these EMMs were obtained. 
They are back-transformed from `emm.src`, in which *the marginal averaging
was done on the log scale*. If we want to back-transform *before* doing the
averaging, we need to call `regrid()` after the reference grid is constructed but
before the averaging takes place:
```{r}
pigs.rg <- ref_grid(pigs.lm)
remm.src <- emmeans(regrid(pigs.rg), "source")
summary(remm.src, infer = TRUE, null = 35)
```
These results all differ from either of the previous two summaries -- again,
because the averaging is done on the `conc` scale rather than the `log(conc)` 
scale.

###### {#regrid2}
<!-- @index `regrid()`!`regrid` vs. `type`; `ref_grid()`!`regrid` argument -->
Note: For those who want to routinely back-transform before averaging,
the `regrid` argument in `ref_grid()` simplifies this. The first two
steps above could have been done more easily as follows:
```{r eval = FALSE}
remm.src <- emmeans(pigs.lm, "source", regrid = "response")
```
But don't get `regrid` and `type` confused. The `regrid` argument is 
passed to `regrid()` (as the `transform` argument) after the reference grid is constructed, whereas the `type`
argument is simply remembered and used by `summary()`. So a similar-looking
call:
```{r eval = FALSE}
emmeans(pigs.lm, "source", type = "response")
```
will compute the results we have seen for `emm.src` -- back-transformed 
*after* averaging on the log scale.

Remember again: When it comes to transformations, timing is everything.

### Implied regridding with certain modes {#regrid3}
<!-- @index Transformations|Special modes; 
             `mode` argument!implied re-gridding; 
             Transformations|Implied re-gridding;
             Re-gridding!Implied  -->
Some model classes provide special argument(s) (typically `mode`) that may cause
transformations or links to be handled early. For example, cumulative link
models for ordinal data allow for a `"prob"` mode that produces estimates of
probabilities for each ordinal level. The reference grid comprises estimates on
a probability scale, and whatever link was used (say, probit) has already been
accounted for, so is not "remembered" for possible later back-transformation. In
that sense, when we use `mode = "prob"`, it is sort of like an implied call to
`regrid()` that takes place *at the time the reference grid is constructed*,
preempting any timing choices you might otherwise have made about handling the
transformation. If there are one or more factors that are averaged over in
estimating marginal means will be averages of the probabilities in the reference
grid; so they will be different than what you would have obtained by keeping
things on the link scale and then computing the probabilities after averaging on
the link scale.

Similar things happen with certain options with multinomial models, 
zero-inflated, or hurdle models. Those special modes are a great convenience
for getting estimates on a scale that is desired, but they also force
you to obtain marginal means of measurements already on that scale.


[Back to Contents](#contents)


## Link functions {#links}
<!-- @index Link functions; Examples!`neuralgia`; Examples!Logistic regression -->
Exactly the same ideas we have presented for response transformations apply to
generalized linear models having non-identity link functions. As far as
**emmeans** is concerned, there is no difference at all.

To illustrate, consider the `neuralgia` dataset provided in the package. These 
data come from an experiment reported in a SAS technical report where different
treatments for neuralgia are compared. The patient's sex is an additional
factor, and their age is a covariate. The response is `Pain`, a binary variable
on whether or not the patient reports neuralgia pain after treatment.
The model suggested in the SAS report is equivalent to the following. We use
it to obtain estimated probabilities of experiencing pain:
```{r}
neuralgia.glm <- glm(Pain ~ Treatment * Sex + Age, family = binomial(), data = neuralgia)
neuralgia.emm <- emmeans(neuralgia.glm, "Treatment", type = "response")
neuralgia.emm
```

###### {#oddsrats}
<!-- @index Odds ratios; Logistic regression!Odds ratios -->
(The note about the interaction is discussed shortly.) Note that the averaging 
over `Sex` is done on the logit scale, *before* the results are back-transformed
for the summary. We may use `pairs()` to compare these estimates; note that 
logits are logs of odds; so this is another instance where log-differences are
back-transformed -- in this case to odds ratios:
```{r}
pairs(neuralgia.emm, reverse = TRUE)
```
So there is evidence of considerably more pain being reported with placebo 
(treatment `P`) than with either of the other two treatments. The estimated odds
of pain with `B` are about half that for `A`, but this finding is not 
statistically significant. (The odds that this is a made-up dataset seem quite
high, but that finding is strictly this author's impression.)

Observe that there is a note in the output for `neuralgia.emm` that the results
may be misleading. It is important to take it seriously, because if two factors
interact, it may be the case that marginal averages of predictions don't reflect
what is happening at any level of the factors being averaged over. To find out,
look at an interaction plot of the fitted model:
```{r fig.alt = "Interaction-plot display of the results of emmeans(neuralgia.glm, ~ Treatment | Sex)"}
emmip(neuralgia.glm, Sex ~ Treatment)
```
There is no practical difference between females and males in the patterns of
response to `Treatment`; so I think most people would be quite comfortable with
the marginal results that are reported here.

Some users prefer risk ratios (ratios of probabilities) rather than odds 
ratios. We will revisit this example [below](#riskrats) after we have discussed 
some more tools.

[Back to Contents](#contents)


## Graphing transformations and links {#trangraph}
<!-- @index Transformations!Graphical display; Examples!`neuralgia`;
    `type = "scale"`; `"scale"` type@scale; -->
There are a few options for displaying transformed results graphically. First, the `type` 
argument works just as it does in displaying a tabular summary. Following through with
the `neuralgia` example, let us display the marginal `Treatment` EMMs on both the
link scale and the response scale (we are opting to do the averaging on the link scale):
```{r, fig.height = 1.5, fig.alt = c("Plot A: Display of the results of confint(neur.Trt.emm)", 'Plot B: Display of the results of confint(neur.Trt.emm, type = "response"). These intervals are markedly skewed right|left for low|high estimated probabilities')}
neur.Trt.emm <- suppressMessages(emmeans(neuralgia.glm, "Treatment"))
plot(neur.Trt.emm)   # Link scale by default
plot(neur.Trt.emm, type = "response")
```

Besides whether or not we see response values, there is a dramatic difference
in the symmetry of the intervals.

For `emmip()` and `plot()` *only* (and currently only with the "ggplot" engine), 
there is also the option of specifying `type = "scale"`,
which causes the response values to be calculated but plotted on a nonlinear scale
corresponding to the transformation or link:
```{r, fig.height = 1.5, fig.alt = "Plot C: On the inside, this plot looks exactly like Plot A above, but the scale is transformed to show the values in Plot B. However, there are not enough tick marks."}
plot(neur.Trt.emm, type = "scale")
```

Notice that the interior part of this plot is identical to the plot on the link scale.
Only the horizontal axis is different. That is because the response values are transformed using the link function to determine the plotting positions of the graphical elements -- putting them back where they started.

As is the case here, nonlinear scales can be confusing to read, and it is very often
true that you will want to display more scale divisions, and even add minor ones.
This is done via adding arguments for the function `ggplot2::scale_x_continuous()`
(see its documentation):
```{r, fig.height = 1.5, fig.alt = "Plot D: Same as Plot C except there are more tick marks so we can discern the values better"}
plot(neur.Trt.emm, type = "scale", breaks = seq(0.10, 0.90, by = 0.10),
     minor_breaks = seq(0.05, 0.95, by = 0.05))
```

When using the `"ggplot"` engine, you always have the option of using **ggplot2** to incorporate a transformed scale -- and it doesn't even have to be the same as the transformation used in the model. For example, here we display the same results on an arcsin-square-root scale.
```{r, fig.height = 1.5, fig.alt = "Plot E: An alternative to Plot D using an arcsin scaling. This scale is less nonlinear than in plot D so the intervals are somewhat skewed, but less so than plot B"}
plot(neur.Trt.emm, type = "response") +
  ggplot2::scale_x_continuous(trans = scales::asn_trans(),
                              breaks = seq(0.10, 0.90, by = 0.10))
```

This comes across as a compromise: not as severe as the logit scaling, and not as distorted as the linear scaling of response values.

Again, the same techniques can be used with `emmip()`, except it is the vertical scale that is affected.

[Back to Contents](#contents)


## Models having both a response transformation and a link function {#tranlink}
<!-- @index Transformations!with link function@link; Examples!`warpbreaks`
            Examples!Gamma regression; `summary()`!`type = "unlink"` -->
It is possible to have a generalized linear model with a non-identity link *and* a response transformation. Here is an example, with the built-in `wapbreaks` dataset:
```{r}
warp.glm <- glm(sqrt(breaks) ~ wool*tension, family = Gamma, data = warpbreaks)
ref_grid(warp.glm)
```
The canonical link for a gamma model is the reciprocal (or inverse); and there is the square-root response transformation besides. If we choose `type = "response"`
in summarizing, we undo *both* transformations:
```{r}
emmeans(warp.glm, ~ tension | wool, type = "response")
```
What happened here is first the linear predictor was back-transformed from the link scale (inverse); then the squares were obtained to back-transform the rest of the way. It is possible to undo the link, and not the response transformation:
```{r}
emmeans(warp.glm, ~ tension | wool, type = "unlink")
```
It is *not* possible to undo the response transformation and leave the link 
in place, because the response was transform first, then the link model was applied;
we have to undo those in reverse order to make sense.

One may also use `"unlink"` as a `transform` argument in `regrid()` or through
`ref_grid()`.

[Back to Contents](#contents)



## Special transformations {#special}
<!-- @index Transformations!Custom; `make.tran()` -->
The `make.tran()` function provides several special transformations and sets
things up so they can be handled in **emmeans** with relative ease. 
(See `help("make.tran", "emmeans")` for descriptions
of what is available.) `make.tran()` works much like `stats::make.link()` in
that it returns a list of functions `linkfun()`, `linkinv()`, etc. that serve
in managing results on a transformed scale. The difference is that most 
transformations with `make.tran()` require additional arguments.

To use this capability in `emmeans()`, it is fortuitous to first obtain the 
`make.tran()` result, and then to use it as the enclosing environment
for fitting the model, with `linkfun` as the transformation. 
For example, suppose the response variable is a percentage and we want to use the response
transformation $\sin^{-1}\sqrt{y/100}$. Then proceed like this:
```{r eval = FALSE}
tran <- make.tran("asin.sqrt", 100)
my.model <- with(tran, 
    lmer(linkfun(percent) ~ treatment + (1|Block), data = mydata))
```
Subsequent calls to `ref_grid()`, `emmeans()`, `regrid()`, etc. will then 
be able to access the transformation information correctly.

The help page for `make.tran()` has an example like this
using a Box-Cox transformation.

[Back to Contents](#contents)

## Specifying a transformation after the fact {#after}
<!-- @index Transformations!Adding after the fact; `update()`!`tran` -->
It is not at all uncommon to fit a model using statements like the following:
```{r eval = FALSE}
mydata <- transform(mydata, logy.5 = log(yield + 0.5))
my.model <- lmer(logy.5 ~ treatment + (1|Block), data = mydata)
```
In this case, there is no way for `ref_grid()` to figure out that a response
transformation was used. What can be done is to update the reference grid
with the required information:
```{r eval = FALSE}
my.rg <- update(ref_grid(my.model), tran = make.tran("genlog", .5))
```
Subsequently, use `my.rg` in place of `my.model` in any `emmeans()` analyses,
and the transformation information will be there.

For standard transformations (those in `stats::make.link()`), just give the name
of the transformation; e.g.,
```{r eval = FALSE}
model.rg <- update(ref_grid(model), tran = "sqrt")
```

## Auto-detected response transformations {#auto}
<!-- @index Transformations!Auto-detected -->
As can be seen in the initial `pigs.lm` example in this vignette,
certain straightforward response transformations such as `log`, `sqrt`, etc. are
automatically detected when `emmeans()` (really, `ref_grid()`) is called on the model
object. In fact, scaling and shifting is supported too; so the preceding example with
`my.model` could have been done more easily by specifying the transformation
directly in the model formula:
```r
my.better.model <- lmer(log(yield + 0.5) ~ treatment + (1|Block), data = mydata)
```
The transformation would be auto-detected, saving you the trouble of adding it later.
Similarly, a response transformation of `2 * sqrt(y + 1)` would be correctly 
auto-detected. A model with a linearly transformed response, e.g. `4*(y - 1)`, 
would *not* be auto-detected, but `4*I(y + -1)` would be interpreted as `4*identity(y + -1)`.
Parsing is such that the response expression must be of the form `mult * fcn(resp + const)`;
operators of `-` and `/` are not recognized.



[Back to Contents](#contents)

## Faking a log transformation {#logs}
<!-- @index Transformations!Faking a log transformation; `regrid()`!`transform = "log"` -->
The `regrid()` function makes it possible to fake a log transformation of the response. Why would you want to do this? So that you can make comparisons using
ratios instead of differences.

Consider the `pigs` example once again, but suppose we had fitted a model with a square-root transformation instead of a log:
```{r}
pigroot.lm <- lm(sqrt(conc) ~ source + factor(percent), data = pigs)
logemm.src <- regrid(emmeans(pigroot.lm, "source"), transform = "log")
confint(logemm.src, type = "response")
pairs(logemm.src, type = "response")
```
These results are not identical, but very similar to the back-transformed
confidence intervals [above](#timing) for the EMMs and the [pairwise ratios in
the "comparisons" vignette](comparisons.html#logs), where the fitted model
actually used a log response.

### Faking other transformations {#faking}
<!-- @index Transformations!faking -->
It is possible to fake transformations other than the log. Just use the same method,
e.g. (results not displayed)
```{r, eval = FALSE}
regrid(emm, transform = "probit")
```
would re-grid the existing `emm` to the probit scale. Note that any estimates in `emm`
outside of the interval $(0,1)$ will be flagged as non-estimable.

The [section on standardized responses](#stdize) gives an example of reverse-engineering 
a standardized response transformation in this way.


### Revisiting the neuralgia example {#riskrats}
<!-- @index Risk ratios; Logistic regression!Risk ratios;
            `regrid()`!to obtain risk ratios;  -->
As was mentioned before in the [`neuralgia` example](#links), some users prefer ratios of probabilities (risk ratios) rather than odds ratios.
The additional machinery of `regrid()` makes this possible. First, do
```{r}
log.emm <- regrid(neuralgia.emm, "log")
```
While each node of `neuralgia.emm` is an estimate of the logit of a probability, the corresponding 
nodes of `log.emm` are estimates of the log of the same probability. Thus, pairwise comparisons
are differences of logs, which are logs of ratios. The risk ratios are thus
obtainable by
```{r}
pairs(log.emm, reverse = TRUE, type = "response")
```
The test statistics and P values differ somewhat from those for the odds ratios because they are computed on the log scale rather than the original logit scale.



###### {#not-logit}
<!-- @index Logistic-like regression!Non-logit links; 
   `regrid()`!probit or other non-logit models;
  Probit regression!Odds or risk ratios -->
We were able to obtain both odds ratios and risk ratios for `neuralgia.glm`.
But what if we had not used the logit link? Then the odds ratios would not
just fall out naturally. However, we can `regrid()` to the
`"logit"` scale if we want odds
ratios, or to `"log"` scale if we want risk ratios. For example,
```{r}
neuralgia.prb <- glm(Pain ~ Treatment * Sex + Age, family = binomial(link = "probit"), 
                     data = neuralgia)
prb.emm <- suppressMessages(emmeans(neuralgia.prb, "Treatment"))
pairs(regrid(prb.emm, "logit"), type = "response", reverse = TRUE)
```
These are vaguely comparable to the odds ratios we obtained with `neuralgia.glm`,
Similar re-gridding for the log will give us risk ratios.


### Alternative scale {#altscale}
<!-- @index Transformations!Percent difference; Percentage differences; Examples!`pigs` -->
It is possible to create a report on an alternative scale by updating the `tran` component.
For example, suppose we want percent differences instead of ratios in the preceding example
with the `pigs` dataset. This is possible by modifying the reverse transformation:
since the usual reverse transformation is a ratio of the form $r = a/b$, we have that
the percentage difference between $a$ and $b$ is $100(a-b)/b = 100(r-1)$. Thus,
```{r}
pct.diff.tran <- list(
    linkfun = function(mu) log(mu/100 + 1),
    linkinv = function(eta) 100 * (exp(eta) - 1),
    mu.eta = function(eta) 100 * exp(eta),
    name = "log(pct.diff)"
)

update(pairs(logemm.src, type = "response"), 
       tran = pct.diff.tran, inv.lbl = "pct.diff", adjust = "none",
       infer = c(TRUE, TRUE))
```
Another way to obtain the same estimates is to directly transform the estimated ratios to
$100r - 100$:
```{r}
contrast(regrid(pairs(logemm.src)), "identity", scale = 100, offset = -100,
         infer = c(TRUE, TRUE))
```
While the estimates are the same, the tests and confidence intervals are different
because they are computed on the re-gridded scale using the standard errors shown,
rather than on the link scale as in the first results.

## Standardized response {#stdize}
<!-- @index Transformations!Standardizing; Transformations!`scale()`;
    `scale()`; Standardized response; Examples!`fiber`; `regrid` argument -->
In some disciplines, it is common to fit a model to a standardized response variable.
R's base function `scale()` makes this easy to do; but it is important to notice that
`scale(y)` is more complicated than, say, `sqrt(y)`, because `scale(y)` requires 
all the values of `y` in order to determine the centering and scaling parameters.
The `ref_grid()` function (called by `emmeans() and others) tries to detect
the scaling parameters. To illustrate:
```{r, message = FALSE}
fiber.lm <- lm(scale(strength) ~ machine * scale(diameter), data = fiber)
emmeans(fiber.lm, "machine")   # on the standardized scale
emmeans(fiber.lm, "machine", type = "response")   # strength scale
```
More interesting (and complex) is what happens with `emtrends()`.
Without anything fancy added, we have
```{r}
emtrends(fiber.lm, "machine", var = "diameter")
```
These slopes are (change in `scale(strength)`) / (change in `diameter`); that is,
we didn't do anything to undo the response transformation, but the trend is
based on exactly the variable specified, `diameter`. To get (change in `strength`) / (change in `diameter`), we need to undo the response transformation, and that is done via `regrid` (which invokes `regrid()` after the reference grid is constructed):
```{r}
emtrends(fiber.lm, "machine", var = "diameter", regrid = "response")
```
What if we want slopes for (change in `scale(strength)`) / (change in `scale(diameter)`)?
This can be done, but it is necessary to manually specify the scaling parameters for `diameter`.
```{r}
with(fiber, c(mean = mean(diameter), sd = sd(diameter)))
emtrends(fiber.lm, "machine", var = "scale(diameter, 24.133, 4.324)")
```
This result is the one most directly related to the regression coefficients:
```{r}
coef(fiber.lm)[4:6]
```
There is a fourth possibility, (change in `strength`) / (change in `scale(diameter)`),
that I leave to the reader.

### What to do if auto-detection fails
Auto-detection of standardized responses is a bit tricky, and doesn't always succeed. 
If it fails, 
a message is displayed and the transformation is ignored. In cases where it doesn't work,
we need to explicitly specify the transformation using `make.tran()`. The methods are exactly 
as shown earlier in this vignette, so we show the code but not the results for 
a hypothetical example.

One method is to fit the model and then add the transformation information later.
In this example, `some.fcn` is a model-fitting function which for some reason doesn't
allow the scaling information to be detected.
```{r, eval = FALSE}
mod <- some.fcn(scale(RT) ~ group + (1|subject), data = mydata)
emmeans(mod, "group", type = "response",
        tran = make.tran("scale", y = mydata$RT))
```
The other, equivalent, method is to create the transformation object first 
and use it in fitting the model:
```{r, eval = FALSE}
mod <- with(make.tran("scale", y = mydata$RT),
            some.fcn(linkfun(RT) ~ group + (1|subject), data = mydata))
emmeans(mod, "group", type = "response")
```

### Reverse-engineering a standardized response
An interesting twist on all this is the reverse situation: Suppose we fitted the model 
*without* the standardized response, but we want to know what the results would be if
we had standardized. Here we reverse-engineer the `fiber.lm` example above:
```{r, message = FALSE}
fib.lm <- lm(strength ~ machine * diameter, data = fiber)

# On raw scale:
emmeans(fib.lm, "machine")

# On standardized scale:
tran <- make.tran("scale", y = fiber$strength)
emmeans(fib.lm, "machine", regrid = tran)
```
In the latter call, the `regrid` argument causes `regrid()` to
be called after the reference grid is constructed.




[Back to Contents](#contents)


## Bias adjustment {#bias-adj}
<!-- @index Transformations!Bias adjustment; Bias adjustment!When back-transforming;
    Means!Generalized; Geometric means; -->
So far, we have discussed ideas related to back-transforming results as a simple way
of expressing results on the same scale as the response. In particular, means
obtained in this way are known as *generalized means*; for example, a log
transformation of the response is associated with geometric means. When the goal
is simply to make inferences about which means are less than which other means,
and a response transformation is used, it is often acceptable to present
estimates and comparisons of these generalized means.
However, sometimes it is important to report results that actually do reflect
expected values of the untransformed response. An example is a financial study,
where the response is in some monetary unit. It may be convenient to use a
response transformation for modeling purposes, but ultimately we may want to
make financial projections in those same units. 

In such settings, we need to
make a bias adjustment when we back-transform, because any nonlinear
transformation biases the expected values of statistical quantities. More
specifically, suppose that we have a response $Y$ and the transformed
response is $U$. To back-transform, we use $Y = h(U)$; and using a Taylor
approximation, $Y \approx h(\eta) + h'(\eta)(U-\eta) +
\frac12h''(\eta)(U-\eta)^2$, so that 
$E(Y) \approx h(\eta) + \frac12h''(\eta)Var(U)$. 
This shows that the amount of needed bias adjustment is approximately
$\frac12h''(\eta)\sigma^2$ where $\sigma$ is the error SD in the model for $U$.
It depends on $\sigma$, and the larger this is, the greater the bias adjustment
is needed. This second-order bias adjustment is what is currently used
in the **emmeans** package when bias-adjustment is requested. There are better or exact adjustments for certain cases, and future updates may incorporate some of those.

### Pigs example revisited {#pigs-biasadj}
<!-- @index Examples!`pigs` -->
Let us compare the estimates in [the overview](#overview) after we apply a bias
adjustment. First, note that an estimate of the residual SD is available via
the `sigma()` function:
```{r}
sigma(pigs.lm)
```
This estimate is used by default. The bias-adjusted EMMs for the sources are:
```{r}
summary(emm.src, type = "response", bias.adj = TRUE)
```
These estimates (and also their SEs) are slightly larger than we had without
bias adjustment. They are estimates of the *arithmetic* mean responses, rather
than the *geometric* means shown in the overview. Had the value of `sigma` been
larger, the adjustment would have been greater. You can experiment with this by
adding a `sigma =` argument to the above call.

### Response transformations vs. link functions {#link-bias}
<!-- @index Transformations!Response versus link functions;
    Bias adjustment!For link functions vs. response transformations; -->
At this point, it is important to point out that the above discussion focuses on
*response* transformations, as opposed to link functions used in generalized linear
models (GLMs). In an ordinary GLM, no bias adjustment is needed, nor is it
appropriate, 
because the link function is just used to define a nonlinear relationship
between the actual response mean $\eta$ and the linear predictor. That is, the 
back-transformed parameter is already the mean. 


#### InsectSprays example {#insects}
<!-- @index Examples!`InsectSprays`;  Bias adjustment!When *not* to use; -->
To illustrate this, consider
the `InsectSprays` data in the **datasets** package. The response variable
is a count, and there is one treatment, the spray that is used. Let us
model the count as a Poisson variable with (by default) a log link; and
obtain the EMMs, with and without a bias adjustment
```{r}
ismod <- glm(count ~ spray, data = InsectSprays, family = poisson())
emmeans(ismod, "spray", type = "response")
```
The above results were computed with no bias adjustment, the default. If you try
```{r eval = FALSE}
emmeans(ismod, "spray", type = "response", bias.adj = TRUE)
```
you will get exactly the same results, plus a warning message that says bias adjustment was disabled.
Why? Because in an ordinary GLM like this, we are *already* modeling the mean counts,
and the link function is not a response transformation as such, just a part of the relationship
we are specifying between the linear predictor and the mean. Given the simple structure of this dataset, we can verify this by noting that the estimates we have correspond exactly to the simple observed mean counts:
```{r}
with(InsectSprays, tapply(count, spray, mean))
```
The point here is that a GLM does not have an additive error
term, that the model is already formulated in terms of the mean, not some
generalized mean. (You can enable the bias-adjustment computations by specifying
a valid `sigma` value; but you should not do so.)

Note that, in a generalized linear *mixed* model, including generalized estimating
equations and such, there *are* additive random components involved, and then bias
adjustment becomes appropriate.

#### CBPP example {#cbpp}
<!-- @index Examples!`cbpp`; Bias adjustment!In GLMMs and GEE models; -->
Consider an example adapted from the help page for `lme4::cbpp`. 
Contagious bovine pleuropneumonia (CBPP) is a disease in African cattle,
and the dataset contains data on incidence of CBPP in several herds of cattle
over four time periods. We will fit a mixed model that accounts for herd variations
as well as overdispersion (variations larger than expected with a simple binomial model): 

```{r, message = FALSE}
require(lme4)
cbpp <- transform(cbpp, unit = 1:nrow(cbpp))
cbpp.glmer <- glmer(cbind(incidence, size - incidence) ~ period + 
                          (1 | herd) +  (1 | unit),
                    family = binomial, data = cbpp)

emm <- emmeans(cbpp.glmer, "period")
summary(emm, type = "response")
```
The above summary reflects the back-transformed estimates, with no bias adjustment. However, the model estimates two independent sources of random variation that probably should be taken into account:
```{r}
lme4::VarCorr(cbpp.glmer)
```
Notably, the over-dispersion SD is considerably greater than the herd SD.
Suppose we want to estimate the marginal probabilities of CBPP incidence,
averaged over herds and over-dispersion variations. For this purpose, we need
the combined effect of these variations; so we compute the overall SD
via the Pythagorean theorem:
```{r}
total.SD = sqrt(0.89107^2 + 0.18396^2)
```
Accordingly, here are the bias-adjusted estimates of the marginal probabilities:
```{r}
summary(emm, type = "response", bias.adjust = TRUE, sigma = total.SD)
```
These estimates are somewhat larger than the unadjusted estimates (actually, any estimates greater than 0.5 would have been adjusted downward). These adjusted estimates are more appropriate for describing the marginal incidence of CBPP for all herds. In fact, these estimates are fairly close to those obtained directly from the incidences in the data:
```{r}
cases <- with(cbpp, tapply(incidence, period, sum))
trials <- with(cbpp, tapply(size, period, sum))
cases / trials
```

Left as an exercise: Revisit the `InsectSprays` example, but (using similar
methods to the above) create a `unit` variable and fit an over-dispersion model.
Compare the results with and without bias adjustment, and evaluate these results
against the earlier results. This is simpler than the CBPP example because there
is only one random effect.

[Back to Contents](#contents)

[Index of all vignette topics](vignette-topics.html) 

---
title: "Re-engineering CLDs"
author: "emmeans package, Version `r packageVersion('emmeans')`"
output: emmeans::.emm_vignette
vignette: >
  %\VignetteIndexEntry{Re-engineering CLDs}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, echo = FALSE, results = "hide", message = FALSE}
knitr::opts_chunk$set(echo = TRUE)
require(emmeans)
require(multcomp, quietly = TRUE)
```

<!-- @index Vignettes!Re-engineering CLDs; Compact letter displays (CLDs) -->

## Contents {#contents}
  1. [Introduction](#intro)
  2. [Grouping by underlining](#underlining)
  3. [Grouping using letters or symbols](#CLDs)
  4. [Simulated example](#CLD-example)
  5. [Alternative CLDs](#alt-CLDs)
     a. [Equivalence sets](#equiv-CLDs)
     b. [Significance sets](#signif-CLDs)
  6. [Conclusions](#concl)
  7. [References](#refs)

[Index of all vignette topics](vignette-topics.html)


### Introduction {#intro}
Compact letter displays (CLDs) are a popular way to display multiple comparisons,
especially when there are more than a few means to compare.
They are problematic, however, because they are prone to misinterpretation (more
details later). Here we present some background on CLDs, and show some adaptations
and alternatives that may be less prone to misinterpretation.

### Grouping by underlining {#underlining}
<!-- @index CLDs!Relation to underlining; Underlining, grouping by -->
CLDs generalize an "underlining" technique shown in some old experimental 
design and analysis textbooks, where results may be displayed something like this:
```
    trt1  ctrl   trt3   trt2   trt4
    ----------
          ------------------
```
The observed means are sorted in increasing order, so in this illustration,
`trt1` has the lowest mean, `ctrl` has the next lowest, and `trt4` has the
highest. The underlines group the means such that the extremes of each group
are *not* significantly different according to a statistical test conducted at a
specified alpha level. So in this illustration, `trt1` is significantly less
than `trt3`, `trt2`, and `trt4`, but not `ctrl`; and in fact `trt4` is
significantly greater than all the others.

This grouping also illustrates the dangers created by careless interpretations.
Some observers of this chart might say that "`trt1` and `ctrl` are equal" and
that "`ctrl`, `trt3`, and `trt2` are equal" -- when in fact we have merely
failed to show they are different. And further confusion results because
mathematical equality is transitive -- that is, these two statements of equality
would imply that `trt1` and `trt2` must be equal, seemingly contradicting the
finding that they are significantly different. Statistical nonsignificance does
*not* have the transitivity property!

### Grouping using letters or symbols {#CLDs}
<!-- @index CLDs!Principles; CLDs!**multcompView** package -->
The underlining method becomes problematic in any case where the standard errors
(SEs) of the comparisons are unequal -- for example if we have unequal sample
sizes, or a model with non-homogeneous variances. When the SEs are unequal, it
is possible, for example, for two adjacent means to be significantly different,
while two more distant ones do not differ significantly. If that happens, we
can't use underlines to group the means. The problem here is that lines are
continuous, and that continuousness forces a continuum of groupings.

However, Piepho (2004) solved this problem by using *symbols* instead of lines,
and creating a display where any two means associated with the same symbol are
deemed to not be statistically different. Using symbols, it is possible to have
non-contiguous groupings, e.g., it is possible for two means to share a symbol
while an intervening one does not share the same symbol. Such a display is
called a *compact letter display*. We do not absolutely require actual letters,
just symbols that can be distinguished from one another. In the case where 
all the differences have equal SEs, the CLD will be the "same" as the result
of grouping lines, in that each distinct symbol will span a contiguous
range of means that can be interpreted as a grouping line.


The R package
**multcompView** (Graves *et al.*, 2019) provides an implementation of the
Piepho algorithm.  The multcomp package (Hothorn *et al.* 2008) provides a
generic `cld()` function, and the **emmeans** package provides a
`cld()` method for `emmGrid` objects. 

[Back to Contents](#contents)


### Simulated example {#CLD-example}
<!-- @index Examples!CLDs (simulated); CLDs!Simulated example -->
As a moving example, we simulate some data from an unbalanced design with 7 
treatments labeled A, B, ..., G; and fit a model to those
```{r}
set.seed(22.10)
mu = c(16, 15, 19, 15, 15, 17, 16)  # true means
n =  c(19, 15, 16, 18, 29,  2, 14)  # sample sizes
foo = data.frame(trt = factor(rep(LETTERS[1:7], n)))
foo$y = rnorm(sum(n), mean = mu[as.numeric(foo$trt)], sd = 1.0)

foo.lm = lm(y ~ trt, data = foo)
```
There are only four distinct true means underlying these seven treatments: Treatments `B`, `D`, and `E` have mean 15, treatments `A` and `G` have mean 16, and treatments `F` and `C`
are solo players with means 17 and 19 respectively.


#### Default CLD
Let's see a compact letter display for the marginal means.
(Call this **CLD #1**)
```{r}
foo.emm = emmeans(foo.lm, "trt")

library(multcomp)
cld(foo.emm)
```
The default "letters" for the **emmeans** implementation are actually numbers,
and we have three groupings indicated by the symbols `1`, `2`, and `3`. This
illustrates a case where grouping lines would not have worked, as we see in the
fact that group `1` is not contiguous. We have  (among other results) that
treatment `A` differs significantly from treatments `B`, `E`, `D`, `G`, and `C`
(at the default 0.05 significance level, with Tukey adjustment for multiple
testing). and that `C` is significantly greater than all the other means since
it is the only mean in group `3`.

An annotation warns that two means in the same group are not necessarily
the same; yet CLDs present a strong visual message that they are. The careless reader who makes this mistake will have trouble with the gap in group `1`, asking how `A` can differ
from `G` and yet `G` and `F`, are "the same." The explanation is that the SE of
`F` is huge, owing to its very small sample size, so it is hard for it to be
*statistically* different from other means. It is almost a gift to obtain a
non-contiguous grouping like this, as it forces the user to think more carefully
about what these grouping do and do not imply.

[Back to Contents](#contents)


### Alternative CLDs {#alt-CLDs}
<!-- @index CLDs!Alternative; CLDs!Re-engineered -->
Given the discussion above, one might wonder if it is possible to construct a CLD
in such a way that means sharing the same symbol *are* actually shown to be the same?
The answer is yes (otherwise we wouldn't have asked the question!) -- and it is quite easy to do,
thanks to two things:

  1. The algorithm for making grouping letters is based on a matrix of Boolean
     values associated with each pair of means, where we set the value `TRUE` for
     any pair that is statistically different (those means must receive different
     grouping letters), and `FALSE` otherwise; and the algorithm works for *any*
     such Boolean matrix
  2. There is such a thing as *equivalence testing*, by which we can establish
     with specified confidence that two means *do not* differ by more than a
     specified threshold $\delta$. One simple way to do this is to conduct *two
     one-sided tests* (TOST) whereby we can conclude that two means are equivalent
     if we show both that the difference exceeds $-\delta$ and is less than
     $+\delta$. We can use this TOST method to set each Boolean pair to `FALSE` is
     they are shown to be equivalent and `TRUE` if not shown to be equivalent.


#### Equivalence sets {#equiv-CLDs}
<!-- @index CLDs!For equivalence sets; Equivalence sets (with CLDs) -->
For our example, suppose, based on subject-matter considerations, that two means
that differ by less than 1.0 can be considered equivalent. In the **emmeans**
setup, we specify that we want equivalence testing simply by providing this
nonzero threshold value as a `delta` argument. In addition, we typically will
not make multiplicity adjustments to equivalence tests. Here is the result we
obtain (call this **CLD #2**)
```{r}
cld(foo.emm, delta = 1, adjust = "none")

```
So we obtain five groupings -- but only two if we ignore those that apply to
only one mean. We have that treatments `B` and `E` can be considered equivalent,
and treatments `E`, `D`, and `G` are considered equivalent. It is also important
to know that we *cannot* say that means in different groups are significantly
different.

Unlike CLD #1, we are showing only groupings of means that
we can *show* to be the same. The first four means, which were grouped together 
earlier, are now assigned to two equivalence groupings. And treatment `F` is not
grouped with any other mean -- which makes sense because we have so little
data on that treatment that we can hardly say anything.


#### Significance sets {#signif-CLDs}
<!-- @index CLDs!For significance sets; Significance sets (with CLDs) -->
Another variation is to simply reverse all the Boolean flags we used in
constructing CLD #1. Then two means will receive the same letter only if they
are significantly different. Thus, we really obtain *un*grouping letters.
We label these groupings "significance sets." The resulting display has
a distinctively different appearance, because common symbols tend to be
far apart rather than contiguous.
(Call this **CLD #3**)

```{r}
cld(foo.emm, signif = TRUE)
```
Here we have five significance sets. By comparing with CLD #1, you can
confirm that each significant difference shown explicitly here corresponds
to one shown implicitly (by *not* sharing a group) in CLD #1.

[Back to Contents](#contents)


### Conclusions {#concl}
Compact letter displays show symbols based on statistical testing results. In
such tests, we have strong conclusions or *findings* -- those that have small
*P* values, and weak conclusions or *non-findings* -- those where the *P* value
is not less than some $\alpha$. When we create visual flags such as grouping
lines or symbols, those come across visually as findings, and the problem with
standard CLDs is that those are the non-findings. We show two simple ways to use
software that creates CLDs so that actual findings are flagged with symbols. It
is hoped that people will find these modifications useful in visually displaying
comparisons among means.

### References {#refs}
<!-- @index CLDs!References -->
Graves, Spencer, Piepho Hans-Pieter, Selzer, Luciano, and Dorai-Raj, Sundar
(2019). **multcompView**: Visualizations of Paired Comparisons. R package
version 0.1-8, `https://CRAN.R-project.org/package=multcompView`
 
Hothorn, Torsten, Bretz, Frank, and Westfall, Peter (2008). Simultaneous
 Inference in General Parametric Models. *Biometrical Journal* **50(3)**,
 346--363.
  
Piepho, Hans-Peter (2004). An algorithm for a letter-based representation of all
pairwise comparisons, *Journal of Computational and Graphical Statistics*
**13(2)** 456--466.

[Back to Contents](#contents)

[Index of all vignette topics](vignette-topics.html)

---
title: "Interaction analysis in emmeans"
author: "emmeans package, Version `r packageVersion('emmeans')`"
output: emmeans::.emm_vignette
vignette: >
  %\VignetteIndexEntry{Interaction analysis in emmeans}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
```{r, echo = FALSE, results = "hide", message = FALSE}
require("emmeans")
options(show.signif.stars = FALSE)
knitr::opts_chunk$set(fig.width = 4.5, class.output = "ro", class.message = "re")
```

<!-- @index Vignettes!Interactions; Interactions!Analysis -->

Models in which predictors interact seem to create a lot of confusion
concerning what kinds of *post hoc* methods should be used. It is hoped that
this vignette will be helpful in shedding some light on how to use the 
**emmeans** package effectively in such situations.

## Contents {#contents}

  1. [Interacting factors](#factors)
     a. [Simple contrasts](#simple)
  2. [Interaction contrasts](#contrasts)
  3. [Multivariate contrasts](#multiv)
  4. [Interactions with covariates](#covariates)
  9. [Summary](#summary)


[Index of all vignette topics](vignette-topics.html)


## Interacting factors {#factors}
<!-- @index Examples!`auto.noise`; Interactions!Possible inappropriateness of marginal means 
  Interactions!Plotting; `emmip()` -->
As an example for this topic, consider the `auto.noise` dataset included with 
the package. This is a balanced 3x2x2 experiment with three replications. The
response -- noise level -- is evaluated with different sizes of cars, types of
anti-pollution filters, on each side of the car being measured.[^1]

[^1]: I sure wish I could ask some questions about how how these data were
collected; for example, are these independent experimental runs, or are some
cars measured more than once? The model is based on the independence assumption,
but I have my doubts.

Let's fit a model and obtain the ANOVA table (because of the scale of the data, 
we believe that the response is recorded in tenths of decibels; 
so we compensate for this by scaling the response):
```{r}
noise.lm <- lm(noise/10 ~ size * type * side, data = auto.noise)
anova(noise.lm)
```
There are statistically strong 2- and 3-way interactions. 

One mistake that a lot of people seem to make is to proceed too hastily to
estimating marginal means (even in the face of all these interactions!). 
They would go straight to analyses like this:
```{r}
emmeans(noise.lm, pairwise ~ size)
```
The analyst-in-a-hurry would thus conclude that the noise level is
higher for medium-sized cars than for small or large ones. 

But as is seen in the message before the output, `emmeans()` valiantly tries
to warn you that it may not be a good idea to average over factors that interact
with the factor of interest. It isn't *always* a bad idea to do this, but
sometimes it definitely is.

What about this time? 
I think a good first step is always to try to visualize the nature of the
interactions before doing any statistical comparisons. The following plot helps.
```{r fig.alt = "emmip plot of type and size, with separate panels for each side. To see the same numerical information, do emmeans(noise.lm, ~ type:size|side)"}
emmip(noise.lm, type ~ size | side)
```

Examining this plot, we see that the "medium" mean is not
always higher; so the marginal means, and the way they compare, does not
represent what is always the case. Moreover,
what is evident in the plot is that the peak for medium-size cars occurs for
only one of the two filter types. So it seems more useful to do the comparisons
of size separately for each filter type. This is easily done, simply by conditioning on `type`:
```{r}
emm_s.t <- emmeans(noise.lm, pairwise ~ size | type)
emm_s.t
```
Not too surprisingly, the statistical comparisons are all different for standard
filters, but with Octel filters, there isn't much of a difference between small
and medium size.

<!-- @index * gazing (star gazing)@star -->
For comparing the levels of other factors, similar judgments must be made. It
may help to construct other interaction plots with the factors in different
roles. In my opinion, almost all meaningful statistical analysis should be
grounded in evaluating the practical impact of the
estimated effects *first*, and seeing if the statistical evidence backs it up. 
Those who put all their attention on how many asterisks
(I call these people "`*` gazers") are ignoring the fact that these don't measure 
the sizes of the effects on a practical scale.[^2]
An effect can be practically negligible and still have a very small *P* value -- 
or practically important but have a large *P* value -- depending on sample size and error variance.
Failure to describe what is actually going on in the data is a failure to do an adequate analysis. 
Use lots of plots, and *think* about the results. For more on this, see the discussion
of *P* values in the ["basics" vignette](basics.html#pvalues).

[^2]: You may have noticed that there are no asterisks in the ANOVA table in
this vignette. I habitually opt out of star-gazing by including
`options(show.signif.stars = FALSE)` in my `.Rprofile` file.

### Simple contrasts {#simple}
<!-- @index Simple comparisons; `contrast()`!`simple`; `contrast()`!`combine` -->
An alternative way to specify conditional contrasts or comparisons is through
the use of the `simple` argument to `contrast()` or `pairs()`, which amounts to
specifying which factors are *not* used as `by` variables. For example, consider:
```{r}
noise.emm <- emmeans(noise.lm, ~ size * side * type)
```
Then `pairs(noise.emm, simple = "size")` is the same as 
`pairs(noise.emm, by = c("side", "type"))`.

One may specify a list for `simple`, in which case separate runs are made with
each element of the list. Thus, `pairs(noise.emm, simple = list("size",
c("side", "type"))` returns two sets of contrasts: comparisons of `size` for
each combination of the other two factors; and comparisons of `side*type`
combinations for each `size`.

A shortcut that generates all simple main-effect comparisons is to use `simple =
"each"`. In this example, the result is the same as obtained using `simple =
list("size", "side", "type")`.

Ordinarily, when `simple` is a list (or equal to `"each"`), a list of contrast sets is returned. However, if the additional argument `combine` is set to `TRUE`, they are all combined into one family:
```{r}
contrast(noise.emm, "consec", simple = "each", combine = TRUE, adjust = "mvt")
```
The dots (`.`) in this result correspond to which simple effect is being
displayed. If we re-run this same call with `combine = FALSE` or omitted, these
twenty comparisons would be displayed in three broad sets of contrasts, each
broken down further by combinations of `by` variables, each separately
multiplicity-adjusted (a total of 16 different tables).

[Back to Contents](#contents)

## Interaction contrasts {#contrasts}
<!-- @index Interactions!Contrasts; `contrast()`!`interaction` 
     Contrasts!of other contrasts@cont -->
An interaction contrast is a contrast of contrasts. For instance, in the auto-noise
example, we may want to obtain the linear and quadratic contrasts of `size` separately
for each `type`, and compare them. Here are estimates of those contrasts:
```{r}
contrast(emm_s.t[[1]], "poly")   ## 'by = "type"' already in previous result 
```
The comparison of these contrasts may be done using the `interaction`
argument in `contrast()` as follows:
```{r}
IC_st <- contrast(emm_s.t[[1]], interaction = c("poly", "consec"), by = NULL)
IC_st
```
(Using `by = NULL` restores `type` to a primary factor in these contrasts.) 
The practical meaning of this is that there isn't a statistical difference in
the linear trends, but the quadratic trend for Octel is greater than for
standard filter types. (Both quadratic trends are negative, so in fact it is the
standard filters that have more pronounced *downward* curvature, as is seen in
the plot.) In case you need to understand more clearly what contrasts are being
estimated, the `coef()` method helps:
```{r}
coef(IC_st)
```
Note that the 4th through 6th contrast coefficients are the negatives of the 1st through 3rd -- thus a comparison of two contrasts.

By the way, "type III" tests of interaction effects can be obtained via interaction contrasts:
```{r}
test(IC_st, joint = TRUE)
```
This result is exactly the same as the *F* test of `size:type` in the `anova` 
output.

The three-way interaction may be explored via interaction contrasts too:
```{r}
emmeans(noise.lm, ~ size*type*side) |>
    contrast(interaction = c("poly", "consec", "consec"))
```
One interpretation of this is that the comparison by `type` of the linear 
contrasts for `size` is different on the left side than on the right side; but 
the comparison of that comparison of the quadratic contrasts, not so much. Refer
again to the plot, and this can be discerned as a comparison of the interaction
in the left panel versus the interaction in the right panel.

<!-- @index `joint_tests()` -->
Finally, **emmeans** provides a `joint_tests()` function that obtains and tests
the interaction contrasts for all effects in the model and compiles them in one
Type-III-ANOVA-like table:
```{r}
joint_tests(noise.lm)
```
You may even add `by` variable(s) to obtain separate ANOVA tables for the
remaining factors:
```{r}
joint_tests(noise.lm, by = "side")
```

[Back to Contents](#contents)

## Multivariate contrasts {#multiv}
<!-- @index Multivariate contrasts; Contrasts!Multivariate; Hotelling's $T^2$;
            `mvcontrast()`    -->
In the preceding sections, the way we addressed interacting factors was to do
comparisons or contrasts of some factors()) separately at levels of other factor(s). 
This leads to a lot of estimates and associated tests.

Another approach is to compare things in a multivariate way. In the auto-noise example,
for example, we have four means (corresponding to the four combinations of `type` and `size`)
with each size of car, and we could consider comparing 
these *sets* of means. Such multivariate comparisons can be done via the *Mahalanobis distance*
(a kind of standardized distance measure) between one set of four means and another.
This is facilitated by the `mvcontrast()` function:
```{r}
mvcontrast(noise.emm, "pairwise", mult.name = c("type", "side"))
```
In this output, the `T.square` values are Hotelling's $T^2$ statistics, which are
the squared Mahalanobis distances among the sets of four means. These results thus 
accomplish a similar objective as the initial comparisons presented in this vignette,
but are not complicated by the issue that the factors interact. (Instead, we lose the 
directionality of the comparisons.) While all comparisons are "significant," the `T.square`
values indicate that large cars are statistically most different from the other sizes.

We may still break things down using `by` variables. Suppose, for example, we wish to 
compare the two filter types for each size of car, without regard to which side:
```{r}
update(mvcontrast(noise.emm, "consec", mult.name = "side", by = "size"), 
       by = NULL)
```

One detail to note about multivariate comparisons: in order to make complete sense,
all the factors involved must interact. Suppose we were to repeat the initial multivariate
comparison after removing all interactions:
```{r}
mvcontrast(update(noise.emm, submodel = ~ side + size + type), 
           "pairwise", mult.name = c("type", "side"))
```
Note that each $F$ ratio now has 1 d.f. Also, note that `T.square = F.ratio`,
and you can verify that these values are equal to the squares of the `t.ratio`s in the initial
example in this vignette ($(-6.147)^2 = 37.786$, etc.). That is, if we ignore all interactions, the multivariate
tests are exactly equivalent to the univariate tests of the marginal means.

[Back to Contents](#contents)



## Interactions with covariates {#covariates}
<!-- @index Interactions!Covariate with factors; `emtrends()`; Examples!`fiber` -->
When a covariate and a factor interact, we typically don't want EMMs themselves,
but rather estimates of *slopes* of the covariate trend for each level of the
factor. As a simple example, consider the `fiber` dataset, and fit a model
including the interaction between `diameter` (a covariate) and `machine` (a factor):
```{r}
fiber.lm <- lm(strength ~ diameter*machine, data = fiber)
```
This model comprises fitting, for each machine, a separate linear trend for
`strength` versus `diameter`. Accordingly, we can estimate and compare the
slopes of those lines via the `emtrends()` function:
```{r}
emtrends(fiber.lm, pairwise ~ machine, var = "diameter")
```
We see the three slopes, but no two of them test as being statistically different.

To visualize the lines themselves, you may use
```{r fig.height = 2, fig.alt = "Plot with three increasing lines. The one for B is highest, the one for C is lower and more-or-less parallel to it, and the one for A is steeper and crosses the other two lines near their ends"}
emmip(fiber.lm, machine ~ diameter, cov.reduce = range)
```

The `cov.reduce = range` argument is passed to `ref_grid()`; it is needed
because by default, each covariate is reduced to only one value (see the
["basics" vignette](basics.html)). Instead, we call the `range()` function to
obtain the minimum and maximum diameter.

######### {#oranges}
<!-- @index Examples!`oranges`; Multivariate models; Interactions!Implied -->
For a more sophisticated example, consider the `oranges` dataset included with
the package. These data concern the sales of two varieties of oranges. The
prices (`price1` and `price2`) were experimentally varied in different stores
and different days, and the responses `sales1` and `sales2` were observed. Let's
consider three multivariate models for these data, with additive effects for
days and stores, and different levels of fitting on the prices:
```{r}
org.quad <- lm(cbind(sales1, sales2) ~ poly(price1, price2, degree = 2)
                                       + day + store, data = oranges)
org.int <- lm(cbind(sales1, sales2) ~ price1 * price2 + day + store, data = oranges)
org.add <- lm(cbind(sales1, sales2) ~ price1 + price2 + day + store, data = oranges)
```
Being a multivariate model, **emmeans** methods will distinguish the responses
as if they were levels of a factor, which we will name "variety". Moreover,
separate effects are estimated for each multivariate response, so there is an
*implied interaction* between `variety` and each of the predictors involving
`price1` and `price2`. (In `org.int`, there is an implied three-way
interaction.) An interesting way to view these models is to look at how they
predict sales of each variety at each observed values of the prices:
```{r fig.alt = "Two panels with sets of curves associated with values of price2. The panel for sales1 shows a tight bundle of curves that decrease with price1. The panel for sales2 shows a looser bundle of curves that diverge as price1 increases"}
emmip(org.quad, price2 ~ price1 | variety, mult.name = "variety", cov.reduce = FALSE)
```

The trends portrayed here are quite sensible: In the left panel, as we increase 
the price of variety 1, sales of that variety will tend to decrease -- and the 
decrease will be faster when the other variety of oranges is low-priced. In the
right panel, as price of variety 1 increases, sales of variety 2 will increase 
when it is low-priced, but could decrease also at high prices because oranges in
general are just too expensive. A plot like this for `org.int` will be similar 
but all the curves will be straight lines; and the one for `plot.add` will have 
all lines parallel. In all models, though, there are implied `price1:variety`
and `price2:variety` interactions, because we have different regression
coefficients for the two responses.

Which model should we use? They are nested models, so they can be compared by 
`anova()`:
```{r}
anova(org.quad, org.int, org.add)
```
It seems like the full-quadratic model has little advantage over the 
interaction model. There truly is nothing magical about a *P* value of 0.05,
and we have enough data that over-fitting is not a hazard; so I like `org.int`.
However, what follows could be done with any of these models.

<!-- @index Trends!Estimating and comparing; `emtrends()` -->
To summarize and test the results compactly, it makes sense to obtain estimates
of a representative trend in each of the left and right panels, and perhaps to
compare them. In turn, that can be done by obtaining the slope of the curve (or
line) at the average value of `price2`. The `emtrends()` function is designed
for exactly this kind of purpose. It uses a difference quotient to estimate the
slope of a line fitted to a given variable. It works just like `emmeans()`
except for requiring the variable to use in the difference quotient. Using the
`org.int` model:
```{r}
emtrends(org.int, pairwise ~ variety, var = "price1", mult.name = "variety")
```
From this, we can say that, starting with `price1` and `price2` both at their
average values, we expect `sales1` to decrease by about .75 per unit
increase in `price1`; meanwhile, there is a suggestion of a slight increase of `sales2`,
but without much statistical evidence. 
Marginally, the first variety has a 0.89 disadvantage relative to
sales of the second variety.

Other analyses (not shown) with `price2` set at a higher value will reduce these
effects, while setting `price2` lower will exaggerate all these effects. If the
same analysis is done with the quadratic model, the the trends are curved, and
so the results will depend somewhat on the setting for `price1`. The graph above
gives an indication of the nature of those changes.

Similar results hold when we analyze the trends for `price2`:
```{r}
emtrends(org.int, pairwise ~ variety, var = "price2", mult.name = "variety")
```
At the averages, increasing the price of variety 2 has the effect of decreasing
sales of variety 2 while slightly increasing sales of variety 1 -- a marginal
difference of about .92.

[Back to Contents](#contents)

## Summary {#summary}
Interactions, by nature, make things more complicated. One must resist pressures 
and inclinations to try to produce simple bottom-line conclusions. Interactions
require more work and more patience; they require presenting more cases -- more
than are presented in the examples in this vignette -- in order to provide a
complete picture.

[Index of all vignette topics](vignette-topics.html)

---
title: "Explanations supplement"
author: "emmeans package, Version `r packageVersion('emmeans')`"
output: emmeans::.emm_vignette
vignette: >
  %\VignetteIndexEntry{Explanations supplement}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, echo = FALSE, results = "hide", message = FALSE}
require("emmeans")
require("MASS")
knitr::opts_chunk$set(fig.width = 4.5, fig.height = 2.0, class.output = "ro",
                      class.message = "re", class.error = "re", class.warning = "re")
###knitr::opts_chunk$set(fig.width = 4.5, fig.height = 2.0)
```

<!-- @index Vignettes!Explanations supplement -->

This vignette provides additional documentation for some methods 
implemented in the **emmeans** package.

[Index of all vignette topics](vignette-topics.html)


## Contents {#contents}

  1. [Sub-models](#submodels)
  2. [Comparison arrows](#arrows)
  3. [Confounded effects in joint tests](#joint_tests)
  4. [Intricacies of offsets](#offsets)



## Sub-models {#submodels}
<!-- @index `submodel`; Alias matrix; EMMs!Projecting to a submodel; 
             Constrained marginal means -->

Estimated marginal means (EMMs) and other statistics computed by the **emmeans**
package are *model-based*: they depend on the model that has been fitted to the
data. In this section we discuss a provision whereby a different underlying
model may be considered. The `submodel` option in `update()` can project EMMs
and other statistics to an alternative universe where a simpler version of the
model has been fitted to the data. Another way of looking at this is that it
constrains certain external effects to be zero -- as opposed to averaging over
them as is otherwise done for marginal means.

Two things to know before getting into details:

  1. The `submodel` option uses information from the fixed-effects portion 
     of the model matrix
  2. Not all model classes are supported for the `submodel` option. 

Now some details. Suppose that we have a fixed-effects model matrix $X$, and let
$X_1$ denote a sub-matrix of $X$ whose columns correspond to a specified
sub-model. (Note: if there are weights, use $X = W^{1/2}X^*$, where $X^*$ is the
model matrix without the weights incorporated.) The trick we use is what is
called the *alias matrix*: $A = (X_1'X_1)^-X_1'X$ where $Z^-$ denotes a
generalized inverse of $Z$. It can be shown that $(X_1'X_1)^-X_1' = A(X'X)^-X'$;
thus, in an ordinary fixed-effects regression model, $b_1 = Ab$ where $b_1$ and
$b$ denote the regression coefficients for the sub-model and full model,
respectively. Thus, given a matrix $L$ such that $Lb$ provides estimates of
interest for the full model, the corresponding estimates for the sub-model are
$L_1b_1$, where $L_1$ is the sub-matrix of $L$ consisting of the columns
corresponding to the columns of $X_1$. Moreover, $L_1b_1 = L_1(Ab) = (L_1A)b$;
that is, we can replace $L$ by $L_1A$ to obtain estimates from the sub-model.
That's all that `update(..., submodel = ...)` does.

Here are some intuitive observations:

  1. Consider the excluded effects, $X_2$, consisting of the columns of $X$ other than $X_1$.
     The corresponding columns of the alias matrix are regression coefficients
     treating  $X_2$ as the response and $X_1$ as the predictors.
  2. Thus, when we obtain predictions via these aliases, we are predicting the
     effects of $X_2$ based on $X_1$.
  3. The columns of the new linear predictor $\tilde L = L_1A$ depend
     only on the columns of $L_1$, and hence not on other columns of $L$.

These three points provide three ways of saying nearly the same thing, namely
that we are excluding the effects in $X_2$. Note that in  a rank-deficient situation,
there are different possible generalized inverses, and so in (1), $A$ is not unique. 
However, the predictions in (2) are unique. In ordinary regression models, (1), (2),
and (3) all apply and will be the same as predictions from re-fitting the model with
model matrix $X_1$; however, in generalized linear models, mixed models, etc., 
re-fitting will likely produce somewhat different results. That is because fitting
such models involves iterative weighting, and the re-fitted models will probably
not have the same weights. However, point (3) will still hold: the predictions
obtained with a submodel will involve only the columns of $L_1$ and hence
constrain all effects outside of the sub-model to be zero. Therefore,
when it really matters to get the correct estimates from the stated sub-model, 
the user should actually fit that sub-model unless the full model is an 
ordinary linear regression.

A technicality: Most writers define the alias matrix as $(X_1'X_1)^-X_1'X_2$,
where $X_2$ denotes that part of $X$ that excludes the columns of $X_1$. We are
including all columns of $X$ here just because it makes the notation very
simple; the $X_1$ portion of $X$ just reduces to the identity (at least in the
case where $X_1$ is full-rank).

A word on computation: 
Like many matrix expressions, we do not compute $A$ directly as shown. Instead,
we use the QR decomposition of $X_1$, obtainable via the R call `Z <- qr(X1)`.
Then the alias matrix is computed via `A <- qr.coef(Z, X)`. In fact, nothing
changes if we use just the $R$ portion of $X = QR$, saving us both memory and
computational effort. The exported function `.cmpMM()` extracts this $R$ matrix,
taking care of any pivoting that might have occurred. And in an `lm` object, the
QR decomposition of $X$ is already saved as a slot. The `qr.coef()` function
works just fine in both the full-rank and rank-deficient cases, but in the
latter situation, some elements of `A` will be `NA`; those correspond to
"excluded" predictors, but that is another way of saying that we are
constraining their regression coefficients to be zero. Thus, we can easily clean
that up via `A[is.na(A)] <- 0`.

If we specify `submodel = "minimal"`, the software figures out the sub-model by
extracting terms involving only factors that have not already been averaged
over. If the user specifies `submodel = "type2"`, an additional step is
performed: Let $X_1$ have only the highest-order effect in the minimal model, and $X_0$ 
denote the matrix of all columns of $X$ whose columns do not contain the effect in $X_1$.
We then replace $Z$
by the QR decomposition of $[I - X_0(X_0'X_0)^-X_0']X_1^*$. This projects
$X_1^*$ onto the null space of $X_0$. The net result is that we obtain estimates
of just the $X_1^*$ effects, after adjusting for all effects that don't contain it
(including the intercept if present). Such estimates have very limited use in
data description, but provide a kind of "Type II" analysis when used in
conjunction with `joint_tests()`. The `"type2"` calculations parallel those
[documented by
SAS](https://documentation.sas.com/?docsetId=statug&docsetTarget=statug_introglmest_sect011.htm&docsetVersion=14.3&locale=en)
for obtaining type II estimable functions in SAS `PROC GLM`. However, we (as well as
`car::Anova()`) define "contained" effects differently
from SAS, treating covariates no differently than factors.

### A note on multivariate models {#mult.submodel}
<!-- @index `submodel`!in a multivariate model; 
            Multivariate models!with `submodel` -->
Recall that **emmeans** generates a constructed factor for the levels of a multivariate
response. That factor (or factors) is completely ignored in any sub-model calculations.
The $X$ and $X_1$ matrices described above involve only the predictors in the right-hand 
side of the model equation . The multivariate response "factor" implicitly interacts
with everything in the right-hand-side model; and the same is true of any sub-model.
So it is not possible to consider sub-models where terms are omitted from among those
multivariate interactions (note that it is also impossible to fit a
multivariate sub-model that excludes those interactions).
The only way to remove consideration of multivariate effects is 
to average over them via a call to `emmeans()`.


[Back to Contents](#contents)

<!-- This section is a rewrite of an answer I provided on Stack Overflow,
  https://stackoverflow.com/questions/61779348/how-does-emmeans-calculate-confidence-intervals-used-to-compare-means/61822331#61822331 -->
  
## Comparison arrows {#arrows}
<!-- @index Comparisons!Graphical!How arrows are determined;
            Comparison arrows!Derivation -->

The `plot()` method for `emmGrid` objects offers the option `comparisons = TRUE`.
If used, the software attempts to construct "comparison arrows" whereby two
estimated marginal means (EMMs) differ significantly if, and only if, their
respective comparison arrows do not overlap. In this section, we explain
how these arrows are obtained.

First, please understand these comparison arrows are decidedly *not* the same as
confidence intervals. Confidence intervals for EMMs are based on the statistical
properties of the individual EMMs, whereas comparison arrows are based on the
statistical properties of *differences* of EMMs.

Let the EMMs be denoted $m_1, m_2, ..., m_k$. For simplicity, let us assume that
these are ordered: $m_1 \le m_2 \le \cdots \le m_k$. Let $d_{ij} = m_j - m_i$
denote the difference between the $i$th and $j$th EMM. Then the $(1 - \alpha)$
confidence interval for the true difference $\delta_{ij} = \mu_j - \mu_i$ is
$$  d_{ij} - e_{ij}\quad\mbox{to}\quad d_{ij} + e_{ij} $$ 
where $e_{ij}$ is the "margin of error"
for the difference; i.e., $e_{ij} = t\cdot SE(d_{ij})$ for some critical value $t$ (equal to $t_{\alpha/2}$ when no multiplicity adjustment is used). Note that
$d_{ij}$ is statistically significant if, and only if, $d_{ij} > e_{ij}$.

Now, how to get the comparison arrows? These arrows are plotted with origins at the $m_i$; we have an arrow of length $L_i$ pointing to the left, and an arrow of length $R_i$ pointing to the right. To compare EMMs $m_i$ and $m_j$ (and remembering that we are supposing that $m_i \le m_j$), we propose to look to see if the arrows extending right from $m_i$ and left from $m_j$ overlap or not. So, ideally, if we want overlap to be identified with statistical non-significance, we want 
$$    R_i + L_j = e_{ij}   \quad\mbox{for all } i < j $$

If we can do that, then the two arrows will overlap if, and only if, $d_{ij} < e_{ij}$.

This is easy to accomplish if all the $e_{ij}$ are equal: just set all $L_i = R_j = \frac12e_{12}$. But with differing $e_{ij}$ values, it may or may not even be possible to obtain suitable arrow lengths. 

The code in **emmeans** uses an *ad hoc* weighted regression method to solve the above equations. We give greater weights to cases where $d_{ij}$ is close to $e_{ij}$, because those are the cases where it is more critical that we get the lengths of the arrows right. 
Once the regression equations are solved, we test to make sure that $R_i + L_j < d_{ij}$ when the difference is significant, and $\ge d_{ij}$ when it is not. If one or more of those checks fails, a warning is issued.

That's the essence of the algorithm. Note, however, that there are a few complications that need to be handled:

  * For the lowest EMM $m_1$, $L_1$ is completely arbitrary because there are no right-pointing arrows with which to compare it; in fact, we don't even need to display that arrow. The same is true of $R_k$ for the largest EMM $m_k$. Moreover, there could be additional unneeded arrows when other $m_i$ are equal to $m_1$ or $m_k$.
  * Depending on the number $k$ of EMMs and the number of tied minima and maxima, the system of equations could be under-determined, over-determined, or just right.
  * It is possible that the solution could result in some $L_i$ or $R_j$ being negative. That would result in an error.

In summary, the algorithm does not always work (in fact it is possible to construct cases where no solution is possible). But we try to do the best we can. The main reason for trying to do this is to encourage people to not ever use confidence intervals for the $m_i$ as a means of testing the comparisons $d_{ij}$. That is almost always incorrect. 

What is better yet is to simply avoid using comparison arrows altogether and use `pwpp()` or `pwpm()` to display the *P* values directly. 

### Examples and tests
Here is a constructed example with specified means and somewhat unequal SEs
```{r, message = FALSE, fig.alt = "Plot A: Comparison arrows for foo. These arrows overlap or not as per the P-values in pairs(foo) are greater or less than 0.05"}
m = c(6.1, 4.5, 5.4,    6.3, 5.5, 6.7)
se2 = c(.3, .4, .37,  .41, .23, .48)^2
lev = list(A = c("a1","a2","a3"), B = c("b1", "b2"))
foo = emmobj(m, diag(se2), levels = lev, linfct = diag(6))
plot(foo, CIs = FALSE, comparisons = TRUE)
```

This came out pretty well. But now let's keep the means and SEs the same but make them correlated. Such correlations happen, for example, in designs with subject effects.
The function below is used to set a specified intra-class correlation, treating `A` as a within-subjects (or split-plot) factor and `B` as a between-subjects (whole-plot) factor.
We'll start with a correlation of 0.3.
```{r, message = FALSE, fig.alt = "Plot B - for foo3. The arrow lengths are more diverse than in plot A, but the arrows still all work correctly"}
mkmat <- function(V, rho = 0, indexes = list(1:3, 4:6)) {
    sd = sqrt(diag(V))
    for (i in indexes)
        V[i,i] = (1 - rho)*diag(sd[i]^2) + rho*outer(sd[i], sd[i])
    V
}
# Intraclass correlation = 0.3
foo3 = foo
foo3@V <- mkmat(foo3@V, 0.3)
plot(foo3, CIs = FALSE, comparisons = TRUE)
```

Same with intraclass correlation of 0.6:
```{r, message = FALSE, fig.alt = "Plot C - for foo6. Not all of the arrows work correctly and we get a warning message with details. The appearance of the arrows is pretty chaotic, with many extending in vastly unequal distances from the means"}
foo6 = foo
foo6@V <- mkmat(foo6@V, 0.6)
plot(foo6, CIs = FALSE, comparisons = TRUE)
```

Now we have a warning that some arrows don't overlap, but should.
We can make it even worse by upping the correlation to 0.8:
```{r, message = FALSE, error = TRUE}
foo8 = foo
foo8@V <- mkmat(foo8@V, 0.8)
plot(foo8, CIs = FALSE, comparisons = TRUE)
```

Now the solution actually leads to negative arrow lengths.

What is happening here is we are continually reducing the SE of within-B comparisons 
while keeping the others the same. These all work out if we use `B` as a `by` variable:
```{r, message = FALSE, fig.alt = "Plot D - foo8, separate panels for each B. This is a nicely behaved plot because we are not mixing together between-B and within-B SEs."}
plot(foo8, CIs = FALSE, comparisons = TRUE, by = "B")
```

Note that the lengths of the comparison arrows are relatively equal within the levels of `B`. Or, we can use `pwpp()` or `pwpm()` to show the *P* values for all comparisons among the six means:
```{r fig.alt = "pwpp of foo6, providing a fairly readable display of the P-values in pairs(foo6)"}
pwpp(foo6, sort = FALSE)
pwpm(foo6)
```

### Related reading on comparisons
The following reference (as well as some articles referenced therein) have been 
suggested to me as relating to similar issues:

Franz, V.H., Loftus, G.R. (2012) Standard errors and confidence intervals in within-subjects designs: Generalizing Loftus and Masson (1994) and avoiding the biases of alternative accounts. *Psychonomic Bulletin and Review* **19**, 395-–404. https://doi.org/10.3758/s13423-012-0230-1

[Back to Contents](#contents)

## Confounded effects in joint tests {#joint_tests}
<!-- @index Confounded effects; `joint_tests()`!Confounded effects;
       Rank deficiency; Missing cells;  Estimable functions -->
When a design has empty cells, this creates estimability issues; and in the
context of `joint_tests()`, this means that certain main- and interaction
effects cannot be estimated, leading to reduced degrees of freedom in the joint
tests. Instead, the estimable parts of these lost effects are pooled together
into a set of contrasts labeled `(confounded)` in the output (as long as
`showconf = TRUE`). Here is a simple illustration based on the `warpbreaks` 
dataset with the last cell removed.
```{r}
options(contrasts = c("contr.treatment", "contr.poly"))   ## ensure system default
w <- warpbreaks[1:40, ]   ### no data for (wool = B, tension = H)
w.lm <- lm(breaks ~ wool * tension, data = w)
w.rg <- ref_grid(w.lm)
(jt <- joint_tests(w.rg))
```
With no empty cells, we would have had 1 d.f. for `wool`, 2 d.f. for `tension`, 
and 2 d.f. for `wool:tension`, for a total of 5 d.f. (6 cells, 6 - 1 = 5 d.f.
for contrasts among them). With the missing cell, we can estimate only 
5 - 1 = 4 d.f. worth of effects, and the `(confounding)` part accounts for
2 of them that are not purely main effects nor interaction effects.

How do we obtain these extra 2 d.f.? Well, first of all, it is easy to 
obtain a joint test of *all* comparisons among the cell means
```{r}
(test.all <- test(contrast(w.rg, "consec"), joint = TRUE))
```
What we do is simply pool together all the contrasts that were tested in `jt`,
and compare them with the above. To do this, we obtain the estimable functions associated with `jt`:
```{r}
(ef <- attr(jt, "est.fcns"))
```
Then we can get a joint test of all these together via some trickery:
```{r}
tmp <- w.rg
tmp@linfct <- do.call(rbind, ef)      # combine EFs into a matrix
tmp@grid <- tmp@grid[1:2, ]           # replace the @linfct and @grid slots
(test.ef <- test(tmp, joint = TRUE))  #  -- that's enough to get the test
```
(Note: In case this isn't obvious, we couldn't have just pooled together the F
ratios for those effects, because they are type-III tests which are not
necessarily independent.)
So now we have `test.all` with the 4 d.f. worth of all effects comparing the
cells, and `test.ef` with just the effects in the top part of the `jt` table.
We can get the unexplained part of `test.all` by d.f.-weighted subtraction:
```{r}
(test.all$df1 * test.all$F.ratio  -  test.ef$df1 * test.ef$F.ratio) /
    (test.all$df1 - test.ef$df1)
```
This gives us the `F.ratio` shown in the `(confounded)` row of
the `jt` results above. That's all there is to it!


[Back to Contents](#contents)


## Intricacies of offsets {#offsets}
<!-- @index Offsets!`offset()` term vs. `offset` argument in model;
     Examples!`Insurance`              -->
Many model-fitting functions provide two ways of specifying model offsets: 
in the model formula, or in a separate `offset` argument. The purpose of this section
is to discuss how to deal with these in **emmeans**, and in particular, why
we decided to handle them differently, even though they seem equivalent.

To illustrate, consider the following two models:
```{r}
require(MASS)
mod1 <- glm(Claims ~ District + Group + Age + offset(log(Holders)),
            data = Insurance,
            family = poisson)
mod2 <- glm(Claims ~ District + Group + Age,
            offset = log(Holders),
            data = Insurance, 
            family = poisson)
```
Both models specify, in different ways, an offset of `log(Holders)`; 
and they both have the same regression coefficients and fitted values.

We do not treat these two models in the same way in **emmeans**.
In `mod1`, the offset is out there as part of the model formula, and so it
is treated as such in making predictions: the linear prediction includes the term
`log(Holders)`, with a regression coefficient of 1. In `mod2`, the model formula 
does not include the variable `Holders`, so we don't use it as a predictor. Instead, 
we regard the `offset` argument as implying that there is a covariate
that is computed in advance of fitting the model, and constrained to have a
regression coefficient of 1. Let's see how this plays out by looking at the reference grids:
```{r}
(rg1 <- ref_grid(mod1))
(rg2 <- ref_grid(mod2))
```
The distinction is in the fourth predictor -- in `mod1`, it is `Holders`,
and in `mod2`, it is a generated covariate named `.static.offset.`.
(We call it a "static" offset because does not depend on model predictors,
whereas `mod1` could be regarded as having a dynamic offset.)
In either case, that fourth variable is set to its mean by default.

Now let's compare some EMMs:
```{r}
emmeans(rg1, "Age")
emmeans(rg2, "Age")
```
We don't get the same estimates. That is because in `mod1` (`rg1`),
the offset that is added is $\log\text{Holders} = \log{364.98} \approx 5.90$,
while in `mod2` (`rg2`), the added offset is $\text{.static.offset.} \approx 4.90$.
Thinking through this, these offsets are the logs of the arithmetic and geometric
means, respectively, of `Holders`; and it is the fact that the offset is logged that
makes those EMMs differ.

With Poisson models, we often want to estimate rates rather than counts; and we 
can do that by specifying an offset of $\log 1 = 0$ to get a rate per holder:
```{r}
emmeans(rg1, "Age", offset = 0, type = "response")
```
We'd get exactly the same estimates using `rg2` because in both cases, we are 
forcing the offset to be zero.

Finally, suppose we want to use different offsets for each age group. This is
done as follows: For `mod1`, we compute `mean(Holders)` separately for each `Age`:
```{r}
emmeans(mod1, "Age", cov.reduce = Holders ~ Age)
```
For `mod2`, `Holders` is not considered a predictor, and instead we need
separate mean offsets:
```{r}
emmeans(mod2, "Age", cov.reduce = .static.offset. ~ Age)
```
In both models, the EMMs increase rather than decrease with age,
reflecting larger numbers of holders as age increases.

[Back to Contents](#contents)


[Index of all vignette topics](vignette-topics.html)

--- 
title: "Working with messy data" 
author: "emmeans package, Version `r packageVersion('emmeans')`"
output: emmeans::.emm_vignette
vignette: > 
  %\VignetteIndexEntry{Working with messy data} 
  %\VignetteEngine{knitr::rmarkdown} 
  %\VignetteEncoding{UTF-8}
--- 
```{r, echo = FALSE, results = "hide", message = FALSE} 
require("emmeans")
require("ggplot2")
options(show.signif.stars = FALSE) 
knitr::opts_chunk$set(fig.width = 4.5, class.output = "ro") 
```
<!-- @index Vignettes!Messy data -->

## Contents {#contents}

  1. [Issues with observational data](#issues) 
  2. [Mediating covariates](#mediators) 
  3. [Mediating factors and weights](#weights)
  3. [Nuisance factors](#nuisance)
  3. [Counterfactuals and G-Computation](#counterfact)
  4. [Sub-models](#submodels)
  5. [Nested fixed effects](#nesting)
     a. [Avoiding mis-identified nesting](#nest-trap)

[Index of all vignette topics](vignette-topics.html)

## Issues with observational data {#issues}
<!-- @index Observational data -->
In experiments, we control the
conditions under which observations are made. Ideally, this leads to balanced
datasets and clear inferences about the effects of those experimental
conditions. In observational data, factor levels are observed rather than
controlled, and in the analysis we control *for* those factors and covariates.
It is possible that some factors and covariates lie in the causal path for other
predictors. Observational studies can be designed in ways to mitigate some of
these issues; but often we are left with a mess. Using EMMs does not solve the
inherent problems in messy, undesigned studies; but they do give us ways to
compensate for imbalance in the data, and allow us to estimate meaningful
effects after carefully considering the ways in which they can be confounded.

####### {#nutrex}
<!-- @index Examples!`nutrition` -->
As an illustration, consider the `nutrition` dataset provided with the package. 
These data are used as an example in Milliken and Johnson (1992), *Analysis of 
Messy Data*, and contain the results of an observational study on nutrition 
education. Low-income mothers are classified by race, age category, and whether 
or not they received food stamps (the `group` factor); and the response variable
is a gain score (post minus pre scores) after completing a nutrition training 
program. First, let's fit a model than includes all main effects and 2-way 
interactions, and obtain its "type II" ANOVA:
```{r}
nutr.lm <- lm(gain ~ (age + group + race)^2, data = nutrition) 
car::Anova(nutr.lm)
```
 There is definitely a
`group` effect and a hint of and interaction with `race`. Here are the EMMs for
those two factors, along with their counts:
```{r}
emmeans(nutr.lm, ~ group * race, calc = c(n = ".wgt."))
```

####### {#nonest}
<!-- @index Non-estimability; Estimability -->
Hmmmm. The EMMs
when `race` is "Hispanic" are not given; instead they are flagged as 
non-estimable. What does that mean? Well, when using a model to make
predictions, it is impossible to do that beyond the linear space of the data
used to fit the model. And we have no data for three of the age groups in the
Hispanic population:
```{r}
with(nutrition, table(race, age))
```
 We can't make
predictions for all the cases we are averaging over in the above EMMs, and that
is why some of them are non-estimable. The bottom line is that we simply cannot
include Hispanics in the mix when comparing factor effects. That's a limitation
of this study that cannot be overcome without collecting additional data. Our
choices for further analysis are to focus only on Black and White populations;
or to focus only on age group 3. For example (the latter):
```{r}
emmeans(nutr.lm, pairwise ~ group | race, at = list(age = "3")) |>
    summary(by = NULL)
```
 (We used trickery with providing a `by` variable, and then taking it
away, to make the output more compact.) Evidently, the training program has been
beneficial to the Black and White groups in that age category. There is no 
conclusion for the Hispanic group -- for which we have very little data.

[Back to Contents](#contents)

## Mediating covariates {#mediators} 
<!-- @index Mediating covariates; Examples!`framing`; Covariates!Mediating -->
The `framing` data in the **mediation**
package has the results of an experiment conducted by Brader et al. (2008) where
subjects were given the opportunity to send a message to Congress regarding
immigration. However, before being offered this, some subjects (`treat = 1`)
were first shown a news story that portrays Latinos in a negative way. Besides
the binary response (whether or not they elected to send a message), the
experimenters also measured `emo`, the subjects' emotional state after the
treatment was applied. There are various demographic variables as well. Let's a
logistic regression model, after changing the labels for `educ` to shorter
strings. 
```{r}
framing <- mediation::framing 
levels(framing$educ) <- c("NA","Ref","< HS", "HS", "> HS","Coll +") 
framing.glm <- glm(cong_mesg ~ age + income + educ + emo + gender * factor(treat), 
    family = binomial, data = framing)
```
 
The conventional way to handle covariates like `emo` is to set them at their
means and use those means for purposes of predictions and EMMs. These adjusted 
means are shown in the following plot. 
```{r fig.alt = "Two panels, each showing a decreasing trend with educ except they increase again with educ=Coll+. In the male panel, the curve for treat 1 is higher than that for treat 2, while the reverse is true in the female panel"}
emmip(framing.glm, treat ~ educ | gender, type = "response") 
```


This plot gives the impression that the effect of `treat` is reversed between 
male and female subjects; and also that the effect of education is not monotone.
Both of these are counter-intuitive.

###### {#med.covred}
<!-- @index `cov.reduce` -->
However, note that the covariate `emo` is measured *post*-treatment. That 
suggests that in fact `treat` (and perhaps other factors) could affect the value
of `emo`; and if that is true (as is in fact established by mediation analysis 
techniques), we should not pretend that `emo` can be set independently of 
`treat` as was done to obtain the EMMs shown above. Instead, let `emo` depend on
`treat` and the other predictors -- easily done using `cov.reduce` -- and we 
obtain an entirely different impression:
```{r fig.alt = "Similar to previous figure but now the curves flatten out with >HS about the same as Coll+. For the maile panel, we still have the trace for treat 1 higher than for treat 0; but in the female panel, they are about the same as each other, and a bit lower than treat 0 for males. To see these results in numerical form, call emmeans() with the same arguments *except* replace the second argument with ~treat*educ|gender"}
emmip(framing.glm, treat ~ educ | gender, type = "response", 
    cov.reduce = emo ~ treat*gender + age + educ + income)
```

The reference grid underlying this plot has different `emo` values for each 
factor combination. The plot suggests that, after taking emotional response into
account, male (but not female) subjects exposed to the negative news story are 
more likely to send the message than are females or those not seeing the 
negative news story. Also, the effect of `educ` is now nearly monotone.

###### {#adjcov}
<!-- @index Covariates!Adjusted -->
By the way, the results in this plot are the same is what you would obtain by
refitting the model with an adjusted covariate
```{r eval = FALSE} 
emo.adj <- resid(lm(emo ~ treat*gender + age + educ + income, data = framing))
```
... and then using ordinary covariate-adjusted means at the means of `emo.adj`. 
This is a technique that is often recommended.

If there is more than one mediating covariate, their settings may be defined in
sequence; for example, if `x1`, `x2`, and `x3` are all mediating covariates, we
might use
```{r eval = FALSE} 
emmeans(..., cov.reduce = list(x1 ~ trt, x2 ~ trt + x1, x3 ~ trt + x1 + x2))
```
 (or possibly with some interactions included as well).

[Back to Contents](#contents)



## Mediating factors and weights {#weights}
<!-- @index Factors!Mediating; `weights`; `emmeans()`!`weights`
            Examples!`nutrition`!`weights` -->
A mediating covariate is one that is
in the causal path; likewise, it is possible to have a mediating *factor*. For
mediating factors, the moral equivalent of the `cov.reduce` technique described
above is to use *weighted* averages in lieu of equally-weighted ones in
computing EMMs. The weights used in these averages should depend on the
frequencies of mediating factor(s). Usually, the `"cells"` weighting scheme
described later in this section is the right approach. In complex situations, it
may be necessary to compute EMMs in stages.

As described in [the "basics" vignette](basics.html#emmeans), EMMs are usually 
defined as *equally-weighted* means of reference-grid predictions. However, 
there are several built-in alternative weighting schemes that are available by 
specifying a character value for `weights` in a call to `emmeans()` or related 
function. The options are `"equal"` (the default), `"proportional"`, `"outer"`,
`"cells"`, and `"flat"`.

The `"proportional"` (or `"prop"` for short) method weights proportionally to
the frequencies (or model weights) of each factor combination that is averaged
over. The `"outer"` method uses the outer product of the marginal frequencies of
each factor that is being averaged over. To explain the distinction, suppose the
EMMs for `A` involve averaging over two factors `B` and `C`. With `"prop"`, we
use the frequencies for each combination of `B` and `C`; whereas for `"outer"`,
first obtain the marginal frequencies for `B` and for `C` and weight
proportionally to the product of these for each combination of `B` and `C`. The
latter weights are like the "expected" counts used in a chi-square test for
independence. Put another way, outer weighting is the same as proportional
weighting applied one factor at a time; the following two would yield the same
results: 
```{r eval = FALSE} 
emmeans(model, "A", weights = "outer") 
emmeans(model, c("A", "B"), weights = "prop") |>  emmeans(weights = "prop") 
```


Using `"cells"` weights gives each prediction the same weight as occurs in the
model; applied to a reference grid for a model with all interactions,
`"cells"`-weighted EMMs are the same as the ordinary marginal means of the data.
With `"flat"` weights, equal weights are used, except zero weight is applied to
any factor combination having no data. Usually, `"cells"` or `"flat"` weighting
will *not* produce non-estimable results, because we exclude empty cells. (That
said, if covariates are linearly dependent with factors, we may still encounter
non-estimable cases.)

Here is a comparison of predictions for `nutr.lm` defined [above](#issues),
using different weighting schemes:
```{r message = FALSE}
sapply(c("equal", "prop", "outer", "cells", "flat"), \(w)
    emmeans(nutr.lm, ~ race, weights = w) |> predict())
```
In the other hand, if we do `group * race` EMMs, only one factor (`age`) is
averaged over; thus, the results for `"prop"` and `"outer"` weights will be
identical in that case.

[Back to Contents](#contents)


## Nuisance factors {#nuisance}
<!-- @index Nuisance factors; `nuisance`; `non.nuisance`; `wt.nuis`;
     Memory usage; Large models; Models!Large -->
Consider a situation where we have a model with 15 factors, each at 5 levels.
Regardless of how simple or complex the model is, the reference grid consists of
all combinations of these factors -- and there are $5^{15}$ of these, or over 30
billion. If there are, say, 100 regression coefficients in the model, then just
the `linfct` slot in the reference grid requires $100\times5^{15}\times8$ bytes of
storage, or almost 23,000 gigabytes. Suppose in addition the model has a
multivariate response with 5 levels. That multiplies *both* the rows and columns
in `linfct`, increasing the storage requirements by a factor of 25. Either way,
your computer can't store that much -- so this definitely qualifies as a messy
situation!

The `ref_grid()` function now provides some relief, in the way of specifying
some of the factors as "nuisance" factors. The reference grid is then
constructed with those factors already averaged-out. So, for example with the
same scenario, if only three of those 15 factors are of primary interest, and we
specify the other 12 as nuisance factors to be averaged, that leaves us with
only $3^5=125$ rows in the reference grid, and hence
$125\times100\times8=10,000$ bytes of storage required for `linfct`. If there is
a 5-level multivariate response, we'll have 625 rows in the reference grid and
$25\times1000=250,000$ bytes in `linfct`. Suddenly a horribly unmanageable
situation becomes quite manageable!

But of course, there is a restriction: nuisance factors must not interact with
any other factors -- not even other nuisance factors. And a multivariate
response (or an implied multivariate response, e.g., in an ordinal model) can
never be a nuisance factor. Under that condition, the average effects of a
nuisance factor are the same regardless of the levels of other factors, making
it possible to pre-average them by considering just one case. 

We specify nuisance factors by listing their names in a `nuisance` argument to 
`ref_grid()` (in `emmeans()`, this argument is passed to `ref_grid)`). Often,
it is much more convenient to give the factors that are *not* nuisance factors, 
via a `non.nuisance` argument. If you do specify
a nuisance factor that does interact with others, or doesn't exist, it is 
quietly excluded from the nuisance list. 

###### {#nuis.example}
<!-- @index Examples!`mtcars` -->
Time for an example. Consider the `mtcars` dataset standard in R, and the model
```{r}
mtcars.lm <- lm(mpg ~ factor(cyl)*am + disp + hp + drat + log(wt) + vs + 
                  factor(gear) + factor(carb), data = mtcars)
```
And let's construct two different reference grids:
```{r}
rg.usual <- ref_grid(mtcars.lm)
rg.usual
nrow(linfct(rg.usual))
rg.nuis = ref_grid(mtcars.lm, non.nuisance = "cyl")
rg.nuis
nrow(linfct(rg.nuis))
```
Notice that we left `am` out of `non.nuisance` and hence included it in `nuisance`.
However, it interacts with `cyl`, so it was not allowed as a nuisance factor.
But `rg.nuis` requires 1/36 as much storage.
There's really nothing else to show, other than to demonstrate that we get the same EMMs
either way, with slightly different annotations:
```{r}
emmeans(rg.usual, ~ cyl * am)
emmeans(rg.nuis, ~ cyl * am)
```

By default, the pre-averaging is done with equal weights. If we specify
`wt.nuis` as anything other than `"equal"`, they are averaged proportionally. As
described above, this really amounts to `"outer"` weights since they are
averaged separately. Let's try it to see how the estimates differ:
```{r}
predict(emmeans(mtcars.lm, ~ cyl * am, non.nuis = c("cyl", "am"), 
                wt.nuis = "prop"))
predict(emmeans(mtcars.lm, ~ cyl * am, weights = "outer"))
```
These are the same as each other, but different from the equally-weighted EMMs
we obtained before. By the way, to help make things consistent, if `weights` is
character, `emmeans()` passes `wt.nuis = weights` to `ref_grid` (if it is
called), unless `wt.nuis` is also specified.

There is a trick to get `emmeans` to use the smallest possible reference grid: 
Pass the `specs` argument to `ref_grid()` as `non.nuisance`. 
But we have to quote it
to delay evaluation, and also use `all.vars()` if (and only if)
`specs` is a formula:
```{r}
emmeans(mtcars.lm, ~ gear | am, non.nuis = quote(all.vars(specs)))
```
Observe that `cyl` was passed over as a nuisance factor because it interacts with
another factor.

### Limiting the size of the reference grid {#rg.limit}
<!-- @index  `rg.limit` option; Memory usage!Limiting -->

We have just seen how easily the size of a reference grid can get out of hand.
The `rg.limit` option (set via `emm_options()` or as an optional argument in `ref_grid()` or `emmeans()`) serves to guard against excessive memory demands. It specifies the number
of allowed rows in the reference grid. But because of the way `ref_grid()` works, this 
check is made *before* any multivariate-response levels are taken into account.
If the limit is exceeded, an error is thrown:
```{r, error = TRUE}
ref_grid(mtcars.lm, rg.limit = 200)
```
The default `rg.limit` is 10,000. With this limit, and if we have 1,000 columns 
in the model matrix, then the size of `linfct` is limited to about 80MB. 
If in addition, there is a 5-level multivariate response, the limit is 2GB --
darn big, but perhaps manageable. Even so, I suspect that the 10000-row default may be
to loose to guard against some users getting into a tight situation.


[Back to Contents](#contents)


## Counterfactuals and G-Computation {#counterfact}
<!-- @index Counterfactuals; Causal inference; G-Computation; -->
G-computation is a method for model-based causal inference originated by JM Robins
(*Mathematical Modelling*, 1986), and we want to remove confounding of treatment effects due to time-varying covariates and such.
The idea is that, under certain assumptions, we can use the model to predict 
*every* subject's response to each treatment -- not just the treatment they 
received. To do this, we make several copies of the whole dataset, substituting
the actual treatment(s) with each of the possible treatment levels; these
provide us with *counterfactual* predictions. We then average those predictions
over each copy of the dataset. Typically, this averaging is done on the response 
scale; that is the interesting case because on the link scale, everything is linear
and we can obtain basically the same results using ordinary `emmeans()` computations
with proportional weights.

An additional consideration is that when we average each of the counterfactual
datasets, we are trying to represent the entire covariate distribution, rather
than conditioning on the cases in the dataset. So it is a good idea to broaden
the covariance estimate using, say, a sandwich estimate.

This kind of computation has just a little bit in common with nuisance variables,
in that the net result is that we can sweep several predictors out of the 
reference grid just by averaging them away. For this to make sense, the predictors
averaged-away will have been observed *before* treatment so that their effects are 
separate from the treatment effects.

The implementation of this in **emmeans** is via the `counterfactuals` argument
in `ref_grid()` (but usually passed from `emmeans()`). We simply specify the
factor(s) we want to keep. This creates an index variable `.obs.no.` to keep
track of the observations in the dataset, and then the reference grid (before
averaging) consists of every observation of the dataset in combination with the
`counterfactuals` combinations.

##### {#neuralgia}
<!-- @index `vcov.`; Sandwich estimators;
    Examples!`neuralgia`!`counterfactuals` -->
As an example, consider the `neuralgia` data, where we have a binary response,
pain, a treatment of interest (two active treatments and placebo), and
pre-treatment predictors of sex, age, and duration of the condition. 
We will include the `vcovHC()` covariance estimate in the **sandwich** package.
```{r}
neuralgia.glm <- glm(Pain ~ Sex + Age + Duration + Treatment,
                     data = neuralgia, family = binomial)
emmeans(neuralgia.glm, "Treatment", counterfactuals = "Treatment",
        vcov. = sandwich::vcovHC)
```
Note that the results are already on the response (probability) scale,
which is the default. Let's compare this with what we get without using counterfactuals (i.e., predicting at each covariate average):
```{r}
emmeans(neuralgia.glm, "Treatment", weights = "prop", type = "response")
```
These results are markedly different; the counterfactual method produces
smaller differences between each of the active treatments and placebo.

[Back to Contents](#contents)


## Sub-models {#submodels}
<!-- @index `submodel`; Models!Constrained 
             Examples!`nutrition`!`submodel` -->
We have just seen that we can assign different weights to the levels of containing
factors. Another option is to constrain the effects of those containing factors
to zero. In essence, that means fitting a different model without those containing
effects; however, for certain models (not all), an `emmGrid` may be updated
with a `submodel` specification so as to impose such a constraint. For illustration,
return again to the nutrition example, and consider the analysis of `group`
and `race` as before, after removing interactions involving `age`:
```{r}
emmeans(nutr.lm, pairwise ~ group | race, submodel = ~ age + group*race) |> 
        summary(by = NULL)
```
If you like, you may confirm that we would obtain exactly the same
estimates if we had fitted that sub-model to the data, except we continue
to use the residual variance from the full model in tests and confidence intervals.
Without the interactions with `age`, all of the marginal means become estimable.
The results are somewhat different from those obtained earlier where
we narrowed the scope to just age 3. These new estimates include all ages,
averaging over them equally, but with constraints that the interaction effects
involving `age` are all zero.

###### {#type2submodel}
<!-- @index `submodel`!`"minimal"` and `"type2"`; Type II analysis
     `joint_tests()`!with `submodel = "type2"`  -->
There are two special character values that may be used with `submodel`.
Specifying `"minimal"` creates a submodel with only the active factors:
```{r}
emmeans(nutr.lm, ~ group * race, submodel = "minimal")
```
This submodel constrains all effects involving `age` to be zero.
Another interesting option is `"type2"`, whereby we in essence analyze the residuals
of the model with all contained or overlapping effects, then constrain the
containing effects to be zero. So what is left if only the interaction
effects of the factors involved. This is most useful with `joint_tests()`:
```{r}
joint_tests(nutr.lm, submodel = "type2")
```
These results are identical to the type II anova obtained 
[at the beginning of this example](#nutrex).

More details on how `submodel` works may be found in 
[`vignette("xplanations")`](xplanations.html#submodels)



[Back to Contents](#contents)


## Nested fixed effects {#nesting}
<!-- @index Nesting -->
A factor `A` is nested in another factor `B` if the levels of `A` have a
different meaning in one level of `B` than in another. Often, nested factors are
random effects---for example, subjects in an experiment may be randomly assigned
to treatments, in which case subjects are nested in treatments---and if we model
them as random effects, these random nested effects are not among the fixed
effects and are not an issue to `emmeans`. But sometimes we have fixed nested
factors.

###### {#cows}
<!-- @index Examples!`cows`; Examples!Nested fixed effects -->
Here is an example of a fictional study of five fictional treatments for some
disease in cows. Two of the treatments are administered by injection, and the
other three are administered orally. There are varying numbers of observations
for each drug. The data and model follow:
```{r}
cows <- data.frame (
    route = factor(rep(c("injection", "oral"), c(5, 9))),
    drug = factor(rep(c("Bovineumab", "Charloisazepam", 
              "Angustatin", "Herefordmycin", "Mollycoddle"), c(3,2,  4,2,3))),
    resp = c(34, 35, 34,   44, 43,      36, 33, 36, 32,   26, 25,   25, 24, 24)
)
cows.lm <- lm(resp ~ route + drug, data = cows)
```
The `ref_grid` function finds a nested structure in this model:
```{r message = FALSE}
cows.rg <- ref_grid(cows.lm)
cows.rg
```

When there is nesting, `emmeans` computes averages separately in each group\ldots
```{r}
route.emm <- emmeans(cows.rg, "route")
route.emm
```
... and insists on carrying along any grouping factors that a factor is nested in:
```{r}
drug.emm <- emmeans(cows.rg, "drug")
drug.emm
```
Here are the associated pairwise comparisons:
```{r}
pairs(route.emm, reverse = TRUE)

pairs(drug.emm, by = "route", reverse = TRUE)
```
In the latter result, the contrast itself becomes a nested factor in the
returned `emmGrid` object. That would not be the case if there had been no `by`
variable.

#### Graphs with nesting
<!-- @index `emmip()`!nested factors; `plot()`!nested factors; `emmip()`!abbreviated labels; **ggplot2** package -->
It can be very helpful to take advantage of special features of **ggplot2** when
graphing results with nested factors. For example, the default plot for the `cows` example
is not ideal:
```{r, fig.width = 5.5, fig.alt = "A panel for each route. This interaction plot has a lot of empty space because all 5 drugs are represented in each panel, and the x axis labels all overlap"}
emmip(cows.rg, ~ drug | route)
```

We can instead remove `route` from the call and instead handle it with **ggplot2**
code to use separate *x* scales; and also abbreviate the drug levels so there is less clutter:
```{r, fig.width = 5.5, fig.alt = "This plot shows the same means as the previous one, but each panel shows only the drugs that are nested in each route"}
require(ggplot2)
emmip(cows.rg, ~ drug, abbr.len = 6) + 
  facet_wrap(~ route, scales = "free_x", space = "free_x")
```

The `space` argument causes the panels to be sized according to the number of means plotted.

Similarly with `plot.emmGrid()`:
```{r, fig.height = 2.5, fig.width = 5.5, fig.alt = "side-by-side CIs and PIs for drugs in each route. Again, with free_y scaling, each panel contains only the drugs involved"}
plot(drug.emm, by = "route", PIs = TRUE) + 
    facet_wrap(~ route, scales = "free_y", space = "free_y")
```


### Auto-identification of nested factors -- avoid being trapped! {#nest-trap}
<!-- @index Nesting!Auto-detection; `ref_grid()`!`nesting` -->
`ref_grid()` and `emmeans()` tries to discover
and accommodate nested structures in the fixed effects. It does this in two
ways: first, by identifying factors whose levels appear in combination with only
one level of another factor; and second, by examining the `terms` attribute of
the fixed effects. In the latter approach, if an interaction `A:B` appears
in the model but `A` is not present as a main effect, then `A` is deemed to
be nested in `B`. Note that this can create a trap: some users take shortcuts
by omitting some fixed effects, knowing that this won't affect the fitted
values. But such shortcuts *do* affect the interpretation of model parameters,
ANOVA tables, etc., and I advise against ever taking such shortcuts. 
Here are some ways you may notice mistakenly-identified nesting:

  * A message is displayed when nesting is detected
  * A `str()` listing of the `emmGrid` object shows a nesting component
  * An `emmeans()` summary unexpectedly includes one or more factors
    that you didn't specify
  * EMMs obtained using `by` factors don't seem to behave right, or
    give the same results with different specifications

To override the auto-detection of nested effects, use the `nesting` argument
in `ref_grid()` or `emmeans()`. Specifying `nesting = NULL` will ignore
all nesting. Incorrectly-discovered nesting can be overcome by specifying 
something akin to `nesting = "A %in% B, C %in% (A * B)"` or, equivalently,
`nesting = list(A = "B",  C = c("A", "B"))`.


[Back to Contents](#contents)

[Index of all vignette topics](vignette-topics.html)

---
title: "Utilities and options for emmeans"
author: "emmeans package, Version `r packageVersion('emmeans')`"
output: emmeans::.emm_vignette
vignette: >
  %\VignetteIndexEntry{Utilities and options}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
```{r, echo = FALSE, results = "hide", message = FALSE} 
require("emmeans")
emm_options(opt.digits = TRUE)
knitr::opts_chunk$set(fig.width = 4.5, class.output = "ro") 
```

<!-- @index Vignettes!Utilities and options -->

## Contents {#contents}

  1. [Updating an `emmGrid` object](#update)
  2. [Setting options](#options)
      a. [Setting and viewing defaults](#defaults)
      b. [Optimal digits to display](#digits)
      c. [Startup options](#startup)
  3. [Combining and subsetting `emmGrid` objects](#rbind)
  4. [Accessing results to use elsewhere](#data)
  5. [Adding grouping factors](#groups)
  6. [Re-labeling and re-leveling an `emmGrid`](#relevel)
  
[Index of all vignette topics](vignette-topics.html)

## Updating an `emmGrid` object {#update}
<!-- @index `update()`; `emmGrid` objects!Modifying -->
Several internal settings are saved when functions like `ref_grid()`, `emmeans()`,
`contrast()`, etc. are run. Those settings can be manipulated via the `update()`
method for `emmGrid`s. To illustrate, consider the `pigs` dataset and model yet again:
```{r}
pigs.lm <- lm(log(conc) ~ source + factor(percent), data = pigs)
pigs.emm <- emmeans(pigs.lm, "source")
pigs.emm
```
We see confidence intervals but not tests, by default. This happens as a result
of internal settings in `pigs.emm.s` that are passed to `summary()` when the
object is displayed. If we are going to work with this object a lot, we might
want to change its internal settings rather than having to rely on explicitly
calling `summary()` with several arguments. If so, just update the internal
settings to what is desired; for example:
```{r}
pigs.emm.s <- update(pigs.emm, infer = c(TRUE, TRUE), null = log(35),
                     calc = c(n = ".wgt."))
pigs.emm.s
```
Note that by adding of `calc`, we have set a default to calculate and
display the sample size when the object is summarized.
See `help("update.emmGrid")` for details on the keywords
that can be changed. Mostly, they are the same as the names of arguments
in the functions that construct these objects.

Of course, we can always get what we want via calls to `test()`, `confint()` or
`summary()` with appropriate arguments. But the `update()` function is more
useful in sophisticated manipulations of objects, or called implicitly via the
`...` or `options` argument in `emmeans()` and other functions. Those options are passed
to `update()` just before the object is returned. For example, we could have
done the above update within the `emmeans()` call as follows (results are not shown because they are the same as before):
```{r eval = FALSE}
emmeans(pigs.lm, "source", infer = c(TRUE, TRUE), null = log(35),
        calc = c(n = ".wgt."))
```

[Back to contents](#contents)

## Setting options {#options}
<!-- @index `get_emm_option()`; Options -->
Speaking of the `options` argument, note that the default in `emmeans()` 
is `options = get_emm_option("emmeans")`. Let's see what that is:
```{r}
get_emm_option("emmeans")
```
So, by default, confidence intervals, but not tests, are displayed
when the result is summarized. The reverse is true for results of 
`contrast()` (and also the default for `pairs()` which calls `contrast()`):
```{r}
get_emm_option("contrast")
```
There are also defaults for a newly constructed reference grid:
```{r}
get_emm_option("ref_grid")
```
The default is to display neither intervals nor tests when summarizing.
In addition, the flag `is.new.rg` is set to `TRUE`, and that is why one 
sees a `str()` listing rather than a summary as the default when the object
is simply shown by typing its name at the console.

### Setting and viewing defaults {#defaults}
<!-- @index `emm_options()`; `emmGrid` objects!Setting defaults for
     `emmeans()`!Changing defaults; `contrast()`!Changing defaults -->
The user may have other preferences. She may want to see both intervals 
and tests whenever contrasts are produced; and perhaps she also wants to
always default to the response scale when transformations or links
are present. We can change the defaults by setting the corresponding options;
and that is done via the `emm_options()` function:
```{r}
emm_options(emmeans = list(type = "response"),
            contrast = list(infer = c(TRUE, TRUE)))
```
Now, new `emmeans()` results and contrasts follow the new defaults:
```{r}
pigs.anal.p <- emmeans(pigs.lm, consec ~ percent)
pigs.anal.p
```
Observe that the contrasts "inherited" the `type = "response"` default from
the EMMs.

NOTE: Setting the above options does *not* change how existing `emmGrid` objects
are displayed; it only affects ones constructed in the future.

There is one more option -- `summary` -- that overrides all other display 
defaults for both existing and future objects. For example, specifying 
`emm_options(summary = list(infer = c(TRUE, TRUE)))` will result in both
intervals and tests being displayed, regardless of their internal defaults,
unless `infer` is explicitly specified in a call to `summary()`.

To temporarily revert to factory defaults in a single call to `emmeans()` or
`contrast()` or `pairs()`, specify `options = NULL` in the call. To reset
everything to factory defaults (which we do presently), null-out all of the
**emmeans** package options:
```{r}
options(emmeans = NULL)
```

### Optimal digits to display {#digits}
<!-- @index Digits, optimizing; `opt.digits` option -->
When an `emmGrid` object is summarized and displayed, the factory default is to
display it with just enough digits as is justified by the standard errors or HPD
intervals of the estimates displayed. You may use the `"opt.digits"` option to
change this. If it is `TRUE` (the default), we display only enough digits as is
justified (but at least 3). If it is set to `FALSE`, the number of digits is set
using the R system's default, `getOption("digits")`; this is often much more
precision than is justified. To illustrate, here is the summary of `pigs.emm`
displayed without optimizing digits. Compare it with the first summary in this 
vignette.
```{r} 
emm_options(opt.digits = FALSE)
pigs.emm
emm_options(opt.digits = TRUE)  # revert to optimal digits
``` 
By the way, setting this option does
*not* round the calculated values computed by `summary.emmGrid()` or saved in a
`summary)emm` object; it simply controls the precision displayed by
`print.summary_emm()`.

### Startup options {#startup}
<!-- @index Startup options; Options!Startup -->
The options accessed by `emm_options()` and `get_emm_option()` are stored in a
list named `emmeans` within R's options environment. Therefore, if you desire
options other than the defaults provided on a regular basis, this can be
easily arranged by specifying them in your startup script for R. For example,
if you want to default to Satterthwaite degrees of freedom for `lmer` models,
and display confidence intervals rather than tests for contrasts,
your `.Rprofile` file could contain the line
```{r eval = FALSE}
options(emmeans = list(lmer.df = "satterthwaite", 
                       contrast = list(infer = c(TRUE, FALSE))))
```

[Back to contents](#contents)

## Combining and subsetting `emmGrid` objects {#rbind}
<!-- @index `emmGrid` objects!Combining and subsetting
     `rbind()`; `+` operator@plus -->
Two or more `emmGrid` objects may be combined using the `rbind()` or `+`
methods. The most common reason (or perhaps the only good reason) to do this
is to combine EMMs or contrasts into one family for purposes of applying
a multiplicity adjustment to tests or intervals. 
A user may want to combine the three pairwise comparisons of sources 
with the three comparisons above of consecutive percents into a single family of six tests with a suitable 
multiplicity adjustment. This is done quite simply:
```{r}
rbind(pairs(pigs.emm.s), pigs.anal.p[[2]])
```
The default adjustment is `"bonferroni"`; we could have specified something different via the `adjust` argument. An equivalent way to combine `emmGrid`s is via the addition
operator. Any options may be provided by `update()`. Below, we combine the same
results into a family but ask for the "exact" multiplicity adjustment.
```{r}
update(pigs.anal.p[[2]] + pairs(pigs.emm.s), adjust = "mvt")
```
Also evident in comparing these results is that settings are obtained from the
first object combined. So in the second output, where they are combined in
reverse order, we get both confidence intervals and tests, and transformation to
the response scale.

###### {#brackets}
<!-- @index Selecting results; Brackets (`[ ]` and `[[ ]]` operators) -->
To subset an `emmGrid` object, just use the subscripting operator `[]`.
For instance,
```{r}
pigs.emm[2:3]
```

## Accessing results to use elsewhere {#data}
<!-- @index `emmGrid` objects!Accessing data; Using results 
     `summary_emm` object!As a data frame; Digits!Displaying more digits;
     Precision!Displaying results with more digits; -->
Sometimes, users want to use the results of an analysis (say, an `emmeans()` call)
in other computations. The `summary()` method creates a `summary_emm` object
that inherits from the `data.frame` class; so one may use the variables therein
just as those in a data frame.

An `emmGrid` object has its own internal structure and we can't directly access
the values we see displayed. If follow-up computations are needed, use
`summary()` (or `confint()` or `test()`), creates a `summary_emm` object which
inherits from `data.frame` -- making it possible to access the values.
For illustration, let's add the widths of the confidence intervals in our example.
```{r}
CIs <- confint(pigs.emm)
CIs$CI.width <- with(CIs, upper.CL - lower.CL)
CIs
```
By the way, the values stored internally are kept to full precision, more than is
typically displayed:
```{r}
CIs$emmean
```
If you want to display more digits, specify so using the `print` method:
```{r}
print(CIs, digits = 5)
```


[Back to contents](#contents)



## Adding grouping factors {#groups}
<!-- @index Grouping factors; `add_grouping()`; Nesting factors!Creating  -->
Sometimes, users want to group levels of a factor into a smaller number of groups.
Those groups may then be, say, averaged separately and compared, or used as a
`by` factor. The `add_grouping()` function serves this purpose. The function
takes four arguments: the object, the name of the grouping factor to be created,
the name of the reference factor that is being grouped, and a vector of level 
names of the grouping factor corresponding to levels of the reference factor.
Suppose for example that we want to distinguish animal and non-animal sources of
protein in the `pigs` example:
```{r}
pigs.emm.ss <- add_grouping(pigs.emm.s, "type", "source",
                            c("animal", "vegetable", "animal"))
str(pigs.emm.ss)
```
Note that the new object has a nesting structure (see more about this in the ["messy-data" vignette](messy-data.html#nesting)), with the reference factor nested in the new grouping factor. Now we can obtain means and comparisons for each group
```{r}
emmeans(pigs.emm.ss, pairwise ~ type)
```

[Back to contents](#contents)



## Re-labeling or re-leveling an `emmGrid` {#relevel}
<!-- @index Re-labeling; Levels!Changing; Labels!Changing; Examples!`warpbreaks`;
    Examples!Welch's *t* comparisons; Welch's *t* comparisons!Example    -->
Sometimes it is desirable to re-label the rows of an `emmGrid`, or cast it in terms of
other factor(s). This can be done via the `levels` argument in `update()`. 

As an example, sometimes a fitted model has a treatment factor that comprises combinations of other factors. In subsequent analysis, we may well want to break it down into the
individual factors' contributions. Consider, for example, the `warpbreaks` data provided with R.
We will define a single factor and fit a non homogeneous-variance model:
```{r, message = FALSE}
warp <- transform(warpbreaks, treat = interaction(wool, tension))
library(nlme)
warp.gls <- gls(breaks ~ treat, weights = varIdent(form = ~ 1|treat), data = warp)
( warp.emm <- emmeans(warp.gls, "treat") )
```
But now we want to re-cast this `emmGrid` into one that has separate factors for `wool` 
and `tension`. We can do this as follows:
```{r}
warp.fac <- update(warp.emm, levels = list(
                wool = c("A", "B"), tension = c("L", "M", "H")))
str(warp.fac)
```
So now we can do various contrasts involving the separate factors:
```{r}
contrast(warp.fac, "consec", by = "wool")
```

Note: When re-leveling to more than one factor, you have to be careful to anticipate
that the levels will be expanded using `expand.grid()`: the first factor in the list
varies the fastest and the last varies the slowest. That was the case in our example, 
but in others, it may not be. Had the levels of `treat` been
ordered as `A.L, A.M, A.H, B.L, B.M, B.H`, then we would have had to specify the levels
of `tension` first and the levels of `wool` second.


[Back to contents](#contents)


[Index of all vignette topics](vignette-topics.html)

---
title: "Confidence intervals and tests in emmeans"
author: "emmeans package, Version `r packageVersion('emmeans')`"
output: emmeans::.emm_vignette
vignette: >
  %\VignetteIndexEntry{Confidence intervals and tests}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
```{r, echo = FALSE, results = "hide", message = FALSE}
require("emmeans")
knitr::opts_chunk$set(fig.width = 4.5, class.output = "ro")
```

<!-- @index Vignettes!Confidence intervals and tests -->

## Contents {#contents}
This vignette describes various ways of summarizing `emmGrid` objects.

  1. [`summary()`, `confint()`, and `test()`](#summary)
  2. [Back-transforming to response scale](#tran)
     (See also the ["transformations" vignette](transformations.html))
  3. [Multiplicity adjustments](#adjust)
  4. [Using "by" variables](#byvars)
  5. [Joint (omnibus) tests](#joint)
  6. [Testing equivalence, noninferiority, nonsuperiority](#equiv)
  7. Graphics (in ["basics" vignette](basics.html#plots))
  
[Index of all vignette topics](vignette-topics.html)

## `summary()`, `confint()`, and `test()` {#summary}
<!-- @index `summary()`; `summary()`!`infer`; `confint()`; `test()`; 
    Confidence intervals; `summary()`!Calculated columns; 
    `summary()`!Show sample size; Sample size, displaying;
            Tests!One- and two-sided; Tests!Nonzero null;
            `summary()`!Bayesian models; `summary()`!`hpd.summary()`;
            Examples!`pigs` -->
The most important method for `emmGrid` objects is `summary()`. For one thing, it is
called by default when you display an `emmeans()` result.
The `summary()` function has a lot of options, and the detailed documentation via 
`help("summary.emmGrid")` is worth a look.

For ongoing illustrations, let's re-create some of the objects in the ["basics" vignette](basics.html) for the `pigs` example:
```{r}
mod4 <- lm(inverse(conc) ~ source + factor(percent), data = pigs)
RG <- ref_grid(mod4)
EMM.source <- emmeans(RG, "source")
```

Just `summary(<object>)` by itself will produce a summary that varies somewhat
according to context. It does this by setting different defaults for the `infer`
argument, which consists of two logical values, specifying confidence intervals 
and tests, respectively. [The exception is models fitted using MCMC methods,
where `summary()` is diverted to the `hpd.summary()` function, a 
preferable summary for many Bayesians.]

The summary of a newly made reference grid will show just estimates and
standard errors, but not confidence intervals or tests (that is, `infer =
c(FALSE, FALSE)`). The summary of an `emmeans()` result, as we see above, will
have intervals, but no tests (i.e., `infer = c(TRUE, FALSE)`); and the result of
a `contrast()` call (see [comparisons and contrasts](comparisons.html)) will
show test statistics and *P* values, but not intervals (i.e., `infer = c(FALSE,
TRUE)`). There are courtesy methods `confint()` and `test()` that just call
`summary()` with the appropriate `infer` setting; for example,
```{r}
test(EMM.source)
```
It is not particularly useful, though, to test these EMMs against the default of 
zero -- which is why tests are not usually shown. It makes a lot more sense
to test them against some target concentration, say 40. And suppose we want to do a one-sided test to see if the concentration is greater than 40. Remembering that the
response is inverse-transformed in this model, and that the inverse transformation
reverses the direction of comparisons, so that a *right*-tailed test on the `conc` scale
becomes a *left*-tailed test on the `inverse(conc)` scale,
```{r}
test(EMM.source, null = inverse(40), side = "<")
```
It is also possible to add calculated columns to the summary, via the `calc` 
argument. The calculations can include any columns up through `df` in the summary,
as well as any variable in the object's `grid` slot. Among the latter are 
usually weights in a column named `.wgt.`, and we can use that to include
sample size in the summary:
```{r}
confint(EMM.source, calc = c(n = ~.wgt.))
```


[Back to Contents](#contents)


## Back-transforming {#tran}
<!-- @index Transformations!Back-transforming; `type`; Response scale -->
Transformations and link functions are supported in several ways in **emmeans**,
making this a complex topic worthy of [its own vignette](transformations.html).
Here, we show just the most basic approach. Namely, specifying the argument
`type = "response"` will cause the displayed results to be back-transformed
to the response scale, when a transformation or link function is incorporated
in the model. For example, let's try the preceding `test()` call again:
```{r}
test(EMM.source, null = inverse(40), side = "<", type = "response")
```
Note what changes and what doesn't change. In the `test()` call, we *still* use
the 1/40 as the null value; `null` must always be specified on the
linear-prediction scale, in this case the inverse. In the output, the displayed
estimates, as well as the `null` value, are shown back-transformed. As well, the
standard errors are altered (using the delta method). However, the *t* ratios
and *P* values are identical to the preceding results. That is, the tests
themselves are still conducted on the linear-predictor scale (as is noted in the
output).

Similar statements apply to confidence intervals on the response scale:
```{r}
confint(EMM.source, side = "<", level = .90, type = "response")
```
With `side = "<"`, an *upper* confidence limit is computed on the inverse scale,
then that limit is back-transformed to the response scale; and since `inverse`
reverses everything, those upper confidence limits become lower ones on the
response scale.
(We have also illustrated 
how to change the confidence level.)

[Back to Contents](#contents)


## Multiplicity adjustments {#adjust}
<!-- @index Multiplicity adjustments; `adjust`; 
     Tukey adjustment; Sidak adjustment -->
Both tests and confidence intervals may be adjusted for simultaneous inference. 
Such adjustments ensure that the confidence coefficient for a whole set of 
intervals is at least the specified level, or to control for multiplicity
in a whole family of tests. This is done via the `adjust`
argument. For `ref_grid()` and `emmeans()` results, the default is `adjust =
"none"`. For most `contrast()` results, `adjust` is often something else,
depending on what type of contrasts are created. For example, pairwise
comparisons default to `adjust = "tukey"`, i.e., the Tukey HSD method.
The `summary()` function sometimes *changes* `adjust` if it is inappropriate.
For example, with
```{r}
confint(EMM.source, adjust = "tukey")
```
the adjustment is changed to the Sidak method because the Tukey adjustment is 
inappropriate unless you are doing pairwise comparisons.

####### {#adjmore}
<!-- @index **mvtnorm** package; Bonferroni adjustment
     Multivariate *t* (`"mvt"`) adjustment; Unadjusted tests -->
An adjustment method that is usually appropriate is Bonferroni; however, it can
be quite conservative. Using `adjust = "mvt"` is the closest to being the
"exact" all-around method "single-step" method, as it uses the multivariate *t*
distribution (and the **mvtnorm** package) with the same covariance structure as
the estimates to determine the adjustment. However, this comes at high
computational expense as the computations are done using simulation techniques.
For a large set of tests (and especially confidence intervals), the
computational lag becomes noticeable if not intolerable.

For tests, `adjust` increases the *P* values over those otherwise obtained with
`adjust = "none"`. Compare the following adjusted tests with the unadjusted ones
previously computed.
```{r}
test(EMM.source, null = inverse(40), side = "<", adjust = "bonferroni")
```

[Back to Contents](#contents)


## "By" variables {#byvars}
<!-- @index `by` groups; Grouping into separate sets -->
Sometimes you want to break a summary down into smaller pieces; for this
purpose, the `by` argument in `summary()` is useful. For example,
```{r}
confint(RG, by = "source")
```
If there is also an `adjust` in force when `by` variables are used, by default, the 
adjustment is made *separately* on each `by` group; e.g., in the above, we would
be adjusting for sets of 4 intervals, not all 12 together (but see "cross-adjustments" below.)

There can be a `by` specification in `emmeans()` (or equivalently, a `|` in the
formula); and if so, it is passed on to `summary()` and used unless overridden
by another `by`. Here are examples, not run:
```{r eval = FALSE}
emmeans(mod4, ~ percent | source)     ### same results as above
summary(.Last.value, by = "percent")       ### grouped the other way
```
Specifying `by = NULL` will remove all grouping.


### Adjustments across `by` groups {#cross-adjust}
<!-- @index cross-group comparisons; `cross.adjust`; `adjust`!vs. `cross.adjust`
     `by` groups!Adjusting across groups; Examples!`warpbreaks` -->
As was mentioned, each `by` group is regarded as a separate family with regards to
the `adjust` procedure. For example, consider a model with interaction for the `warpbreaks` data,
and construct pairwise comparisons of `tension` by `wool`:
```{r}
warp.lm <- lm(breaks ~ wool * tension, data = warpbreaks)
warp.pw <- pairs(emmeans(warp.lm, ~ tension | wool))
warp.pw
```
We have two sets of 3 comparisons, and the (default) Tukey adjustment is made *separately* in each group.

However, sometimes we want the multiplicity adjustment to be broader.
This broadening can be done in two ways. One is to remove the `by` variable,
which then treats all results as one family. In our example:
```{r}
test(warp.pw, by = NULL, adjust = "bonferroni")
```
This accomplishes the goal of putting all the comparisons in one family of 6
comparisons. Note that the Tukey adjustment may not be used here because we no
longer have *one* set of pairwise comparisons.

An alternative is to specify `cross.adjust`, which specifies an additional adjustment method
to apply to corresponding sets of within-group adjusted *P* values:
```{r}
test(warp.pw, adjust = "tukey", cross.adjust = "bonferroni")
```
These adjustments are less conservative than the previous result, but it is still
a conservative adjustment to the set of 6 tests. Had we also specified `adjust = "bonferroni"`,
we would have obtained the same adjusted *P* values as we obtained with `by = NULL`.

### Simple comparisons {#simple}
<!-- @index Simple comparisons; `contrast()`!`simple`; `simple = "each"` -->
There is also a `simple` argument for `contrast()` that is in essence 
the inverse of `by`; the contrasts are run using everything *except* the
specified variables as `by` variables. To illustrate, let's consider 
the model for `pigs` that includes the interaction (so that the levels
of one factor compare differently at levels of the other factor).
```{r}
mod5 <- lm(inverse(conc) ~ source * factor(percent), data = pigs)
RG5 <- ref_grid(mod5)
contrast(RG5, "consec", simple = "percent")
```
In fact, we may do *all* one-factor comparisons by specifying `simple = "each"`.
This typically produces a lot of output, so use it with care.

[Back to Contents](#contents)


## Joint tests {#joint}
<!-- @index `test()`!`joint = TRUE`; `joint`; Type III tests -->
From the above, we already know how to test individual results. For pairwise comparisons (details in [the "comparisons" vignette](comparisons.html)), we might do
```{r}
PRS.source <- pairs(EMM.source)
PRS.source
```
But suppose we want an *omnibus* test that all these comparisons are zero.
Easy enough, using the `joint` argument in `test` (note: the `joint` argument
is *not* available in `summary()`; only in `test()`):
```{r}
test(PRS.source, joint = TRUE)
```
Notice that there are three comparisons, but only 2 d.f. for the test,
as cautioned in the message.

The test produced with `joint = TRUE` is a "type III" test (assuming the default
equal weights are used to obtain the EMMs). See more on these types of tests for
higher-order effects in the ["interactions" vignette section on
contrasts](interactions.html#contrasts).

####### {#joint_tests}
<!-- @index `joint_tests()`; Type III tests; Analysis of variance!Type III -->
For convenience, there is also a `joint_tests()` function that performs
joint tests of contrasts among each term in a model or `emmGrid` object.
```{r}
joint_tests(RG5)
```
The tests of main effects are of families of contrasts; those for interaction
effects are for interaction contrasts. These results are essentially the same as
a "Type-III ANOVA", but may differ in situations where there are empty cells or
other non-estimability issues, or if generalizations are present such as unequal
weighting. (Another distinction is that sums of squares and mean squares are not
shown; that is because these really are tests of contrasts among predictions,
and they may or may not correspond to model sums of squares.) 

One may use `by` variables with `joint_tests`. For example:
```{r}
joint_tests(RG5, by = "source")
```
In some models, it is possible to specify `submodel = "type2"`, thereby obtaining 
something akin to a Type II analysis of variance. See the [messy-data vignette](messy-data.html#type2submodel) for an example.


[Back to Contents](#contents)


## Testing equivalence, noninferiority, and nonsuperiority {#equiv}
<!-- @index Tests!Equivalence; Tests!Non-inferiority; `test()`!`delta` -->
The `delta` argument in `summary()` or `test()` allows the user to
specify a threshold value to use in a test of equivalence, non-inferiority, 
or non-superiority. An equivalence test is kind of a backwards significance 
test, where small *P* values are associated with small differences relative 
to a specified threshold value `delta`.
The help page for `summary.emmGrid` gives the details of 
these tests.
Suppose in the present example, we consider two sources to be equivalent if they 
are within 0.005 of each other. We can test this as follows:
```{r}
test(PRS.source, delta = 0.005, adjust = "none")
```
Using the 0.005 threshold, the *P* value is quite small for comparing soy and skim, 
providing some statistical evidence that their difference is enough smaller than the
threshold to consider them equivalent.

[Back to Contents](#contents)


## Graphics {#graphics}
Graphical displays of `emmGrid` objects are described in the 
["basics" vignette](basics.html#plots)

[Index of all vignette topics](vignette-topics.html)

---
title: "Models supported by emmeans"
author: "emmeans package, Version `r packageVersion('emmeans')`"
output: emmeans::.emm_vignette
vignette: >
  %\VignetteIndexEntry{Models supported by emmeans}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

Here we document what model objects may be used with **emmeans**, and some
special features of some of them that may be accessed by passing additional
arguments through `ref_grid` or `emmeans()`.

Certain objects are affected by optional arguments to functions that
construct `emmGrid` objects, including `ref_grid()`,
`emmeans()`, `emtrends()`, and `emmip()`. When
"*arguments*" are mentioned in the subsequent quick reference and
object-by-object documentation, we are talking about arguments in these
constructors.

Some options cause transformations and links to be resolved at the time the
reference grid is created, thus performing a kind of implied re-gridding
at the very start. Such options are marked in the table with a "twiddle" (e.g., `"prob"~`). 
For those options, no links or transformations are passed along. 

If a model type is not included here, users may be able to obtain
usable results via the `qdrg()` function; see its help page.
Package developers may support their models by writing appropriate `recover_data`
and `emm_basis` methods. See the package documentation for 
`extending-emmeans` and `vignette("xtending")` for details.

[Index of all vignette topics](vignette-topics.html)

## Quick reference for supported objects and options {#quickref}
<!-- @index Models!Quick reference -->

Here is an alphabetical list of model classes that are supported, and
the arguments that apply. Detailed documentation follows, with objects
grouped by the code in the "Group" column. 
Scroll down or follow the links to those groups for more information.

|Object.class |Package  |Group    |Arguments / notes (Suffix of `~` indicates re-gridding)       |
|:------------|:--------|:-------:|:------------------------------------------------------------|
|aov          |stats    |[A](#A)  |                                                             |
|aovList      |stats    |[V](#V)  |Best with balanced designs, orthogonal coding                |
|averaging    |MuMIn    |[I](#I)  |`formula`, `subset` (see details)                            |
|betareg      |betareg  |[B](#B)  |`mode = c("link", "precision", "phi.link",`                  |
|             |         |         |`  "variance"~, "quantile"~)`                                |
|brmsfit      |brms     |[P](#P)  |Supported in **brms** package                                |
|carbayes     |CARBayes |[S](#S)  |`data` is required                                           |
|clm          |ordinal  |[O](#O)  |`mode = c("latent"~, "linear.predictor", "cum.prob"~,`       |
|             |         |         |`  "exc.prob"~, "prob"~, "mean.class"~, "scale")`            |
|clmm         |ordinal  |[O](#O)  |Like `clm` but no `"scale"` mode                             |
|coxme        |coxme    |[G](#G)  |                                                             |
|coxph        |survival |[G](#G)  |                                                             |
|gam          |mgcv     |[G](#G)  |`freq = FALSE`, `unconditional = FALSE`,                     |
|             |         |         |`what = c("location", "scale", "shape", "rate", "prob.gt.0")`|
|gamm         |mgcv     |[G](#G)  |`call = object$gam$call`                                     |
|Gam          |gam      |[G](#G)  |`nboot = 800`                                                |
|gamlss       |gamlss   |[H](#H)  |`what = c("mu", "sigma", "nu", "tau")`                       |
|gee          |gee      |[E](#E)  |`vcov.method = c("naive", "robust")`                         |
|geeglm       |geepack  |[E](#E)  |`vcov.method = c("vbeta", "vbeta.naiv", "vbeta.j1s",`        |
|             |         |         |`"vbeta.fij", "robust", "naive")` or a matrix                |
|geese        |geepack  |[E](#E)  |Like `geeglm`                                                |
|glm          |stats    |[G](#G)  |                                                             |
|glm.nb       |MASS     |[G](#G)  |                                                             |
|glmerMod     |lme4     |[G](#G)  |                                                             |
|glmgee       |glmtoolbox |[E](#E)|`vcov.method = c("robust", "df-adjusted", "model",`          |
|             |           |       |    `"bias-corrected", "jackknife")`                         |
|glmmadmb     |glmmADMB |         |No longer supported                                          |
|glmmPQL      |MASS     |[G](#G)  |inherits `lm` support                                        |
|glmmTMB      |glmmTMB  |[P](#P)  |Supported in **glmmTMB** package                             |
|gls          |nlme     |[K](#K)  |`mode = c("auto", "df.error", "satterthwaite", "asymptotic")`|
|gnls         |nlme     |[A](#A)  |Supports `params` part. Requires `param = "<name>"`          |
|hurdle       |pscl     |[C](#C)  |`mode = c("response", "count", "zero", "prob0"),`            |
|             |         |         |`lin.pred = c(FALSE~, TRUE)`                                 |
|lavaan       |lavaan    |[P](#P)  |Supported by the **semTools** package (see `? lavaan2emmeans`) |
|lm           |stats    |[A](#A)  |Several other classes inherit from this and may be supported |
|lme          |nlme     |[K](#K)  |`sigmaAdjust = c(TRUE, FALSE),`                              |
|             |         |         |`mode = c("auto", containment", "satterthwaite", "asymptotic"),`|
|             |         |         |`extra.iter = 0`                                             |
|lmerMod      |lme4     |[L](#L)  |`lmer.df = c("kenward-roger", "satterthwaite", "asymptotic")`, |
|             |         |         |`pbkrtest.limit = 3000`,  `disable.pbkrtest = FALSE`.        |
|             |         |         |`emm_options(lmer.df =, pbkrtest.limit =, disable.pbkrtest =)` |
|logistf      |glmmTMB  |[P](#P)  |Supported in **logistf** package                             |
|lqm,lqmm     |lqmm     |[Q](#Q)  |`tau = "0.5"` (must match an entry in `object$tau`)          |
|             |         |         |Optional: `method`, `R`, `seed`, `startQR` (must be fully spelled-out) |
|manova       |stats    |[M](#M)  |`mult.name`, `mult.levs`                                     |
|maov         |stats    |[M](#M)  |`mult.name`, `mult.levs`                                     |
|mblogit      |mclogit  |[N](#N)  |`mode = c("prob"~, "latent")`                                |
|             |         |         |Always include response in specs for `emmeans()`             |
|mcmc         |mcmc     |[S](#S)  |May require `formula`, `data`                                |
|MCMCglmm     |MCMCglmm |[S](#S)  |(see also [M](#M#))  `mult.name`, `mult.levs`, `trait`,      |
|             |         |         |`mode = c("default", "multinomial")`; `data` is required     |
|mira         |mice     |[I](#I)  |Optional arguments per class of `$analyses` elements         |
|mixed        |afex     |[P](#P)  |Supported in **afex** package                                |
|mlm          |stats    |[M](#M)  |`mult.name`, `mult.levs`                                     |
|mmer         |sommer   |[G](#G)  |                                                             |
|mmrm         |mmrm     |[P](#P)  |Supported in the **mmrm** package                            |
|multinom     |nnet     |[N](#N)  |`mode = c("prob"~, "latent")`                                |
|             |         |         |Always include response in specs for `emmeans()`             |
|nauf         |nauf.*xxx* |[P](#P)  |Supported in **nauf** package                              |
|nlme         |nlme     |[A](#A)  |Supports fixed part. Requires `param = "<name>"`             |
|polr         |MASS     |[O](#O)  |`mode = c("latent"~, "linear.predictor", "cum.prob"~,`       |
|             |         |         |`"exc.prob"~, "prob"~, "mean.class"~)`                       |
|rlm          |MASS     |[A](#A)  |inherits `lm` support                                        |
|rms          |rms      |[O](#O)  |`mode = ("middle"~, "latent"~, "linear.predictor",`          |
|             |         |         |`"cum.prob"~, "exc.prob"~, "prob"~, "mean.class"~)`          |
|rq,rqs       |quantreg |[Q](#Q)  |`tau = object$tau`                                           |
|             |         |         |Creates a pseudo-factor `tau` with levels `tau`              |
|             |         |         |Optional: `se`, `R`, `bsmethod`, etc.                        |
|rlmerMod     |robustlmm|[P](#P)  |Supported in **robustlmm** package                           |
|rsm          |rsm      |[P](#P)  |Supported in **rsm** package                                 |
|stanreg      |rstanarm |[S](#S)  |Args for `stanreg_`*xxx* similar to those for *xxx*          |
|survreg      |survival |[A](#A)  |                                                             |
|svyglm       |survey   |[A](#A)  |                                                             |
|svyolr       |survey   |[O](#O)  |Piggybacks on `polr` support                                 |
|zeroinfl     |pscl     |[C](#C)  |`mode = c("response", "count", "zero", "prob0")`,            |
|             |         |         |`lin.pred = c(FALSE~, TRUE)`                                  |


## Group A -- "Standard" or minimally supported models {#A}

Models in this group, such as `lm`, do not have unusual features that
need special support; hence no extra arguments are needed.
Some may require `data` in the call.


## Group B -- Beta regression {#B}
<!-- @index Beta regression; `betareg` models -->

The additional `mode` argument for `betareg` objects has possible
values of `"response"`, `"link"`, `"precision"`,
`"phi.link"`, `"variance"`, and `"quantile"`, which have the
same meaning as the `type` argument in `predict.betareg` -- with
the addition that `"phi.link"` is like `"link"`, but for the
precision portion of the model. When `mode = "quantile"` is specified,
the additional argument `quantile` (a numeric scalar or vector)
specifies which quantile(s) to compute; the default is 0.5 (the median). Also
in `"quantile"` mode, an additional variable `quantile` is added to
the reference grid, and its levels are the values supplied.
Modes `"variance"` and `"quantile"` cause an implied re-grid.

[Back to quick reference](#quickref)

## Group C -- Count models {#C}
<!-- @index Count regression; `hurdle` models; `zeroinfl` models -->

Two optional arguments -- `mode` and `lin.pred` -- are provided.
The `mode` argument has possible values `"response"` (the default),
`"count"`, `"zero"`, or `"prob0"`. `lin.pred` is logical
and defaults to `FALSE`.

With `lin.pred = FALSE`, the results are comparable to those returned by
`predict(..., type = "response")`, `predict(..., type = "count")`,
`predict(..., type = "zero")`, or `predict(..., type = "prob")[,
1]`. See the documentation for `predict.hurdle` and
`predict.zeroinfl`. Note that specifying `lin.pred = FALSE` causes re-gridding to take place.

The option `lin.pred = TRUE` only applies to `mode = "count"` and
`mode = "zero"`. The results returned are on the linear-predictor scale,
with the same transformation as the link function in that part of the model.
The predictions for a reference grid with `mode = "count"`,
`lin.pred = TRUE`, and `type = "response"` will be the same as
those obtained with `lin.pred = FALSE` and `mode = "count"`;
however, any EMMs derived from these grids will be different, because the
averaging is done on the log-count scale and the actual count scale,
respectively -- thereby producing geometric means versus arithmetic means of
the predictions.

If the `vcov.` argument is used (see details in the documentation for `ref_grid`),
it must yield a matrix of the same size as would be obtained using
`vcov.hurdle` or `vcov.zeroinfl` with its
`model` argument set to `("full", "count", "zero")` in respective
correspondence with `mode` of `("mean", "count", "zero")`. If
`vcov.` is a function, it must support the `model` argument.

[Back to quick reference](#quickref)

## Group E -- GEE models {#E}
<!-- @index GEE models --> 

These models all have more than one covariance estimate available, and it may
be selected by supplying a string as the `vcov.method` argument. It is
partially matched with the available choices shown in the quick reference. In
`geese` and `geeglm`, the aliases `"robust"` (for
`"vbeta"`) and `"naive"` (for `"vbeta.naiv"` are also
accepted.

If a matrix or function is supplied as `vcov.method`, it is
interpreted as a `vcov.` specification as described for `...`
in the documentation for `ref_grid`.


## Group G -- Generalized linear models and relatives {#G}
<!-- @index Generalized linear models; `glm`*xxx* models; 
    Generalized additive models; `gam` models; `mmer` models -->
Most models in this group receive only standard support as in [Group A](#A), but
typically the tests and confidence intervals are asymptotic. Thus the
`df` column for tabular results will be `Inf`.

Some objects in this group may require that the original or reference dataset 
be provided when calling `ref_grid()` or `emmeans()`.

For `coxph` objects, the estimates we obtain are comparable to running 
`predict.coxph()` with `reference = "zero"`; that is, no covariate centering is done.
The user may use `at` to specify adjusted covariate values. For example, the 
default reference grid sets each covariate to its mean, so estimates comparable
to `predict.coxph(..., reference = "sample")` would be obtained by specifying
`at = list(x = 0)`.

In the case of `mgcv::gam` objects, there are optional `freq` and
`unconditional` arguments as is detailed in the documentation for
`mgcv::vcov.gam()`. Both default to `FALSE`. The value of `unconditional` 
matters only if `freq = FALSE` and `object$Vc` is non-null.

For `mgcv::gamm` objects, `emmeans()` results are based on the `object$gam`
part. Unfortunately, that is missing its `call` component, so the user
must supply it in the `call` argument (e.g., `call = quote(gamm(y ~ s(x), data = dat))`) 
or give the dataset in the `data` argument. Alternatively (and recommended), 
you may first set `object$gam$call` to the quoted call ahead of time.
The `what` arguments are used to select which model formula to use: 
`"location", "scale"` apply to `gaulss` and `gevlss`
families, `"shape"` applies only to `gevlss`, and `"rate", "prob.gt.0"` 
apply to `ziplss`.

With `gam::Gam` objects, standard errors are estimated using a bootstrap
method when there are any smoothers involved. Accordingly, there is an 
optional `nboot` argument
that sets the number of bootstrap replications used to estimate the 
variances and covariances of the smoothing portions of the model.
Generally, it is better to use models fitted via `mgcv::gam()` rather
than `gam::gam()`.

[Back to quick reference](#quickref)

## Group H -- `gamlss` models {#H}
<!-- @index `gamlss` models -->
The `what` argument has possible values of `"mu"` (default), `"sigma"`, `"nu"`, 
or `"tau"` depending on which part of the model you want results for.
Currently, there is no support when the selected part of the model contains
a smoothing method like `pb()`. 

## Group I -- Multiple models (via imputation or averaging) {#I}
<!-- @index Multiple imputation; Model averaging; `averaging` models; `mira` models -->
These objects are the results of fitting several models with different predictor
subsets or imputed values. The `bhat` and `V` slots are obtained via averaging
and, in the case of multiple imputation, adding a multiple of the between-imputation 
covariance per Rubin's rules, along with an associated degrees-of-freedom
adjustment (Barnard & Rubin 1999).
In the case of `mira` models with model classes not
supported by **emmeans**, [GitHub issue #446](https://github.com/rvlenth/emmeans/issues/446) 
includes a function `pool_estimates_for_qdrg()` that may be useful for obtaining
results via `qdrg()`. Another useful link may be a page that shows [how to pool several `emmeans`
results](https://github.com/adrianolszewski/Useful-R-codes/blob/main/Pooling%20emmeans%20objects.md); that is, instead of pooling the models and then running `emmeans()`, we do just the reverse.

Support for `MuMIn::averaging` objects may be somewhat dodgy, as it is not clear
that all supported model classes will work. The object *must* have a
`"modelList"` attribute (obtained by constructing the object explicitly from a
model list or by including `fit = TRUE` in the call). And each model should be
fitted with `data` as a **named** argument in the call; or else provide a `data`
argument in the call to `emmeans()` or `ref_grid()`. Only ``full'' averaging is
supported; conditional averaging can result in non-positive-definite covariance
matrices, and so cannot be considered. No estimability checking is done at
present (not clear what we even mean by it).

Also, be aware that support for `averaging` objects does *not* pay attention to the class of the models being averaged (nor any `emmeans` options associated with that class, such as alternative modes or d.f. methods), and that you can only obtain direct results of linear predictions or back-transformations thereof (`type = "response"`). It does *not* take apart multivariate models, nor multiple-intercept models (e.g. ordinal ones). (But keep reading...)

Finally, note that special care is needed with models having multiple components (e.g. hurdle models), where there is essentially more than one set of coefficients. We can handle only one at a time. A `subset` argument must be provided to specify which coefficients to use; that can be a *named* vector of integers (where the names are the names of the actual model terms), or the special character values `"prefix:pfx"`, `"pfx"`, or `wrap:wrp` which picks out all coefficients whose names are prefixed by `pfx` (e.g., `pfxtrtB`) or wrapped by `wrp` (e.g., `wrp(trtB)`). A `formula` option is also available for specifying an appropriate formula other than `object$formula` when that is not suitable.

In many cases (especially with multivariate or ordinal models), you are better off making a copy of one of the averaged models that has all required terms, and hacking that object by replacing the coefficients by `coef(object, full = TRUE)` and the covariance matrix by `vcov(object, full = TRUE)`. You need to be careful to match the names of the coefficients correctly, and to use the same indexes to permute the rows and columns of the covariance matrix. You then use the `emmeans` support, including any options available, for the class of the model object, rather than for class `averaging`. An example is given towards the end of [Issue 442](https://github.com/rvlenth/emmeans/issues/442).


## Group K -- `gls` and `lme` models {#K}
<!-- @index `lme` models; `gls` models; Satterthwaite d.f.; Containment d.f.; 
     `appx-satterthwaite` method; `boot-satterthwaite` method -->
The `sigmaAdjust` argument is a logical value that defaults to `TRUE`. It is
comparable to the `adjustSigma` option in `nlme::summary.lme` (the name-mangling
is to avoid conflicts with the often-used `adjust` argument), and determines
whether or not a degrees-of-freedom adjustment is performed with models fitted
using the ML method.

The optional `mode` argument affects the degrees of freedom. The `mode =
"satterthwaite"` option determines degrees of freedom via the Satterthwaite
method: If `s^2` is the estimate of some variance, then its Satterthwaite d.f.
is `2*s^4 / Var(s^2)`. In case our numerical methods for this fail, we also
offer `mode = "appx-satterthwaite"` as a backup, by which quantities related to
`Var(s^2)` are obtained by randomly perturbing the response values. Currently,
only `"appx-satterthwaite"` is available for `lme` objects, and it is used
if `"satterthwaite"` is requested. 
Because `appx-satterthwaite` is simulation-based, results may vary if the same 
analysis is repeated. An `extra.iter` argument may be added to request additional simulation runs (at [possibly considerable] cost of repeating the model-fitting that many more times).
(Note: Previously, `"appx-satterthwaite"` was
termed `"boot-satterthwaite"`; this is still supported for backward compatibility.
The "boot" was abandoned because it is really an approximation method, not
a bootstrap method in the sense as many statistical methods.)

An alternative method is `"df.error"` (for `gls`) and
`"containment"` (for `lme`). `df.error` is just the error degrees of freedom for
the model, minus the number of extra random effects estimated; it generally
over-estimates the degrees of freedom. The `asymptotic` mode simply sets the 
degrees of freedom to infinity.
`"containment"` mode (for `lme` models) determines the degrees of
freedom for the coarsest grouping involved in the contrast or linear function
involved, so it tends to under-estimate the degrees of freedom.
The default is `mode = "auto"`, which uses Satterthwaite if there are estimated 
random effects and the non-Satterthwaite option otherwise.

User reports indicate that models with special terms like `poly()` are not adequately
supported by `gls` in that the needed basis is not recoverable from its `terms`
component. This is not a problem with `lme`.

The `extra.iter` argument is ignored unless the d.f. method is (or defaults to) 
`appx-satterthwaite`.

[Back to quick reference](#quickref)

## Group L -- `lmerMod` models {#L}
<!-- @index `lmerMod` models; Kenward-Roger d.f.; Satterthwaite d.f. -->
There is an optional `lmer.df` argument that defaults to 
`get_EMM_option("lmer.df")` (which in turn defaults to 
`"kenward-roger"`). The possible values are `"kenward-roger"`,
`"satterthwaite"`, and `"asymptotic"` (these are partially matched and 
case-insensitive). With `"kenward-roger"`, d.f. are obtained using code
from the **pbkrtest** package, if installed. With `"satterthwaite"`,
d.f. are obtained using code from the **lmerTest** package, if installed. 
With `"asymptotic"`, or if the needed package is not installed, d.f. are
set to `Inf`. (For backward compatibility, the user may specify
`mode` in lieu of `lmer.df`.)

A by-product of the Kenward-Roger method is that the covariance matrix is 
adjusted using `pbkrtest::vcovAdj()`. This can require considerable
computation; so to avoid that overhead, the user should opt for the 
Satterthwaite or asymptotic method; or, for backward compatibility, may 
disable the use of **pbkrtest** via `emm_options(disable.pbkrtest =
TRUE)` (this does not disable the **pbkrtest** package entirely, just its
use in **emmeans**). The computation time required depends roughly on the
number of observations, *N*, in the design matrix (because a major part
of the computation involves inverting an *N* x *N* matrix). Thus,
**pbkrtest** is automatically disabled if *N* exceeds the value of 
`get_emm_option("pbkrtest.limit")`, for which the factory default is 3000.
(The user may also specify `pbkrtest.limit` or `disable.pbkrtest` as an
argument in the call to `emmeans()` or `ref_grid()`)

Similarly to the above, the `disable.lmerTest` and `lmerTest.limit` options
or arguments affect whether Satterthwaite methods can be implemented.

The `df` argument may be used to specify some other degrees of freedom.
Note that if `df` and `method = "kenward-roger"` are both
specified, the covariance matrix is adjusted but the K-R degrees of freedom
are not used. 

Finally, note that a user-specified covariance matrix 
(via the `vcov.` argument) will also disable the Kenward-Roger method; in
that case, the Satterthwaite method is used in place of Kenward-Roger.

[Back to quick reference](#quickref)

## Group M -- Multivariate models {#M}
<!-- @index Multivariate models -->
When there is a multivariate response, the different responses are treated as
if they were levels of a factor -- named `rep.meas` by default. The
`mult.name` argument may be used to change this name. The
`mult.levs` argument may specify a named list of one or more sets of
levels. If this has more than one element, then the multivariate levels are
expressed as combinations of the named factor levels via the function
`base::expand.grid`.


## N - Multinomial responses {#N}
<!-- @index `mlm` models; Multinomial models -->
The reference grid includes a pseudo-factor with the same name and levels as
the multinomial response. (If the response is an expression, the name of that pseudo-factor
will be the first name in the expression; e.g., if the model formula is 
`cbind(col1, col2) ~ trt`, the grid factors will be `cbind` and `trt` and the
levels of `cbind` will be `1` and `2`.) (You can change the assigned name for the
multinomial response via the `mult.resp` argument.)

There is an optional `mode` argument which
should match `"prob"` or `"latent"`. With `mode = "prob"`, the
reference-grid predictions consist of the estimated multinomial
probabilities -- and this implies a re-gridding so no
link functions are passed on. The `"latent"` mode returns the linear predictor,
re-centered so that it averages to zero over the levels of the response
variable (similar to sum-to-zero contrasts). Thus each latent variable can be
regarded as the log probability at that level minus the average log
probability over all levels.

There are two optional arguments: `mode` and `rescale` (which
defaults to `c(0, 1)`).

Please note that, because the probabilities sum to 1 (and the latent values
sum to 0) over the multivariate-response levels, all sensible results from
`emmeans()` must involve that response as one of the factors. For example,
if `resp` is a response with *k* levels, `emmeans(model, ~ resp
| trt)` will yield the estimated multinomial distribution for each
`trt`; but `emmeans(model, ~ trt)` will just yield the average
probability of 1/*k* for each `trt`.


[Back to quick reference](#quickref)

## Group O - Ordinal responses {#O}
<!-- @index Ordinal models; `clm` models; `polreg` models -->
The reference grid for ordinal models will include all variables that appear in
the main model
as well as those in the `scale` or `nominal` models (if provided). 
There are two optional arguments: `mode` (a character string) and 
`rescale` (which defaults to `c(0, 1)`). `mode` should match 
one of `"latent"` (the default), `"linear.predictor"`, 
`"cum.prob"`, `"exc.prob"`, `"prob"`, `"mean.class"`, or
`"scale"` -- see the quick reference and note which are supported.
With the exception of `"linear.predictor"`, all of these modes do an implied regrid.

With `mode = "latent"`, the reference-grid predictions are made on the
scale of the latent variable implied by the model. The scale and location of
this latent variable are arbitrary, and may be altered via `rescale`.
The predictions are multiplied by `rescale[2]`, then added to `rescale[1]`.
Keep in mind that the scaling is related to the link function used
in the model; for example, changing from a probit link to a logistic link
will inflate the latent values by around $\pi/\sqrt{3}$, all
other things being equal. `rescale` has no effect for other values of
`mode`. Even though the latent means comprise a re-scaling of the linear predictor,
we regard this as a re-gridding, and the cumulative link function is 
not included in the reference grid.

With `mode = "linear.predictor"`, `mode = "cum.prob"`, and
`mode = "exc.prob"`, the boundaries between categories (i.e.,
thresholds) in the ordinal response are included in  the reference grid as a
pseudo-factor named `cut`. The reference-grid predictions are then of
the cumulative probabilities at each threshold (for `mode =
"cum.prob"`), exceedance probabilities (one minus cumulative probabilities,
for `mode = "exc.prob"`), or the link function thereof (for `mode =
"linear.predictor"`).

With `mode = "prob"`, a pseudo-factor with the same name as the model's
response variable is created, and the grid predictions are of the
probabilities of each class of the ordinal response. With
`"mean.class"`, the returned results are means of the ordinal response,
interpreted as a numeric value from 1 to the number of classes, using the
`"prob"` results as the estimated probability distribution for each
case.

With `mode = "scale"`, and the fitted object incorporates a scale model,
EMMs are obtained for the factors in the scale model (with a log response)
instead of the response model. The grid is constructed using only the factors
in the scale model.

Any grid point that is non-estimable by either the location or the scale
model (if present) is set to `NA`, and any EMMs involving such a
grid point will also be non-estimable. A consequence of this is that if there
is a rank-deficient `scale` model, then *all* latent responses
become non-estimable because the predictions are made using the average
log-scale estimate.

`rms` models have an additional `mode`. With `mode = "middle"`
(this is the default), the middle intercept is used, comparable to the
default for `rms::Predict()`. This is quite similar in
concept to `mode = "latent"`, where all intercepts are averaged
together.

[Back to quick reference](#quickref)

## Group P -- Other packages {#P}

Models in this group have their **emmeans** support provided by another package
(usually the one that implements the model-fitting procedure). Users should refer to the
package documentation for details on **emmeans** support. In some cases, a
package's models may have been supported here in **emmeans**; if so, the 
other package's support overrides it.


## Group Q -- Quantile regression {#Q}

The elements
of `tau` are included in the reference grid as a pseudo-factor named
`tau`. In these models, the covariance matrix is obtained via the model's
`summary()` method with `covariance = TRUE`. The user may specify one or more of
the other arguments for `summary` (e.g., `se = "boot"`) or to be passed to the `...`
argument. 

A caveat is that when there is more than one `tau` value, we do not have
estimates of the covariances between regression coefficients associated with
different `tau`s. Thus, a contrast involving different `tau`s can be estimated
but its SE will be `NA`. Also, due to `NA`s in the covariance matrix, the
`"mvt"` adjustment is unavailable.

*Note:* Older versions of this `rq` and `rqs` support *required* `tau`; now it is
optional. Only `tau` values included in `object$tau` are allowed; others are
silently ignored. It is more efficient (and less memory-greedy) to specify `tau` 
than to subset them using the `at` argument to `ref_grid`.



## Group S -- Sampling (MCMC) methods {#S}
<!-- @index Bayesian models; `mcmc` objects; `stanreg` objects; `brmsfit` objects
  `as.mcmc()`; `summary()`!Bayesian models -->
Models fitted using MCMC methods contain a sample from the posterior
distribution of fixed-effect coefficients. In some cases (e.g., results of
`MCMCpack::MCMCregress()` and `MCMCpack::MCMCpoisson()`), the object may include a
`"call"` attribute that `emmeans()` can use to reconstruct the data
and obtain a basis for the EMMs. If not, a `formula` and
`data` argument are provided that may help produce the right results. In
addition, the `contrasts` specifications are not necessarily recoverable
from the object, so the system default must match what was actually used in
fitting the model.

The `summary.emmGrid()` method provides credibility intervals (HPD intervals) of
the results, and ignores the frequentist-oriented arguments (`infer`, `adjust`,
etc.) An `as.mcmc()` method is provided that creates an `mcmc` object that can
be summarized or plotted using the **coda** package (or others that support
those objects). It provides a posterior sample of EMMs, or contrasts thereof,
for the given reference grid, based on the posterior sample of the fixed effects
from the model object.

In `MCMCglmm` objects, the `data` argument is required; however, if you
save it as a member of the model object (e.g., `object$data = quote(mydata)`),
that removes the necessity of specifying it in each call.
The special keyword `trait` is used in some models. 
When the response is multivariate and numeric, `trait` is generated automatically
as a factor in the reference grid, and the arguments `mult.levels` can be used to
name its levels. In other models such as a multinomial model, use the
`mode` argument to specify the type of model, and `trait = <factor name>` 
to specify the name of the data column that contains the levels of the factor response.

The **brms** package version 2.13 and later, has its own `emmeans` support. 
Refer to the documentation in that package.


[Back to quick reference](#quickref)

## Group V -- `aovList` objects (also used with `afex_aov` objects) {#V}
<!-- @index `aovList` objects; `afex_aov` objects -->
Support for these objects is limited. To avoid strong biases in the predictions,
it is strongly recommended that when fitting the model, the `contrasts`
attribute of all factors should be of a type that sums to zero -- for example,
`"contr.sum"`, `"contr.poly"`, or `"contr.helmert"` but *not*
`"contr.treatment"`.  If that is found not to be the case, the model is
re-fitted using sum-to-zero contrasts (thus requiring additional computation).
Doing so does *not* remove all bias in the EMMs unless the design is perfectly 
balanced, and an annotation is added to warn of that. This bias cancels out 
when doing comparisons and contrasts.

Only intra-block estimates of covariances are used. That is, if a factor appears
in more than one error stratum, only the covariance structure from its lowest
stratum is used in estimating standard errors. Degrees of freedom are obtained
using the Satterthwaite method. In general, `aovList` support is best with
balanced designs, with due caution in the use of contrasts. If a `vcov.`
argument is supplied, it must yield a single covariance matrix for the unique
fixed effects (not a set of them for each error stratum). In that case, the
degrees of freedom are set to `NA`.

[Back to quick reference](#quickref)

[Index of all vignette topics](vignette-topics.html)

---
title: "Comparisons and contrasts in emmeans"
author: "emmeans package, Version `r packageVersion('emmeans')`"
output: emmeans::.emm_vignette
vignette: >
  %\VignetteIndexEntry{Comparisons and contrasts}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
```{r, echo = FALSE, results = "hide", message = FALSE}
require("emmeans")
knitr::opts_chunk$set(fig.width = 4.5, class.output = "ro")
```

<!-- @index Vignettes!Comparisons -->

## Contents
This vignette covers techniques for comparing EMMs at levels of a factor
predictor, and other related analyses.

  1. [Pairwise comparisons](#pairwise)
  2. [Other contrasts](#contrasts)
  3. [Formula interface](#formulas)
  4. [Custom contrasts and linear functions](#linfcns)
  5. [Special behavior with log transformations](#logs)
  6. Interaction contrasts (in ["interactions" vignette](interactions.html#contrasts))
  7. Multivariate contrasts (in ["interactions" vignette](interactions.html#multiv))
  
[Index of all vignette topics](vignette-topics.html)

## Pairwise comparisons {#pairwise}
<!-- @index Pairwise comparisons; Contrasts!Pairwise; `pairs()` -->
The most common follow-up analysis for models having factors as predictors is
to compare the EMMs with one another. This may be done simply via the `pairs()`
method for `emmGrid` objects. In the code below, we obtain the EMMs for `source` for
the `pigs` data, and then compare the sources pairwise.
```{r}
pigs.lm <- lm(log(conc) ~ source + factor(percent), data = pigs)
pigs.emm.s <- emmeans(pigs.lm, "source")
pairs(pigs.emm.s)
```
<!-- @index `adjust`; `summary()`!`adjust`; `summary()`!`infer` -->
In its out-of-the-box configuration, `pairs()` sets two defaults for 
[`summary()`](confidence-intervals.html#summary): `adjust = "tukey"`
(multiplicity adjustment), and `infer = c(FALSE, TRUE)` (test statistics, not
confidence intervals). You may override these, of course, by calling `summary()`
on the result with different values for these.

In the example above, EMMs for later factor levels are subtracted from those for
earlier levels; if you want the comparisons to go in the other direction, use
`pairs(pigs.emm.s, reverse = TRUE)`. Also, in multi-factor situations,
you may specify `by` factor(s) to perform the comparisons separately at the 
levels of those factors.

### Matrix displays {#pwpm}
<!-- @index `pwpm()`; Pairwise comparisons!Matrix displays; -->
The numerical main results associated with pairwise comparisons can be presented
compactly in matrix form via the `pwpm()` function. We simply hand it the
`emmGrid` object to use in making the comparisons:
```{r}
pwpm(pigs.emm.s)
```
This matrix shows the EMMs along the diagonal, $P$ values in the upper triangle,
and the differences in the lower triangle. Options exist to switch off any one of these
and to switch which triangle is used for the latter two. Also, optional
arguments are passed. For instance, we can reverse the direction of the comparisons,
suppress the display of EMMs, swap where the $P$ values go,
and perform non-inferiority tests with a threshold of 0.05 as follows:
```{r}
pwpm(pigs.emm.s, means = FALSE, flip = TRUE,     # args for pwpm()
     reverse = TRUE,                             # args for pairs()
     side = ">", delta = 0.05, adjust = "none")  # args for test()
```
With all three *P* values so small, we have fish, soy, and skim in increasing order of
non-inferiority based on the given threshold.

When more than one factor is present, an existing or newly specified `by` variables()
can split the results into l list of matrices.


### Effect size
<!-- @index Effect size; Cohen's *d*; `eff_size()` -->
Some users desire standardized effect-size measures. Most popular is probably
Cohen's *d*, which is defined as the observed difference, divided by the
population SD; and obviously Cohen effect sizes are close cousins of pairwise
differences. They are available via the `eff_size()` function, where the user
must specify the `emmGrid` object with the means to be compared, the estimated
population SD `sigma`, and its degrees of freedom `edf`. This is illustrated with the 
current example:
```{r}
eff_size(pigs.emm.s, sigma = sigma(pigs.lm), edf = 23)
```
The confidence intervals shown take into account the error in estimating `sigma` as
well as the error in the differences. Note that the intervals are narrower if we
claim that we know `sigma` perfectly (i.e., infinite degrees of freedom):
```{r}
eff_size(pigs.emm.s, sigma = sigma(pigs.lm), edf = Inf)
```
Note that `eff_size()` expects the object with the means, not the differences.
If you want to use the differences, use the `method` argument to specify that
you don't want to compute pairwise differences again; e.g.,
```{r, eval = FALSE}
eff_size(pairs(pigs.emm.s), sigma = sigma(pigs.lm), edf = 23, method = "identity")
```
(results are identical to the first effect sizes shown).


### Graphical comparisons {#graphical}
<!-- @index Comparisons!Graphical -->
Comparisons may be summarized graphically via the `comparisons` argument
in `plot.emm()`:
```{r fig.height = 1.5, fig.alt = "side-by-side CIs with comparison arrows added"}
plot(pigs.emm.s, comparisons = TRUE)
```

The blue bars are confidence intervals for the EMMs, and the red arrows are for 
the comparisons among them. If an arrow from one mean overlaps an arrow from
another group, the difference is not "significant," based on the `adjust` setting
(which defaults to `"tukey"`) and the value of `alpha` (which defaults to 0.05). 
See the ["xplanations" supplement](xplanations.html#arrows) for details on how 
these are derived.

*Note:* Don't *ever* use confidence intervals for EMMs to perform comparisons; they 
can be very misleading. Use the comparison arrows instead; or better yet, use `pwpp()`.

*A caution:* it really is not good practice to draw a
bright distinction based on whether or not a *P* value exceeds some cutoff.
This display does dim such distinctions somewhat by allowing the viewer 
to judge whether a *P* value is close to `alpha` one way or the other; but a better
strategy is to simply obtain all the *P* values using `pairs()`, and look
at them individually.

#### Pairwise *P*-value plots {#pwpp}
<!-- @index `pwpp()`; Pairwise *P*-value plots; Comparisons!Displaying *P* values -->
In trying to develop an alternative to compact letter displays (see next subsection), we 
devised the "pairwise *P*-value plot" displaying all the *P* values in pairwise comparisons:
```{r fig.alt = "Pairwise P-value plot that shows, for each pair of means, a vertical line segment whose horizontal position is the Tukey-adjusted P-value for that comparison. The endpoints of the line segments align with the vertical scale showing the soy levels and their means. This particular plot shows that skim-fish and soy-fish are highyly significant, while skim-soy has a P-value just over 0.05"}
pwpp(pigs.emm.s)
```

Each comparison is associated with a vertical line segment that joins the scale
positions of the two EMMs being compared, and whose horizontal position is determined
by the *P* value of that comparison. 

This kind of plot can get quite "busy" as the number of means being compared
goes up. For example, suppose we include the interactions in the model for the pigs
data, and compare all 12 cell means:
```{r, fig.width = 9, fig.alt = "pwpp for all 78 pairwise comparisons of cell means. It is pretty hard to digest due to its complexity. To see all this information in text form, call pwpm(pigs.cells)"}
pigs.lmint <- lm(log(conc) ~ source * factor(percent), data = pigs)
pigs.cells <- emmeans(pigs.lmint, ~ source * percent)
pwpp(pigs.cells, type = "response")
```

While this plot has a lot of stuff going on, consider looking at it row-by-row.
Next to each EMM, we can visualize the *P* values of all 11 comparisons
with each other EMM (along with their color codes).
Also, note that we can include arguments that are passed to `summary()`; in this case,
to display the back-transformed means. 

If we are willing to forgo the diagonal comparisons (where neither factor has a
common level), we can make this a lot less cluttered via a `by` specification:
```{r, fig.width = 6, fig.alt = "pwpp presented in separate panels for each source. Each panel has just 6 P-value bars, making it much less cluttered than the previous figure"}
pwpp(pigs.cells, by = "source", type = "response")
```

In this latter plot we can see that the comparisons with `skim` as the source 
tend to be statistically stronger. This is also an opportunity to remind the user
that multiplicity adjustments are made relative to each `by` group. For example,
comparing `skim:9` versus `skim:15` has a Tukey-adjusted *P* value somewhat greater
than 0.1 when all are in one family of 12 means, but about 0.02 relative to
a smaller family of 4 means as depicted in the three-paneled plot.


#### Compact letter displays (CLDs) {#CLD}
<!-- @index `cld()`; Compact letter displays;
             Comparisons!Displaying as groups -->
Another way to depict comparisons is by *compact letter displays*, whereby
two EMMs sharing one or more grouping symbols are not "significantly" different.
These may be generated by the `multcomp::cld()` function.
I really recommend against this kind of display, though, and decline to illustrate it.
These displays promote visually the idea that two means that are "not
significantly different" are to be judged as being equal; and that is a very wrong
interpretation. In addition, they draw an artificial "bright line" between
*P* values on either side of `alpha`, even ones that are very close.

[Back to Contents](#contents)


## Other contrasts {#contrasts}
<!-- @index `contrast()`; `coef()`; Contrasts!Polynomial -->
Pairwise comparisons are an example of linear functions of EMMs.
You may use `coef()` to see the coefficients of these linear functions:
```{r}
coef(pairs(pigs.emm.s))
```
The pairwise comparisons correspond to columns of the above results.
For example, the first pairwise comparison, `fish - soy`, gives coefficients
of 1, -1, and 0 to fish, soy, and skim, respectively. In cases, such as this
one, where each column of coefficients sums to zero, the linear functions
are termed *contrasts*

The `contrast()` function provides for general contrasts (and linear functions,
as well) of factor levels. Its second argument, `method`, is used to specify
what method is to be used. In this section we describe the built-in ones,
where we simply provide the name of the built-in method. Consider, for example,
the factor `percent` in the model `pigs.lm` . It is treated as a factor in 
the model, but it corresponds to equally-spaced values of a numeric variable.
In such cases, users often want to compute orthogonal polynomial contrasts:
```{r}
pigs.emm.p <- emmeans(pigs.lm, "percent")
ply <- contrast(pigs.emm.p, "poly")
ply

coef(ply)
```
We obtain tests for the linear, quadratic, and cubic trends. The coefficients
are those that can be found in tables in many experimental-design texts.
It is important to understand that the estimated linear contrast is *not* the
slope of a line fitted to the data. It is simply a contrast having coefficients
that increase linearly. It *does* test the linear trend, however.

<!-- @index Dunnett method; `pairwise` contrasts; `revpairwise` contrasts; `trt.vs.ctrl` contrasts;
            `eff` contrasts; `consec` contrasts -->
There are a number of other named contrast methods, for example `"trt.vs.ctrl"`,
`"eff"`, and `"consec"`. The `"pairwise"` and `"revpairwise"` methods in `contrast()` are the same as `Pairs()` and `pairs(..., reverse = TRUE)`. See
`help("contrast-methods")` for details.

### Normalized contrasts {#nrmlze}
<!-- @index Normalized contrasts; `nrmlz.emmc`; Contrasts!Normalized -->
Sometimes users like contrast coefficients to be normalized -- i.e., the sum of 
the squares of each set of contrast coefficients is equal to 1. This is possible with
any contrast family by specifying `"nrmlz"` as the method and the desired contrast family
as the `family` argument. For example,
```{r}
nrmlz.emmc(1:4, family = "helmert")
contrast(pigs.emm.s, "nrmlz", family = "pairwise")
```
Note that the above estimates and SEs differ from those in the example at [the
beginning of this vignette](#pairwise), but the the *t* ratios and *P* values
are the same, because all we have done is rescale those comparisons.

[Back to Contents](#contents)



## Formula interface {#formulas}
<!-- @index `specs`!Formula; Contrasts!Formula; `emm_list` object
             Examples!`oranges` -->
If you already know what contrasts you will want before calling `emmeans()`, 
a quick way to get them is to specify the method as the left-hand side of the formula in its second argument. For example, with the `oranges` dataset
provided in the package,
```{r}
org.aov <- aov(sales1 ~ day + Error(store), data = oranges,
               contrasts = list(day = "contr.sum"))
org.emml <- emmeans(org.aov, consec ~ day)
org.emml
```
The contrasts shown are the day-to-day changes.

This two-sided formula technique is quite convenient, but it can also create
confusion. For one thing, the result is not an `emmGrid` object anymore; it is a
`list` of `emmGrid` objects, called an `emm_list`. You may need to be cognizant of
that if you are to do further contrasts or other analyzes. For example if you
want `"eff"` contrasts as well, you need to do `contrast(org.emml[[1]],
"eff")` or `contrast(org.emml, "eff", which = 1)`.

Another issue is that it may be unclear which part of the results is
affected by certain options. For example, if you were to add `adjust = "bonf"`
to the `org.emm` call above, would the Bonferroni adjustment be applied to the
EMMs, or to the contrasts? (See the documentation if interested; but the best practice is to avoid such dilemmas.)

[Back to Contents](#contents)

## Custom contrasts and linear functions {#linfcns}
<!-- @index Contrasts!Custom; `.emmc` functions@emmc; `contrast()`!`adjust` -->
The user may write a custom contrast function for use in `contrast()`.
What's needed is a function having the desired name with `".emmc"` appended,
that generates the needed coefficients as a list or data frame. The
function should take a vector of levels as its first argument, 
and any optional parameters as additional arguments. It should also always have
a `...` argument to allow for unspecified arguments that may occur in the
call. 

As an example,
suppose we want to compare every third level of a treatment. 
The following function provides for this:
```{r}
skip_comp.emmc <- function(levels, skip = 1, reverse = FALSE, ...) {
    if((k <- length(levels)) < skip + 1)
        stop("Need at least ", skip + 1, " levels")
    coef <- data.frame()
    coef <- as.data.frame(lapply(seq_len(k - skip - 1), function(i) {
        sgn <- ifelse(reverse, -1, 1)
        sgn * c(rep(0, i - 1), 1, rep(0, skip), -1, rep(0, k - i - skip - 1))
    }))
    names(coef) <- sapply(coef, function(x)
        paste(which(x == 1), "-", which(x == -1)))
    attr(coef, "adjust") = "fdr"   # default adjustment method
    coef
}
```
To test it, try 5 levels:
```{r}
skip_comp.emmc(1:5)

skip_comp.emmc(1:5, skip = 0, reverse = TRUE)
```
(The latter is the same as `"consec"` contrasts.)
Now try it with the `oranges` example we had previously:
```{r}
contrast(org.emml[[1]], "skip_comp", skip = 2, reverse = TRUE)
```

####### {#linfct}
<!-- @index Linear functions; `contrast()`!Linear functions -->
The `contrast()` function may in fact be used to compute arbitrary linear
functions of EMMs. Suppose for some reason we want to estimate the quantities
$\lambda_1 = \mu_1+2\mu_2-7$ and $\lambda_2 = 3\mu_2-2\mu_3+1$, where the
$\mu_j$ are the population values of the `source` EMMs in the `pigs` example. 
This may be done by providing the coefficients in a list, and the added
constants in the `offset` argument:
```{r}
LF <- contrast(pigs.emm.s, 
               list(lambda1 = c(1, 2, 0), lambda2 = c(0, 3, -2)),
               offset = c(-7, 1))
confint(LF, adjust = "bonferroni")
```

[Back to Contents](#contents)

## Special properties of log (and logit) transformations {#logs}
<!-- @index Transformations!Log; Comparisons!with logs@logs;
            Comparisons!Back-transforming -->
Suppose we obtain EMMs for a model having a response transformation 
or link function. In most cases, when we compute contrasts of those EMMs,
there is no natural way to express those contrasts on anything other
than the transformed scale. For example, in a model fitted using `glm()`
with the `gamma()` family, the default link function is the inverse.
Predictions on such a model are estimates of $1/\mu_j$ for various $j$.
Comparisons of predictions will be estimates of $1/\mu_j - 1/\mu_{k}$
for $j \ne k$. There is no natural way to back-transform these 
differences to some other interpretable scale.

However, logs are an exception, in that 
$\log\mu_j - \log\mu_k = \log(\mu_j/\mu_k)$. Accordingly, when `contrast()`
(or `pairs()`) notices that the response is on the log scale, it back-transforms 
contrasts to ratios when results are to be of `response` type. For example:
```{r}
pairs(pigs.emm.s, type = "lp")

pairs(pigs.emm.s, type = "response")
```
<!-- @index Contrasts!Tests of!with transformations@trans -->
As is true of EMM summaries with `type = "response"`, the tests and confidence
intervals are done before back-transforming. The ratios estimated here are
actually ratios of *geometric* means. In general, a model with a log response is
in fact a model for *relative* effects of any of its linear predictors, and this
back-transformation to ratios goes hand-in-hand with that.

In generalized linear models, this behaviors will occur in two common cases:
Poisson or count regression, for which the usual link is the log; and logistic
regression, because logits are logs of odds ratios.

[Back to Contents](#contents)



[Index of all vignette topics](vignette-topics.html)

---
title: "Quick start guide for emmeans"
author: "emmeans package, Version `r packageVersion('emmeans')`"
output: emmeans::.emm_vignette
vignette: >
  %\VignetteIndexEntry{A quick-start guide for emmeans}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
```{r, echo = FALSE, results = "hide", message = FALSE}
require("emmeans")
knitr::opts_chunk$set(fig.width = 4.5, class.output = "ro")
```

<!-- @index Quick-start guide; **emmeans** package!Quick-start guide -->

## Contents {#contents}
  1. [The three basic steps](#steps)
  2. [One-factor model](#one-factor)
  3. [Two factors, no interaction](#additive)
  4. [Two interacting factors](#interactions)
  5. [Three or more factors](#multi-factor)
  6. [Additional options](#options)
  7. [What you see versus what you get](#objects)
  8. [Common things that can go wrong](#problems)
  9. [Further reading](#more)


## The three basic steps {#steps}
Much of what you do with the **emmeans** package involves these three basic steps:

  0. Fit a *good* model to your data, and do reasonable checks to make sure
     it adequately explains the respons(es) and reasonably meets underlying 
     statistical assumptions. Modeling is not the focus of **emmeans**, but this
     is an extremely important step because **emmeans** does not analyze your data,
     it summarizes your model. If it is a bad model, you will likely get 
     misleading results from this package -- the garbage in, garbage out principle.
     If you're not sure whether your model is any good, this is a good time
     to get statistical consulting help. This really *is* like rocket science, not
     just a matter of getting programs to run.
  1. Run  `EMM <- emmeans(...)` (see scenarios below) to obtain estimates of 
     means or marginal means
  2. Run `contrast(EMM, ...)` or `pairs(EMM)` one or more times to obtain
     estimates of contrasts or pairwise comparisons among the means.

**Note:** A lot of users have developed the habit of running something like 
`emmeans(model, pairwise ~ factor(s))`, which conflates steps 1 and 2. We recommend
*against* doing this because it often yields output you don't want or need -- especially
when there is more than one factor. You are better off keeping steps 1 and 2 separate.
What you do in step 2 depends on how many factors you have, and how they relate.
     
### One-factor model {#one-factor}
If a one-factor model fits well and the factor is named `treatment`, do
```r
EMM <- emmeans(model, "treatment")   # or emmeans(model, ~ treatment)
EMM    # display the means

### pairwise comparisons
contrast(EMM, "pairwise")    # or pairs(EMM)
```
You may specify other contrasts in the second argument of the `contrast()` call, e.g. `"trt.vs.ctrl", ref = 1` (compare each mean to the first),
or `"consec"` (compare 2 vs 1, 3 vs 2, etc.), or `"poly", max.degree = 3` (polynomial contrasts)

### Two factors, no interaction {#additive}
If the model fits well and factors are named `treat` and `dose`, and they don't interact, follow 
the same steps as for one factor at a time.
That is, something like
```
(EMM1 <- emmeans(model, ~ treat))
pairs(EMM1)

(EMM2 <- emmeans(model, ~ dose))
pairs(EMM2)
```
These analyses will yield the estimated *marginal* means for
each factor, and comparisons/contrasts thereof.


[Back to Contents](#contents)

### Two interacting factors {#interactions}
In this case, unless the interaction effect is negligible, we usually want to 
do "simple comparisons" of the cell means. That is, compare
or contrast the means separately, holding one factor fixed at each level.
```r
EMM <- emmeans(model, ~ treat * dose)
EMM    # display the cell means

### Simple pairwise comparisons...
pairs(EMM, simple = "treat")    # compare treats for each dose -- "simple effects"
pairs(EMM, simple = "dose")     # compare doses for each treat
```
The default is to apply a separate Tukey adjustment to the *P* values in each
`by` group (so if each group has just 2 means, no adjustment at all is applied).
If you want to adjust the whole family combined, you need to undo the `by` variable
and specify the desired adjustment (which *can't* be Tukey because that method
is invalid when you have more than one set of pairwise comparisons.) For example
```r
test(pairs(EMM, by = "dose"), by = NULL, adjust = "mvt")
```
#### Diagonal comparisons
If the "diagonal" comparisons (where *both* factors differ) are of
interest, you would do `pairs(EMM)` without a `by` variable. But you get a lot more
comparisons this way.

#### Interaction contrasts
Sometimes you may want to examine *interaction contrasts*, which are contrasts of contrasts. The thing to know here is that `contrast()` or (`pairs()`) creates the same kind of object as `emmeans()`, so you can run them multiple times. For example,
```r
CON <- pairs(EMM, by = "dose")
contrast(CON, "consec", by = NULL)    # by = NULL is essential here!
```
Or equivalently, the named argument `interaction` can be used
```r
contrast(EMM, interaction = c("pairwise", "consec"))
```

### Three or more factors {#multi-factor}
After you have mastered the strategies for two factors, you can adapt them to
three or more factors as appropriate, based on how they interact and what you need.

[Back to Contents](#contents)


## Additional options {#options}
  1. See the help files for *both* `emmeans()` and `ref_grid()` for additional
     arguments that may prove useful. Many of the most useful arguments are
     passed to `ref_grid()`.
  2. There are a number of vignettes provided with the package that include
     examples and discussions for different kinds of situations. There is also an 
     [index of vignette topics](vignette-topics.html).

## What you see versus what you get {#objects}
Most non-graphical functions in the **emmeans** package produce one of two classes of objects. The functions `emmeans()`, `emtrends()`, `ref_grid()`, `contrast()`, and `pairs()` return `emmGrid` objects (or lists thereof, class `emm_list`). For example
```
EMM <- emmeans(mod, "Treatment")
```
The functions `summary()`, `confint()`, `test()`, `joint_tests()`, and others
return `summary_emm` objects (or lists thereof, class `summary_eml`):
```
SEMM <- summary(EMM)
```
If you display `EMM` and `SEMM`, they *look* identical; that's because `emmGrid` objects are displayed using `summary()`. But they are not identical. `EMM` has all the ingredients needed to do further analysis, e.g. `contrast(EMM, "consec")` will estimate comparisons between consecutive `Treatment` means. But `SEMM` is just an annotated data frame and we can do no further analysis with it. Similarly, we can change how `EMM` is displayed via arguments to `summary()` or relatives, while in `SEMM`, everything has been computed and those results are locked-in.


## Common things that can go wrong {#problems}

### Only one mean is obtained -- or fewer than expected {#covar}
<!-- @index Only one mean; Too few means -->
This is probably the most common issue, and it can happen when a treatment is coded as a 
**numeric predictor** rather than a factor. Instead of getting a mean for each 
treatment, you get a mean at the average of those numerical values. 

  1. In such cases, the model is often inappropriate; you should replace 
     `treatment` with `factor(treatment)` and re-fit the model.
  2. In a situation where it *is* appropriate to consider the treatment as a
     quantitative predictor, then you can get separate means at specified
     values by adding an argument like `at = list(treatment = c(3,5,7))`
     to the `emmeans()` call.
  3. When you have a numerical predictor interacting with a factor, it may be
     useful to estimate its slope for each level of that factor. See the documentation
     for `emtrends()`
     
### Having trouble with follow-up analyses, and the `pairwise ~ ...` recipe {#pairwise}
<!-- @index `pairwise ~ factors`!Why you should avoid this -->
The basic object returned by `emmeans()` and `contrast()` is of class `emmGrid`,
and additional `emmeans()` and `contrast()` calls can accept `emmGrid` objects.
However, some options create *lists* of `emmGrid` objects, and that makes things 
a bit confusing. The most common case is using a call like 
`emmeans(model, pairwise ~ treat * dose)`, which computes the means *and* 
all pairwise comparisons -- a list of two `emmGrid`s. If you try to obtain additional
contrasts, say, of this result, `contrast()` makes a guess that you want to
run it on just the first element. 

This causes confusion (I know, because I get a lot of questions about it).
I recommend that you avoid using the `pairwise ~` construct altogether: 
Get your means in one step, and get your contrasts in separate step(s).
The `pairwise ~` construct is generally useful if you have only one factor; 
otherwise, it likely gives you results you don't want.

## Further reading {#more}
There are several of these vignettes that offer more details and 
more advanced topics. [An index of all these vignette topics is available here](vignette-topics.html).

The strings linked below are the names of the vignettes; i.e., they can
also be accessed via `vignette("`*name*`", "emmeans")`

  * Models that are supported in **emmeans** (there are lots of them)
    ["models"](models.html)
  * Basic ideas that underlie estimated marginal means (EMMs):
    ["basics"](basics.html). These concepts
    emphasize *experimental* data, as distinct from *observational* studies.
  * Confidence intervals and tests: 
    ["confidence-intervals"](confidence-intervals.html)
  * Often, users want to compare or contrast EMMs: ["comparisons"](comparisons.html)
  * Working with response transformations and link functions:
    ["transformations"](transformations.html)
  * Multi-factor models with interactions: ["interactions"](interactions.html)
  * Making predictions from your model: ["predictions"](predictions.html)
  * Examples of more sophisticated models (e.g., mixed, ordinal, MCMC)
    ["sophisticated"](sophisticated.html)
  * Working with messy data, counterfactuals, mediating covariates, 
    and nested effects: 
    ["messy-data"](messy-data.html). Here is where you may see more on
    how **emmeans** might help with observational data.
  * Utilities for working with `emmGrid` objects: ["utilities"](utilities.html)
  * Adding **emmeans** support to your package: ["xtending"](xtending.html)
  * Explanations of some unusual aspects of **emmeans**: ["xplanations"](xplanations.html)
    and some custom variations on compact letter displays: 
    ["re-engineering-clds"](re-engineering-clds.html)
  * Frequently asked questions: ["FAQs"](FAQs.html)


[Back to Contents](#contents)

[Index of all vignette topics](vignette-topics.html)

--- 
title: "Working with messy data" 
author: "emmeans package, Version `r packageVersion('emmeans')`"
output: emmeans::.emm_vignette
vignette: > 
  %\VignetteIndexEntry{Working with messy data} 
  %\VignetteEngine{knitr::rmarkdown} 
  %\VignetteEncoding{UTF-8}
--- 
```{r, echo = FALSE, results = "hide", message = FALSE} 
require("emmeans")
require("ggplot2")
options(show.signif.stars = FALSE) 
knitr::opts_chunk$set(fig.width = 4.5, class.output = "ro") 
```
<!-- @index Vignettes!Messy data -->

## Contents {#contents}

  1. [Issues with observational data](#issues) 
  2. [Mediating covariates](#mediators) 
  3. [Mediating factors and weights](#weights)
  3. [Nuisance factors](#nuisance)
  3. [Counterfactuals and G-Computation](#counterfact)
  4. [Sub-models](#submodels)
  5. [Nested fixed effects](#nesting)
     a. [Avoiding mis-identified nesting](#nest-trap)

[Index of all vignette topics](vignette-topics.html)

## Issues with observational data {#issues}
<!-- @index Observational data -->
In experiments, we control the
conditions under which observations are made. Ideally, this leads to balanced
datasets and clear inferences about the effects of those experimental
conditions. In observational data, factor levels are observed rather than
controlled, and in the analysis we control *for* those factors and covariates.
It is possible that some factors and covariates lie in the causal path for other
predictors. Observational studies can be designed in ways to mitigate some of
these issues; but often we are left with a mess. Using EMMs does not solve the
inherent problems in messy, undesigned studies; but they do give us ways to
compensate for imbalance in the data, and allow us to estimate meaningful
effects after carefully considering the ways in which they can be confounded.

####### {#nutrex}
<!-- @index Examples!`nutrition` -->
As an illustration, consider the `nutrition` dataset provided with the package. 
These data are used as an example in Milliken and Johnson (1992), *Analysis of 
Messy Data*, and contain the results of an observational study on nutrition 
education. Low-income mothers are classified by race, age category, and whether 
or not they received food stamps (the `group` factor); and the response variable
is a gain score (post minus pre scores) after completing a nutrition training 
program. First, let's fit a model than includes all main effects and 2-way 
interactions, and obtain its "type II" ANOVA:
```{r}
nutr.lm <- lm(gain ~ (age + group + race)^2, data = nutrition) 
car::Anova(nutr.lm)
```
 There is definitely a
`group` effect and a hint of and interaction with `race`. Here are the EMMs for
those two factors, along with their counts:
```{r}
emmeans(nutr.lm, ~ group * race, calc = c(n = ".wgt."))
```

####### {#nonest}
<!-- @index Non-estimability; Estimability -->
Hmmmm. The EMMs
when `race` is "Hispanic" are not given; instead they are flagged as 
non-estimable. What does that mean? Well, when using a model to make
predictions, it is impossible to do that beyond the linear space of the data
used to fit the model. And we have no data for three of the age groups in the
Hispanic population:
```{r}
with(nutrition, table(race, age))
```
 We can't make
predictions for all the cases we are averaging over in the above EMMs, and that
is why some of them are non-estimable. The bottom line is that we simply cannot
include Hispanics in the mix when comparing factor effects. That's a limitation
of this study that cannot be overcome without collecting additional data. Our
choices for further analysis are to focus only on Black and White populations;
or to focus only on age group 3. For example (the latter):
```{r}
emmeans(nutr.lm, pairwise ~ group | race, at = list(age = "3")) |>
    summary(by = NULL)
```
 (We used trickery with providing a `by` variable, and then taking it
away, to make the output more compact.) Evidently, the training program has been
beneficial to the Black and White groups in that age category. There is no 
conclusion for the Hispanic group -- for which we have very little data.

[Back to Contents](#contents)

## Mediating covariates {#mediators} 
<!-- @index Mediating covariates; Examples!`framing`; Covariates!Mediating -->
The `framing` data in the **mediation**
package has the results of an experiment conducted by Brader et al. (2008) where
subjects were given the opportunity to send a message to Congress regarding
immigration. However, before being offered this, some subjects (`treat = 1`)
were first shown a news story that portrays Latinos in a negative way. Besides
the binary response (whether or not they elected to send a message), the
experimenters also measured `emo`, the subjects' emotional state after the
treatment was applied. There are various demographic variables as well. Let's a
logistic regression model, after changing the labels for `educ` to shorter
strings. 
```{r}
framing <- mediation::framing 
levels(framing$educ) <- c("NA","Ref","< HS", "HS", "> HS","Coll +") 
framing.glm <- glm(cong_mesg ~ age + income + educ + emo + gender * factor(treat), 
    family = binomial, data = framing)
```
 
The conventional way to handle covariates like `emo` is to set them at their
means and use those means for purposes of predictions and EMMs. These adjusted 
means are shown in the following plot. 
```{r fig.alt = "Two panels, each showing a decreasing trend with educ except they increase again with educ=Coll+. In the male panel, the curve for treat 1 is higher than that for treat 2, while the reverse is true in the female panel"}
emmip(framing.glm, treat ~ educ | gender, type = "response") 
```


This plot gives the impression that the effect of `treat` is reversed between 
male and female subjects; and also that the effect of education is not monotone.
Both of these are counter-intuitive.

###### {#med.covred}
<!-- @index `cov.reduce` -->
However, note that the covariate `emo` is measured *post*-treatment. That 
suggests that in fact `treat` (and perhaps other factors) could affect the value
of `emo`; and if that is true (as is in fact established by mediation analysis 
techniques), we should not pretend that `emo` can be set independently of 
`treat` as was done to obtain the EMMs shown above. Instead, let `emo` depend on
`treat` and the other predictors -- easily done using `cov.reduce` -- and we 
obtain an entirely different impression:
```{r fig.alt = "Similar to previous figure but now the curves flatten out with >HS about the same as Coll+. For the maile panel, we still have the trace for treat 1 higher than for treat 0; but in the female panel, they are about the same as each other, and a bit lower than treat 0 for males. To see these results in numerical form, call emmeans() with the same arguments *except* replace the second argument with ~treat*educ|gender"}
emmip(framing.glm, treat ~ educ | gender, type = "response", 
    cov.reduce = emo ~ treat*gender + age + educ + income)
```

The reference grid underlying this plot has different `emo` values for each 
factor combination. The plot suggests that, after taking emotional response into
account, male (but not female) subjects exposed to the negative news story are 
more likely to send the message than are females or those not seeing the 
negative news story. Also, the effect of `educ` is now nearly monotone.

###### {#adjcov}
<!-- @index Covariates!Adjusted -->
By the way, the results in this plot are the same is what you would obtain by
refitting the model with an adjusted covariate
```{r eval = FALSE} 
emo.adj <- resid(lm(emo ~ treat*gender + age + educ + income, data = framing))
```
... and then using ordinary covariate-adjusted means at the means of `emo.adj`. 
This is a technique that is often recommended.

If there is more than one mediating covariate, their settings may be defined in
sequence; for example, if `x1`, `x2`, and `x3` are all mediating covariates, we
might use
```{r eval = FALSE} 
emmeans(..., cov.reduce = list(x1 ~ trt, x2 ~ trt + x1, x3 ~ trt + x1 + x2))
```
 (or possibly with some interactions included as well).

[Back to Contents](#contents)



## Mediating factors and weights {#weights}
<!-- @index Factors!Mediating; `weights`; `emmeans()`!`weights`
            Examples!`nutrition`!`weights` -->
A mediating covariate is one that is
in the causal path; likewise, it is possible to have a mediating *factor*. For
mediating factors, the moral equivalent of the `cov.reduce` technique described
above is to use *weighted* averages in lieu of equally-weighted ones in
computing EMMs. The weights used in these averages should depend on the
frequencies of mediating factor(s). Usually, the `"cells"` weighting scheme
described later in this section is the right approach. In complex situations, it
may be necessary to compute EMMs in stages.

As described in [the "basics" vignette](basics.html#emmeans), EMMs are usually 
defined as *equally-weighted* means of reference-grid predictions. However, 
there are several built-in alternative weighting schemes that are available by 
specifying a character value for `weights` in a call to `emmeans()` or related 
function. The options are `"equal"` (the default), `"proportional"`, `"outer"`,
`"cells"`, and `"flat"`.

The `"proportional"` (or `"prop"` for short) method weights proportionally to
the frequencies (or model weights) of each factor combination that is averaged
over. The `"outer"` method uses the outer product of the marginal frequencies of
each factor that is being averaged over. To explain the distinction, suppose the
EMMs for `A` involve averaging over two factors `B` and `C`. With `"prop"`, we
use the frequencies for each combination of `B` and `C`; whereas for `"outer"`,
first obtain the marginal frequencies for `B` and for `C` and weight
proportionally to the product of these for each combination of `B` and `C`. The
latter weights are like the "expected" counts used in a chi-square test for
independence. Put another way, outer weighting is the same as proportional
weighting applied one factor at a time; the following two would yield the same
results: 
```{r eval = FALSE} 
emmeans(model, "A", weights = "outer") 
emmeans(model, c("A", "B"), weights = "prop") |>  emmeans(weights = "prop") 
```


Using `"cells"` weights gives each prediction the same weight as occurs in the
model; applied to a reference grid for a model with all interactions,
`"cells"`-weighted EMMs are the same as the ordinary marginal means of the data.
With `"flat"` weights, equal weights are used, except zero weight is applied to
any factor combination having no data. Usually, `"cells"` or `"flat"` weighting
will *not* produce non-estimable results, because we exclude empty cells. (That
said, if covariates are linearly dependent with factors, we may still encounter
non-estimable cases.)

Here is a comparison of predictions for `nutr.lm` defined [above](#issues),
using different weighting schemes:
```{r message = FALSE}
sapply(c("equal", "prop", "outer", "cells", "flat"), \(w)
    emmeans(nutr.lm, ~ race, weights = w) |> predict())
```
In the other hand, if we do `group * race` EMMs, only one factor (`age`) is
averaged over; thus, the results for `"prop"` and `"outer"` weights will be
identical in that case.

[Back to Contents](#contents)


## Nuisance factors {#nuisance}
<!-- @index Nuisance factors; `nuisance`; `non.nuisance`; `wt.nuis`;
     Memory usage; Large models; Models!Large -->
Consider a situation where we have a model with 15 factors, each at 5 levels.
Regardless of how simple or complex the model is, the reference grid consists of
all combinations of these factors -- and there are $5^{15}$ of these, or over 30
billion. If there are, say, 100 regression coefficients in the model, then just
the `linfct` slot in the reference grid requires $100\times5^{15}\times8$ bytes of
storage, or almost 23,000 gigabytes. Suppose in addition the model has a
multivariate response with 5 levels. That multiplies *both* the rows and columns
in `linfct`, increasing the storage requirements by a factor of 25. Either way,
your computer can't store that much -- so this definitely qualifies as a messy
situation!

The `ref_grid()` function now provides some relief, in the way of specifying
some of the factors as "nuisance" factors. The reference grid is then
constructed with those factors already averaged-out. So, for example with the
same scenario, if only three of those 15 factors are of primary interest, and we
specify the other 12 as nuisance factors to be averaged, that leaves us with
only $3^5=125$ rows in the reference grid, and hence
$125\times100\times8=10,000$ bytes of storage required for `linfct`. If there is
a 5-level multivariate response, we'll have 625 rows in the reference grid and
$25\times1000=250,000$ bytes in `linfct`. Suddenly a horribly unmanageable
situation becomes quite manageable!

But of course, there is a restriction: nuisance factors must not interact with
any other factors -- not even other nuisance factors. And a multivariate
response (or an implied multivariate response, e.g., in an ordinal model) can
never be a nuisance factor. Under that condition, the average effects of a
nuisance factor are the same regardless of the levels of other factors, making
it possible to pre-average them by considering just one case. 

We specify nuisance factors by listing their names in a `nuisance` argument to 
`ref_grid()` (in `emmeans()`, this argument is passed to `ref_grid)`). Often,
it is much more convenient to give the factors that are *not* nuisance factors, 
via a `non.nuisance` argument. If you do specify
a nuisance factor that does interact with others, or doesn't exist, it is 
quietly excluded from the nuisance list. 

###### {#nuis.example}
<!-- @index Examples!`mtcars` -->
Time for an example. Consider the `mtcars` dataset standard in R, and the model
```{r}
mtcars.lm <- lm(mpg ~ factor(cyl)*am + disp + hp + drat + log(wt) + vs + 
                  factor(gear) + factor(carb), data = mtcars)
```
And let's construct two different reference grids:
```{r}
rg.usual <- ref_grid(mtcars.lm)
rg.usual
nrow(linfct(rg.usual))
rg.nuis = ref_grid(mtcars.lm, non.nuisance = "cyl")
rg.nuis
nrow(linfct(rg.nuis))
```
Notice that we left `am` out of `non.nuisance` and hence included it in `nuisance`.
However, it interacts with `cyl`, so it was not allowed as a nuisance factor.
But `rg.nuis` requires 1/36 as much storage.
There's really nothing else to show, other than to demonstrate that we get the same EMMs
either way, with slightly different annotations:
```{r}
emmeans(rg.usual, ~ cyl * am)
emmeans(rg.nuis, ~ cyl * am)
```

By default, the pre-averaging is done with equal weights. If we specify
`wt.nuis` as anything other than `"equal"`, they are averaged proportionally. As
described above, this really amounts to `"outer"` weights since they are
averaged separately. Let's try it to see how the estimates differ:
```{r}
predict(emmeans(mtcars.lm, ~ cyl * am, non.nuis = c("cyl", "am"), 
                wt.nuis = "prop"))
predict(emmeans(mtcars.lm, ~ cyl * am, weights = "outer"))
```
These are the same as each other, but different from the equally-weighted EMMs
we obtained before. By the way, to help make things consistent, if `weights` is
character, `emmeans()` passes `wt.nuis = weights` to `ref_grid` (if it is
called), unless `wt.nuis` is also specified.

There is a trick to get `emmeans` to use the smallest possible reference grid: 
Pass the `specs` argument to `ref_grid()` as `non.nuisance`. 
But we have to quote it
to delay evaluation, and also use `all.vars()` if (and only if)
`specs` is a formula:
```{r}
emmeans(mtcars.lm, ~ gear | am, non.nuis = quote(all.vars(specs)))
```
Observe that `cyl` was passed over as a nuisance factor because it interacts with
another factor.

### Limiting the size of the reference grid {#rg.limit}
<!-- @index  `rg.limit` option; Memory usage!Limiting -->

We have just seen how easily the size of a reference grid can get out of hand.
The `rg.limit` option (set via `emm_options()` or as an optional argument in `ref_grid()` or `emmeans()`) serves to guard against excessive memory demands. It specifies the number
of allowed rows in the reference grid. But because of the way `ref_grid()` works, this 
check is made *before* any multivariate-response levels are taken into account.
If the limit is exceeded, an error is thrown:
```{r, error = TRUE}
ref_grid(mtcars.lm, rg.limit = 200)
```
The default `rg.limit` is 10,000. With this limit, and if we have 1,000 columns 
in the model matrix, then the size of `linfct` is limited to about 80MB. 
If in addition, there is a 5-level multivariate response, the limit is 2GB --
darn big, but perhaps manageable. Even so, I suspect that the 10000-row default may be
to loose to guard against some users getting into a tight situation.


[Back to Contents](#contents)


## Counterfactuals and G-Computation {#counterfact}
<!-- @index Counterfactuals; Causal inference; G-Computation; -->
G-computation is a method for model-based causal inference originated by JM Robins
(*Mathematical Modelling*, 1986), and we want to remove confounding of treatment effects due to time-varying covariates and such.
The idea is that, under certain assumptions, we can use the model to predict 
*every* subject's response to each treatment -- not just the treatment they 
received. To do this, we make several copies of the whole dataset, substituting
the actual treatment(s) with each of the possible treatment levels; these
provide us with *counterfactual* predictions. We then average those predictions
over each copy of the dataset. Typically, this averaging is done on the response 
scale; that is the interesting case because on the link scale, everything is linear
and we can obtain basically the same results using ordinary `emmeans()` computations
with proportional weights.

An additional consideration is that when we average each of the counterfactual
datasets, we are trying to represent the entire covariate distribution, rather
than conditioning on the cases in the dataset. So it is a good idea to broaden
the covariance estimate using, say, a sandwich estimate.

This kind of computation has just a little bit in common with nuisance variables,
in that the net result is that we can sweep several predictors out of the 
reference grid just by averaging them away. For this to make sense, the predictors
averaged-away will have been observed *before* treatment so that their effects are 
separate from the treatment effects.

The implementation of this in **emmeans** is via the `counterfactuals` argument
in `ref_grid()` (but usually passed from `emmeans()`). We simply specify the
factor(s) we want to keep. This creates an index variable `.obs.no.` to keep
track of the observations in the dataset, and then the reference grid (before
averaging) consists of every observation of the dataset in combination with the
`counterfactuals` combinations.

##### {#neuralgia}
<!-- @index `vcov.`; Sandwich estimators;
    Examples!`neuralgia`!`counterfactuals` -->
As an example, consider the `neuralgia` data, where we have a binary response,
pain, a treatment of interest (two active treatments and placebo), and
pre-treatment predictors of sex, age, and duration of the condition. 
We will include the `vcovHC()` covariance estimate in the **sandwich** package.
```{r}
neuralgia.glm <- glm(Pain ~ Sex + Age + Duration + Treatment,
                     data = neuralgia, family = binomial)
emmeans(neuralgia.glm, "Treatment", counterfactuals = "Treatment",
        vcov. = sandwich::vcovHC)
```
Note that the results are already on the response (probability) scale,
which is the default. Let's compare this with what we get without using counterfactuals (i.e., predicting at each covariate average):
```{r}
emmeans(neuralgia.glm, "Treatment", weights = "prop", type = "response")
```
These results are markedly different; the counterfactual method produces
smaller differences between each of the active treatments and placebo.

[Back to Contents](#contents)


## Sub-models {#submodels}
<!-- @index `submodel`; Models!Constrained 
             Examples!`nutrition`!`submodel` -->
We have just seen that we can assign different weights to the levels of containing
factors. Another option is to constrain the effects of those containing factors
to zero. In essence, that means fitting a different model without those containing
effects; however, for certain models (not all), an `emmGrid` may be updated
with a `submodel` specification so as to impose such a constraint. For illustration,
return again to the nutrition example, and consider the analysis of `group`
and `race` as before, after removing interactions involving `age`:
```{r}
emmeans(nutr.lm, pairwise ~ group | race, submodel = ~ age + group*race) |> 
        summary(by = NULL)
```
If you like, you may confirm that we would obtain exactly the same
estimates if we had fitted that sub-model to the data, except we continue
to use the residual variance from the full model in tests and confidence intervals.
Without the interactions with `age`, all of the marginal means become estimable.
The results are somewhat different from those obtained earlier where
we narrowed the scope to just age 3. These new estimates include all ages,
averaging over them equally, but with constraints that the interaction effects
involving `age` are all zero.

###### {#type2submodel}
<!-- @index `submodel`!`"minimal"` and `"type2"`; Type II analysis
     `joint_tests()`!with `submodel = "type2"`  -->
There are two special character values that may be used with `submodel`.
Specifying `"minimal"` creates a submodel with only the active factors:
```{r}
emmeans(nutr.lm, ~ group * race, submodel = "minimal")
```
This submodel constrains all effects involving `age` to be zero.
Another interesting option is `"type2"`, whereby we in essence analyze the residuals
of the model with all contained or overlapping effects, then constrain the
containing effects to be zero. So what is left if only the interaction
effects of the factors involved. This is most useful with `joint_tests()`:
```{r}
joint_tests(nutr.lm, submodel = "type2")
```
These results are identical to the type II anova obtained 
[at the beginning of this example](#nutrex).

More details on how `submodel` works may be found in 
[`vignette("xplanations")`](xplanations.html#submodels)



[Back to Contents](#contents)


## Nested fixed effects {#nesting}
<!-- @index Nesting -->
A factor `A` is nested in another factor `B` if the levels of `A` have a
different meaning in one level of `B` than in another. Often, nested factors are
random effects---for example, subjects in an experiment may be randomly assigned
to treatments, in which case subjects are nested in treatments---and if we model
them as random effects, these random nested effects are not among the fixed
effects and are not an issue to `emmeans`. But sometimes we have fixed nested
factors.

###### {#cows}
<!-- @index Examples!`cows`; Examples!Nested fixed effects -->
Here is an example of a fictional study of five fictional treatments for some
disease in cows. Two of the treatments are administered by injection, and the
other three are administered orally. There are varying numbers of observations
for each drug. The data and model follow:
```{r}
cows <- data.frame (
    route = factor(rep(c("injection", "oral"), c(5, 9))),
    drug = factor(rep(c("Bovineumab", "Charloisazepam", 
              "Angustatin", "Herefordmycin", "Mollycoddle"), c(3,2,  4,2,3))),
    resp = c(34, 35, 34,   44, 43,      36, 33, 36, 32,   26, 25,   25, 24, 24)
)
cows.lm <- lm(resp ~ route + drug, data = cows)
```
The `ref_grid` function finds a nested structure in this model:
```{r message = FALSE}
cows.rg <- ref_grid(cows.lm)
cows.rg
```

When there is nesting, `emmeans` computes averages separately in each group\ldots
```{r}
route.emm <- emmeans(cows.rg, "route")
route.emm
```
... and insists on carrying along any grouping factors that a factor is nested in:
```{r}
drug.emm <- emmeans(cows.rg, "drug")
drug.emm
```
Here are the associated pairwise comparisons:
```{r}
pairs(route.emm, reverse = TRUE)

pairs(drug.emm, by = "route", reverse = TRUE)
```
In the latter result, the contrast itself becomes a nested factor in the
returned `emmGrid` object. That would not be the case if there had been no `by`
variable.

#### Graphs with nesting
<!-- @index `emmip()`!nested factors; `plot()`!nested factors; **ggplot2** package -->
It can be very helpful to take advantage of special features of **ggplot2** when
graphing results with nested factors. For example, the default plot for the `cows` example
is not ideal:
```{r, fig.width = 5.5, fig.alt = "A panel for each route. This interaction plot has a lot of empty space because all 5 drugs are represented in each panel, and the x axis labels all overlap"}
emmip(cows.rg, ~ drug | route)
```

We can instead remove `route` from the call and instead handle it with **ggplot2**
code to use separate *x* scales:
```{r, fig.width = 5.5, fig.alt = "This plot shows the same means as the previous one, but each panel shows only the drugs that are nested in each route"}
require(ggplot2)
emmip(cows.rg, ~ drug) + facet_wrap(~ route, scales = "free_x")
```

Similarly with `plot.emmGrid()`:
```{r, fig.height = 2.5, fig.width = 5.5, fig.alt = "side-by-side CIs and PIs for drugs in each route. Again, with free_y scaling, each panel contains only the drugs involved"}
plot(drug.emm, PIs = TRUE) + 
    facet_wrap(~ route, nrow = 2, scales = "free_y")
```


### Auto-identification of nested factors -- avoid being trapped! {#nest-trap}
<!-- @index Nesting!Auto-detection; `ref_grid()`!`nesting` -->
`ref_grid()` and `emmeans()` tries to discover
and accommodate nested structures in the fixed effects. It does this in two
ways: first, by identifying factors whose levels appear in combination with only
one level of another factor; and second, by examining the `terms` attribute of
the fixed effects. In the latter approach, if an interaction `A:B` appears
in the model but `A` is not present as a main effect, then `A` is deemed to
be nested in `B`. Note that this can create a trap: some users take shortcuts
by omitting some fixed effects, knowing that this won't affect the fitted
values. But such shortcuts *do* affect the interpretation of model parameters,
ANOVA tables, etc., and I advise against ever taking such shortcuts. 
Here are some ways you may notice mistakenly-identified nesting:

  * A message is displayed when nesting is detected
  * A `str()` listing of the `emmGrid` object shows a nesting component
  * An `emmeans()` summary unexpectedly includes one or more factors
    that you didn't specify
  * EMMs obtained using `by` factors don't seem to behave right, or
    give the same results with different specifications

To override the auto-detection of nested effects, use the `nesting` argument
in `ref_grid()` or `emmeans()`. Specifying `nesting = NULL` will ignore
all nesting. Incorrectly-discovered nesting can be overcome by specifying 
something akin to `nesting = "A %in% B, C %in% (A * B)"` or, equivalently,
`nesting = list(A = "B",  C = c("A", "B"))`.


[Back to Contents](#contents)

[Index of all vignette topics](vignette-topics.html)

---
title: "Quick start guide for **emmeans**"
author: "emmeans package, Version `r packageVersion('emmeans')`"
output: emmeans::.emm_vignette
vignette: >
  %\VignetteIndexEntry{A quick-start guide for emmeans}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
```{r, echo = FALSE, results = "hide", message = FALSE}
require("emmeans")
knitr::opts_chunk$set(fig.width = 4.5, class.output = "ro")
```

<!-- @index Quick-start guide; **emmeans** package!Quick-start guide -->

## Contents {#contents}
  1. [The three basic steps](#steps)
  2. [One-factor model](#one-factor)
  3. [Two factors, no interaction](#additive)
  4. [Two interacting factors](#interactions)
  5. [Three or more factors](#multi-factor)
  6. [Additional options](#options)
  7. [What you see versus what you get](#objects)
  8. [Common things that can go wrong](#problems)
  9. [Further reading](#more)


## The three basic steps {#steps}
Much of what you do with the **emmeans** package involves these three basic steps:

  0. Fit a *good* model to your data, and do reasonable checks to make sure
     it adequately explains the respons(es) and reasonably meets underlying 
     statistical assumptions. Modeling is not the focus of **emmeans**, but this
     is an extremely important step because **emmeans** does not analyze your data,
     it summarizes your model. If it is a bad model, you will likely get 
     misleading results from this package -- the garbage in, garbage out principle.
     If you're not sure whether your model is any good, this is a good time
     to get statistical consulting help. This really *is* like rocket science, not
     just a matter of getting programs to run.
  1. Run  `EMM <- emmeans(...)` (see scenarios below) to obtain estimates of 
     means or marginal means
  2. Run `contrast(EMM, ...)` or `pairs(EMM)` one or more times to obtain
     estimates of contrasts or pairwise comparisons among the means.

**Note:** A lot of users have developed the habit of running something like 
`emmeans(model, pairwise ~ factor(s))`, which conflates steps 1 and 2. We recommend
*against* doing this because it often yields output you don't want or need -- especially
when there is more than one factor. You are better off keeping steps 1 and 2 separate.
What you do in step 2 depends on how many factors you have, and how they relate.
     
### One-factor model {#one-factor}
If a one-factor model fits well and the factor is named `treatment`, do
```r
EMM <- emmeans(model, "treatment")   # or emmeans(model, ~ treatment)
EMM    # display the means

### pairwise comparisons
contrast(EMM, "pairwise")    # or pairs(EMM)
```
You may specify other contrasts in the second argument of the `contrast()` call, e.g. `"trt.vs.ctrl", ref = 1` (compare each mean to the first),
or `"consec"` (compare 2 vs 1, 3 vs 2, etc.), or `"poly", max.degree = 3` (polynomial contrasts)

### Two factors, no interaction {#additive}
If the model fits well and factors are named `treat` and `dose`, and they don't interact, follow 
the same steps as for one factor at a time.
That is, something like
```
(EMM1 <- emmeans(model, ~ treat))
pairs(EMM1)

(EMM2 <- emmeans(model, ~ dose))
pairs(EMM2)
```
These analyses will yield the estimated *marginal* means for
each factor, and comparisons/contrasts thereof.


[Back to Contents](#contents)

### Two interacting factors {#interactions}
In this case, unless the interaction effect is negligible, we usually want to 
do "simple comparisons" of the cell means. That is, compare
or contrast the means separately, holding one factor fixed at each level.
```r
EMM <- emmeans(model, ~ treat * dose)
EMM    # display the cell means

### Simple pairwise comparisons...
pairs(EMM, simple = "treat")    # compare treats for each dose -- "simple effects"
pairs(EMM, simple = "dose")     # compare doses for each treat
```
The default is to apply a separate Tukey adjustment to the *P* values in each
`by` group (so if each group has just 2 means, no adjustment at all is applied).
If you want to adjust the whole family combined, you need to undo the `by` variable
and specify the desired adjustment (which *can't* be Tukey because that method
is invalid when you have more than one set of pairwise comparisons.) For example
```r
test(pairs(EMM, by = "dose"), by = NULL, adjust = "mvt")
```
#### Diagonal comparisons
If the "diagonal" comparisons (where *both* factors differ) are of
interest, you would do `pairs(EMM)` without a `by` variable. But you get a lot more
comparisons this way.

#### Interaction contrasts
Sometimes you may want to examine *interaction contrasts*, which are contrasts of contrasts. The thing to know here is that `contrast()` or (`pairs()`) creates the same kind of object as `emmeans()`, so you can run them multiple times. For example,
```r
CON <- pairs(EMM, by = "dose")
contrast(CON, "consec", by = NULL)    # by = NULL is essential here!
```
Or equivalently, the named argument `interaction` can be used
```r
contrast(EMM, interaction = c("pairwise", "consec"))
```

### Three or more factors {#multi-factor}
After you have mastered the strategies for two factors, you can adapt them to
three or more factors as appropriate, based on how they interact and what you need.

[Back to Contents](#contents)


## Additional options {#options}
  1. See the help files for *both* `emmeans()` and `ref_grid()` for additional
     arguments that may prove useful. Many of the most useful arguments are
     passed to `ref_grid()`.
  2. There are a number of vignettes provided with the package that include
     examples and discussions for different kinds of situations. There is also an 
     [index of vignette topics](vignette-topics.html).

## What you see versus what you get {#objects}
Most non-graphical functions in the **emmeans** package produce one of two classes of objects. The functions `emmeans()`, `emtrends()`, `ref_grid()`, `contrast()`, and `pairs()` return `emmGrid` objects (or lists thereof, class `emm_list`). For example
```
EMM <- emmeans(mod, "Treatment")
```
The functions `summary()`, `confint()`, `test()`, `joint_tests()`, and others
return `summary_emm` objects (or lists thereof, class `summary_eml`):
```
SEMM <- summary(EMM)
```
If you display `EMM` and `SEMM`, they *look* identical; that's because `emmGrid` objects are displayed using `summary()`. But they are not identical. `EMM` has all the ingredients needed to do further analysis, e.g. `contrast(EMM, "consec")` will estimate comparisons between consecutive `Treatment` means. But `SEMM` is just an annotated data frame and we can do no further analysis with it. Similarly, we can change how `EMM` is displayed via arguments to `summary()` or relatives, while in `SEMM`, everything has been computed and those results are locked-in.


## Common things that can go wrong {#problems}

### Only one mean is obtained -- or fewer than expected {#covar}
<!-- @index Only one mean; Too few means -->
This is probably the most common issue, and it can happen when a treatment is coded as a 
**numeric predictor** rather than a factor. Instead of getting a mean for each 
treatment, you get a mean at the average of those numerical values. 

  1. In such cases, the model is often inappropriate; you should replace 
     `treatment` with `factor(treatment)` and re-fit the model.
  2. In a situation where it *is* appropriate to consider the treatment as a
     quantitative predictor, then you can get separate means at specified
     values by adding an argument like `at = list(treatment = c(3,5,7))`
     to the `emmeans()` call.
  3. When you have a numerical predictor interacting with a factor, it may be
     useful to estimate its slope for each level of that factor. See the documentation
     for `emtrends()`
     
### Having trouble with follow-up analyses, and the `pairwise ~ ...` recipe {#pairwise}
<!-- @index `pairwise ~ factors`!Why you should avoid this -->
The basic object returned by `emmeans()` and `contrast()` is of class `emmGrid`,
and additional `emmeans()` and `contrast()` calls can accept `emmGrid` objects.
However, some options create *lists* of `emmGrid` objects, and that makes things 
a bit confusing. The most common case is using a call like 
`emmeans(model, pairwise ~ treat * dose)`, which computes the means *and* 
all pairwise comparisons -- a list of two `emmGrid`s. If you try to obtain additional
contrasts, say, of this result, `contrast()` makes a guess that you want to
run it on just the first element. 

This causes confusion (I know, because I get a lot of questions about it).
I recommend that you avoid using the `pairwise ~` construct altogether: 
Get your means in one step, and get your contrasts in separate step(s).
The `pairwise ~` construct is generally useful if you have only one factor; 
otherwise, it likely gives you results you don't want.

## Further reading {#more}
There are several of these vignettes that offer more details and 
more advanced topics. [An index of all these vignette topics is available here](vignette-topics.html).

The strings linked below are the names of the vignettes; i.e., they can
also be accessed via `vignette("`*name*`", "emmeans")`

  * Models that are supported in **emmeans** (there are lots of them)
    ["models"](models.html)
  * Basic ideas that underlie estimated marginal means (EMMs):
    ["basics"](basics.html). These concepts
    emphasize *experimental* data, as distinct from *observational* studies.
  * Confidence intervals and tests: 
    ["confidence-intervals"](confidence-intervals.html)
  * Often, users want to compare or contrast EMMs: ["comparisons"](comparisons.html)
  * Working with response transformations and link functions:
    ["transformations"](transformations.html)
  * Multi-factor models with interactions: ["interactions"](interactions.html)
  * Making predictions from your model: ["predictions"](predictions.html)
  * Examples of more sophisticated models (e.g., mixed, ordinal, MCMC)
    ["sophisticated"](sophisticated.html)
  * Working with messy data, counterfactuals, mediating covariates, 
    and nested effects: 
    ["messy-data"](messy-data.html). Here is where you may see more on
    how **emmeans** might help with observational data.
  * Utilities for working with `emmGrid` objects: ["utilities"](utilities.html)
  * Adding **emmeans** support to your package: ["xtending"](xtending.html)
  * Explanations of some unusual aspects of **emmeans**: ["xplanations"](xplanations.html)
    and some custom variations on compact letter displays: 
    ["re-engineering-clds"](re-engineering-clds.html)
  * Frequently asked questions: ["FAQs"](FAQs.html)


[Back to Contents](#contents)

[Index of all vignette topics](vignette-topics.html)

---
title: "Index of vignette topics"
author: "emmeans package, Version `r packageVersion('emmeans')`"
output: emmeans::.emm_vignette
vignette: >
  %\VignetteIndexEntry{Index of vignette topics}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<div class="vigindex" id="Index follows this line">



### Jump to:  [A](#a) [B](#b) [C](#c) [D](#d) [E](#e) [F](#f) [G](#g) [H](#h) [I](#i) [J](#j) [K](#k) [L](#l) [M](#m) [N](#n) [O](#o) [P](#p) [Q](#q) [R](#r) [S](#s) [T](#t) [U](#u) [V](#v) [W](#w) [X](#x) [Z](#z)  {#topnav}




### A {#a}


  * [`add_grouping()`](utilities.html#groups)
  * [`addl.vars`](sophisticated.html#random-slopes)
  * `adjust`
      * [in *comparisons: pairwise*](comparisons.html#pairwise)
      * [in *confidence-intervals: adjust*](confidence-intervals.html#adjust)
      * [vs. `cross.adjust`](confidence-intervals.html#cross-adjust)
  * [Adjusted means](basics.html#RG)
  * [Adjusted R-squared](basics.html#pigs)
  * [`afex_aov` objects](models.html#V)
  * [Alias matrix](xplanations.html#submodels)
  * [Analysis of subsets of data](FAQs.html#model)
  * Analysis of variance
      * [versus *post hoc* comparisons](FAQs.html#anova)
      * [Type III](confidence-intervals.html#joint_tests)
  * [`aovList` objects](models.html#V)
  * [`appx-satterthwaite` method](models.html#K)
  * Arguments
      * [`...`](basics.html#arguments)
      * [Passed to `ref_grid()`](basics.html#arguments)
  * `as.mcmc()`
      * [in *models: S*](models.html#S)
      * [in *sophisticated: bayesxtra*](sophisticated.html#bayesxtra)
  * [ASA Statement on *P* values](basics.html#pvalues)
  * [Asymptotic tests](sophisticated.html#dfoptions)
  * [ATOM](basics.html#pvalues)
  * [`averaging` models](models.html#I)


[Back to top](#topnav)

### B {#b}


  * [Back-transforming](basics.html#transf)
  * [Balanced allocation](basics.html#exper)
  * [Bayes factor](sophisticated.html#bayesxtra)
  * Bayesian models
      * [in *models: S*](models.html#S)
      * [in *sophisticated: mcmc*](sophisticated.html#mcmc)
  * [**bayesplot** package](sophisticated.html#bayesxtra)
  * [**bayestestR** package](sophisticated.html#bayesxtra)
  * [Beta regression](models.html#B)
  * [`betareg` models](models.html#B)
  * Bias adjustment
      * [For link functions vs. response transformations](transformations.html#link-bias)
      * [in Bayesian models](sophisticated.html#bias-adj-mcmc)
      * [In GLMMs and GEE models](transformations.html#cbpp)
      * [When back-transforming](transformations.html#bias-adj)
      * [When *not* to use](transformations.html#insects)
      * [with random slopes](sophisticated.html#random-slopes)
  * [Bonferroni adjustment](confidence-intervals.html#adjmore)
  * [`boot-satterthwaite` method](models.html#K)
  * [Brackets (`[ ]` and `[[ ]]` operators)](utilities.html#brackets)
  * [`brmsfit` objects](models.html#S)
  * [`by` groups](confidence-intervals.html#byvars)
      * [Adjusting across groups](confidence-intervals.html#cross-adjust)
      * [Identical comparisons](FAQs.html#additive)


[Back to top](#topnav)

### C {#c}


  * [Causal inference](messy-data.html#counterfact)
  * [`cld()`](comparisons.html#CLD)
  * CLDs
      * [Alternative](re-engineering-clds.html#alt-CLDs)
      * [For equivalence sets](re-engineering-clds.html#equiv-CLDs)
      * [For significance sets](re-engineering-clds.html#signif-CLDs)
      * [**multcompView** package](re-engineering-clds.html#CLDs)
      * [Principles](re-engineering-clds.html#CLDs)
      * [Re-engineered](re-engineering-clds.html#alt-CLDs)
      * [References](re-engineering-clds.html#refs)
      * [Relation to underlining](re-engineering-clds.html#underlining)
      * [Simulated example](re-engineering-clds.html#CLD-example)
  * [`clm` models](models.html#O)
  * [**coda** package](sophisticated.html#bayesxtra)
  * [`coef()`](comparisons.html#contrasts)
  * [Cohen's *d*](comparisons.html#pwpm)
  * [Compact letter displays](comparisons.html#CLD)
  * [Compact letter displays (CLDs)](re-engineering-clds.html)
  * Comparison arrows
      * [Derivation](xplanations.html#arrows)
  * Comparisons
      * [Back-transforming](comparisons.html#logs)
      * [Displaying as groups](comparisons.html#CLD)
      * [Displaying *P* values](comparisons.html#pwpp)
      * [Graphical](comparisons.html#graphical)
          * [How arrows are determined](xplanations.html#arrows)
      * [with logs](comparisons.html#logs)
      * [Obtaining differences rather than ratios](FAQs.html#ratios)
      * [with overlapping CIs](FAQs.html#CIerror)
  * [Comparisons result in `(nothing)`](FAQs.html#nopairs)
  * [Confidence intervals](confidence-intervals.html#summary)
      * [Overlapping](FAQs.html#CIerror)
  * [`confint()`](confidence-intervals.html#summary)
  * [Confounded effects](xplanations.html#joint_tests)
  * [Confounding](basics.html#emms)
  * [`consec` contrasts](comparisons.html#contrasts)
  * [Constrained marginal means](xplanations.html#submodels)
  * [Consultants](basics.html#recs3)
  * [Containment d.f.](models.html#K)
  * [`contrast()`](comparisons.html#contrasts)
      * [`adjust`](comparisons.html#linfcns)
      * [Changing defaults](utilities.html#defaults)
      * [`combine`](interactions.html#simple)
      * [`interaction`](interactions.html#contrasts)
      * [Linear functions](comparisons.html#linfct)
      * `simple`
          * [in *confidence-intervals: simple*](confidence-intervals.html#simple)
          * [in *interactions: simple*](interactions.html#simple)
  * Contrasts
      * [of other contrasts](interactions.html#contrasts)
      * [Custom](comparisons.html#linfcns)
      * [Formula](comparisons.html#formulas)
      * [Multivariate](interactions.html#multiv)
      * [Pairwise](comparisons.html#pairwise)
      * [Polynomial](comparisons.html#contrasts)
      * Tests of
          * [with transformations](comparisons.html#logs)
  * [Controlled experiments](basics.html#exper)
  * [Count regression](models.html#C)
  * [Counterfactuals](messy-data.html#counterfact)
  * [`cov.reduce`](messy-data.html#med.covred)
  * Covariates
      * [Adjusted](messy-data.html#adjcov)
      * [`cov.keep`](FAQs.html#numeric)
      * [`cov.reduce`](FAQs.html#numeric)
      * [Derived](basics.html#depcovs)
      * [`emmeans()` doesn't work](FAQs.html#numeric)
      * [Interacting with factors](FAQs.html#trends)
      * [Mediating](messy-data.html#mediators)
  * [cross-group comparisons](confidence-intervals.html#cross-adjust)
  * [`cross.adjust`](confidence-intervals.html#cross-adjust)


[Back to top](#topnav)

### D {#d}


  * [Degrees of freedom](sophisticated.html#dfoptions)
      * [Infinite](FAQs.html#asymp)
  * Digits
      * [Displaying more digits](utilities.html#data)
  * [Digits, optimizing](utilities.html#digits)
  * [Dunnett method](comparisons.html#contrasts)


[Back to top](#topnav)

### E {#e}


  * [`eff` contrasts](comparisons.html#contrasts)
  * [`eff_size()`](comparisons.html#pwpm)
  * [Effect size](comparisons.html#pwpm)
  * [`emm_basis()`](xtending.html#intro)
      * [Arguments and returned value](xtending.html#ebreqs)
      * [Communicating with `recover_data()`](xtending.html#communic)
      * [Dispatching](xtending.html#dispatch)
      * [Hook functions](xtending.html#hooks)
      * [for `lqs` objects](xtending.html#eblqs)
      * [for `rsm` objects](xtending.html#ebrsm)
  * [`emm_list` object](comparisons.html#formulas)
  * [`emm_options()`](utilities.html#defaults)
  * [`.emmc` functions](comparisons.html#linfcns)
  * **emmeans** package
      * [Exporting extensions to](xtending.html#exporting)
      * [Quick-start guide](AQuickStart.html)
  * [`emmeans()`](basics.html#emms)
      * [And the underlying model](FAQs.html#nowelch)
      * [Arguments passed to `ref_grid()`](basics.html#arguments)
      * [Changing defaults](utilities.html#defaults)
      * [Fastest way to get wrong answers](FAQs.html#fastest)
      * [Surprising results from](FAQs.html#transformations)
      * `weights`
          * [in *basics: weights*](basics.html#weights)
          * [in *messy-data: weights*](messy-data.html#weights)
      * [With transformations](transformations.html#regrid)
  * [`emmGrid` objects](basics.html#emmobj)
      * [Accessing data](utilities.html#data)
      * [Combining and subsetting](utilities.html#rbind)
      * [Modifying](utilities.html#update)
      * [Setting defaults for](utilities.html#defaults)
  * `emmip()`
      * [in *basics: emmip*](basics.html#emmip)
      * [in *basics: plots*](basics.html#plots)
      * [in *interactions: factors*](interactions.html#factors)
      * [nested factors](messy-data.html#cows)
  * EMMs
      * [Projecting to a submodel](xplanations.html#submodels)
      * [What are they?](FAQs.html#what)
  * `emtrends()`
      * [in *interactions: covariates*](interactions.html#covariates)
      * [in *interactions: oranges*](interactions.html#oranges)
  * [Equivalence sets (with CLDs)](re-engineering-clds.html#equiv-CLDs)
  * [`estHook`](xtending.html#hooks)
  * [Estimability](messy-data.html#nonestex)
  * [Estimability issues](FAQs.html#NAs)
  * [Estimable functions](xplanations.html#joint_tests)
  * Estimated marginal means
      * [Definition of](basics.html#refgrid)
      * [Demonstrated](basics.html#emms)
      * [Manually computed](basics.html#refgrid)
      * [Reference grid](basics.html#emms)
      * [vs. ordinary marginal means](basics.html#emms)
  * Examples
      * [`auto.noise`](interactions.html#factors)
      * [Bayesian model](sophisticated.html#mcmc)
      * `cbpp`
          * [in *sophisticated: mcmc*](sophisticated.html#mcmc)
          * [in *transformations: cbpp*](transformations.html#cbpp)
      * [`ChickWeight`](sophisticated.html#random-slopes)
      * [CLDs (simulated)](re-engineering-clds.html#CLD-example)
      * [`cows`](messy-data.html#cows)
      * [`feedlot`](predictions.html#feedlot)
      * `fiber`
          * [in *interactions: covariates*](interactions.html#covariates)
          * [in *transformations: stdize*](transformations.html#stdize)
      * [`framing`](messy-data.html#mediators)
      * [Gamma regression](transformations.html#tranlink)
      * [`InsectSprays`](transformations.html#insects)
      * [`Insurance`](xplanations.html#offsets)
      * [Insurance claims (SAS)](sophisticated.html#offsets)
      * [Logistic regression](transformations.html#links)
      * [`lqs` objects](xtending.html#lqs)
      * [`MOats`](basics.html#multiv)
      * `mtcars`
          * [in *basics: depcovs*](basics.html#depcovs)
          * [in *messy-data: nuis.example*](messy-data.html#nuis.example)
      * [Multivariate](basics.html#multiv)
      * [Nested fixed effects](messy-data.html#cows)
      * `neuralgia`
          * [`counterfactuals`](messy-data.html#neuralgia)
          * [in *transformations: links*](transformations.html#links)
          * [in *transformations: trangraph*](transformations.html#trangraph)
      * [`nutrition`](messy-data.html#nutrex)
          * [`submodel`](messy-data.html#submodels)
          * [`weights`](messy-data.html#weights)
      * [`Oats`](sophisticated.html#lmer)
      * `oranges`
          * [in *comparisons: formulas*](comparisons.html#formulas)
          * [in *interactions: oranges*](interactions.html#oranges)
      * [Ordinal model](sophisticated.html#ordinal)
      * `pigs`
          * [in *basics: pigs*](basics.html#pigs)
          * [in *confidence-intervals: summary*](confidence-intervals.html#summary)
          * [in *transformations: altscale*](transformations.html#altscale)
          * [in *transformations: overview*](transformations.html#overview)
          * [in *transformations: pigs-biasadj*](transformations.html#pigs-biasadj)
      * [`rlm` objects](xtending.html#rlm)
      * [Robust regression](xtending.html#rlm)
      * [Split-plot experiment](sophisticated.html#lmer)
      * [Unbalanced data](basics.html#pigs)
      * `warpbreaks`
          * [in *confidence-intervals: cross-adjust*](confidence-intervals.html#cross-adjust)
          * [in *transformations: tranlink*](transformations.html#tranlink)
          * [in *utilities: relevel*](utilities.html#relevel)
      * [Welch's *t* comparisons](utilities.html#relevel)
      * [`wine`](sophisticated.html#ordinal)
  * Expected marginal means
      * [Foundations of](basics.html#found)
      * [Reliance on a model](basics.html#models)
  * [Experimental versus observational data](basics.html#exper)
  * [Exporting output](basics.html#formatting)
  * Extending **emmeans**
      * [Exports useful to developers](xtending.html#exported)
      * [Restrictions](xtending.html#dispatch)


[Back to top](#topnav)

### F {#f}


  * *F* test
      * [vs. pairwise comparisons](FAQs.html#anova)
      * [Role in *post hoc* tests](basics.html#recs1)
  * Factors
      * [Mediating](messy-data.html#weights)
  * [Formatting results](basics.html#formatting)
  * [Foundations of EMMs](basics.html#found)
  * [Frequently asked questions](FAQs.html)


[Back to top](#topnav)

### G {#g}


  * [G-Computation](messy-data.html#counterfact)
  * [`gam` models](models.html#G)
  * [`gamlss` models](models.html#H)
  * [GEE models](models.html#E)
  * [Generalized additive models](models.html#G)
  * [Generalized linear models](models.html#G)
  * [Geometric means](transformations.html#bias-adj)
  * [Get the model right first](FAQs.html#nowelch)
  * [`get_emm_option()`](utilities.html#options)
  * **ggplot2** package
      * [in *basics: ggplot*](basics.html#ggplot)
      * [in *messy-data: cows*](messy-data.html#cows)
  * [GIGO (garbage in, garbage out)](basics.html#models)
  * [`glm`*xxx* models](models.html#G)
  * [`gls` models](models.html#K)
  * [Graphical displays](basics.html#plots)
  * [Grouping factors](utilities.html#groups)
  * [Grouping into separate sets](confidence-intervals.html#byvars)


[Back to top](#topnav)

### H {#h}


  * [Hook functions](xtending.html#hooks)
  * [Hotelling's $T^2$](interactions.html#multiv)
  * [`hpd.summary()`](sophisticated.html#mcmc)
  * [`hurdle` models](models.html#C)


[Back to top](#topnav)

### I {#i}


  * [Indicator variables](basics.html#RG)
  * [Infinite degrees of freedom](FAQs.html#asymp)
  * [Interaction-style plots](basics.html#emmip)
  * Interactions
      * [Analysis](interactions.html)
      * [Contrasts](interactions.html#contrasts)
      * [Covariate with factors](interactions.html#covariates)
      * [Implied](interactions.html#oranges)
      * [Plotting](interactions.html#factors)
      * [Possible inappropriateness of marginal means](interactions.html#factors)
  * [`inverse()`](basics.html#pigs)


[Back to top](#topnav)

### J {#j}


  * [`joint`](confidence-intervals.html#joint)
  * `joint_tests()`
      * [in *confidence-intervals: joint_tests*](confidence-intervals.html#joint_tests)
      * [Confounded effects](xplanations.html#joint_tests)
      * [in *interactions: contrasts*](interactions.html#contrasts)
      * [with `submodel = "type2"`](messy-data.html#type2submodel)


[Back to top](#topnav)

### K {#k}


  * [`kable`](basics.html#formatting)
  * [Kenward-Roger d.f.](models.html#L)


[Back to top](#topnav)

### L {#l}


  * Labels
      * [Changing](utilities.html#relevel)
  * [Large models](messy-data.html#nuisance)
  * [Latin squares](basics.html#exper)
  * [Least-squares means](FAQs.html#what)
  * Levels
      * [Changing](utilities.html#relevel)
  * [Linear functions](comparisons.html#linfct)
  * [Link functions](transformations.html#links)
  * [`lme` models](models.html#K)
  * `lmerMod` models
      * [in *models: L*](models.html#L)
      * [in *sophisticated: lmer*](sophisticated.html#lmer)
      * [System options for](sophisticated.html#lmerOpts)
  * Logistic-like regression
      * [Non-logit links](transformations.html#not-logit)
  * Logistic regression
      * [Odds ratios](transformations.html#oddsrats)
      * [Risk ratios](transformations.html#riskrats)
      * [Surprising results](FAQs.html#transformations)
  * LSD
      * [protected](basics.html#recs2)


[Back to top](#topnav)

### M {#m}


  * [`make.tran()`](transformations.html#special)
  * [`mcmc` objects](models.html#S)
  * Means
      * [Generalized](transformations.html#bias-adj)
      * [Weighted](basics.html#weights)
  * [Mediating covariates](messy-data.html#mediators)
  * [Memory usage](messy-data.html#nuisance)
      * [Limiting](messy-data.html#rg.limit)
  * [`mira` models](models.html#I)
  * [`misc` attribute and argument](xtending.html#communic)
  * [Missing cells](xplanations.html#joint_tests)
  * [`mlm` models](models.html#N)
  * [`mmer` models](models.html#G)
  * Model
      * [Get it right first](basics.html#recs2)
      * Importance of
          * [in *basics: models*](basics.html#models)
          * [in *FAQs: fastest*](FAQs.html#fastest)
      * [Importance of getting it right](FAQs.html#nowelch)
  * [Model averaging](models.html#I)
  * Modeling
      * [`pigs` example](basics.html#pigs)
      * [Response transformation](basics.html#pigs)
  * [Modelling](basics.html#models)
  * Models
      * [Constrained](messy-data.html#submodels)
      * [Large](messy-data.html#nuisance)
      * [Quick reference](models.html#quickref)
      * [Unsupported](FAQs.html#qdrg)
  * [Multi-factor studies](FAQs.html#interactions)
  * [Multinomial models](models.html#N)
  * [Multiple imputation](models.html#I)
  * [Multiplicity adjustments](confidence-intervals.html#adjust)
  * [Multivariate contrasts](interactions.html#multiv)
  * Multivariate models
      * [in *basics: multiv*](basics.html#multiv)
      * [in *interactions: oranges*](interactions.html#oranges)
      * [in *models: M*](models.html#M)
      * [with `submodel`](xplanations.html#mult.submodel)
  * [Multivariate *t* (`"mvt"`) adjustment](confidence-intervals.html#adjmore)
  * [`mvcontrast()`](interactions.html#multiv)
  * [**mvtnorm** package](confidence-intervals.html#adjmore)


[Back to top](#topnav)

### N {#n}


  * [`NA`s in the output](FAQs.html#NAs)
  * [Nesting](messy-data.html#nesting)
      * [Auto-detection](messy-data.html#nest-trap)
  * Nesting factors
      * [Creating](utilities.html#groups)
  * [Non-estimability](messy-data.html#nonestex)
  * [`non.nuisance`](messy-data.html#nuisance)
  * [`NonEst` values](FAQs.html#NAs)
  * [`(nothing)` in output](FAQs.html#nopairs)
  * [`nuisance`](messy-data.html#nuisance)
  * [Nuisance factors](messy-data.html#nuisance)


[Back to top](#topnav)

### O {#o}


  * Observational data
      * [in *basics: weights*](basics.html#weights)
      * [in *messy-data: issues*](messy-data.html#issues)
  * [Observational versus experimental data](basics.html#exper)
  * [Odds ratios](transformations.html#oddsrats)
      * [but I wanted differences](FAQs.html#ratios)
  * Offsets
      * [Models with offsets](sophisticated.html#offsets)
      * [`offset()` term vs. `offset` argument in model](xplanations.html#offsets)
  * [Only one mean](AQuickStart.html#covar)
  * [`opt.digits` option](utilities.html#digits)
  * [Options](utilities.html#options)
      * [Startup](utilities.html#startup)
  * Ordinal models
      * [Latent scale](sophisticated.html#ordinal)
      * [Linear-predictor scale](sophisticated.html#ordlp)
      * [in *models: O*](models.html#O)
      * [`prob` and `mean.class`](sophisticated.html#ordprob)
      * [in *sophisticated: ordinal*](sophisticated.html#ordinal)


[Back to top](#topnav)

### P {#p}


  * *P* values
      * [Adjusted](basics.html#recs1)
      * [Adjustment is ignored](FAQs.html#noadjust)
      * [Interpreting](basics.html#pvalues)
  * [`pairs()`](comparisons.html#pairwise)
  * `pairwise ~ factors`
      * [Why you should avoid this](AQuickStart.html#pairwise)
  * [Pairwise comparisons](comparisons.html#pairwise)
      * [Matrix displays](comparisons.html#pwpm)
  * [`pairwise` contrasts](comparisons.html#contrasts)
  * [Pairwise *P*-value plots](comparisons.html#pwpp)
  * [`params`](basics.html#params)
  * [Percentage differences](transformations.html#altscale)
  * [`plot()`](basics.html#plots)
      * [nested factors](messy-data.html#cows)
  * [`plot.emmGrid()`](basics.html#plot.emmGrid)
  * Plots
      * [of confidence intervals](basics.html#plot.emmGrid)
      * [of EMMs](basics.html#plots)
      * [Enhancing with **ggplot2** functions](basics.html#ggplot)
      * [Interaction-style](basics.html#plots)
  * [`+` operator](utilities.html#rbind)
  * Poisson regression
      * [Surprising results](FAQs.html#transformations)
  * [`polreg` models](models.html#O)
  * [Polynomial regression](basics.html#depcovs)
  * Pooled *t*
      * [Instead of Welch's *t*](FAQs.html#nowelch)
  * [`postGridHook`](xtending.html#hooks)
  * [Practices, recommended](basics.html#recs1)
  * Precision
      * [Displaying results with more digits](utilities.html#data)
  * Predictions
      * Bayesian models
          * [in *predictions: bayes*](predictions.html#bayes)
          * [in *sophisticated: predict-mcmc*](sophisticated.html#predict-mcmc)
      * [Error SD](predictions.html#sd-estimate)
      * [graphics](predictions.html#feedlot)
      * [on Particular strata](predictions.html#strata)
      * [Posterior predictive distribution](sophisticated.html#predict-mcmc)
      * [Reference grid](predictions.html#ref-grid)
      * [Total SD](predictions.html#feedlot)
  * [`print.summary_emm()`](basics.html#emmobj)
  * Probit regression
      * [Odds or risk ratios](transformations.html#not-logit)
  * [`pwpm()`](comparisons.html#pwpm)
  * [`pwpp()`](comparisons.html#pwpp)


[Back to top](#topnav)

### Q {#q}


  * [`qdrg()`](FAQs.html#qdrg)
  * [Quadratic terms](basics.html#depcovs)
  * [Quick-start guide](AQuickStart.html)


[Back to top](#topnav)

### R {#r}


  * [R-squared](basics.html#pigs)
  * Random predictors
      * [Accessing levels](sophisticated.html#random-slopes)
  * Random slopes
      * [Bias adjustment](sophisticated.html#random-slopes)
  * [Rank deficiency](xplanations.html#joint_tests)
  * Ratios
      * [but I wanted differences](FAQs.html#ratios)
  * [`rbind()`](utilities.html#rbind)
  * [Re-labeling](utilities.html#relevel)
  * [Recommended practices](basics.html#recs1)
  * [`recover_data()`](xtending.html#intro)
      * [Communicating with `emm_basis()`](xtending.html#communic)
      * [`data` and `params` arguments](xtending.html#rdargs)
      * [Dispatching](xtending.html#dispatch)
      * [Error handling](xtending.html#rderrs)
      * [for `lqs` objects](xtending.html#rd.lqs)
      * [for `rsm` objects](xtending.html#rdrsm)
  * `recover_data.call()`
      * [`frame` argument](xtending.html#rdargs)
  * [`ref_grid()`](basics.html#RG)
      * [`at`](basics.html#RG)
      * `cov.reduce`
          * [in *basics: plot.emmGrid*](basics.html#plot.emmGrid)
          * [in *basics: RG*](basics.html#RG)
      * [`mult.name`](basics.html#multiv)
      * [`nesting`](messy-data.html#nest-trap)
      * [`offset`](sophisticated.html#offsets)
      * [`regrid` argument](transformations.html#regrid2)
  * [Reference grid](basics.html#emms)
      * [Covariates](basics.html#RG)
      * [Defined](basics.html#RG)
      * [Indicator variables](basics.html#RG)
      * [Predictions on](basics.html#refgrid)
  * Reference grids
      * [Prediction on](predictions.html#ref-grid)
  * [Region of practical equivalence](sophisticated.html#bayesxtra)
  * [Registering `recover_data` and `emm_basis` methods](xtending.html#exporting)
  * [`regrid` argument](transformations.html#stdize)
  * [`regrid()`](transformations.html#regrid)
      * [probit or other lon-logit models](transformations.html#not-logit)
      * [`regrid` vs. `type`](transformations.html#regrid2)
      * [to obtain risk ratios](transformations.html#riskrats)
      * [`transform = "log"`](transformations.html#logs)
  * [Residual plots](basics.html#pigs)
  * [Response scale](confidence-intervals.html#tran)
  * [Response transformations](basics.html#transf)
  * [`revpairwise` contrasts](comparisons.html#contrasts)
  * [`rg.limit` option](messy-data.html#rg.limit)
  * [Risk ratios](transformations.html#riskrats)
  * [RMarkdown](basics.html#formatting)
  * [ROPE](sophisticated.html#bayesxtra)
  * [**rsm** package](xtending.html#rsm)
  * [`rstanarm`](sophisticated.html#mcmc)


[Back to top](#topnav)

### S {#s}


  * [Sample size, displaying](confidence-intervals.html#summary)
  * [Sandwich estimators](messy-data.html#neuralgia)
  * Satterthwaite d.f.
      * [in *models: K*](models.html#K)
      * [in *models: L*](models.html#L)
  * [`"scale"` type](transformations.html#trangraph)
  * [`scale()`](transformations.html#stdize)
  * [Selecting results](utilities.html#brackets)
  * [Sidak adjustment](confidence-intervals.html#adjust)
  * Significance
      * [Assessing](basics.html#pvalues)
  * [Significance sets (with CLDs)](re-engineering-clds.html#signif-CLDs)
  * [`simple = "each"`](confidence-intervals.html#simple)
  * Simple comparisons
      * [in *confidence-intervals: simple*](confidence-intervals.html#simple)
      * [in *FAQs: interactions*](FAQs.html#interactions)
      * [in *interactions: simple*](interactions.html#simple)
  * [Simpson's paradox](basics.html#emms)
  * `specs`
      * [Formula](comparisons.html#formulas)
  * [Standardized response](transformations.html#stdize)
  * [`stanreg` objects](models.html#S)
  * [* gazing (star gazing)](interactions.html#factors)
  * [Startup options](utilities.html#startup)
  * [Statistical consultants](basics.html#recs3)
  * [Statistics is hard](basics.html#recs3)
  * [`str()`](basics.html#emmobj)
  * `submodel`
      * [`"minimal"` and `"type2"`](messy-data.html#type2submodel)
      * [in a multivariate model](xplanations.html#mult.submodel)
      * [in *messy-data: submodels*](messy-data.html#submodels)
      * [in *xplanations: submodels*](xplanations.html#submodels)
  * [Subsets of data](FAQs.html#model)
  * `summary()`
      * [`adjust`](comparisons.html#pairwise)
      * [in *basics: emmobj*](basics.html#emmobj)
      * Bayesian models
          * [in *confidence-intervals: summary*](confidence-intervals.html#summary)
          * [in *models: S*](models.html#S)
      * [Calculated columns](confidence-intervals.html#summary)
      * [in *confidence-intervals: summary*](confidence-intervals.html#summary)
      * [HPD intervals](sophisticated.html#mcmc)
      * [`hpd.summary()`](confidence-intervals.html#summary)
      * `infer`
          * [in *comparisons: pairwise*](comparisons.html#pairwise)
          * [in *confidence-intervals: summary*](confidence-intervals.html#summary)
      * [Show sample size](confidence-intervals.html#summary)
      * [`type = "unlink"`](transformations.html#tranlink)
  * [`summary_emm` object](basics.html#emmobj)
      * [As a data frame](utilities.html#data)


[Back to top](#topnav)

### T {#t}


  * [*t* tests vs. *z* tests](FAQs.html#asymp)
  * [`test()`](confidence-intervals.html#summary)
      * [`delta`](confidence-intervals.html#equiv)
      * [`joint = TRUE`](confidence-intervals.html#joint)
  * Tests
      * [Equivalence](confidence-intervals.html#equiv)
      * [Noninferiority](confidence-intervals.html#equiv)
      * [Nonzero null](confidence-intervals.html#summary)
      * [One- and two-sided](confidence-intervals.html#summary)
  * [Too few means](AQuickStart.html#covar)
  * [Transformations](basics.html#transf)
      * [Adding after the fact](transformations.html#after)
      * [Auto-detected](transformations.html#auto)
      * [Back-transforming](confidence-intervals.html#tran)
      * [Bias adjustment](transformations.html#bias-adj)
      * [Custom](transformations.html#special)
      * [faking](transformations.html#faking)
      * [Faking a log transformation](transformations.html#logs)
      * [Graphical display](transformations.html#trangraph)
      * [with link function](transformations.html#tranlink)
      * [Log](comparisons.html#logs)
      * [Overview](transformations.html#overview)
      * [Percent difference](transformations.html#altscale)
      * [Re-gridding](transformations.html#regrid)
      * [Response versus link functions](transformations.html#link-bias)
      * [`scale()`](transformations.html#stdize)
      * [Standardizing](transformations.html#stdize)
      * [Timing is everything](transformations.html#timing)
  * Trends
      * [Estimating and comparing](interactions.html#oranges)
  * [`trt.vs.ctrl` contrasts](comparisons.html#contrasts)
  * [Tukey adjustment](confidence-intervals.html#adjust)
      * [Ignored or changed](FAQs.html#notukey)
  * [`type`](confidence-intervals.html#tran)
  * [`type = "response"`](basics.html#transf)
  * [`type = "scale"`](transformations.html#trangraph)
  * [Type II analysis](messy-data.html#type2submodel)
  * Type III tests
      * [in *confidence-intervals: joint*](confidence-intervals.html#joint)
      * [in *confidence-intervals: joint_tests*](confidence-intervals.html#joint_tests)


[Back to top](#topnav)

### U {#u}


  * [Unadjusted tests](confidence-intervals.html#adjmore)
  * [Underlining, grouping by](re-engineering-clds.html#underlining)
  * [`update()`](utilities.html#update)
      * [`tran`](transformations.html#after)
  * [Using results](utilities.html#data)


[Back to top](#topnav)

### V {#v}


  * [Variables that are not predictors](basics.html#params)
  * [`vcov.`](messy-data.html#neuralgia)
  * [`vcovHook`](xtending.html#hooks)
  * Vignettes
      * [Comparisons](comparisons.html)
      * [Confidence intervals and tests](confidence-intervals.html)
      * [Explanations supplement](xplanations.html)
      * [Extending **emmeans**](xtending.html)
      * [FAQS](FAQs.html)
      * [Interactions](interactions.html)
      * [Messy data](messy-data.html)
      * [Models](models.html)
      * [Predictions](predictions.html)
      * [Re-engineering CLDs](re-engineering-clds.html)
      * [Sophisticated models](sophisticated.html)
      * [Transformations and link functions](transformations.html)
      * [Utilities and options](utilities.html)


[Back to top](#topnav)

### W {#w}


  * Weighting
      * [Equal weights](basics.html#exper)
  * [`weights`](messy-data.html#weights)
  * [Welch's *t* comparisons](FAQs.html#nowelch)
      * [Example](utilities.html#relevel)
  * [`wt.nuis`](messy-data.html#nuisance)


[Back to top](#topnav)

### X {#x}


  * [`xtable` method](basics.html#formatting)


[Back to top](#topnav)

### Z {#z}


  * [*z* tests](sophisticated.html#dfoptions)
      * [vs. *t* tests](FAQs.html#asymp)
  * [`zeroinfl` models](models.html#C)


[Back to top](#topnav)


*Index generated by the [vigindex](https://github.com/rvlenth/vigindex) package.*

---
title: "Basics of estimated marginal means"
author: "emmeans package, Version `r packageVersion('emmeans')`"
output: emmeans::.emm_vignette
vignette: >
  %\VignetteIndexEntry{Basics of EMMs}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
```{r, echo = FALSE, results = "hide", message = FALSE}
require("emmeans")
require("ggplot2")
knitr::opts_chunk$set(fig.width = 4.5, class.output = "ro")
```



## [Contents](#contents)

  1. [Foundations](#found)
     a. [Emphasis on experimental data](#exper)
     b. [Emphasis on models](#models)
     c. [Illustration: `pigs` experiment](#pigs)
     d. [Estimated marginal means](#emms)
     e. [The reference grid, and definition of EMMs](#refgrid)
     f. [More on the reference grid](#RG)
  2. [Other topics](#othertopics)
     a. [Passing arguments](#arguments)
     b. [Transformations](#transf)
     c. [Derived covariates](#depcovs)
     d. [Non-predictor variables](#params)
     e. [Graphical displays](#plots)
     f. [Formatting results](#formatting)
     g. [Using weights](#weights)
     h. [Multivariate responses](#multiv)
  3. [Objects, structures, and methods](#emmobj)
  4. [P values, "significance", and recommendations](#pvalues)
  5. [Summary](#summary)
  6. [Further reading](#more)
  
[Index of all vignette topics](vignette-topics.html)

## Foundations {#found}
<!-- @index Foundations of EMMs; Expected marginal means!Foundations of -->

### Emphasis on experimental data {#exper}
<!-- @index Experimental versus observational data;
            Observational versus experimental data;  
            Controlled experiments; Latin squares; Balanced allocation;
            Weighting!Equal weights; -->
To start off with, we should emphasize that the underpinnings of estimated marginal 
means -- and much of what the **emmeans** package offers -- relate more to
*experimental* data than to *observational* data. In observational data,
we sample from some population, and the goal of statistical analysis is to
characterize that population in some way. In contrast, with experimental data,
the experimenter controls the environment under which test runs are conducted,
and in which responses are observed and recorded. Thus with experimentation,
the population is an abstract entity consisting of potential outcomes of test
runs made under conditions we enforce, rather than a physical entity that we 
observe without changing it.

We say this because the default behavior of the `emmeans()` function is to average
groups together with equal weights; this is common in analysis of experiments,
but not common in analysis of observational data; and I think that misunderstandings
about this underlie some criticisms such as [are found here](https://stats.stackexchange.com/questions/332167/what-are-ls-means-useful-for) and [here](https://stats.stackexchange.com/questions/510862/is-least-squares-means-lsmeans-statistical-nonsense/510923#510923).

Consider, for example, a classic Latin square experimental design. RA Fisher and
others expounded on such designs. Suppose we want to compare four treatments,
say fertilizers, in an agricultural experiment. A Latin square plan would
involve dividing a parcel of land into four rows and four columns, defining 16
plots. Then we apply one of the fertilizers to each plot in such a way that each
fertilizer appears once in each row and once in each column (and thus, each
row and each column contains all four fertilizers). This scheme, to some extent,
controls for possible spatial effects within the land parcel. To compare
the fertilizer, we average together the response values (say, yield of a crop)
observed on the four plots where each fertilizer was used. It seems right to
average these together with equal weight, because each experimental condition
seems equally valid and there is no reason to give one more weight than another.
In this illustration, the fertilizer means are not marginal means of some
physical population; they are simply the means obtained under the four test
conditions defined by the experiment.

### Emphasis on models {#models}
<!-- @index Modelling; Expected marginal means!Reliance on a model;
            Model!Importance of; GIGO (garbage in, garbage out); -->
The **emmeans** package requires you to fit a model to your data. All the
results obtained in **emmeans** rely on this model. So, really, the analysis
obtained is really an analysis of the model, not the data. This analysis does
depend on the data, but only insofar as the fitted model depends on the data. We
use predictions from this model to compute estimated marginal means (EMMs),
which will be defined more explicitly below. For now, there are two things to
know:

  1. If you change the model, that changes the EMMs
  2. If the model fits poorly, the EMMs represent the data poorly
     (the garbage in, garbage out principle)

So to use this package to analyze your data, the most important first step is to
fit a good model.

[Back to Contents](#contents)


### Illustration: `pigs` experiment {#pigs}
<!-- @index Examples!`pigs`; Examples!Unbalanced data; 
            Modeling!Response transformation; Modeling!`pigs` example; 
            `inverse()`; Residual plots; R-squared; Adjusted R-squared -->
Consider the `pigs` dataset provided with the package (`help("pigs")` provides
details). These data come from an experiment where pigs are given
different percentages of protein (`percent`) from different sources (`source`)
in their diet, and later we measured the concentration (`conc`) of leucine.
The `percent` values are quantitative, but we chose those particular values
deliberately, and (at least initially) we want separate estimates at each `percent` 
level; that is, we want to view `percent` as a factor, not a quantitative predictor.

As discussed, our first task is to come up with a good model. Doing so requires
a lot of skill, and we don't want to labor too much over the details; you really 
need other references to deal with this aspect adequately. But we will briefly 
discuss five models and settle on one of them:
```{r}
mod1 <- lm(conc ~ source * factor(percent), data = pigs)
mod2 <- update(mod1, . ~ source + factor(percent))   # no interaction
```
These models have $R^2$ values of 0.808 and 0.700, and adjusted $R^2$ values of
0.684 and 0.634. `mod1` is preferable to `mod2`, suggesting we need the
interaction term. However, a residual-vs-predicted plot of `mod2` has a classic
"horn" shape (curving and fanning out), indicating a situation where a response
transformation might help better than including the interaction. 

It turns out that an inverse transformation, (`1/conc`) really serves us well.
(Perhaps this isn't too surprising, as concentrations are typically determined
by titration, in which the actual measurements are volumes; and these are
reciprocally related to concentrations, i.e., amounts *per* unit volume.)

So here are three more models:
```{r}
mod3 <- update(mod1, inverse(conc) ~ .)
mod4 <- update(mod2, inverse(conc) ~ .)     # no interaction
mod5 <- update(mod4, . ~ source + percent)  # linear term for percent
```
(Note: We could have used `1/conc` as the response variable, but **emmeans**
provides an equivalent `inverse()` function that will prove more advantageous
later.) The residual plots for these models look a lot more like a random scatter
of points (and that is good). The $R^2$ values for these models are 0.818, 0.787,
and 0.749, respectively; and the adjusted $R^2$s are 0.700, 0.740, and 0.719.
`mod4` has the best adjusted $R^2$ and will be our choice.

[Back to Contents](#contents)


### Estimated marginal means {#emms}
<!-- @index Estimated marginal means!Demonstrated; 
            Estimated marginal means!Reference grid; Reference grid;
            Estimated marginal means!vs. ordinary marginal means;
            Simpson's paradox; Confounding;  `emmeans()`; -->
Now that we have a good model, let's use the `emmeans()` function to obtain
estimated marginal means (EMMs). We'll explain them later.
```{r}
(EMM.source <- emmeans(mod4, "source"))
(EMM.percent <- emmeans(mod4, "percent"))
```
Let's compare these with the ordinary marginal means (OMMs) on `inverse(conc)`:
```{r}
with(pigs, tapply(inverse(conc), source, mean))
with(pigs, tapply(inverse(conc), percent, mean))
```
Both sets of OMMs are vaguely similar to the corresponding EMMs. However,
please note that the EMMs for `percent` form a decreasing sequence, while
the the OMMs decrease but then increase at the end.

### The reference grid, and definition of EMMs {#refgrid}
<!-- @index Estimated marginal means!Definition of; 
            Reference grid!Predictions on;
            Estimated marginal means!Manually computed; -->
Estimated marginal means are defined as marginal means of model predictions
over the grid comprising all factor combinations -- called the *reference grid*.
For the example at hand, the reference grid is
```{r}
(RG <- expand.grid(source = levels(pigs$source), percent = unique(pigs$percent)))
```
To get the EMMs, we first need to obtain predictions on this grid:
```{r}
(preds <- matrix(predict(mod4, newdata = RG), nrow = 3))
```
then obtain the marginal means of these predictions:
```{r}
apply(preds, 1, mean)   # row means -- for source
apply(preds, 2, mean)   # column means -- for percent
```
These marginal averages match the EMMs obtained earlier via `emmeans()`.

Now let's go back to the comparison with the ordinary marginal means. The
`source` levels are represented by the columns of `pred`; and note that each row
of `pred` is a decreasing set of values. So it is no wonder that the marginal
means -- the EMMs for `source` -- are decreasing. That the OMMs for `percent` do
not behave this way is due to the imbalance in sample sizes:
```{r}
with(pigs, table(source, percent))
```
This shows that the OMMs of the last column give most of the weight (3/5) to the
first source, which tends to have higher `inverse(conc)`, making the OMM for 18
percent higher than that for 15 percent, even though the reverse is true with
every level of `source`. This kind of disconnect is an example of *Simpson's
paradox,* in which a confounding factor can distort your findings. The EMMs are
not subject to this paradox, but the OMMs are, when the sample sizes are
correlated with the expected values.

In summary, we obtain a references grid of all factor combinations, obtain model
predictions on that grid, and then the expected marginal means are estimated as
equally-weighted marginal averages of those predictions. Those EMMs are not
subject to confounding by other factors, such as might happen with ordinary
marginal means of the data. Moreover, unlike OMMs, EMMs are based on a model
that is fitted to the data.

[Back to Contents](#contents)


### More on the reference grid {#RG}
<!-- @index Reference grid!Defined; `ref_grid()`; 
            `ref_grid()`!`at`; `ref_grid()`!`cov.reduce`;
            Reference grid!Covariates; Adjusted means; 
            Reference grid!Indicator variables; Indicator variables;  -->
In the previous section, we discussed the reference grid as being the set of all
factor combinations. It is slightly more complicated than that when we have
numerical predictors (AKA covariates) in the model. By default, we use the
average of each covariate -- thus not enlarging the number of combinations
comprising the grid. Using the covariate average(s) yields what are often called
*adjusted means*. There is one exception, though: if a covariate has only two
different values, we treat it as a factor having those two levels. For example,
a model could include an indicator variable `male` that is `1` if the subject is
male, and `0` otherwise. Then `male` would be viewed as a factor with levels `0`
and `1`. Note, again, that the reference grid is formulated from the model
we are using.

We can see a snapshot of the reference grid via the `ref_grid` function; for example
```{r}
(RG4 <- ref_grid(mod4))
ref_grid(mod5)
```
The reference grid for `mod5` is different from that for `mod4` because in those models, `percent` is a factor in `mod4` and a covariate in `mod5`.

It is possible to modify the reference grid. In the context of the present example,
it might be inetersting to compare EMMs based on `mod4` and `mod5`, and we can put
them on an equal footing by using the same `percent` values as reference levels:
```{r}
(RG5 <- ref_grid(mod5, at = list(percent = c(9, 12, 15, 18))))
```
We could also have done this using
```{r eval = FALSE}
(RG5 <- ref_grid(mod5, cov.reduce = FALSE)
```
... which tells `ref_grid()` to set covariate levels using unique values.
It's safer to use `at` because `cov.reduce` affects *all* covariates
instead of specific ones.
###### {#emmip}
<!-- @index `emmip()`; Interaction-style plots; -->
The two models' predictions can be compared using interaction-style plots
via the `emmip()` function
```{r fig.alt = "interaction-style plots of 'RG4' and 'RG5'. These show parallel trends along 'percent' for each 'source'. The one for 'RG5' consists of parallel straigt lines. The values plotted here can be obtained via 'summary(RG4)' and 'summary(RG5)'"}
emmip(RG4, source ~ percent, style = "factor")
emmip(RG5, source ~ percent, style = "factor")
```
Both plots show three parallel trends, because neither model includes an 
interaction term; but of course for `mod5`, those trends are straight lines.


[Back to Contents](#contents)


## Other topics {#othertopics}


### Passing arguments {#arguments}
<!-- @index Arguments!Passed to `ref_grid()`; Arguments!`...`
     `emmeans()`!Arguments passed to `ref_grid()`; -->
Quite a few functions in the **emmeans** package, including `emmeans()` and `emmip()`,
can take either a model object or a reference-grid object as their first argument.
Thus we can obtain EMMs for `mod5` directly from `RG5`, e.g.
```{r}
emmeans(RG5, "source")
```
These are slightly different results than we had earlier for `mod4`.
In these functions where the model and the reference grid are interchangeable, 
the first thing the function does is to check which it is; and if it is a model
object, it constructs the reference grid. When it does that, it passes
its arguments to `ref_grid()` in case they are needed. For instance, the
above EMMs could have been obtained using
```{r, eval = FALSE}
emmeans(mod5, "source", at = list(percent = c(9, 12, 15, 18)))
## (same results as above)
```
It is a great convenience to be able to pass arguments to `ref_grid()`, but it
also can confuse new users, because if we look at the help page for `emmeans()`,
it does not list `at` as a possible argument. It is mentioned, though, if you
look at the `...` argument. So develop a habit of looking at documentation for
other functions, especially `ref_grid()`, for other arguments that may affect 
your results.


[Back to Contents](#contents)


### Transformations {#transf}
<!-- @index Transformations; Response transformations; Back-transforming;
            `type = "response"`; --> 
In our running example with `pigs`, by now you are surely tired of seeing
all the answers on the `inverse(conc)` scale. What about estimating things on the
`conc` scale? You may have noticed that the `inverse` transformation has not
been forgotten; it is mentioned in the annotations below the `emmeans()` output.
[I'd also comment that having used `inverse(conc)` rather than `1/conc` as the
response variable in the model has made it easier to sort things out, because
`inverse()` is a named transformation that `emmeans()` can work with.]
We can back-transform the results by specifying `type = "response"` in any
function call where it makes sense. For instance,
```{r fig.alt = "interaction-style plots for 'RG4' after back-transforming. Compared to the plots of 'RG4' without back-transforming, these trends increase rather than decrease (due to the inverse transformation) and they fan-out somewhat as 'percent' increases. The values plotted here are obtainable via 'summary(RG4, type = \"response\")'"}
emmeans(RG4, "source", type = "response")
emmip(RG4, source ~ percent, type = "response")
```
We are now on the `conc` scale, and that will likely be less confusing. Compared with the earlier plots in which the trends were decreasing and parallel, this plot has them increasing (because of the inverse relationship) and non-parallel. An interaction that
occurs on the response scale is pretty well explained by a model with no interactions on the inverse scale.

Transformations have a lot of nuances, and we refer you to the [vignette of transformations](transformations.html) for more details.

[Back to Contents](#contents)



### Derived covariates {#depcovs}
<!-- @index Covariates!Derived; Quadratic terms; Polynomial regression;
            Examples!`mtcars`;        -->

You need to be careful when one covariate depends on the value of another. To
illustrate using the `datasets::mtcars` data, suppose we want to predict `mpg` using `cyl` (number of cylinders) as a factor `disp` (displacement) as a covariate, and
include a quadratic term for `disp`. Here are two equivalent models:
```{r}
mcmod1 <- lm(mpg ~ factor(cyl) + disp + I(disp^2), data = mtcars)
mtcars <- transform(mtcars, 
                    dispsq = disp^2)
mcmod2 <- lm(mpg ~ factor(cyl) + disp + dispsq, data = mtcars)
```
These two models have exactly the same predicted values. But look at the EMMs:
```{r}
emmeans(mcmod1, "cyl")
emmeans(mcmod2, "cyl")
```
Wow! Those are really different results -- even though the models are
equivalent. Why is this -- and which (if either) is right? To understand, look
at the reference grids:
```{r}
ref_grid(mcmod1)
ref_grid(mcmod2)
```
For both models, the reference grid uses the `disp` mean of 230.72. But for
`mcmod2`, `dispsq` is a separate covariate, and it is set to its mean of 68113. 
This is not right, because it is impossible to have `disp` equal to 230.72 and its
square equal to 68113 at the same time!
If we use consistent values of `disp` and`dispsq`, we get the same results as for `mcmod1`:
```{r}
emmeans(mcmod2, "cyl", at = list(disp = 230.72, dispsq = 230.72^2))
```

In summary, for polynomial models and others where some covariates depend on
others in nonlinear ways, it is definitely best to include that dependence in
the model formula (as in `mcmod1`) using `I()` or `poly()` expressions, or
alter the reference grid so that the dependency among covariates is correct.

[Back to Contents](#contents)



### Non-predictor variables {#params}
<!-- @index `params`; Variables that are not predictors; -->
Reference grids are derived using the variables in the right-hand side of the model
formula. But sometimes, these variables are not actually predictors. For example:
```{r, eval = FALSE}
deg <- 2
mod <- lm(y ~ treat * poly(x, degree = deg), data = mydata)
```
If we call `ref_grid()` or `emmeans()` with this model, it will try to construct
a grid of values of `treat`, `x`, and `deg` -- causing an error because `deg` is
not a predictor in this model. To get things to work correctly, you need to name
`deg` in a `params` argument, e.g.,
```{r, eval = FALSE}
emmeans(mod, ~ treat | x, at = list(x = 1:3), params = "deg")
```


[Back to Contents](#contents)

### Graphical displays {#plots}
<!-- @index Graphical displays; Plots!of EMMs@emms; Plots!Interaction-style; 
            `emmip()`; `plot()`;  -->
The results of `ref_grid()` or `emmeans()` (these are objects of class `emmGrid`)
may be plotted in two different 
ways. One we have already seen is an interaction-style plot, using `emmip()`. 

The formula
specification we used in `emmip(RG4, source ~ percent)` sets the *x* variable to be the one on the right-hand side and the "trace"
factor (what is used to define the different curves) on the left.

###### {#plot.emmGrid}
<!-- @index Plots!of confidence intervals@conf; `plot.emmGrid()`;
            `ref_grid()`!`cov.reduce`; -->
The other graphics option offered is the `plot()` method for `emmGrid` objects. 
Let's consider a different model for the `mtcars` data with both `cyl` and `disp`
as covariates
```{r}
mcmod3 <- lm(mpg ~ disp * cyl, data = mtcars)
```
In
the following, we display the estimates and 95% confidence intervals for
`RG4` in separate panels for each `source`.


```{r fig.alt = "Plot of side-by-side confidence intervals for 'cyl' means, in 3 panels corresponding to 'disp' values of 100, 200, and 300. The values plotted here are those in 'summary(EMM3)'"}
EMM3 <- emmeans(mcmod3, ~ cyl | disp, 
                at = list(cyl = c(4,6,8), disp = c(100,200,300)))
plot(EMM3)
```

This plot illustrates, as much as anything else, how silly it is to try to
predict mileage for a 4-cylinder car having high displacement, or an 8-cylinder
car having low displacement. The widths of the intervals give us a clue that we
are extrapolating. A better idea is to acknowledge that displacement largely
depends on the number of cylinders. So here is yet another way to 
use `cov.reduce` to modify the reference grid:
```{r}
mcrg <- ref_grid(mcmod3, at = list(cyl = c(4,6,8)),
                         cov.reduce = disp ~ cyl)
mcrg @ grid
```
The `ref_grid` call specifies that `disp` depends on `cyl`; so a linear model 
is fitted with the given formula and its fitted values are used as the `disp`
values -- only one for each `cyl`. If we plot this grid, the results are 
sensible, reflecting what the model predicts for typical cars with each 
number of cylinders:
```{r fig.height = 1.5, fig.alt = "Side-by-side CIs for cyl marginal means. The values plotted are obtainable via 'summary(mcrg)'"}
plot(mcrg)
```

###### {#ggplot}
<!-- @index **ggplot2** package; Plots!Enhancing with **ggplot2** functions -->
Wizards with the **ggplot2** package can further enhance these plots if 
they like. For example, we can add the data to an interaction plot -- this
time we opt to include confidence intervals and put the three sources 
in separate panels:
```{r fig.alt = "Enhanced interaction plot with CIs and observed data added; we have separate panels for the 3 diets, and the 4 percent conentrations in each panel"}
require("ggplot2")
emmip(mod4, ~ percent | source, CIs = TRUE, type = "response") +
    geom_point(aes(x = percent, y = conc), data = pigs, pch = 2, color = "blue")
```

[Back to Contents](#contents)


### Formatting results {#formatting}
<!-- @index Formatting results; `kable`; Exporting output; `xtable` method; 
  RMarkdown; -->
If you want to include `emmeans()` results in a report, you might want to have it
in a nicer format than just the printed output. We provide a little bit of help for this,
especially if you are using RMarkdown or SWeave to prepare the report. 
There is an `xtable` method for exporting these results, which we do not illustrate
here but it works similarly to `xtable()` in other contexts. Also, the `export` option 
the `print()` method allows the user to save exactly what is seen in the printed
output as text, to be saved or formatted as the user likes (see the documentation for `print.emmGrid` for details).
Here is an example using one of the objects above:
```{r, eval = FALSE}
ci <- confint(mcrg, level = 0.90, adjust = "scheffe")
xport <- print(ci, export = TRUE)
cat("<font color = 'blue'>\n")
knitr::kable(xport$summary, align = "r")
for (a in xport$annotations) cat(paste(a, "<br>"))
cat("</font>\n")
```
```{r, results = "asis", echo = FALSE}
ci <- confint(mcrg, level = 0.90, adjust = "scheffe")
xport <- print(ci, export = TRUE)
cat("<font color = 'blue'>\n")
knitr::kable(xport$summary, align = "r")
for (a in xport$annotations) cat(paste(a, "<br>"))
cat("</font>\n")
```


[Back to Contents](#contents)

### Using weights {#weights}
<!-- @index Means!Weighted; `emmeans()`!`weights`; Observational data; -->
As we have mentioned, `emmeans()` uses equal weighting by default, based on its foundations in experimental situations. When you have observational data, you are more likely to use unequal weights that more accurately characterize the population.
Accordingly, a `weights` argument is provided in `emmeans()`. For example, using
`weights = "cells"` in the call will weight the predictions according to their
cell frequencies (recall this information is retained in the reference grid).
This produces results comparable to ordinary marginal means:
```{r}
emmeans(mod4, "percent", weights = "cells")
```
Note that, as in the ordinary marginal means we obtained long ago,
the highest estimate is for `percent = 15` rather than `percent = 18`. It is
interesting to compare this with the results for a model that includes only
`percent` as a predictor.
```{r}
mod6 <- lm(inverse(conc) ~ factor(percent), data = pigs)
emmeans(mod6, "percent")
```
The EMMs in these two tables are identical, so in some sense, 
`weights = "cells"` amounts to throwing-out the uninvolved factors. 
However, note that these
outputs show markedly different standard errors. That is because the model
`mod4` accounts for variations due to `source` while `mod6` does not. The lesson
here is that it is possible to obtain statistics comparable to ordinary marginal
means, while still accounting for variations due to the factors that are being
averaged over.

[Back to Contents](#contents)

### Multivariate responses {#multiv}
<!-- @index Multivariate models; Examples!`MOats`
     Examples!Multivariate; `ref_grid()`!`mult.name` -->
The **emmeans** package supports various multivariate models. When there
is a multivariate response, the dimensions of that response are treated as if
they were levels of a factor. For example, the `MOats` dataset provided in the
package has predictors `Block` and `Variety`, and a four-dimensional response
`yield` giving yields observed with varying amounts of nitrogen added to the soil.
Here is a model and reference grid:
```{r}
MOats.lm <- lm (yield ~ Block + Variety, data = MOats)
ref_grid (MOats.lm, mult.name = "nitro")
```
So, `nitro` is regarded as a factor having 4 levels corresponding to the 4
dimensions of `yield`. We can subsequently obtain EMMs for any of the factors
`Block`, `Variety`, `nitro`, or combinations thereof. The argument `mult.name =
"nitro"` is optional; if it had been excluded, the multivariate levels would
have been named `rep.meas`.

[Back to Contents](#contents)





## Objects, structures, and methods {#emmobj}
<!-- @index `emmGrid` objects -->
The `ref_grid()` and `emmeans()` functions are introduced previously.
These functions, and a few related ones,
return an object of class `emmGrid`. From previously defined objects:
```{r}
class(RG4)
class(EMM.source)
```
If you simply show these objects, you get different-looking results:
```{r}
RG4

EMM.source
```
This is based on guessing what users most need to see when displaying the object.
You can override these defaults; for example to just see a quick summary of
what is there, do
```{r}
str(EMM.source)
```
<!-- @index `str()`; `summary()`; `summary_emm` object; `print.summary_emm()` -->
The most important method for `emmGrid` objects is `summary()`. It is used as the
print method for displaying an `emmeans()` result. For this reason, arguments for 
`summary()` may also be specified within most functions that produce `these kinds of
results.`emmGrid` objects. For example:
```{r}
# equivalent to summary(emmeans(mod4, "percent"), level = 0.90, infer = TRUE))
emmeans(mod4, "percent", level = 0.90, infer = TRUE)
```


This `summary()`
method for `emmGrid` objects) actually produces a `data.frame`, but with extra bells 
and whistles:
```{r}
class(summary(EMM.source))
```
This can be useful to know because if you want to actually *use* `emmeans()` results
in other computations, you should save its summary, and then you can access those
results just like you would access data in a data frame. The `emmGrid` object itself
is not so accessible. There is a `print.summary_emm()` function that is what
actually produces the output you see above -- a data frame with extra
annotations.



[Back to Contents](#contents)




## P values, "significance", and recommendations {#pvalues}
<!-- @index Significance!Assessing; 
    *P* values!Interpreting; ATOM; 
    ASA Statement on *P* values -->
There is some debate among statisticians and researchers about the appropriateness
of *P* values, and that the term "statistical significance" can be misleading. 
If you have a small *P* value, it *only* means that the
effect being tested is unlikely to be explained by chance variation alone, in
the context of the current study and the current statistical model underlying
the test. If you have a large *P* value, it *only* means that the observed
effect could plausibly be due to chance alone: it is *wrong* to conclude that
there is no effect.

The American Statistical Association has for some time been advocating very
cautious use of *P* values (Wasserstein *et al.* 2014) because it is
too often misinterpreted, and too often used carelessly. Wasserstein *et al.* (2019) 
even goes so far as to advise against *ever* using the term "statistically significant".
The 43 articles it accompanies in the same issue of *TAS*, recommend a
number of alternatives. I do not agree with all that is said in the main
article, and there are portions that are too cutesy or wander off-topic.
Further, it is quite dizzying to try to digest all the accompanying articles,
and to reconcile their disagreeing viewpoints. I do agree with one frequent point: that
there is really no substantive difference between $P=.051$ and $P=.049$,
and that one should avoid making sweeping statements based on a hard cutoff
at $P=.05$ or some other value.

For some time I included a summary of Wasserstein *et al.*'s
recommendations and their *ATOM* paradigm (Acceptance of uncertainty, Thoughtfulness, Openness,
Modesty). But in the meantime, I have handled a large number of user questions, and many of
those have made it clear to me that there are more important fish to fry in  a vignette 
section like this. It is just a fact that *P* values are used, and are useful. So I have my own set of recommendations regarding them.

#### A set of comparisons or well-chosen contrasts is more useful and interpretable than an omnibus *F* test {#recs1}
<!-- @index Recommended practices; Practices, recommended;
     *P* values!Adjusted; *F* test!Role in *post hoc* tests; -->
*F* tests are useful for model selection, but don't tell you anything specific about the nature of an effect. If *F* has a small *P* value, it suggests that there is some effect, somewhere. It doesn't
even necessarily imply that any two means differ statistically.

#### Use *adjusted* *P* values
When you run a bunch of tests, there is a risk of
making too many type-I errors, and adjusted *P* values (e.g., the Tukey adjustment 
for pairwise comparisons) keep you from making too many mistakes. That said,
it is possible to go overboard; and it's usually reasonable to regard each
"by" group as a separate family of tests for purposes of adjustment.
    
#### It is *not* necessary to have a significant *F* test as a prerequisite to doing comparisons or contrasts {#recs2}
<!-- @index LSD!protected; Model!Get it right first -->

... as long as an appropriate adjustment is used. There do
exist rules such as the "protected LSD" by which one is given license to do unadjusted
comparisons provided the $F$ statistic is "significant." However, this is a very weak
form of protection for which the justification is, basically, "if $F$ is significant,
you can say absolutely anything you want."

#### Get the model right first
Everything the **emmeans** package does is an interpretation of the model that you
fitted to the data. If the model is bad, you will get bad results from `emmeans()` and other
functions. Every single limitation of your model, be it presuming constant error variance,
omitting interaction terms, etc., becomes a limitation of the results `emmeans()` produces.
So do a responsible job of fitting the model. And if you don't know what's meant by that...

#### Consider seeking the advice of a statistical consultant {#recs3}
<!-- @index Statistical consultants; Consultants; Statistics is hard -->
Statistics is *hard*. It is a lot more than just running programs and copying output.
We began this vignette by emphasizing we need to start with a good model; that
is an artful task, and certainly what is shown here only hints at
what is required; you may need help with it.
It is *your* research; is it important that it be done right?
Many academic statistics and biostatistics departments can refer you to someone 
who can help.



[Back to Contents](#contents)


## Summary of main points {#summary}
  * EMMs are derived from a *model*. A different model for the same data may lead
    to different EMMs.
  * EMMs are based on a *reference grid* consisting of all combinations
    of factor levels, with each covariate set to its average (by default).
  * For purposes of defining the reference grid, dimensions of
    a multivariate response are treated as levels of a factor.
  * EMMs are then predictions on this reference grid, or marginal
    averages thereof (equally weighted by default).
  * Reference grids may be modified using `at` or other arguments
    for `ref_grid()`
  * Reference grids and `emmeans()` results may be plotted via `plot()`
    (for parallel confidence intervals) or `emmip()` (for an interaction-style
    plot).
  * Be cautious with the terms "significant" and "nonsignificant", and don't ever
    interpret a "nonsignificant" result as saying that there is no effect.
  * Follow good statistical practices such as getting the model right first, 
    and using adjusted *P* values
    for appropriately chosen families of comparisons or contrasts.
    
[Back to Contents](#contents)

### References
 Wasserstein RL, Lazar NA (2016)
   "The ASA's Statement on *p*-Values: Context, Process, and Purpose,"
   *The American Statistician*, **70**, 129--133,
   https://doi.org/10.1080/00031305.2016.1154108
 
 Wasserstein RL, Schirm AL, Lazar, NA (2019)
   "Moving to a World Beyond 'p < 0.05',"
   *The American Statistician*, **73**, 1--19,
   https://doi.org/10.1080/00031305.2019.1583913



## Further reading {#more}
The reader is referred to other vignettes for more details and advanced use.
The strings linked below are the names of the vignettes; i.e., they can
also be accessed via `vignette("`*name*`", "emmeans")`

  * Models that are supported in **emmeans** (there are lots of them)
    ["models"](models.html)
  * Confidence intervals and tests: 
    ["confidence-intervals"](confidence-intervals.html)
  * Often, users want to compare or contrast EMMs: ["comparisons"](comparisons.html)
  * Working with response transformations and link functions:
    ["transformations"](transformations.html)
  * Multi-factor models with interactions: ["interactions"](interactions.html)
  * Working with messy data and nested effects: ["messy-data"](messy-data.html)
  * Making predictions from your model: ["predictions"](predictions.html)
  * Examples of more sophisticated models (e.g., mixed, ordinal, MCMC)
    ["sophisticated"](sophisticated.html)
  * Utilities for working with `emmGrid` objects: ["utilities"](utilities.html)
  * Frequently asked questions: ["FAQs"](FAQs.html)
  * Adding **emmeans** support to your package: ["xtending"](xtending.html)

[Back to Contents](#contents)

[Index of all vignette topics](vignette-topics.html)

---
title: "Confidence intervals and tests in emmeans"
author: "emmeans package, Version `r packageVersion('emmeans')`"
output: emmeans::.emm_vignette
vignette: >
  %\VignetteIndexEntry{Confidence intervals and tests}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
```{r, echo = FALSE, results = "hide", message = FALSE}
require("emmeans")
knitr::opts_chunk$set(fig.width = 4.5, class.output = "ro")
```

<!-- @index Vignettes!Confidence intervals and tests -->

## Contents {#contents}
This vignette describes various ways of summarizing `emmGrid` objects.

  1. [`summary()`, `confint()`, and `test()`](#summary)
  2. [Back-transforming to response scale](#tran)
     (See also the ["transformations" vignette](transformations.html))
  3. [Multiplicity adjustments](#adjust)
  4. [Using "by" variables](#byvars)
  5. [Joint (omnibus) tests](#joint)
  6. [Testing equivalence, noninferiority, nonsuperiority](#equiv)
  7. Graphics (in ["basics" vignette](basics.html#plots))
  
[Index of all vignette topics](vignette-topics.html)

## `summary()`, `confint()`, and `test()` {#summary}
<!-- @index `summary()`; `summary()`!`infer`; `confint()`; `test()`; 
    Confidence intervals; `summary()`!Calculated columns; 
    `summary()`!Show sample size; Sample size, displaying;
            Tests!One- and two-sided; Tests!Nonzero null;
            `summary()`!Bayesian models; `summary()`!`hpd.summary()`;
            Examples!`pigs` -->
The most important method for `emmGrid` objects is `summary()`. For one thing, it is
called by default when you display an `emmeans()` result.
The `summary()` function has a lot of options, and the detailed documentation via 
`help("summary.emmGrid")` is worth a look.

For ongoing illustrations, let's re-create some of the objects in the ["basics" vignette](basics.html) for the `pigs` example:
```{r}
mod4 <- lm(inverse(conc) ~ source + factor(percent), data = pigs)
RG <- ref_grid(mod4)
EMM.source <- emmeans(RG, "source")
```

Just `summary(<object>)` by itself will produce a summary that varies somewhat
according to context. It does this by setting different defaults for the `infer`
argument, which consists of two logical values, specifying confidence intervals 
and tests, respectively. [The exception is models fitted using MCMC methods,
where `summary()` is diverted to the `hpd.summary()` function, a 
preferable summary for many Bayesians.]

The summary of a newly made reference grid will show just estimates and
standard errors, but not confidence intervals or tests (that is, `infer =
c(FALSE, FALSE)`). The summary of an `emmeans()` result, as we see above, will
have intervals, but no tests (i.e., `infer = c(TRUE, FALSE)`); and the result of
a `contrast()` call (see [comparisons and contrasts](comparisons.html)) will
show test statistics and *P* values, but not intervals (i.e., `infer = c(FALSE,
TRUE)`). There are courtesy methods `confint()` and `test()` that just call
`summary()` with the appropriate `infer` setting; for example,
```{r}
test(EMM.source)
```
It is not particularly useful, though, to test these EMMs against the default of 
zero -- which is why tests are not usually shown. It makes a lot more sense
to test them against some target concentration, say 40. And suppose we want to do a one-sided test to see if the concentration is greater than 40. Remembering that the
response is inverse-transformed in this model, and that the inverse transformation
reverses the direction of comparisons, so that a *right*-tailed test on the `conc` scale
becomes a *left*-tailed test on the `inverse(conc)` scale,
```{r}
test(EMM.source, null = inverse(40), side = "<")
```
It is also possible to add calculated columns to the summary, via the `calc` 
argument. The calculations can include any columns up through `df` in the summary,
as well as any variable in the object's `grid` slot. Among the latter are 
usually weights in a column named `.wgt.`, and we can use that to include
sample size in the summary:
```{r}
confint(EMM.source, calc = c(n = ~.wgt.))
```


[Back to Contents](#contents)


## Back-transforming {#tran}
<!-- @index Transformations!Back-transforming; `type`; Response scale -->
Transformations and link functions are supported in several ways in **emmeans**,
making this a complex topic worthy of [its own vignette](transformations.html).
Here, we show just the most basic approach. Namely, specifying the argument
`type = "response"` will cause the displayed results to be back-transformed
to the response scale, when a transformation or link function is incorporated
in the model. For example, let's try the preceding `test()` call again:
```{r}
test(EMM.source, null = inverse(40), side = "<", type = "response")
```
Note what changes and what doesn't change. In the `test()` call, we *still* use
the 1/40 as the null value; `null` must always be specified on the
linear-prediction scale, in this case the inverse. In the output, the displayed
estimates, as well as the `null` value, are shown back-transformed. As well, the
standard errors are altered (using the delta method). However, the *t* ratios
and *P* values are identical to the preceding results. That is, the tests
themselves are still conducted on the linear-predictor scale (as is noted in the
output).

Similar statements apply to confidence intervals on the response scale:
```{r}
confint(EMM.source, side = "<", level = .90, type = "response")
```
With `side = "<"`, an *upper* confidence limit is computed on the inverse scale,
then that limit is back-transformed to the response scale; and since `inverse`
reverses everything, those upper confidence limits become lower ones on the
response scale.
(We have also illustrated 
how to change the confidence level.)

[Back to Contents](#contents)


## Multiplicity adjustments {#adjust}
<!-- @index Multiplicity adjustments; `adjust`; 
     Tukey adjustment; Sidak adjustment -->
Both tests and confidence intervals may be adjusted for simultaneous inference. 
Such adjustments ensure that the confidence coefficient for a whole set of 
intervals is at least the specified level, or to control for multiplicity
in a whole family of tests. This is done via the `adjust`
argument. For `ref_grid()` and `emmeans()` results, the default is `adjust =
"none"`. For most `contrast()` results, `adjust` is often something else,
depending on what type of contrasts are created. For example, pairwise
comparisons default to `adjust = "tukey"`, i.e., the Tukey HSD method.
The `summary()` function sometimes *changes* `adjust` if it is inappropriate.
For example, with
```{r}
confint(EMM.source, adjust = "tukey")
```
the adjustment is changed to the Sidak method because the Tukey adjustment is 
inappropriate unless you are doing pairwise comparisons.

####### {#adjmore}
<!-- @index **mvtnorm** package; Bonferroni adjustment
     Multivariate *t* (`"mvt"`) adjustment; Unadjusted tests -->
An adjustment method that is usually appropriate is Bonferroni; however, it can
be quite conservative. Using `adjust = "mvt"` is the closest to being the
"exact" all-around method "single-step" method, as it uses the multivariate *t*
distribution (and the **mvtnorm** package) with the same covariance structure as
the estimates to determine the adjustment. However, this comes at high
computational expense as the computations are done using simulation techniques.
For a large set of tests (and especially confidence intervals), the
computational lag becomes noticeable if not intolerable.

For tests, `adjust` increases the *P* values over those otherwise obtained with
`adjust = "none"`. Compare the following adjusted tests with the unadjusted ones
previously computed.
```{r}
test(EMM.source, null = inverse(40), side = "<", adjust = "bonferroni")
```

[Back to Contents](#contents)


## "By" variables {#byvars}
<!-- @index `by` groups; Grouping into separate sets -->
Sometimes you want to break a summary down into smaller pieces; for this
purpose, the `by` argument in `summary()` is useful. For example,
```{r}
confint(RG, by = "source")
```
If there is also an `adjust` in force when `by` variables are used, by default, the 
adjustment is made *separately* on each `by` group; e.g., in the above, we would
be adjusting for sets of 4 intervals, not all 12 together (but see "cross-adjustments" below.)

There can be a `by` specification in `emmeans()` (or equivalently, a `|` in the
formula); and if so, it is passed on to `summary()` and used unless overridden
by another `by`. Here are examples, not run:
```{r eval = FALSE}
emmeans(mod4, ~ percent | source)     ### same results as above
summary(.Last.value, by = "percent")       ### grouped the other way
```
Specifying `by = NULL` will remove all grouping.


### Adjustments across `by` groups {#cross-adjust}
<!-- @index cross-group comparisons; `cross.adjust`; `adjust`!vs. `cross.adjust`
     `by` groups!Adjusting across groups; Examples!`warpbreaks` -->
As was mentioned, each `by` group is regarded as a separate family with regards to
the `adjust` procedure. For example, consider a model with interaction for the `warpbreaks` data,
and construct pairwise comparisons of `tension` by `wool`:
```{r}
warp.lm <- lm(breaks ~ wool * tension, data = warpbreaks)
warp.pw <- pairs(emmeans(warp.lm, ~ tension | wool))
warp.pw
```
We have two sets of 3 comparisons, and the (default) Tukey adjustment is made *separately* in each group.

However, sometimes we want the multiplicity adjustment to be broader.
This broadening can be done in two ways. One is to remove the `by` variable,
which then treats all results as one family. In our example:
```{r}
test(warp.pw, by = NULL, adjust = "bonferroni")
```
This accomplishes the goal of putting all the comparisons in one family of 6
comparisons. Note that the Tukey adjustment may not be used here because we no
longer have *one* set of pairwise comparisons.

An alternative is to specify `cross.adjust`, which specifies an additional adjustment method
to apply to corresponding sets of within-group adjusted *P* values:
```{r}
test(warp.pw, adjust = "tukey", cross.adjust = "bonferroni")
```
These adjustments are less conservative than the previous result, but it is still
a conservative adjustment to the set of 6 tests. Had we also specified `adjust = "bonferroni"`,
we would have obtained the same adjusted *P* values as we obtained with `by = NULL`.

### Simple comparisons {#simple}
<!-- @index Simple comparisons; `contrast()`!`simple`; `simple = "each"` -->
There is also a `simple` argument for `contrast()` that is in essence 
the inverse of `by`; the contrasts are run using everything *except* the
specified variables as `by` variables. To illustrate, let's consider 
the model for `pigs` that includes the interaction (so that the levels
of one factor compare differently at levels of the other factor).
```{r}
mod5 <- lm(inverse(conc) ~ source * factor(percent), data = pigs)
RG5 <- ref_grid(mod5)
contrast(RG5, "consec", simple = "percent")
```
In fact, we may do *all* one-factor comparisons by specifying `simple = "each"`.
This typically produces a lot of output, so use it with care.

[Back to Contents](#contents)


## Joint tests {#joint}
<!-- @index `test()`!`joint = TRUE`; `joint`; Type III tests -->
From the above, we already know how to test individual results. For pairwise comparisons (details in [the "comparisons" vignette](comparisons.html)), we might do
```{r}
PRS.source <- pairs(EMM.source)
PRS.source
```
But suppose we want an *omnibus* test that all these comparisons are zero.
Easy enough, using the `joint` argument in `test` (note: the `joint` argument
is *not* available in `summary()`; only in `test()`):
```{r}
test(PRS.source, joint = TRUE)
```
Notice that there are three comparisons, but only 2 d.f. for the test,
as cautioned in the message.

The test produced with `joint = TRUE` is a "type III" test (assuming the default
equal weights are used to obtain the EMMs). See more on these types of tests for
higher-order effects in the ["interactions" vignette section on
contrasts](interactions.html#contrasts).

####### {#joint_tests}
<!-- @index `joint_tests()`; Type III tests; Analysis of variance!Type III -->
For convenience, there is also a `joint_tests()` function that performs
joint tests of contrasts among each term in a model or `emmGrid` object.
```{r}
joint_tests(RG5)
```
The tests of main effects are of families of contrasts; those for interaction
effects are for interaction contrasts. These results are essentially the same as
a "Type-III ANOVA", but may differ in situations where there are empty cells or
other non-estimability issues, or if generalizations are present such as unequal
weighting. (Another distinction is that sums of squares and mean squares are not
shown; that is because these really are tests of contrasts among predictions,
and they may or may not correspond to model sums of squares.) 

One may use `by` variables with `joint_tests`. For example:
```{r}
joint_tests(RG5, by = "source")
```
In some models, it is possible to specify `submodel = "type2"`, thereby obtaining 
something akin to a Type II analysis of variance. See the [messy-data vignette](messy-data.html#type2submodel) for an example.


[Back to Contents](#contents)


## Testing equivalence, noninferiority, and nonsuperiority {#equiv}
<!-- @index Tests!Equivalence; Tests!Noninferiority; `test()`!`delta` -->
The `delta` argument in `summary()` or `test()` allows the user to
specify a threshold value to use in a test of equivalence, noninferiority, 
or nonsuperiority. An equivalence test is kind of a backwards significance 
test, where small *P* values are associated with small differences relative 
to a specified threshold value `delta`.
The help page for `summary.emmGrid` gives the details of 
these tests.
Suppose in the present example, we consider two sources to be equivalent if they 
are within 0.005 of each other. We can test this as follows:
```{r}
test(PRS.source, delta = 0.005, adjust = "none")
```
Using the 0.005 threshold, the *P* value is quite small for comparing soy and skim, 
providing some statistical evidence that their difference is enough smaller than the
threshold to consider them equivalent.

[Back to Contents](#contents)


## Graphics {#graphics}
Graphical displays of `emmGrid` objects are described in the 
["basics" vignette](basics.html#plots)

[Index of all vignette topics](vignette-topics.html)

---
title: "For developers: Extending **emmeans**"
author: "emmeans package, Version `r packageVersion('emmeans')`"
output: emmeans::.emm_vignette
vignette: >
  %\VignetteIndexEntry{For developers: Extending emmeans}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
```{r, echo = FALSE, results = "hide", message = FALSE}
require("emmeans")
knitr::opts_chunk$set(fig.width = 4.5, class.output = "ro")
set.seed(271828)
```

<!-- @index Vignettes!Extending **emmeans** -->

## Contents {#contents}
This vignette explains how developers may incorporate **emmeans** support in their packages.
If you are a user looking for a quick way to obtain results for an unsupported model,
you are probably better off trying to use the `qdrg()` function.

  1. [Introduction](#intro)
  2. [Data example](#dataex)
  3. [Supporting `rlm` objects](#rlm)
  4. [Supporting `lqs` objects](#lqs)
  5. [Communication between methods](#communic)
  5. [Hook functions](#hooks)
  6. [Exported methods from **emmeans**](#exported)
  7. [Existing support for `rsm` objects](#rsm)
  7. [Dispatching and restrictions](#dispatch)
  8. [Exporting and registering your methods](#exporting)
  9. [Conclusions](#concl)
  
[Index of all vignette topics](vignette-topics.html)


## Introduction {#intro}
<!-- @index `recover_data()`; `emm_basis()` -->
Suppose you want to use **emmeans** for some type of model that it doesn't (yet) support. Or, suppose you have developed a new package with a fancy model-fitting function, and you'd like it to work with **emmeans**. What can you do? Well, there is hope because **emmeans** is designed to be extended. 

The first thing to do is to look at the help page for extending the package:
```{r eval=FALSE}
help("extending-emmeans", package="emmeans")
```
It gives details about the fact that you need to write two S3 methods, `recover_data` and `emm_basis`, for the class of object that your model-fitting function returns. The `recover_data`  method is needed to recreate the dataset so that the reference grid can be identified. The `emm_basis` method then determines the linear functions needed to evaluate each point in the reference grid and to obtain associated information---such as the variance-covariance matrix---needed to do estimation and testing.

These methods must also be exported from your package so that they are available
to users. See the section on [exporting the methods](#exporting) for details and
suggestions.

This vignette presents an example where suitable methods are developed, and discusses a few issues that arise.

[Back to Contents](#contents)


## Data example {#dataex}
The **MASS** package contains various functions that do robust or outlier-resistant model fitting. We will cobble together some **emmeans** support for these. But first, let's create a suitable dataset (a simulated two-factor experiment) for testing.
```{r}
fake = expand.grid(rep = 1:5, A = c("a1","a2"), B = c("b1","b2","b3"))
fake$y = c(11.46,12.93,11.87,11.01,11.92,17.80,13.41,13.96,14.27,15.82,
           23.14,23.75,-2.09,28.43,23.01,24.11,25.51,24.11,23.95,30.37,
           17.75,18.28,17.82,18.52,16.33,20.58,20.55,20.77,21.21,20.10)
```
The `y` values were generated using predetermined means and Cauchy-distributed errors. There are some serious outliers in these data.

## Supporting `rlm` {#rlm}
<!-- @index Examples!Robust regression; Examples!`rlm` objects -->
The **MASS** package provides an `rlm` function that fits robust-regression models using *M* estimation. We'll fit a model using the default settings for all tuning parameters:
```{r}
library(MASS)
fake.rlm = rlm(y ~ A * B, data = fake)

library(emmeans)
emmeans(fake.rlm, ~ B | A)
```
The first lesson to learn about extending **emmeans** is that sometimes, it already works! It works here because `rlm` objects inherit from `lm`, which is supported by the **emmeans** package, and `rlm` objects aren't enough different to create any problems.

[Back to Contents](#contents)


## Supporting `lqs` objects {#lqs}
<!-- @index Examples!`lqs` objects -->
The **MASS** resistant-regression functions `lqs`, `lmsreg`, and `ltsreg` are another story, however. They create `lqs` objects that are not extensions of any other class, and have other issues, including not even having a `vcov` method. So for these, we really do need to write new methods for `lqs` objects. First, let's fit a model.
```{r}
fake.lts = ltsreg(y ~ A * B, data = fake)
```

### The `recover_data` method {#rd.lqs}
<!-- @index `recover_data()!for `lqs` objects@lqs -->
It is usually an easy matter to write a `recover_data` method. Look at the one for `lm` objects:
```{r}
emmeans:::recover_data.lm
```
Note that all it does is obtain the `call` component and call the method for class `call`, with additional arguments for its `terms` component and `na.action`. It happens that we can access these attributes in exactly the same way as for `lm` objects; so:
```{r}
recover_data.lqs = emmeans:::recover_data.lm
```
Let's test it:
```{r}
rec.fake = recover_data(fake.lts)
head(rec.fake)
```
Our recovered data excludes the response variable `y` (owing to the `delete.response` call), and this is fine. 

#### Special arguments {#rdargs}
<!-- @index `recover_data()!`data` and `params` arguments;
    `recover_data.call()`!`frame` argument -->
By the way, there are two special arguments `data` and `params` that may be handed to `recover_data` via `ref_grid` or `emmeans` or a related function; and you may need to provide for if you don't use the `recover_data.call` function. The `data` argument is needed to cover a desperate situation that occurs with certain kinds of models where the underlying data information is not saved with the object---e.g., models that are fitted by iteratively modifying the data. In those cases, the only way to recover the data is to for the user to give it explicitly, and `recover_data` just adds a few needed attributes to it. 

The `params` argument is needed when the model formula refers to variables besides predictors. For example, a model may include a spline term, and the knots are saved in the user's environment as a vector and referred to in the call to fit the model. In trying to recover the data, we try to construct a data frame containing all the variables present on the right-hand side of the model, but if some of those are scalars or of different lengths than the number of observations, an error occurs. So you need to exclude any names in `params` when reconstructing the data.

Many model objects contain the model frame as a slot; for example, a model fitted with
`lm(..., model = TRUE)` has a member `$model` containing the model frame. This can
be useful for recovering the data, provided none of the predictors are transformed
(when predictors are transformed, the original predictor values are not in the model
frame so it's harder to recover them). Therefore, when the model frame is
available in the model object, it should be provided in the `frame` argument of `recover_data.call()`; then when `data = NULL`, a check is made on `trms`, and if it
has no function calls, then `data` is set to `frame`. Of course, in the rarer case 
where the original data are available in the model object, specify that as `data`.

#### Error handling {#rderrs}
<!-- @index `recover_data()!Error handling -->
If you check for any error conditions in `recover_data`, simply have it return a character string with the desired message, rather than invoking `stop`. This provides a cleaner exit. The reason is that whenever `recover_data` throws an error, an informative message suggesting that `data` or `params` be provided is displayed. But a character return value is tested for and throws a different error with your string as the message.

### The `emm_basis` method {#ebreqs}
<!-- @index `emm_basis()`!Arguments and returned value -->
The `emm_basis` method has four required arguments:
```{r}
args(emmeans:::emm_basis.lm)
```
These are, respectively, the model object, its `terms` component (at least for the right-hand side of the model), a `list` of levels of the factors, and the grid of predictor combinations that specify the reference grid. 

The function must obtain six things and return them in a named `list`. They are
the matrix `X` of linear functions for each point in the reference grid, the
regression coefficients `bhat`; the variance-covariance matrix `V`; a matrix
`nbasis` for non-estimable functions; a function `dffun(k,dfargs)` for computing
degrees of freedom for the linear function `sum(k*bhat)`; and a list `dfargs` of
arguments to pass to `dffun`. Optionally, the returned list may include a
`model.matrix` element (the model matrix for the data or a compact version
thereof obtained via `.cmpMM()`), which, if included, enables the `submodel`
option.

To write your own `emm_basis` function, examining some of the existing methods can help; but the best resource is the `predict` method for the object in question, looking carefully to see what it does to predict values for a new set of predictors (e.g., `newdata` in `predict.lm`). Following this advice, let's take a look at it:
```{r}
MASS:::predict.lqs
```
###### {#eblqs}
<!-- @index `emm_basis()`!for `lqs` objects@lqs -->
Based on this, here is a listing of an `emm_basis` method for `lqs` objects:
```{r}
emm_basis.lqs = function(object, trms, xlev, grid, ...) { 
    m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
    X = model.matrix(trms, m, contrasts.arg = object$contrasts) 
    bhat = coef(object) 
    Xmat = model.matrix(trms, data=object$model)                      # 5
    V = rev(object$scale)[1]^2 * solve(t(Xmat) %*% Xmat)
    nbasis = matrix(NA) 
    dfargs = list(df = nrow(Xmat) - ncol(Xmat))
    dffun = function(k, dfargs) dfargs$df
    list(X = X, bhat = bhat, nbasis = nbasis, V = V,                  #10
         dffun = dffun, dfargs = dfargs)
}
```
Before explaining it, let's verify that it works:
```{r}
emmeans(fake.lts, ~ B | A)
```
Hooray! Note the results are comparable to those we had for `fake.rlm`, albeit the standard errors are quite a bit smaller. (In fact, the SEs could be misleading; a better method for estimating covariances should probably be implemented, but that is beyond the scope of this vignette.)

[Back to Contents](#contents)

### Dissecting `emm_basis.lqs`
Let's go through the listing of this method, line-by-line:

  * Lines 2--3: Construct the linear functions, `X`. This is a pretty standard 
    two-step process: First obtain a model frame, `m`, for the grid of
    predictors, then pass it as data to `model.matrix` to create the associated
    design matrix. As promised, this code is essentially identical to what you
    find in `predict.lqs`.
  * Line 4: Obtain the coefficients, `bhat`. Most model objects have a `coef` method.
  * Lines 5--6: Obtain the covariance matrix, `V`, of `bhat`. In many models,
    this can be obtained using the object's `vcov` method. But not in this case.
    Instead, I cobbled one together using the inverse of the **X'X** matrix
    as in ordinary regression, and the variance estimate found in the last
    element of the `scale` element of the object. This probably under-estimates
    the variances and distorts the covariances, because robust estimators have
    some efficiency loss.
  * Line 7: Compute the basis for non-estimable functions. This applies only
    when there is a possibility of rank deficiency in the model. But `lqs` methods
    don't allow rank deficiencies, so it we have fitted such a model, we can be sure that
    all linear functions are estimable; we signal that by
    setting `nbasis` equal to a 1 x 1 matrix of `NA`. If rank deficiency were
    possible, the **estimability** package (which is required by **emmeans**)
    provides a `nonest.basis` function that makes this fairly painless---I would
    have coded `nbasis = estimability::nonest.basis(Xmat)`. 
    
    There some subtleties you need to know regarding estimability. Suppose the model
    is rank-deficient, so that the design matrix **X** has *p* columns but
    rank *r* < *p*. In that case, `bhat` should be of length *p* (not *r*), and there
    should be *p* - *r* elements equal to `NA`, corresponding to columns of **X**
    that were excluded from the fit. Also, `X` should have all *p* columns. In other
    words, do not alter or throw-out columns of `X` or their corresponding elements
    of `bhat`---even those with `NA` coefficients---as they are essential for
    assessing estimability. `V` should be *r* x *r*, however---the covariance
    matrix for the non-excluded predictors.

  * Lines 8--9: Obtain `dffun` and `dfargs`. This is a little awkward because it is     designed to allow support for mixed models, where approximate methods may be
    used to obtain degrees of freedom. The function `dffun` is expected to have two
    arguments: `k`, the vector of coefficients of `bhat`, and `dfargs`, a list
    containing any additional arguments. In this case (and in many other models),
    the degrees of freedom are the same regardless of `k`. We put the required
    degrees of freedom in `dfargs` and write `dffun` so that it simply returns that
    value. (Note: If asymptotic tests and CIs are desired, return `Inf`
    degrees of freedom.)
  * Line 10: Return these results in a named list.

[Back to Contents](#contents)


## Communication between methods {#communic}
<!-- @index `emm_basis()`!Communicating with `recover_data()`;
   `recover_data()!Communicating with `emm_basis()`; `misc` attribute and argument -->
If you need to pass information obtained in `recover_data()` to the `emm_basis()` method,
simply incorporate it as `attr(data, "misc")` where `data` is the dataset returned by
`recover_data()`. Subsequently, that attribute is available in `emm_grid()` by adding a
`misc` argument.


## Hook functions {#hooks}
<!-- @index Hook functions; `emm_basis()`!Hook functions 
     `estHook`; `vcovHook`; `postGridHook` -->
Most linear models supported by **emmeans** have straightforward structure: Regression coefficients, their covariance matrix, and a set of linear functions that define the reference grid. However, a few are more complex. An example is the `clm` class in the **ordinal** package, which allows a scale model in addition to the location model. When a scale model is used, the scale parameters are included in the model matrix, regression coefficients, and covariance matrix, and we can't just use the usual matrix operations to obtain estimates and standard errors. To facilitate using custom routines for these tasks, the `emm_basis.clm` function function provided in **emmeans** includes, in its `misc` part, the names (as character constants) of two "hook" functions: `misc$estHook` has the name of the function to call when computing estimates, standard errors, and degrees of freedom (for the `summary` method); and `misc$vcovHook` has the name of the function to call to obtain the covariance matrix of the grid values (used by the `vcov` method). These functions are called in lieu of the usual built-in routines for these purposes, and return the appropriately sized matrices. 

In addition, you may want to apply some form of special post-processing after the reference grid is constructed. To provide for this, give the name of your function to post-process the object in `misc$postGridHook`. Again, `clm` objects (as well as `polr` in the **MASS** package) serve as an example. They allow a `mode` specification that in two cases, calls for post-processing. The `"cum.prob"` mode uses the `regrid` function to transform the linear predictor to the cumulative-probability scale. And the `"prob"` mode performs this, as well as applying the contrasts necessary to convert the cumulative probabilities into the class probabilities.

[Back to Contents](#contents)

## Exported methods from **emmeans** {#exported}
<!-- @index Extending **emmeans**!Exports useful to developers -->
For package developers' convenience, **emmeans** exports some of its S3 methods
for `recover_data` and/or `emm_basis`---use `methods("recover_data")` and
`methods("emm_basis")` to discover which ones. It may be that all you need is to
invoke one of those methods and perhaps make some small changes---especially if
your model-fitting algorithm makes heavy use of an existing model type supported
by **emmeans**. For those methods that are not exported, use `recover_data()`
and `.emm_basis()`, which run in **emmeans**'s namespace, thus providing access
to all available methods..

A few additional functions are exported because they may be useful to developers. They are as follows:

  * `emmeans::.all.vars(expr, retain)` Some users of your package may include `$` or
    `[[]]` operators in their model formulas. If you need to get the variable
    names, `base::all.vars` will probably not give you what you need. For
    example, if `form = ~ data$x + data[[5]]`, then `base::all.vars(form)`
    returns the names `"data"` and `"x"`, whereas `emmeans::.all.vars(form)`
    returns the names `"data$x"` and `"data[[5]]"`. The `retain` argument may be
    used to specify regular expressions for patterns to retain as parts of
    variable names.

  * `emmeans::.diag(x, nrow, ncol)` The base `diag` function has a booby trap whereby,
    for example, `diag(57.6)` returns a 57 x 57 identity matrix rather than
    a 1 x 1 matrix with 57.6 as its only element. But
    `emmeans::.diag(57.6)` will return the latter. The function works identically
    to `diag` except for its tail run around the identity-matrix trap.
  * `emmeans::.aovlist.dffun(k, dfargs)` This function is exported because it is needed
    for computing degrees of freedom for models fitted using `aov`, but it may be
    useful for other cases where Satterthwaite degrees-of-freedom calculations are
    needed. It requires the `dfargs` slot to contain analogous contents.
  * `emmeans::.get.offset(terms, grid)` If `terms` is a model formula containing an
    `offset` call, this is will compute that offset in the context of `grid` (a
    `data.frame`).
  * `emmeans::.my.vcov(object, ...)` In a call to `ref_grid`, `emmeans`, etc., the user may
    use `vcov.` to specify an alternative function or matrix to use as the
    covariance matrix of the fixed-effects coefficients. This function supports
    that feature. Calling `.my.vcov` in place of the `vcov` method will
    substitute the user's `vcov.` when it is specified.
  * `emmeans::.std.link.labels(fam, misc)` This is useful in `emm_basis` methods for
    generalized linear models. Call it with `fam` equal to the `family` object for
    your model, and `misc` either an existing list, or just `list()` if none. It
    returns a new `misc` list containing the link function and, in some cases,
    extra features that are used for certain types of link functions (e.g., for a
    log link, the setups for returning ratio comparisons with `type = "response"`).
  * `emmeans::.num.key(levs, key)` Returns integer indices of elements of 
    `key` in `levs` when `key` is a character vector; or just returns integer values
    if already integer. Also throws an error if levels are mismatched or indices exceed 
    legal range. This is useful in custom contrast functions (`.emmc` functions).
  * `emmeans::.get.excl(levs, exclude, include)` This is support for the `exclude` and 
    `include` arguments of contrast functions. It checks legality and returns an integer 
    vector of `exclude` indices in `levs`, given specified integer or character
    arguments `exclude` and `include`. In your `.emmc` function, 
    `exclude` should default to `integer(0)` and `include` should have no default.
  * `emmeans::.cmpMM(X, weights, assign)` creates a compact version of the
    model matrix `X` (or, preferably, its QR decomposition). 
    This is useful if we want an `emm_basis()` method to return a `model.matrix` element.
    The returned result is
    just the R portion of the QR decomposition of `diag(sqrt(weights)) %*% X`, with the 
    `assign` attribute added. If `X` is a `qr` object, we assume the weights are
    already incorporated, as is true of the `qr` slot of a `lm` object. 

[Back to Contents](#contents)


## Existing support for `rsm` objects {#rsm}
<!-- @index **rsm** package -->
As a nontrivial example of how an existing package supports **emmeans**, we show the support offered by the **rsm** package. Its `rsm` function  returns an `rsm` object which is an extension of the `lm` class. Part of that extension has to do with `coded.data` structures whereby, as is typical in response-surface analysis, models are fitted to variables that have been linearly transformed (coded) so that the scope of each predictor is
represented by plus or minus 1 on the coded scale. 

Without any extra support in **rsm**, `emmeans` will work just fine with `rsm` objects; but if the data are coded, it becomes awkward to present results in terms of the original predictors on their original, uncoded scale. The `emmeans`-related methods in **rsm** provide a `mode` argument that may be used to specify whether we want to work with coded or uncoded data. The possible values for `mode` are `"asis"` (ignore any codings, if present), `"coded"` (use the coded scale), and `"decoded"` (use the decoded scale). The first two are actually the same in that no decoding is done; but it seems clearer to provide separate options because they represent two different situations.

### The `recover_data` method {#rdrsm}
<!-- @index `recover_data()`!for `rsm` objects@rsm -->
Note that coding is a *predictor* transformation, not a response transformation (we could have that, too, as it's already supported by the **emmeans** infrastructure). So, to handle the `"decode"` mode, we will need to actually decode the predictors used to construct he reference grid. That means we need to make `recover_data` a lot fancier! Here it is:

```{r}
recover_data.rsm = function(object, data, mode = c("asis", "coded", "decoded"), ...) {
    mode = match.arg(mode)
    cod = rsm::codings(object)
    fcall = object$call
    if(is.null(data))                                                 # 5
        data = emmeans::recover_data(fcall, 
                   delete.response(terms(object)), object$na.action, 
                   weights = weights(object), ...)
    if (!is.null(cod) && (mode == "decoded")) {
        pred = cpred = attr(data, "predictors")
        trms = attr(data, "terms")                                    #10
        data = rsm::decode.data(rsm::as.coded.data(data, formulas = cod))
        for (form in cod) {
            vn = all.vars(form)
            if (!is.na(idx <- grep(vn[1], pred))) { 
                pred[idx] = vn[2]                                     #15
                cpred = setdiff(cpred, vn[1])
            }
        }
        attr(data, "predictors") = pred
        new.trms = update(trms, reformulate(c("1", cpred)))           #20
        attr(new.trms, "orig") = trms
        attr(data, "terms") = new.trms
        attr(data, "misc") = cod
    }
    data
}
```

Lines 2--7 ensure that `mode` is legal, retrieves the codings from the object, and obtain the results we would get from `recover_data` had it been an `lm` object. If `mode` is not `"decoded"`, *or* if no codings were used, that's all we need. Otherwise, we need to return the decoded data. However, it isn't quite that simple, because the model equation is still defined on the coded scale. Rather than to try to translate the model coefficients and covariance matrix to the decoded scale, we elected to remember what we will need to do later to put things back on the coded scale. In lines 9--10, we retrieve the attributes of the recovered data that provide the predictor names and `terms` object on the coded scale. In line 11, we replace the recovered data with the decoded data. 

By the way, the codings comprise a list of formulas with the coded name on the left and the original variable name on the right. It is possible that only some of the predictors are coded (for example, blocking factors will not be). In the `for` loop in lines 12--18, the coded predictor names are replaced with their decoded names. For technical reasons to be discussed later, we also remove these coded predictor names from a copy, `cpred`, of the list of all predictors in the coded model. In line 19, the `"predictors"` attribute of `data` is replaced with the modified version.

Now, there is a nasty technicality. The `ref_grid` function in **emmeans** has a few lines of code after `recover_data` is called that determine if any terms in the model convert covariates to factors or vice versa; and this code uses the model formula. That formula involves variables on the coded scale, and those variables are no longer present in the data, so an error will occur if it tries to access them. Luckily, if we simply take those terms out of the formula, it won't hurt because those coded predictors would not have been converted in that way. So in line 20, we update `trms` with a simpler model with the coded variables excluded (the intercept is explicitly included to ensure there will be a right-hand side even is `cpred` is empty). We save that as the `terms` attribute, and the original terms as a new `"orig"` attribute to be retrieved later. The `data` object, modified or not, is returned. If data have been decoded, `ref_grid` will construct its grid using decoded variables.

In line 23, we save the codings as the `"misc"` attribute, to be accessed later by `emm_basis()`.

### The `emm_basis` method {#ebrsm}
<!-- @index `emm_basis()`!for `rsm` objects@rsm -->
Now comes the `emm_basis` method that will be called after the grid is defined. 
It is listed below:
```{r}
emm_basis.rsm = function(object, trms, xlev, grid, 
                         mode = c("asis", "coded", "decoded"), misc, ...) {
    mode = match.arg(mode)
    cod = misc
    if(!is.null(cod) && mode == "decoded") {                          # 5
        grid = rsm::coded.data(grid, formulas = cod)
        trms = attr(trms, "orig")
    }
    
    m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)     #10
    X = model.matrix(trms, m, contrasts.arg = object$contrasts)
    bhat = as.numeric(object$coefficients) 
    V = emmeans::.my.vcov(object, ...)
    
    if (sum(is.na(bhat)) > 0)                                         #15
        nbasis = estimability::nonest.basis(object$qr)
    else
        nbasis = estimability::all.estble
    dfargs = list(df = object$df.residual)
    dffun = function(k, dfargs) dfargs$df                             #20

    list(X = X, bhat = bhat, nbasis = nbasis, V = V, 
         dffun = dffun, dfargs = dfargs, misc = list())
}
```

This is much simpler. The coding formulas are obtained from `misc` (line 4) so that we don't have to re-obtain them from the object. All we have to do is determine if decoding was done (line 5); and, if so, convert the grid back to the coded scale (line 6) and recover the original `terms` attribute (line 7). The rest is borrowed directly from the `emm_basis.lm` method in **emmeans**. Note that line 13 uses one of the exported functions we described in the preceding section. Lines 15--18 use functions from the **estimability** package to handle the possibility that the model is rank-deficient.


### A demonstration {#demo}
Here's a demonstration of this **rsm** support. The standard example for `rsm` fits a second-order model `CR.rs2` to a dataset organized in two blocks and with two coded predictors.
```{r results = "hide", warning = FALSE, message = FALSE}
library("rsm")
example("rsm")   ### (output is not shown) ###
```

First, let's look at some results on the coded scale---which are the same as for an ordinary `lm` object.
```{r}
emmeans(CR.rs2, ~ x1 * x2, mode = "coded", 
        at = list(x1 = c(-1, 0, 1), x2 = c(-2, 2)))
```
Now, the coded variables `x1` and `x2` are derived from these coding formulas for predictors `Time` and `Temp`:
```{r}
codings(CR.rs1)
```
Thus, for example, a coded value of `x1 = 1` corresponds to a time of 85 + 1 x 5 = 90. Here are some results working with decoded predictors. Note that the `at` list must now be given in terms of `Time` and `Temp`:
```{r}
emmeans(CR.rs2, ~ Time * Temp, mode = "decoded", 
        at = list(Time = c(80, 85, 90), Temp = c(165, 185)))
```
Since the supplied settings are the same on the decoded scale as were used on the coded scale, the EMMs are identical to those in the previous output.


## Dispatching and restrictions {#dispatch}
<!-- @index Extending **emmeans**!Restrictions; 
    `recover_data()`!Dispatching; `emm_basis()`!Dispatching -->
The **emmeans** package has internal support for a number of model classes.
When `recover_data()` and `emm_basis()` are dispatched, a search is made
for external methods for a given class; and if found, those methods are
used instead of the internal ones. However, certain restrictions apply
when you aim to override an existing internal method:

  1. The class name being extended must appear in the first or second
     position in the results of `class(object)`. That is, you may have a
     base class for which you provide `recover_data()` and `emm_basis()` 
     methods, and those will also work for *direct* descendants thereof;
     but any class in third place or later in the inheritance is ignored.
  2. Certain classes vital to the correct operation of the package, e.g.,
     `"lm"`, `"glm"`, etc., may not be overridden.
     
If there are no existing internal methods for the class(es) you provide
methods for, there are no restrictions on them.


## Exporting and registering your methods {#exporting}
<!-- @index **emmeans** package!Exporting extensions to 
     Registering `recover_data` and `emm_basis` methods -->
To make the methods available to users of your package, the methods must be exported.
R and CRAN are evolving in a way that having S3 methods in the registry is
increasingly important; so it is a good idea to provide for that.
The problem is not all of your package users will have **emmeans** installed.

Thus, registering the methods must be done conditionally. We provide a
courtesy function `.emm_register()` to make this simple. 
Suppose that your package offers two model classes `foo` and `bar`, and it
includes the corresponding functions `recover_data.foo`, `recover_data.bar`,
`emm_basis.foo`, and `emm_basis.bar`.
Then to register these methods, add or modify the
`.onLoad` function in your package (traditionally saved in the source file `zzz.R`):
```r
.onLoad <- function(libname, pkgname) {
    if (requireNamespace("emmeans", quietly = TRUE))
        emmeans::.emm_register(c("foo", "bar"), pkgname)
}

```
You should also add `emmeans (>= 1.4)` and `estimability` (which is required by **emmeans**)
to the `Suggests` field of your `DESCRIPTION` file.


[Back to Contents](#contents)


## Conclusions {#concl}
It is relatively simple to write appropriate methods that work with **emmeans** for model objects it does not support. I hope this vignette is helpful for understanding how. Furthermore, if you are the developer of a package that fits linear models, I encourage you to include `recover_data` and `emm_basis` methods for those classes of objects, so that users have access to **emmeans** support.

[Back to Contents](#contents)


[Index of all vignette topics](vignette-topics.html)

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{nutrition}
\alias{nutrition}
\title{Nutrition data}
\format{
A data frame with 107 observations and 4 variables:
\describe{
  \item{\code{age}}{a factor with levels \code{1}, \code{2}, \code{3},
    \code{4}. Mother's age group.}
  \item{\code{group}}{a factor with levels \code{FoodStamps}, \code{NoAid}.
    Whether or not the family receives food stamp assistance.}
  \item{\code{race}}{a factor with levels \code{Black}, \code{Hispanic},
    \code{White}. Mother's race.}
  \item{\code{gain}}{a numeric vector (the response variable). Gain score
    (posttest minus pretest) on knowledge of nutrition.}
}
}
\source{
Milliken, G. A. and Johnson, D. E. (1984)
\emph{Analysis of Messy Data -- Volume I: Designed Experiments}. 
Van Nostrand, ISBN 0-534-02713-7.
}
\usage{
nutrition
}
\description{
This observational dataset involves three factors, but where several factor 
combinations are missing. It is used as a case study in Milliken and Johnson,
Chapter 17, p.202. (You may also find it in the second edition, p.278.)
}
\details{
A survey was conducted by home economists ``to study how much
lower-socioeconomic-level mothers knew about nutrition and to judge the
effect of a training program designed to increase their knowledge of
nutrition.'' This is a messy dataset with several empty cells.
}
\examples{
nutr.aov <- aov(gain ~ (group + age + race)^2, data = nutrition)

# Summarize predictions for age group 3
nutr.emm <- emmeans(nutr.aov, ~ race * group, at = list(age="3"))
                   
emmip(nutr.emm, race ~ group)

# Hispanics seem exceptional; but this doesn't test out due to very sparse data
pairs(nutr.emm, by = "group")
pairs(nutr.emm, by = "race")
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xtable-method.R
\name{xtable.emmGrid}
\alias{xtable.emmGrid}
\alias{xtable.summary_emm}
\alias{print.xtable_emm}
\title{Using \code{xtable} for EMMs}
\usage{
\method{xtable}{emmGrid}(x, caption = NULL, label = NULL, align = NULL,
  digits = 4, display = NULL, auto = FALSE, ...)

\method{xtable}{summary_emm}(x, caption = NULL, label = NULL,
  align = NULL, digits = 4, display = NULL, auto = FALSE, ...)

\method{print}{xtable_emm}(x, type = getOption("xtable.type", "latex"),
  include.rownames = FALSE, sanitize.message.function = footnotesize, ...)
}
\arguments{
\item{x}{Object of class \code{emmGrid}}

\item{caption}{Passed to \code{\link[xtable]{xtableList}}}

\item{label}{Passed to \code{xtableList}}

\item{align}{Passed to \code{xtableList}}

\item{digits}{Passed to \code{xtableList}}

\item{display}{Passed to \code{xtableList}}

\item{auto}{Passed to \code{xtableList}}

\item{...}{Arguments passed to \code{\link{summary.emmGrid}}}

\item{type}{Passed to \code{\link[xtable]{print.xtable}}}

\item{include.rownames}{Passed to \code{print.xtable}}

\item{sanitize.message.function}{Passed to \code{print.xtable}}
}
\value{
The \code{xtable} methods return an \code{xtable_emm}
object, for which its print method is \code{print.xtable_emm} .
}
\description{
These methods provide support for the \pkg{xtable} package, enabling 
polished presentations of tabular output from \code{\link{emmeans}}
and other functions.
}
\details{
The methods actually use \code{\link[xtable]{xtableList}},
because of its ability to display messages such as those for P-value
adjustments. These methods return an object of class \code{"xtable_emm"} --
an extension of \code{"xtableList"}. Unlike other \code{xtable} methods, the
number of digits defaults to 4; and degrees of freedom and \emph{t} ratios
are always formatted independently of \code{digits}. The \code{print} method
uses \code{\link[xtable:xtableList]{print.xtableList}}, and any \code{\dots} arguments are
passed there.
}
\examples{
if(requireNamespace("xtable"))
    emm_example("xtable")
    # Use emm_example("xtable", list = TRUE) # to just list the code
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emmGrid-methods.R
\name{regrid}
\alias{regrid}
\title{Reconstruct a reference grid with a new transformation or simulations}
\usage{
regrid(object, transform = c("response", "mu", "unlink", "none", "pass",
  links), inv.link.lbl = "response", predict.type,
  bias.adjust = get_emm_option("back.bias.adj"), sigma, N.sim,
  sim = mvtnorm::rmvnorm, ...)
}
\arguments{
\item{object}{An object of class \code{emmGrid}}

\item{transform}{Character, list, or logical value. If \code{"response"},
\code{"mu"}, or \code{TRUE}, the inverse transformation is applied to the
estimates in the grid (but if there is both a link function and a response
transformation, \code{"mu"} back-transforms only the link part); if
\code{"none"} or \code{FALSE}, \code{object} is re-gridded so that its
\code{bhat} slot contains \code{predict(object)} and its \code{linfct} slot
is the identity. Any internal transformation information is preserved. If
\code{transform = "pass"}, the object is not re-gridded in any way (this
may be useful in conjunction with \code{N.sim}).

If \code{transform} is a character value in \code{links} (which is the set
of valid arguments for the \code{\link{make.link}} function, excepting
\code{"identity"}), or if \code{transform} is a list of the same form as
returned by \code{make.links} or \code{\link{make.tran}}, the results are
formulated as if the response had been transformed with that link function.}

\item{inv.link.lbl}{Character value. This applies only when \code{transform} 
is in \code{links}, and is used to label the predictions if subsequently summarized
with \code{type = "response"}.}

\item{predict.type}{Character value. If provided, the returned object is
updated with the given type to use by default by \code{summary.emmGrid}
(see \code{\link{update.emmGrid}}).  This may be useful if, for example,
when one specifies \code{transform = "log"} but desires summaries to be
produced by default on the response scale.}

\item{bias.adjust}{Logical value for whether to adjust for bias in
back-transforming (\code{transform = "response"}). This requires a valid value of 
\code{sigma} to exist in the object or be specified.}

\item{sigma}{Error SD assumed for bias correction (when 
\code{transform = "response"} and a transformation
is in effect). If not specified,
\code{object@misc$sigma} is used, and a warning is issued if it is not found.}

\item{N.sim}{Integer value. If specified and \code{object} is based on a 
frequentist model (i.e., does not have a posterior sample), then a fake 
posterior sample is generated using the function \code{sim}.}

\item{sim}{A function of three arguments (no names are assumed).
If \code{N.sim} is supplied with a frequentist model, this function is called
with respective arguments \code{N.sim}, \code{object@bhat}, and \code{object@V}.
The default is the multivariate normal distribution.}

\item{...}{Ignored.}
}
\value{
An \code{emmGrid} object with the requested changes
}
\description{
The typical use of this function is to cause EMMs to be computed on
a different scale, e.g., the back-transformed scale rather than the 
linear-predictor scale. In other words, if you want back-transformed 
results, do you want to average and then back-transform, or 
back-transform and then average?
}
\details{
The \code{regrid} function reparameterizes an existing \code{ref.grid} so
that its \code{linfct} slot is the identity matrix and its \code{bhat} slot
consists of the estimates at the grid points. If \code{transform} is
\code{TRUE}, the inverse transform is applied to the estimates. Outwardly,
when \code{transform = "response"}, the result of \code{\link{summary.emmGrid}}
after applying \code{regrid} is identical to the summary of the original
object using \samp{type="response"}. But subsequent EMMs or
contrasts will be conducted on the new scale -- which is
the reason this function exists. 

This function may also be used to simulate a sample of regression
coefficients for a frequentist model for subsequent use as though it were a
Bayesian model. To do so, specify a value for \code{N.sim} and a sample is
simulated using the function \code{sim}. The grid may be further processed in
accordance with the other arguments; or if \code{transform = "pass"}, it is
simply returned with the only change being the addition of the simulated
sample.
}
\note{
Another way to use \code{regrid} is to supply a \code{regrid} 
  argument to \code{\link{ref_grid}} (either directly of indirectly via
  \code{\link{emmeans}}), in which case its value is passed to \code{regrid} as
  \code{transform}. This is often a simpler approach if the reference
  grid has not already been constructed.
}
\section{Degrees of freedom}{
  
In cases where the
degrees of freedom depended on the linear function being estimated (e.g.,
Satterthwaite method), the d.f.
from the reference grid are saved, and a kind of \dQuote{containment} method
is substituted in the returned object, whereby the calculated d.f. for a new
linear function will be the minimum d.f. among those having nonzero
coefficients. This is kind of an \emph{ad hoc} method, and it can
over-estimate the degrees of freedom in some cases. An annotation is
displayed below any subsequent summary results stating that the 
degrees-of-freedom method is inherited from the previous method at
the time of re-gridding.
}

\examples{
pigs.lm <- lm(log(conc) ~ source + factor(percent), data = pigs)
rg <- ref_grid(pigs.lm)

# This will yield EMMs as GEOMETRIC means of concentrations:
(emm1 <- emmeans(rg, "source", type = "response"))
pairs(emm1) ## We obtain RATIOS

# This will yield EMMs as ARITHMETIC means of concentrations:
(emm2 <- emmeans(regrid(rg, transform = "response"), "source"))
pairs(emm2)  ## We obtain DIFFERENCES
# Same result, useful if we hadn't already created 'rg'
# emm2 <- emmeans(pigs.lm, "source", regrid = "response")

# Simulate a sample of regression coefficients
set.seed(2.71828)
rgb <- regrid(rg, N.sim = 200, transform = "pass")
emmeans(rgb, "source", type = "response")  ## similar to emm1
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emmeans.R
\name{emmobj}
\alias{emmobj}
\title{Construct an \code{emmGrid} object from scratch}
\usage{
emmobj(bhat, V, levels, linfct = diag(length(bhat)), df = NA, dffun,
  dfargs = list(), post.beta = matrix(NA), nesting = NULL, se.bhat,
  se.diff, ...)
}
\arguments{
\item{bhat}{Numeric. Vector of regression coefficients}

\item{V}{Square matrix. Covariance matrix of \code{bhat}}

\item{levels}{Named list or vector. Levels of factor(s) that define the
estimates defined by \code{linfct}. If not a list, we assume one factor
named \code{"level"}}

\item{linfct}{Matrix. Linear functions of \code{bhat} for each combination 
of \code{levels}.}

\item{df}{Numeric value or function with arguments \code{(x, dfargs)}. If a
number, that is used for the degrees of freedom. If a function, it should
return the degrees of freedom for \code{sum(x*bhat)}, with any additional
parameters in \code{dfargs}.}

\item{dffun}{Overrides \code{df} if specified. This is a convenience
to match the slot names of the returned object.}

\item{dfargs}{List containing arguments for \code{df}.
This is ignored if df is numeric.}

\item{post.beta}{Matrix whose columns comprise a sample from the posterior
distribution of the regression coefficients (so that typically, the column
averages will be \code{bhat}). A 1 x 1 matrix of \code{NA} indicates that
such a sample is unavailable.}

\item{nesting}{Nesting specification as in \code{\link{ref_grid}}. This is
ignored if \code{model.info} is supplied.}

\item{se.bhat, se.diff}{Alternative way of specifying \code{V}. If \code{se.bhat}
is specified, \code{V} is constructed using \code{se.bhat}, the standard errors of \code{bhat},
and \code{se.diffs}, the standard errors of its pairwise differences. \code{se.diff}
should be a vector of length \code{k*(k-1)/2} where \code{k} is the length
of \code{se.bhat}, and its elements should be in the order \code{12,13,...,1k,23,...2k,...}.
If \code{se.diff} is missing, \code{V} is computed as if the \code{bhat} are independent.}

\item{...}{Arguments passed to \code{\link{update.emmGrid}}}
}
\value{
An \code{emmGrid} object
}
\description{
This allows the user to incorporate results obtained by some analysis
into an \code{emmGrid} object, enabling the use of \code{emmGrid} methods
to perform related follow-up analyses.
}
\details{
The arguments must be conformable. This includes that the length of
 \code{bhat}, the number of columns of \code{linfct}, and the number of
 columns of \code{post.beta} must all be equal. And that the product of
 lengths in \code{levels} must be equal to the number of rows of
 \code{linfct}. The \code{grid} slot of the returned object is generated 
 by \code{\link{expand.grid}} using \code{levels} as its arguments. So the
 rows of \code{linfct} should be in corresponding order.
 
The functions \code{qdrg} and \code{\link{emmobj}} are close cousins, in that
they both produce \code{emmGrid} objects. When starting with summary
statistics for an existing grid, \code{emmobj} is more useful, while
\code{qdrg} is more useful when starting from an unsupported fitted model.
}
\examples{
# Given summary statistics for 4 cells in a 2 x 2 layout, obtain 
# marginal means and comparisons thereof. Assume heteroscedasticity
# and use the Satterthwaite method
levels <- list(trt = c("A", "B"), dose = c("high", "low"))
ybar <- c(57.6, 43.2, 88.9, 69.8)
s <-    c(12.1, 19.5, 22.8, 43.2)
n <-    c(44,   11,   37,   24)
se2 = s^2 / n
Satt.df <- function(x, dfargs)
    sum(x * dfargs$v)^2 / sum((x * dfargs$v)^2 / (dfargs$n - 1))
    
expt.emm <- emmobj(bhat = ybar, V = diag(se2),
    levels = levels, linfct = diag(c(1, 1, 1, 1)),
    df = Satt.df, dfargs = list(v = se2, n = n), estName = "mean")

( trt.emm <- emmeans(expt.emm, "trt") )
( dose.emm <- emmeans(expt.emm, "dose") )

rbind(pairs(trt.emm), pairs(dose.emm), adjust = "mvt")

### Create an emmobj from means and SEs
### (This illustration reproduces the MOats example for Variety = "Victory")
means = c(71.50000,  89.66667, 110.83333, 118.50000)
semeans = c(5.540591, 6.602048, 8.695358, 7.303221)
sediffs = c(7.310571,  9.894724,  7.463615, 10.248306,  4.935698,  8.694507)
foo = emmobj(bhat = means, se.bhat = semeans, se.diff = sediffs, 
             levels = list(nitro = seq(0, .6, by = .2)), df = 10)
plot(foo, comparisons = TRUE)

}
\seealso{
\code{\link{qdrg}}, an alternative that is useful when starting 
  with a fitted model not supported in \pkg{emmeans}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transformations.R
\name{make.tran}
\alias{make.tran}
\alias{inverse}
\title{Response-transformation extensions}
\usage{
make.tran(type = c("genlog", "power", "boxcox", "sympower", "asin.sqrt",
  "atanh", "bcnPower", "scale"), alpha = 1, beta = 0, param, y, inner, ...)

inverse(y)
}
\arguments{
\item{type}{The name of a standard transformation supported by \code{stat::make.link},
or of a special transformation described under Details.}

\item{alpha, beta}{Numeric parameters needed for special transformations.}

\item{param}{If non-missing, this specifies either
\code{alpha} or \code{c(alpha, beta)} (provided for backward compatibility).
Also, for the same reason, if \code{alpha} is of length more than 1,
it is taken as \code{param}.}

\item{y}{A numeric response variable used (\emph{and required}) with \code{type = "scale"}, 
where \code{scale(y)} determines \code{alpha} and \code{beta}.}

\item{inner}{another transformation. See the section on compound transformations}

\item{...}{Additional arguments passed to other functions/methods}
}
\value{
A \code{list} having at least the same elements as those returned by
  \code{\link{make.link}}. The \code{linkfun} component is the transformation
  itself. Each of the functions is associated with an environment where any 
  parameter values are defined.

\code{inverse} returns the reciprocal of its argument. It allows
  the \code{"inverse"} link to be auto-detected as a response transformation.
}
\description{
The \code{make.tran} function creates the needed information to perform
transformations of the response
variable, including inverting the transformation and estimating variances of
back-transformed predictions via the delta method. \code{make.tran} is
similar to \code{\link{make.link}}, but it covers additional transformations.
The result can be used as an environment in which the model is fitted, or as
the \code{tran} argument in \code{\link{update.emmGrid}} (when the given
transformation was already applied in an existing model).
}
\note{
The \code{genlog} transformation is technically unneeded, because
  a response transformation of the form \code{log(y + c)} is now auto-detected 
  by \code{\link{ref_grid}}.

We modify certain \code{\link{make.link}} results in transformations
  where there is a restriction on valid prediction values, so that reasonable
  inverse predictions are obtained, no matter what. For example, if a
  \code{sqrt} transformation was used but a predicted value is negative, the
  inverse transformation is zero rather than the square of the prediction. A
  side effect of this is that it is possible for one or both confidence
  limits, or even a standard error, to be zero.
}
\section{Details}{

The \code{make.tran} function returns a
suitable list of functions for several popular transformations. Besides being
usable with \code{update}, the user may use this list as an enclosing
environment in fitting the model itself, in which case the transformation is
auto-detected when the special name \code{linkfun} (the transformation
itself) is used as the response transformation in the call. See the examples
below.

The primary purpose of \code{make.tran} is to support transformations that
require additional parameters, specified as \code{alpha} and \code{beta};
these are the onse shown in the argument-matching list. However, standard
transformations supported by \code{stats::make.link} are also supported.
In the following discussion of ones requiring parameters, 
we use \eqn{\alpha} and \eqn{\beta} to
denote \code{alpha} and \code{beta}, and \eqn{y} to denote the response variable.
The \code{type} argument specifies the following transformations:
\describe{
\item{\code{"genlog"}}{Generalized logarithmic transformation: \eqn{\log_\beta(y +
  \alpha)}, where \eqn{y > -\alpha}.
  When \eqn{\beta = 0} (the default), we use \eqn{\log_e(y + \alpha)}}
\item{\code{"power"}}{Power transformation: \eqn{(y-\beta)^\alpha}, where \eqn{y > \beta}.
  When \eqn{\alpha = 0}, \eqn{\log(y-\beta)} is used instead.}
\item{\code{"boxcox"}}{The Box-Cox transformation (unscaled by the geometric
  mean): \eqn{((y - \beta)^\alpha - 1) / \alpha}, where \eqn{y > \beta}. 
  When \eqn{\alpha = 0}, \eqn{\log(y - \beta)}
  is used.}
\item{\code{"sympower"}}{A symmetrized power transformation on the whole real
  line:
  \eqn{|y - \beta|^\alpha\cdot sign(y - \beta)}. There are no restrictions on \eqn{y}, but we
  require \eqn{\alpha > 0} in order for the transformation to be monotone and
  continuous.}
\item{\code{"asin.sqrt"}}{Arcsin-square-root transformation:
  \eqn{\sin^{-1}(y/\alpha)^{1/2}}. Typically, \code{alpha} will be either 1 (default) or 100.}
\item{\code{"atanh"}}{Arctanh transformation:
  \eqn{\tanh^{-1}(y/\alpha)}. Typically, \code{alpha} will be either 1 (default) or 100.}
\item{\code{"bcnPower"}}{Box-Cox with negatives allowed, as described for the 
  \code{bcnPower} function in the \pkg{car} package. It is defined as the Box-Cox
  transformation \eqn{(z^\alpha - 1) / \alpha} of the variable \eqn{z = y + (y^2+\beta^2)^{1/2}}. 
  Note that this requires both parameters and that \code{beta > 0}.}
\item{\code{"scale"}}{This one is a little different than the others, in that
  \code{alpha} and \code{beta} are ignored; instead, they are determined by calling 
  \code{scale(y, ...)}. The user should give as \code{y} the response variable in the
  model to be fitted to its scaled version.}

}

Note that with the \code{"power"}, \code{"boxcox"}, or \code{"sympower"} transformations, 
the argument \code{beta} specifies a location shift. 
In the \code{"genpower"} transformation, \code{beta} specifies
the base of the logarithm -- however, quirkily, the default of \code{beta = 0}
is taken to be the natural logarithm. For example,
\code{make.tran(0.5, 10)} sets up the \eqn{\log_{10}(y + \frac12)}
transformation. In the \code{"bcnPower"} transformation, \code{beta}
must be specified as a positive value.

For purposes of back-transformation, the \samp{sqrt(y) + sqrt(y+1)}
transformation is treated exactly the same way as \samp{2*sqrt(y)}, because
both are regarded as estimates of \eqn{2\sqrt\mu}.
}

\section{Cases where \code{make.tran} may not be needed}{

For standard transformations with no parameters, we usually don't need to use
\code{make.tran}; just the name of the transformation is all that is needed.
The functions \code{\link{emmeans}}, \code{\link{ref_grid}}, and related ones
automatically detect response transformations that are recognized by
examining the model formula. These are \code{log}, \code{log2}, \code{log10},
\code{log1p},
\code{sqrt}, \code{logit}, \code{probit}, \code{cauchit}, \code{cloglog}; as
well as (for a response variable \code{y}) \code{asin(sqrt(y))},
\code{asinh(sqrt(y))}, \code{atanh(y)}, and \code{sqrt(y) + sqrt(y+1)}. 
In addition, any
constant multiple of these (e.g., \code{2*sqrt(y)}) is auto-detected and
appropriately scaled (see also the \code{tran.mult} argument in
\code{\link{update.emmGrid}}).

A few additional transformations may be specified as character strings and
are auto-detected: \code{"identity"}, \code{"1/mu^2"},
\code{"inverse"}, \code{"reciprocal"}, \code{"log10"}, \code{"log2"},
\code{"asin.sqrt"}, \code{"asinh.sqrt"}, and \code{"atanh"}.
}

\section{Compound transformations}{

A transformation that is a function of another function can be created by
specifying \code{inner} for the other function. For example, the
transformation \eqn{1/\sqrt{y}} can be created either by
\code{make.tran("inverse", inner = "sqrt")} or by \code{make.tran("power",
-0.5)}. In principle, transformations can be compounded to any depth.
Also, if \code{type} is \code{"scale"}, \code{y} is replaced by 
\code{inner$linkfun(y)}, because that will be the variable that is scaled.
}

\examples{
# Fit a model using an oddball transformation:
bctran <- make.tran("boxcox", 0.368)
warp.bc <- with(bctran, 
    lm(linkfun(breaks) ~ wool * tension, data = warpbreaks))
# Obtain back-transformed LS means:    
emmeans(warp.bc, ~ tension | wool, type = "response")

### Using a scaled response...
# Case where it is auto-detected:
mod <- lm(scale(yield[, 1]) ~ Variety, data = MOats)
emmeans(mod, "Variety", type = "response")

# Case where scaling is not auto-detected -- and what to do about it:
copt <- options(contrasts = c("contr.sum", "contr.poly"))
mod.aov <- aov(scale(yield[, 1]) ~ Variety + Error(Block), data = MOats)
emm.aov <- suppressWarnings(emmeans(mod.aov, "Variety", type = "response"))

# Scaling was not retrieved, but we can do:
emm.aov <- update(emm.aov, tran = make.tran("scale", y = MOats$yield[, 1]))
emmeans(emm.aov, "Variety", type = "response")

### Compound transformations
# The following amount to the same thing:
t1 <- make.tran("inverse", inner = "sqrt")
t2 <- make.tran("power", -0.5)

options(copt)


\dontrun{
### An existing model 'mod' was fitted with a y^(2/3) transformation...
  ptran = make.tran("power", 2/3)
  emmeans(mod, "treatment", tran = ptran)
}

pigs.lm <- lm(inverse(conc) ~ source + factor(percent), data = pigs)
emmeans(pigs.lm, "source", type = "response")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{oranges}
\alias{oranges}
\title{Sales of oranges}
\format{
A data frame with 36 observations and 6 variables:
\describe{
  \item{\code{store}}{a factor with levels \code{1} \code{2} \code{3}
    \code{4} \code{5} \code{6}. The store that was observed.}
  \item{\code{day}}{a factor with levels \code{1} \code{2} \code{3}
    \code{4} \code{5} \code{6}. The day the observation was taken (same for
    each store).}
  \item{\code{price1}}{a numeric vector. Price of variety 1.}
  \item{\code{price2}}{a numeric vector. Price of variety 2.}
  \item{\code{sales1}}{a numeric vector. Sales (per customer) of variety 1.}
  \item{\code{sales2}}{a numeric vector. Sales (per customer) of variety 2.}
}
}
\source{
This is (or once was) available as a SAS sample dataset.
}
\usage{
oranges
}
\description{
This example dataset on sales of oranges has two factors, two covariates, and
two responses. There is one observation per factor combination.
}
\examples{
# Example on p.244 of Littell et al.
oranges.lm <- lm(sales1 ~ price1*day, data = oranges)
emmeans(oranges.lm, "day")

# Example on p.246 of Littell et al.
emmeans(oranges.lm, "day", at = list(price1 = 0))

# A more sensible model to consider, IMHO (see vignette("interactions"))
org.mlm <- lm(cbind(sales1, sales2) ~ price1 * price2 + day + store, 
              data = oranges)
}
\references{
Littell, R., Stroup W., Freund, R. (2002) \emph{SAS For Linear Models} (4th
edition). SAS Institute. ISBN 1-59047-023-0.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cld-emm.R
\name{cld.emmGrid}
\alias{cld.emmGrid}
\alias{cld.emm_list}
\title{Compact letter displays}
\usage{
\method{cld}{emmGrid}(object, details = FALSE, sort = TRUE, by,
  alpha = 0.05, Letters = c("1234567890", LETTERS, letters),
  reversed = decreasing, decreasing = FALSE, signif.sets = FALSE,
  delta = 0, ...)

\method{cld}{emm_list}(object, ..., which = 1)
}
\arguments{
\item{object}{An object of class \code{emmGrid}}

\item{details}{Logical value determining whether detailed information on tests of
pairwise comparisons is displayed}

\item{sort}{Logical value determining whether the EMMs are sorted before the comparisons
are produced. When \code{TRUE}, the results are displayed according to
\code{reversed}.}

\item{by}{Character value giving the name or names of variables by which separate
families of comparisons are tested. If NULL, all means are compared.
If missing, the object's \code{by.vars} setting, if any, is used.}

\item{alpha}{Numeric value giving the significance level for the comparisons}

\item{Letters}{Character vector of letters to use in the display. Any strings of
length greater than 1 are expanded into individual characters}

\item{reversed, decreasing}{Logical value (passed to \code{multcompView::multcompLetters}.)
If \code{TRUE}, the order of use of the letters is reversed.
Either \code{reversed} or \code{decreasing} may be specified, thus providing
compatibility with both \code{multcompView::multcompLetters(..., reversed, ...)} and
\code{multcomp::cld(..., decreasing, ...)}.
In addition, if both \code{sort} and \code{reversed} are TRUE, the sort
order of results is reversed.}

\item{signif.sets}{Logical value. If \code{FALSE} (and \code{delta = 0}), a 
\sQuote{traditional}
compact-letter display is constructed with groupings representing sets of
estimates that are not statistically different. If \code{TRUE}, the criteria
are reversed so that two estimates sharing the same symbol test as significantly
different. See also \code{delta}.}

\item{delta}{Numeric value passed to \code{\link{test.emmGrid}}. If this
is positive, it is used as an equivalence threshold in the TOST procedure for
two-sided equivalence testing. In the resulting compact letter display,
two estimates share the same grouping letter only if they are found to be
statistically equivalent -- that is, groupings reflect actual \emph{findings}
of equivalence rather than failure to find a significant difference.
When \code{delta} is nonzero, \code{signif.sets} is ignored.}

\item{...}{Arguments passed to \code{\link{contrast}} (for example,
an \code{adjust} method)}

\item{which}{Which element of the \code{emm_list} object to process
(If length exceeds one, only the first one is used)}
}
\value{
A \code{\link[=summary.emmGrid]{summary_emm}} object showing the estimated marginal means
plus an additional column labeled \code{.group} (when \code{signif.sets = FALSE}), 
\code{.signif.set} (when \code{signif.sets = TRUE}), or \code{.equiv.set} 
(when \code{delta > 0}).
}
\description{
A method for \code{multcomp::cld()} is provided for users desiring to produce 
compact-letter displays (CLDs). 
This method uses the Piepho (2004) algorithm (as implemented in the
\pkg{multcompView} package) to generate a compact letter display of all
pairwise comparisons of estimated marginal means. The function obtains (possibly
adjusted) P values for all pairwise comparisons of means, using the
\code{\link{contrast}} function with \code{method = "pairwise"}. When a P
value exceeds \code{alpha}, then the two means have at least one letter in
common.
}
\note{
We warn that the default display encourages a poor
practice in interpreting significance tests. Such CLDs are misleading because they
visually group means with comparisons \emph{P} > \code{alpha} as though they 
are equal, when in fact we have only failed to prove that they differ.
A better alternative if one wants to show groupings is to specify an equivalence
threshold \code{delta}; then groupings will be based on actual findings of
equivalence. Another way to display actual findings is to set
\code{signif.sets = TRUE}, so that estimates in the same group are those 
found to be statistically \emph{different}. Obviously, these different options
require different interpretations of the results; the annotations and the label
given the final column help guide how to assess the results.

As further alternatives, consider \code{\link{pwpp}} (graphical display of \emph{P} 
values) or \code{\link{pwpm}} (matrix display).
}
\examples{
if(requireNamespace("multcomp"))
    emm_example("cld-multcomp")
    # Use emm_example("cld-multcomp", list = TRUE) # to just list the code

}
\references{
Piepho, Hans-Peter (2004) An algorithm for a letter-based 
  representation of all pairwise comparisons, 
  Journal of Computational and Graphical Statistics, 
  13(2), 456-466.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interfacing.R, R/zzz.R, R/helpers.R,
%   R/0nly-internal.R, R/aovlist-support.R, R/emm-contr.R, R/countreg-support.R
\name{extending-emmeans}
\alias{extending-emmeans}
\alias{recover_data}
\alias{recover_data.call}
\alias{emm_basis}
\alias{.emm_register}
\alias{.std.link.labels}
\alias{.combine.terms}
\alias{.aovlist.dffun}
\alias{.cmpMM}
\alias{.get.excl}
\alias{.get.offset}
\alias{.my.vcov}
\alias{.all.vars}
\alias{.diag}
\alias{.num.key}
\alias{.emm_vignette}
\alias{.hurdle.support}
\alias{.zi.support}
\title{Support functions for model extensions}
\usage{
recover_data(object, ...)

\method{recover_data}{call}(object, trms, na.action, data = NULL,
  params = "pi", frame, pwts, addl.vars, ...)

emm_basis(object, trms, xlev, grid, ...)

.emm_register(classes, pkgname, qdrg = FALSE)

.std.link.labels(fam, misc)

.combine.terms(...)

.aovlist.dffun(k, dfargs)

.cmpMM(X, weights = rep(1, nrow(X)), assign = attr(X$qr, "assign"))

.get.excl(levs, exc, inc)

.get.offset(terms, grid)

.my.vcov(object, vcov. = .statsvcov, ...)

.all.vars(expr, retain = c("\\\\$", "\\\\[\\\\[", "\\\\]\\\\]", "'", "\\""),
  ...)

.diag(x, nrow, ncol)

.num.key(levs, key)

.emm_vignette(css = system.file("css", "clean-simple.css", package =
  "emmeans"), highlight = NULL, ...)

.hurdle.support(cmu, cshape, cp0, cmean, zmu, zshape, zp0)

.zi.support(zmu, zshape, zp0)
}
\arguments{
\item{object}{An object of the same class as is supported by a new method.}

\item{...}{Additional parameters that may be supported by the method.}

\item{trms}{The \code{\link{terms}} component of \code{object} (typically with
the response deleted, e.g. via \code{\link{delete.response}})}

\item{na.action}{Integer vector of indices of observations to ignore; or
\code{NULL} if none}

\item{data}{Data frame. Usually, this is \code{NULL}. However, if non-null,
this is used in place of the reconstructed dataset. It must have all of the
predictors used in the model, and any factor levels must match those used
in fitting the model.}

\item{params}{Character vector giving the names of any variables in the model
formula that are \emph{not} predictors. For example, a spline model may involve
a local variable \code{knots} that is not a predictor, but its value is
needed to fit the model. Names of parameters not actually used are harmless,
and the default value \code{"pi"} (the only numeric constant in base R)
is provided in case the model involves it. An example involving splines
may be found at \url{https://github.com/rvlenth/emmeans/issues/180}.}

\item{frame}{Optional \code{data.frame}. Many model objects contain the 
model frame used when fitting the model. In cases where there are no 
predictor transformations, this model frame has all the original predictor
values and so is usable for recovering the data. Thus, if \code{frame} is
non-missing and \code{data} is \code{NULL}, a check is made on \code{trms}
and if there are no function calls, we use \code{data = frame}. This
can be helpful because it provides a modicum of security against the
possibility that the original data used when fitting the model has been
altered or removed.}

\item{pwts}{Optional vector of prior weights. Typically, this may be obtained
from the fitted \code{model} via \code{weights(model)}. If this is provided,
it is used to set weights as long as it is non-\code{NULL} and the same length 
as the number of rows of the data.}

\item{addl.vars}{Character value or vector specifying additional predictors
to include in the reference grid. These must be names of variables that
exist, or you will get an error. 
This may be useful if you need to do
additional computations later on that depend on these variables; e.g., 
bias adjustments for random slopes of variables not among the fixed predictors.}

\item{xlev}{Named list of factor levels (\emph{excluding} ones coerced to 
factors in the model formula)}

\item{grid}{A \code{data.frame} (provided by \code{ref_grid}) containing 
the predictor settings needed in the reference grid}

\item{classes}{Character names of one or more classes to be registered.
The package must contain the functions \code{recover_data.foo} and
\code{emm_basis.foo} for each class \code{foo} listed in \code{classes}.}

\item{pkgname}{Character name of package providing the methods (usually
should be the second argument of \code{.onLoad})}

\item{qdrg}{Logical value. If \code{FALSE}, the \code{recover_data} and
\code{emm_basis} methods are registered. If \code{TRUE}, the \code{qdrg}
method for each class is registered instead.}

\item{fam}{Result of call to \code{family(object)}}

\item{misc}{A \code{list} intended for the \code{@misc} slot of an \code{emmGrid} object}

\item{k, dfargs}{Arguments to \code{.aovlist.dffun}, which is made available as a 
convenience to developers providing support similar to that provided for 
\code{aovlist} objects}

\item{X, weights, assign}{Arguments for \code{.cmpMM}, which compacts a model
matrix \code{X} into a much smaller matrix that has the same row space.
Specifically, it returns the R portion of its QR decomposition. If \code{X}
is already of class \code{qr}, it is used directly. \code{weights} should be
the weights used in the model fit, and \code{assign} is used for unravelling
any pivoting done by \code{\link{qr}}.}

\item{levs, key}{The \code{.num.key} function returns the numeric indices of
the levels in \code{levs} to the set of all levels in \code{key}}

\item{exc, inc}{Arguments for \code{.get.excl} which is useful
in writing \code{.emmc} functions for generating contrast coefficients,
and supports arguments \code{exclude} or \code{include} for excluding
or specifying which levels to use.}

\item{terms}{A \code{terms} component}

\item{vcov.}{Function or matrix that returns a suitable covariance matrix.
The default is \code{.statsvcov} which is \code{stats::vcov}. The \code{.my.vcov} 
function should be called in place of \code{\link{vcov}}, and it supports the user 
being able to specify a different matrix or function via the
optional \code{vcov.} argument.}

\item{expr, retain}{Arguments for \code{.all.vars}, which is an alternative to \code{\link{all.vars}}
that has special provisions for retaining the special characters in \code{retain},
thus allowing model specifications like \code{y ~ data$trt * df[["dose"]]}}

\item{x, nrow, ncol}{Arguments for \code{.diag}, which is an alternative to 
\code{\link{diag}} that lacks its idiosyncrasy of returning an
identity matrix when \code{x} is of length 1.}

\item{css, package, highlight}{Arguments for \code{.emm_vignette}, which is
a clean and simple alternative to such as \code{html_document} for use
as the output style of a Markdown file. All the vignettes in the
\pkg{emmeans} package use this output style.}

\item{cmu, zmu}{In \code{.hurdle.support} and \code{.zi.support}, 
these specify a vector of back-transformed 
estimates for the count and zero model, respectively}

\item{cshape, zshape}{Shape parameter for the count and zero model, respectively}

\item{cp0, zp0}{Function of \code{(mu, shape)} for computing Prob(Y = 0)
for the count and zero model, respectively}

\item{cmean}{Function of \code{(mu, shape)} for computing the mean of the
count model. Typically, this just returns \code{mu}}
}
\value{
The \code{recover_data} method must return a \code{\link{data.frame}}
  containing all the variables that appear as predictors in the model,
  and attributes \code{"call"}, \code{"terms"}, \code{"predictors"},
  and \code{"responses"}. (\code{recover_data.call} will 
  provide these attributes.)

The \code{emm_basis} method should return a \code{list} with the
  following elements:
\describe{
\item{X}{The matrix of linear functions over \code{grid}, having the same
  number of rows as \code{grid} and the number of columns equal to the length
  of \code{bhat}.}
\item{bhat}{The vector of regression coefficients for fixed effects. This
  should \emph{include} any \code{NA}s that result from rank deficiencies.}
\item{nbasis}{A matrix whose columns form a basis for non-estimable functions
  of beta, or a 1x1 matrix of \code{NA} if there is no rank deficiency.}
\item{V}{The estimated covariance matrix of \code{bhat}.}
\item{dffun}{A function of \code{(k, dfargs)} that returns the degrees of
  freedom associated with \code{sum(k * bhat)}.}
\item{dfargs}{A \code{list} containing additional arguments needed for
  \code{dffun}}.
} %%% end of describe

\code{.std.link.llabels} returns a modified version of \code{misc}
  with the appropriate information included corresponding to the information in \code{fam}

\code{combine.terms} returns a \code{terms} object resulting
  from combining all the terms or formulas in \code{...}.

\code{.get.offset} returns the values, based on \code{grid}, of 
any \code{offset} component in \code{terms}

\code{.hurdle.support} returns a matrix with 3 rows containing the
      estimated mean responses and the differentials wrt \code{cmu} and \code{zmu},
      resp.

\code{.zi.support} returns a matrix with 2 rows containing the
      estimated probabilities of 0 and the differentials wrt \code{mu}.
      See the section on hurdle and zero-inflated models.
}
\description{
This documents some functions and methods that may be useful to package 
developers wishing to add support for \pkg{emmeans} for their model objects.A user
or package developer may add \pkg{emmeans} support for a model
class by writing \code{recover_data} and \code{emm_basis} methods
for that class. (Users in need for a quick way to obtain results for a model
that is not supported may be better served by the \code{\link{qdrg}} function.)
There are several other exported functions that may be useful. See the
"xtending" vignette for more details.
}
\note{
Without an explicit \code{data} argument, \code{recover_data} returns
   the \emph{current version} of the dataset. If the dataset has changed
   since the model was fitted, then this will not be the data used to fit
   the model. It is especially important to know this in simulation studies
   where the data are randomly generated or permuted, and in cases where
   several datasets are processed in one step (e.g., using \code{dplyr}).
   In those cases, users should be careful to provide the actual data
   used to fit the model in the \code{data} argument.
}
\section{Details}{

To create a reference grid, the \code{ref_grid} function needs to reconstruct
the data used in fitting the model, and then obtain a matrix of linear
functions of the regression coefficients for a given grid of predictor
values. These tasks are performed by calls to \code{recover_data} and
\code{emm_basis} respectively. A vignette giving details and examples
is available via \href{../doc/xtending.html}{vignette("xtending", "emmeans")}

To extend \pkg{emmeans}'s support to additional model types, one need only
write S3 methods for these two functions. The existing methods serve as
helpful guidance for writing new ones.  Most of the work for
\code{recover_data} can be done by its method for class \code{"call"},
providing the \code{terms} component and \code{na.action} data as additional
arguments. Writing an \code{emm_basis} method is more involved, but the
existing methods (e.g., \code{emmeans:::emm_basis.lm}) can serve as models.
Certain \code{recover_data} and \code{emm_basis} methods are exported from
\pkg{emmeans}. (To find out, do \code{methods("recover_data")}.) If your
object is based on another model-fitting object, it
may be that all that is needed is to call one of these exported methods and
perhaps make modifications to the results. Contact the developer if you need
others of these exported.

If the model has a multivariate response, \code{bhat} needs to be
\dQuote{flattened} into a single vector, and \code{X} and \code{V} must be
constructed consistently.

In models where a non-full-rank result is possible (often, you can tell by
seeing if there is a \code{singular.ok} argument in the model-fitting
function), \code{\link{summary.emmGrid}} and its relatives check the
estimability of each
prediction, using the \code{\link[estimability]{nonest.basis}} function in
the \pkg{estimability} package.

The models already supported are detailed in \href{../doc/models.html}{the
"models" vignette}. Some packages may provide additional \pkg{emmeans}
support for its object classes.
}

\section{Communication between methods}{

If the \code{recover_data} method generates information needed by \code{emm_basis},
that information may be incorporated by creating a \code{"misc"} attribute in the
returned recovered data. That information is then passed as the \code{misc} 
argument when \code{ref_grid} calls \code{emm_basis}.
}

\section{Optional hooks}{

Some models may need something other than standard linear estimates and
standard errors. If so, custom functions may be pointed to via the items
\code{misc$estHook}, \code{misc$vcovHook} and \code{misc$postGridHook}. If
just the name of the hook function is provided as a character string, then it
is retrieved using \code{\link{get}}.

The \code{estHook} function should have arguments \samp{(object, do.se, tol,
...)} where \code{object} is the \code{emmGrid} object,
\code{do.se} is a logical flag for whether to return the standard error, and
\code{tol} is the tolerance for assessing estimability. It should return a
matrix with 3 columns: the estimates, standard errors (\code{NA} when
\code{do.se==FALSE}), and degrees of freedom (\code{NA} for asymptotic). The
number of rows should equal \samp{nrow(linfct(object)}. The
\code{vcovHook} function should have arguments \samp{(object, tol, ...)} as
described. It should return the covariance matrix for the estimates. Finally,
\code{postGridHook}, if present, is called at the very end of
\code{ref_grid}; it takes one argument, the constructed \code{object}, and
should return a suitably modified \code{emmGrid} object.
}

\section{Registering S3 methods for a model class}{

The \code{.emm_register} function is provided as a convenience to conditionally 
register your
S3 methods for a model class, \code{recover_data.foo} and \code{emm_basis.foo},
where \code{foo} is the class name. Your package should implement an
\code{.onLoad} function and call \code{.emm_register} if \pkg{emmeans} is
installed. See the example.
}

\section{Support for Hurdle and Zero-inflated models}{

  The functions \code{.hurdle.support} and \code{.zi.support} help facilitate
  calculations needed to estimate the mean response (count model and zero model
  combined) of these models. \code{.hurdle.support} returns a matrix of three rows.
  The first is the estimated mean for a hurdle model, and the 2nd and 3rd rows are
  differentials for the count and zero models, which needed for delta-method
  calculations. To use these, regard the \code{@linfct} slot as comprising
  two sets of columns, for the count and zero models respectively. To do
  the delta method calculations, multiply the rows of the count part by its 
  differentials times \code{link$mu.eta} evcaluated at that part of the linear predictor.
  Do the same for the zero part, using its differentials and \code{mu.eta}.
  If the resulting matrix is \bold{A}, then the covariance of the mean response
  is \bold{AVA'} where \bold{V}is the \code{@V} slot of the object.
  
  The function \code{zi.support} works the same way, only it is much simpler,
  and is used to estimate the probability of 0 and its differential for either 
  part of a zero-inflated model or hurdle model.
  
  See the code for \code{emm_basis.zeroinfl} and \code{emm_basis.hurdle}
  for how these are used with models fitted by the \pkg{pscl} package.
}

\examples{
\dontrun{
#--- If your package provides recover_data and emm_grid methods for class 'mymod',
#--- put something like this in your package code -- say in zzz.R:
  .onLoad <- function(libname, pkgname) {
    if (requireNamespace("emmeans", quietly = TRUE))
      emmeans::.emm_register("mymod", pkgname)
  }
}
}
\seealso{
\href{../doc/xtending.html}{Vignette on extending emmeans}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.emm.R
\name{plot.emmGrid}
\alias{plot.emmGrid}
\alias{plot.summary_emm}
\title{Plot an \code{emmGrid} or \code{summary_emm} object}
\usage{
\method{plot}{emmGrid}(x, y, type, CIs = TRUE, PIs = FALSE,
  comparisons = FALSE, colors, alpha = 0.05, adjust = "tukey",
  int.adjust = "none", intervals, ...)

\method{plot}{summary_emm}(x, y, horizontal = TRUE, CIs = TRUE, xlab, ylab,
  facetlab = "label_both", layout, scale = NULL, colors, intervals,
  plotit = TRUE, ...)
}
\arguments{
\item{x}{Object of class \code{emmGrid} or \code{summary_emm}}

\item{y}{(Required but ignored)}

\item{type}{Character value specifying the type of prediction desired
(matching \code{"linear.predictor"}, \code{"link"}, or \code{"response"}).
See details under \code{\link{summary.emmGrid}}.
In addition, the user may specify \code{type = "scale"}, in which case
a transformed scale (e.g., a log scale) is displayed based on the transformation
or link function used. Additional customization of this scale is available through
including arguments to \code{ggplot2::scale_x_continuous} in \code{...} .}

\item{CIs}{Logical value. If \code{TRUE}, confidence intervals are
plotted for each estimate.}

\item{PIs}{Logical value. If \code{TRUE}, prediction intervals are
plotted for each estimate. If \code{object} is a Bayesian model,
this requires the \code{...} arguments to include
\code{frequentist = TRUE} and \code{sigma =} (some value).
Note that the \code{PIs} option is \emph{not} available with
\code{summary_emm} objects -- only for \code{emmGrid} objects.
Also, prediction intervals are not available
with \code{engine = "lattice"}.}

\item{comparisons}{Logical value. If \code{TRUE}, \dQuote{comparison arrows}
are added to the plot, in such a way that the degree to which arrows
overlap reflects as much as possible the significance of the comparison of
the two estimates. (A warning is issued if this can't be done.)
Note that comparison arrows are not available with `summary_emm` objects.}

\item{colors}{Optional character vector of colors to use for estimates, CIs, PIs, 
and comparison arrows, respectively. If missing, these are derived from
\code{ink}, \code{paper}, and \code{accent} in the theme's \code{geom} element.
\code{ink} is used for the estimates, \code{accent} is used for comparison arrows,
and a contrasting color to \code{accent} is blended with \code{paper} for
the intervals. 
If just one color is given, estimates and intervals are made from
various blends of that color and \code{paper}, and \code{accent}
color is used for comparison arrows.}

\item{alpha}{The significance level to use in constructing comparison arrows}

\item{adjust}{Character value: Multiplicity adjustment method for comparison arrows \emph{only}.}

\item{int.adjust}{Character value: Multiplicity adjustment method for the plotted confidence intervals \emph{only}.}

\item{intervals}{If specified, it is used to set \code{CIs}. This is the previous
argument name for \code{CIs} and is provided for backward compatibility.}

\item{...}{Additional arguments passed to \code{\link{update.emmGrid}}, 
\code{\link{summary.emmGrid}},
\code{\link{predict.emmGrid}}, or
\code{\link[lattice:xyplot]{dotplot}}}

\item{horizontal}{Logical value specifying whether the intervals should be
plotted horizontally or vertically}

\item{xlab}{Character label for horizontal axis}

\item{ylab}{Character label for vertical axis}

\item{facetlab}{Character or function method used to label facets in a 
multi-panel plot (with the \code{ggplot} engine).
Default is \code{"label_both"}, meaning that both factor names and levels are shown,
You can use \code{"label_value"} to save space, or \code{"label_context"}
to decide automatically (often wrongly). See the help page for 
\code{ggplot2::labellers}}

\item{layout}{Numeric value passed to \code{\link[lattice:xyplot]{dotplot}}
when \code{engine == "lattice"}.}

\item{scale}{Object of class \code{trans} (in the \pkg{scales} package) to
specify a nonlinear scale. This is used in lieu of \code{type = "scale"} when
plotting a \code{summary_emm} object created with \code{type = "response"}.
This is ignored with other types of summaries.}

\item{plotit}{Logical value. If \code{TRUE}, a graphical object is returned;
if \code{FALSE}, a data.frame is returned containing all the values
used to construct the plot.}
}
\value{
If \code{plotit = TRUE}, a graphical object is returned.

  If \code{plotit = FALSE}, a \code{data.frame} with the table of
  EMMs that would be plotted. In the latter case, the estimate being plotted
  is named \code{the.emmean}, and any factors involved have the same names as
  in the object. Confidence limits are named \code{lower.CL} and
  \code{upper.CL}, prediction limits are named \code{lpl} and \code{upl}, and
  comparison-arrow limits are named \code{lcmpl} and \code{ucmpl}.
  There is also a variable named \code{pri.fac} which contains the factor 
  combinations that are \emph{not} among the \code{by} variables.
}
\description{
Methods are provided to plot EMMs as side-by-side CIs, and optionally to display 
  \dQuote{comparison arrows} for displaying pairwise comparisons.
}
\note{
In order to play nice with the plotting functions,
any variable names that are not syntactically correct (e.g., contain spaces)
are altered using \code{\link{make.names}}.
}
\section{Details}{

If any \code{by} variables are in force, the plot is divided into separate
panels. For
\code{"summary_emm"} objects, the \code{\dots} arguments in \code{plot}
are passed \emph{only} to \code{dotplot}, whereas for \code{"emmGrid"}
objects, the object is updated using \code{\dots} before summarizing and
plotting.

In plots with \code{comparisons = TRUE}, the resulting arrows are only
approximate, and in some cases may fail to accurately reflect the pairwise
comparisons of the estimates -- especially when estimates having large and
small standard errors are intermingled in just the wrong way. Note that the
maximum and minimum estimates have arrows only in one direction, since there
is no need to compare them with anything higher or lower, respectively. See
the \href{../doc/xplanations.html#arrows}{\code{vignette("xplanations",
"emmeans")}} for details on how these are derived.

If \code{adjust} or \code{int.adjust} are not supplied, they default to the 
internal \code{adjust} setting saved in \code{pairs(x)} and \code{x} 
respectively (see \code{\link{update.emmGrid}}).
}

\examples{
warp.lm <- lm(breaks ~ wool * tension, data = warpbreaks)
warp.emm <- emmeans(warp.lm, ~ tension | wool)
plot(warp.emm)

plot(warp.emm, PIs = TRUE, comparisons = TRUE)

with_emm_options(gg.theme = ggplot2::theme_dark(), 
    plot(warp.emm, PIs = TRUE, comparisons = TRUE))

plot(warp.emm, by = NULL, comparisons = TRUE, adjust = "none", 
     horizontal = FALSE, colors = "blue")

### Using a transformed scale (also demonstrating 'facetlab' argument)
pigs.lm <- lm(log(conc + 2) ~ source * factor(percent), data = pigs)
pigs.emm <- emmeans(pigs.lm, ~ percent | source)
plot(pigs.emm, type = "scale", breaks = seq(20, 100, by = 10), 
     facetlab = "label_value")

# Based on a summary. 
# To get a transformed axis, must specify 'scale'; but it does not necessarily
# have to be the same as the actual response transformation
pigs.ci <- confint(pigs.emm, type = "response")
plot(pigs.ci, scale = scales::log10_trans(), 
     facetlab = \(x) ggplot2::label_both(x, sep = " = "))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pwpp.R
\name{pwpp}
\alias{pwpp}
\title{Pairwise P-value plot}
\usage{
pwpp(emm, method = "pairwise", by, sort = TRUE, values = TRUE,
  rows = ".", xlab, ylab, xsub = "", plim = numeric(0), add.space = 0,
  aes, ...)
}
\arguments{
\item{emm}{An \code{emmGrid} object}

\item{method}{Character or list. Passed to \code{\link{contrast}}, and defines 
the contrasts to be displayed. Any contrast method may be used,
provided that each contrast includes one coefficient of \code{1},
one coefficient of \code{-1}, and the rest \code{0}. That is, calling
\code{contrast(object, method)} produces a set of comparisons, each with
one estimate minus another estimate.}

\item{by}{Character vector of variable(s) in the grid to condition on. These will
create different panels, one for each level or level-combination.
Grid factors not in \code{by} are the \emph{primary} factors: 
whose levels or level combinations are compared pairwise.}

\item{sort}{Logical value. If \code{TRUE}, levels of the factor combinations are
ordered by their marginal means. If \code{FALSE}, they appear in
order based on the existing ordering of the factor levels involved.
Note that the levels are ordered the same way in all panels, and in
many cases this implies that the means in any particular panel
will \emph{not} be ordered even when \code{sort = TRUE}.}

\item{values}{Logical value. If \code{TRUE}, the values of the EMMs are included
in the plot. When there are several side-by-side panels due
to \code{by} variable(s), the labels showing values start
stealing a lot of space from the plotting area; in those cases,
it may be desirable to specify \code{FALSE} or use \code{rows}
so that some panels are vertically stacked.}

\item{rows}{Character vector of which \code{by} variable(s) are used to define
rows of the panel layout. Those variables in \code{by} not included in 
\code{rows} define columns in the array of panels.
A \code{"."} indicates that only one row
is used, so all panels are stacked side-by-side.}

\item{xlab}{Character label to use in place of the default for the P-value axis.}

\item{ylab}{Character label to use in place of the default for the primary-factor axis.}

\item{xsub}{Character label used as caption at the lower right of the plot.}

\item{plim}{numeric vector of value(s) between 0 and 1. These are included
among the observed p values so that the range of tick marks includes at
least the range of \code{plim}. Choosing \code{plim = c(0,1)} will ensure
the widest possible range.}

\item{add.space}{Numeric value to adjust amount of space used for value labels. Positioning
of value labels is tricky, and depends on how many panels and the
physical size of the plotting region. This parameter allows the user to
adjust the position. Changing it by one unit should shift the position by
about one character width (right if positive, left if negative).
Note that this interacts with \code{aes$label} below.}

\item{aes}{optional named list of lists. Entries considered are \code{point}, 
\code{segment}, and \code{label}, and contents are passed to the respective
\code{ggplot2::geom_xxx()} functions. These affect rendering of points, 
line segments joining them, and value labels. 
Defaults are \code{point = list(size = 2)},
\code{segment = list()}, and \code{label = list(size = 2.5)}.}

\item{...}{Additional arguments passed to \code{contrast} and \code{\link{summary.emmGrid}}, 
as well as to \code{geom_segment} and \code{geom_label}}
}
\description{
Constructs a plot of P values associated with pairwise comparisons of 
estimated marginal means.
}
\details{
Factor levels (or combinations thereof) are plotted on the vertical scale, and P values
are plotted on the horizontal scale. Each P value is plotted twice -- at
vertical positions corresponding to the levels being compared -- and connected by
a line segment. Thus, it is easy to visualize which P values are small and large,
and which levels are compared. In addition, factor levels are color-coded, and the points
and half-line segments appear in the color of the other level.
The P-value scale is nonlinear, so as to stretch-out smaller P values and
compress larger ones.
P values smaller than 0.0004 are altered and plotted in a way that makes 
  them more distinguishable from one another.

If \code{xlab}, \code{ylab}, and \code{xsub} are not provided, reasonable labels
are created. \code{xsub} is used to note special features; e.g., equivalence
thresholds or one-sided tests.
}
\note{
If \code{emm} is the result of a Bayesian analysis, the plot is based on
  summaries with \code{frequentist = TRUE}.

The \pkg{ggplot2} and \pkg{scales} packages must be installed in order 
  for \code{pwpp} to work.

Additional plot aesthetics are available by adding them to the returned object;
  see the examples
}
\examples{
pigs.lm <- lm(log(conc) ~ source * factor(percent), data = pigs)
emm = emmeans(pigs.lm, ~ percent | source)
pwpp(emm)
pwpp(emm, method = "trt.vs.ctrl1", type = "response", side = ">")

# custom aesthetics:
my.aes <- list(point = list(shape = "square"), 
               segment = list(linetype = "dashed", color = "red"),
               label = list(family = "serif", fontface = "italic"))
my.pal <- c("darkgreen", "blue", "magenta", "orange")
pwpp(emm, aes = my.aes) + ggplot2::scale_color_manual(values = my.pal)

}
\seealso{
A numerical display of essentially the same results is available
  from \code{\link{pwpm}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emm-example.R
\name{emm_example}
\alias{emm_example}
\title{Run or list additional examples}
\usage{
emm_example(name, run = !list, list = FALSE, ...)
}
\arguments{
\item{name}{Character name of file to run. We look for a file with this name
(with \code{".R"} appended) in the system files provided with \pkg{emmeans}.}

\item{run}{Logical choosing whether or not to run the example code}

\item{list}{Logical choosing whether or not to list the example code}

\item{...}{Used only by the developer}
}
\description{
This function exists so as to provide cleaner-looking examples in
help files when it must be run conditionally on another package.
Typically we want to run the code (\code{run = TRUE} is the default),
or otherwise just list it on the console (\code{list = TRUE}).
}
\examples{
# List an example
emm_example("qdrg-biglm", list = TRUE)

# Run an example
if (require(biglm))
    emm_example("qdrg-biglm")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multiv.R
\name{mvregrid}
\alias{mvregrid}
\title{Multivariate regridding}
\usage{
mvregrid(object, transform = "response", mult.name, newname = mult.name,
  newlevs, fcn, ...)
}
\arguments{
\item{object}{An \code{emmGrid} object}

\item{transform}{The transformation to use in re-gridding. If \code{"response"},
we apply the inverse of the multivariate transformation in \code{object@misc$tran};
otherwise, we re-grid as if \code{transform} had been applied to the multivariate
response. (Note that this will entail first re-gridding to the response scale
if necessary.)}

\item{mult.name}{The name of the multivariate factor to be transformed.
If missing, we use \code{object@roles$multresp}, and throw an error message 
if it is \code{NULL} or ambiguous; in that case, the user must repeat the
call with \code{mult.name} specified.}

\item{newname}{the name to be given to the newly transformed variable}

\item{newlevs}{levels of the newly created factor (must conform to
the number of columns created by \code{fcn}). If missing, we use the column names
of the newly created variable.}

\item{fcn}{The multivariate function to apply. If character, we look for it 
in the namespace of the \pkg{compositions} package.}

\item{...}{Additional arguments passed to \code{fcn}}
}
\value{
A new \code{emmGrid} object with the newly created factor as its last factor
}
\description{
This function is similar to \code{\link{regrid}} except it performs a
multivariate transformation. This is useful, for instance, in multivariate
models that have a compositional response.
}
\details{
If a multivariate response transformation was used in fitting the model,
its name is auto-detected, and in that case we need not specify \code{fcn}
as long as its inverse can be found in the namespace of the \pkg{compositions}
package. (That package need not be installed unless \code{fcn} is a character 
value.) For some such models, auto-detection process throws a
warning message, especially if \code{cbind} is also present in the model
formula.

Currently, no bias-adjustment option is available.
}
\examples{
if(requireNamespace("compositions"))
    emm_example("mvregrid")
    # Use emm_example("mvregrid", list = TRUE) # to see just the code

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{ubds}
\alias{ubds}
\title{Unbalanced dataset}
\format{
A data frame with 100 observations, 5 variables,
  and a special \code{"cells"} attribute:
\describe{
  \item{A}{Factor with levels 1, 2, and 3}
  \item{B}{Factor with levels 1, 2, and 3}
  \item{C}{Factor with levels 1, 2, and 3}
  \item{x}{A numeric variable}
  \item{y}{A numeric variable}
}
In addition, \code{attr(ubds, "cells")} consists of a named list of length 27 with the row numbers for
each combination of \code{A, B, C}. For example, 
\code{attr(ubds, "cells")[["213"]]} has the row numbers corresponding
to levels \code{A == 2, B == 1, C == 3}. The entries are ordered by
length, so the first entry is the cell with the lowest frequency.
}
\usage{
ubds
}
\description{
This is a simulated unbalanced dataset with three factors
and two numeric variables. There are true relationships among these variables.
This dataset can be useful in testing or illustrating messy-data situations.
There are no missing data, and there is at least one observation for every 
factor combination; however, the \code{"cells"} attribute makes it simple
to construct subsets that have empty cells.
}
\examples{
 # Omit the three lowest-frequency cells
 low3 <- unlist(attr(ubds, "cells")[1:3]) 
 messy.lm <- lm(y ~ (x + A + B + C)^3, data = ubds, subset = -low3)
  
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multiv.R
\name{mvcontrast}
\alias{mvcontrast}
\title{Multivariate contrasts}
\usage{
mvcontrast(object, method = "eff", mult.name = object@roles$multresp,
  null = 0, by = object@misc$by.vars, adjust = c("sidak",
  p.adjust.methods), show.ests = FALSE, ...)
}
\arguments{
\item{object}{An object of class \code{emmGrid}}

\item{method}{A contrast method, per \code{\link{contrast.emmGrid}}}

\item{mult.name}{Character vector of names of the factors whose levels
define the multivariate means to contrast. If the model itself has a
multivariate response, that is what is used. Otherwise, \code{mult.name}
\emph{must} be specified.}

\item{null}{Scalar or conformable vector of null-hypothesis values to test against}

\item{by}{Any \code{by} variable(s). These should not include the primary
variables to be contrasted. For convenience, the \code{by} variable is
nulled-out if it would result in no primary factors being contrasted.}

\item{adjust}{Character value of a multiplicity adjustment method
(\code{"none"} for no adjustment). The available adjustment methods are
more limited that in \code{contrast}, and any default adjustment returned
via \code{method} is ignored.}

\item{show.ests}{Logical flag determining whether the multivariate means 
are displayed}

\item{...}{Additional arguments passed to \code{contrast}}
}
\value{
An object of class \code{summary_emm} containing the multivariate
  test results; or a list of the estimates and the tests if \code{show.ests}
  is \code{TRUE}. The test results include the Hotelling \eqn{T^2} statistic,
  \eqn{F} ratios, degrees of freedom, and \eqn{P} values.
}
\description{
This function displays tests of multivariate comparisons or contrasts.
The contrasts are constructed at each level of the variable in \code{mult.name},
and then we do a multivariate test that the vector of estimates is equal to
\code{null} (zero by default). The \emph{F} statistic and degrees
of freedom are determined via the Hotelling distribution. that is, if there are
\eqn{m} error degrees of freedom and multivariate dimensionality \eqn{d}, then
the resulting \eqn{F} statistic has degrees of freedom \eqn{(d, m - d + 1)}
as shown in Hotelling (1931).
}
\note{
If some interactions among the primary and \code{mult.name} factors are
absent, the covariance of the multivariate means is singular; this situation
is accommodated, but the result has reduced degrees of freedom and a message
is displayed. If there are other abnormal conditions such as non-estimable
results, estimates are shown as \code{NA}.

While designed primarily for testing contrasts, multivariate tests of the
mean vector itself can be implemented via \code{method = "identity")} (see
the examples).
}
\examples{
MOats.lm <- lm(yield ~ Variety + Block, data = MOats)
MOats.emm <- emmeans(MOats.lm, ~ Variety | rep.meas)
mvcontrast(MOats.emm, "consec", show.ests = TRUE)  # mult.name defaults to rep.meas

# Test each mean against a specified null vector
mvcontrast(MOats.emm, "identity", name = "Variety", 
           null = c(80, 100, 120, 140), adjust = "none")
# (Note 'name' is passed to contrast() and overrides default name "contrast")

# 'mult.name' need not refer to a multivariate response
mvcontrast(MOats.emm, "trt.vs.ctrl1", mult.name = "Variety")

}
\references{
Hotelling, Harold (1931) "The generalization of Student's ratio", 
  \emph{Annals of Mathematical Statistics} 2(3), 360–378. doi:10.1214/aoms/1177732979
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{neuralgia}
\alias{neuralgia}
\title{Neuralgia data}
\format{
A data frame with 60 observations and 5 variables:
\describe{
  \item{\code{Treatment}}{Factor with 3 levels \code{A}, \code{B}, and \code{P}.
    The latter is placebo}
  \item{\code{Sex}}{Factor with two levels \code{F} and \code{M}}
  \item{\code{Age}}{Numeric covariate -- patient's age in years}
  \item{\code{Duration}}{Numeric covariate -- duration of the condition before
    beginning treatment}
  \item{\code{Pain}}{Binary response factor with levels \code{No} and \code{Yes}}
}
}
\source{
Cai, Weijie (2014) \emph{Making Comparisons Fair: How LS-Means Unify 
  the Analysis of Linear Models}, SAS Institute, Inc. Technical paper 142-2014,
  page 12, 
  \url{http://support.sas.com/resources/papers/proceedings14/SAS060-2014.pdf}
}
\usage{
neuralgia
}
\description{
These data arise from a study of analgesic effects of treatments of elderly
patients who have neuralgia. Two treatments and a placebo are compared. The 
response variable is whether the patient reported pain or not. Researchers
recorded the age and gender of 60 patients along with the duration of
complaint before the treatment began.
}
\examples{
# Model and analysis shown in the SAS report:
neuralgia.glm <- glm(Pain ~ Treatment * Sex + Age, family = binomial(),
   data = neuralgia) 
pairs(emmeans(neuralgia.glm, ~ Treatment, at = list(Sex = "F")), 
    reverse = TRUE, type = "response", adjust = "bonferroni")

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers.R
\name{lsmeans}
\alias{lsmeans}
\alias{wrappers}
\alias{lstrends}
\alias{lsmip}
\alias{lsm}
\alias{lsmobj}
\alias{lsm.options}
\alias{get.lsm.option}
\title{Wrappers for alternative naming of EMMs}
\usage{
lsmeans(...)

lstrends(...)

lsmip(...)

lsm(...)

lsmobj(...)

lsm.options(...)

get.lsm.option(x, default = emm_defaults[[x]])
}
\arguments{
\item{...}{Arguments passed to the corresponding \code{em}\emph{xxxx} function}

\item{x}{Character name of desired option}

\item{default}{default value to return if \code{x} not found}
}
\value{
The result of the call to \code{em}\emph{xxxx}, suitably modified.

\code{get.lsm.option} and \code{lsm.options} remap options from
  and to corresponding options in the \pkg{emmeans} options system.
}
\description{
These are wrappers for \code{\link{emmeans}} and related functions to provide
backward compatibility, or for users who may prefer to
use other terminology than \dQuote{estimated marginal means} -- namely 
\dQuote{least-squares means}. These functions also provide the functionality
formerly provided by the \pkg{lsmeans} package, which is now just a front-end
for \pkg{emmeans}.
}
\details{
For each function with \code{ls}\emph{xxxx} in its name,
the same function named \code{em}\emph{xxxx} is called. Any estimator names or 
list items beginning with \dQuote{em} are replaced with \dQuote{ls} 
before the results are returned
}
\examples{
pigs.lm <- lm(log(conc) ~ source + factor(percent), data = pigs)
lsmeans(pigs.lm, "source")
}
\seealso{
\code{\link{emmeans}}, \code{\link{emtrends}}, \code{\link{emmip}},
         \code{\link{emm}}, \code{\link{emmobj}}, \code{\link{emm_options}},
         \code{\link{get_emm_option}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emm-list.R
\name{emm_list}
\alias{emm_list}
\alias{contrast.emm_list}
\alias{pairs.emm_list}
\alias{test.emm_list}
\alias{confint.emm_list}
\alias{plot.emm_list}
\alias{coef.emm_list}
\alias{linfct.emm_list}
\alias{str.emm_list}
\alias{summary.emm_list}
\alias{print.emm_list}
\alias{as.data.frame.emm_list}
\alias{as.data.frame.summary_eml}
\title{The \code{emm_list} class}
\usage{
\method{contrast}{emm_list}(object, ..., which = NULL)

\method{pairs}{emm_list}(x, ..., which = NULL)

\method{test}{emm_list}(object, ..., which = seq_along(object))

\method{confint}{emm_list}(object, ..., which = seq_along(object))

\method{plot}{emm_list}(x, ..., which = 1)

\method{coef}{emm_list}(object, ..., which = NULL)

\method{linfct}{emm_list}(object, ..., which = seq_along(object))

\method{str}{emm_list}(object, ...)

\method{summary}{emm_list}(object, ..., which = seq_along(object))

\method{print}{emm_list}(x, ...)

\method{as.data.frame}{emm_list}(x, ...)

\method{as.data.frame}{summary_eml}(x, row.names = NULL, optional = FALSE,
  which, ...)
}
\arguments{
\item{object, x}{an object of class \code{emm_list}}

\item{...}{additional arguments passed to corresponding \code{emmGrid}
method. In addition, the user may include a logical argument \code{drop}
that is akin to \code{\link{drop}} and the argument of the same name in
subscripting matrices and data frames. When \code{drop} is \code{TRUE} (the
default), then when the result is a \code{list} of length 1, the
\code{list} structure is removed.}

\item{which}{integer vector specifying which elements to select;
if \code{NULL},
we try to guess which elements make sense. Usually, this is all elements having 
names that start with \sQuote{em} or \sQuote{ls},
or the first element if no matches are found. However, in \code{coef.emm_list},
these are the ones we \emph{exclude}.}

\item{row.names, optional}{Required arguments of \code{as.data.frame}, ignored}
}
\value{
a \code{list} of objects returned by the corresponding \code{emmGrid}
  method (thus, often, another \code{emm_list} object). However, if
  \code{which} has length 1, the one result is not wrapped in a list.

\code{summary.emm_list} returns an object
of class \code{summary_eml}, which is a list of \code{summary_emm}
objects.

The \code{as.data.frame} methods return a single data frame via
\code{as.data.frame(rbind(x))}.
See also \code{\link{rbind.emm_list}} and \code{\link{as.data.frame.emmGrid}}
}
\description{
An \code{emm_list} object is simply a list of
\code{\link[=emmGrid-class]{emmGrid}} objects. Such a list is returned,
for example, by \code{\link{emmeans}} with a two-sided formula or a list as its
\code{specs} argument. Several methods for this class are provided, as detailed below.
Typically, these methods just quietly do the same thing as their \code{emmGrid}
methods, using the first element of the list. You can specify \code{which}
to select a different element, or just run the corresponding \code{emmGrid}
method on \code{object[[k]]}.
}
\note{
The \code{plot} method uses only the first element of \code{which}; the others are ignored.

No \code{export} option is provided for printing an \code{emm_list}
(see \code{\link{print.emmGrid}}). If you wish to export these objects, you 
must do so separately for each element in the list.
}
\examples{
mod <- lm(conc ~ source, data = pigs)
obj <- emmeans(mod, pairwise ~ source)

linfct(obj)

coef(obj)     # done only for the contrasts

contrast(obj, "consec")  # done only for the means

contrast(obj, "eff", drop = FALSE)   # kept as a list
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emm-contr.R
\name{contrast-methods}
\alias{contrast-methods}
\alias{pairwise.emmc}
\alias{emmc-functions}
\alias{revpairwise.emmc}
\alias{tukey.emmc}
\alias{poly.emmc}
\alias{opoly.emmc}
\alias{trt.vs.ctrl.emmc}
\alias{trt.vs.ctrl1.emmc}
\alias{trt.vs.ctrlk.emmc}
\alias{dunnett.emmc}
\alias{eff.emmc}
\alias{del.eff.emmc}
\alias{consec.emmc}
\alias{mean_chg.emmc}
\alias{helmert.emmc}
\alias{nrmlz.emmc}
\alias{wtcon.emmc}
\alias{identity.emmc}
\title{Contrast families}
\usage{
pairwise.emmc(levs, exclude = integer(0), include, ...)

revpairwise.emmc(levs, exclude = integer(0), include, ...)

tukey.emmc(levs, reverse = FALSE, ...)

poly.emmc(levs, max.degree = min(6, k - 1), ...)

opoly.emmc(levs, max.degree = min(6, k - 1), scores, exclude = integer(0),
  include, ...)

trt.vs.ctrl.emmc(levs, ref = 1, reverse = FALSE, exclude = integer(0),
  include, ...)

trt.vs.ctrl1.emmc(levs, ref = 1, ...)

trt.vs.ctrlk.emmc(levs, ref = length(levs), ...)

dunnett.emmc(levs, ref = 1, ...)

eff.emmc(levs, exclude = integer(0), include, wts = rep(1, length(levs)),
  ...)

del.eff.emmc(levs, exclude = integer(0), include, wts = rep(1,
  length(levs)), ...)

consec.emmc(levs, reverse = FALSE, exclude = integer(0), include, ...)

mean_chg.emmc(levs, reverse = FALSE, exclude = integer(0), include, ...)

helmert.emmc(levs, exclude = integer(0), include, ...)

nrmlz.emmc(levs, family, ...)

wtcon.emmc(levs, wts = rep(1, length(levs)), cmtype = "GrandMean", ...)

identity.emmc(levs, exclude = integer(0), include, ...)
}
\arguments{
\item{levs}{Vector of factor levels}

\item{exclude}{integer vector of indices, or character vector of levels to
exclude from consideration. These levels will receive weight 0 in all
contrasts. Character levels must exactly match elements of \code{levs}.}

\item{include}{integer or character vector of levels to include (the
complement of \code{exclude}). An error will result if the user specifies
both \code{exclude} and \code{include}.}

\item{...}{Additional arguments, passed to related methods as appropriate}

\item{reverse}{Logical value to determine the direction of comparisons}

\item{max.degree}{Integer specifying the maximum degree of polynomial contrasts}

\item{scores}{Set of values of length \code{length(levs)} over which
orthogonal polynomials are computed. The default scores are the 
consecutive integers \code{seq_along(levs)}.
(If \code{exclude} or \code{include}
are used, the default scores are subsetted accordingly; and if \code{scores} 
is specified, its length must be the same as that of the subsetted \code{levs}).}

\item{ref}{Integer(s) or character(s) specifying which level(s) to use 
as the reference. Character values must exactly match elements of \code{levs}
(including any enhancements -- see examples)}

\item{wts}{Optional weights to use with \code{eff.emmc} and \code{del.eff.emmc} contrasts.
These default to equal weights.
If \code{exclude} or \code{include} are specified, \code{wts} may be
either the same length as \code{levs} or the length of the included levels.
In the former case, weights for any excluded levels are set to zero.
\code{wts} has no impact on the results unless there are at least
three levels included in the contrast.}

\item{family}{name of contrast family to use}

\item{cmtype}{the \code{type} argument passed to \code{\link[multcomp]{contrMat}}}
}
\value{
A data.frame, each column containing contrast coefficients for levs.
  The "desc" attribute is used to label the results in emmeans, and the
  "adjust" attribute gives the default adjustment method for multiplicity.
}
\description{
Functions with an extension of \code{.emmc} provide for named contrast
families. One of the standard ones documented here may be used, or the user
may write such a function.
}
\details{
Each standard contrast family has a default multiple-testing adjustment as
noted below. These adjustments are often only approximate; for a more
exacting adjustment, use the interfaces provided to \code{glht} in the
\pkg{multcomp} package.

\code{pairwise.emmc}, \code{revpairwise.emmc}, and \code{tukey.emmc} generate
contrasts for all pairwise comparisons among estimated marginal means at the
levels in levs. The distinction is in which direction they are subtracted.
For factor levels A, B, C, D, \code{pairwise.emmc} generates the comparisons
A-B, A-C, A-D, B-C, B-D, and C-D, whereas \code{revpairwise.emmc} generates
B-A, C-A, C-B, D-A, D-B, and D-C. \code{tukey.emmc} invokes
\code{pairwise.emmc} or \code{revpairwise.emmc} depending on \code{reverse}.
The default multiplicity adjustment method is \code{"tukey"}, which is only
approximate when the standard errors differ.

\code{poly.emmc} and \code{opoly.emmc} generate orthogonal polynomial contrasts.
\code{poly.emmc} uses equally-spaced factor levels; coefficients are derived from the
\code{\link[stats]{poly}} function, but an \emph{ad hoc} algorithm is used to
scale them to integer coefficients that are (usually) the same as in
published tables of orthogonal polynomial contrasts. On the other hand, 
\code{opoly.emmc}'s coefficients are always normalized (sum of squares equals 1),
but allows the user to choose alternate reference points in \code{scores}, as in the
\code{\link{contr.poly}} function.
In both cases, the default multiplicity adjustment method is \code{"none"}.

\code{trt.vs.ctrl.emmc} and its relatives generate contrasts for comparing
one level (or the average over specified levels) with each of the other
levels. The argument \code{ref} should be the index(es) (not the labels) of
the reference level(s). \code{trt.vs.ctrl1.emmc} is the same as
\code{trt.vs.ctrl.emmc} with a reference value of 1, and
\code{trt.vs.ctrlk.emmc} is the same as \code{trt.vs.ctrl} with a reference
value of \code{length(levs)}. \code{dunnett.emmc} is the same as
\code{trt.vs.ctrl}. The default multiplicity adjustment method is
\code{"dunnettx"}, a close approximation to the Dunnett adjustment.
\emph{Note} in all of these functions, it is illegal to have any overlap
between the \code{ref} levels and the \code{exclude} levels. If any is found,
an error is thrown.

\code{consec.emmc} and \code{mean_chg.emmc} are useful for contrasting
treatments that occur in sequence. For a factor with levels A, B, C, D,
\code{consec.emmc} generates the comparisons B-A, C-B, and D-C, while
\code{mean_chg.emmc} generates the contrasts (B+C+D)/3 - A, (C+D)/2 -
(A+B)/2, and D - (A+B+C)/3. With \code{reverse = TRUE}, these differences go
in the opposite direction.

\code{eff.emmc} and \code{del.eff.emmc} generate contrasts that compare each
level with the average over all levels (in \code{eff.emmc}) or over all other
levels (in \code{del.eff.emmc}). These differ only in how they are scaled.
For a set of k EMMs, \code{del.eff.emmc} gives weight 1 to one EMM and weight
-1/(k-1) to the others, while \code{eff.emmc} gives weights (k-1)/k and -1/k
respectively, as in subtracting the overall EMM from each EMM. The default
multiplicity adjustment method is \code{"fdr"}. This is a Bonferroni-based
method and is slightly conservative; see \code{\link[stats]{p.adjust}}.

\code{nrmlz.emmc} is a wrapper that can be used with any other \code{.emmc} 
function that will normalize the contrast coefficients so that the sum of its 
squares equals 1. Just provide the root name of the function in \code{family},
along with any other arguments to pass to it.

\code{wtcon.emmc} generates weighted contrasts based on the function 
\code{\link[multcomp]{contrMat}} function in the \pkg{multcomp} package,
using the provided \code{type} as documented there. If the user provides 
\code{wts}, they have to conform to the length of \code{levs}; however,
if \code{wts} is not specified, \code{contrast} will fill-in what is
required, and usually this is safer (especially when \code{by != NULL}
which usually means that the weights are different in each \code{by} group).

\code{identity.emmc} simply returns the identity matrix (as a data frame),
minus any columns specified in \code{exclude}. It is potentially useful in
cases where a contrast function must be specified, but none is desired.
}
\note{
Caution is needed in cases where the user alters the ordering of
  results (e.g., using the the \code{"[...]"} operator), because the
  contrasts generated depend on the order of the levels provided. For
  example, suppose \code{trt.vs.ctrl1} contrasts are applied to two \code{by}
  groups with levels ordered (Ctrl, T1, T2) and (T1, T2, Ctrl) respectively,
  then the contrasts generated will be for (T1 - Ctrl, T2 - Ctrl) in the
  first group and (T2 - T1, Ctrl - T1) in the second group, because the first
  level in each group is used as the reference level.
}
\examples{
warp.lm <- lm(breaks ~ wool*tension, data = warpbreaks)
warp.emm <- emmeans(warp.lm, ~ tension | wool)
contrast(warp.emm, "poly")
contrast(warp.emm, "trt.vs.ctrl", ref = "M")
\dontrun{
## Same when enhanced labeling is used:
contrast(warp.emm, "trt.vs.ctrl", 
         enhance.levels = "tension", ref = "tensionM")}

# Comparisons with grand mean
contrast(warp.emm, "eff")
# Comparisons with a weighted grand mean
contrast(warp.emm, "eff", wts = c(2, 5, 3))

# Compare only low and high tensions
# Note pairs(emm, ...) calls contrast(emm, "pairwise", ...)
pairs(warp.emm, exclude = 2)
# (same results using exclude = "M" or include = c("L","H") or include = c(1,3))

# Same contrasts as above but with normalized contrast coefficients
contrast(warp.emm, "nrmlz", family = "pairwise", include = c(1, 3))

### Setting up a custom contrast function
revhelmert.emmc <- function(levs, ...) {
    M <- as.data.frame(contr.helmert(levs)[rev(seq_along(levs)), ])
    names(M) <- paste(rev(levs)[-1],"vs later")
    attr(M, "desc") <- "reverse Helmert contrasts"
    M
}
contrast(warp.emm, "revhelmert")

\dontrun{
# See what is used for polynomial contrasts with 6 levels
emmeans:::poly.emmc(1:6)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{MOats}
\alias{MOats}
\title{Oats data in multivariate form}
\format{
A data frame with 18 observations and 3 variables
\describe{
  \item{\code{Variety}}{a factor with levels \code{Golden Rain},
    \code{Marvellous}, \code{Victory}}
  \item{\code{Block}}{an ordered factor with levels \code{VI} < \code{V} <
    \code{III} < \code{IV} < \code{II} < \code{I}}
  \item{\code{yield}}{a matrix with 4 columns, giving the yields with
    nitrogen concentrations of 0, .2, .4, and .6.}
}
}
\source{
The dataset \code{\link[nlme]{Oats}} in the \pkg{nlme} package.
}
\usage{
MOats
}
\description{
This is the \code{Oats} dataset provided in the \pkg{nlme} package, but it is
rearranged as one multivariate observation per plot.
}
\details{
These data arise from a split-plot experiment reported by Yates (1935) and
used as an example in Pinheiro and Bates (2000) and other texts. Six blocks
were divided into three whole plots, randomly assigned to the three varieties
of oats. The whole plots were each divided into 4 split plots and randomized
to the four concentrations of nitrogen.
}
\examples{
MOats.lm <- lm (yield ~ Block + Variety, data = MOats)
MOats.rg <- ref_grid (MOats.lm, mult.name = "nitro")
emmeans(MOats.rg, ~ nitro | Variety)
}
\references{
Pinheiro, J. C. and Bates D. M. (2000) \emph{Mixed-Effects Models in S and
S-PLUS}, Springer, New York. (Appendix A.15)

Yates, F. (1935) Complex experiments, \emph{Journal of the Royal Statistical
Society} Suppl. 2, 181-247
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{auto.noise}
\alias{auto.noise}
\title{Auto Pollution Filter Noise}
\format{
A data frame with 36 observations on the following 4 variables.
  \describe{
  \item{\code{noise}}{Noise level in decibels (but see note) - a numeric vector.}
  \item{\code{size}}{The size of the vehicle - an ordered factor with
    levels \code{S}, \code{M}, \code{L}.}
  \item{\code{type}}{Type of anti-pollution filter - a factor with levels
    \code{Std} and \code{Octel}}
  \item{\code{side}}{The side of the car where measurement was taken -- a
    factor with levels \code{L} and \code{R}.}
  }
}
\source{
The dataset was obtained from the Data and Story Library (DASL)
  at Carnegie-Mellon University. Apparently it has since been removed. The
  original dataset was altered by assigning meaningful names to the factors
  and sorting the observations in random order as if this were the run order
  of the experiment.
}
\usage{
auto.noise
}
\description{
Three-factor experiment comparing pollution-filter noise for two filters,
three sizes of cars, and two sides of the car.
}
\details{
The data are from a statement by Texaco, Inc., to the Air and Water Pollution
Subcommittee of the Senate Public Works Committee on June 26, 1973.    
Mr. John McKinley, President of Texaco, cited an automobile filter developed
by Associated Octel Company as effective in reducing pollution. However, 
questions had been raised about the effects of filters on vehicle performance, 
fuel consumption, exhaust gas back pressure, and silencing. On the last 
question, he referred to the data included here as evidence that the silencing
properties of the Octel filter were at least equal to those of standard silencers.
}
\note{
While the data source claims that \code{noise} is measured in decibels,
  the values are implausible. I believe that these measurements are actually
  in tenths of dB (centibels?). Looking at the values in the dataset, note 
  that every measurement ends in 0 or 5, and it is reasonable to believe that
  measurements are accurate to the nearest half of a decibel.
  %%% Thanks to an email communication from a speech/hearing scientist
}
\examples{
# (Based on belief that noise/10 is in decibel units)
noise.lm <- lm(noise/10 ~ size * type * side, data = auto.noise)

# Interaction plot of predictions
emmip(noise.lm, type ~ size | side)

# Confidence intervals
plot(emmeans(noise.lm, ~ size | side*type))

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{fiber}
\alias{fiber}
\title{Fiber data}
\format{
A data frame with 15 observations and 3 variables:
\describe{
  \item{\code{machine}}{a factor with levels \code{A} \code{B} \code{C}. 
    This is the primary factor of interest.}
  \item{\code{strength}}{a numeric vector. The response variable.}
  \item{\code{diameter}}{a numeric vector. A covariate.}
}
}
\source{
Montgomery, D. C. (2013) \emph{Design and Analysis of Experiments}
  (8th ed.). John Wiley and Sons, ISBN 978-1-118-14692-7.
}
\usage{
fiber
}
\description{
Fiber data from Montgomery Design (8th ed.), p.656 (Table 15.10). Useful as a
simple analysis-of-covariance example.
}
\details{
The goal of the experiment is to compare the mean breaking strength of fibers
produced by the three machines. When testing this, the technician also
measured the diameter of each fiber, and this measurement may be used as a
concomitant variable to improve precision of the estimates.
}
\examples{
fiber.lm <- lm(strength ~ diameter + machine, data=fiber)
ref_grid(fiber.lm)

# Covariate-adjusted means and comparisons
emmeans(fiber.lm, pairwise ~ machine)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emmGrid-methods.R
\docType{data}
\name{emm_options}
\alias{emm_options}
\alias{get_emm_option}
\alias{with_emm_options}
\alias{emm_defaults}
\title{Set or change emmeans options}
\format{
An object of class \code{list} of length 25.
}
\usage{
emm_options(..., disable)

get_emm_option(x, default = emm_defaults[[x]])

with_emm_options(..., expr)

emm_defaults
}
\arguments{
\item{...}{Option names and values (see Details)}

\item{disable}{If non-missing, this will reset all options to their defaults 
if \code{disable} tests \code{TRUE} (but first save them for possible later 
restoration). Otherwise, all previously saved options
are restored. This is important for bug reporting; please see the section below
on reproducible bugs. When \code{disable} is specified, the other arguments are ignored.}

\item{x}{Character value - the name of an option to be queried}

\item{default}{Value to return if \code{x} is not found}

\item{expr}{Expression to evaluate. If missing, the last element of \code{...} is used.}
}
\value{
\code{emm_options} returns the current options (same as the result 
  of \samp{getOption("emmeans")}) -- invisibly, unless called with no arguments.

\code{get_emm_option} returns the currently stored option for \code{x}, 
  or its default value if not found.

\code{with_emm_options()} temporarily sets the options in \code{...}, then 
  evaluates \code{try(expr)} and returns the result.
}
\description{
Use \code{emm_options} to set or change various options that are used in
the \pkg{emmeans} package. These options are set separately for different contexts in
which \code{emmGrid} objects are created, in a named list of option lists.
}
\details{
\pkg{emmeans}'s options are stored as a list in the system option \code{"emmeans"}. 
Thus, \code{emm_options(foo = bar)} is the same as 
\code{options(emmeans = list(..., foo = bar))} where \code{...} represents any
previously existing options. The list \code{emm_defaults} contains the default
values in case the corresponding element of system option \code{emmeans} is \code{NULL}.

Currently, the following main list entries are supported:
\describe{
\item{\code{ref_grid}}{A named \code{list} of defaults for objects created by
\code{\link{ref_grid}}. This could affect other objects as well. For example,
if \code{emmeans} is called with a fitted model object, it calls
\code{ref_grid} and this option will affect the resulting \code{emmGrid}
object.}
\item{\code{emmeans}}{A named \code{list} of defaults for objects created by
  \code{\link{emmeans}} or \code{\link{emtrends}}.}
\item{\code{contrast}}{A named \code{list} of defaults for objects created by
  \code{\link{contrast.emmGrid}} or \code{\link{pairs.emmGrid}}.}
\item{\code{summary}}{A named \code{list} of defaults used by the methods
  \code{\link{summary.emmGrid}}, \code{\link{predict.emmGrid}}, \code{\link{test.emmGrid}},
  \code{\link{confint.emmGrid}}, and \code{\link{emmip}}. The only option that can
  affect the latter four is \code{"predict.type"}.}
\item{\code{allow.na.levs}}{A logical value that if \code{TRUE} (the default), allows
  \code{NA} to be among the levels of a factor. Older versions of \pkg{emmeans} did not
  allow this. So if problems come up (say in a messy dataset that includes incomplete cases), 
  try setting this to \code{FALSE}.}
\item{\code{sep}}{A character value to use as a separator in labeling factor combinations.
  Such labels are potentially used in several places such as \code{\link{contrast}} and 
  \code{\link{plot.emmGrid}} when combinations of factors are compared or plotted.
  The default is \code{" "}.}
\item{\code{parens}}{Character vector that determines which labels are parenthesized
  when they are contrasted. The first element is a regular expression, and the second and
  third elements are used as left and right parentheses. 
  See details for the \code{parens} argument in \code{\link{contrast}}. The default
  will parenthesize labels containing the four arithmetic operators, 
  using round parentheses.}
\item{\code{cov.keep}}{The default value of \code{cov.keep} in \code{\link{ref_grid}}.
  Defaults to \code{"2"}, i.e., two-level covariates are treated like factors.}
\item{\code{graphics.engine}}{A character value matching 
  \code{c("ggplot", "lattice")}, setting the default engine to use in
  \code{\link{emmip}} and \code{\link{plot.emmGrid}}.  Defaults to \code{"ggplot"}.}
\item{\code{gg.theme}}{An integer. Set it to 1 or 2 if you want the appearance
  of plots used in \pkg{emmeans} version 1.x.x or 2.x.x. Or set it to the actual theme
  that you want to use, e.g., \code{emm_options(gg.theme = ggplot2::theme_dark())}}
\item{\code{msg.interaction}}{A logical value controlling whether or not
  a message is displayed when \code{emmeans} averages over a factor involved
  in an interaction. It is probably not appropriate to do this, unless
  the interaction is weak. Defaults to \code{TRUE}.}
\item{\code{msg.nesting}}{A logical value controlling whether or not to
  display a message when a nesting structure is auto-detected. The existence
  of such a structure affects computations of EMMs. Sometimes, a nesting
  structure is falsely detected -- namely when a user has omitted some
  main effects but included them in interactions. This does not change the
  model fit, but it produces a different parameterization that is picked
  up when the reference grid is constructed. Defaults to \code{TRUE}.}
\item{\code{rg.limit}}{An integer value setting a limit on the number of rows
  in a newly constructed reference grid. This is checked based on the number of
  levels of the factors involved; but it excludes the levels of any multivariate
  responses because those are not yet known. The reference grid consists of all
  possible combinations of the predictors, and this can become huge if there are
  several factors. An error is thrown if this limit is exceeded. One can use the 
  \code{nuisance} argument of \code{\link{ref_grid}} to collapse on nuisance
  factors, thus making the grid smaller. Defaults to 10,000.}
\item{\code{simplify.names}}{A logical value controlling whether to
  simplify (when possible) names in the model formula that refer to datasets --
  for example, should we simplify a predictor name like \dQuote{\code{data$trt}}
  to just \dQuote{\code{trt}}? Defaults to \code{TRUE}.}
\item{\code{opt.digits}}{A logical value controlling the precision with which
  summaries are printed. If \code{TRUE} (default), the number of digits
  displayed is just enough to reasonably distinguish estimates from the ends
  of their confidence intervals; but always at least 3 digits. If
  \code{FALSE}, the system value \code{getOption("digits")} is used.}
\item{\code{pval.digits}}{An integer indicating the precision with which
  p-values are printed. Integers between \code{2} and \code{6} are acceptable; the default is 
\code{4}. Floating point numbers are truncated. If a number outside the accepted range is
used, the nearest acceptable integer will be used instead. Non-numeric arguments are ignored.}
\item{\code{back.bias.adj}}{A logical value controlling whether we 
  try to adjust bias when back-transforming. If \code{FALSE}, we use naive
  back transformation. If \code{TRUE} \emph{and \code{sigma} is available and valid}, a
  second-order adjustment is applied to estimate the mean on the response
  scale. A warning is issued if no valid \code{sigma} is available}
\item{\code{enable.submodel}}{A logical value. If \code{TRUE}, enables support 
  for selected model classes to implement the \code{submodel} option. If
  \code{FALSE}, this support is disabled. Setting this option to \code{FALSE}
  could save excess memory consumption.}
  
}%%% end describe{}
Some other options have more specific purposes:
\describe{
\item{\code{estble.tol}}{Tolerance for determining estimability in
rank-deficient cases. If absent, the value in \code{emm_defaults$estble.tol)}
is used.}
\item{\code{save.ref_grid}}{Logical value of \code{TRUE} if you wish the 
latest reference grid created to be saved in \code{.Last.ref_grid}.
The default is \code{FALSE}.}
\item{Options for \code{lme4::lmerMod} models}{Options \code{lmer.df},
\code{disable.pbkrtest}, \code{pbkrtest.limit}, \code{disable.lmerTest},
and \code{lmerTest.limit}
options affect how degrees of freedom are computed for \code{lmerMod} objects
produced by the \pkg{lme4} package). See that section of the "models" vignette
for details.}
\item{\code{post.ci.method}}{method for estimating confidence intervals for Bayesian models. 
Options are \code{"HPD"} (default) or \code{"quantile"}.} 
} %%%%%% end \describe
}
\section{Reproducible bugs}{

Most options set display attributes and such that are not likely to be associated
with bugs in the code. However, some other options (e.g., \code{cov.keep})
are essentially configuration settings that may affect how/whether the code
runs, and the settings for these options may cause subtle effects that may be
hard to reproduce. Therefore, when sending a bug report, please create a reproducible
example and make sure the bug occurs with all options set at their defaults.
This is done by preceding it with  \code{emm_options(disable = TRUE)}. 

By the way, \code{disable} works like a stack (LIFO buffer), in that \code{disable = TRUE}
is equivalent to \code{emm_options(saved.opts = emm_options())} and 
\code{emm_options(disable = FALSE)} is equivalent to 
\code{options(emmeans = get_emm_option("saved.opts"))}. To completely erase
all options, use \code{options(emmeans = NULL)}
}

\section{Startup options}{

\pkg{emmeans}'s options are kept in the system options in a list names \code{emmeans}.
Thus, if the user wants certain options set on startup, this can be arranged by
setting the \code{emmeans} option in one's startup script. For example,
the user's \code{.Rprofile} file could contain:
\preformatted{    options(emmeans = list(summary = list(predict.type = "response"), 
                           lmer.df = "satterthwaite"))}
}

\examples{
\dontrun{
emm_options(ref_grid = list(level = .90),
            contrast = list(infer = c(TRUE,FALSE)),
            estble.tol = 1e-6)
# Sets default confidence level to .90 for objects created by ref.grid
# AS WELL AS emmeans called with a model object (since it creates a 
# reference grid). In addition, when we call 'contrast', 'pairs', etc.,
# confidence intervals rather than tests are displayed by default.
}

\dontrun{
emm_options(disable.pbkrtest = TRUE)
# This forces use of asymptotic methods for lmerMod objects.
# Set to FALSE or NULL to re-enable using pbkrtest.
}

# See tolerance being used for determining estimability
get_emm_option("estble.tol")

\dontrun{
# Set all options to their defaults
emm_options(disable = TRUE)
# ... and perhaps follow with code for a minimal reproducible bug,
#     which may include emm_options() clls if they are pertinent ...

# restore options that had existed previously
emm_options(disable = FALSE)
}


# Temporarily alter the formatting of labels in an interaction contrast:
pigs.lm <- lm(inverse(conc) ~ source * factor(percent), data = pigs)
with_emm_options(parens = c("\\\\-", "<", ">"), sep = ": ",
    contrast(contrast(ref_grid(pigs.lm), "consec", by = "percent"),
        "consec", by = NULL))

}
\seealso{
\code{\link{update.emmGrid}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emmeans.R
\name{emmeans}
\alias{emmeans}
\title{Estimated marginal means (Least-squares means)}
\usage{
emmeans(object, specs, by = NULL, fac.reduce = function(coefs) apply(coefs,
  2, mean), contr, options = get_emm_option("emmeans"), weights, offset, ...,
  tran)
}
\arguments{
\item{object}{An object of class \code{emmGrid}; or a fitted model object
that is supported, such as the result of a call to \code{lm} or
\code{lmer}. Many fitted-model objects are supported; see
\href{../doc/models.html}{\code{vignette("models", "emmeans")}} for details.}

\item{specs}{A \code{character} vector specifying the names of the predictors
over which EMMs are desired. \code{specs} may also be a \code{formula}
or a \code{list} (optionally named) of valid \code{spec}s. Use of formulas
is described in the Overview section below. 
Specifying \code{.} as the only factor name
creates a list of specifications for all model terms.

\bold{Note:} We recommend \emph{against} using two-sided formulas; see the
note below for \code{contr}.}

\item{by}{A character vector specifying the names of predictors to condition on.}

\item{fac.reduce}{A function that combines the rows of a matrix into a single
vector. This implements the ``marginal averaging'' aspect of EMMs. 
The default is the mean of the rows. Typically if it is overridden,
it would be some kind of weighted mean of the rows. If \code{fac.reduce} is
nonlinear, bizarre results are likely, and EMMs will not be
interpretable. NOTE: If the \code{weights} argument is non-missing,
\code{fac.reduce} is ignored.}

\item{contr}{A character value or \code{list} specifying contrasts to be
added. See \code{\link{contrast}}. \bold{Note:} \code{contr} is ignored when
\code{specs} is a formula. \bold{Note 2:}: We recommend \emph{against} using this
argument; obtaining means and obtaining contrasts are two different things,
and it is best to do them in separate steps, using the \code{\link{contrast}}
function for the contrasts.}

\item{options}{If non-\code{NULL}, a named \code{list} of arguments to pass
to \code{\link{update.emmGrid}}, just after the object is constructed.
(Options may also be included in \code{...}; see the \sQuote{options}
section below.)}

\item{weights}{Character value, numeric vector, or numeric matrix specifying
weights to use in averaging predictions. See \dQuote{Weights} section below.
Also, if \code{object} is not already a reference grid, \code{weights}
(if it is character) is passed to \code{ref_grid} as \code{wt.nuis} in case 
nuisance factors are specified. We can override this by specifying 
\code{wt.nuis} explicitly.
This more-or-less makes the weighting of nuisance factors consistent with
that of primary factors.}

\item{offset}{Numeric vector or scalar. If specified, this adds an offset to
the predictions, or overrides any offset in the model or its
reference grid. If a vector of length differing from the number of rows in 
the result, it is subsetted or cyclically recycled.}

\item{...}{When \code{object} is not already a \code{"emmGrid"}
object, these arguments are passed to \code{\link{ref_grid}}. Common
examples are \code{at}, \code{cov.reduce}, \code{data}, \code{type}, 
\code{regrid}, \code{df}, \code{nesting}, and \code{vcov.}.
Model-type-specific options (see
\href{../doc/models.html}{\code{vignette("models", "emmeans")}}), commonly
\code{mode}, may be used here as well. In addition, if the model formula
contains references to variables that are not predictors, you must provide
a \code{params} argument with a list of their names.
These arguments may also be used in lieu of \code{options}. See the 
\sQuote{Options} section below.}

\item{tran}{Placeholder to prevent it from being included in \code{...}.
If non-missing, it is added to `options`. See the \sQuote{Options}
section.}
}
\value{
When \code{specs} is a \code{character} vector or one-sided formula,
  an object of class \code{"emmGrid"}. A number of methods
  are provided for further analysis, including
  \code{\link{summary.emmGrid}}, \code{\link{confint.emmGrid}},
  \code{\link{test.emmGrid}}, \code{\link{contrast.emmGrid}},
  and \code{\link{pairs.emmGrid}}.
When \code{specs} is a \code{list} or a \code{formula} having a left-hand
side, the return value is an \code{\link{emm_list}} object, which is simply a
\code{list} of \code{emmGrid} objects.
}
\description{
Compute estimated marginal means (EMMs) for specified factors
or factor combinations in a linear model; and optionally, comparisons or
contrasts among them. EMMs are also known as least-squares means.
}
\details{
Users should also consult the documentation for \code{\link{ref_grid}}, 
because many important options for EMMs are implemented there, via the 
\code{...} argument.
}
\section{Overview}{

Estimated marginal means or EMMs (sometimes called least-squares means) are
predictions from a linear model over a \emph{reference grid}; or marginal
averages thereof. The \code{\link{ref_grid}} function identifies/creates the
reference grid upon which \code{emmeans} is based.

For those who prefer the terms \dQuote{least-squares means} or
\dQuote{predicted marginal means}, functions \code{lsmeans} and
\code{pmmeans} are provided as wrappers. See \code{\link{wrappers}}.

If \code{specs} is a \code{formula}, it should be of the form \code{~ specs},
\code{~ specs | by}, \code{contr ~ specs}, or \code{contr ~ specs | by}. The
formula is parsed and the variables therein are used as the arguments
\code{specs}, \code{by}, and \code{contr} as indicated. The left-hand side is
optional (and we don't recommend it), but if specified it should be the 
name of a contrast family (e.g., \code{pairwise}). Operators like
\code{*} or \code{:} are needed in the formula to delineate names, but
otherwise are ignored.

We now also allow using \code{.} in \code{specs}. If this is done, we run
\code{\link{joint_tests}} on the side to determine all relevant model terms,
then replace \code{specs} with a corresponding list of specifications. This is
a convenience, but it can create a sizeable \code{emm_list} object and it is
coded rather inefficiently. While it is permissible to include contrasts
via a \code{contr} argument or formula left-hand-side, we recommend instead
doing this in a follow-up test with \code{\link{contrast.emm_list}}. \emph{Caution}:
In models with nested fixed effects, using \code{.} creates results where
nested factors interact with nesting factors; in those cases, any contrasts 
you specify will go across nests, which is likely not what is desired.

In the special case where the mean (or weighted mean) of all the predictions
is desired, specify \code{specs} as \code{~ 1} or \code{"1"}.

A number of standard contrast families are provided. They can be identified 
as functions having names ending in \code{.emmc} -- see the documentation
for \code{\link{emmc-functions}} for details -- including how to write your
own \code{.emmc} function for custom contrasts.
}

\section{Weights}{

If \code{weights} is a vector, its length must equal
  the number of predictions to be averaged to obtain each EMM.
  If a matrix, each row of the matrix is used in turn, wrapping back to the
  first row as needed.  When in doubt about what is being averaged (or how
  many), first call \code{emmeans} with \code{weights = "show.levels"}.
  
If \code{weights} is a string, it should partially match one of the following:
\describe{
\item{\code{"equal"}}{Use an equally weighted average.}
\item{\code{"proportional"}}{Weight in proportion to the frequencies (in the
  original data) of the factor combinations that are averaged over.}
\item{\code{"outer"}}{Weight in proportion to each individual factor's
  marginal frequencies. Thus, the weights for a combination of factors are the
  outer product of the one-factor margins}
\item{\code{"cells"}}{Weight according to the frequencies of the cells being
  averaged.}
\item{\code{"flat"}}{Give equal weight to all cells with data, and ignore
  empty cells.}
\item{\code{"show.levels"}}{This is a convenience feature for understanding
  what is being averaged over. Instead of a table of EMMs, this causes the
  function to return a table showing the levels that are averaged over, in the
  order that they appear.}
}
Outer weights are like the 'expected' counts in a chi-square test of
independence, and will yield the same results as those obtained by
proportional averaging with one factor at a time. All except \code{"cells"}
uses the same set of weights for each mean. In a model where the predicted
values are the cell means, cell weights will yield the raw averages of the
data for the factors involved. Using \code{"flat"} is similar to
\code{"cells"}, except nonempty cells are weighted equally and empty cells
are ignored.
}

\section{Counterfactuals}{

Counterfactual reference grids (see the documentation for \code{\link{ref_grid}})
contain pairs of imputed and actual factor levels, and are handled in a special way.
For starters, the \code{weights} argument is ignored and we always use 
\code{"cells"} weights.
Our understanding is that if factors \code{A, B} are specified as counterfactuals, 
the marginal means for \code{A} should still be the same as if \code{A} were the only 
counterfactual. Accordingly, in computing these marginal means, we
exclude all cases where \code{B != actual_B}, because if \code{A} were the only
counterfactual, \code{B} will stay at its actual level.
We also take special pains to "remember" information about actual and 
imputed levels of counterfactuals so that appropriate results are obtained when
\code{emmeans} is applied to a previous \code{emmeans} result.
}

\section{Offsets}{

Unlike in \code{ref_grid}, an offset need not be scalar. If not enough values
are supplied, they are cyclically recycled. For a vector of offsets, it is 
important to understand that the ordering of results goes with the first 
name in \code{specs} varying fastest. If there are any \code{by} factors,
those vary slower than all the primary ones, but the first \code{by} variable
varies the fastest within that hierarchy. See the examples.
}

\section{Options and \code{...}}{

  Arguments that could go in \code{options} may instead be included in \code{...},
  typically, arguments such as \code{type}, \code{infer}, etc. that in essence
  are passed to \code{\link{summary.emmGrid}}. Arguments in both places are 
  overridden by the ones in \code{...}.
  
  There is a danger that \code{...} arguments could partially match those used
  by both \code{ref_grid} and \code{update.emmGrid}, creating a conflict.
  If these occur, usually they can be resolved by providing complete (or at least 
  longer) argument names; or by isolating non-\code{ref_grid} arguments in
  \code{options}; or by calling \code{ref_grid} separately and passing the
  result as \code{object}. See a not-run example below.
  
  Also, when \code{specs} is a two-sided formula, or \code{contr} is specified,
  there is potential confusion concerning which \code{...} arguments
  apply to the means, and which to the contrasts. When such confusion is possible,
  we suggest doing things separately 
  (a call to \code{emmeans} with no contrasts, followed by a call to 
  \code{\link{contrast}}). We treat
  \code{adjust} as a special case: it is applied to the \code{emmeans} results 
  \emph{only} if there are
  no contrasts specified, otherwise it is passed only to \code{contrast}.
}

\examples{
warp.lm <- lm(breaks ~ wool * tension, data = warpbreaks)
emmeans (warp.lm,  ~ wool | tension)
# or equivalently emmeans(warp.lm, "wool", by = "tension")

# 'adjust' argument ignored in emmeans, passed to contrast part...
emmeans (warp.lm, poly ~ tension | wool, adjust = "sidak")

# 'adjust' argument NOT ignored ...
emmeans (warp.lm, ~ tension | wool, adjust = "sidak")

# Get all sets of EMMs for this model
( allsets <- emmeans(warp.lm, ".") )
contrast(allsets, "eff")    # all effects


\dontrun{
  ### Offsets: Consider a silly example:
  emmeans(warp.lm, ~ tension | wool, offset = c(17, 23, 47)) @ grid
  # note that offsets are recycled so that each level of tension receives
  # the same offset for each wool.
  # But using the same offsets with ~ wool | tension will probably not
  # be what you want because the ordering of combinations is different.
}
}
\seealso{
\code{\link{ref_grid}}, \code{\link{contrast}}, 
\href{../doc/models.html}{vignette("models", "emmeans")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S4-classes.R
\docType{class}
\name{emmGrid-class}
\alias{emmGrid-class}
\title{The \code{emmGrid} class}
\description{
The \code{emmGrid} class encapsulates linear functions of regression
parameters, defined over a grid of predictors. This includes reference
grids and grids of marginal means thereof (aka estimated marginal means).
Objects of class `emmGrid` may be used independently of the underlying model
object. Instances are created primarily by \code{\link{ref_grid}} and
\code{\link{emmeans}}, and several related functions.
}
\section{Slots}{

\describe{
\item{\code{model.info}}{list. Contains the elements \code{call} (the call that
produced the model), \code{terms} (its \code{terms} object), and
\code{xlev} (factor-level information)}

\item{\code{roles}}{list. Contains at least the elements \code{predictors}, 
\code{responses}, and \code{multresp}. Each is a character vector of names 
of these variables.}

\item{\code{grid}}{data.frame. Contains the combinations of the variables that define
the reference grid. In addition, there is an auxiliary column named
\code{".wgt."} holding the observed frequencies or weights for each factor
combination (excluding covariates). If the model has one or more
\code{\link{offset}()} calls, there is an another auxiliary column named
\code{".offset."}. Auxiliary columns are not considered part of the
reference grid. (However, any variables included in \code{offset} calls
\emph{are} in the reference grid.)}

\item{\code{levels}}{list. Each entry is a character vector with the distinct levels
of each variable in the reference grid. Note that \code{grid} is obtained
by applying the function \code{\link{expand.grid}} to this list}

\item{\code{matlevs}}{list. Like \code{levels} but has the levels of any matrices in
the original dataset. Matrix columns are always concatenated and treated as
a single variable for purposes of the reference grid}

\item{\code{linfct}}{matrix. Each row consists of the linear function of the
regression coefficients for predicting its corresponding element of the
reference grid. The rows of this matrix go in one-to-one correspondence
with the rows of \code{grid}, and the columns with elements of \code{bhat}.}

\item{\code{bhat}}{numeric. The regression coefficients. If there is a multivariate
response, the matrix of coefficients is flattened to a single vector, and
\code{linfct} and \code{V} redefined appropriately. Important: \code{bhat}
must \emph{include} any \code{NA} values produced as a result of 
collinearity in the predictors. These are taken care of later in the 
estimability check.}

\item{\code{nbasis}}{matrix. The basis for the non-estimable functions of the
regression coefficients. Every EMM will correspond to a linear combination
of rows of \code{linfct}, and that result must be orthogonal to all the
columns of \code{nbasis} in order to be estimable. If everything is
estimable, \code{nbasis} should be a 1 x 1 matrix of \code{NA}.}

\item{\code{V}}{matrix. The symmetric variance-covariance matrix of \code{bhat}}

\item{\code{dffun}}{function having two arguments. \code{dffun(k, dfargs)} should
return the degrees of freedom for the linear function \code{sum(k*bhat)},
or \code{NA} if unavailable}

\item{\code{dfargs}}{list. Used to hold any additional information needed by
\code{dffun}.}

\item{\code{misc}}{list. Additional information used by methods. These include at
least the following: \code{estName} (the label for the estimates of linear
functions), and the default values of \code{infer}, \code{level}, and
\code{adjust} to be used in the \code{\link{summary.emmGrid}} method. Elements in
this slot may be modified if desired using the \code{\link{update.emmGrid}}
method.}

\item{\code{post.beta}}{matrix. A sample from the posterior distribution of the
regression coefficients, if MCMC methods were used; or a 1 x 1 matrix of
\code{NA} otherwise. When it is non-trivial, the \code{\link{as.mcmc.emmGrid}}
method returns \code{post.beta \%*\% t(linfct)}, which is a sample from the
posterior distribution of the EMMs.}
}}

\section{Methods}{

  All methods for these objects are S3 methods except for \code{show}. 
  They include \code{\link{[.emmGrid}}, \code{\link{as.glht.emmGrid}},
  \code{\link{as.mcmc.emmGrid}}, \code{\link{as.mcmc.list.emmGrid}} (see \pkg{coda}),
  \code{\link{cld.emmGrid}} (see \pkg{multcomp}),
  \code{\link{coef.emmGrid}}, \code{\link{confint.emmGrid}}, 
  \code{\link{contrast.emmGrid}}, \code{\link{pairs.emmGrid}},
  \code{\link{plot.emmGrid}}, \code{\link{predict.emmGrid}}, \code{\link{print.emmGrid}},
  \code{\link{rbind.emmGrid}}, \code{show.emmGrid}, \code{\link{str.emmGrid}}, 
  \code{\link{summary.emmGrid}}, \code{\link{test.emmGrid}}, 
  \code{\link{update.emmGrid}}, \code{\link{vcov.emmGrid}}, and 
  \code{\link{xtable.emmGrid}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R, R/test.R
\name{summary.emmGrid}
\alias{summary.emmGrid}
\alias{confint.emmGrid}
\alias{test}
\alias{test.emmGrid}
\alias{predict.emmGrid}
\alias{as.data.frame.emmGrid}
\alias{[.summary_emm}
\title{Summaries, predictions, intervals, and tests for \code{emmGrid} objects}
\usage{
\method{summary}{emmGrid}(object, infer, level, adjust, by,
  cross.adjust = "none", type, df, calc, null, delta, side, frequentist,
  bias.adjust = get_emm_option("back.bias.adj"), sigma, ...)

\method{confint}{emmGrid}(object, parm, level = 0.95, ...)

test(object, null, ...)

\method{test}{emmGrid}(object, null = 0, joint = FALSE, verbose = FALSE,
  rows, by, status = FALSE, ...)

\method{predict}{emmGrid}(object, type, interval = c("none", "confidence",
  "prediction"), level = 0.95,
  bias.adjust = get_emm_option("back.bias.adj"), sigma, ...)

\method{as.data.frame}{emmGrid}(x, row.names = NULL, optional,
  check.names = TRUE, destroy.annotations = FALSE, ...)

\method{[}{summary_emm}(x, ..., as.df = FALSE)
}
\arguments{
\item{object}{An object of class \code{"emmGrid"} (see \link{emmGrid-class})}

\item{infer}{A vector of one or two logical values. The first determines
whether confidence intervals are displayed, and the second determines
whether \emph{t} tests and \emph{P} values are displayed. If only one value
is provided, it is used for both.}

\item{level}{Numerical value between 0 and 1. Confidence level for confidence
intervals, if \code{infer[1]} is \code{TRUE}.}

\item{adjust}{Character value naming the method used to adjust \eqn{p} values
or confidence limits; or to adjust comparison arrows in \code{plot}. See
the P-value adjustments section below.}

\item{by}{Character name(s) of variables to use for grouping into separate 
tables. This affects the family of tests considered in adjusted \emph{P}
values.}

\item{cross.adjust}{Character: \eqn{p}-value adjustment method to 
additionally apply \emph{across} 
the \code{by} groups. See the section on P-value adjustments for details.}

\item{type}{Character: type of prediction desired. This only has an effect if
there is a known transformation or link function. \code{"response"} 
specifies that the inverse transformation be applied. \code{"mu"} (or 
equivalently, \code{"unlink"}) is usually the same as \code{"response"},
but in the case where the model has both a link function and a response 
transformation, only the link part is back-transformed. Other valid values 
are \code{"link"}, \code{"lp"}, and \code{"linear.predictor"}; these are
equivalent, and request that results be shown for the linear predictor,
with no back-transformation. The default is \code{"link"}, unless the 
\code{"predict.type"} option is in force; see \code{\link{emm_options}},
and also the section below on transformations and links.}

\item{df}{Numeric. If non-missing, a constant number of degrees of freedom to
use in constructing confidence intervals and \emph{P} values (\code{NA}
specifies asymptotic results).}

\item{calc}{Named list of character value(s) or formula(s).
The expressions in \code{char} are evaluated and appended to the
summary, just after the \code{df} column. The expression may include
any names up through \code{df} in the summary, any additional names in 
\code{object@grid} (such as \code{.wgt.} or \code{.offset.}), or any
earlier elements of \code{calc}.}

\item{null}{Numeric. Null hypothesis value(s), on the linear-predictor scale,
against which estimates are tested. May be a single value used for all, or
a numeric vector of length equal to the number of tests in each family
(i.e., \code{by} group in the displayed table).}

\item{delta}{Numeric value (on the linear-predictor scale). If zero, ordinary
tests of significance are performed. If positive, this specifies a
threshold for testing equivalence (using the TOST or two-one-sided-test
method), non-inferiority, or non-superiority, depending on \code{side}. See
Details for how the test statistics are defined.}

\item{side}{Numeric or character value specifying whether the test is
left-tailed (\code{-1}, \code{"-"}, \code{"<"}, \code{"left"}, or
\code{"nonsuperiority"}); right-tailed (\code{1}, \code{"+"}, \code{">"},
\code{"right"}, or \code{"noninferiority"}); or two-sided (\code{0},
\code{2}, \code{"!="}, \code{"two-sided"}, \code{"both"},
\code{"equivalence"}, or \code{"="}). See the special section below for
more details.}

\item{frequentist}{Ignored except if a Bayesian model was fitted. If missing
or \code{FALSE}, the object is passed to \code{\link{hpd.summary}}. Otherwise, 
a logical value of \code{TRUE} will have it return a frequentist summary.}

\item{bias.adjust}{Logical value for whether to adjust for bias in
back-transforming (\code{type = "response"}). This requires a valid value of 
\code{sigma} to exist in the object or be specified.}

\item{sigma}{Error SD assumed for bias correction (when 
\code{type = "response"} and a transformation
is in effect), or for constructing prediction intervals. If not specified,
\code{object@misc$sigma} is used, and a warning is issued if it is not found
or not valid.
\emph{Note:} \code{sigma} may be a vector, but be careful that it correctly
corresponds (perhaps after recycling) to the order of the reference grid.}

\item{...}{Optional arguments such as \code{scheffe.rank} 
(see \dQuote{P-value adjustments}). 
In \code{confint.emmGrid}, 
\code{predict.emmGrid}, and 
\code{test.emmGrid}, these arguments are passed to
\code{summary.emmGrid}. These arguments are also passed to \code{mvregrid}
when we have a multivariate transformation.}

\item{parm}{(Required argument for \code{confint} methods, but not used)}

\item{joint}{Logical value. If \code{FALSE}, the arguments are passed to 
\code{\link{summary.emmGrid}} with \code{infer=c(FALSE, TRUE)}. If \code{joint = 
TRUE}, a joint test of the hypothesis L beta = null is performed, where L 
is \code{linfct(object)} and beta is the vector of fixed effects estimated 
by \code{object@betahat}. This will be either an \emph{F} test or a 
chi-square (Wald) test depending on whether degrees of freedom are 
available. See also \code{\link{joint_tests}}.}

\item{verbose}{Logical value. If \code{TRUE} and \code{joint = TRUE}, a table
of the effects being tested is printed.}

\item{rows}{Integer values. The rows of L to be tested in the joint test. If
missing, all rows of L are used. If not missing, \code{by} variables are
ignored.}

\item{status}{logical. If \code{TRUE}, a \code{note} column showing status
flags (for rank deficiencies and estimability issues) is displayed even 
when empty. If \code{FALSE}, the column is included only if there are 
such issues.}

\item{interval}{Type of interval desired (partial matching is allowed): 
\code{"none"} for no intervals,
  otherwise confidence or prediction intervals with given arguments, 
  via \code{\link{confint.emmGrid}}. 
  Note: prediction intervals are not available
  unless the model family is \code{"gaussian"}.}

\item{x}{object of the given class}

\item{row.names}{passed to \code{\link{as.data.frame}}}

\item{optional}{required argument, but ignored in \code{as.data.frame.emmGrid}}

\item{check.names}{passed to \code{\link{data.frame}}}

\item{destroy.annotations}{Logical value. If \code{FALSE}, an object of class
\code{summary_emm} is returned (which inherits from \code{data.frame}),
but if displayed, details like confidence levels, P-value adjustments, 
transformations, etc. are also shown. But unlike the result
of \code{summary}, the number of digits displayed
is obtained from \code{getOption("digits")} rather than using the
optimal digits algorithm we usually use. Thus, it is formatted more like a 
regular data frame, but with any annotations and groupings still intact.
If \code{TRUE} (not recommended), a \dQuote{plain vanilla} data frame is 
returned, based on \code{row.names} and \code{check.names}.}

\item{as.df}{Logical value. With \code{x[..., as.df = TRUE]}, the result is
object is coerced to a \code{\link{data.frame}} before the subscripting 
is applied. With \code{as.df = FALSE}, the result is
returned as a \code{summary_emm} object when possible.}
}
\value{
\code{summary.emmGrid}, \code{confint.emmGrid}, and
  \code{test.emmGrid} return an object of class \code{"summary_emm"}, which
  is an extension of \code{\link{data.frame}} but with a special \code{print}
  method that displays it with custom formatting. For models fitted using
  MCMC methods, the call is diverted to \code{\link{hpd.summary}} (with 
  \code{prob} set to \code{level}, if specified); one may
  alternatively use general MCMC summarization tools with the 
  results of \code{as.mcmc}.

\code{predict} returns a vector of predictions for each row of \code{object@grid}.

The \code{as.data.frame} method returns an object that inherits 
  from \code{"data.frame"}.
}
\description{
These are the primary methods for obtaining numerical or tabular results from
an \code{emmGrid} object. \code{summary.emmGrid} is the general function for
summarizing \code{emmGrid} objects. It also serves as the print method for
these objects; so for convenience, \code{summary()} arguments may be included
in calls to functions such as \code{\link{emmeans}} and
\code{\link{contrast}} that construct \code{emmGrid} objects. Note that by
default, summaries for Bayesian models are diverted to
\code{\link{hpd.summary}}.
}
\details{
\code{confint.emmGrid} is equivalent to \code{summary.emmGrid with 
infer = c(TRUE, FALSE)}. The function \code{test.emmGrid}, when called with 
\code{joint = FALSE}, is equivalent to \code{summary.emmGrid} with \code{infer = c(FALSE, TRUE)}. 

With \code{joint = TRUE}, \code{test.emmGrid} calculates the Wald test of the
hypothesis \code{linfct \%*\% bhat = null}, where \code{linfct} and
\code{bhat} refer to slots in \code{object} (possibly subsetted according to
\code{by} or \code{rows}). An error is thrown if any row of \code{linfct} is
non-estimable. It is permissible for the rows of \code{linfct} to be linearly
dependent, as long as \code{null == 0}, in which case a reduced set of 
contrasts is tested. Linear dependence and nonzero \code{null} cause an 
error. The returned object has an additional \code{"est.fcns"} attribute, which
is a list of the linear functions associated with the joint test.
}
\note{
In doing testing and a transformation and/or link is in force, any
  \code{null} and/or \code{delta} values specified must always be on the
  scale of the linear predictor, regardless of the setting for `type`. If
  \code{type = "response"}, the null value displayed in the summary table 
  will be back-transformed from the value supplied by the user. But the
  displayed \code{delta} will not be changed, because there (often) is
  not a natural way to back-transform it.

When we have \code{type = "response"}, and \code{bias.adj = TRUE},
  the \code{null} value displayed in the output is both back-transformed
  and bias-adjusted, leading to a rather non-intuitive-looking null value.
  However, since the tests themselves are performed on the link scale,
  this is the response value at which a *P* value of 1 would be obtained.

The default \code{show} method for \code{emmGrid} objects (with the
  exception of newly created reference grids) is \code{print(summary())}.
  Thus, with ordinary usage of \code{\link{emmeans}} and such, it is
  unnecessary to call \code{summary} unless there is a need to
  specify other than its default options.

If a data frame is needed, \code{summary}, \code{confint},
  and \code{test} serve this need. \code{as.data.frame} routes to
  \code{summary} by default; calling it with \code{destroy.annotations = TRUE}
  is not recommended for exactly that reason.
  If you want to see more digits in the output, use
  \code{print(summary(object), digits = ...)}; and if you \emph{always} want
  to see more digits, use \code{emm_options(opt.digits = FALSE)}.
}
\section{Defaults}{

  The \code{misc} slot in \code{object} may contain default values for
  \code{by}, \code{calc}, \code{infer}, \code{level}, \code{adjust}, 
  \code{type}, \code{null}, \code{side}, and \code{delta}. 
  These defaults vary depending
  on the code that created the object. The \code{\link{update}} method may be
  used to change these defaults. In addition, any options set using 
  \samp{emm_options(summary = ...)} will trump those stored in the object's 
  \code{misc} slot.
}

\section{Transformations and links}{

  With \code{type = "response"}, the transformation assumed can be found in
  \samp{object@misc$tran}, and its label, for the summary is in
  \samp{object@misc$inv.lbl}. Any \eqn{t} or \eqn{z} tests are still performed
  on the scale of the linear predictor, not the inverse-transformed one.
  Similarly, confidence intervals are computed on the linear-predictor scale,
  then inverse-transformed. 
  
  Be aware that only univariate transformations and links are
  supported in this way. Some multivariate transformations are supported by 
  \code{\link{mvregrid}}.
}

\section{Bias adjustment when back-transforming}{

  When \code{bias.adjust} is \code{TRUE}, then back-transformed estimates
  are adjusted by adding 
  \eqn{0.5 h''(u)\sigma^2}, where \eqn{h} is the inverse transformation and
  \eqn{u} is the linear predictor. This is based on a second-order Taylor
  expansion. There are better or exact adjustments for certain specific
  cases, and these may be incorporated in future updates.
  
  Note: In certain models, e.g., those with non-gaussian families,
  \code{sigma} is initialized as \code{NA}, and so by default, bias adjustment
  is skipped and a warning is issued. You may override this by specifying a
  value for \code{sigma}. However, \emph{with ordinary generalized linear models,
  bias adjustment is inappropriate} and you should not try to do it. With GEEs and GLMMs,
  you probably should \emph{not} use \code{sigma(model)}, and instead you should create an
  appropriate value using the estimated random effects, e.g., from \code{VarCorr(model)}.
  An example is provided in the \dQuote{transformations} vignette.
  
  \emph{A word of caution:} This bias-adjustment method is merely a one-term
  correction, and it multiplies \code{sigma^2}. When \code{sigma} is reasonably
  small relative to the scale of the response, this is fine; but when
  \code{sigma} is large, this bias adjustment can produce bizarre and
  out-of-range results. Consider for example a logit or probit model where
  the back-transformed response is constrained to the interval [0, 1]; if
  \code{sigma} is more than 1, the adjustment can 
  easily pull the estimate outside of these constraints.
}

\section{P-value adjustments}{

  The \code{adjust} argument specifies a multiplicity adjustment for tests or
  confidence intervals. This adjustment always is applied \emph{separately}
  to each table or sub-table that you see in the printed output (see
  \code{\link{rbind.emmGrid}} for how to combine tables). If there are non-estimable
  cases in a \code{by} group, those cases are \emph{excluded} before determining
  the adjustment; that means there could be different adjustments in different groups.
  
  The valid values of \code{adjust} are as follows:
  \describe{
  \item{\code{"tukey"}}{Uses the Studentized range distribution with the number
    of means in the family. (Available for two-sided cases only.)}
  \item{\code{"scheffe"}}{Computes \eqn{p} values from the \eqn{F}
    distribution, according to the Scheffe critical value of
    \eqn{\sqrt{rF(\alpha; r, d)}}{sqrt[r*qf(alpha, r, d)]}, where \eqn{d} is
    the error degrees of freedom and \eqn{r} is the rank of the set of linear
    functions under consideration. By default, the value of \code{r} is
    computed from \code{linfct(object)} for each by group; however, if the
    user specifies an argument matching \code{scheffe.rank}, its value will
    be used instead. Ordinarily, if there are \eqn{k} means involved, then
    \eqn{r = k - 1} for a full set of contrasts involving all \eqn{k} means, and
    \eqn{r = k} for the means themselves. (The Scheffe adjustment is available
    for two-sided cases only.)}
  \item{\code{"sidak"}}{Makes adjustments as if the estimates were independent
    (a conservative adjustment in many cases).}
  \item{\code{"bonferroni"}}{Multiplies \eqn{p} values, or divides significance
    levels by the number of estimates. This is a conservative adjustment.}
  \item{\code{"dunnettx"}}{Uses our own\emph{ad hoc} approximation to the 
    Dunnett distribution for a family of estimates having pairwise
    correlations of \eqn{0.5} (as is true when comparing treatments with a
    control with equal sample sizes). The accuracy of the approximation
    improves with the number of simultaneous estimates, and is much faster
    than \code{"mvt"}. (Available for two-sided cases only.)}
  \item{\code{"mvt"}}{Uses the multivariate \eqn{t} distribution to assess the
    probability or critical value for the maximum of \eqn{k} estimates. This
    method produces the same \eqn{p} values and intervals as the default
    \code{summary} or \code{confint} methods to the results of
    \code{\link{as.glht}}. In the context of pairwise comparisons or comparisons
    with a control, this produces \dQuote{exact} Tukey or Dunnett adjustments,
    respectively. However, the algorithm (from the \pkg{mvtnorm} package) uses a
    Monte Carlo method, so results are not exactly repeatable unless the same
    random-number seed is used (see \code{\link[base:Random]{set.seed}}). As the family
    size increases, the required computation time will become noticeable or even
    intolerable, making the \code{"tukey"}, \code{"dunnettx"}, or others more
    attractive.}
  \item{\code{"none"}}{Makes no adjustments to the \eqn{p} values.}
  } %%%%%%%%%%%%%%%% end \describe {}

  For tests, not confidence intervals, the Bonferroni-inequality-based adjustment
  methods in \code{\link{p.adjust}} are also available (currently, these
  include \code{"holm"}, \code{"hochberg"}, \code{"hommel"},
  \code{"bonferroni"}, \code{"BH"}, \code{"BY"}, \code{"fdr"}, and
  \code{"none"}). If a \code{p.adjust.methods} method other than
  \code{"bonferroni"} or \code{"none"} is specified for confidence limits, the
  straight Bonferroni adjustment is used instead. Also, if an adjustment method
  is not appropriate (e.g., using \code{"tukey"} with one-sided tests, or with
  results that are not pairwise comparisons), a more appropriate method
  (usually \code{"sidak"}) is substituted.

  In some cases, confidence and \eqn{p}-value adjustments are only approximate
  -- especially when the degrees of freedom or standard errors vary greatly
  within the family of tests. The \code{"mvt"} method is always the correct
  one-step adjustment, but it can be very slow. One may use
  \code{\link{as.glht}} with methods in the \pkg{multcomp} package to obtain
  non-conservative multi-step adjustments to tests.
  
  \emph{Warning:} Non-estimable cases are \emph{included} in the family to which adjustments
  are applied. You may wish to subset the object using the \code{[]} operator
  to work around this problem.
  
  The \code{cross.adjust} argument is a way of specifying a multiplicity
  adjustment across the \code{by} groups (otherwise by default, each group is
  treated as a separate family in regards to multiplicity adjustments). It
  applies only to \eqn{p} values. Valid options are one of the
  \code{p.adjust.methods} or \code{"sidak"}. This argument is ignored unless
  it is other than \code{"none"}, there is more than one \code{by} group, and
  they are all the same size. Under those conditions, we first use
  \code{adjust} to determine the within-group adjusted \eqn{p} values.
  Imagine each group's adjusted \eqn{p} values arranged in side-by-side
  columns, thus forming a matrix with the number of columns equal to the
  number of \code{by} groups. Then we use the \code{cross.adjust} method to
  further adjust the adjusted \eqn{p} values in each row of this matrix. Note
  that an \emph{overall} Bonferroni (or Sidak) adjustment is obtainable by
  specifying \emph{both} \code{adjust} and \code{cross.adjust} as
  \code{"bonferroni"} (or \code{"sidak"}). However, less conservative (but
  yet conservative) overall adjustments are available when it is possible to
  use an \dQuote{exact} within-group method (e.g., \code{adjust = "tukey"}
  for pairwise comparisons) and \code{cross.adjust} as a conservative
  adjustment. [\code{cross.adjust} methods other than \code{"none"}, 
  \code{"bonferroni"}, or \code{"sidak"} do not seem advisable, but other 
  \code{p.adjust} methods are available if you can make sense of them.]
}

\section{Tests of significance, nonsuperiority, noninferiority, or equivalence}{

  When \code{delta = 0}, test statistics are the usual tests of significance.
  They are of the form 
  \samp{(estimate - null)/SE}. Notationally: 
  \describe{
    \item{Significance}{\eqn{H_0: \theta = \theta_0}  versus \cr
       \eqn{H_1: \theta < \theta_0} (left-sided), or\cr
      \eqn{H_1: \theta > \theta_0} (right-sided), or\cr
      \eqn{H_1: \theta \ne \theta_0} (two-sided)\cr
      The test statistic is\cr
      \eqn{t = (Q - \theta_0)/SE}\cr 
      where \eqn{Q} is our estimate of \eqn{\theta};
      then left, right, or two-sided \eqn{p} values are produced, 
      depending on \code{side}.}
  }
  When \code{delta} is positive, the test statistic depends on \code{side} as
  follows.
  \describe{
  \item{Left-sided (nonsuperiority)}{\eqn{H_0: \theta \ge \theta_0 + \delta}
    versus \eqn{H_1: \theta < \theta_0 + \delta}\cr 
    \eqn{t = (Q - \theta_0 - \delta)/SE}\cr 
    The \eqn{p} value is the lower-tail probability.}
  \item{Right-sided (noninferiority)}{\eqn{H_0: \theta \le \theta_0 - \delta}
    versus \eqn{H_1: \theta > \theta_0 - \delta}\cr 
    \eqn{t = (Q - \theta_0 + \delta)/SE}\cr
    The \eqn{p} value is the upper-tail probability.}
  \item{Two-sided (equivalence)}{\eqn{H_0: |\theta - \theta_0| \ge \delta}
    versus \eqn{H_1: |\theta - \theta_0| < \delta}\cr
    \eqn{t = (|Q - \theta_0| - \delta)/SE}\cr
    The \eqn{p} value is the \emph{lower}-tail probability.\cr
    Note that \eqn{t} is the maximum of \eqn{t_{nonsup}} and \eqn{-t_{noninf}}. 
    This is equivalent to choosing the less 
    significant result in the two-one-sided-test (TOST) procedure.}
  } %%%%%%%%%%%% end \describe{}
}

\section{Non-estimable cases}{

  When the model is rank-deficient, each row \code{x} of \code{object}'s
  \code{linfct} slot is checked for estimability. If \code{sum(x*bhat)}
  is found to be non-estimable, then the string \code{NonEst} is displayed for the
  estimate, and associated statistics are set to \code{NA}. 
  The estimability check is performed
  using the orthonormal basis \code{N} in the \code{nbasis} slot for the null
  space of the rows of the model matrix. Estimability fails when
  \eqn{||Nx||^2 / ||x||^2} exceeds \code{tol}, which by default is
  \code{1e-8}. You may change it via \code{\link{emm_options}} by setting
  \code{estble.tol} to the desired value.
  
  See the warning above that non-estimable cases are still included when
  determining the family size for \emph{P}-value adjustments.
}

\section{Warning about potential misuse of P values}{

  Some in the statistical and scientific community argue that
  the term \dQuote{statistical significance} should be completely abandoned, and
  that criteria such as \dQuote{p < 0.05} never be used to assess the
  importance of an effect. These practices can be too misleading and are prone to abuse.
  See \href{../doc/basics.html#pvalues}{the \dQuote{basics} vignette} for more
  discussion.
}

\examples{
warp.lm <- lm(breaks ~ wool * tension, data = warpbreaks)
warp.emm <- emmeans(warp.lm, ~ tension | wool)
warp.emm    # implicitly runs 'summary'

confint(warp.emm, by = NULL, level = .90)

# --------------------------------------------------------------
pigs.lm <- lm(log(conc) ~ source + factor(percent), data = pigs)
pigs.emm <- emmeans(pigs.lm, "percent", type = "response")
summary(pigs.emm)    # (inherits type = "response")
summary(pigs.emm, calc = c(n = ".wgt."))  # Show sample size

# For which percents is EMM non-inferior to 35, based on a 10\% threshold?
# Note the test is done on the log scale even though we have type = "response"
test(pigs.emm, null = log(35), delta = log(1.10), side = ">")

con <- contrast(pigs.emm, "consec")
test(con)

test(con, joint = TRUE)

# default Scheffe adjustment - rank = 3
summary(con, infer = c(TRUE, TRUE), adjust = "scheffe")

# Consider as some of many possible contrasts among the six cell means
summary(con, infer = c(TRUE, TRUE), adjust = "scheffe", scheffe.rank = 5)

# Show estimates to more digits
print(test(con), digits = 7)

# --------------------------------------------------------------
# Cross-adjusting P values
prs <- pairs(warp.emm)   # pairwise comparisons of tension, by wool
test(prs, adjust = "tukey", cross.adjust = "bonferroni")

# Same comparisons taken as one big family (more conservative)
test(prs, adjust = "bonferroni", by = NULL)

}
\seealso{
\code{\link{hpd.summary}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emmeans.R, R/emm-list.R
\name{as.list.emmGrid}
\alias{as.list.emmGrid}
\alias{as.emm_list}
\alias{as.emmGrid}
\title{Convert to and from \code{emmGrid} objects}
\usage{
\method{as.list}{emmGrid}(x, model.info.slot = FALSE, ...)

as.emm_list(object, ...)

as.emmGrid(object, ...)
}
\arguments{
\item{x}{An \code{emmGrid} object}

\item{model.info.slot}{Logical value: Include the \code{model.info} slot?
Set this to \code{TRUE} if you want to preserve the original call and 
information needed by the \code{submodel} option.
If \code{FALSE}, only the nesting information (if any) is saved}

\item{...}{In \code{as.emmGrid}, additional arguments passed to 
\code{\link{update.emmGrid}} before returning the object. This
argument is ignored in \code{as.list.emmGrid}}

\item{object}{Object to be converted to class \code{emmGrid}. It may
be a \code{list} returned by \code{as.list.emmGrid}, or a \code{ref.grid}
or \code{lsmobj} object created by \pkg{emmeans}'s predecessor, the 
\pkg{lsmeans} package. An error is thrown if \code{object} cannot
be converted.}
}
\value{
\code{as.list.emmGrid} returns an object of class \code{list}.

\code{as.emm_list} returns an object of class \code{emm_list}.

\code{as.emmGrid} returns an object of class \code{emmGrid}. 
    However, in fact, both \code{as.emmGrid} and \code{as.emm_list} check for an
    attribute in \code{object} to decide whether to return an \code{emmGrid} 
    or \code{emm_list)} object.
}
\description{
These are useful utility functions for creating a compact version of an
\code{emmGrid} object that may be saved and later reconstructed, or for
converting old \code{ref.grid} or \code{lsmobj} objects into \code{emmGrid}
objects.
}
\details{
An \code{emmGrid} object is an S4 object, and as such cannot be saved in a
text format or saved without a lot of overhead. By using \code{as.list},
the essential parts of the object are converted to a list format that can be
easily and compactly saved for use, say, in another session or by another user.
Providing this list as the arguments for \code{\link{emmobj}} allows the user 
to restore a working \code{emmGrid} object.
}
\examples{
pigs.lm <- lm(log(conc) ~ source + factor(percent), data = pigs)
pigs.sav <- as.list(ref_grid(pigs.lm))

pigs.anew <- as.emmGrid(pigs.sav)
emmeans(pigs.anew, "source")
}
\seealso{
\code{\link{emmobj}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rbind.R, R/emm-list.R, R/nested.R
\name{rbind.emmGrid}
\alias{rbind.emmGrid}
\alias{+.emmGrid}
\alias{[.emmGrid}
\alias{head.emmGrid}
\alias{tail.emmGrid}
\alias{subset.emmGrid}
\alias{rbind.emm_list}
\alias{rbind.summary_emm}
\alias{force_regular}
\title{Combine or subset \code{emmGrid} objects}
\usage{
\method{rbind}{emmGrid}(..., deparse.level = 1, adjust = "bonferroni")

\method{+}{emmGrid}(e1, e2)

\method{[}{emmGrid}(x, i, adjust, drop.levels = TRUE, ...)

\method{head}{emmGrid}(x, n = 6, ...)

\method{tail}{emmGrid}(x, n = 6, ...)

\method{subset}{emmGrid}(x, subset, ...)

\method{rbind}{emm_list}(..., which, adjust = "bonferroni")

\method{rbind}{summary_emm}(..., which)

force_regular(object)
}
\arguments{
\item{...}{In \code{rbind}, object(s) of class \code{emmGrid} or \code{summary_emm}.
In others, additional arguments passed to other methods}

\item{deparse.level}{(required but not used)}

\item{adjust}{Character value passed to \code{\link{update.emmGrid}}}

\item{e1, e2, x, object}{Objects of class \code{emmGrid}}

\item{i}{Integer vector of indexes}

\item{drop.levels}{Logical value. If \code{TRUE}, the \code{"levels"} slot in
the returned object is updated to hold only the predictor levels that actually occur}

\item{n}{integer number of entries to include (or exclude if negative)}

\item{subset}{logical expression indicating which rows of the grid to keep}

\item{which}{Integer vector of subset of elements to use;
if missing, we use all elements.}
}
\value{
A revised object of class \code{emmGrid}

The result of \code{e1 + e2} is the same as \code{rbind(e1, e2)}

The \code{rbind} method for \code{emm_list} objects simply combines 
the \code{emmGrid} objects comprising the first element of \code{...}.
Note that the returned object is not yet summarized, so any \code{adjust}
parameters apply to the combined \code{emmGrid}.

The \code{rbind} method for \code{summary_emm} objects (or a list thereof)
returns a single \code{summary_emm} object. This combined object
\emph{preserves} any adjusted P values or confidence limits in the
original summaries, since those quantities have already been computed.

\code{force_regular} adds extra (invisible) rows to an \code{emmGrid} object
  to make it a regular grid (all combinations of factors). This regular structure is 
  needed by \code{emmeans}. An object can become irregular by, for example,
  subsetting rows, or by obtaining contrasts of a nested structure.
}
\description{
These functions provide methods for \code{\link[base:cbind]{rbind}} and
\code{\link[base:Extract]{[}} that may be used to combine \code{emmGrid} objects
together, or to extract a subset of cases. The primary reason for 
doing this would be to obtain multiplicity-adjusted results for smaller
or larger families of tests or confidence intervals.
}
\note{
\code{rbind} throws an error if there are incompatibilities in
  the objects' coefficients, covariance structures, etc. But they 
  are allowed to have different factors; a missing level \code{'.'}
  is added to factors as needed.

These functions generally reset \code{by.vars} to \code{NULL};
so if you want to keep any \dQuote{by} variables, you should follow-up
with \code{\link{update.emmGrid}}.
}
\examples{
warp.lm <- lm(breaks ~ wool * tension, data = warpbreaks)
warp.rg <- ref_grid(warp.lm)

# Do all pairwise comparisons within rows or within columns, 
# all considered as one faily of tests:
w.t <- pairs(emmeans(warp.rg, ~ wool | tension))
t.w <- pairs(emmeans(warp.rg, ~ tension | wool))
rbind(w.t, t.w, adjust = "mvt")
update(w.t + t.w, adjust = "fdr")  ## same as above except for adjustment

# Show only 3 of the 6 cases
summary(warp.rg[c(2, 4, 5)])

# After-the-fact 'at' specification
subset(warp.rg, wool == "A")  ## or warp.rg |> subset(wool == "A")


### Working with 'emm_list' objects
mod <- lm(conc ~ source + factor(percent), data = pigs)
all <- emmeans(mod, list(src = pairwise ~ source, pct = consec ~ percent))
rbind(all, which = c(2, 4), adjust = "mvt")

### Irregular object
tmp <- warp.rg[-1]
## emmeans(tmp, "tension")   # will fail because tmp is irregular
emmeans(force_regular(tmp), "tension")   # will show some results
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emtrends.R
\name{emtrends}
\alias{emtrends}
\title{Estimated marginal means of linear trends}
\usage{
emtrends(object, specs, var, delta.var = 0.001 * rng, max.degree = 1, ...)
}
\arguments{
\item{object}{A supported model object (\emph{not} a reference grid)}

\item{specs}{Specifications for what marginal trends are desired -- as in
\code{\link{emmeans}}. If \code{specs} is missing or \code{NULL},
\code{emmeans} is not run and the reference grid for specified trends
is returned.}

\item{var}{Character value giving the name of a variable with respect to 
which a difference quotient of the linear predictors is computed. In order
for this to be useful, \code{var} should be a numeric predictor that
interacts with at least one factor in \code{specs}. Then instead of
computing EMMs, we compute and compare the slopes of the \code{var} trend
over levels of the specified other predictor(s). As in EMMs, marginal
averages are computed for the predictors in \code{specs} and \code{by}.
See also the \dQuote{Generalizations} section below.}

\item{delta.var}{The value of \emph{h} to use in forming the difference
quotient \eqn{(f(x+h) - f(x))/h}. Changing it (especially changing its
sign) may be necessary to avoid numerical problems such as logs of negative
numbers. The default value is 1/1000 of the range of \code{var} over the
dataset.}

\item{max.degree}{Integer value. The maximum degree of trends to compute (this
is capped at 5). If greater than 1, an additional factor \code{degree} is
added to the grid, with corresponding numerical derivatives of orders
\code{1, 2, ..., max.degree} as the estimates.}

\item{...}{Additional arguments passed to \code{\link{ref_grid}} or 
\code{\link{emmeans}} as appropriate. See Details.}
}
\value{
An \code{emmGrid} or \code{emm_list} object, according to \code{specs}.
See \code{\link{emmeans}} for more details on when a list is returned.
}
\description{
The \code{emtrends} function is useful when a fitted model involves a
numerical predictor \eqn{x}  interacting with another predictor \code{a}
(typically a factor). Such models specify that \eqn{x} has a different trend
depending on \eqn{a}; thus, it may be of interest to estimate and compare
those trends. Analogous to the \code{\link{emmeans}} setting, we construct a
reference grid of these predicted trends, and then possibly average them over
some of the predictors in the grid.
}
\details{
The function works by constructing reference grids for \code{object} with 
various values of \code{var}, and then calculating difference quotients of predictions
from those reference grids. Finally, \code{\link{emmeans}} is called with
the given \code{specs}, thus computing marginal averages as needed of
the difference quotients. Any \code{...} arguments are passed to the
\code{ref_grid} and \code{\link{emmeans}}; examples of such optional
arguments include optional arguments (often \code{mode}) that apply to
specific models; \code{ref_grid} options such as \code{data}, \code{at},
\code{cov.reduce}, \code{mult.names}, \code{nesting}, or \code{transform};
and \code{emmeans} options such as \code{weights} (but please avoid
\code{trend} or \code{offset}.
}
\note{
In earlier versions of \code{emtrends}, the first argument was named
\code{model} rather than \code{object}. (The name was changed because of
potential mis-matching with a \code{mode} argument, which is an option for
several types of models.) For backward compatibility, \code{model} still works
\emph{provided all arguments are named}.

It is important to understand that trends computed by \code{emtrends} are
\emph{not} equivalent to polynomial contrasts in a parallel model where
\code{var} is regarded as a factor. That is because the model \code{object}
here is assumed to fit a smooth function of \code{var}, and the estimated
trends reflect \emph{local} behavior at particular value(s) of \code{var};
whereas when \code{var} is modeled as a factor and polynomial contrasts are
computed, those contrasts represent the \emph{global} pattern of changes over
\emph{all} levels of \code{var}. 

See the \code{pigs.poly} and \code{pigs.fact} examples below for an
illustration. The linear and quadratic trends depend on the value of 
\code{percent}, but the cubic trend is constant (because that is true of
a cubic polynomial, which is the underlying model). The cubic contrast
in the factorial model has the same P value as for the cubic trend,
again because the cubic trend is the same everywhere.
}
\section{Generalizations}{

Instead of a single predictor, the user may specify some monotone function of
one variable, e.g., \code{var = "log(dose)"}. If so, the chain rule is
applied. Note that, in this example, if \code{object} contains
\code{log(dose)} as a predictor, we will be comparing the slopes estimated by
that model, whereas specifying \code{var = "dose"} would perform a
transformation of those slopes, making the predicted trends vary depending on
\code{dose}.
}

\examples{
fiber.lm <- lm(strength ~ diameter*machine, data=fiber)
# Obtain slopes for each machine ...
( fiber.emt <- emtrends(fiber.lm, "machine", var = "diameter") )
# ... and pairwise comparisons thereof
pairs(fiber.emt)

# Suppose we want trends relative to sqrt(diameter)...
emtrends(fiber.lm, ~ machine | diameter, var = "sqrt(diameter)", 
         at = list(diameter = c(20, 30)))

# Obtaining a reference grid
mtcars.lm <- lm(mpg ~ poly(disp, degree = 2) * (factor(cyl) + factor(am)), data = mtcars)

# Center trends at mean disp for each no. of cylinders
mtcTrends.rg <- emtrends(mtcars.lm, var = "disp", 
                          cov.reduce = disp ~ factor(cyl))
summary(mtcTrends.rg)  # estimated trends at grid nodes
emmeans(mtcTrends.rg, "am", weights = "prop")


### Higher-degree trends ...

pigs.poly <- lm(conc ~ poly(percent, degree = 3), data = pigs)
emt <- emtrends(pigs.poly, ~ degree | percent, "percent", max.degree = 3,
                at = list(percent = c(9, 13.5, 18)))
       # note: 'degree' is an extra factor created by 'emtrends'
       
summary(emt, infer = c(TRUE, TRUE))

# Compare above results with poly contrasts when 'percent' is modeled as a factor ...
pigs.fact <- lm(conc ~ factor(percent), data = pigs)
emm <- emmeans(pigs.fact, "percent")

contrast(emm, "poly")
# Some P values are comparable, some aren't! See Note in documentation
}
\seealso{
\code{\link{emmeans}}, \code{\link{ref_grid}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/models.R
\name{models}
\alias{models}
\title{Models supported in \pkg{emmeans}}
\description{
Documentation for models has been moved to a vignette. To access it,
use \href{../doc/models.html}{\code{vignette("models", "emmeans")}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\name{untidy}
\alias{untidy}
\title{Dare to be un-"tidy"!}
\description{
Users who use \pkg{emmeans} functions as part of a pipeline -- or post-process 
those results in some other way -- are likely missing some important information.
}
\details{
Your best bet is to display the actual results without any post-processing.
That's because \code{emmeans} and its relatives have their own \code{summary} 
and \code{print} methods that display annotations that may be helpful in
explaining what you have. If you just pipe the results into the next step,
those annotations are stripped away and you never see them. Statistical
analysis is not just a workflow; it is a discipline that involves care in
interpreting intermediate results, and thinking before moving on.
}
\examples{
neur.glm <- glm(Pain ~ Treatment + Sex + Age, family = binomial(),
            data = neuralgia)
            
### The actual results with annotations (e.g. ests are on logit scale):
emmeans(neur.glm, "Treatment")

### Post-processed results lose the annotations
if(requireNamespace("tibble")) {
    emmeans(neur.glm, "Treatment") |> tibble::as_tibble()
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/factors.R
\name{comb_facs}
\alias{comb_facs}
\alias{split_fac}
\alias{add_grouping}
\alias{add_submodels}
\alias{permute_levels}
\title{Manipulate factors in a reference grid}
\usage{
comb_facs(object, facs, newname = paste(facs, collapse = "."),
  drop = FALSE, ...)

split_fac(object, fac, newfacs, ...)

add_grouping(object, newname, refname, newlevs, ...)

add_submodels(object, ..., newname = "model")

permute_levels(object, fac, pos)
}
\arguments{
\item{object}{An object of class \code{emmGrid}}

\item{facs}{Character vector. The names of the factors to combine}

\item{newname}{Character name of grouping factor to add (different from any
existing factor in the grid)}

\item{drop}{Logical value. If \code{TRUE}, any levels of the new factor that 
are dropped if all occurrences in the newly reconstructed object have 
weight zero. If \code{FALSE}, all levels are retained. 
(This argument is ignored if there is no \code{.wgt.} column
in \code{object@grid}.)}

\item{...}{arguments passed to other methods}

\item{fac}{The name of a factor that is part of the grid in \code{object}}

\item{newfacs}{A named list with the names of new factors
and their levels. The names must not already exist in the object,
and the product of the lengths of the levels must equal the number
of levels of \code{fac}.}

\item{refname}{Character name(s) of the reference factor(s)}

\item{newlevs}{Character vector or factor of the same length as that of the (combined) levels for 
\code{refname}. The grouping factor \code{newname} will have the unique
values of \code{newlevs} as its levels. The order of levels in \code{newlevs}
is the same as the order of the level combinations produced by 
\code{\link{expand.grid}} applied to the levels of \code{refname} -- that is, the
first factor's levels change the fastest and the last one's vary the slowest.}

\item{pos}{Integer vector consisting of some permutation of the sequence
\code{1:k}, where \code{k} is the number of levels of \code{fac}.
This determines which position each level of \code{fac} will occupy
after the levels are permuted; thus, if the
levels of \code{fac} are \code{A,B,C,D}, and \code{pos = c(3,1,2,4)}, 
then the permuted levels will be \code{B,C,A,D}.}
}
\value{
A modified object of class \code{emmGrid}
}
\description{
These functions manipulate the levels of factors comprising a reference
grid by combining factor levels, splitting a factor's levels into 
combinations of newly-defined factors, creating a grouping factor in which 
factor(s) levels are nested, or permuting the order of levels of a factor
}
\section{The \code{comb_facs} function}{

\code{comb_facs} combines the levels of factors into a single factor
in the reference grid (similar to \code{\link{interaction}}). This new factor
replaces the factors that comprise it.

\emph{Additional note:}
The choice of whether to drop levels or not can make a profound difference.
If the goal is to combine factors for use in \code{joint_tests}, we advise \emph{against} 
\code{drop = TRUE} because that might change the weights used in deriving marginal means.
If combining factors in a nested structure, dropping unused cases can considerably reduce 
the storage required.
}

\section{The \code{split_fac} function}{

The levels in \code{newfacs} are expanded via \code{\link{expand.grid}} into
combinations of levels, and the factor \code{fac} is replaced by those 
factor combinations. Unlike \code{add_grouping}, this creates a crossed, 
rather than a nested structure. Note that the order of factor combinations
is systematic with the levels of first factor in \code{newfacs} varying 
the fastest; and those factor combinations are assigned respectively
to the levels of \code{fac} as displayed in \code{str(object)}.
}

\section{The \code{add_grouping} function}{

This function adds a grouping factor to an existing reference grid or other 
\code{emmGrid} object, such that the levels of one or more existing factors (call them the
reference factors) are mapped to a smaller number of levels of the new
grouping factor. The reference factors are then nested in a 
new grouping factor named \code{newname}, and a new nesting structure
\code{refname \%in\% newname}.
This facilitates obtaining marginal means of the grouping factor, and 
contrasts thereof.


\emph{Additional notes:} By default, the levels of \code{newname} will be ordered
  alphabetically. To dictate a different ordering of levels, supply 
  \code{newlevs} as a \code{factor} having its levels in the desired order.
  
When \code{refname} specifies more than one factor, this can
  fundamentally (and permanently) change what is meant by the levels of those
  individual factors. For instance, in the \code{gwrg} example below, there
  are two levels of \code{wool} nested in each \code{prod}; and that implies
  that we now regard these as four different kinds of wool. Similarly, there
  are five different tensions (L, M, H in prod 1, and L, M in prod 2).
}

\section{The \code{add_submodels} function}{

This function updates \code{object} with a named list of submodels specified in 
\code{...}. These are \code{rbind}ed together and the corresponding rows
for each submodel are assigned a factor named \code{newname} with levels 
equal to the names in \code{...}. This facilitates comparing estimates obtained
from different submodels. For this to work, the underlying model object must be of
a class supported by the \code{submodel} argument of \code{\link{update.emmGrid}}.
}

\section{The \code{permute_levels} function}{

This function permutes the levels of \code{fac}. The returned object
has the same factors, same \code{by} variables, but with the levels
of \code{fac} permuted. 
The order of the columns in \code{object@grid} may be altered.

NOTE: \code{fac} must not be nested in another factor. \code{permute_levels} 
throws an error when \code{fac} is nested.

NOTE: Permuting the levels of a numeric predictor is tricky. For example,
if you want to display the new ordering of levels in \code{emmip()},
you must add the arguments \code{style = "factor"} and \code{nesting.order = TRUE}.
}

\examples{
mtcars.lm <- lm(mpg ~ factor(vs)+factor(cyl)*factor(gear), data = mtcars)
(v.c.g <- ref_grid(mtcars.lm))
(v.cg <- comb_facs(v.c.g, c("cyl", "gear")))
  
# One use is obtaining a single test for the joint contributions of two factors:
joint_tests(v.c.g)

joint_tests(v.cg)

# undo the 'comb_facs' operation:
split_fac(v.cg, "cyl.gear", list(cyl = c(4, 6, 8), gear = 3:5))

IS.glm <- glm(count ~ spray, data = InsectSprays, family = poisson)
IS.emm <- emmeans(IS.glm, "spray")
IS.new <- split_fac(IS.emm, "spray", list(A = 1:2, B = c("low", "med", "hi")))
str(IS.new)

fiber.lm <- lm(strength ~ diameter + machine, data = fiber)
( frg <- ref_grid(fiber.lm) )

# Suppose the machines are two different brands
brands <- factor(c("FiberPro", "FiberPro", "Acme"), levels = c("FiberPro", "Acme"))
( gfrg <- add_grouping(frg, "brand", "machine", brands) )

emmeans(gfrg, "machine")

emmeans(gfrg, "brand")

### More than one reference factor
warp.lm <- lm(breaks ~ wool * tension, data = warpbreaks)
gwrg <- add_grouping(ref_grid(warp.lm), 
    "prod",  c("tension", "wool"),  c(2, 1, 1,  1, 2, 1))
        # level combinations:         LA MA HA  LB MB HB

emmeans(gwrg, ~ wool * tension)   # some NAs due to impossible combinations

emmeans(gwrg, "prod")

## Using 'add_submodels' to compare adjusted and unadjusted means
fibint.lm <- lm(strength ~ machine * diameter, data = fiber)
fibsub <- add_submodels(emmeans(fibint.lm, "machine"), 
    full = ~ ., additive = ~ . - machine:diameter, unadj = ~ machine)
emmeans(fibsub, pairwise ~ model | machine, adjust = "none")

# Permuting factor levels...
str(v.c.g)
str(permute_levels(v.c.g, "cyl", c(2,3,1)))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MCMC-support.R
\name{hpd.summary}
\alias{hpd.summary}
\title{Summarize an emmGrid from a Bayesian model}
\usage{
hpd.summary(object, prob, by, type, point.est = median,
  ci.method = get_emm_option("post.ci.method"), delta,
  bias.adjust = get_emm_option("back.bias.adj"), sigma, ...)
}
\arguments{
\item{object}{an \code{emmGrid} object having a non-missing \code{post.beta} slot}

\item{prob}{numeric probability content for HPD intervals (note: when not specified,
the current \code{level} option is used; see \code{\link{emm_options}})}

\item{by}{factors to use as \code{by} variables}

\item{type}{prediction type as in \code{\link{summary.emmGrid}}}

\item{point.est}{function to use to compute the point estimates from the 
posterior sample for each grid point}

\item{ci.method}{character value matching \code{"HPD"} (default) or \code{"quantile"}
(but actually not case-sensitive).
The default is to use HPD intervals (which are the shortest possible intervals). 
Alternatively, choosing \code{"quantile"} uses the quantiles of the posterior
having equal tail probabilities.}

\item{delta}{Numeric equivalence threshold (on the linear predictor scale 
regardless of \code{type}).
See the section below on equivalence testing.}

\item{bias.adjust}{Logical value for whether to adjust for bias in
back-transforming (\code{type = "response"}). This requires a value of 
\code{sigma} to exist in the object or be specified.}

\item{sigma}{Error SD assumed for bias correction (when 
\code{type = "response"}. If not specified,
\code{object@misc$sigma} is used, and a warning if it is not found or invalid.
\emph{Note:} \code{sigma} may be a vector, as long as it conforms to the 
number of observations in the posterior sample.}

\item{...}{required but not used}
}
\value{
an object of class \code{summary_emm}
}
\description{
This function computes point estimates and HPD (or quantile) intervals for each
factor combination in \code{object@emmGrid}. While this function
may be called independently, it is called automatically by the S3 method
\code{\link{summary.emmGrid}} when the object is based on a Bayesian model.
(Note: the \code{level} argument, or its default, is passed as \code{prob}).
}
\note{
HPD intervals require the \pkg{coda} package to be installed on your system;
otherwise an error is thrown. (So one way to sidestep that error is to specify 
\code{ci.method = "quantile"}).
}
\section{Equivalence testing note}{

  If \code{delta} is positive, two columns labeled \code{p.equiv} and
  \code{odds.eq} are appended to the summary. \code{p.equiv} is the fraction
  of posterior estimates having absolute values less than \code{delta}. The
  \code{odds.eq} column is just \code{p.equiv} converted to an odds ratio; so
  it is the posterior odds of equivalence.
  
  A high value of \code{p.equiv} is evidence
  in favor of equivalence. It can be used to obtain something equivalent
  (in spirit) to the frequentist Schuirmann (TOST) procedure, whereby we would
  conclude equivalence at significance level \eqn{\alpha} if the \eqn{(1 - 2\alpha)}
  confidence interval falls entirely in the interval \eqn{[-\delta, \delta]}.
  Similarly in the Bayesian context, an equally strong argument for
  equivalence is obtained if \code{p.equiv} exceeds \eqn{1 - 2\alpha}.
  
  A closely related quantity is the ROPE (region of practical equivalence),
  obtainable via \code{bayestestR::rope(object, range = c(-delta, delta))}.
  Its value is approximately \code{100 * p.equiv / 0.95} if the default
  \code{ci = 0.95} is used. See also \pkg{bayestestR}'s 
  \href{https://github.com/easystats/bayestestR/issues/567}{issue #567}.
  
  Finally, a Bayes factor for equivalence is obtainable by dividing 
  \code{odds.eq} by the prior odds of equivalence, assessed or elicited separately.
}

\examples{
if(require("coda")) 
    emm_example("hpd.summary-coda")
    # Use emm_example("hpd.summary-coda", list = TRUE) # to see just the code

}
\seealso{
summary.emmGrid
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{feedlot}
\alias{feedlot}
\title{Feedlot data}
\format{
A data frame with 67 observations and 4 variables:
\describe{
  \item{\code{herd}}{a factor with levels \code{9} \code{16} \code{3}
    \code{32} \code{24} \code{31} \code{19} \code{36} \code{34} \code{35}
    \code{33}, designating the herd that a feeder calf came from.}
  \item{\code{diet}}{a factor with levels \code{Low} \code{Medium}
    \code{High}: the energy level of the diet given the animal.}
  \item{\code{swt}}{a numeric vector: the weight of the animal at slaughter.}
  \item{\code{ewt}}{a numeric vector: the weight of the animal at entry to the feedlot.}
}
}
\source{
Urquhart NS (1982) Adjustment in covariates when one factor affects
  the covariate. \emph{Biometrics} 38, 651-660.
}
\usage{
feedlot
}
\description{
This is an unbalanced analysis-of-covariance example, where one covariate is
affected by a factor. Feeder calves from various herds enter a feedlot, where
they are fed one of three diets. The weight of the animal at entry is the
covariate, and the weight at slaughter is the response.
}
\details{
The data arise from a Western Regional Research Project conducted at New
Mexico State University. Calves born in 1975 in commercial herds entered a
feedlot as yearlings. Both diets and herds are of interest as factors. The
covariate, \code{ewt}, is thought to be dependent on \code{herd} due to
different genetic backgrounds, breeding history, etc. The levels of
\code{herd} ordered to similarity of genetic background.

Note: There are some empty cells in the cross-classification of 
\code{herd} and \code{diet}.
}
\examples{
feedlot.lm <- lm(swt ~ ewt + herd*diet, data = feedlot)

# Obtain EMMs with a separate reference value of ewt for each 
# herd. This reproduces the last part of Table 2 in the reference
emmeans(feedlot.lm,  ~ diet | herd,  cov.reduce = ewt ~ herd)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pwpp.R
\name{pwpm}
\alias{pwpm}
\title{Pairwise P-value matrix (plus other statistics)}
\usage{
pwpm(emm, by, reverse = FALSE, pvals = TRUE, means = TRUE,
  diffs = TRUE, flip = FALSE, digits, ...)
}
\arguments{
\item{emm}{An \code{emmGrid} object}

\item{by}{Character vector of variable(s) in the grid to condition on. These
will create different matrices, one for each level or level-combination.
If missing, \code{by} is set to \code{emm@misc$by.vars}.
Grid factors not in \code{by} are the \emph{primary} factors:
whose levels or level combinations are compared pairwise.}

\item{reverse}{Logical value passed to \code{\link{pairs.emmGrid}}.
Thus, \code{FALSE} specifies \code{"pairwise"} comparisons 
(earlier vs. later), and \code{TRUE} specifies \code{"revpairwise"}
comparisons (later vs. earlier).}

\item{pvals}{Logical value. If \code{TRUE}, the pairwise differences 
of the EMMs are included in each matrix according to \code{flip}.}

\item{means}{Logical value. If \code{TRUE}, the estimated marginal means
(EMMs) from \code{emm} are included in the matrix diagonal(s).}

\item{diffs}{Logical value. If \code{TRUE}, the pairwise differences 
of the EMMs are included in each matrix according to \code{flip}.}

\item{flip}{Logical value that determines where P values and differences 
are placed. \code{FALSE} places the P values in the upper triangle
and differences in the lower, and \code{TRUE} does just the opposite.}

\item{digits}{Integer. Number of digits to display. If missing,
an optimal number of digits is determined.}

\item{...}{Additional arguments passed to \code{\link{contrast.emmGrid}} and 
\code{\link{summary.emmGrid}}. You should \emph{not} include \code{method}
here, because pairwise comparisons are always used.}
}
\value{
A matrix or `list` of matrices, one for each `by` level.
}
\description{
This function presents results from \code{emmeans} and pairwise comparisons
thereof in a compact way. It displays a matrix (or matrices) of estimates,
pairwise differences, and P values. The user may opt to exclude any of these
via arguments \code{means}, \code{diffs}, and \code{pvals}, respectively.
To control the direction of the pairwise differences, use \code{reverse};
and to control what appears in the upper and lower triangle(s), use \code{flip}.
Optional arguments are passed to \code{contrast.emmGrid} and/or 
\code{summary.emmGrid}, making it possible to control what estimates 
and tests are displayed.
}
\note{
If \code{emm} is the result of a Bayesian analysis, \code{pwpm} is
  based on a frequentist analysis
}
\examples{
warp.lm <- lm(breaks ~ wool * tension, data = warpbreaks)
warp.emm <- emmeans(warp.lm, ~ tension | wool)

pwpm(warp.emm)

# use dot options to specify noninferiority tests
pwpm(warp.emm, by = NULL, side = ">", delta = 5, adjust = "none")
}
\seealso{
A graphical display of essentially the same results is available
  from \code{\link{pwpp}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emmeans-package.R
\name{emmeans-package}
\alias{emmeans-package}
\title{Estimated marginal means (aka Least-squares means)}
\description{
This package provides methods for obtaining estimated marginal means (EMMs, also 
known as least-squares means) for factor combinations in a variety of models.
Supported models include [generalized linear] models, models for counts,
multivariate, multinomial and ordinal responses, survival models, GEEs, and
Bayesian models. For the latter, posterior samples of EMMs are provided.
The package can compute contrasts or linear
combinations of these marginal means with various multiplicity adjustments.
One can also estimate and contrast slopes of trend lines.
Some graphical displays of these results are provided.
}
\section{Overview}{

\describe{
\item{Vignettes}{A number of vignettes are provided to help the user get
acquainted with the \pkg{emmeans} package and see some examples.}

\item{Concept}{Estimated marginal means (see Searle \emph{et al.} 1980 are
popular for summarizing linear models that include factors. For balanced
experimental designs, they are just the marginal means. For unbalanced data,
they in essence estimate the marginal means you \emph{would} have observed
that the data arisen from a balanced experiment. Earlier developments
regarding these techniques were developed in a least-squares context and are
sometimes referred to as \dQuote{least-squares means}. Since its early
development, the concept has expanded far beyond least-squares settings.}

\item{Reference grids}{ The implementation in \pkg{emmeans} relies on our own
concept of a \emph{reference grid}, which is an array of factor and predictor
levels. Predictions are made on this grid, and estimated marginal means (or
EMMs) are defined as averages of these predictions over zero or more
dimensions of the grid. The function \code{\link{ref_grid}} explicitly
creates a reference grid that can subsequently be used to obtain
least-squares means. The object returned by \code{ref_grid} is of class
\code{"emmGrid"}, the same class as is used for estimated marginal means (see
below).

Our reference-grid framework expands slightly upon Searle \emph{et al.}'s
definitions of EMMs, in that it is possible to include multiple levels of
covariates in the grid. }

\item{Models supported}{As is mentioned in the package description, many
types of models are supported by the package. 
See \href{../doc/models.html}{vignette("models", "emmeans")} for full details. 
Some models may require other packages be
installed in order to  access all of the available features.
For models not explicitly supported, it may still be possible to do basic
post hoc analyses of them via the \code{\link{qdrg}} function.}

\item{Estimated marginal means}{
The \code{\link{emmeans}} function computes EMMs given a fitted model (or a
previously constructed \code{emmGrid} object), using a specification indicating
what factors to include. The \code{\link{emtrends}} function creates the same
sort of results for estimating and comparing slopes of fitted lines. Both
return an \code{emmGrid} object.}

\item{Summaries and analysis}{
The \code{\link{summary.emmGrid}}  method may be used to display an \code{emmGrid}
object. Special-purpose summaries are available via \code{\link{confint.emmGrid}} and
\code{\link{test.emmGrid}}, the latter of which can also do a joint test of several
estimates. The user may specify by variables, multiplicity-adjustment
methods, confidence levels, etc., and if a transformation or link function is
involved, may reverse-transform the results to the response scale.}

\item{Contrasts and comparisons}{
The \code{\link{contrast}} method for \code{emmGrid} objects is used to obtain
contrasts among the estimates; several standard contrast families are
available such as deviations from the mean, polynomial contrasts, and
comparisons with one or more controls. Another \code{emmGrid} object is returned,
which can be summarized or further analyzed. For convenience, a \code{pairs.emmGrid}
method is provided for the case of pairwise comparisons. 
}
\item{Graphs}{The \code{\link{plot.emmGrid}} method will display
side-by-side confidence intervals for the estimates, and/or
\dQuote{comparison arrows} whereby the *P* values of pairwise differences
can be observed by how much the arrows overlap. The \code{\link{emmip}} function
displays estimates like an interaction plot, multi-paneled if there are by
variables. These graphics capabilities require the \pkg{lattice} package be
installed.}

\item{MCMC support}{When a model is fitted using MCMC methods, the posterior
chains(s) of parameter estimates are retained and converted into posterior
samples of EMMs or contrasts thereof. These may then be summarized or plotted
like any other MCMC results, using tools in, say \pkg{coda} or
\pkg{bayesplot}.}

\item{\pkg{multcomp} interface}{The \code{\link{as.glht}} function and
\code{glht} method for \code{emmGrid}s provide an interface to the
\code{glht} function in the \pkg{multcomp} package, thus
providing for more exacting simultaneous estimation or testing. The package
also provides an \code{\link{emm}} function that works as an alternative to
\code{mcp} in a call to \code{glht}.
}
} %%% end describe
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emmip.R
\name{emmip}
\alias{emmip}
\alias{emmip.default}
\alias{emmip_ggplot}
\alias{emmip_lattice}
\title{Interaction-style plots for estimated marginal means}
\usage{
emmip(object, formula, ...)

\method{emmip}{default}(object, formula, type, CIs = FALSE, PIs = FALSE,
  style, engine = get_emm_option("graphics.engine"), plotit = TRUE,
  nesting.order = FALSE, abbr.len = c(0, 0), ...)

emmip_ggplot(emms, style = "factor", dodge = 0.2, xlab = labs$xlab,
  ylab = labs$ylab, tlab = labs$tlab, facetlab = "label_both", scale,
  dotarg = list(shape = "circle"), linearg = list(linetype = "solid"),
  CIarg = list(alpha = 0.4), PIarg = list(alpha = 0.25), col = NULL, ...)

emmip_lattice(emms, style = "factor", xlab = labs$xlab, ylab = labs$ylab,
  tlab = labs$tlab, pch = c(1, 2, 6, 7, 9, 10, 15:20), lty = 1,
  col = NULL, ...)
}
\arguments{
\item{object}{An object of class \code{emmGrid}, or a fitted model of a class
supported by the \pkg{emmeans} package}

\item{formula}{Formula of the form 
\code{trace.factors ~ x.factors | by.factors}. The EMMs are
plotted against \code{x.factor} for each level of \code{trace.factors}.
\code{by.factors} is optional, but if present, it determines separate
panels. Each element of this formula may be a single factor in the model,
or a combination of factors using the \code{*} operator.}

\item{...}{Additional arguments passed to \code{\link{emmeans}} (when
\code{object} is not already an \code{emmGrid} object),
\code{predict.emmGrid}, 
\code{emmip_ggplot}, or \code{emmip_lattice}.}

\item{type}{As in \code{\link{predict.emmGrid}}, this determines
whether we want to inverse-transform the predictions
(\code{type = "response"}) or not (any other choice). The default is
\code{"link"}, unless the \code{"predict.type"} option is in force; see
\code{\link{emm_options}}.
In addition, the user may specify \code{type = "scale"} to create a
transformed scale for the vertical axis based on \code{object}'s response 
transformation or link function.}

\item{CIs}{Logical value. If \code{TRUE}, confidence intervals (or HPD intervals
for Bayesian models) are added to the plot 
(works only with \code{engine = "ggplot"}).}

\item{PIs}{Logical value. If \code{TRUE}, prediction intervals are added to the plot 
(works only with \code{engine = "ggplot"}). This is allowed only if the
underlying model family is \code{"gaussian"}.
If both \code{CIs} and
\code{PIs} are \code{TRUE}, the prediction intervals will be somewhat
longer, lighter, and thinner than the confidence intervals. Additional
parameters to \code{\link{predict.emmGrid}} (e.g., \code{sigma}) may be passed via
\code{...}. For Bayesian models, PIs require \code{frequentist = TRUE} and 
a value for \code{sigma}.}

\item{style}{Optional character value. This has an effect only when the
horizontal variable is a single numeric variable. If \code{style} is
unspecified or \code{"numeric"}, the horizontal scale will be numeric and
curves are plotted using lines (and no symbols). With \code{style =
"factor"}, the horizontal variable is treated as the levels of a factor
(equally spaced along the horizontal scale), and curves are plotted using
lines and symbols. When the horizontal variable is character or factor, or
a combination of more than one predictor, \code{"factor"} style is always used.}

\item{engine}{Character value matching \code{"ggplot"} (default), 
\code{"lattice"}, or \code{"none"}. The graphics engine to be used to produce the plot.
These require, respectively, the \pkg{ggplot2} or \pkg{lattice} package to
be installed. Specifying \code{"none"} is equivalent to setting \code{plotit = FALSE}.}

\item{plotit}{Logical value. If \code{TRUE}, a graphical object is returned;
if \code{FALSE}, a data.frame is returned containing all the values
used to construct the plot.}

\item{nesting.order}{Logical value. If \code{TRUE}, factors that are nested
are presented in order according to their nesting factors, even if those nesting
factors are not present in \code{formula}. If \code{FALSE}, only the
variables in \code{formula} are used to order the variables.}

\item{abbr.len}{Integer vector of length 1 or 2 used to specify that factor levels
be abbreviated to the specified lengths, using \code{\link{abbreviate}}. Any values
less than 1 are ignored. \code{abbr.len[1]} applies to a factor plotted along the 
horizontal axis, and \code{abbr.len[2]} (if present) applies to trace factors (shown in a legend)
or by factors (shown in facet labels).}

\item{emms}{A \code{data.frame} created by calling \code{emmip} with
\code{plotit = FALSE}. Certain variables and attributes are expected
to exist in this data frame; see the section detailing the rendering functions.}

\item{dodge}{Numerical amount passed to \code{ggplot2::position_dodge} 
by which points and intervals are offset so they do not collide.}

\item{xlab, ylab, tlab}{Character labels for the horizontal axis, vertical
axis, and traces (the different curves), respectively. The \code{emmip}
function generates these automatically and provides therm via the \code{labs} 
attribute, but the user may override these if desired.}

\item{facetlab}{Labeller for facets (when by variables are in play). Use
\code{"label_value"} to show just the factor levels, or \code{"label_both"}
to show both the factor names and factor levels; \code{"label_context"}
decides which based on how many \code{by} factors there are. See the
documentation for \code{ggplot2::labellers}.}

\item{scale}{If not missing, an object of class \code{scales::trans} specifying
a (usually) nonlinear scaling for the vertical axis. For example, 
\code{scales = scales::log_trans()} specifies a logarithmic scale. For
fine-tuning purposes, additional
arguments to \code{ggplot2::scale_y_continuous} may be included in \code{...} .}

\item{dotarg}{\code{list}
of arguments passed to \code{geom_point} to customize appearance of points.
If this includes \code{shape} and it is of length greater than 1, it is interpreted
as a shape palette instead.}

\item{linearg}{\code{list}
of arguments passed to \code{geom_line} to customize appearance of lines.
If this includes \code{linetype} and it is of length greater than 1,
it is interpreted as a linetype.}

\item{CIarg, PIarg}{\code{list}s
of arguments passed to \code{geom_linerange} to customize appearance of intervals.
(Note: the \code{linetype} aesthetic defaults to \code{"solid"} under the hood)}

\item{col}{With \code{emmip_ggplot}, this adds \code{color = col} (not
\code{colour}) to all of the \code{*arg} lists. This is intended for setting a
common color for everything, such as a black-and-white plot. 
With \code{emmip_lattice}, \code{col} specifies the colors to use
for each group, recycled as needed. If not specified, the default trellis
colors are used.}

\item{pch}{(Lattice only) The plotting characters to use for each group (i.e., levels of
\code{trace.factors}). They are recycled as needed.}

\item{lty}{(Lattice only) The line types to use for each group. Recycled as needed.}
}
\value{
If \code{plotit = FALSE}, a \code{data.frame} (actually, a
  \code{summary_emm} object) with the table of EMMs that would be plotted.
  The variables plotted are named \code{xvar} and \code{yvar}, and the trace
  factor is named \code{tvar}. This data frame has an added \code{"labs"}
  attribute containing the labels \code{xlab}, \code{ylab}, and \code{tlab}
  for these respective variables. The confidence limits are also
  included, renamed \code{LCL} and \code{UCL}.

If \code{plotit = TRUE}, the function
  returns an object of class \code{"ggplot"} or a \code{"trellis"}, depending
  on \code{engine}.
}
\description{
Creates an interaction plot of EMMs based on a fitted model and a simple
formula specification.
}
\note{
Conceptually, this function is equivalent to 
  \code{\link{interaction.plot}} where the summarization function is thought 
  to return the EMMs.
}
\section{Details}{

If \code{object} is a fitted model, \code{\link{emmeans}} is called with an
appropriate specification to obtain estimated marginal means for each
combination of the factors present in \code{formula} (in addition, any 
arguments in \code{\dots} that match \code{at}, \code{trend}, 
\code{cov.reduce}, or \code{fac.reduce} are passed to \code{emmeans}). 
Otherwise, if \code{object} is an \code{emmGrid} object, its first element is 
used, and it must contain one estimate for each combination of the factors
present in \code{formula}.
}

\section{Rendering functions}{

The functions \code{emmip_ggplot} and \code{emmip_lattice}
are called when \code{plotit == TRUE} to render the plots; 
but they may also be called later on an object saved via \code{plotit = FALSE}
(or \code{engine = "none"}). The functions require that \code{emms} contains variables
\code{xvar}, \code{yvar}, and \code{tvar}, and attributes \code{"labs"} and \code{"vars"}.
Confidence intervals are plotted if variables \code{LCL} and \code{UCL} exist;
and prediction intervals are plotted if \code{LPL} and \code{UPL} exist.
Finally, it must contain the variables named in \code{attr(emms, "vars")}.

In \code{emmip_ggplot}, colors, linetypes, and shapes are all assigned to
groups (according to \code{tvar}) unless overridden. So, for example, one may 
have different symbols for each group by simply specifying \code{dotarg = list()}.
}

\examples{
#--- Three-factor example
noise.lm = lm(noise ~ size * type * side, data = auto.noise)

# Separate interaction plots of size by type, for each side
emmip(noise.lm, type ~ size | side)

# Same using the "lattice" engine
emmip(noise.lm, type ~ size | side, engine = "lattice")

# One interaction plot, using combinations of size and side as the x factor
# ... with added confidence intervals and some formatting changes
emmip(noise.lm, type ~ side * size, CIs = TRUE,
    CIarg = list(linewidth = 1.5, alpha = 1, color = "orange"),
    dotarg = list(size = 2, shape = "square", color = "black"))
    
# Same using legacy theme
with_emm_options(gg.theme = 1,
    emmip(noise.lm, type ~ side * size, CIs = TRUE,
        CIarg = list(linewidth = 1.5, alpha = 1, color = "orange"),
        dotarg = list(size = 2, shape = "square", color = "black")))

# Create a black-and-white version of above with different linetypes
# (Let the linetypes and symbols default to the palette)
emmip(noise.lm, type ~ side * size, CIs = TRUE, col = "black",
      linearg = list(), dotarg = list(size = 4), CIarg = list(alpha = 1)) +
    ggplot2::theme_bw()

# One interaction plot using combinations of type and side as the trace factor
emmip(noise.lm, type * side ~ size)

# Individual traces in panels
emmip(noise.lm, ~ size | type * side)

# Example for the 'style' argument
fib.lm = lm(strength ~ machine * sqrt(diameter), data = fiber)
fib.rg = ref_grid(fib.lm, at = list(diameter = c(12, 14, 15, 25, 36)))
emmip(fib.rg, machine ~ diameter)   # curves (because diameter is numeric)
emmip(fib.rg, machine ~ diameter, style = "factor")  # points and lines

# For an example using extra ggplot2 code, see 'vignette("messy-data")',
# in the section on nested models.

### Options with transformations or link functions
neuralgia.glm <- glm(Pain ~ Treatment * Sex + Age, family = binomial(), 
                     data = neuralgia) 

# On link scale:
emmip(neuralgia.glm, Treatment ~ Sex)

# On response scale:
emmip(neuralgia.glm, Treatment ~ Sex, type = "response")

# With transformed axis scale and custom scale divisions
emmip(neuralgia.glm, Treatment ~ Sex, type = "scale",
    breaks = seq(0.10, 0.90, by = 0.10))
}
\seealso{
\code{\link{emmeans}}, \code{\link{interaction.plot}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MCMC-support.R
\name{as.mcmc.emmGrid}
\alias{as.mcmc.emmGrid}
\alias{mcmc-support}
\alias{as.mcmc.emm_list}
\alias{as.mcmc.list.emmGrid}
\alias{as.mcmc.list.emm_list}
\title{Support for MCMC-based estimation}
\usage{
\method{as.mcmc}{emmGrid}(x, names = TRUE, sep.chains = TRUE, likelihood,
  NE.include = FALSE, ...)

\method{as.mcmc}{emm_list}(x, which = 1, ...)

\method{as.mcmc.list}{emmGrid}(x, names = TRUE, ...)

\method{as.mcmc.list}{emm_list}(x, which = 1, ...)
}
\arguments{
\item{x}{An object of class \code{emmGrid}}

\item{names}{Logical scalar or vector specifying whether variable names are
appended to levels in the column labels for the \code{as.mcmc} or
\code{as.mcmc.list} result -- e.g., column names of \code{treat A} and
\code{treat B} versus  just \code{A} and \code{B}. When there is more than
one variable involved, the elements of \code{names} are used cyclically.}

\item{sep.chains}{Logical value. If \code{TRUE}, and there is more than one
MCMC chain available, an \code{\link[coda]{mcmc.list}} object is returned
by \code{as.mcmc}, with separate EMMs posteriors in each chain.}

\item{likelihood}{Character value or function. If given, simulations are made from 
the corresponding posterior predictive distribution. If not given, we obtain
the posterior distribution of the parameters in \code{object}. See Prediction
section below.}

\item{NE.include}{Logical value. If \code{TRUE}, non-estimable columns are
kept but returned as columns of \code{NA} values (this may create errors or
warnings in subsequent analyses using, say, \pkg{coda}). If \code{FALSE},
non-estimable columns are dropped, and a warning is issued. (If all are
non-estimable, an error is thrown.)}

\item{...}{arguments passed to other methods}

\item{which}{item in the \code{emm_list} to use}
}
\value{
An object of class \code{\link[coda]{mcmc}} or \code{\link[coda]{mcmc.list}}.
}
\description{
When a model is fitted using Markov chain Monte Carlo (MCMC) methods, 
its reference grid contains a \code{post.beta} slot. These functions 
transform those posterior samples to posterior samples of EMMs or
related contrasts. They can then be summarized or plotted using,
e.g., functions in the \pkg{coda} package.
}
\section{Details}{

When the object's \code{post.beta} slot is non-trivial, \code{as.mcmc} will
return an \code{\link[coda]{mcmc}} or \code{\link[coda]{mcmc.list}} object
that can be summarized or plotted using methods in the \pkg{coda} package.
In these functions, \code{post.beta} is transformed by post-multiplying it by
\code{t(linfct)}, creating a sample from the posterior distribution of LS
means. In \code{as.mcmc}, if \code{sep.chains} is \code{TRUE} and there is in
fact more than one chain, an \code{mcmc.list} is returned with each chain's
results. The \code{as.mcmc.list} method is guaranteed to return an
\code{mcmc.list}, even if it comprises just one chain.
}

\section{Prediction}{

When \code{likelihood} is specified, it is used to simulate values from the
posterior predictive distribution corresponding to the given likelihood and
the posterior distribution of parameter values. Denote the likelihood 
function as \eqn{f(y|\theta,\phi)}, where \eqn{y} is a response, \eqn{\theta}
is the parameter estimated in \code{object}, and \eqn{\phi} comprises zero or
more additional parameters to be specified. If \code{likelihood} is a 
function, that function should take as its first argument a vector of 
\eqn{\theta} values (each corresponding to one row of \code{object@grid}).
Any \eqn{\phi} values should be specified as additional named function
arguments, and passed to \code{likelihood} via \code{...}. This function should 
simulate values of \eqn{y}.

A few standard likelihoods are available by specifying \code{likelihood} as
a character value. They are:
\describe{
  \item{\code{"normal"}}{The normal distribution with mean \eqn{\theta} and
  standard deviation specified by additional argument \code{sigma}}
  \item{\code{"binomial"}}{The binomial distribution with success probability 
    \eqn{theta}, and number of trials specified by \code{trials}}
  \item{\code{"poisson"}}{The Poisson distribution with mean \eqn{theta} 
    (no additional parameters)}
  \item{\code{"gamma"}}{The gamma distribution with scale parameter \eqn{\theta}
    and shape parameter specified by \code{shape}}
}
}

\examples{
if(requireNamespace("coda")) 
    emm_example("as.mcmc-coda")
    # Use emm_example("as.mcmc-coda", list = TRUE) # to see just the code
    
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emmGrid-methods.R
\name{str.emmGrid}
\alias{str.emmGrid}
\alias{print.emmGrid}
\alias{vcov.emmGrid}
\alias{linfct}
\alias{linfct.default}
\title{Miscellaneous methods for \code{emmGrid} objects}
\usage{
\method{str}{emmGrid}(object, ...)

\method{print}{emmGrid}(x, ..., export = FALSE)

\method{vcov}{emmGrid}(object, ..., sep = get_emm_option("sep"))

linfct(object, ...)

\method{linfct}{default}(object, ...)
}
\arguments{
\item{object}{An \code{emmGrid} object}

\item{...}{(required but not used)}

\item{x}{An \code{emmGrid} object}

\item{export}{Logical value. If \code{FALSE}, the object is printed.
If \code{TRUE}, a list is invisibly returned, which contains character
elements named \code{summary} and \code{annotations} that may be saved
or displayed as the user sees fit. \code{summary} is a character matrix
(or list of such matrices, if a \code{by} variable is in effect).
\code{annotations} is a character vector of the annotations that would
have been printed below the summary or summaries.}

\item{sep}{separator for pasting levels in creating row and column
names for \code{vcov()} results}
}
\value{
The \code{vcov} method returns a symmetric matrix of variances and
  covariances for \code{predict.emmGrid(object, type = "lp")}

The \code{linfct} function and method returns the \code{linfct} slot of \code{object}.
}
\description{
Miscellaneous methods for \code{emmGrid} objects
}
\examples{
warp.lm <- lm(breaks ~ wool * tension, data = warpbreaks)
warp.emm <- emmeans(warp.lm, ~ tension | wool)
vcov(warp.emm) |> zapsmall()

vcov(pairs(warp.emm), sep = "|") |> zapsmall()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contrast.R
\name{contrast}
\alias{contrast}
\alias{contrast.emmGrid}
\alias{pairs.emmGrid}
\alias{coef.emmGrid}
\alias{weights.emmGrid}
\title{Contrasts and linear functions of EMMs}
\usage{
contrast(object, ...)

\method{contrast}{emmGrid}(object, method = "eff", interaction = FALSE, by,
  offset = NULL, scale = NULL, name = "contrast",
  options = get_emm_option("contrast"), type, adjust, simple,
  combine = FALSE, ratios = TRUE, parens, enhance.levels = TRUE, wts,
  ...)

\method{pairs}{emmGrid}(x, reverse = FALSE, ...)

\method{coef}{emmGrid}(object, ...)

\method{weights}{emmGrid}(object, ...)
}
\arguments{
\item{object}{An object of class \code{emmGrid}}

\item{...}{Additional arguments passed to other methods}

\item{method}{Character value giving the root name of a contrast method (e.g.
\code{"pairwise"} -- see \link{emmc-functions}). Alternatively, a function
of the same form, or a named \code{list} of coefficients (for a contrast or
linear function) that must each conform to the number of results in each
\code{by} group. In a multi-factor situation, the factor levels are
combined and treated like a single factor.}

\item{interaction}{Character vector, logical value, or list. If this is specified,
\code{method} is ignored. See the \dQuote{Interaction contrasts} section
below for details.}

\item{by}{Character names of variable(s) to be used for ``by'' groups. The
contrasts or joint tests will be evaluated separately for each combination
of these variables. If \code{object} was created with by groups, those are
used unless overridden. Use \code{by = NULL} to use no by groups at all.}

\item{offset, scale}{Numeric vectors of the same length as each \code{by} group.
The \code{scale} values, if supplied, multiply their respective linear estimates, and
any \code{offset} values are added. Scalar values are also allowed. 
(These arguments are ignored when \code{interaction} is specified.)}

\item{name}{Character name to use to override the default label for contrasts
used in table headings or subsequent contrasts of the returned object.}

\item{options}{If non-\code{NULL}, a named \code{list} of arguments to pass
to \code{\link{update.emmGrid}}, just after the object is constructed.}

\item{type}{Character: prediction type (e.g., \code{"response"}) -- added to
\code{options}}

\item{adjust}{Character: adjustment method (e.g., \code{"bonferroni"}) --
added to \code{options}}

\item{simple}{Character vector or list: Specify the factor(s) \emph{not} in
\code{by}, or a list thereof. See the section below on simple contrasts.}

\item{combine}{Logical value that determines what is returned when
\code{simple} is a list. See the section on simple contrasts.}

\item{ratios}{Logical value determining how log and logit transforms are
handled. These transformations are exceptional cases in that there is a
valid way to back-transform contrasts: differences of logs are logs of
ratios, and differences of logits are odds ratios. If \code{ratios = TRUE}
and summarized with \code{type = "response"}, \code{contrast} results are
back-transformed to ratios whenever we have true contrasts (coefficients
sum to zero). For other transformations, there is no natural way to
back-transform contrasts, so even when summarized with \code{type = "response"},
contrasts are computed and displayed on the linear-predictor scale. Similarly, 
if \code{ratios = FALSE}, log and logit transforms are treated in the same way as
any other transformation.}

\item{parens}{character or \code{NULL}. If a character value, the labels for levels
being contrasted are parenthesized if they match the regular expression in 
\code{parens[1]} (via \code{\link{grep}}). The default is \code{emm_option("parens")}.
Optionally, \code{parens} may contain second and third elements specifying
what to use for left and right parentheses (default \code{"("} and \code{")"}).
Specify \code{parens = NULL} or \code{parens = "a^"} (which won't match anything)
to disable all parenthesization.}

\item{enhance.levels}{character or logical. 
If character, the levels of the named factors that are contrasted are enhanced 
by appending the name of the factor; e.g., if a factor named \code{"trt"} has
levels \code{A} and \code{B}, a \code{trt} comparison is labeled \code{trtA - trtB}.
If \code{enhance.levels} is logical, then if \code{TRUE} (the default), 
only factors with numeric levels are enhanced; and of
course if \code{FALSE}, no levels are enhanced.
The levels of \code{by} variables are not enhanced, and any 
names of factors that don't exist are silently ignored. 
To enhance the labels beyond what is done here, change them
directly via \code{\link[=update.emmGrid]{levels<-}}.}

\item{wts}{The \code{wts} argument for some contrast methods. You should omit
this argument unless you want unequal \code{wts}. Otherwise we recommend
specifying \code{wts = NA} which instructs that \code{wts} be obtained from
\code{object}, \emph{separately} for each \code{by} group. If numerical
\code{wts} are specified, they must
conform to the number of levels in each \code{by} group, and those same weights
are used in each group.}

\item{x}{An \code{emmGrid} object}

\item{reverse}{Logical value - determines whether to use \code{"pairwise"} (if \code{TRUE}) or \code{"revpairwise"} (if \code{FALSE}).}
}
\value{
\code{contrast} and \code{pairs} return an object of class
  \code{emmGrid}. Its grid will correspond to the levels of the contrasts and
  any \code{by} variables. The exception is that an \code{\link{emm_list}}
  object is returned if \code{simple} is a list and \code{combine} is
  \code{FALSE}.

\code{coef} returns a \code{data.frame} containing the "parent" object's grid, 
along with columns named \code{c.1, c.2, ...} containing the contrast coefficients
used to produce the linear functions embodied in the object. \code{coef()} only
returns coefficients if \code{object} is the result of a call to \code{contrast()},
and the parent object is the object that was handed to \code{contrast}. This
is most useful for understanding interaction contrasts.

\code{weights} returns the weights stored for each row of \code{object},
   or a vector of 1s if no weights are saved.
}
\description{
These methods provide for follow-up analyses of \code{emmGrid} objects:
Contrasts, pairwise comparisons, tests, and confidence intervals. They may
also be used to compute arbitrary linear functions of predictions or EMMs.
}
\note{
When \code{object} has a nesting structure (this can be seen via
  \code{str(object)}), then any grouping factors involved are forced into
  service as \code{by} variables, and the contrasts are thus computed
  separately in each nest. This in turn may lead to an irregular grid in the
  returned \code{emmGrid} object, which may not be valid for subsequent
  \code{emmeans} calls.
}
\section{Pairs method}{
 The call \code{pairs(object)} is equivalent to
  \code{contrast(object, method = "pairwise")}; and \code{pairs(object,
  reverse = TRUE)} is the same as \code{contrast(object, method =
  "revpairwise")}.
}

\section{Interaction contrasts}{
 When \code{interaction} is specified,
  interaction contrasts are computed. Specifically contrasts are generated
  for each factor separately, one at a time; and these contrasts are applied
  to the object (the first time around) or to the previous result
  (subsequently). (Any factors specified in \code{by} are skipped.) The final
  result comprises contrasts of contrasts, or, equivalently, products of
  contrasts for the factors involved. Any named elements of \code{interaction}
  are assigned to contrast methods; others are assigned in order of
  appearance in \code{object@levels}. The contrast factors in the resulting 
  \code{emmGrid} object are ordered the same as in \code{interaction}.
  
  \code{interaction} may be a character vector or list of valid contrast
  methods (as documented for the \code{method} argument). If the vector or
  list is shorter than the number needed, it is recycled. Alternatively, if
  the user specifies \code{contrast = TRUE}, the contrast specified in
  \code{method} is used for all factors involved.
}

\section{Simple contrasts}{

  \code{simple} is essentially the complement of \code{by}: When
  \code{simple} is a character vector, \code{by} is set to all the factors in
  the grid \emph{except} those in \code{simple}. If \code{simple} is a list,
  each element is used in turn as \code{simple}, and assembled in an
  \code{"emm_list"}. To generate \emph{all} simple main effects, use
  \code{simple = "each"} (this works unless there actually is a factor named
  \code{"each"}). Note that a non-missing \code{simple} will cause \code{by}
  to be ignored.
  
  Ordinarily, when \code{simple} is a list or \code{"each"}, the return value
  is an \code{\link{emm_list}} object with each entry in correspondence with
  the entries of \code{simple}. However, with \code{combine = TRUE}, the
  elements are all combined into one family of contrasts in a single
  \code{\link[=emmGrid-class]{emmGrid}} object using
  \code{\link{rbind.emmGrid}}.. In that case, the \code{adjust} argument sets
  the adjustment method for the combined set of contrasts.
}

\examples{
warp.lm <- lm(breaks ~ wool*tension, data = warpbreaks)
(warp.emm <- emmeans(warp.lm, ~ tension | wool))

contrast(warp.emm, "poly")    # inherits 'by = "wool"' from warp.emm

### Custom contrast coefs (we already have wool as 'by' thus 3 means to contrast)
contrast(warp.emm, list(mid.vs.ends = c(-1,2,-1)/2, lo.vs.hi = c(1,0,-1)))

pairs(warp.emm)

# Effects (dev from mean) of the 6 factor combs, with enhanced levels:
contrast(warp.emm, "eff", by = NULL, 
    enhance.levels = c("wool", "tension"))  
    
pairs(warp.emm, simple = "wool") # same as pairs(warp.emm, by = "tension")

# Do all "simple" comparisons, combined into one family
pairs(warp.emm, simple = "each", combine = TRUE)

\dontrun{

## Note that the following are NOT the same:
contrast(warp.emm, simple = c("wool", "tension"))
contrast(warp.emm, simple = list("wool", "tension"))
## The first generates contrasts for combinations of wool and tension
##   (same as by = NULL)
## The second generates contrasts for wool by tension, and for 
##   tension by wool, respectively.
}

# An interaction contrast for tension:wool
tw.emm <- contrast(warp.emm, interaction = c(tension = "poly", wool = "consec"), 
                   by = NULL)
tw.emm          # see the estimates
coef(tw.emm)    # see the contrast coefficients

# Use of scale and offset
#   an unusual use of the famous stack-loss data...
mod <- lm(Water.Temp ~ poly(stack.loss, degree = 2), data = stackloss)
(emm <- emmeans(mod, "stack.loss", at = list(stack.loss = 10 * (1:4))))
# Convert results from Celsius to Fahrenheit:
confint(contrast(emm, "identity", scale = 9/5, offset = 32))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ref-grid.R
\name{ref_grid}
\alias{ref_grid}
\title{Create a reference grid from a fitted model}
\usage{
ref_grid(object, at, cov.reduce = mean,
  cov.keep = get_emm_option("cov.keep"), mult.names, mult.levs,
  options = get_emm_option("ref_grid"), data, df, type, regrid, nesting,
  offset, sigma, counterfactuals, nuisance = character(0), non.nuisance,
  wt.nuis = "equal", rg.limit = get_emm_option("rg.limit"), ...)
}
\arguments{
\item{object}{An object produced by a supported model-fitting function, such
as \code{lm}. Many models are supported. See
\href{../doc/models.html}{\code{vignette("models", "emmeans")}}.}

\item{at}{Optional named list of levels for the corresponding variables}

\item{cov.reduce}{A function, logical value, or formula; or a named list of
these. Each covariate \emph{not} specified in \code{cov.keep} or \code{at}
is reduced according to these specifications. See the section below on
\dQuote{Using \code{cov.reduce} and \code{cov.keep}}.}

\item{cov.keep}{Character vector: names of covariates that are \emph{not}
to be reduced; these are treated as factors and used in weighting calculations.
\code{cov.keep} may also include integer value(s), and if so, the maximum
of these is used to set a threshold such that any covariate having no more
than that many unique values is automatically included in \code{cov.keep}.}

\item{mult.names}{Character value: the name(s) to give to the
pseudo-factor(s) whose levels delineate the elements of a multivariate
response. If this is provided, it overrides the default name(s) used for
\code{class(object)} when it has a multivariate response (e.g., the default
is \code{"rep.meas"} for \code{"mlm"} objects).}

\item{mult.levs}{A named list of levels for the dimensions of a multivariate
response. If there is more than one element, the combinations of levels are
used, in \code{\link{expand.grid}} order. The (total) number of levels must
match the number of dimensions. If \code{mult.name} is specified, this
argument is ignored.}

\item{options}{If non-\code{NULL}, a named \code{list} of arguments to pass
to \code{\link{update.emmGrid}}, just after the object is constructed.}

\item{data}{A \code{data.frame} to use to obtain information about the
predictors (e.g. factor levels). If missing, then
\code{\link{recover_data}} is used to attempt to reconstruct the data. See
the note with \code{\link{recover_data}} for an important precaution.}

\item{df}{Numeric value. This is equivalent to specifying \code{options(df =
df)}. See \code{\link{update.emmGrid}}.}

\item{type}{Character value. If provided, this is saved as the
\code{"predict.type"} setting. See \code{\link{update.emmGrid}} and the
section below on prediction types and transformations.}

\item{regrid}{Character, logical, or list. If non-missing, the reference
grid is reconstructed via \code{\link{regrid}} with the argument
\code{transform = regrid}. See the section below on prediction types and
transformations. \emph{Note:} This argument was named \code{transform} in
version 1.7.2 and earlier. For compatibility with old code, \code{transform}
is still accepted if found among \code{...}, 
as long as it doesn't match \code{tran}.}

\item{nesting}{If the model has nested fixed effects, this may be specified
here via a character vector or named \code{list} specifying the nesting
structure. Specifying \code{nesting} overrides any nesting structure that
is automatically detected. See the section below on Recovering or Overriding 
Model Information.}

\item{offset}{Numeric scalar value (if a vector, only the first element is
used). This may be used to add an offset, or override offsets based on the
model. A common usage would be to specify \code{offset = 0} for a Poisson
regression model, so that predictions from the reference grid become rates
relative to the offset that had been specified in the model.}

\item{sigma}{Numeric value to use for subsequent predictions or
back-transformation bias adjustments. If not specified, we use
\code{sigma(object)}, if available, and \code{NULL} otherwise.
Note: This applies only when the family is \code{"gaussian"}; for other families,
\code{sigma} is set to \code{NA} and cannot be overridden.}

\item{counterfactuals}{\code{counterfactuals} specifies character
names of counterfactual factors. If this is non-missing, a reference grid
is created consisting of combinations of counterfactual levels 
and the actual levels of those same factors.
This grid is always converted to the response transformation scale
and averaged over the actual factor levels. See the section below 
on counterfactuals.}

\item{nuisance, non.nuisance, wt.nuis}{If \code{nuisance} is a vector of predictor names,
those predictors are omitted from the reference grid. Instead, the result 
will be as if we had averaged over the levels of those factors, with either 
equal or proportional weights as specified in \code{wt.nuis} (see the 
\code{weights} argument in \code{\link{emmeans}}). The factors in 
\code{nuisance} must not interact with other factors, not even other
nuisance factors. Specifying nuisance factors can save considerable
storage and computation time, and help avoid exceeding the maximum
reference-grid size (\code{get_emm_option("rg.limit")}). 
(\emph{Note:} For certain models where the \code{emm_basis} method returns a
re-gridded parameterization, nuisance factors cannot be used, and an error
is thrown.)}

\item{rg.limit}{Integer limit on the number of reference-grid rows to allow
(checked before any multivariate responses are included).}

\item{...}{Optional arguments passed to \code{\link{summary.emmGrid}},
\code{\link{emm_basis}}, and
\code{\link{recover_data}}, such as \code{params}, \code{vcov.} (see
\bold{Covariance matrix} below), or options such as \code{mode} for
specific model types (see \href{../doc/models.html}{vignette("models",
"emmeans")}).}
}
\value{
An object of the S4 class \code{"emmGrid"} (see
  \code{\link{emmGrid-class}}). These objects encapsulate everything needed
  to do calculations and inferences for estimated marginal means, and contain
  nothing that depends on the model-fitting procedure.
}
\description{
Using a fitted model object, determine a reference grid for which estimated
marginal means are defined. The resulting \code{ref_grid} object encapsulates
all the information needed to calculate EMMs and make inferences on them.
}
\details{
To users, the \code{ref_grid} function itself is important because most of
its arguments are in effect arguments of \code{\link{emmeans}} and related
functions, in that those functions pass their \code{...} arguments to
\code{ref_grid}.

The reference grid consists of combinations of independent variables over
which predictions are made. 
Estimated marginal means are defined as these
predictions, or marginal averages thereof. The grid is determined by first
reconstructing the data used in fitting the model (see
\code{\link{recover_data}}), or by using the \code{data.frame} provided in
\code{data}. 

By \dQuote{independent variables,} we mean (in most cases) the results of
\code{all.vars()} applied to the fixed-effects part of the right-hand side of
the model formula. Any random effects are excluded. Thus, if the model
formula in an \code{lme4::lmer} call is \code{yield ~ fert +
seed*density + log(rain) + (1|block/plot)}, the independent variables are
\code{fert}, \code{seed}, \code{density}, and \code{rain} (not
\code{log(rain)}). In multivariate models, the dimension of the multivariate
response is also considered an independent variable.


The default reference grid is determined by the observed levels
of any factors, the ordered unique values of character-valued predictors, and
the results of \code{cov.reduce} for numeric predictors. These may be
overridden using \code{at}. See also the section below on
recovering/overriding model information.
}
\note{
The system default for \code{cov.keep} causes models
  containing indicator variables to be handled differently than in
  \pkg{emmeans} version 1.4.1 or earlier. To replicate older
  analyses, change the default via 
  \samp{emm_options(cov.keep = character(0))}.

Some earlier versions of \pkg{emmeans} offer a \code{covnest} argument.
  This is now obsolete; if \code{covnest} is specified, it is harmlessly
  ignored. Cases where it was needed are now handled appropriately via the
  code associated with \code{cov.keep}.
}
\section{Using \code{cov.reduce} and \code{cov.keep}}{
 
  The \code{cov.keep} argument was not available in \pkg{emmeans} versions
  1.4.1 and earlier. Any covariates named in this list are treated as if they
  are factors: all the unique levels are kept in the reference grid. The user
  may also specify an integer value, in which case any covariate having no more
  than that number of unique values is implicitly included in \code{cov.keep}.
  The default for \code{cov.keep} is set and retrieved via the 
  \code{\link{emm_options}} framework, and the system default is \code{"2"},
  meaning that covariates having only two unique values are automatically
  treated as two-level factors. See also the Note below on backward compatibility.
  
  There is a subtle distinction between including a covariate in \code{cov.keep}
  and specifying its values manually in \code{at}: Covariates included in 
  \code{cov.keep} are treated as factors for purposes of weighting, while
  specifying levels in \code{at} will not include the covariate in weighting.
  See the \code{mtcars.lm} example below for an illustration.
  
  \code{cov.reduce} may be a function,
  logical value, formula, or a named list of these.
  If a single function, it is applied to each covariate.
  If logical and \code{TRUE}, \code{mean} is used. If logical and
  \code{FALSE}, it is equivalent to including all covariates in
  \code{cov.keep}. Use of \samp{cov.reduce = FALSE} is inadvisable because it
  can result in a huge reference grid; it is far better to use
  \code{cov.keep}.

  If a formula (which must be two-sided), then a model is fitted to that
  formula using \code{\link{lm}}; then in the reference grid, its response
  variable is set to the results of \code{\link{predict}} for that model,
  with the reference grid as \code{newdata}. (This is done \emph{after} the
  reference grid is determined.) A formula is appropriate here when you think
  experimental conditions affect the covariate as well as the response.
  
  To allow for situations where a simple \code{lm()} call as described above won't
  be adequate, a formula of the form \code{ext ~ fcnname} is also supported,
  where the left-hand side may be \code{ext}, \code{extern}, or
  \code{external} (and must \emph{not} be a predictor name) and the
  right-hand side is the name of an existing function. The function is called
  with one argument, a data frame with columns for each variable in the
  reference grid. The function is expected to use that frame as new data to
  be used to obtain predictions for one or more models; and it should return
  a named list or data frame with replacement values for one or more of the
  covariates.

  If \code{cov.reduce} is a named list, then the above criteria are used to
  determine what to do with covariates named in the list. (However, formula
  elements do not need to be named, as those names are determined from the
  formulas' left-hand sides.) Any unresolved covariates are reduced using
  \code{"mean"}.

  Any \code{cov.reduce} of \code{cov.keep} specification for a covariate 
  also named in \code{at} is ignored.
}

\section{Interdependent covariates}{
 Care must be taken when covariate values
  depend on one another. For example, when a polynomial model was fitted
  using predictors \code{x}, \code{x2} (equal to \code{x^2}), and \code{x3}
  (equal to \code{x^3}), the reference grid will by default set \code{x2} and
  \code{x3} to their means, which is inconsistent. The user should instead
  use the \code{at} argument to set these to the square and cube of
  \code{mean(x)}. Better yet, fit the model using a formula involving
  \code{poly(x, 3)} or \code{I(x^2)} and \code{I(x^3)}; then there is only
  \code{x} appearing as a covariate; it will be set to its mean, and the
  model matrix will have the correct corresponding quadratic and cubic terms.
}

\section{Matrix covariates}{
 Support for covariates that appear in the dataset
  as matrices is very limited. If the matrix has but one column, it is
  treated like an ordinary covariate. Otherwise, with more than one column,
  each column is reduced to a single reference value -- the result of
  applying \code{cov.reduce} to each column (averaged together if that
  produces more than one value); you may not specify values in \code{at}; and
  they are not treated as variables in the reference grid, except for
  purposes of obtaining predictions.
}

\section{Recovering or overriding model information}{
 Ability to support a
  particular class of \code{object} depends on the existence of
  \code{recover_data} and \code{emm_basis} methods -- see
  \link{extending-emmeans} for details. The call
  \code{methods("recover_data")} will help identify these.

  \bold{Data.} In certain models, (e.g., results of
  \code{\link[lme4]{glmer.nb}}), it is not possible to identify the original
  dataset. In such cases, we can work around this by setting \code{data}
  equal to the dataset used in fitting the model, or a suitable subset. Only
  the complete cases in \code{data} are used, so it may be necessary to
  exclude some unused variables. Using \code{data} can also help save
  computing, especially when the dataset is large. In any case, \code{data}
  must represent all factor levels used in fitting the model. It
  \emph{cannot} be used as an alternative to \code{at}. (Note: If there is a
  pattern of \code{NAs} that caused one or more factor levels to be excluded
  when fitting the model, then \code{data} should also exclude those levels.)

  \bold{Covariance matrix.} By default, the variance-covariance matrix for
  the fixed effects is obtained from \code{object}, usually via its
  \code{\link{vcov}} method. However, the user may override this via a
  \code{vcov.} argument, specifying a matrix or a function. If a matrix, it
  must be square and of the same dimension and parameter order of the fixed
  effects. If a function, must return a suitable matrix when it is called
  with arguments \code{(object, ...)}. Be careful with possible 
  unintended conflicts with arguments in \code{...}; for example, 
  \code{sandwich::vcovHAC()} has optional arguments \code{adjust} and \code{weights}
  that may be intended for \code{emmeans()} but will also be passed to \code{vcov.()}.

  \bold{Nested factors.} Having a nesting structure affects marginal
  averaging in \code{emmeans} in that it is done separately for each level
  (or combination thereof) of the grouping factors. \code{ref_grid} tries to
  discern which factors are nested in other factors, but it is not always
  obvious, and if it misses some, the user must specify this structure via
  \code{nesting}; or later using \code{\link{update.emmGrid}}. The
  \code{nesting} argument may be a character vector, a named \code{list}, 
  or \code{NULL}.
  If a \code{list}, each name should be the name of a single factor in the
  grid, and its entry a character vector of the name(s) of its grouping
  factor(s). \code{nested} may also be a character value of the form
  \code{"factor1 \%in\% (factor2*factor3)"} (the parentheses are optional).
  If there is more than one such specification, they may be appended
  separated by commas, or as separate elements of a character vector. For
  example, these specifications are equivalent: \code{nesting = list(state =
  "country", city = c("state", "country")}, \code{nesting = "state \%in\%
  country, city \%in\% (state*country)"}, and \code{nesting = c("state \%in\%
  country", "city \%in\% state*country")}.
}

\section{Predictors with subscripts and data-set references}{
 When the fitted
  model contains subscripts or explicit references to data sets, the
  reference grid may optionally be post-processed to simplify the variable
  names, depending on the \code{simplify.names} option (see
  \code{\link{emm_options}}), which by default is \code{TRUE}. For example,
  if the model formula is \code{data1$resp ~ data1$trt + data2[[3]] +
  data2[["cov"]]}, the simplified predictor names (for use, e.g., in the
  \code{specs} for \code{\link{emmeans}}) will be \code{trt},
  \code{data2[[3]]}, and \code{cov}. Numerical subscripts are not simplified;
  nor are variables having simplified names that coincide, such as if
  \code{data2$trt} were also in the model.

  Please note that this simplification is performed \emph{after} the
  reference grid is constructed. Thus, non-simplified names must be used in
  the \code{at} argument (e.g., \code{at = list(`data2["cov"]` = 2:4)}.

  If you don't want names simplified, use \code{emm_options(simplify.names =
  FALSE)}.
}

\section{Prediction types and transformations}{

  Transformations can exist because of a link function in a generalized linear model, 
  or as a response transformation, or even both. In many cases, they are auto-detected,
  for example a model formula of the form \code{sqrt(y) ~ ...}. Even transformations
  containing multiplicative or additive constants, such as \code{2*sqrt(y + pi) ~ ...},
  are auto-detected. A response transformation of \code{y + 1 ~ ...} is \emph{not}
  auto-detected, but \code{I(y + 1) ~ ...} is interpreted as \code{identity(y + 1) ~ ...}.
  A warning is issued if it gets too complicated.
  Complex transformations like the Box-Cox transformation are not auto-detected; but see 
  the help page for \code{\link{make.tran}} for information on some advanced methods.
  
  There is a subtle difference
  between specifying \samp{type = "response"} and \samp{regrid =
  "response"}. While the summary statistics for the grid itself are the same,
  subsequent use in \code{\link{emmeans}} will yield different results if
  there is a response transformation or link function. With \samp{type =
  "response"}, EMMs are computed by averaging together predictions on the
  \emph{linear-predictor} scale and then back-transforming to the response
  scale; while with \samp{regrid = "response"}, the predictions are
  already on the response scale so that the EMMs will be the arithmetic means
  of those response-scale predictions. To add further to the possibilities,
  \emph{geometric} means of the response-scale predictions are obtainable via
  \samp{regrid = "log", type = "response"}. See also the help page for 
  \code{\link{regrid}}.
  
  \emph{Order-of-processing issues:} 
  The \code{regrid} argument, if present, is acted on immediately after the reference 
  grid is constructed, while some of the \code{...} arguments may be used to
  update the object at the very end. Thus, code like
  \code{ref_grid(mod, tran = "sqrt", regrid = "response")} will not work correctly
  if the intention was to specify the response transformation, because the re-grid 
  is done \emph{before} it processes \code{tran = "sqrt"}. To get the intended
  result, do
  \code{regrid(ref_grid(mod, tran = "sqrt"), transform = "response")}.
}

\section{Counterfactuals}{

  If \code{counterfactuals} is specified, the rows of the entire dataset
  become part of the reference grid, and the other reference levels are
  confined to those named in \code{counterfactuals}. In this type of analysis
  (called G-computation), we substitute (or impute) each combination of counterfactual
  levels into the entire dataset. Thus, predictions from this grid are those
  of each observation under each of the counterfactual levels. For this to
  make sense, we require an assumption of exchangeability of these levels.
  
  This grid is always converted to the response scale, as G-computation on
  the linear-predictor scale produces the same results as ordinary weighted EMMs.
  If we have counterfactual factors \code{A, B}, the reference grid also includes
  factors \code{actual_A, actual_B} which are used to track which observations
  originally had the \code{A, B} levels before they were changed by the 
  counterfactuals code. We average the response-scale predictions for each
  combination of actual levels and imputed levels (and multivariate levels, 
  if any). See additional discussion of how \code{\link{emmeans}} handles
  counterfactuals under that documentation.
  
  Currently, counterfactuals are not supported when the reference grid
  requires post-processing (e.g., ordinal models with \code{mode = "prob"}).
  Cases where we have nested factor levels can be complicated if mixed-in with counterfactuals, 
  and we make no guarantees. 
  Note that past implementations included arguments \code{wt.counter} and \code{avg.counter},
  which are now deprecated and are just ignored if specified.
}

\section{Optional side effect}{
 If the \code{save.ref_grid} option is set to
  \code{TRUE} (see \code{\link{emm_options}}),
  The most recent result of \code{ref_grid}, whether
  called directly or indirectly via \code{\link{emmeans}},
  \code{\link{emtrends}}, or some other function that calls one of these, is
  saved in the user's environment as \code{.Last.ref_grid}. This facilitates
  checking what reference grid was used, or reusing the same reference grid
  for further calculations. This automatic saving is disabled by default, but
  may be enabled via \samp{emm_options(save.ref_grid = TRUE)}.
}

\examples{
fiber.lm <- lm(strength ~ machine*diameter, data = fiber)
ref_grid(fiber.lm)

ref_grid(fiber.lm, at = list(diameter = c(15, 25)))

\dontrun{
# We could substitute the sandwich estimator vcovHAC(fiber.lm)
# as follows:
summary(ref_grid(fiber.lm, vcov. = sandwich::vcovHAC))
}

# If we thought that the machines affect the diameters
# (admittedly not plausible in this example), then we should use:
ref_grid(fiber.lm, cov.reduce = diameter ~ machine)

### Model with indicator variables as predictors:
mtcars.lm <- lm(mpg ~ disp + wt + vs * am, data = mtcars)
(rg.default <- ref_grid(mtcars.lm))
(rg.nokeep <- ref_grid(mtcars.lm, cov.keep = character(0)))
(rg.at <- ref_grid(mtcars.lm, at = list(vs = 0:1, am = 0:1)))

# Two of these have the same grid but different weights:
rg.default@grid
rg.at@grid

### Using cov.reduce formulas...
# Above suggests we can vary disp indep. of other factors - unrealistic
rg.alt <- ref_grid(mtcars.lm, at = list(wt = c(2.5, 3, 3.5)),
    cov.reduce = disp ~ vs * wt)
rg.alt@grid

# Alternative to above where we model sqrt(disp)
disp.mod <- lm(sqrt(disp) ~ vs * wt, data = mtcars)
disp.fun <- function(dat)
    list(disp = predict(disp.mod, newdata = dat)^2)
rg.alt2 <- ref_grid(mtcars.lm, at = list(wt = c(2.5, 3, 3.5)),
    cov.reduce = external ~ disp.fun)
rg.alt2@grid


# Multivariate example
MOats.lm = lm(yield ~ Block + Variety, data = MOats)
ref_grid(MOats.lm, mult.names = "nitro")
# Silly illustration of how to use 'mult.levs' to make comb's of two factors
ref_grid(MOats.lm, mult.levs = list(T=LETTERS[1:2], U=letters[1:2]))

# Comparing estimates with and without counterfactuals
neuralgia.glm <- glm(Pain ~ Treatment + Sex + Age + Duration, 
                     family = binomial(), data = neuralgia)
emmeans(neuralgia.glm, "Treatment", type = "response")

emmeans(neuralgia.glm, "Treatment", counterfactuals = "Treatment")


# Using 'params'
require("splines")
my.knots = c(2.5, 3, 3.5)
mod = lm(Sepal.Length ~ Species * ns(Sepal.Width, knots = my.knots), data = iris)
## my.knots is not a predictor, so need to name it in 'params'
ref_grid(mod, params = "my.knots") 

}
\seealso{
Reference grids are of class \code{\link[=emmGrid-class]{emmGrid}},
  and several methods exist for them -- for example
  \code{\link{summary.emmGrid}}. Reference grids are fundamental to
  \code{\link{emmeans}}. Supported models are detailed in
  \href{../doc/models.html}{\code{vignette("models", "emmeans")}}.
  See \code{\link{update.emmGrid}} for details of arguments that can be in 
  \code{options} (or in \code{...}).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glht-support.R
\name{emm}
\alias{emm}
\alias{glht-support}
\alias{glht.emmGrid}
\alias{glht.emmlf}
\alias{modelparm.emmwrap}
\alias{as.glht}
\alias{as.glht.emmGrid}
\title{Support for \code{multcomp::glht}}
\usage{
emm(...)

as.glht(object, ...)

\method{as.glht}{emmGrid}(object, ...)
}
\arguments{
\item{...}{In \code{emm}, the \code{specs}, \code{by}, and \code{contr}
arguments you would normally supply to \code{\link{emmeans}}. Only
\code{specs} is required. Otherwise, arguments are passed to other
methods. You may also include a \code{which} argument; see Details.}

\item{object}{An object of class \code{emmGrid} or \code{emm_list}}
}
\value{
\code{emm} returns an object of an intermediate class for which
  there is a \code{multcomp::glht} method.

\code{as.glht} returns an object of class \code{glht} or \code{glht_list}
  according to whether \code{object} is of class \code{emmGrid} or \code{emm_list}. 
  See Details below for more on \code{glht_list}s.
}
\description{
These functions and methods provide an interface between \pkg{emmeans} and
the \code{multcomp::glht} function for simultaneous inference provided
by the \pkg{multcomp} package.
}
\note{
The multivariate-\eqn{t} routines used by \code{glht} require that all
  estimates in the family have the same integer degrees of freedom. In cases
  where that is not true, a message is displayed that shows what df is used.
  The user may override this via the \code{df} argument.
}
\section{Details for \code{emm}}{
  
\code{emm} is meant to be called only \emph{from} \code{"glht"} as its second
(\code{linfct}) argument. It works similarly to \code{multcomp::mcp},
except with \code{specs} (and optionally \code{by} and \code{contr}
arguments) provided as in a call to \code{\link{emmeans}}.

If the specifications in \code{...} would result in a list (i.e., an
\code{emm_list} object), then by default, only the last element of that list
is passed to \code{glht}. However, if \code{...} contains a \code{which}
argument consisting of integer values, the list elements with those indexes
are selected and combined and passed on to \code{glht}. No checking is done
on whether the indexes are valid, and the keyword \code{which} must be spelled-out.
}

\section{Details for \code{as.glht}}{

When no \code{by} variable is in force, we obtain a \code{glht} object; otherwise
it is a \code{glht_list}. The latter is defined in \pkg{emmeans}, not \pkg{multcomp},
and is simply a \code{list} of \code{glht} objects. 
Appropriate convenience methods \code{coef},
\code{confint}, \code{plot}, \code{summary}, and \code{vcov} are provided,
which simply apply the corresponding \code{glht} methods to each member.
}

\examples{
if(require(multcomp, quietly = TRUE)) 
    emm_example("glht-multcomp") 
    # Use emm_example("glht-multcomp", list = TRUE) # to see just the code
    
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{pigs}
\alias{pigs}
\title{Effects of dietary protein on free plasma leucine concentration in pigs}
\format{
A data frame with 29 observations and 3 variables:
\describe{
  \item{source}{Source of protein in the diet (factor with 3 levels: 
    fish meal, soybean meal, dried skim milk)}
  \item{percent}{Protein percentage in the diet (numeric with 4 values:
    9, 12, 15, and 18)}
  \item{conc}{Concentration of free plasma leucine, in mcg/ml}
}
}
\source{
Windels HF (1964) PhD thesis, Univ. of Minnesota. (Reported as
  Problem 10.8 in Oehlert G (2000) \emph{A First Course in Design and
  Analysis of Experiments}, licensed under Creative Commons,
  \url{http://users.stat.umn.edu/~gary/Book.html}.) Observations 7, 22, 23,
  31, 33, and 35 have been omitted, creating a more notable imbalance.
}
\usage{
pigs
}
\description{
A two-factor experiment with some observations lost
}
\examples{
  pigs.lm <- lm(inverse(conc) ~ source + factor(percent), data = pigs)
  emmeans(pigs.lm, "source")
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qdrg.R
\name{qdrg}
\alias{qdrg}
\alias{qdrg.default}
\title{Quick and dirty reference grid}
\usage{
qdrg(formula, data, coef, vcov, df, mcmc, object, subset, weights, contrasts,
  link, qr, ordinal, ...)

\method{qdrg}{default}(formula = stats::formula(object),
  data = try(recover_data.lm(object), silent = TRUE),
  coef = stats::coef(object), vcov = stats::vcov(object),
  df = stats::df.residual(object), mcmc, object, subset,
  weights = stats::weights(object), contrasts = object$contrasts,
  link = ifelse(!is.null(lnk <- object$family$link), lnk, object$link),
  qr = object$qr, ordinal, ...)
}
\arguments{
\item{formula}{Formula for the fixed effects}

\item{data}{Dataset containing the variables in the model}

\item{coef}{Fixed-effect regression coefficients (must conform to formula)}

\item{vcov}{Variance-covariance matrix of the fixed effects}

\item{df}{Error degrees of freedom}

\item{mcmc}{Posterior sample of fixed-effect coefficients}

\item{object}{Optional model object. \emph{This rarely works!}; 
but if provided, we try to set 
other arguments based on an expectation that `object` has a similar
structure to `lm` objects. See Details.}

\item{subset}{Subset of \code{data} used in fitting the model}

\item{weights}{Weights used in fitting the model}

\item{contrasts}{List of contrasts specified in fitting the model}

\item{link}{Link function (character or list) used, if a generalized linear model.
(Note: response transformations are auto-detected from \code{formula})}

\item{qr}{QR decomposition of the model matrix; used only if there are \code{NA}s
in \code{coef}.}

\item{ordinal}{\code{list} with elements \code{dim} and \code{mode}.
\code{ordinal$dim} (integer) is the number of levels in an ordinal response. If 
\code{ordinal} is provided, the intercept terms are modified appropriate to predicting 
an ordinal response, as described in \code{vignette("models")}, Section O,
using \code{ordinal$mode} as the \code{mode} argument (if not
provided, \code{"latent"} is assumed).
(All modes are supported except `scale`)
For this to work, we expect
the first \code{ordinal$dim - 1} elements of \code{coef} to be the
estimated threshold parameters, followed by the coefficients for the
linear predictor. Also, if \code{mode} requires back-transforming (e.g.,
\code{"prob"} or \code{"mean.class"}), the user may need to supply the \code{link}
for it to work correctly.}

\item{...}{Optional arguments passed to \code{\link{ref_grid}}}
}
\value{
An \code{emmGrid} object constructed from the arguments
}
\description{
This function may make it possible to compute a reference grid for a model 
object that is otherwise not supported.
}
\details{
Usually, you need to provide either \code{object}; or
\code{formula}, \code{coef}, \code{vcov}, \code{data}, and perhaps other
parameters. It is often fairly straightforward to figure out how to get
these from the model \code{object}; see the documentation for the model class that
was fitted. Sometimes one or more of these quantities contains extra parameters,
and if so, you may need to subset them to make everything conformable. For a given \code{formula} and \code{data},
you can find out what is needed via \code{colnames(model.matrix(formula, data))}.
(However, for an ordinal model, we expect the first \code{ordinal.dim - 1} coefficients
to replace \code{(Intercept)}. And for a multivariate model, we expect \code{coef} 
to be a matrix with these row names, and \code{vcov} to have as many rows and columns as
the total number of elements of \code{coef}.)

If \code{object} is specified, this function serves as a generic for dispatching
a method for \code{object}'s class. See the arguments for \code{qdrg.default} to
see how the arguments are determined by default. For many \code{lm}- or \code{glm}-like
models, it may suffice override just one or two of these
defaults. The possibility of a custom \code{qdrg} method also provides a
minimal way for package developers to provide \pkg{emmeans} support: it doesn't
allow directly applying \code{emmeans()} on the model, but at least the user
can obtain a reference grid, and then go from there. Note that when using a `qdrg` 
method, it is best for the user to specify `object =` explicitly in the
call, since `object` is not the first argument of `qdrg()`. However, if the
first argument is not a formula, the function is retried with it as \code{object}.

The functions \code{\link{qdrg}} and \code{emmobj} are close cousins, in that
they both produce \code{emmGrid} objects. When starting with summary
statistics for an existing grid, \code{emmobj} is more useful, while
\code{qdrg} is more useful when starting from a fitted model.
}
\note{
For backwards compatibility, an argument \code{ordinal.dim} is invisibly 
supported as part of \code{...}, and if present, sets 
\code{ordinal = list(dim = ordinal.dim, mode = "latent")}
}
\section{Rank deficiencies}{

Different model-fitting packages take different approaches when the model
matrix is singular, but \code{qdrg} tries to reconcile them by comparing the
linear functions created by \code{formula} to \code{coefs} and \code{vcov}.
We may then use the \pkg{estimability} package to determine what quantities
are estimable. For reconciling to work properly, \code{coef} should be named
and \code{vcov} should have dimnames. To disable this name-matching
action, remove the names from \code{coef}, e.g., by calling \code{unname()}.
No reconciliation is attempted in multivariate-response cases. For more
details on estimability, see the documentation in the \pkg{estimability}
package.
}

\examples{
# In these examples, use emm_example(..., list = TRUE) # to see just the code

if (require(biglm, quietly = TRUE)) 
    emm_example("qdrg-biglm")
    
if(require(coda, quietly = TRUE) && require(lme4, quietly = TRUE)) 
    emm_example("qdrg-coda")
    
if(require(ordinal, quietly = TRUE)) 
    emm_example("qdrg-ordinal")

}
\seealso{
\code{\link{emmobj}} for an alternative way to construct an \code{emmGrid}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eff-size.R
\name{eff_size}
\alias{eff_size}
\title{Calculate Cohen effect sizes and confidence bounds thereof}
\usage{
eff_size(object, sigma, edf, method = "pairwise", ...)
}
\arguments{
\item{object}{an \code{\link[=emmGrid-class]{emmGrid}} object, 
typically one defining the EMMs to 
be contrasted. If instead, \code{class(object) == "emm_list"},
such as is produced by \code{emmeans(model, pairwise ~ treatment)},
a message is displayed; the contrasts already therein are used; and 
\code{method} is replaced by \code{"identity"}.}

\item{sigma}{numeric scalar, value of the population SD.}

\item{edf}{numeric scalar that specifies the equivalent degrees of freedom
for the \code{sigma}. This is a way of specifying the uncertainty in \code{sigma},
in that we regard our estimate of \code{sigma^2} as being proportional to
a chi-square random variable with \code{edf} degrees of freedom. (\code{edf} should
not be confused with the \code{df} argument that may be passed via \code{...}
to specify the degrees of freedom to use in \eqn{t} statistics and confidence intervals.)}

\item{method}{the contrast method to use to define the effects.
This is passed to \code{\link{contrast}} after the elements of \code{object}
are scaled.}

\item{...}{Additional arguments passed to \code{contrast}}
}
\value{
an \code{\link[=emmGrid-class]{emmGrid}} object containing the effect sizes
}
\description{
Standardized effect sizes are typically calculated using pairwise differences of estimates,
divided by the SD of the population providing the context for those effects.
This function calculates effect sizes from an \code{emmGrid} object,
and confidence intervals for them, accounting for uncertainty in both the estimated
effects and the population SD.
}
\details{
Any \code{by} variables specified in \code{object} will remain in force in the returned
effects, unless overridden in the optional arguments.

For models having a single random effect, such as those fitted using
\code{\link{lm}}; in that case, the \code{stats::sigma} and
\code{stats::df.residual} functions may be useful for specifying \code{sigma}
and \code{edf}. For models with more than one random effect, \code{sigma} may
be based on some combination of the random-effect variances. 

Specifying \code{edf} can be rather unintuitive but is also relatively
uncritical; but the smaller the value, the wider the confidence intervals for
effect size. The value of \code{sqrt(2/edf)} can be interpreted as the
relative accuracy of \code{sigma}; for example, with \code{edf = 50},
\eqn{\sqrt(2/50) = 0.2}, meaning that \code{sigma} is accurate to plus or
minus 20 percent. Note in an example below, we tried two different \code{edf}
values as kind of a bracketing/sensitivity-analysis strategy. A value of
\code{Inf} is allowable, in which case you are assuming that \code{sigma} is
known exactly. Obviously, this narrows the confidence intervals for the
effect sizes -- unrealistically if in fact \code{sigma} is unknown.
}
\note{
The effects are always computed on the scale of the \emph{linear-predictor};
any response transformation or link function is completely ignored. If you
wish to base the effect sizes on the response scale, it is \emph{not} enough
to replace \code{object} with \code{regrid(object)}, because this
back-transformation changes the SD required to compute effect sizes.

\strong{Paired data:} Be careful with paired-data situations, where Cohen's d is typically referenced to
the SD of the \emph{paired differences} rather than the \emph{residual} SD.
You may need to enlarge \code{sigma} by a factor of \code{sqrt(2)} to obtain
comparable results with other software.

\strong{Disclaimer:} There is substantial disagreement among practitioners on
what is the appropriate \code{sigma} to use in computing effect sizes; or,
indeed, whether \emph{any} effect-size measure is appropriate for some
situations. The user is completely responsible for specifying 
appropriate parameters (or for failing to do so).

Cohen effect sizes do not even exist for generalized linear models or other
models lacking an additive residual error term.

The examples here illustrate a sobering message that effect sizes are often not nearly as accurate as you may think.
}
\section{Computation}{

This function uses calls to \code{\link{regrid}} to put the estimated
marginal means (EMMs) on the log scale. Then an extra element is added to
this grid for the log of \code{sigma} and its standard error (where we assume
that \code{sigma} is uncorrelated with the log EMMs). Then a call to
\code{\link{contrast}} subtracts \code{log{sigma}} from each of the log EMMs,
yielding values of \code{log(EMM/sigma)}.
Finally, the results are re-gridded back to the original scale and the
desired contrasts are computed using \code{method}. In the log-scaling
part, we actually rescale the absolute values and keep track of the signs.
}

\examples{
fiber.lm <- lm(strength ~ diameter + machine, data = fiber)

emm <- emmeans(fiber.lm, "machine")
eff_size(emm, sigma = sigma(fiber.lm), edf = df.residual(fiber.lm))

# or equivalently:
eff_size(pairs(emm), sigma(fiber.lm), df.residual(fiber.lm), method = "identity")


### Mixed model example:
if (require(nlme)) withAutoprint({
  Oats.lme <- lme(yield ~ Variety + factor(nitro), 
                  random = ~ 1 | Block / Variety,
                  data = Oats)
  # Combine variance estimates
  VarCorr(Oats.lme)
  (totSD <- sqrt(214.4724 + 109.6931 + 162.5590))
  # I figure edf is somewhere between 5 (Blocks df) and 51 (Resid df)
  emmV <- emmeans(Oats.lme, ~ Variety)
  eff_size(emmV, sigma = totSD, edf = 5)
  eff_size(emmV, sigma = totSD, edf = 51)
}, spaced = TRUE)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test.R
\name{joint_tests}
\alias{joint_tests}
\alias{make.meanint}
\alias{meanint}
\alias{make.symmint}
\alias{symmint}
\title{Compute joint tests of the terms in a model}
\usage{
joint_tests(object, by = NULL, show0df = FALSE, showconf = TRUE,
  cov.reduce = make.meanint(1), ...)

make.meanint(delta = 1, npts = 2)

meanint(x)

make.symmint(ctr, delta = 1, npts = 2)

symmint(ctr)
}
\arguments{
\item{object}{a fitted model, \code{emmGrid}, or \code{emm_list}. If the
latter, its first element is used.}

\item{by}{character names of \code{by} variables. Separate sets of tests are
run for each combination of these.}

\item{show0df}{logical value; if \code{TRUE}, results with zero numerator
degrees of freedom are displayed, if \code{FALSE} they are skipped}

\item{showconf}{logical value.
When we have models with estimability issues (e.g., missing cells), then with
\code{showconf = TRUE}, we test any remaining effects that are not purely
due to contrasts of a single term. If found, they are labeled
\code{(confounded)}. See
\code{vignette("xplanations")} for more information.}

\item{cov.reduce}{a function.
If \code{object} is a fitted model, it is
replaced by \code{ref_grid(object, cov.reduce = cov.reduce, ...)}.
For this purpose, the functions \code{meanint} and \code{symmint} are
available for returning an interval around the mean or around zero,
respectively. Se the section below on covariates.}

\item{...}{additional arguments passed to \code{ref_grid} and \code{emmeans}}

\item{delta, ctr}{arguments for \code{make.meanint} and \code{make.symmint}.
\code{delta} sets the distance each side of the center, so that the
width of the interval is \code{2*delta}.}

\item{npts}{number of points to include in the interval}

\item{x}{argument for \code{meanint} and \code{symmint}}
}
\value{
a \code{summary_emm} object (same as is produced by 
  \code{\link{summary.emmGrid}}). All effects for which there are no
  estimable contrasts are omitted from the results. 
  There may be an additional row named \code{(confounded)} which accounts
  for additional degrees of freedom for effects not accounted for in the 
  preceding rows.
  
  The returned object also includes an \code{"est.fcns"} attribute, which is a
  named list containing the linear functions associated with each joint test.
  Each row of these is standardized to have length 1. 
  No estimable functions are included for confounded effects.

\code{make.meanint} returns the function 
\code{function(x) mean(x) + delta * c(-1, 1)},
  and \code{make.symmint(ctr, delta)} returns the function
\code{function(x) ctr + delta * c(-1, 1)}
        (which does not depend on \code{x}).
The cases with \code{delta = 1}, \code{meanint = make.meanint(1)} and
\code{symmint(ctr) = make.symmint(ctr, 1)} are retained for
back-compatibility reasons. These functions are available primarily for use
with \code{cov.reduce}.
}
\description{
This function produces an analysis-of-variance-like table based on linear
functions of predictors in a model or \code{emmGrid} object. Specifically,
the function constructs, for each combination of factors (or covariates
reduced to two or more levels), a set of (interaction) contrasts via
\code{\link{contrast}}, and then tests them using \code{\link{test}} with
\code{joint = TRUE}. Optionally, one or more of the predictors may be used as
\code{by} variable(s), so that separate tables of tests are produced for
each combination of them.
}
\details{
In models with only factors, no covariates, these tests correspond to
\dQuote{type III} tests a la \pkg{SAS}, as long as equal-weighted averaging
is used and there are no estimability issues. When covariates are present and
they interact with factors, the results depend on how the covariate is
handled in constructing the reference grid. See the section on covariates
below. The point that one must always remember is that \code{joint_tests}
always tests contrasts among EMMs, in the context of the reference grid,
whereas SAS's type III tests are tests of model coefficients -- which may or may
not have anything to do with EMMs or contrasts.
}
\note{
\code{joint_tests} is flaky with models having nested fixed effects. In
some cases, terms that could be relevant are not identified, or confounded
with unidentifiable terms.
}
\section{Dealing with covariates}{

A covariate (or any other predictor) must have \emph{more than one value in 
the reference grid} in order to test its effect and be included in the results.
Therefore, when \code{object} is a model, we default to \code{cov.reduce = meanint}
which sets each covariate at a symmetric interval about its mean. But
when \code{object} is an existing reference grid, it often has only one value 
for covariates, in which case they are excluded from the joint tests.

While having two points is sufficient when the covariate term has a linear trend,
you need more than two when some kind of curved trend (polynomial, spline, etc.)
is present -- else \code{joint_tests()} will not show enough degrees of freedom 
for terms involving the covariate. You may specify these points manually using \code{at},
or by including an \code{npts} argument in \code{cov.reduce}, via \code{make.meanint}
or \code{make.symmint()}. With some kinds of curved trends, the joint tests of
covariate terms may become somewhat meaningless.

Covariates present further complications in that their values in the
reference grid can affect the joint tests of \emph{other} effects. When
covariates are centered around their means (the default), then the tests we
obtain can be described as joint tests of covariate-adjusted means; and that
is our intended use here. However, some software such as \pkg{SAS} and
\code{car::Anova} adopt the convention of centering covariates around zero;
and for that purpose, one can use \code{cov.reduce = symmint(0)} when calling
with a model object (or in constructing a reference grid). However, adjusted
means with covariates set at or around zero do not make much sense in the
context of interpreting estimated marginal means, unless the covariate means
really are zero.

See the examples below with the \code{toy} dataset.
}

\examples{
pigs.lm <- lm(log(conc) ~ source * factor(percent), data = pigs)

(jt <- joint_tests(pigs.lm))             ## will be same as type III ANOVA

### Estimable functions associated with "percent"
attr(jt, "est.fcns") $ "percent"

joint_tests(pigs.lm, weights = "outer")  ## differently weighted

joint_tests(pigs.lm, by = "source")      ## separate joint tests of 'percent'

### Comparisons with type III tests in SAS
toy = data.frame(
    treat = rep(c("A", "B"), c(4, 6)),
    female = c(1, 0, 0, 1,   0, 0, 0, 1, 1, 0 ),
    resp = c(17, 12, 14, 19, 28, 26, 26, 34, 33, 27))
toy.fac = lm(resp ~ treat * factor(female), data = toy)
toy.cov = lm(resp ~ treat * female, data = toy)
# (These two models have identical fitted values and residuals)

# -- SAS output we'd get with toy.fac --
## Source          DF    Type III SS    Mean Square   F Value   Pr > F
## treat            1    488.8928571    488.8928571    404.60   <.0001
## female           1     78.8928571     78.8928571     65.29   0.0002
## treat*female     1      1.7500000      1.7500000      1.45   0.2741
# 
# -- SAS output we'd get with toy.cov --
## Source          DF    Type III SS    Mean Square   F Value   Pr > F
## treat            1    252.0833333    252.0833333    208.62   <.0001
## female           1     78.8928571     78.8928571     65.29   0.0002
## female*treat     1      1.7500000      1.7500000      1.45   0.2741

joint_tests(toy.fac)
joint_tests(toy.cov)   # female is regarded as a 2-level factor by default

## Treat 'female' as a numeric covariate (via cov.keep = 0)
## ... then tests depend on where we center things

# Center around the mean
joint_tests(toy.cov, cov.keep = 0, cov.reduce = make.meanint(delta = 1))
# Center around zero (like SAS's results for toy.cov)
joint_tests(toy.cov, cov.keep = 0, cov.reduce = make.symmint(ctr = 0, delta = 1))
# Center around 0.5 (like SAS's results for toy.fac)
joint_tests(toy.cov, cov.keep = 0, cov.reduce = range)

### Example with empty cells and confounded effects
low3 <- unlist(attr(ubds, "cells")[1:3]) 
ubds.lm <- lm(y ~ A*B*C, data = ubds, subset = -low3)

# Show overall joint tests by C:
ref_grid(ubds.lm, by = "C") |> contrast("consec") |> test(joint = TRUE)

# Break each of the above into smaller components:
joint_tests(ubds.lm, by = "C")

}
\seealso{
\code{\link{test}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emmGrid-methods.R, R/summary.R
\name{update.emmGrid}
\alias{update.emmGrid}
\alias{levels<-.emmGrid}
\alias{update.summary_emm}
\title{Update an \code{emmGrid} object}
\usage{
\method{update}{emmGrid}(object, ..., silent = FALSE)

\method{levels}{emmGrid}(x) <- value

\method{update}{summary_emm}(object, by.vars, mesg, ...)
}
\arguments{
\item{object}{An \code{emmGrid} object}

\item{...}{Options to be set. These must match a list of known options (see
Details)}

\item{silent}{Logical value. If \code{FALSE} (the default), a message is
displayed if any options are not matched. If \code{TRUE}, no messages are
shown.}

\item{x}{an \code{emmGrid} object}

\item{value}{\code{list} or replacement levels. See the documentation for
\code{update.emmGrid} with the \code{levels} argument, 
as well as the section below on \dQuote{Replaciong levels}}

\item{by.vars, mesg}{Attributes that can be altered in \code{update.summary_emm}}
}
\value{
an updated \code{emmGrid} object.

\code{levels<-} replaces the levels of the object in-place.
  See the section on replacing levels for details.
}
\description{
Objects of class \code{emmGrid} contain several settings that affect such things as
what arguments to pass to \code{\link{summary.emmGrid}}. 
The \code{update} method allows safer management of these settings than
by direct modification of its slots.
}
\note{
When it makes sense, an option set by \code{update} will persist into 
future results based on that object. But some options are disabled as well.
For example, a \code{calc} option will be nulled-out if \code{contrast}
is called, because it probably will not make sense to do the same 
calculations on the contrast results, and in fact the variable(s) needed
may not even still exist.
\code{factor(percent)}.
}
\section{Details}{

The names in \code{\dots} are partially matched against those that are valid, and if a match is found, it adds or replaces the current setting. The valid names are

\describe{
\item{\code{tran}, \code{tran2}}{(\code{list} or \code{character}) specifies
the transformation which, when inverted, determines the results displayed by
\code{\link{summary.emmGrid}}, \code{\link{predict.emmGrid}}, or \code{\link{emmip}} when
\code{type="response"}. The value may be the name of a standard
transformation from \code{\link{make.link}} or additional ones supported by
name, such as \code{"log2"}; or, for a custom transformation, a \code{list}
containing at least the functions \code{linkinv} (the inverse of the
transformation) and \code{mu.eta} (the derivative thereof). The
\code{\link{make.tran}} function returns such lists for a number of popular
transformations. See the help page of \code{\link{make.tran}} for details as
well as information on the additional named transformations that are
supported. \code{tran2} is just like \code{tran} except it is a second
transformation (i.e., a response transformation in a generalized linear
model).}

\item{\code{tran.mult}}{Multiple for \code{tran}. For example, for the
response transformation \samp{2*sqrt(y)} (or \samp{sqrt(y) + sqrt(y + 1)},
for that matter), we should have \code{tran = "sqrt"} and \code{tran.mult =
2}. If absent, a multiple of 1 is assumed.}

\item{\code{tran.offset}}{Additive constant before a transformation is applied.
For example, a response transformation of \code{log(y + pi)} has
\code{tran.offset  = pi}. If no value is present, an offset of 0 is assumed.}

\item{\code{estName}}{(\code{character}) is the column label used for
displaying predictions or EMMs.}

\item{\code{inv.lbl}}{(\code{character)}) is the column label to use for
predictions or EMMs when \code{type="response"}.}

\item{\code{by.vars}}{(\code{character)} vector or \code{NULL}) the variables
used for grouping in the summary, and also for defining subfamilies in a call
to \code{\link{contrast}}.}

\item{\code{pri.vars}}{(\code{character} vector) are the names of the grid
variables that are not in \code{by.vars}. Thus, the combinations of their
levels are used as columns in each table produced by \code{\link{summary.emmGrid}}.}

\item{\code{alpha}}{(numeric) is the default significance level for tests, in
\code{\link{summary.emmGrid}} as well as \code{\link{plot.emmGrid}}
when \samp{CIs = TRUE}. Be cautious that methods that depend on
specifying \code{alpha} are prone to abuse. See the
discussion in \href{../doc/basics.html#pvalues}{\code{vignette("basics", "emmeans")}}.}

\item{\code{adjust}}{(\code{character)}) is the default for the \code{adjust}
argument in \code{\link{summary.emmGrid}}.}

\item{\code{cross.adjust}}{(\code{character)}) is the default for the \code{cross.adjust}
argument in \code{\link{summary.emmGrid}} (used for adjusting between groups).}

\item{\code{famSize}}{(integer) is the number of means involved in a family of
inferences; used in Tukey adjustment}

\item{\code{infer}}{(\code{logical} vector of length 2) is the default value
of \code{infer} in \code{\link{summary.emmGrid}}.}

\item{\code{level}}{(numeric) is the default confidence level, \code{level},
in \code{\link{summary.emmGrid}}. \emph{Note:} You must specify all five letters 
of \sQuote{level} to distinguish it from the slot name \sQuote{levels}.}

\item{\code{df}}{(numeric) overrides the default degrees of freedom with a
specified single value.}

\item{\code{calc}}{(list) additional calculated columns. See \code{\link{summary.emmGrid}}.}

\item{\code{null}}{(numeric) null hypothesis for \code{summary} or
\code{test} (taken to be zero if missing).}

\item{\code{side}}{(numeric or character) \code{side} specification for for
\code{summary} or \code{test} (taken to be zero if missing).}

\item{\code{sigma}}{(numeric) Error SD to use in predictions and for bias-adjusted
back-transformations}

\item{\code{delta}}{(numeric) \code{delta} specification for \code{summary}
or \code{test} (taken to be zero if missing).}

\item{\code{predict.type} or \code{type}}{(character) sets the default method
of displaying predictions in \code{\link{summary.emmGrid}},
\code{\link{predict.emmGrid}}, and \code{\link{emmip}}. Valid values are
\code{"link"} (with synonyms \code{"lp"} and \code{"linear"}), or
\code{"response"}.}

\item{\code{bias.adjust}, \code{frequentist}}{(logical) These
are used by \code{summary} if the value of these arguments are not specified.}

\item{\code{estType}}{(\code{character}) is used internally to determine 
what \code{adjust} methods are appropriate. It should match one of 
\samp{c("prediction", "contrast", "pairs")}. As an example of why this is needed,
the Tukey adjustment should only be used for pairwise comparisons 
(\code{estType = "pairs"}); if \code{estType} is some other string, Tukey
adjustments are not allowed.}

\item{\code{avgd.over}}{(\code{character)} vector) are the names of the 
variables whose levels are averaged over in obtaining marginal averages of 
predictions, i.e., estimated marginal means. Changing this might produce a 
misleading printout, but setting it to \code{character(0)} will suppress the 
\dQuote{averaged over} message in the summary.}

\item{\code{initMesg}}{(\code{character}) is a string that is added to the
beginning of any annotations that appear below the \code{\link{summary.emmGrid}}
display.}

\item{\code{methDesc}}{(\code{character}) is a string that may be used for
creating names for a list of \code{emmGrid} objects. }

\item{\code{nesting}}{(Character or named \code{list}) specifies the nesting
structure. See \dQuote{Recovering or overriding model information} in the
documentation for \code{\link{ref_grid}}. The current nesting structure is
displayed by \code{\link{str.emmGrid}}.}

\item{\code{levels}}{named \code{list} of new levels for the elements of the
current \code{emmGrid}. The list name(s) are used as new variable names, and
if needed, the list is expanded using \code{expand.grid}. These results replace
current variable names and levels. This specification changes the \code{levels},
\code{grid}, \code{roles}, and \code{misc} slots in the updated \code{emmGrid},
and resets \code{pri.vars}, \code{by.vars}, \code{adjust}, \code{famSize},
and \code{avgd.over}. In addition, if there is nesting of factors, that may be 
altered; a warning is issued if it involves something other than mere name changes.
\emph{Note:} All six letters of \code{levels} is needed in order to distinguish
it from \code{level}.}

\item{\code{submodel}}{\code{formula} or \code{character} value specifying a 
submodel (requires this feature being supported by underlying methods 
for the model class). When specified, the \code{linfct} slot is replaced by 
its aliases for the specified sub-model. Any factors in the sub-model that 
do not appear in the model matrix are ignored, as are any interactions that 
are not in the main model, and any factors associate with multivariate responses. 
The estimates displayed are then computed as if 
the sub-model had been fitted. (However, the standard errors will be based on the
error variance(s) of the full model.) 
\emph{Note:} The formula should refer only to predictor names, \emph{excluding} any
function calls (such as \code{factor} or \code{poly}) that appear in the 
original model formula. See the example.

The character values allowed should partially 
match \code{"minimal"} or \code{"type2"}. With \code{"minimal"}, the sub-model
is taken to be the one only involving the surviving factors in \code{object}
(the ones averaged over being omitted). Specifying \code{"type2"} is the same as
\code{"minimal"} except only the highest-order term in the submodel is retained,
and all effects not containing it are orthogonalized-out. Thus, in a purely linear
situation such as an \code{lm} model, the joint test
of the modified object is in essence a type-2 test as in \code{car::Anova}.

Please note that it is possible (or even likely) that there will be disparity
between the \code{grid} and \code{linfct} slots when a submodel is used. This is
because \code{grid} contains the \emph{claimed} values of the predictors and
\code{linfct} contains \emph{aliases} of them computed from the submodel.

For some objects such as generalized linear models, specifying \code{submodel}
will typically not produce the same estimates or type-2 tests as would be
obtained by actually fitting a separate model with those specifications.
The reason is that those models are fitted by iterative-reweighting methods,
whereas the \code{submodel} calculations preserve the final weights used in
fitting the full model.}

\item{(any other slot name)}{If the name matches an element of
\code{slotNames(object)} other than \code{levels}, that slot is replaced by 
the supplied value, if it is of the required class (otherwise an error occurs). 

The user must be very careful in
replacing slots because they are interrelated; for example, the lengths
and dimensions of \code{grid}, \code{linfct}, \code{bhat}, and \code{V} must
conform.}
} %%%%%%% end \describe
}

\section{Replacing levels}{

The \code{levels<-} method uses \code{update.emmGrid} to replace the
levels of one or more factors. This method allows selectively replacing
the levels of just one factor (via subsetting operators), whereas 
\code{update(x, levels = list(...))} requires a list of \emph{all} factors
and their levels. If any factors are to be renamed, we must replace all
levels and include the new names in the replacements. See the examples.
}

\section{Method for \code{summary_emm} objects}{

This method exists so that we can change the way a summary is displayed,
by changing the by variables or the annotations.
}

\examples{
# Using an already-transformed response:
pigs.lm <- lm(log(conc) ~ source * factor(percent), data = pigs)

# Reference grid that knows about the transformation
# and asks to include the sample size in any summaries:
pigs.rg <- update(ref_grid(pigs.lm), tran = "log", 
                    predict.type = "response",
                    calc = c(n = ~.wgt.))
emmeans(pigs.rg, "source")

# Obtain estimates for the additive model
# [Note that the submodel refers to 'percent', not 'factor(percent)']
emmeans(pigs.rg, "source", submodel = ~ source + percent)

# Type II ANOVA
joint_tests(pigs.rg, submodel = "type2")

## Changing levels of one factor
newrg <- pigs.rg
levels(newrg)$source <- 1:3
newrg

## Unraveling a previously standardized covariate
zd = scale(fiber$diameter)
fibz.lm <- lm(strength ~ machine * zd, data = fiber)
(fibz.rg <- ref_grid(fibz.lm, at = list(zd = -2:2)))   ### 2*SD range
lev <- levels(fibz.rg)
levels(fibz.rg) <- list (
    machine = lev$machine,
    diameter = with(attributes(zd), 
                    `scaled:center` + `scaled:scale` * lev$zd) )
fibz.rg

### Compactify results with a by variable
update(joint_tests(pigs.rg, by = "source"), by = NULL)
}
\seealso{
\code{\link{emm_options}}
}
